@@ -22,6 +22,8 @@ class QWidget;
 
 #include "parameter.h"
 
+class XAbstractMessageHandler;
+
 class login2 : public QDialog, public Ui::login2
 {
   Q_OBJECT
@@ -58,7 +60,6 @@ class login2 : public QDialog, public Ui::login2
     virtual void sHandleButton();
     virtual void sOpenHelp();
     virtual void sLogin();
-    virtual void sOptions();
 
   private:
     bool _captive;
@@ -75,6 +76,8 @@ class login2 : public QDialog, public Ui::login2
     QString _cloudDatabaseURL;
     QString _cCompany;
     QString _connAppName;
+
+    XAbstractMessageHandler* _handler;
 };
 
 #endif
