@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2018 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -22,12 +22,12 @@
 
 #include "addresscluster.h"
 #include "contact.h"
-#include "inputManager.h"
 #include "mqlutil.h"
 #include "shipTo.h"
 #include "vendor.h"
 #include "vendorAddress.h"
 #include "warehouse.h"
+#include "errorReporter.h"
 
 address::address(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
@@ -51,6 +51,8 @@ address::address(QWidget* parent, const char* name, bool modal, Qt::WindowFlags
     _uses->addColumn(tr("Web Address"),	100, Qt::AlignLeft, true, "cntct_webaddr");
 
     _charass->setType("ADDR");
+
+    _close = false;
 }
 
 address::~address()
@@ -69,15 +71,6 @@ enum SetResponse address::set(const ParameterList &pParams)
   QVariant param;
   bool     valid;
 
-  param = pParams.value("addr_id", &valid);
-  if (valid)
-  {
-    _captive = true;
-    _addr->setId(param.toInt());
-    sPopulate();
-    _charass->setId(_addr->id());
-  }
-
   param = pParams.value("mode", &valid);
   if (valid)
   {
@@ -88,11 +81,10 @@ enum SetResponse address::set(const ParameterList &pParams)
       int addrSaveResult = _addr->save(AddressCluster::CHANGEONE);
       if (addrSaveResult < 0)
       {
-	systemError(this, tr("There was an error creating a new address (%).\n"
-			     "Check the database server log for errors.")
-			  .arg(addrSaveResult),
-		    __FILE__, __LINE__);
-	return UndefinedError;
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Address"),
+                  tr("There was an error creating a new address (%1).\n"
+                     "Check the database server log for errors.") .arg(addrSaveResult), __FILE__, __LINE__);
+        return UndefinedError;
       }
       _comments->setId(_addr->id());
       _charass->setId(_addr->id());
@@ -103,25 +95,37 @@ enum SetResponse address::set(const ParameterList &pParams)
       _mode = cEdit;
     }
     else if (param.toString() == "view")
-    {
-      _mode = cView;
+      setViewMode();
+  }
 
-      _editAddrUse->hide();
-      disconnect(_uses, SIGNAL(itemSelected(int)), _editAddrUse, SLOT(animateClick()));
-      connect(_uses, SIGNAL(itemSelected(int)), _viewAddrUse, SLOT(animateClick()));
-
-      _addr->setEnabled(false);
-      _notes->setEnabled(false);
-      _comments->setReadOnly(true);
-      _editAddrUse->setEnabled(false);
-      _charass->setReadOnly(true);
-      _buttonBox->setStandardButtons(QDialogButtonBox::Close);
-    }
+  param = pParams.value("addr_id", &valid);
+  if (valid)
+  {
+    _captive = true;
+    _addr->setId(param.toInt());
+    sPopulate();
+    _charass->setId(_addr->id());
   }
 
   return NoError;
 }
 
+void address::setViewMode()
+{
+  _mode = cView;
+
+  _editAddrUse->hide();
+  disconnect(_uses, SIGNAL(itemSelected(int)), _editAddrUse, SLOT(animateClick()));
+  connect(_uses, SIGNAL(itemSelected(int)), _viewAddrUse, SLOT(animateClick()));
+
+  _addr->setEnabled(false);
+  _notes->setEnabled(false);
+  _comments->setReadOnly(true);
+  _editAddrUse->setEnabled(false);
+  _charass->setReadOnly(true);
+  _buttonBox->setStandardButtons(QDialogButtonBox::Close);
+}
+
 void address::sSave()
 {
    internalSave();
@@ -132,27 +136,23 @@ void address::internalSave(AddressCluster::SaveFlags flag)
 {
   _addr->setNotes(_notes->toPlainText());
 
+  int oldId      = _addr->id();
   int saveResult = _addr->save(flag);
-  if (-2 == saveResult)
+  if (saveResult == -2)
   {
-    int answer = QMessageBox::question(this,
-		    tr("Saving Shared Address"),
-		    tr("There are multiple Contacts sharing this Address.\n"
-		       "If you save this Address, the Address for all "
-		       "of these Contacts will be changed. Would you like to "
-		       "save this Address?"),
-		    QMessageBox::No | QMessageBox::Default, QMessageBox::Yes);
-    if (QMessageBox::No == answer)
+    AddressCluster::SaveFlags answer = AddressCluster::askForSaveMode(_addr->id(), this);
+    if (answer == AddressCluster::CHECK)
       return;
-    saveResult = _addr->save(AddressCluster::CHANGEALL);
-  }
-  if (0 > saveResult)	// NOT else if
-  {
-    systemError(this, tr("There was an error saving this address (%1).\n"
-			 "Check the database server log for errors.")
-		      .arg(saveResult),
-		__FILE__, __LINE__);
+    saveResult = _addr->save(answer);
   }
+  if (saveResult < 0)	// NOT else if
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Address"),
+              tr("There was an error saving this address (%1).\n"
+                 "Check the database server log for errors.") .arg(saveResult), __FILE__, __LINE__);
+  else if (saveResult != oldId)
+    QMessageBox::information(this, tr("Not Saved"),
+                             tr("This change was not saved because it would create a duplicate. "
+                                "Merge the two addresses instead."));
 }
 
 void address::reject()
@@ -165,10 +165,10 @@ void address::reject()
     rejectAddress.bindValue(":addr_id", _addr->id());
     rejectAddress.bindValue(":number", _addr->number().toInt());
     rejectAddress.exec();
-    if (rejectAddress.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Rejecting Address"),
+                                  rejectAddress, __FILE__, __LINE__))
     {
-      systemError(this, rejectAddress.lastError().databaseText(), __FILE__, __LINE__);
-      return;
+        return;
     }
   }
 
@@ -177,6 +177,43 @@ void address::reject()
 
 void address::sPopulate()
 {
+  if (_mode == cEdit && !_lock.acquire("addr", _addrid, AppLock::Interactive))
+    setViewMode();
+
+  _close = false;
+
+  foreach (QWidget* widget, QApplication::allWidgets())
+  {
+    if (!widget->isWindow() || !widget->isVisible())
+      continue;
+
+    address *w = qobject_cast<address*>(widget);
+
+    if (w && w != this && w->id()==_addrid)
+    {
+      // detect "i'm my own grandpa"
+      QObject *p;
+      for (p = parent(); p && p != w ; p = p->parent())
+        ; // do nothing
+      if (p == w)
+      {
+        QMessageBox::warning(this, tr("Cannot Open Recursively"),
+                             tr("This address is already open and cannot be "
+                                "raised. Please close windows to get to it."));
+        _close = true;
+      } else if (p) {
+        w->setFocus();
+        if (omfgThis->showTopLevel())
+        {
+          w->raise();
+          w->activateWindow();
+        }
+        _close = true;
+      }
+      break;
+    }
+  }
+
   _notes->setText(_addr->notes());
   _comments->setId(_addr->id());
   _charass->setId(_addr->id());
@@ -196,7 +233,6 @@ void address::sPopulate()
 
 void address::sPopulateMenu(QMenu *pMenu)
 {
-  QAction *menuItem;
   QString editStr = tr("Edit...");
   QString viewStr = tr("View...");
 
@@ -205,18 +241,18 @@ void address::sPopulateMenu(QMenu *pMenu)
     case 1:
       if (_privileges->check("MaintainAllContacts") &&
 	  (cNew == _mode || cEdit == _mode))
-	menuItem = pMenu->addAction(editStr, this, SLOT(sEditContact()));
+    (void)pMenu->addAction(editStr, this, SLOT(sEditContact()));
       else if (_privileges->check("ViewAllContacts"))
-	menuItem = pMenu->addAction(viewStr, this, SLOT(sViewContact()));
+    (void)pMenu->addAction(viewStr, this, SLOT(sViewContact()));
 
       break;
 
     case 2:	// ship-to
       if (_privileges->check("MaintainShiptos") &&
 	  (cNew == _mode || cEdit == _mode))
-	menuItem = pMenu->addAction(editStr, this, SLOT(sEditShipto()));
+    (void)pMenu->addAction(editStr, this, SLOT(sEditShipto()));
       else if (_privileges->check("ViewShiptos"))
-	menuItem = pMenu->addAction(viewStr, this, SLOT(sViewShipto()));
+    (void)pMenu->addAction(viewStr, this, SLOT(sViewShipto()));
 
       break;
 
@@ -234,18 +270,18 @@ void address::sPopulateMenu(QMenu *pMenu)
     case 4:	// vendaddr
       if (_privileges->check("MaintainVendorAddresses") &&
 	  (cNew == _mode || cEdit == _mode))
-	menuItem = pMenu->addAction(editStr, this, SLOT(sEditVendorAddress()));
+    (void)pMenu->addAction(editStr, this, SLOT(sEditVendorAddress()));
       else if (_privileges->check("ViewVendorAddresses"))
-	menuItem = pMenu->addAction(viewStr, this, SLOT(sViewVendorAddress()));
+    (void)pMenu->addAction(viewStr, this, SLOT(sViewVendorAddress()));
 
       break;
 
     case 5:	// warehouse
       if (_privileges->check("MaintainWarehouses") &&
 	  (cNew == _mode || cEdit == _mode))
-	menuItem = pMenu->addAction(editStr, this, SLOT(sEditWarehouse()));
+    (void)pMenu->addAction(editStr, this, SLOT(sEditWarehouse()));
       else if (_privileges->check("ViewWarehouses"))
-	menuItem = pMenu->addAction(viewStr, this, SLOT(sViewWarehouse()));
+    (void)pMenu->addAction(viewStr, this, SLOT(sViewWarehouse()));
 
       break;
 
@@ -418,3 +454,25 @@ void address::sViewWarehouse()
   newdlg.set(params);
   newdlg.exec();
 }
+
+int address::id()
+{
+  return _addrid;
+}
+
+void address::setVisible(bool visible)
+{
+  if (_close)
+    close();
+  else
+    XDialog::setVisible(visible);
+}
+
+void address::done(int result)
+{
+  if (!_lock.release())
+    ErrorReporter::error(QtCriticalMsg, this, tr("Locking Error"),
+                         _lock.lastError(), __FILE__, __LINE__);
+
+  XDialog::done(result);
+}
