@@ -27,6 +27,10 @@ public slots:
     virtual void sEdit();
     virtual void sView();
     virtual void sDelete();
+    virtual void sNewProspect();
+    virtual void sEditProspect();
+    virtual void sViewProspect();
+    virtual void sOpenProspect(QString);
 
 };
 
