@@ -18,6 +18,10 @@
 #include "distributeInventory.h"
 #include "inputManager.h"
 #include "storedProcErrorLookup.h"
+#include "errorReporter.h"
+#include "guiErrorCheck.h"
+
+#define DEBUG false
 
 adjustmentTrans::adjustmentTrans(QWidget* parent, const char * name, Qt::WindowFlags fl)
     : XWidget(parent, name, fl)
@@ -61,6 +65,8 @@ adjustmentTrans::adjustmentTrans(QWidget* parent, const char * name, Qt::WindowF
     _tab->removeTab(0);
 
   _item->setFocus();
+  _itemsiteId = 0;
+  _controlledItem = false;
 }
 
 adjustmentTrans::~adjustmentTrans()
@@ -79,7 +85,6 @@ enum SetResponse adjustmentTrans::set(const ParameterList &pParams)
   QVariant param;
   bool     valid;
   int      invhistid = -1;
-  bool     noQty     = true;
 
   param = pParams.value("itemsite_id", &valid);
   if (valid)
@@ -99,10 +104,9 @@ enum SetResponse adjustmentTrans::set(const ParameterList &pParams)
     _qty->setDouble(param.toDouble());
     _qty->setEnabled(false);
     _afterQty->setDouble(param.toDouble());
+
     _absolute->setChecked(true);
     _adjustmentTypeGroup->setEnabled(false);
-
-    noQty = false;
   }
 
   param = pParams.value("invhist_id", &valid);
@@ -151,10 +155,10 @@ enum SetResponse adjustmentTrans::set(const ParameterList &pParams)
         _notes->setText(setAdjustment.value("invhist_comments").toString());
         _item->setItemsiteid(setAdjustment.value("invhist_itemsite_id").toInt());
       }
-      else if (setAdjustment.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Transaction"),
+                                    setAdjustment, __FILE__, __LINE__))
       {
-	systemError(this, setAdjustment.lastError().databaseText(), __FILE__, __LINE__);
-	return UndefinedError;
+          return UndefinedError;
       }
 
     }
@@ -166,6 +170,8 @@ enum SetResponse adjustmentTrans::set(const ParameterList &pParams)
 void adjustmentTrans::sPost()
 {
   XSqlQuery adjustmentPost;
+  XSqlQuery cleanup;  
+  int itemlocSeries = 0;
   double qty = _qty->toDouble();
   double cost = _cost->toDouble();
 
@@ -175,51 +181,73 @@ void adjustmentTrans::sPost()
     cost = (_cost->toDouble() - _cachedValue);
   }
 
-  struct {
-    bool        condition;
-    QString     msg;
-    QWidget     *widget;
-  } error[] = {
-    { ! _item->isValid(),
-      tr("You must select an Item before posting this transaction."), _item },
-    { _qty->text().length() == 0 || qty == 0,
-      tr("<p>You must enter a Quantity before posting this Transaction."),
-      _qty },
-    { _costAdjust->isEnabled() && _costAdjust->isChecked() && _costManual->isChecked() && (_cost->text().length() == 0 || cost == 0),
-      tr("<p>You must enter a total cost value for the inventory to be transacted."),
-      _cost },
-    { _costMethod == "A" && _afterQty->toDouble() < 0,
-      tr("<p>Average cost adjustments may not result in a negative quantity on hand."),
-      _qty },
-    { true, "", NULL }
-  };
- 
-  int errIndex;
-  for (errIndex = 0; ! error[errIndex].condition; errIndex++)
-    ;
-  if (! error[errIndex].msg.isEmpty())
-  {
-    QMessageBox::critical(this, tr("Cannot Post Transaction"),
-                          error[errIndex].msg);
-    error[errIndex].widget->setFocus();
+  QList<GuiErrorCheck>errors;
+    errors<<GuiErrorCheck(!_item->isValid(), _item,
+      tr("You must select an Item before posting this transaction."))
+    <<GuiErrorCheck(_qty->text().length() == 0 || qty == 0, _qty,
+      tr("<p>You must enter a Quantity before posting this Transaction."))
+    <<GuiErrorCheck(_costAdjust->isEnabled() && _costAdjust->isChecked() && _costManual->isChecked()
+      && (_cost->text().length() == 0 || cost == 0), _cost,
+      tr("<p>You must enter a total cost value for the inventory to be transaction."))
+    <<GuiErrorCheck( _costMethod == "A" && _afterQty->toDouble() < 0, _qty,
+      tr("<p>Average cost adjustments may not result in a negative quantity on hand."));
+
+  if (GuiErrorCheck::reportErrors(this,tr("Cannot Post Transaction"),errors))
     return;
+
+  // Get parent series id
+  XSqlQuery parentSeries;
+  parentSeries.prepare("SELECT NEXTVAL('itemloc_series_seq') AS result;");
+  parentSeries.exec();
+  if (parentSeries.first())
+    itemlocSeries = parentSeries.value("result").toInt();
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Failed to Retrieve the Next itemloc_series_seq"),
+                            parentSeries, __FILE__, __LINE__))
+    return;
+  
+  if (DEBUG)
+    qDebug() << "adjustmentTrans::sPost itemlocSeries: " << itemlocSeries;
+
+  // Stage cleanup function to be called on error
+  cleanup.prepare("SELECT deleteitemlocseries(:itemlocSeries, TRUE);");
+  cleanup.bindValue(":itemlocSeries", itemlocSeries);
+
+  // If controlled item: create the parent itemlocdist record, call distributeInventory::seriesAdjust
+  if (_controlledItem)
+  {
+    XSqlQuery parentItemlocdist;
+    parentItemlocdist.prepare("SELECT createitemlocdistparent(:itemsite_id, :qty, 'AD'::TEXT, NULL, :itemlocSeries) AS result;");
+    parentItemlocdist.bindValue(":itemsite_id", _itemsiteId);
+    parentItemlocdist.bindValue(":qty", qty);
+    parentItemlocdist.bindValue(":itemlocSeries", itemlocSeries);
+    parentItemlocdist.exec();
+    if (parentItemlocdist.first())
+    {
+      if (distributeInventory::SeriesAdjust(itemlocSeries, this, QString(), QDate(), QDate(), true)
+        == XDialog::Rejected)
+      {
+        cleanup.exec();
+        QMessageBox::information(this, tr("Inventory Adjustment"),
+                                 tr("Transaction Canceled") );
+        return;
+      }
+    }
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Creating itemlocdist Records"),
+                              parentItemlocdist, __FILE__, __LINE__))
+    {
+      return;
+    }
   }
 
-  XSqlQuery rollback;
-  rollback.prepare("ROLLBACK;");
-
-  adjustmentPost.exec("BEGIN;");	// because of possible distribution cancelations
-  adjustmentPost.prepare( "SELECT invAdjustment(itemsite_id, :qty, :docNumber,"
-             "                     :comments, :date, :cost) AS result "
-             "FROM itemsite "
-             "WHERE ( (itemsite_item_id=:item_id)"
-             " AND (itemsite_warehous_id=:warehous_id) );" );
+  // Post inventory transaction
+  adjustmentPost.prepare( "SELECT invAdjustment(:itemsite_id, :qty, :docNumber, :comments, :date, "
+                          " :cost, NULL, :itemlocSeries, true) AS result;");
   adjustmentPost.bindValue(":qty", qty);
-  adjustmentPost.bindValue(":docNumber", _documentNum->text());
+  adjustmentPost.bindValue(":docNumber",   _documentNum->text());
   adjustmentPost.bindValue(":comments",    _notes->toPlainText());
-  adjustmentPost.bindValue(":item_id", _item->id());
-  adjustmentPost.bindValue(":warehous_id", _warehouse->id());
+  adjustmentPost.bindValue(":itemsite_id", _itemsiteId);
   adjustmentPost.bindValue(":date",        _transDate->date());
+  adjustmentPost.bindValue(":itemlocSeries", itemlocSeries);
   if(!_costAdjust->isChecked())
     adjustmentPost.bindValue(":cost", 0.0);
   else if(_costManual->isChecked())
@@ -227,28 +255,7 @@ void adjustmentTrans::sPost()
   adjustmentPost.exec();
   if (adjustmentPost.first())
   {
-    int result = adjustmentPost.value("result").toInt();
-    if (result < 0)
-    {
-      rollback.exec();
-      systemError(this, storedProcErrorLookup("invAdjustment", result),
-                  __FILE__, __LINE__);
-    }
-    else if (adjustmentPost.lastError().type() != QSqlError::NoError)
-    {
-      systemError(this, adjustmentPost.lastError().databaseText(), __FILE__, __LINE__);
-      return;
-    }
-
-    if (distributeInventory::SeriesAdjust(adjustmentPost.value("result").toInt(), this) == XDialog::Rejected)
-    {
-      rollback.exec();
-      QMessageBox::information(this, tr("Inventory Adjustment"),
-                               tr("Transaction Canceled") );
-      return;
-    }
-
-    adjustmentPost.exec("COMMIT;");
+    // Continue to cleanup
     if (_captive)
       close();
     else
@@ -265,19 +272,20 @@ void adjustmentTrans::sPost()
       _item->setFocus();
     }
   }
-  else if (adjustmentPost.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting Transaction"),
+                                adjustmentPost, __FILE__, __LINE__))
   {
-    rollback.exec();
-    systemError(this, adjustmentPost.lastError().databaseText(), __FILE__, __LINE__);
+    cleanup.exec();
     return;
   }
   else
   {
-    rollback.exec();
-    systemError( this,
-                tr("<p>No transaction was done because Item %1 "
-                   "was not found at Site %2.")
-                .arg(_item->itemNumber()).arg(_warehouse->currentText()));
+    cleanup.exec();
+    ErrorReporter::error(QtCriticalMsg, this, tr("Post Transaction Cancelled"),
+                         tr("<p>No transaction was done because Item %1 "
+                            "was not found at Site %2.")
+                         .arg(_item->itemNumber()).arg(_warehouse->currentText()),
+                         __FILE__, __LINE__);
   }
 }
 
@@ -286,7 +294,10 @@ void adjustmentTrans::sPopulateQOH()
   XSqlQuery populateAdjustment;
   if (_mode != cView)
   {
-    populateAdjustment.prepare( "SELECT itemsite_freeze, itemsite_qtyonhand, itemsite_costmethod, itemsite_value "
+    populateAdjustment.prepare("SELECT itemsite_id, itemsite_freeze, itemsite_qtyonhand, "
+               "  itemsite_costmethod, itemsite_value, "
+               "  itemsite_loccntrl, "
+               "  (itemsite_controlmethod IN ('L', 'S') ) AS lscntrl "
                "FROM itemsite "
                "WHERE ( (itemsite_item_id=:item_id)"
                " AND (itemsite_warehous_id=:warehous_id));" );
@@ -297,8 +308,11 @@ void adjustmentTrans::sPopulateQOH()
     _absolute->setStyleSheet("");
     if (populateAdjustment.first())
     {
+      _itemsiteId = populateAdjustment.value("itemsite_id").toInt();
       _cachedValue = populateAdjustment.value("itemsite_value").toDouble();
       _cachedQOH = populateAdjustment.value("itemsite_qtyonhand").toDouble();
+      _controlledItem = populateAdjustment.value("lscntrl").toBool() || 
+        populateAdjustment.value("itemsite_loccntrl").toBool();
       if(_cachedQOH == 0.0)
         _costManual->setChecked(true);
       _beforeQty->setDouble(populateAdjustment.value("itemsite_qtyonhand").toDouble());
@@ -315,10 +329,10 @@ void adjustmentTrans::sPopulateQOH()
       else
         _qty->setValidator(new QIntValidator(this));
     }
-    else if (populateAdjustment.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Information"),
+                                  populateAdjustment, __FILE__, __LINE__))
     {
-      systemError(this, populateAdjustment.lastError().databaseText(), __FILE__, __LINE__);
-      return;
+        return;
     }
 
     sPopulateQty();
@@ -367,4 +381,3 @@ void adjustmentTrans::sCostUpdated()
   else
     _unitCost->setDouble(_cost->toDouble() / _qty->toDouble());
 }
-
