@@ -37,12 +37,14 @@ protected slots:
 
 private:
     int _mode;
+    int _itemsiteId;
     double _cachedQOH;
     double _cachedValue;
     bool _captive;
+    bool _controlledItem;
     QButtonGroup* _adjustmentTypeGroupInt;
     QString _costMethod;
 
 };
 
-#endif // ADJUSTMENTTRANS_H
+#endif // ADJUSTMENTTRANS_H
\ No newline at end of file
