@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -32,11 +32,16 @@ public slots:
     virtual SetResponse set( const ParameterList & pParams );
     virtual void sSave();
     virtual void sClose();
+    virtual void sDetermineTaxAmount();
+    virtual bool sInitializeMemo();
     virtual void sPopulateVendInfo( int vend_id );
     virtual void sPopulateDueDate();
     virtual void sTaxDetail();
+    virtual void sPrintOnPost(int temp_id);
     virtual void sReleaseNumber();
+    virtual void sToggleAccount();
     virtual void sCalcBalance();
+    virtual void sViewMode();
 
 protected slots:
     virtual void languageChange();
