@@ -17,8 +17,10 @@
 #include <metasql.h>
 
 #include "arCreditMemoApplication.h"
+#include "getGLDistDate.h"
 #include "mqlutil.h"
 #include "storedProcErrorLookup.h"
+#include "errorReporter.h"
 
 applyARCreditMemo::applyARCreditMemo(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
@@ -90,18 +92,59 @@ void applyARCreditMemo::sPost()
   XSqlQuery applyPost;
   populate(); // repeat in case someone else has updated applications
 
+  QDate applyDate = QDate::currentDate();
+  while (_privileges->check("ChangeCashRecvPostDate"))
+  {
+    getGLDistDate newdlg(this, "", true);
+    newdlg.sSetDefaultLit(tr("Current Date"));
+    if (newdlg.exec() == XDialog::Accepted)
+    {
+      applyDate = newdlg.date();
+      if (applyDate.isNull())
+        applyDate = QDate::currentDate();
+
+      XSqlQuery closedPeriod;
+      closedPeriod.prepare("SELECT period_closed "
+                           "  FROM period "
+                           " WHERE :distdate BETWEEN period_start AND period_end;");
+      closedPeriod.bindValue(":distdate", applyDate);
+      closedPeriod.bindValue(":aropen_id", _aropenid);
+      closedPeriod.exec();
+      if (closedPeriod.first() && !closedPeriod.value("period_closed").toBool())
+        break;
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting A/R CM"),
+                                    closedPeriod, __FILE__, __LINE__))
+        return;
+      else
+      {
+        if (QMessageBox::question(this, tr("Change Date?"),
+                                  tr("Could not post application because the accounting period for "
+                                     "the posting date is closed. Try again with a different "
+                                     "date?"),
+                                  QMessageBox::Yes | QMessageBox::No,
+                                  QMessageBox::No) == QMessageBox::Yes)
+          continue;
+        else
+          return;
+      }
+    }
+    else
+      return;
+  }
+
   XSqlQuery rollback;
   rollback.prepare("ROLLBACK;");
 
   applyPost.exec("BEGIN;");
-  if (applyPost.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting A/R CM"),
+                                applyPost, __FILE__, __LINE__))
   {
-    systemError(this, applyPost.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
-  applyPost.prepare("SELECT postARCreditMemoApplication(:aropen_id) AS result;");
+  applyPost.prepare("SELECT postARCreditMemoApplication(:aropen_id, :applydate) AS result;");
   applyPost.bindValue(":aropen_id", _aropenid);
+  applyPost.bindValue(":applydate", applyDate);
   applyPost.exec();
   if (applyPost.first())
   {
@@ -110,21 +153,22 @@ void applyARCreditMemo::sPost()
     if (result < 0)
     {
       rollback.exec();
-      systemError(this, storedProcErrorLookup("postARCreditMemoApplication", result),
-		  __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting A/R CM"),
+                                      applyPost, __FILE__, __LINE__);
       return;
     }
   }
   else if (applyPost.lastError().type() != QSqlError::NoError)
   {
     rollback.exec();
-    systemError(this, applyPost.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting A/R CM"),
+                                  applyPost, __FILE__, __LINE__);
     return;
   }
   applyPost.exec("COMMIT;");
-  if (applyPost.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting A/R CM"),
+                                applyPost, __FILE__, __LINE__))
   {
-    systemError(this, applyPost.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -142,13 +186,14 @@ void applyARCreditMemo::sApplyBalance()
     int result = applyApplyBalance.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("applyARCreditMemoToBalance", result));
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Applying A/R CM"),
+                                      applyApplyBalance, __FILE__, __LINE__);
       return;
     }
   }
-  else if (applyApplyBalance.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Applying A/R CM"),
+                                applyApplyBalance, __FILE__, __LINE__))
   {
-    systemError(this, applyApplyBalance.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -167,13 +212,14 @@ void applyARCreditMemo::sApplyLineBalance()
     int result = applyApplyLineBalance.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("applyARCreditMemoToBalance", result));
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Applying A/R CM"),
+                                      applyApplyLineBalance, __FILE__, __LINE__);
       return;
     }
   }
-  else if (applyApplyLineBalance.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Applying A/R CM"),
+                                applyApplyLineBalance, __FILE__, __LINE__))
   {
-    systemError(this, applyApplyLineBalance.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -201,8 +247,8 @@ void applyARCreditMemo::sClear()
   applyClear.bindValue(":sourceAropenid", _aropenid);
   applyClear.bindValue(":targetAropenid", _aropen->id());
   applyClear.exec();
-  if (applyClear.lastError().type() != QSqlError::NoError)
-      systemError(this, applyClear.lastError().databaseText(), __FILE__, __LINE__);
+  ErrorReporter::error(QtCriticalMsg, this, tr("Error Clearing Application of A/R CM"),
+                                applyClear, __FILE__, __LINE__);
 
   populate();
 }
@@ -214,8 +260,8 @@ void applyARCreditMemo::sClose()
              "WHERE (arcreditapply_source_aropen_id=:sourceAropenid);" );
   applyClose.bindValue(":sourceAropenid", _aropenid);
   applyClose.exec();
-  if (applyClose.lastError().type() != QSqlError::NoError)
-      systemError(this, applyClose.lastError().databaseText(), __FILE__, __LINE__);
+  ErrorReporter::error(QtCriticalMsg, this, tr("Error Clearing Application of A/R CM"),
+                                applyClose, __FILE__, __LINE__);
 
   reject();
 }
@@ -264,7 +310,8 @@ void applyARCreditMemo::populate()
     _docDate->setDate(applypopulate.value("aropen_docdate").toDate(), true);
   }
   else
-      systemError(this, applypopulate.lastError().databaseText(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving A/R CM Information"),
+                                applypopulate, __FILE__, __LINE__);
 
   MetaSQLQuery mql = mqlLoad("arOpenApplications", "detail");
   ParameterList params;
@@ -274,9 +321,9 @@ void applyARCreditMemo::populate()
   params.append("source_aropen_id", _aropenid);
   applypopulate = mql.toQuery(params);
   _aropen->populate(applypopulate);
-  if (applypopulate.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving A/R CM Information"),
+                                applypopulate, __FILE__, __LINE__))
   {
-    systemError(this, applypopulate.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
