@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2016 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -74,7 +74,6 @@ arOpenItem::arOpenItem(QWidget* parent, const char* name, bool modal, Qt::Window
 
   _journalNumber->setEnabled(false);
 
-  _altAccntid->setType(GLCluster::cRevenue | GLCluster::cExpense);
 }
 
 arOpenItem::~arOpenItem()
@@ -274,6 +273,7 @@ void arOpenItem::sSave()
                "    aropen_commission_due=:aropen_commission_due, aropen_notes=:aropen_notes,"
                "    aropen_rsncode_id=:aropen_rsncode_id, "
                "    aropen_curr_id=:curr_id, "
+               "    aropen_curr_rate = currrate(:curr_id, :aropen_docdate), "
                "    aropen_taxzone_id=:taxzone "
                "WHERE (aropen_id=:aropen_id);" );
   }
