@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -11,14 +11,16 @@
 #include "assignItemToPlannerCode.h"
 
 #include <QMessageBox>
+#include <QPushButton>
 #include <QVariant>
 
+#include "guiErrorCheck.h"
+
 assignItemToPlannerCode::assignItemToPlannerCode(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
 {
   setupUi(this);
 
-    // signals and slots connections
   connect(_item, SIGNAL(valid(bool)), _buttonBox->button(QDialogButtonBox::Ok), SLOT(setEnabled(bool)));
   connect(_buttonBox, SIGNAL(accepted()), this, SLOT(sAssign()));
   connect(_buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
@@ -40,12 +42,15 @@ void assignItemToPlannerCode::languageChange()
 void assignItemToPlannerCode::sAssign()
 {
   XSqlQuery assignAssign;
-  if(!_plannerCode->isValid())
-  {
-    QMessageBox::warning(this, tr("No Planner Code Selected"),
-      tr("You must select a Planner Code to assign before continuing.") );
-    return;
-  }
+  QList<GuiErrorCheck> errors;
+  errors << GuiErrorCheck(!_item->isValid(), _item,
+                          tr("You must select a valid Item before continuing.") )
+         << GuiErrorCheck(!_plannerCode->isValid(), _plannerCode,
+                          tr("You must select a Planner Code to assign before continuing.") )
+  ;
+
+  if (GuiErrorCheck::reportErrors(this, tr("Missing Information"), errors))
+      return;
 
   QString sql( "UPDATE itemsite "
                "SET itemsite_plancode_id=:plancode_id "
