@@ -32,6 +32,7 @@ public slots:
     virtual void sNameChanged(QString);
     virtual void sSave();
     virtual void populate();
+    virtual void sTogglePrefix();
 
 protected:
     QValidator *_accountValidator;
@@ -40,6 +41,7 @@ protected:
 protected slots:
     virtual void languageChange();
     virtual void sHandleTransmitGroup();
+    virtual void sHandleType();
 
 private:
     int _mode;
