@@ -41,7 +41,7 @@ BOM::BOM(QWidget* parent, const char* name, Qt::WindowFlags fl)
   connect(_showExpired, SIGNAL(toggled(bool)), this, SLOT(sFillList()));
   connect(_showFuture, SIGNAL(toggled(bool)), this, SLOT(sFillList()));
   connect(_bomitem, SIGNAL(populateMenu(QMenu*,QTreeWidgetItem*,int)), this, SLOT(sPopulateMenu(QMenu*)));
-  connect(_close, SIGNAL(clicked()), this, SLOT(sClose()));
+  connect(_close, SIGNAL(clicked()), this, SLOT(close()));
   connect(_view, SIGNAL(clicked()), this, SLOT(sView()));
   connect(_save, SIGNAL(clicked()), this, SLOT(sSave()));
   connect(_expire, SIGNAL(clicked()), this, SLOT(sExpire()));
@@ -158,6 +158,7 @@ enum SetResponse BOM::set(const ParameterList &pParams)
       _requiredQtyPer->setEnabled(false);
       _documents->setReadOnly(true);
       _save->setEnabled(false);
+      _comments->setReadOnly(true);
       
       connect(_bomitem, SIGNAL(itemSelected(int)), _view, SLOT(animateClick()));
       
@@ -660,12 +661,6 @@ void BOM::sFillList(int pItemid, bool)
   }
 }
 
-void BOM::sClose()
-{
-  if(sCheckRequiredQtyPer())
-    close();
-}
-
 bool BOM::sCheckRequiredQtyPer()
 {
   if(cView == _mode || !_doRequireQtyPer->isChecked() || _totalQtyPerCache == 0.0)
