@@ -18,6 +18,7 @@
 #include <openreports.h>
 
 #include "guiclient.h"
+#include "errorReporter.h"
 
 buyCard::buyCard(QWidget* parent, const char* name, Qt::WindowFlags fl)
     : XWidget(parent, name, fl)
@@ -124,6 +125,12 @@ void buyCard::sHandleVendor(int pVendorid)
 
 void buyCard::sHandleItemSource(int pItemsrcid)
 {
+  if (pItemsrcid < 0)
+  {
+    _poitem->clear();
+    return;
+  }
+
   XSqlQuery item;
   item.prepare( "SELECT itemsrc_vend_item_descrip, itemsrc_item_id, itemsrc_comments "
                 "FROM itemsrc "
@@ -165,9 +172,9 @@ void buyCard::sHandleItemSource(int pItemsrcid)
       return;
     XSqlQuery poitem = mql.toQuery(params);
     _poitem->populate(poitem);
-    if (poitem.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Information"),
+                                  poitem, __FILE__, __LINE__))
     {
-      systemError(this, poitem.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
