@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -24,7 +24,15 @@ class cashReceipt : public XWidget, public Ui::cashReceipt
 public:
     cashReceipt(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::Window);
     ~cashReceipt();
-    virtual bool save( bool partial );
+    Q_INVOKABLE virtual bool save( bool partial );
+    Q_INVOKABLE virtual int  id();
+    Q_INVOKABLE virtual int  ccpayId();
+    Q_INVOKABLE virtual int  setCcpayId( int ccpayId );
+    Q_INVOKABLE virtual int  mode();
+    Q_INVOKABLE virtual int  transType();
+    Q_INVOKABLE virtual bool isCcEdit();
+    Q_INVOKABLE virtual bool isPosted();
+    Q_INVOKABLE virtual bool isOverApplied();
 
 public slots:
     virtual SetResponse set(const ParameterList & pParams );
@@ -44,7 +52,7 @@ public slots:
     virtual void sMoveDown();
     virtual void sMoveUp();
     virtual void sNewCreditCard();
-    virtual void sPopulateCustomerInfo( int );
+    virtual void sPopulateCustomerInfo(int = 0);
     virtual void sSave();
     virtual void sUpdateBalance();
     virtual void sViewCreditCard();
@@ -53,19 +61,28 @@ public slots:
     virtual void sHandleAltExchRate();
     virtual void sUpdateGainLoss();
 
+    virtual ParameterList getParams();
+    virtual void grpFillApplyList();
+    virtual void updateCustomerGroup();
+    virtual bool postReceipt();
+    virtual void activateButtons(bool c=true);
+
 protected slots:
     virtual void languageChange();
     virtual void sSearchDocNumChanged();
 
+signals:
+    void populated();
 
 private:
+    int     _cashrcptid;
+    int     _ccpayid;
     int     _mode;
     int     _transType;
-    int     _cashrcptid;
     bool    _ccEdit;
-    QString _origFunds;
     bool    _overapplied;
     bool    _posted;
+    QString _origFunds;
     QDate   _mindate;
 
 };
