@@ -13,7 +13,7 @@
 
 #include <QVariant>
 #include <QValidator>
-#include <QMessageBox>
+#include "guiErrorCheck.h"
 
 /*
  *  Constructs a cashReceiptItem as a child of 'parent', with the
@@ -113,13 +113,13 @@ enum SetResponse cashReceiptItem::set(const ParameterList &pParams)
 
 void cashReceiptItem::sSave()
 {
-  if (_amountToApply->localValue() + _discountAmount->localValue() > _openAmount->localValue())
-  {
-    QMessageBox::warning( this, tr("Cannot Apply"),
-      tr("You may not apply more than the balance of this item.") );
-    _amountToApply->setFocus();
+  double epsilon = 0.005;
+  QList<GuiErrorCheck> errors;
+  errors<< GuiErrorCheck((_amountToApply->localValue() + _discountAmount->localValue()) - _openAmount->localValue() > epsilon, _amountToApply,
+                         tr("You may not apply more than the balance of this item."))
+  ;
+  if (GuiErrorCheck::reportErrors(this, tr("Cannot Apply"), errors))
     return;
-  }
 
   int sense = 1;
   if (_docType->text() == "C" || _docType->text() == "R")
@@ -244,6 +244,10 @@ void cashReceiptItem::sDiscount()
   if(newdlg.exec() != XDialog::Rejected)
   {
     _discountAmount->setLocalValue(newdlg._amount->localValue());
-    _amountToApply->setLocalValue(_openAmount->localValue() - _discountAmount->localValue());
+    if(_mode == cNew)
+      _amountToApply->setLocalValue(_openAmount->localValue() - _discountAmount->localValue());
+    else
+      _amountToApply->setLocalValue(qMin(_amountToApply->localValue(),
+                                         _openAmount->localValue() - _discountAmount->localValue()));
   }
 }
