@@ -49,9 +49,6 @@ cashReceiptsEditList::cashReceiptsEditList(QWidget* parent, const char* name, Qt
   _cashrcpt->addColumn(tr("Amount"),        _bigMoneyColumn, Qt::AlignRight, true, "cashrcpt_amount");
   _cashrcpt->addColumn(tr("Currency"),      _currencyColumn, Qt::AlignLeft,  true, "currabbr");
 
-  if (omfgThis->singleCurrency())
-      _cashrcpt->hideColumn("cashrcpt_curr_id");
-
   if (_privileges->check("PostCashReceipts"))
     connect(_cashrcpt, SIGNAL(valid(bool)), _post, SLOT(setEnabled(bool)));
     
@@ -143,13 +140,15 @@ void cashReceiptsEditList::sDelete()
     int result = cashDelete.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("deleteCashRcpt", result));
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Cash Receipt Entry"),
+                             storedProcErrorLookup("deleteCashRcpt", result),
+                             __FILE__, __LINE__);
       return;
     }
   }
-  else if (cashDelete.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Cash Receipt Entry"),
+                                cashDelete, __FILE__, __LINE__))
   {
-    systemError(this, cashDelete.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   sFillList();
@@ -160,15 +159,18 @@ void cashReceiptsEditList::sPost()
   XSqlQuery cashPost;
   bool changeDate = false;
   QDate newDate = QDate();
+  QDate seriesDate;
   
   if (_privileges->check("ChangeCashRecvPostDate"))
   {
     getGLDistDate newdlg(this, "", true);
     newdlg.sSetDefaultLit(tr("Distribution Date"));
+    newdlg.sSetSeriesLit(tr("Deposit Date"));
     if (newdlg.exec() == XDialog::Accepted)
     {
       newDate = newdlg.date();
       changeDate = (newDate.isValid());
+      seriesDate = newdlg.seriesDate();
     }
     else
       return;
@@ -178,12 +180,14 @@ void cashReceiptsEditList::sPost()
 
   XSqlQuery tx;
   tx.exec("BEGIN;");
-  cashPost.exec("SELECT fetchJournalNumber('C/R') AS journalnumber;");
+  cashPost.prepare("SELECT fetchJournalNumber('C/R', :seriesDate) AS journalnumber;");
+  cashPost.bindValue(":seriesDate", seriesDate);
+  cashPost.exec();
   if (cashPost.first())
     journalNumber = cashPost.value("journalnumber").toInt();
-  else if (cashPost.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting Cash Receipt Entry"),
+                                cashPost, __FILE__, __LINE__))
   {
-    systemError(this, cashPost.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -222,16 +226,17 @@ void cashReceiptsEditList::sPost()
       int result = cashPost.value("result").toInt();
       if (result < 0)
       {
-        systemError(this, storedProcErrorLookup("postCashReceipt", result),
-                    __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting Cash Receipt Entry"),
+                               storedProcErrorLookup("postCashReceipt", result),
+                               __FILE__, __LINE__);
         tx.exec("ROLLBACK;");
         return;
       }
     }
-    else if (cashPost.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting Cash Receipt Entry"),
+                                  cashPost, __FILE__, __LINE__))
     {
-      systemError(this, cashPost.lastError().databaseText(), __FILE__, __LINE__);
-      tx.exec("ROLLBACK;");
+      tx.exec("ROLLBACK");
       return;
     }
   }
