@@ -22,7 +22,7 @@ class changeWoQty : public XDialog, public Ui::changeWoQty
     Q_OBJECT
 
 public:
-    changeWoQty(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    changeWoQty(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~changeWoQty();
 
 public slots:
