@@ -27,6 +27,7 @@
 
 #include <metasql.h>
 #include "errorReporter.h"
+#include "guiErrorCheck.h"
 
 #define DEBUG false
 
@@ -152,11 +153,26 @@ characteristic::characteristic(QWidget* parent, const char* name, bool modal, Qt
 
   _d = new characteristicPrivate(this);
 
+  QString _charSql = "SELECT rownum AS id, char_group "
+                     " FROM ( "
+                     "   SELECT distinct char_group, "
+                     "          DENSE_RANK() OVER (ORDER BY  char_group) AS rownum "
+                     "   FROM char"
+                     "   WHERE char_group IS NOT NULL "
+                     "   ORDER BY char_group) foo;";
+  _charGroup->populate(_charSql);
+
   connect(_buttonBox, SIGNAL(accepted()), this, SLOT(sSave()));
   connect(_name, SIGNAL(editingFinished()), this, SLOT(sCheck()));
   connect(_new, SIGNAL(clicked()), this, SLOT(sNew()));
   connect(_charoptView, SIGNAL(clicked(QModelIndex)), this, SLOT(sCharoptClicked(QModelIndex)));
   connect(_delete, SIGNAL(clicked()), this, SLOT(sDelete()));
+
+  _validator->append(0, "[Y|N]");
+  _validator->append(1, "\\S+");
+  _validator->append(2, "[1-9]\\d{0,3}");
+  _validator->append(3, "[A-Z]\\d{5}[1-9]");
+  _validator->append(4, "(https?:\\/\\/(?:www\\.|(?!www))[^\\s\\.]+\\.[^\\s]{2,}|www\\.[^\\s]+\\.[^\\s]{2,})");
 }
 
 characteristic::~characteristic()
@@ -208,7 +224,9 @@ enum SetResponse characteristic::set(const ParameterList &pParams)
     {
       _d->setMode(cView);
       _name->setEnabled(false);
+      _charGroup->setEnabled(false);
       _search->setEnabled(false);
+      _unique->setEnabled(false);
       _useGroup->setEnabled(false);
       _order->setEnabled(false);
       _mask->setEnabled(false);
@@ -227,16 +245,15 @@ void characteristic::sSave()
 // TODO: verify that _mask      applies to all existing charass
 // TODO: verify that _validator applies to all existing charass
   XSqlQuery characteristicSave;
-  if (_name->text().trimmed().isEmpty())
-  {
-    QMessageBox::critical(this, tr("Missing Name"),
-			  tr("<p>You must name this Characteristic before "
-			     "saving it."));
-    _name->setFocus();
-    return;
-  }
 
-  bool allClear = true;
+  QList<GuiErrorCheck>errors;
+  errors<<GuiErrorCheck(_name->text().trimmed().isEmpty(), _name,
+                        tr("<p>You must name this Characteristic before saving it."));
+
+  if(GuiErrorCheck::reportErrors(this,tr("Unable To Save Characteristic"),errors))
+      return;
+
+   bool allClear = true;
   foreach (QCheckBox *cb, _d->checkboxMap.values())
   {
     if (cb && cb->isChecked())
@@ -272,11 +289,11 @@ void characteristic::sSave()
     characteristicSave.prepare( "INSERT INTO char "
                "( char_id, char_name, char_options, char_attributes,"
                "  char_notes, char_mask, char_validator, char_type, "
-               "  char_order, char_search ) "
+               "  char_order, char_search, char_unique, char_group ) "
                "VALUES "
                "( :char_id, :char_name, :char_options, :char_attributes,"
                "  :char_notes, :char_mask, :char_validator, :char_type, "
-               "  :char_order, :char_search );" );
+               "  :char_order, :char_search, :char_unique, :char_group );" );
 
     characteristicSave.bindValue(":char_type", _type->currentIndex());
   }
@@ -289,11 +306,14 @@ void characteristic::sSave()
                "    char_mask=:char_mask,"
                "    char_validator=:char_validator, "
                "    char_order=:char_order, "
-               "    char_search=:char_search "
+               "    char_search=:char_search, "
+               "    char_unique=:char_unique, "
+               "    char_group=:char_group "
                "WHERE (char_id=:char_id);" );
 
   characteristicSave.bindValue(":char_id", _d->charid);
   characteristicSave.bindValue(":char_name", _name->text());
+  characteristicSave.bindValue(":char_group", _charGroup->currentText());
 
   characteristicSave.bindValue(":char_options",     QVariant(false));
   characteristicSave.bindValue(":char_attributes",  QVariant(false));
@@ -305,6 +325,7 @@ void characteristic::sSave()
     characteristicSave.bindValue(":char_validator",   _validator->currentText());
   characteristicSave.bindValue(":char_order", _order->value());
   characteristicSave.bindValue(":char_search", QVariant(_search->isChecked()));
+  characteristicSave.bindValue(":char_unique", QVariant(_unique->isChecked()));
   characteristicSave.exec();
   if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Characteristic"),
                            characteristicSave, __FILE__, __LINE__))
@@ -359,6 +380,11 @@ void characteristic::populate()
   if (charq.first())
   {
     _name->setText(charq.value("char_name").toString());
+#if QT_VERSION >= 0x050000
+    _charGroup->setCurrentText(charq.value("char_group").toString());
+#else
+    _charGroup->setCode(charq.value("char_group").toString());
+#endif
     _description->setText(charq.value("char_notes").toString());
     _mask->setText(charq.value("char_mask").toString());
     _validator->setText(charq.value("char_validator").toString());
@@ -366,6 +392,7 @@ void characteristic::populate()
     _type->setEnabled(false);
     _order->setValue(charq.value("char_order").toInt());
     _search->setChecked(charq.value("char_search").toBool());
+    _unique->setChecked(charq.value("char_unique").toBool());
   }
   else if (ErrorReporter::error(QtCriticalMsg, this,
                            tr("Error Getting Characteristic"),
@@ -445,9 +472,9 @@ void characteristic::sDelete()
     QMessageBox::critical(this, tr("Error"), tr("This value is in use and can not be deleted."));
     return;
   }
-  else if (qry.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Characteristic Option"),
+                                qry, __FILE__, __LINE__))
   {
-    systemError(this, qry.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
