@@ -19,34 +19,17 @@
 
 #include "ui_characteristic.h"
 
+class characteristicPrivate;
+
 class characteristic : public XDialog, public Ui::characteristic
 {
     Q_OBJECT
 
 public:
-    characteristic(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    characteristic(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~characteristic();
 
-    enum Type { Text, List, Date };
-
-    enum Use { Addresses     = 0x00000001,
-               Contacts      = 0x00000002,
-               CRMAccounts   = 0x00000004,
-               Customers     = 0x00000008,
-               Employees     = 0x00000010,
-               Incidents     = 0x00000020,
-               Items         = 0x00000040,
-               LotSerial     = 0x00000080,
-               Opportunities = 0x00000100,
-               Quotes         = 0x00000200,
-               SalesOrders    = 0x00000400,
-               Invoices       = 0x00000800,
-               Vendors        = 0x00001000,
-               PurchaseOrders = 0x00002000,
-               Vouchers       = 0x00004000,
-               Projects       = 0x00008000,
-    	       Tasks  	      = 0x00010000	
-             };
+    enum Type { Text, List, Date, Number };
 
 public slots:
     virtual enum SetResponse set(const ParameterList & pParams );
@@ -64,9 +47,7 @@ protected slots:
 
 
 private:
-    int _mode;
-    int _charid;
-    QSqlTableModel *_charoptModel;
+    characteristicPrivate *_d;
 };
 
 #endif // CHARACTERISTIC_H
