@@ -31,6 +31,7 @@ check::check(QWidget* parent, const char* name, Qt::WindowFlags fl)
   _items->addColumn(tr("A/P Doc."),   _orderColumn, Qt::AlignRight, true, "apopen_docnumber");
   _items->addColumn(tr("A/R Doc."),   _orderColumn, Qt::AlignRight, true, "aropen_docnumber");
   _items->addColumn(tr("Doc. Date"),   _dateColumn, Qt::AlignCenter,true, "checkitem_docdate");
+  _items->addColumn(tr("Amount (pre-discount)"),     _moneyColumn, Qt::AlignRight, true, "checkitem_amount_prediscount");
   _items->addColumn(tr("Amount"),     _moneyColumn, Qt::AlignRight, true, "checkitem_amount");
   _items->addColumn(tr("Discount"),    _itemColumn, Qt::AlignRight, true, "checkitem_discount");
   _items->addColumn(tr("Amount (in %1)").arg(CurrDisplay::baseCurrAbbr()),
@@ -71,13 +72,20 @@ void check::sPopulate()
 {
   XSqlQuery checkPopulate;
   MetaSQLQuery mql("SELECT checkhead.*,"
-                   "       checkrecip_number, checkrecip_name, checkrecip_type"
+                   "       checkrecip_number, checkrecip_name, "
+                   "       CASE checkrecip_type WHEN 'V' THEN <? value('vendor') ?>"
+                   "                            WHEN 'C' THEN <? value('customer') ?>"
+                   "                            WHEN 'T' THEN <? value('taxauth') ?>"
+                   "       END AS checkrecip_type"
                    "  FROM checkhead"
                    "  JOIN checkrecip ON (checkhead_recip_id=checkrecip_id"
                    "                 AND checkhead_recip_type=checkrecip_type)"
                    " WHERE (checkhead_id=<? value('checkid')?>);");
   ParameterList params;
   params.append("checkid", _checkid);
+  params.append("vendor", tr("Vendor"));
+  params.append("customer", tr("Customer"));
+  params.append("taxauth", tr("Tax Authority"));
   checkPopulate = mql.toQuery(params);
   if (checkPopulate.first())
   {
@@ -141,6 +149,7 @@ void check::sFillList()
                      "       checkitem_vouchernumber, checkitem_invcnumber,"
                      "       checkitem_cmnumber, checkitem_ranumber,"
                      "       checkitem_docdate, checkitem_amount, checkitem_discount,"
+                     "       (checkitem_amount + checkitem_discount) as checkitem_amount_prediscount,"
                      "       apopen_docnumber, aropen_docnumber,"
                      "       currToBase(<? value('curr_id') ?>, checkitem_amount,"
                      "                  checkitem_docdate) AS baseamount,"
@@ -162,4 +171,4 @@ void check::sFillList()
       return;
     }
   }
-}
+}
\ No newline at end of file
