@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -18,6 +18,7 @@
 
 #include "guiclient.h"
 #include "mqlutil.h"
+#include "errorReporter.h"
 
 configureCRM::configureCRM(QWidget* parent, const char* name, bool /*modal*/, Qt::WindowFlags fl)
     : XAbstractConfigure(parent, fl)
@@ -128,6 +129,9 @@ bool configureCRM::sSave()
   configq.bindValue(":innumber", _nextInNumber->text().toInt());
   configq.exec();
   
+  if (QString(numberGenerationTypes[_acctGeneration->currentIndex()]) != "M" && _nextAcctNumber->text().toInt() < 1)
+    _nextAcctNumber->setText("1");
+
   configq.prepare( "SELECT setNextCRMAccountNumber(:acnumber);" );
   configq.bindValue(":acnumber", _nextAcctNumber->text().toInt());
   configq.exec();
@@ -227,9 +231,9 @@ void configureCRM::sStrictCountryChanged(bool p)
         return;
       }
     }
-    else if (activeq.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Confirming Country Information"),
+                                  activeq, __FILE__, __LINE__))
     {
-      systemError(this, activeq.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
 
@@ -260,9 +264,9 @@ void configureCRM::sStrictCountryChanged(bool p)
         }
       }
     }
-    else if (allq.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Confirming Country Information"),
+                                  allq, __FILE__, __LINE__))
     {
-      systemError(this, allq.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
