@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2018 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -12,11 +12,11 @@
 
 #include <QMessageBox>
 #include <QVariant>
+#include <QCloseEvent>
 
 #include <metasql.h>
 #include <mqlutil.h>
 
-#include "characteristicAssignment.h"
 #include "contactcluster.h"
 #include "crmaccount.h"
 #include "customer.h"
@@ -38,6 +38,7 @@
 #include "warehouse.h"
 #include "xsqlquery.h"
 #include <time.h>
+#include "guiErrorCheck.h"
 
 struct privSet {
   bool canEdit;
@@ -53,6 +54,7 @@ class contactPrivate
     {
       _activeCache = false;
       _mode        = cView;
+      _close       = false;
     };
 
     ~contactPrivate()
@@ -61,6 +63,8 @@ class contactPrivate
 
     bool     _activeCache;
     int      _mode;
+    bool     _close;
+    AppLock  _lock;
     contact *_parent;
 
     struct privSet rowPrivs(XTreeWidgetItem *row)
@@ -200,7 +204,7 @@ class contactPrivate
     };
 };
 
-contact::contact(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)
+contact::contact(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
 {
   setupUi(this);
@@ -214,11 +218,8 @@ contact::contact(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)
   connect(_buttonBox,            SIGNAL(accepted()), this, SLOT(sSave()));
   connect(_buttonBox,            SIGNAL(rejected()), this, SLOT(sClose()));
   connect(_crmAccount,           SIGNAL(newId(int)), _contact, SLOT(setSearchAcct(int)));
-  connect(_deleteCharacteristic, SIGNAL(clicked()), this, SLOT(sDeleteCharass()));
   connect(_detachUse,            SIGNAL(clicked()), this, SLOT(sDetachUse()));
-  connect(_editCharacteristic,   SIGNAL(clicked()), this, SLOT(sEditCharass()));
   connect(_editUse,              SIGNAL(clicked()), this, SLOT(sEditUse()));
-  connect(_newCharacteristic,    SIGNAL(clicked()), this, SLOT(sNewCharass()));
   connect(_showOrders,       SIGNAL(toggled(bool)), this, SLOT(sFillList()));
   connect(_uses,               SIGNAL(valid(bool)), this, SLOT(sHandleValidUse(bool)));
   connect(_uses, SIGNAL(populateMenu(QMenu*, XTreeWidgetItem*)), this, SLOT(sPopulateUsesMenu(QMenu*)));
@@ -234,8 +235,7 @@ contact::contact(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)
   connect(omfgThis,                SIGNAL(vendorsUpdated()), this, SLOT(sFillList()));
   connect(omfgThis,             SIGNAL(warehousesUpdated()), this, SLOT(sFillList()));
 
-  _charass->addColumn(tr("Characteristic"), _itemColumn, Qt::AlignLeft, true, "char_name");
-  _charass->addColumn(tr("Value"),          -1,          Qt::AlignLeft, true, "charass_value");
+  _charass->setType("CNTCT");
 
   _uses->addColumn(tr("Used by"),         100, Qt::AlignLeft, true, "type");
   _uses->addColumn(tr("Number"), _orderColumn, Qt::AlignLeft, true, "number");
@@ -255,6 +255,7 @@ contact::contact(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)
   _owner->setType(UsernameLineEdit::UsersActive);
   _owner->setEnabled(_privileges->check("EditOwner"));
 
+  _cntctid = -1;
 }
 
 contact::~contact()
@@ -273,12 +274,53 @@ enum SetResponse contact::set(const ParameterList &pParams)
   QVariant param;
   bool     valid;
 
+  param = pParams.value("mode", &valid);
+  if (valid)
+  {
+    if (param.toString() == "new")
+    {
+      _data->_mode = cNew;
+      XSqlQuery getq;
+      getq.exec("SELECT fetchNextNumber('ContactNumber') AS result;");
+      getq.first();
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting Number"),
+                               getq, __FILE__, __LINE__))
+        return UndefinedError;
+      _number->setText(getq.value("result").toString());
+      _contact->setNumber(_number->text());
+      _contact->setFirst("Contact" + QDateTime::currentDateTime().toString());
+      int cntctSaveResult = _contact->save(AddressCluster::CHANGEONE);
+      if (cntctSaveResult < 0)
+      {
+        ErrorReporter::error(QtCriticalMsg, this, tr("Saving Placeholder"),
+                             tr("<p>There was an error creating a new contact (%). "
+                                "Check the database server log for errors.")
+                             .arg(cntctSaveResult), __FILE__, __LINE__);
+        return UndefinedError;
+      }
+      _comments->setId(_contact->id());
+      _documents->setId(_contact->id());
+      _charass->setId(_contact->id());
+      _contact->setFirst("");
+      _contact->setOwnerUsername(omfgThis->username());
+      _tabWidget->setTabEnabled(_tabWidget->indexOf(_usesTab), false);
+    }
+    else if (param.toString() == "edit")
+    {
+      _data->_mode = cEdit;
+    }
+    else if (param.toString() == "view")
+      setViewMode();
+  }
+
   param = pParams.value("cntct_id", &valid);
   if (valid)
   {
     _contact->setId(param.toInt());
     _comments->setId(_contact->id());
     _documents->setId(_contact->id());
+    _charass->setId(_contact->id());
+    _cntctid = _contact->id();
     sPopulate();
   }
 
@@ -314,64 +356,22 @@ enum SetResponse contact::set(const ParameterList &pParams)
   if (valid)
     _contact->addressWidget()->setCountry(param.toString());
 
-  param = pParams.value("mode", &valid);
-  if (valid)
-  {
-    if (param.toString() == "new")
-    {
-      _data->_mode = cNew;
-      XSqlQuery getq;
-      getq.exec("SELECT fetchNextNumber('ContactNumber') AS result;");
-      getq.first();
-      if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting Number"),
-                               getq, __FILE__, __LINE__))
-        return UndefinedError;
-      _number->setText(getq.value("result").toString());
-      _contact->setNumber(_number->text());
-      _contact->setFirst("Contact" + QDateTime::currentDateTime().toString());
-      int cntctSaveResult = _contact->save(AddressCluster::CHANGEONE);
-      if (cntctSaveResult < 0)
-      {
-        ErrorReporter::error(QtCriticalMsg, this, tr("Saving Placeholder"),
-                             tr("<p>There was an error creating a new contact (%). "
-                                "Check the database server log for errors.")
-                             .arg(cntctSaveResult), __FILE__, __LINE__);
-        return UndefinedError;
-      }
-      _comments->setId(_contact->id());
-      _documents->setId(_contact->id());
-      _contact->setFirst("");
-      connect(_charass, SIGNAL(valid(bool)), _editCharacteristic, SLOT(setEnabled(bool)));
-      connect(_charass, SIGNAL(valid(bool)), _deleteCharacteristic, SLOT(setEnabled(bool)));
-      _contact->setOwnerUsername(omfgThis->username());
-    }
-    else if (param.toString() == "edit")
-    {
-      _data->_mode = cEdit;
-
-      connect(_charass, SIGNAL(valid(bool)), _editCharacteristic, SLOT(setEnabled(bool)));
-      connect(_charass, SIGNAL(valid(bool)), _deleteCharacteristic, SLOT(setEnabled(bool)));
-    }
-    else if (param.toString() == "view")
-    {
-      _data->_mode = cView;
-
-      _buttonBox->setStandardButtons(QDialogButtonBox::Close);
-
-      _contact->setEnabled(FALSE);
-      _notes->setEnabled(FALSE);
-      _comments->setReadOnly(true);
-      _documents->setReadOnly(true);
-      _newCharacteristic->setEnabled(FALSE);
-      _editCharacteristic->setEnabled(FALSE);
-      _deleteCharacteristic->setEnabled(FALSE);
-      _charass->setEnabled(FALSE);
-    }
-  }
-
   return NoError;
 }
 
+void contact::setViewMode()
+{
+  _data->_mode = cView;
+
+  _buttonBox->setStandardButtons(QDialogButtonBox::Close);
+
+  _contact->setEnabled(false);
+  _notes->setEnabled(false);
+  _comments->setReadOnly(true);
+  _documents->setReadOnly(true);
+  _charass->setReadOnly(true);
+}
+
 void contact::sPopulateUsesMenu(QMenu* pMenu)
 {
   QAction *editAction = 0;
@@ -497,14 +497,22 @@ void contact::sClose()
   reject();
 }
 
+void contact::closeEvent(QCloseEvent *pEvent)
+{
+  if(_data->_mode == cNew && _cntctid == -1)
+    sClose();
+
+  XDialog::closeEvent(pEvent);
+}
+
 void contact::sSave()
 {
-  if (_contact->first().isEmpty() && _contact->last().isEmpty())
-  {
-    QMessageBox::information(this, tr("Contact Blank"),
-                             tr("<p>You must fill in a contact first or last name as a minimum before saving."));
-    return;
-  }
+  QList<GuiErrorCheck> errors;
+    errors<< GuiErrorCheck(_contact->first().isEmpty() && _contact->last().isEmpty(), _contact,
+                           tr("You must fill in a contact first or last name as a minimum before saving."))
+    ;
+    if (GuiErrorCheck::reportErrors(this, tr("Cannot Save Contact"), errors))
+      return;
 
   if (_data->_activeCache && ! _contact->active())
   {
@@ -543,21 +551,10 @@ void contact::sSave()
   int saveResult = addr->save(AddressCluster::CHECK);
   if (-2 == saveResult)
   {
-    int answer = 2;     // Cancel
-    answer = QMessageBox::question(this, tr("Question Saving Address"),
-                    tr("<p>There are multiple Contacts sharing this Address. "
-                       "What would you like to do?"),
-                    tr("Change This One"),
-                    tr("Change Address for All"),
-                    tr("Cancel"),
-                    2, 2);
-
-    if (0 == answer)
-      saveResult = addr->save(AddressCluster::CHANGEONE);
-    else if (1 == answer)
-      saveResult = addr->save(AddressCluster::CHANGEALL);
-    else
+    AddressCluster::SaveFlags answer = AddressCluster::askForSaveMode(addr->id());
+    if (answer == AddressCluster::CHECK)
       return;
+    saveResult = addr->save(answer);
   }
   if (saveResult < 0)   // check from errors for CHECK and CHANGE* saves
   {
@@ -583,12 +580,49 @@ void contact::sSave()
                          .arg(saveResult), __FILE__, __LINE__);
     return;
   }
-
+  _cntctid = _contact->id();
   done(_contact->id());
 }
 
 void contact::sPopulate()
 {
+  if (_data->_mode == cEdit && !_data->_lock.acquire("cntct", _cntctid, AppLock::Interactive))
+    setViewMode();
+
+  _data->_close = false;
+
+  foreach (QWidget* widget, QApplication::allWidgets())
+  {
+    if (!widget->isWindow() || !widget->isVisible())
+      continue;
+
+    contact *w = qobject_cast<contact*>(widget);
+
+    if (w && w != this && w->id()==_cntctid)
+    {
+      // detect "i'm my own grandpa"
+      QObject *p;
+      for (p = parent(); p && p != w ; p = p->parent())
+        ; // do nothing
+      if (p == w)
+      {
+        QMessageBox::warning(this, tr("Cannot Open Recursively"),
+                             tr("This contact is already open and cannot be "
+                                "raised. Please close windows to get to it."));
+        _data->_close = true;
+      } else if (p) {
+        w->setFocus();
+        if (omfgThis->showTopLevel())
+        {
+          w->raise();
+          w->activateWindow();
+        }
+        _data->_close = true;
+      }
+      break;
+    }
+  }
+
   _number->setText(_contact->number());
   _active->setChecked(_contact->active());
   _crmAccount->setId(_contact->crmAcctId());
@@ -598,65 +632,9 @@ void contact::sPopulate()
   sFillList();
 }
 
-void contact::sNewCharass()
-{
-  ParameterList params;
-  params.append("mode", "new");
-  params.append("cntct_id", _contact->id());
-
-  characteristicAssignment newdlg(this, "", TRUE);
-  newdlg.set(params);
-
-  if (newdlg.exec() != XDialog::Rejected)
-    sFillList();
-}
-
-void contact::sEditCharass()
-{
-  ParameterList params;
-  params.append("mode", "edit");
-  params.append("charass_id", _charass->id());
-
-  characteristicAssignment newdlg(this, "", TRUE);
-  newdlg.set(params);
-
-  if (newdlg.exec() != XDialog::Rejected)
-    sFillList();
-}
-
-void contact::sDeleteCharass()
-{
-  XSqlQuery delq;
-  delq.prepare("DELETE FROM charass WHERE (charass_id=:charass_id);");
-  delq.bindValue(":charass_id", _charass->id());
-  delq.exec();
-  if (ErrorReporter::error(QtCriticalMsg, this, tr("Deleting Characteristic"),
-                          delq, __FILE__, __LINE__))
-    return;
-
-  sFillList();
-}
-
 void contact::sFillList()
 {
-  XSqlQuery charq;
-  charq.prepare( "SELECT charass_id, char_name, "
-             " CASE WHEN char_type < 2 THEN "
-             "   charass_value "
-             " ELSE "
-             "   formatDate(charass_value::date) "
-             "END AS charass_value "
-             "FROM charass, char "
-             "WHERE ((charass_target_type='CNTCT')"
-             " AND   (charass_char_id=char_id)"
-             " AND   (charass_target_id=:cntct_id) ) "
-             "ORDER BY char_order, char_name;" );
-  charq.bindValue(":cntct_id", _contact->id());
-  charq.exec();
-  _charass->populate(charq);
-  if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting Characteristics"),
-                           charq, __FILE__, __LINE__))
-    return;
+  _charass->setId(_contact->id());
 
   QString errmsg;
   bool    ok = false;
@@ -1309,3 +1287,25 @@ void contact::sViewWarehouse()
   newdlg.set(params);
   newdlg.exec();
 }
+
+int contact::id()
+{
+  return _cntctid;
+}
+
+void contact::setVisible(bool visible)
+{
+  if (_data->_close)
+    close();
+  else
+    XDialog::setVisible(visible);
+}
+
+void contact::done(int result)
+{
+  if (!_data->_lock.release())
+    ErrorReporter::error(QtCriticalMsg, this, tr("Locking Error"),
+                         _data->_lock.lastError(), __FILE__, __LINE__);
+
+  XDialog::done(result);
+}
