@@ -20,6 +20,7 @@
 #include "copyContract.h"
 #include "guiclient.h"
 #include "parameterwidget.h"
+#include "errorReporter.h"
 
 contracts::contracts(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "contracts", fl)
@@ -164,9 +165,9 @@ void contracts::sDelete()
                           "WHERE (itemsrc_contrct_id=:contrct_id);");
       itemDelete.bindValue(":contrct_id", list()->id());
       itemDelete.exec();
-      if (itemDelete.lastError().type() != QSqlError::NoError)
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Contract"),
+                                    itemDelete, __FILE__, __LINE__))
       {
-        systemError(this, itemDelete.lastError().databaseText(), __FILE__, __LINE__);
         return;
       }
     }
@@ -176,9 +177,9 @@ void contracts::sDelete()
                           "WHERE (itemsrc_contrct_id=:contrct_id);");
       itemDelete.bindValue(":contrct_id", list()->id());
       itemDelete.exec();
-      if (itemDelete.lastError().type() != QSqlError::NoError)
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Contract"),
+                                    itemDelete, __FILE__, __LINE__))
       {
-        systemError(this, itemDelete.lastError().databaseText(), __FILE__, __LINE__);
         return;
       }
     }
@@ -187,9 +188,9 @@ void contracts::sDelete()
                         "WHERE (contrct_id=:contrct_id);");
     itemDelete.bindValue(":contrct_id", list()->id());
     itemDelete.exec();
-    if (itemDelete.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Contract"),
+                                  itemDelete, __FILE__, __LINE__))
     {
-      systemError(this, itemDelete.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
 
