@@ -22,11 +22,12 @@ class correctProductionPosting : public XDialog, public Ui::correctProductionPos
     Q_OBJECT
 
 public:
-    correctProductionPosting(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    correctProductionPosting(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~correctProductionPosting();
 
     Q_INVOKABLE virtual bool captive() const { return _captive; }
     Q_INVOKABLE virtual bool okToPost();
+    Q_INVOKABLE int handleSeriesAdjustBeforePost();
 
 public slots:
     virtual enum SetResponse set(const ParameterList & pParams );
