@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -29,6 +29,9 @@ createRecurringItems::createRecurringItems(QWidget* parent, const char* name, Qt
   connect(_incidents, SIGNAL(toggled(bool)), this, SLOT(sHandleButtons()));
   connect(_projects,  SIGNAL(toggled(bool)), this, SLOT(sHandleButtons()));
   connect(_todoItems, SIGNAL(toggled(bool)), this, SLOT(sHandleButtons()));
+
+  sHandleButtons();
+  _submit->setVisible(_metrics->boolean("EnableBatchManager"));
 }
 
 createRecurringItems::~createRecurringItems()
@@ -145,7 +148,7 @@ void createRecurringItems::sSubmit()
       submitAction newdlg(this, "", true);
       newdlg.set(params);
 
-      if(! newdlg.exec() == XDialog::Accepted)
+      if(newdlg.exec() != XDialog::Accepted)
         errors.append("Submit cancelled");
       else
         count += 1;
