@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2018 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -31,7 +31,6 @@
 #include "taxAuthority.h"
 #include "user.h"
 #include "vendor.h"
-#include "vendorWorkBench.h"
 
 #define DEBUG false
 
@@ -120,6 +119,7 @@ crmaccount::crmaccount(QWidget* parent, const char* name, Qt::WindowFlags fl)
   _taxauthId    = -1;
   _username     = "";
   _vendId       = -1;
+  _closed       = false;
   _comments->setId(-1);
   _documents->setId(-1);
   _charass->setId(-1);
@@ -184,7 +184,7 @@ enum SetResponse crmaccount::set(const ParameterList &pParams)
       XSqlQuery insq;
       insq.prepare("INSERT INTO crmacct(crmacct_number, crmacct_name,"
                    "                    crmacct_active, crmacct_type)"
-                   "  SELECT CASE WHEN :number > 1 THEN "
+                   "  SELECT CASE WHEN :number >= 1 THEN "
                    "           CAST(:number AS TEXT) "
                    "         ELSE 'TEMPORARY' || (last_value + 1) "
                    "         END, "
@@ -237,6 +237,40 @@ enum SetResponse crmaccount::set(const ParameterList &pParams)
     setViewMode();
   }
 
+  _closed = false;
+
+  foreach (QWidget* widget, QApplication::allWidgets())
+  {
+    if (!widget->isWindow() || !widget->isVisible())
+      continue;
+
+    crmaccount *w = qobject_cast<crmaccount*>(widget);
+
+    if (w && w != this && w->id()==_crmacctId)
+    {
+      // detect "i'm my own grandpa"
+      QObject *p;
+      for (p = parent(); p && p != w ; p = p->parent())
+        ; // do nothing
+      if (p == w)
+      {
+        QMessageBox::warning(this, tr("Cannot Open Recursively"),
+                             tr("This account is already open and cannot be "
+                                "raised. Please close windows to get to it."));
+        _closed = true;
+      } else if (p) {
+        w->setFocus();
+        if (omfgThis->showTopLevel())
+        {
+          w->raise();
+          w->activateWindow();
+        }
+        _closed = true;
+      }
+      break;
+    }
+  }
+
   sHandleChildButtons();
 
   return NoError;
@@ -317,6 +351,21 @@ void crmaccount::sClose()
       return;
     }
 
+    if (_metrics->boolean("LotSerialControl"))
+    {
+      XSqlQuery deleteReg;
+      deleteReg.prepare("DELETE FROM lsreg "
+                        "      WHERE lsreg_crmacct_id=:crmacct_id;");
+      deleteReg.bindValue(":crmacct_id", _crmacctId);
+      deleteReg.exec();
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Error deleting Registrations from Account"),
+                               deleteReg, __FILE__, __LINE__))
+      {
+        rollback.exec();
+        return;
+      }
+    }
+
     // TODO: handle _username
     // can't delete in alpha order because of dependencies
     struct {
@@ -517,6 +566,14 @@ void crmaccount::sSave()
     return;
   }
 
+  disconnect(omfgThis, SIGNAL(customersUpdated(int, bool)), this, SLOT(sUpdateRelationships()));
+  disconnect(omfgThis, SIGNAL(employeeUpdated(int)), this, SLOT(sUpdateRelationships()));
+  disconnect(omfgThis, SIGNAL(prospectsUpdated()),   this, SLOT(sUpdateRelationships()));
+  disconnect(omfgThis, SIGNAL(salesRepUpdated(int)), this, SLOT(sUpdateRelationships()));
+  disconnect(omfgThis, SIGNAL(taxAuthsUpdated(int)), this, SLOT(sUpdateRelationships()));
+  disconnect(omfgThis, SIGNAL(vendorsUpdated()),     this, SLOT(sUpdateRelationships()));
+  disconnect(omfgThis, SIGNAL(userUpdated(QString)), this, SLOT(sUpdateRelationships()));
+
   omfgThis->sCrmAccountsUpdated(_crmacctId);
   omfgThis->sCustomersUpdated(-1, true);
   omfgThis->sEmployeeUpdated(-1);
@@ -977,16 +1034,6 @@ void crmaccount::doDialog(QWidget *parent, const ParameterList & pParams)
   omfgThis->handleNewWindow(ci);
 }
 
-void crmaccount::sVendorInfo()
-{
-  ParameterList params;
-  params.append("vend_id", _vendId);
-
-  vendorWorkBench *newdlg = new vendorWorkBench(this, "vendorWorkBench", Qt::Window);
-  newdlg->set(params);
-  omfgThis->handleNewWindow(newdlg);
-}
-
 void crmaccount::sCustomerToggled()
 {
   if (_customer->isChecked())
@@ -1236,8 +1283,6 @@ void crmaccount::sHandleChildButtons()
     menuItem=vendorMenu->addAction(tr("View..."), this, SLOT(sViewVendor()));
     menuItem->setEnabled(_privileges->check("ViewVendors") ||
                          _privileges->check("MaintainVendors"));
-    menuItem=vendorMenu->addAction(tr("Workbench..."), this, SLOT(sVendorInfo()));
-    menuItem->setEnabled(_privileges->check("MaintainVendors"));
     _vendorButton->setMenu(vendorMenu);
   }
   else
@@ -1344,3 +1389,11 @@ void crmaccount::sUser()
   newdlg->set(params);
   omfgThis->handleNewWindow(newdlg);
 }
+
+void crmaccount::setVisible(bool visible)
+{
+  if (_closed)
+    close();
+  else
+    XWidget::setVisible(visible);
+}
