@@ -68,7 +68,8 @@ crmaccounts::crmaccounts(QWidget* parent, const char*, Qt::WindowFlags fl)
   parameterWidget()->append(tr("State Pattern"), "addr_state_pattern", ParameterWidget::Text);
   parameterWidget()->append(tr("Postal Code Pattern"), "addr_postalcode_pattern", ParameterWidget::Text);
   parameterWidget()->append(tr("Country Pattern"), "addr_country_pattern", ParameterWidget::Text);
-  parameterWidget()->applyDefaultFilterSet();
+  parameterWidget()->append(tr("Create Date on or After"), "startCreateDate", ParameterWidget::Date);
+  parameterWidget()->append(tr("Create Date on or Before"),   "endCreateDate",   ParameterWidget::Date);
 
   connect(omfgThis, SIGNAL(crmAccountsUpdated(int)),     this, SLOT(sFillList()));
   connect(omfgThis, SIGNAL(customersUpdated(int, bool)), this, SLOT(sFillList()));
@@ -101,9 +102,11 @@ crmaccounts::crmaccounts(QWidget* parent, const char*, Qt::WindowFlags fl)
   list()->addColumn(tr("User"),           70, Qt::AlignCenter, false, "usr");
   list()->addColumn(tr("Employee"),       70, Qt::AlignCenter, false, "emp");
   list()->addColumn(tr("Sales Rep"),      70, Qt::AlignCenter, false, "salesrep");
+  list()->addColumn(tr("Create Date"), _dateColumn, Qt::AlignLeft, false, "crmacct_created" );
+
+  list()->setSelectionMode(QAbstractItemView::ExtendedSelection);
 
   setupCharacteristics("CRMACCT");
-  parameterWidget()->applyDefaultFilterSet();
 
   connect(list(), SIGNAL(itemSelected(int)), this, SLOT(sOpen()));
 
@@ -123,42 +126,53 @@ void crmaccounts::sNew()
 
 void crmaccounts::sView()
 {
-  ParameterList params;
-  params.append("mode", "view");
-  params.append("crmacct_id", list()->id());
+  foreach (XTreeWidgetItem *item, list()->selectedItems())
+  {
+    ParameterList params;
+    params.append("mode", "view");
+    params.append("crmacct_id", item->id());
 
-  crmaccount* newdlg = new crmaccount();
-  newdlg->set(params);
-  omfgThis->handleNewWindow(newdlg);
+    crmaccount* newdlg = new crmaccount();
+    newdlg->set(params);
+    omfgThis->handleNewWindow(newdlg);
+  }
 }
 
 void crmaccounts::sDelete()
 {
   if (QMessageBox::question(this, tr("Delete?"),
-                            tr("Are you sure you want to delete this Account?"),
+                            tr("Are you sure you want to delete these Account(s)?"),
                             QMessageBox::Yes,
                             QMessageBox::No | QMessageBox::Default) == QMessageBox::No)
     return;
 
   XSqlQuery delq;
   delq.prepare("DELETE FROM crmacct WHERE crmacct_id = :crmacct_id;");
-  delq.bindValue(":crmacct_id", list()->id());
-  delq.exec();
-  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error deleting Account"),
-                           delq, __FILE__, __LINE__))
-    return;
+
+  foreach (XTreeWidgetItem *item, list()->selectedItems())
+  {
+    delq.bindValue(":crmacct_id", item->id());
+    delq.exec();
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error deleting Account"),
+                             delq, __FILE__, __LINE__))
+      return;
+  }
+
   sFillList();
 }
 
 void crmaccounts::sEdit()
 {
-  ParameterList params;
-  params.append("mode", "edit");
-  params.append("crmacct_id", list()->id());
+  foreach (XTreeWidgetItem *item, list()->selectedItems())
+  {
+    ParameterList params;
+    params.append("mode", "edit");
+    params.append("crmacct_id", item->id());
 
-  crmaccount* newdlg = new crmaccount();
-  newdlg->set(params);
-  omfgThis->handleNewWindow(newdlg);
+    crmaccount* newdlg = new crmaccount();
+    newdlg->set(params);
+    omfgThis->handleNewWindow(newdlg);
+  }
 }
 
 void crmaccounts::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem *, int)
