@@ -16,6 +16,7 @@
 
 #include "customCommand.h"
 #include "guiclient.h"
+#include "errorReporter.h"
 
 customCommands::customCommands(QWidget* parent, const char* name, Qt::WindowFlags fl)
     : XWidget(parent, name, fl)
@@ -79,9 +80,9 @@ void customCommands::sDelete()
   customDelete.bindValue(":cmd_id", _commands->id());
   if(customDelete.exec())
     sFillList();
-  else if (customDelete.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Custom Command"),
+                                customDelete, __FILE__, __LINE__))
   {
-    systemError(this, customDelete.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -100,9 +101,9 @@ void customCommands::sFillList()
          "        'Manufacture','CRM','Sales','Accounting','System'))) "
          " ORDER BY cmd_module, cmd_title;");
   _commands->populate(customFillList);
-  if (customFillList.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Custom Command Information"),
+                                customFillList, __FILE__, __LINE__))
   {
-    systemError(this, customFillList.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
