@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2018 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -10,7 +10,6 @@
 
 #include "characteristic.h"
 #include "display.h"
-#include "xlineedit.h"
 #include "ui_display.h"
 
 #include <QSqlError>
@@ -21,151 +20,127 @@
 #include <QToolButton>
 
 #include <metasql.h>
-#include <mqlutil.h>
+#include <metasql.h>
 #include <orprerender.h>
 #include <orprintrender.h>
 #include <renderobjects.h>
 #include <parameter.h>
 #include <previewdialog.h>
 
-#include "../scriptapi/parameterlistsetup.h"
+#include "parameterlistsetup.h"
+#include "errorReporter.h"
+#include "displayprivate.h"
 
-class displayPrivate : public Ui::display
+displayPrivate::displayPrivate(::display *parent)
+    : QObject(parent),
+      _useAltId(false),
+      _queryOnStartEnabled(false),
+      _autoUpdateEnabled(false),
+      _filterChanged(false),
+      _parent(parent)
 {
-public:
-  displayPrivate(::display * parent) : _parent(parent)
-  {
-    setupUi(_parent);
-    _parameterWidget->setVisible(false);
-    _queryonstart->hide(); // hide until query on start enabled
-    _autoupdate->hide(); // hide until auto update is enabled
-    _search->hide();
-    _searchLit->hide();
-    _listLabelFrame->setVisible(false);
-    _useAltId = false;
-    _queryOnStartEnabled = false;
-    _autoUpdateEnabled = false;
+  setupUi(_parent);
 
-    // Build Toolbar even if we hide it so we get actions
-    _newBtn = new QToolButton(_toolBar);
-    _newBtn->setObjectName("_newBtn");
-    _newBtn->setFocusPolicy(Qt::NoFocus);
-    _newAct = _toolBar->addWidget(_newBtn);
-    _newAct->setVisible(false);
+  _parameterWidget->setVisible(false);
+  _queryonstart->hide(); // hide until query on start enabled
+  _autoupdate->hide(); // hide until auto update is enabled
+  _search->hide();
+  _searchLit->hide();
+  _listLabelFrame->setVisible(false);
 
-    _closeBtn = new QToolButton(_toolBar);
-    _closeBtn->setObjectName("_closeBtn");
-    _closeBtn->setFocusPolicy(Qt::NoFocus);
-    _closeAct = _toolBar->addWidget(_closeBtn);
+  // Build Toolbar even if we hide it so we get actions
+  _newBtn = new QToolButton(_toolBar);
+  _newBtn->setObjectName("_newBtn");
+  _newBtn->setFocusPolicy(Qt::NoFocus);
+  _newAct = _toolBar->addWidget(_newBtn);
+  _newAct->setVisible(false);
 
-    _sep1 = _toolBar->addSeparator();
+  _closeBtn = new QToolButton(_toolBar);
+  _closeBtn->setObjectName("_closeBtn");
+  _closeBtn->setFocusPolicy(Qt::NoFocus);
+  _closeAct = _toolBar->addWidget(_closeBtn);
 
-    // Move parameter widget controls into toolbar
-    _moreBtn = new QToolButton(_toolBar);
-    _moreBtn->setObjectName("_moreBtn");
-    _moreBtn->setFocusPolicy(Qt::NoFocus);
-    _moreBtn->setCheckable(true);
-    _moreAct = _toolBar->addWidget(_moreBtn);
-    _moreAct->setVisible(false);
+  _sep1 = _toolBar->addSeparator();
 
-    QLabel* filterListLit = _parent->findChild<QLabel*>("_filterListLit");
-    filterListLit->setContentsMargins(3,0,3,0);
-    XComboBox* filterList = _parent->findChild<XComboBox*>("_filterList");
+  // Move parameter widget controls into toolbar
+  _moreBtn = new QToolButton(_toolBar);
+  _moreBtn->setObjectName("_moreBtn");
+  _moreBtn->setFocusPolicy(Qt::NoFocus);
+  _moreBtn->setCheckable(true);
+  _moreAct = _toolBar->addWidget(_moreBtn);
+  _moreAct->setVisible(false);
 
-    _filterLitAct = _toolBar->insertWidget(_moreAct, filterListLit);
-    _filterLitAct->setVisible(false);
+  QLabel* filterListLit = _parent->findChild<QLabel*>("_filterListLit");
+  filterListLit->setContentsMargins(3,0,3,0);
+  XComboBox* filterList = _parent->findChild<XComboBox*>("_filterList");
 
-    _filterAct = _toolBar->insertWidget(_moreAct, filterList);
-    _filterAct->setVisible(false);
+  _filterLitAct = _toolBar->insertWidget(_moreAct, filterListLit);
+  _filterLitAct->setVisible(false);
 
-    _sep2 = _toolBar->addSeparator();
-    _sep2->setVisible(false);
+  _filterAct = _toolBar->insertWidget(_moreAct, filterList);
+  _filterAct->setVisible(false);
 
-    // Print buttons
-    _printBtn = new QToolButton(_toolBar);
-    _printBtn->setObjectName("_printBtn");
-    _printBtn->setFocusPolicy(Qt::NoFocus);
-    _printAct = _toolBar->addWidget(_printBtn);
-    _printAct->setVisible(false); // hide the print button until a reportName is set
+  _sep2 = _toolBar->addSeparator();
+  _sep2->setVisible(false);
 
-    _previewBtn = new QToolButton(_toolBar);
-    _previewBtn->setObjectName("_previewBtn");
-    _printBtn->setFocusPolicy(Qt::NoFocus);
-    _previewAct = _toolBar->addWidget(_previewBtn);
-    _previewAct->setVisible(false); // hide the preview button until a reportName is set
-    _sep3 = _toolBar->addSeparator();
-    _sep3->setVisible(false);
+  _expandBtn = new QToolButton(_toolBar);
+  _expandBtn->setObjectName("_expandBtn");
+  _expandBtn->setFocusPolicy(Qt::NoFocus);
+  _expandAct = _toolBar->addWidget(_expandBtn);
+  _expandAct->setVisible(false);
 
-    // Optional search widget in toolbar
-    _search->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
-    _searchAct = _toolBar->addWidget(_search);
-    _searchAct->setVisible(false);
+  _collapseBtn = new QToolButton(_toolBar);
+  _collapseBtn->setObjectName("_collapseBtn");
+  _collapseBtn->setFocusPolicy(Qt::NoFocus);
+  _collapseAct = _toolBar->addWidget(_collapseBtn);
+  _collapseAct->setVisible(false);
 
-    // Remaining buttons in toolbar
-    _queryBtn = new QToolButton(_toolBar);
-    _queryBtn->setObjectName("_queryBtn");
-    _queryBtn->setFocusPolicy(Qt::NoFocus);
-    _queryAct = _toolBar->addWidget(_queryBtn);
+  // Print buttons
+  _printBtn = new QToolButton(_toolBar);
+  _printBtn->setObjectName("_printBtn");
+  _printBtn->setFocusPolicy(Qt::NoFocus);
+  _printAct = _toolBar->addWidget(_printBtn);
+  _printAct->setVisible(false); // hide the print button until a reportName is set
 
-    // Menu actions for query options
-    _queryMenu = new QMenu(_queryBtn);
-    _queryOnStartAct = new QAction(_queryMenu);
-    _queryOnStartAct->setCheckable(true);
-    _queryOnStartAct->setVisible(false);
-    _queryMenu->addAction(_queryOnStartAct);
+  _previewBtn = new QToolButton(_toolBar);
+  _previewBtn->setObjectName("_previewBtn");
+  _printBtn->setFocusPolicy(Qt::NoFocus);
+  _previewAct = _toolBar->addWidget(_previewBtn);
+  _previewAct->setVisible(false); // hide the preview button until a reportName is set
+  _sep3 = _toolBar->addSeparator();
+  _sep3->setVisible(false);
 
-    _autoUpdateAct = new QAction(_queryMenu);
-    _autoUpdateAct->setCheckable(true);
-    _autoUpdateAct->setVisible(false);
-    _queryMenu->addAction(_autoUpdateAct);
+  // Optional search widget in toolbar
+  _search->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
+  _searchAct = _toolBar->addWidget(_search);
+  _searchAct->setVisible(false);
 
-    _parent->layout()->setContentsMargins(0,0,0,0);
-    _parent->layout()->setSpacing(0);
-  }
+  // Remaining buttons in toolbar
+  _queryBtn = new QToolButton(_toolBar);
+  _queryBtn->setObjectName("_queryBtn");
+  _queryBtn->setFocusPolicy(Qt::NoFocus);
+  _queryAct = _toolBar->addWidget(_queryBtn);
 
-  bool setParams(ParameterList &);
-  void setupCharacteristics(QStringList uses);
-  void print(ParameterList, bool, bool);
+  // Menu actions for query options
+  _queryMenu = new QMenu(_queryBtn);
+  _queryOnStartAct = new QAction(_queryMenu);
+  _queryOnStartAct->setCheckable(true);
+  _queryOnStartAct->setVisible(false);
+  _queryMenu->addAction(_queryOnStartAct);
 
-  QString reportName;
-  QString metasqlName;
-  QString metasqlGroup;
+  _autoUpdateAct = new QAction(_queryMenu);
+  _autoUpdateAct->setCheckable(true);
+  _autoUpdateAct->setVisible(false);
+  _queryMenu->addAction(_autoUpdateAct);
 
-  bool _useAltId;
-  bool _queryOnStartEnabled;
-  bool _autoUpdateEnabled;
+  _parent->layout()->setContentsMargins(0,0,0,0);
+  _parent->layout()->setSpacing(0);
+}
 
-  QAction* _newAct;
-  QAction* _closeAct;
-  QAction* _sep1;
-  QAction* _filterLitAct;
-  QAction* _filterAct;
-  QAction* _moreAct;
-  QAction* _sep2;
-  QAction* _printAct;
-  QAction* _previewAct;
-  QAction* _sep3;
-  QAction* _searchAct;
-  QAction* _queryAct;
-  QAction* _queryOnStartAct;
-  QAction* _autoUpdateAct;
-
-  QMenu* _queryMenu;
-
-  QToolButton * _newBtn;
-  QToolButton * _closeBtn;
-  QToolButton * _moreBtn;
-  QToolButton * _queryBtn;
-  QToolButton * _previewBtn;
-  QToolButton * _printBtn;
-
-  QList<QVariant> _charidstext;
-  QList<QVariant> _charidslist;
-  QList<QVariant> _charidsdate;
-
-private:
-  ::display * _parent;
-};
+void displayPrivate::sFilterChanged()
+{
+  _filterChanged = true;
+}
 
 void displayPrivate::print(ParameterList pParams, bool showPreview, bool forceSetParams)
 {
@@ -266,7 +241,7 @@ void displayPrivate::setupCharacteristics(QStringList uses)
     column = QString("char%1").arg(chars.value("char_id").toString());
     name = chars.value("char_name").toString();
     _list->addColumn(name, -1, Qt::AlignLeft , false, column );
-    if (chartype == characteristic::Text)
+    if (chartype == characteristic::Text || chartype == characteristic::Number)
     {
       _charidstext.append(chars.value("char_id").toInt());
       _parameterWidget->append(name, column, ParameterWidget::Text);
@@ -274,9 +249,10 @@ void displayPrivate::setupCharacteristics(QStringList uses)
     else if (chartype == characteristic::List)
     {
       _charidslist.append(chars.value("char_id").toInt());
-      QString sql =   QString("SELECT charopt_value, charopt_value "
-                              "FROM charopt "
-                              "WHERE (charopt_char_id=%1);")
+      QString sql = QString("SELECT charopt_value, charopt_value"
+                            "  FROM charopt"
+                            " WHERE charopt_char_id = %1"
+                            " ORDER BY charopt_order, charopt_value;")
           .arg(chars.value("char_id").toInt());
       _parameterWidget->append(name, column, ParameterWidget::Multiselect, QVariant(), false, sql);
     }
@@ -315,38 +291,44 @@ bool displayPrivate::setParams(ParameterList &params)
   params.append("char_id_date_list", _charidsdate);
 
   // Handle text based sections of clause
-  for (int i = 0; i < _charidstext.count(); i++)
+  foreach (QVariant columnid, _charidstext)
   {
-    column = QString("char%1").arg(_charidstext.at(i).toString());
+    column = QString("char%1").arg(columnid.toString());
     param = params.value(column, &valid);
     if (valid)
-      clauses.append(QString("charass_alias%1.charass_value ~* '%2'").arg(_charidstext.at(i).toString()).arg(param.toString()));
+      clauses.append(QString("charass_alias%1.charass_value ~* :char%1").arg(columnid.toString()));
   }
   // Handle text based sections of clause
-  for (int i = 0; i < _charidslist.count(); i++)
+  foreach (QVariant columnid, _charidslist)
   {
-    column = QString("char%1").arg(_charidslist.at(i).toString());
+    column = QString("char%1").arg(columnid.toString());
     param = params.value(column, &valid);
     if (valid)
     {
-      QStringList list = param.toStringList();
-      clauses.append(QString("charass_alias%1.charass_value IN ('%2') ").arg(_charidslist.at(i).toString()).arg(list.join("','")));
+      QStringList list;
+      for (int j = 0; j < param.toStringList().count(); j++)
+        list << QString(":char%1_%2").arg(columnid.toString()).arg(j);
+
+      if (list.size())
+        clauses.append(QString("charass_alias%1.charass_value IN (%2) ").arg(columnid.toString()).arg(list.join(",")));
+      else
+        clauses.append("FALSE");
     }
   }
   // Handle date based sections of clause
-  for (int i = 0; i < _charidsdate.count(); i++)
+  foreach (QVariant columnid, _charidsdate)
   {
     // Look for start date
-    column = QString("char%1startDate").arg(_charidsdate.at(i).toString());
+    column = QString("char%1startDate").arg(columnid.toString());
     param = params.value(column, &valid);
     if (valid)
-      clauses.append(QString("charass_alias%1.charass_value::date >= '%2'").arg(_charidsdate.at(i).toString()).arg(param.toString()));
+      clauses.append(QString("charass_alias%1.charass_value::date >= :char%1startDate").arg(columnid.toString()));
 
     // Look for end date
-    column = QString("char%1endDate").arg(_charidsdate.at(i).toString());
+    column = QString("char%1endDate").arg(columnid.toString());
     param = params.value(column, &valid);
     if (valid)
-      clauses.append(QString("charass_alias%1.charass_value::date <= '%2'").arg(_charidsdate.at(i).toString()).arg(param.toString()));
+      clauses.append(QString("charass_alias%1.charass_value::date <= :char%1endDate").arg(columnid.toString()));
   }
   if (clauses.count())
     params.append("charClause", clauses.join(" AND ").prepend(" AND "));
@@ -365,6 +347,8 @@ display::display(QWidget* parent, const char* name, Qt::WindowFlags flags)
   _data->_newBtn->setText(tr("New"));
   _data->_closeBtn->setText(tr("Close"));
   _data->_moreBtn->setText(tr("More"));
+  _data->_expandBtn->setText(tr("Expand All"));
+  _data->_collapseBtn->setText(tr("Collapse All"));
   _data->_printBtn->setText(tr("Print"));
   _data->_previewBtn->setText(tr("Preview"));
   _data->_queryBtn->setText(tr("Query"));
@@ -378,7 +362,9 @@ display::display(QWidget* parent, const char* name, Qt::WindowFlags flags)
   _data->_printAct->setShortcut(QKeySequence::Print);
 
   _data->_search->setNullStr(tr("search"));
-  _data->_searchAct->setShortcut(QKeySequence::InsertParagraphSeparator);
+  QList<QKeySequence> shortcuts;
+  shortcuts << QKeySequence(Qt::Key_Enter) << QKeySequence(Qt::Key_Return);
+  _data->_searchAct->setShortcuts(shortcuts);
   _data->_searchAct->setShortcutContext(Qt::WidgetWithChildrenShortcut);
 
   // Set tooltips
@@ -386,10 +372,14 @@ display::display(QWidget* parent, const char* name, Qt::WindowFlags flags)
   _data->_closeBtn->setToolTip(_data->_closeBtn->text() + " " + _data->_closeAct->shortcut().toString(QKeySequence::NativeText));
   _data->_queryBtn->setToolTip(_data->_queryBtn->text() + " " + _data->_queryAct->shortcut().toString(QKeySequence::NativeText));
   _data->_printBtn->setToolTip(_data->_printBtn->text() + " " + _data->_printAct->shortcut().toString(QKeySequence::NativeText));
+  _data->_expandBtn->setToolTip(_data->_expandBtn->text());
+  _data->_collapseBtn->setToolTip(_data->_collapseBtn->text());
 
   connect(_data->_newBtn, SIGNAL(clicked()), _data->_newAct, SLOT(trigger()));
   connect(_data->_closeBtn, SIGNAL(clicked()), _data->_closeAct, SLOT(trigger()));
   connect(_data->_moreBtn, SIGNAL(clicked(bool)), filterButton, SLOT(setChecked(bool)));
+  connect(_data->_expandBtn, SIGNAL(clicked()), _data->_expandAct, SLOT(trigger()));
+  connect(_data->_collapseBtn, SIGNAL(clicked()), _data->_collapseAct, SLOT(trigger()));
   connect(_data->_printBtn, SIGNAL(clicked()), _data->_printAct, SLOT(trigger()));
   connect(_data->_previewBtn, SIGNAL(clicked()), _data->_previewAct, SLOT(trigger()));
   connect(_data->_queryBtn, SIGNAL(clicked()), _data->_queryAct, SLOT(trigger()));
@@ -403,6 +393,8 @@ display::display(QWidget* parent, const char* name, Qt::WindowFlags flags)
   connect(_data->_newAct, SIGNAL(triggered()), this, SLOT(sNew()));
   connect(_data->_closeAct, SIGNAL(triggered()), this, SLOT(close()));
   connect(_data->_queryAct, SIGNAL(triggered()), this, SLOT(sFillList()));
+  connect(_data->_expandAct, SIGNAL(triggered()), this, SLOT(sExpand()));
+  connect(_data->_collapseAct, SIGNAL(triggered()), this, SLOT(sCollapse()));
   connect(_data->_printAct, SIGNAL(triggered()), this, SLOT(sPrint()));
   connect(_data->_previewAct, SIGNAL(triggered()), this, SLOT(sPreview()));
   connect(_data->_searchAct, SIGNAL(triggered()), this, SLOT(sFillList()));
@@ -425,8 +417,16 @@ void display::languageChange()
 
 void display::showEvent(QShowEvent * e)
 {
+  loadScriptEngine();
   XWidget::showEvent(e);
 
+  // don't overwrite the user's filter when the window is minimized
+  if (! _data->_filterChanged)
+  {
+    parameterWidget()->applyDefaultFilterSet();
+    connect(parameterWidget(), SIGNAL(filterChanged()), _data, SLOT(sFilterChanged()), Qt::UniqueConnection);
+  }
+
   if (_data->_queryOnStartEnabled &&
       _data->_queryonstart->isChecked())
     emit fillList();
@@ -462,6 +462,16 @@ QAction * display::closeAction()
   return _data->_closeAct;
 }
 
+QAction * display::expandAction()
+{
+  return _data->_expandAct;
+}
+
+QAction * display::collapseAction()
+{
+  return _data->_collapseAct;
+}
+
 QAction * display::filterSeparator()
 {
   return _data->_sep1;
@@ -507,14 +517,12 @@ QString display::searchText()
 bool display::setParams(ParameterList & params)
 {
   bool ret = _data->setParams(params);
-  if(engine() && engine()->globalObject().property("setParams").isFunction())
+
+  if (ret)
   {
-    QScriptValue paramArg = ParameterListtoScriptValue(engine(), params);
-    QScriptValue tmp = engine()->globalObject().property("setParams").call(QScriptValue(), QScriptValueList() << paramArg);
-    ret = ret && tmp.toBool();
-    params.clear();
-    ParameterListfromScriptValue(paramArg, params);
+    ret = setScriptableParams(params);
   }
+
   return ret;
 }
 
@@ -588,6 +596,26 @@ bool display::closeVisible() const
   return _data->_closeAct->isVisible();
 }
 
+void display::setExpandVisible(bool show)
+{
+  _data->_expandAct->setVisible(show);
+}
+
+bool display::expandVisible() const
+{
+  return _data->_expandAct->isVisible();
+}
+
+void display::setCollapseVisible(bool show)
+{
+  _data->_collapseAct->setVisible(show);
+}
+
+bool display::collapseVisible() const
+{
+  return _data->_collapseAct->isVisible();
+}
+
 void display::setParameterWidgetVisible(bool show)
 {
   _data->_parameterWidget->setVisible(show);
@@ -684,6 +712,19 @@ void display::sNew()
 {
 }
 
+void display::sExpand()
+{
+    if (_data->_list)
+        _data->_list->expandAll();
+}
+
+void display::sCollapse()
+{
+    if (_data->_list)
+        _data->_list->collapseAll();
+}
+
+
 void display::sPrint()
 {
   sPrint(ParameterList());
@@ -718,19 +759,55 @@ void display::sFillList(ParameterList pParams, bool forceSetParams)
       return;
   }
   int itemid = _data->_list->id();
-  bool ok = true;
-  QString errorString;
-  MetaSQLQuery mql = MQLUtil::mqlLoad(_data->metasqlGroup, _data->metasqlName, errorString, &ok);
-  if(!ok)
+  MetaSQLQuery mql(omfgThis->_mqlhash->value(_data->metasqlGroup, _data->metasqlName));
+  XSqlQuery xq = mql.toQuery(pParams, QSqlDatabase(), false);
+
+  QString column;
+  QVariant param;
+  bool valid;
+
+  foreach (QVariant columnid, _data->_charidstext)
   {
-    systemError(this, errorString, __FILE__, __LINE__);
-    return;
+    column = QString("char%1").arg(columnid.toString());
+    param = pParams.value(column, &valid);
+    if (valid)
+      xq.bindValue(QString(":%1").arg(column), param.toString());
   }
-  XSqlQuery xq = mql.toQuery(pParams);
+
+  foreach (QVariant columnid, _data->_charidslist)
+  {
+    column = QString("char%1").arg(columnid.toString());
+    param = pParams.value(column, &valid);
+    if (valid)
+    {
+      QStringList list = param.toStringList();
+      for (int j = 0; j < list.count(); j++)
+        xq.bindValue(QString(":%1_%2").arg(column).arg(j), list.at(j));
+    }
+  }
+
+  foreach (QVariant columnid, _data->_charidsdate)
+  {
+    // Look for start date
+    column = QString("char%1startDate").arg(columnid.toString());
+    param = pParams.value(column, &valid);
+    if (valid)
+      xq.bindValue(QString(":%1").arg(column), param.toString());
+
+    // Look for end date
+    column = QString("char%1endDate").arg(columnid.toString());
+    param = pParams.value(column, &valid);
+    if (valid)
+      xq.bindValue(QString(":%1").arg(column), param.toString());
+  }
+
+  xq.exec();
+
   _data->_list->populate(xq, itemid, _data->_useAltId);
   if (xq.lastError().type() != QSqlError::NoError)
   {
-    systemError(this, xq.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Information"),
+                           xq, __FILE__, __LINE__);
     return;
   }
   emit fillListAfter();
