@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -44,6 +44,11 @@ public:
     Q_INVOKABLE void setCloseVisible(bool);
     Q_INVOKABLE bool closeVisible() const;
 
+    Q_INVOKABLE void setExpandVisible(bool);
+    Q_INVOKABLE bool expandVisible() const;
+    Q_INVOKABLE void setCollapseVisible(bool);
+    Q_INVOKABLE bool collapseVisible() const;
+
     Q_INVOKABLE void setParameterWidgetVisible(bool);
     Q_INVOKABLE bool parameterWidgetVisible() const;
 
@@ -69,6 +74,8 @@ public:
     Q_INVOKABLE QAction * querySeparator();
     Q_INVOKABLE QAction * searchAction();
     Q_INVOKABLE QAction * queryAction();
+    Q_INVOKABLE QAction * expandAction();
+    Q_INVOKABLE QAction * collapseAction();
 
     Q_INVOKABLE QString   searchText();
 
@@ -77,6 +84,8 @@ public:
 
 public slots:
     virtual void sNew();
+    virtual void sExpand();
+    virtual void sCollapse();
     virtual void sPrint();
     virtual void sPrint(ParameterList, bool = false);
     virtual void sPreview();
@@ -87,11 +96,11 @@ public slots:
 
 protected:
     Q_INVOKABLE ParameterList getParams();
-    virtual void showEvent(QShowEvent*);
 
 protected slots:
     virtual void languageChange();
     virtual void sAutoUpdateToggled();
+    virtual void showEvent(QShowEvent *);
 
 signals:
     void fillList();
