@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -17,6 +17,8 @@
 
 #include "errorReporter.h"
 #include "apOpenItem.h"
+#include "applyAPCreditMemo.h"
+#include "printApOpenItem.h"
 #include "miscVoucher.h"
 #include "voucher.h"
 
@@ -43,17 +45,12 @@ dspAPOpenItemsByVendor::dspAPOpenItemsByVendor(QWidget* parent, const char*, Qt:
   list()->addColumn(tr("Due Date"),     _dateColumn,     Qt::AlignCenter, true,  "apopen_duedate" );
   list()->addColumn(tr("Amount"),       _moneyColumn,    Qt::AlignRight,  true,  "apopen_amount"  );
   list()->addColumn(tr("Paid"),         _moneyColumn,    Qt::AlignRight,  true,  "paid"  );
-  list()->addColumn(tr("Balance"),      -1,              Qt::AlignRight,  true,  "balance"  );
-  list()->addColumn(tr("Currency"),     _currencyColumn, Qt::AlignLeft,   true,  "currAbbr"   );
+  list()->addColumn(tr("Balance"),      -1,              Qt::AlignRight,  !omfgThis->singleCurrency(),  "balance"  );
+  list()->addColumn(tr("Currency"),     _currencyColumn, Qt::AlignLeft,   !omfgThis->singleCurrency(),  "currAbbr"   );
   list()->addColumn(tr("Base Balance"), _moneyColumn,    Qt::AlignRight,  true,  "base_balance"  );
   list()->addColumn(tr("Status"),       _moneyColumn,    Qt::AlignCenter, false, "apopen_status"  );
 
-  if (omfgThis->singleCurrency())
-  {
-    list()->hideColumn(10);
-    list()->hideColumn(11);
-  }
-  else
+  if (!omfgThis->singleCurrency())
   {
     XSqlQuery qq;
     qq.prepare("SELECT currConcat(baseCurrId()) AS currConcat;");
@@ -155,6 +152,17 @@ void dspAPOpenItemsByVendor::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem *select
       menuItem->setEnabled(_privileges->check("EditAPOpenItem"));
     }	
   }
+
+  menuItem = pMenu->addAction(tr("Print..."), this, SLOT(sPrintItem()));
+  if (!_privileges->check("EditAPOpenItem"))
+    menuItem->setEnabled(false);
+
+  if (list()->currentItem()->text("f_doctype") == tr("Credit Memo"))
+  {
+    pMenu->addSeparator();
+    menuItem = pMenu->addAction(tr("Apply Credit Memo..."), this, SLOT(sApplyAPOpenCM()));
+    menuItem->setEnabled(_privileges->check("ApplyAPMemos"));
+  }
 }
 
 void dspAPOpenItemsByVendor::sViewVoucher()
@@ -220,10 +228,41 @@ void dspAPOpenItemsByVendor::sView()
     sFillList();
 }
 
+void dspAPOpenItemsByVendor::sApplyAPOpenCM()
+{
+  ParameterList params;
+  params.append("apopen_id", list()->id());
+
+  applyAPCreditMemo newdlg(this, "", true);
+  newdlg.set(params);
+
+  if (newdlg.exec() != XDialog::Rejected)
+    sFillList();
+}
+
+void dspAPOpenItemsByVendor::sPrintItem()
+{
+  ParameterList params;
+  params.append("apopen_id", list()->id());
+
+  printApOpenItem newdlg(this, "", true);
+  if (newdlg.set(params) == NoError)
+    newdlg.exec();
+}
+
 bool dspAPOpenItemsByVendor::setParams(ParameterList & params)
 {
+  if (!display::setParams(params))
+    return false;
+
   _vendorGroup->appendValue(params);
-  _dates->appendValue(params);
+  if (_docDate->isChecked())
+    _dates->appendValue(params);
+  else
+  {
+    params.append("startDueDate", _dates->startDate());
+    params.append("endDueDate", _dates->endDate());
+  }
   params.append("asofDate", _asOf->date());
 
   // have both in case we add a third option
