@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -19,6 +19,7 @@
 #include "salesOrderItem.h"
 #include "parameterwidget.h"
 #include "printPackingList.h"
+#include "errorReporter.h"
 
 dspBacklog::dspBacklog(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspBacklog", fl)
@@ -48,8 +49,6 @@ dspBacklog::dspBacklog(QWidget* parent, const char*, Qt::WindowFlags fl)
   if (_metrics->boolean("MultiWhs"))
     parameterWidget()->append(tr("Site"), "warehous_id", ParameterWidget::Site);
 
-  parameterWidget()->applyDefaultFilterSet();
-
   list()->setSelectionMode(QAbstractItemView::ExtendedSelection);
 
   list()->addColumn(tr("S/O #/Line #"),_itemColumn, Qt::AlignLeft,  true, "coitem_linenumber");
@@ -61,6 +60,7 @@ dspBacklog::dspBacklog(QWidget* parent, const char*, Qt::WindowFlags fl)
   list()->addColumn(tr("Ordered"),      _qtyColumn, Qt::AlignRight, true, "coitem_qtyord");
   list()->addColumn(tr("Shipped"),      _qtyColumn, Qt::AlignRight, true, "coitem_qtyshipped");
   list()->addColumn(tr("Balance"),      _qtyColumn, Qt::AlignRight, true, "qtybalance");
+  list()->addColumn(tr("QOH"),          _qtyColumn, Qt::AlignRight, true, "qtyonhand");
   if (_privileges->check("ViewCustomerPrices") || _privileges->check("MaintainCustomerPrices"))
   {
     list()->addColumn(tr("Base Ext. Price"), _bigMoneyColumn, Qt::AlignRight,  true, "baseextpricebalance");
@@ -175,9 +175,9 @@ void dspBacklog::sAddToPackingListBatch()
     dspAddToPackingListBatch.prepare("SELECT addToPackingListBatch(:sohead_id) AS result;");
     dspAddToPackingListBatch.bindValue(":sohead_id", ((XTreeWidgetItem*)(selected[i]))->id());
     dspAddToPackingListBatch.exec();
-    if (dspAddToPackingListBatch.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Packing List Batch Information"),
+                                  dspAddToPackingListBatch, __FILE__, __LINE__))
     {
-      systemError(this, dspAddToPackingListBatch.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
