@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -50,6 +50,9 @@ void dspBriefEarnedCommissions::languageChange()
 
 bool dspBriefEarnedCommissions::setParams(ParameterList &params)
 {
+  if (!display::setParams(params))
+    return false;
+
   if (!_dates->allValid())
   {
     if (!_dates->startDate().isValid() && isVisible())
