@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -16,7 +16,7 @@
 
 #include "item.h"
 
-dspCapacityUOMsByParameter::dspCapacityUOMsByParameter(QWidget* parent, const char* name, Qt::WFlags fl)
+dspCapacityUOMsByParameter::dspCapacityUOMsByParameter(QWidget* parent, const char* name, Qt::WindowFlags fl)
     : display(parent, name, fl)
 {
   setupUi(optionsWidget());
@@ -32,6 +32,9 @@ void dspCapacityUOMsByParameter::languageChange()
 
 bool dspCapacityUOMsByParameter::setParams(ParameterList &params)
 {
+  if (!display::setParams(params))
+    return false;
+
   _parameter->appendValue(params);
 
   return true;
