@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -13,6 +13,9 @@
 #include <QSqlError>
 #include <QVariant>
 
+#include "errorReporter.h"
+#include "xtreewidget.h"
+
 dspCostedSingleLevelBOM::dspCostedSingleLevelBOM(QWidget* parent, const char*, Qt::WindowFlags fl)
     : dspCostedBOMBase(parent, "dspCostedSingleLevelBOM", fl)
 {
@@ -29,6 +32,10 @@ bool dspCostedSingleLevelBOM::setParams(ParameterList &params)
 
 void dspCostedSingleLevelBOM::sFillList()
 {
+  ParameterList params;
+  if (!setParams(params))
+    return;
+  
   dspCostedBOMBase::sFillList();
 
   XSqlQuery qq;
@@ -44,9 +51,9 @@ void dspCostedSingleLevelBOM::sFillList()
     last = new XTreeWidgetItem(list(), last, -1, -1, tr("Standard Cost"), "" );
     last->setText(11, qq.value("standard").toString());
   }
-  else if (qq.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving BOM Cost Information"),
+                                qq, __FILE__, __LINE__))
   {
-    systemError(this, qq.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
