@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -21,6 +21,7 @@
 #include "expenseTrans.h"
 #include "materialReceiptTrans.h"
 #include "countTag.h"
+#include "xtreewidget.h"
 
 dspDetailedInventoryHistoryByLocation::dspDetailedInventoryHistoryByLocation(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspDetailedInventoryHistoryByLocation", fl)
@@ -102,7 +103,8 @@ void dspDetailedInventoryHistoryByLocation::sPopulateLocationInfo(int pLocationi
 {
   XSqlQuery qq;
   qq.prepare( "SELECT formatBoolYN(location_netable) AS netable,"
-             "       formatBoolYN(location_restrict) AS restricted "
+             "       formatBoolYN(location_restrict) AS restricted, "
+             "       formatBoolYN(location_active)   AS active "
              "FROM location, whsinfo "
              "WHERE ( (location_warehous_id=warehous_id)"
              " AND (location_id=:location_id) );" );
@@ -112,6 +114,7 @@ void dspDetailedInventoryHistoryByLocation::sPopulateLocationInfo(int pLocationi
   {
     _netable->setText(qq.value("netable").toString());
     _restricted->setText(qq.value("restricted").toString());
+    _active->setText(qq.value("active").toString());
   }
 }
 
@@ -177,6 +180,9 @@ void dspDetailedInventoryHistoryByLocation::sPopulateMenu(QMenu *menuThis, QTree
 
 bool dspDetailedInventoryHistoryByLocation::setParams(ParameterList &params)
 {
+  if (!display::setParams(params))
+    return false;
+
   params.append("byLocation");
  
   if (!_dates->startDate().isValid())
