@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -21,6 +21,7 @@
 #include "materialReceiptTrans.h"
 #include "countTag.h"
 #include "parameterwidget.h"
+#include "xtreewidget.h"
 
 dspDetailedInventoryHistoryByLotSerial::dspDetailedInventoryHistoryByLotSerial(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspDetailedInventoryHistoryByLotSerial", fl)
@@ -78,6 +79,19 @@ dspDetailedInventoryHistoryByLotSerial::dspDetailedInventoryHistoryByLotSerial(Q
   list()->addColumn(tr("Qty. Before"),  _qtyColumn,          Qt::AlignRight,  true,  "lshist_qty_before"  );
   list()->addColumn(tr("Qty. After"),   _qtyColumn,          Qt::AlignRight,  true,  "lshist_qty_after"  );
 
+  list()->addColumn(tr("Customer #/Name"),              170, Qt::AlignLeft,   false, "customerinfo");
+  list()->addColumn(tr("ShipTo Name"),                  170, Qt::AlignLeft,   false, "shiptoname");
+  list()->addColumn(tr("ShipTo Line1"),                 200, Qt::AlignLeft,   false, "shiptol1");
+  list()->addColumn(tr("ShipTo Line2"),                 200, Qt::AlignLeft,   false, "shiptol2");
+  list()->addColumn(tr("ShipTo Line3"),                 200, Qt::AlignLeft,   false, "shiptol3");
+  list()->addColumn(tr("City/State/Zip"),               200, Qt::AlignLeft,   false, "citystatezip");
+  list()->addColumn(tr("Country"),                      120, Qt::AlignLeft,   false, "country");
+  list()->addColumn(tr("Contact"),                      120, Qt::AlignLeft,   false, "contact");
+  list()->addColumn(tr("Contact Phone"),                100, Qt::AlignLeft,   false, "contact_phone");
+  list()->addColumn(tr("Contact Fax"),                  100, Qt::AlignLeft,   false, "contact_fax");
+  list()->addColumn(tr("Contact Email"),                175, Qt::AlignLeft,   false, "contact_email");
+  list()->addColumn(tr("Phone"),                        100, Qt::AlignLeft,   false, "phone");
+  
   parameterWidget()->append(tr("Start Date"), "startDate", ParameterWidget::Date);
   parameterWidget()->append(tr("End Date"),   "endDate",   ParameterWidget::Date);
   if (_metrics->boolean("MultiWhs"))
@@ -173,6 +187,9 @@ void dspDetailedInventoryHistoryByLotSerial::sSelect()
 
 bool dspDetailedInventoryHistoryByLotSerial::setParams(ParameterList &params)
 {
+  if (!display::setParams(params))
+    return false;
+
   QString trace = "N";
   params = parameterWidget()->parameters();
 
@@ -212,7 +229,7 @@ bool dspDetailedInventoryHistoryByLotSerial::setParams(ParameterList &params)
   else
   {
     params.append("pattern", "t");
-    params.append("lotSerial", _lotSerialPattern->text().trimmed());
+    params.append("lotSerial", _lotSerialPattern->text().trimmed().toUpper());
   }
 
   return true;
