@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -24,6 +24,7 @@
 #include "transferTrans.h"
 #include "createCountTagsByItem.h"
 #include "guiclient.h"
+#include "xtreewidget.h"
 
 #define COST_COL	6
 #define VALUE_COL	7
@@ -187,6 +188,9 @@ void dspExpiredInventoryByClassCode::sFillList()
 
 bool dspExpiredInventoryByClassCode::setParams(ParameterList &params)
 {
+  if (!display::setParams(params))
+    return false;
+
   if (_perishability->isChecked())
   {
     params.append("perishability");
