@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -16,6 +16,7 @@
 #include <QVariant>
 
 #include "dspInventoryHistory.h"
+#include "xtreewidget.h"
 
 dspIndentedWhereUsed::dspIndentedWhereUsed(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspIndentedWhereUsed", fl)
@@ -74,6 +75,9 @@ enum SetResponse dspIndentedWhereUsed::set(const ParameterList &pParams)
 
 bool dspIndentedWhereUsed::setParams(ParameterList &params)
 {
+  if (!display::setParams(params))
+    return false;
+
   if (!_item->isValid())
   {
     QMessageBox::warning( this, tr("Enter a Valid Item Number"),
