@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -27,6 +27,7 @@
 #include "salesOrder.h"
 #include "storedProcErrorLookup.h"
 #include "workOrder.h"
+#include "errorReporter.h"
 
 dspInventoryAvailabilityByCustomerType::dspInventoryAvailabilityByCustomerType(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspInventoryAvailabilityByCustomerType", fl)
@@ -93,6 +94,9 @@ enum SetResponse dspInventoryAvailabilityByCustomerType::set(const ParameterList
 
 bool dspInventoryAvailabilityByCustomerType::setParams(ParameterList &params)
 {
+  if (!display::setParams(params))
+    return false;
+
   _custtype->appendValue(params);
 
   if(_onlyShowShortages->isChecked())
@@ -335,15 +339,15 @@ void dspInventoryAvailabilityByCustomerType::sReserveLineBalance()
     int result = qq.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("reserveSoLineBalance", result),
-                  __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                             storedProcErrorLookup("reserveSoLineBalance", result),
+                             __FILE__, __LINE__);
       return;
     }
   }
-  else if (qq.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                qq, __FILE__, __LINE__))
   {
-    systemError(this, tr("Error\n") +
-                      qq.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -356,10 +360,9 @@ void dspInventoryAvailabilityByCustomerType::sUnreserveStock()
   qq.prepare("UPDATE coitem SET coitem_qtyreserved=0 WHERE coitem_id=:soitem_id;");
   qq.bindValue(":soitem_id", list()->altId());
   qq.exec();
-  if (qq.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Updating Item Information"),
+                                qq, __FILE__, __LINE__))
   {
-    systemError(this, tr("Error\n") +
-                      qq.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
