@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -25,6 +25,7 @@
 #include "reserveSalesOrderItem.h"
 #include "storedProcErrorLookup.h"
 #include "workOrder.h"
+#include "errorReporter.h"
 
 dspInventoryAvailabilityBySalesOrder::dspInventoryAvailabilityBySalesOrder(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspInventoryAvailabilityBySalesOrder", fl)
@@ -95,6 +96,9 @@ enum SetResponse dspInventoryAvailabilityBySalesOrder::set(const ParameterList &
 
 bool dspInventoryAvailabilityBySalesOrder::setParams(ParameterList &params)
 {
+  if (!display::setParams(params))
+    return false;
+
   if(!_so->isValid())
   {
     QMessageBox::warning(this, tr("No Sales Order Selected"),
@@ -345,15 +349,15 @@ void dspInventoryAvailabilityBySalesOrder::sReserveLineBalance()
     int result = dspReserveLineBalance.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("reserveSoLineBalance", result),
-                  __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                             storedProcErrorLookup("reserveSoLineBalance", result),
+                             __FILE__, __LINE__);
       return;
     }
   }
-  else if (dspReserveLineBalance.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                dspReserveLineBalance, __FILE__, __LINE__))
   {
-    systemError(this, tr("Error\n") +
-                      dspReserveLineBalance.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -366,10 +370,9 @@ void dspInventoryAvailabilityBySalesOrder::sUnreserveStock()
   dspUnreserveStock.prepare("UPDATE coitem SET coitem_qtyreserved=0 WHERE coitem_id=:soitem_id;");
   dspUnreserveStock.bindValue(":soitem_id", list()->altId());
   dspUnreserveStock.exec();
-  if (dspUnreserveStock.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Updating Item Information"),
+                                dspUnreserveStock, __FILE__, __LINE__))
   {
-    systemError(this, tr("Error\n") +
-                      dspUnreserveStock.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
