@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -13,12 +13,14 @@
 #include <QAction>
 #include <QMenu>
 #include <QMessageBox>
+#include "guiErrorCheck.h"
 #include <QSqlError>
 #include <QVariant>
 
 #include "arOpenItem.h"
 #include "creditMemo.h"
 #include "dspInvoiceInformation.h"
+#include "errorReporter.h"
 
 dspInvoiceRegister::dspInvoiceRegister(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspInvoiceRegister", fl)
@@ -82,9 +84,9 @@ enum SetResponse dspInvoiceRegister::set(const ParameterList &pParams)
       _dates->setStartDate(dspet.value("period_start").toDate());
       _dates->setEndDate(dspet.value("period_end").toDate());
     }
-    else if (dspet.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Invoicing Information"),
+                                  dspet, __FILE__, __LINE__))
     {
-      systemError(this, dspet.lastError().databaseText(), __FILE__, __LINE__);
       return UndefinedError;
     }
   }
@@ -176,9 +178,9 @@ void dspInvoiceRegister::sViewCreditMemo()
       newdlg.exec();
     }
   }
-  else if (dspViewCreditMemo.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Credit Memo Information"),
+                                dspViewCreditMemo, __FILE__, __LINE__))
   {
-    systemError(this, dspViewCreditMemo.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   else
@@ -211,12 +213,12 @@ void dspInvoiceRegister::sViewInvoice()
 
 bool dspInvoiceRegister::setParams(ParameterList &params)
 {
-  if(!_dates->allValid())
-  {
-    QMessageBox::warning(this, tr("Invalid Date Range"),
-      tr("You must specify a valid date range.") );
+  QList<GuiErrorCheck> errors;
+  errors<< GuiErrorCheck(!_dates->allValid(), _dates,
+                         tr("You must specify a valid date range."))
+  ;
+  if (GuiErrorCheck::reportErrors(this, tr("Invalid Date Range"), errors))
     return false;
-  }
 
   _dates->appendValue(params);
 
@@ -233,6 +235,10 @@ bool dspInvoiceRegister::setParams(ParameterList &params)
 
 void dspInvoiceRegister::sFillList()
 {
+  ParameterList params;
+  if (!setParams(params))
+    return;
+  
   display::sFillList();
   list()->expandAll();
 
