@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -12,6 +12,8 @@
 
 #include <QMessageBox>
 
+#include "xtreewidget.h"
+
 dspItemCostHistory::dspItemCostHistory(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspItemCostHistory", fl)
 {
@@ -28,15 +30,9 @@ dspItemCostHistory::dspItemCostHistory(QWidget* parent, const char*, Qt::WindowF
   list()->addColumn(tr("Time"),    _timeDateColumn, Qt::AlignCenter,true, "costhist_date");
   list()->addColumn(tr("User"),         _qtyColumn, Qt::AlignCenter,true, "username");
   list()->addColumn(tr("Old"),         _costColumn, Qt::AlignRight, true, "costhist_oldcost");
-  list()->addColumn(tr("Currency"),_currencyColumn, Qt::AlignLeft,  true, "oldcurr");
+  list()->addColumn(tr("Currency"),_currencyColumn, Qt::AlignLeft,  !omfgThis->singleCurrency(), "oldcurr");
   list()->addColumn(tr("New"),         _costColumn, Qt::AlignRight, true, "costhist_newcost");
-  list()->addColumn(tr("Currency"),_currencyColumn, Qt::AlignLeft,  true, "newcurr");
-
-  if (omfgThis->singleCurrency())
-  {
-    list()->hideColumn("oldcurr");
-    list()->hideColumn("newcurr");
-  }
+  list()->addColumn(tr("Currency"),_currencyColumn, Qt::AlignLeft,  !omfgThis->singleCurrency(), "newcurr");
 }
 
 void dspItemCostHistory::languageChange()
@@ -47,6 +43,9 @@ void dspItemCostHistory::languageChange()
 
 bool dspItemCostHistory::setParams(ParameterList &params)
 {
+  if (!display::setParams(params))
+    return false;
+
   if(!_item->isValid())
   {
     QMessageBox::warning( this, tr("Item Required"),
