@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -40,6 +40,7 @@ dspItemCostsByClassCode::dspItemCostsByClassCode(QWidget* parent, const char*, Q
   list()->addColumn(tr("Std. Cost"),   _costColumn,  Qt::AlignRight,  true,  "scost"  );
   list()->addColumn(tr("Act. Cost"),   _costColumn,  Qt::AlignRight,  true,  "acost"  );
   list()->addColumn(tr("% Var."),      _costColumn,  Qt::AlignRight,  false,  "percent_variance" );
+   list()->addColumn(tr("Item Type"), _orderColumn,  Qt::AlignLeft,   true,  "itemtype"   );
 }
 
 void dspItemCostsByClassCode::languageChange()
@@ -70,11 +71,16 @@ enum SetResponse dspItemCostsByClassCode::set(const ParameterList &pParams)
 
 bool dspItemCostsByClassCode::setParams(ParameterList &params)
 {
+  if (!display::setParams(params))
+    return false;
+
   params.append("byClassCode");
   _classCode->appendValue(params);
 
-  if(_onlyShowZero->isChecked())
-    params.append("onlyShowZeroCosts");
+  if(_onlyShowZeroStd->isChecked())
+    params.append("onlyShowZeroStdCosts");
+  if(_onlyShowZeroLast->isChecked())
+    params.append("onlyShowZeroLastCosts");
 
   if(_onlyShowDiff->isChecked())
     params.append("onlyShowDiffCosts");
@@ -82,6 +88,24 @@ bool dspItemCostsByClassCode::setParams(ParameterList &params)
   if(!_showInactive->isChecked())
     params.append("onlyShowActive");
 
+  if(_cbPurchase->isChecked())
+      params.append("showPurchased");
+
+  if(_cbMfg->isChecked())
+    params.append("showMfg");
+
+  if(_cbOS->isChecked())
+    params.append("showOS");
+
+  if(_cbCosting->isChecked())
+    params.append("showCosting");
+
+  if(_cbPlan->isChecked())
+    params.append("showPlanning");
+
+  if(_cbOther->isChecked())
+    params.append("showOther");
+
   XSqlQuery qq;
   qq.exec("SELECT locale_cost_scale "
          "FROM locale, usr "
