@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -17,18 +17,18 @@ class dspItemSources : public display
 {
     Q_OBJECT
 
-public:
-    dspItemSources(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = Qt::Window);
+    Q_INVOKABLE virtual bool setParams(ParameterList &);
 
-    virtual bool setParams(ParameterList &);
+public:
+    dspItemSources(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::Window);
 
 public slots:
     virtual void sPopulateMenu(QMenu * menuThis, QTreeWidgetItem*, int);
     virtual void sNew();
     virtual void sEdit();
     virtual void sView();
-    virtual void sCopy();
     virtual void sDefault();
+    virtual void sCopy();
     virtual void sDelete();
     virtual void sBuyCard();
     virtual void sReceipts();
