@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -13,6 +13,8 @@
 #include <QVariant>
 #include <QMessageBox>
 
+#include "xtreewidget.h"
+
 dspMaterialUsageVarianceByItem::dspMaterialUsageVarianceByItem(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspMaterialUsageVarianceByItem", fl)
 {
@@ -53,6 +55,9 @@ void dspMaterialUsageVarianceByItem::languageChange()
 
 bool dspMaterialUsageVarianceByItem::setParams(ParameterList &params)
 {
+  if (!display::setParams(params))
+    return false;
+
   if(!_dates->allValid())
   {
     QMessageBox::warning(this, tr("Invalid Date Range"),
@@ -74,4 +79,3 @@ bool dspMaterialUsageVarianceByItem::setParams(ParameterList &params)
   return true;
 }
 
-
