@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -14,6 +14,7 @@
 #include <QMessageBox>
 
 #include "inputManager.h"
+#include "xtreewidget.h"
 
 dspMaterialUsageVarianceByWorkOrder::dspMaterialUsageVarianceByWorkOrder(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspMaterialUsageVarianceByWorkOrder", fl)
@@ -51,9 +52,12 @@ void dspMaterialUsageVarianceByWorkOrder::languageChange()
 
 bool dspMaterialUsageVarianceByWorkOrder::setParams(ParameterList &params)
 {
+  if (!display::setParams(params))
+    return false;
+
   if(!_wo->isValid())
   {
-    QMessageBox::warning(this, tr("Invalid W/O"),
+    QMessageBox::warning(this, tr("Work Order Required"),
       tr("You must specify a Work Order.") );
     return false;
   }
