@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -69,9 +69,10 @@ dspOrderActivityByProject::dspOrderActivityByProject(QWidget* parent, const char
 
   disconnect(newAction(), SIGNAL(triggered()), this, SLOT(sNew()));
   connect(newAction(), SIGNAL(triggered()), this, SLOT(sNewProjectTask()));
-  connect(_showSo, SIGNAL(checked()), this, SLOT(sFillList()));
-  connect(_showPo, SIGNAL(checked()), this, SLOT(sFillList()));
-  connect(_showWo, SIGNAL(checked()), this, SLOT(sFillList()));
+  connect(_project,   SIGNAL(newId(int)), this, SLOT(sFillList()));
+  connect(_showSo, SIGNAL(toggled(bool)), this, SLOT(sFillList()));
+  connect(_showPo, SIGNAL(toggled(bool)), this, SLOT(sFillList()));
+  connect(_showWo, SIGNAL(toggled(bool)), this, SLOT(sFillList()));
 
   QToolButton * newBtn = (QToolButton*)toolBar()->widgetForAction(newAction());
   newBtn->setPopupMode(QToolButton::MenuButtonPopup);
@@ -175,8 +176,11 @@ void dspOrderActivityByProject::sPopulateMenu(QMenu * pMenu, QTreeWidgetItem*, i
 
   if(list()->altId() == 35)
   {
-    menuItem = pMenu->addAction(tr("Edit Invoice..."), this, SLOT(sEdit()));
-    menuItem->setEnabled(_privileges->check("MaintainMiscInvoices"));
+    if (list()->rawValue("status") != "Posted")
+    {
+      menuItem = pMenu->addAction(tr("Edit Invoice..."), this, SLOT(sEdit()));
+      menuItem->setEnabled(_privileges->check("MaintainMiscInvoices"));
+    }
 
     menuItem = pMenu->addAction(tr("View Invoice..."), this, SLOT(sView()));
     menuItem->setEnabled(_privileges->check("MaintainMiscInvoices") ||
@@ -185,8 +189,11 @@ void dspOrderActivityByProject::sPopulateMenu(QMenu * pMenu, QTreeWidgetItem*, i
 
   if(list()->altId() == 37)
   {
-    menuItem = pMenu->addAction(tr("Edit Invoice Item..."), this, SLOT(sEdit()));
-    menuItem->setEnabled(_privileges->check("MaintainMiscInvoices"));
+    if (list()->rawValue("status") != "Posted")
+    {
+      menuItem = pMenu->addAction(tr("Edit Invoice Item..."), this, SLOT(sEdit()));
+      menuItem->setEnabled(_privileges->check("MaintainMiscInvoices"));
+    }
 
     menuItem = pMenu->addAction(tr("View Invoice Item..."), this, SLOT(sView()));
     menuItem->setEnabled(_privileges->check("MaintainMiscInvoices") ||
@@ -426,10 +433,12 @@ void dspOrderActivityByProject::sView()
 
 bool dspOrderActivityByProject::setParams(ParameterList &params)
 {
-  if(_project->id() == -1 && _run)
+  if (!display::setParams(params))
+    return false;
+
+  if(_project->id() == -1)
   {
-    QMessageBox::warning(this, tr("Project Required"),
-      tr("You must specify a Project."));
+    list()->clear();
     return false;
   }
 
