@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -20,6 +20,7 @@
 #include "reprioritizeWo.h"
 #include "reschedulePoitem.h"
 #include "rescheduleWo.h"
+#include "xtreewidget.h"
 
 dspOrders::dspOrders(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspOrders", fl)
@@ -202,6 +203,9 @@ void dspOrders::sChangePoitemQty()
 
 bool dspOrders::setParams(ParameterList & params)
 {
+  if (!display::setParams(params))
+    return false;
+
   if ( !((_item->isValid()) &&
        ( (_leadTime->isChecked()) || (_byDays->isChecked()) ||
          ((_byDate->isChecked()) && (_date->isValid())) ||
