@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -37,8 +37,8 @@ dspPartiallyShippedOrders::dspPartiallyShippedOrders(QWidget* parent, const char
   _dates->setStartNull(tr("Earliest"), omfgThis->startOfTime(), true);
   _dates->setEndNull(tr("Latest"), omfgThis->endOfTime(), true);
 
-  list()->addColumn(tr("Hold"),        0,           Qt::AlignCenter,true, "cohead_holdtype");
-  list()->addColumn(tr("Sale Type"),  _orderColumn, Qt::AlignLeft,  true,  "saletype_descr");
+  list()->addColumn(tr("Hold"),        0,           Qt::AlignCenter,true, "holdtype");
+  list()->addColumn(tr("Sale Type"),  _orderColumn, Qt::AlignLeft,  true, "saletype_descr");
   list()->addColumn(tr("S/O #"),      _orderColumn, Qt::AlignRight, true, "cohead_number");
   list()->addColumn(tr("Customer"),    -1,          Qt::AlignLeft,  true, "cust_name");
   list()->addColumn(tr("Hold Type"),   _dateColumn, Qt::AlignCenter,true, "f_holdtype");
@@ -83,6 +83,9 @@ void dspPartiallyShippedOrders::sHandlePrices(bool pShowPrices)
 
 bool dspPartiallyShippedOrders::setParams(ParameterList &params)
 {
+  if (!display::setParams(params))
+    return false;
+
   _warehouse->appendValue(params);
   if (_dates->allValid())
     _dates->appendValue(params);
@@ -97,6 +100,7 @@ bool dspPartiallyShippedOrders::setParams(ParameterList &params)
   params.append("pack",   tr("Pack"));
   params.append("return", tr("Return"));
   params.append("ship",   tr("Ship"));
+  params.append("tax",    tr("Tax"));
   params.append("other",  tr("Other"));
 
   if (omfgThis->singleCurrency())
