@@ -20,7 +20,7 @@ class dspPendingAvailability : public display, public Ui::dspPendingAvailability
     Q_OBJECT
 
 public:
-    dspPendingAvailability(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = Qt::Window);
+    dspPendingAvailability(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::Window);
 
     virtual bool setParams(ParameterList &);
 
