@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -11,6 +11,7 @@
 #include "dspPoDeliveryDateVariancesByItem.h"
 
 #include <QMessageBox>
+#include "xtreewidget.h"
 
 dspPoDeliveryDateVariancesByItem::dspPoDeliveryDateVariancesByItem(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspPoDeliveryDateVariancesByItem", fl)
@@ -51,6 +52,9 @@ void dspPoDeliveryDateVariancesByItem::languageChange()
 
 bool dspPoDeliveryDateVariancesByItem::setParams(ParameterList &pParams)
 {
+  if (!display::setParams(pParams))
+    return false;
+
   pParams.append("byItem");
   pParams.append("item_id", _item->id());
 
