@@ -20,7 +20,7 @@ class dspPoHistory : public display, public Ui::dspPoHistory
     Q_OBJECT
 
 public:
-    dspPoHistory(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = Qt::Window);
+    dspPoHistory(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::Window);
 
     virtual bool setParams(ParameterList &);
 
