@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -13,7 +13,7 @@
 #include <QMessageBox>
 #include <QSqlError>
 
-#include "guiclient.h"
+#include "xtreewidget.h"
 
 dspPoItemReceivingsByItem::dspPoItemReceivingsByItem(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspPoItemReceivingsByItem", fl)
@@ -58,8 +58,25 @@ void dspPoItemReceivingsByItem::languageChange()
   retranslateUi(this);
 }
 
+enum SetResponse dspPoItemReceivingsByItem::set(const ParameterList &pParams)
+{
+  display::set(pParams);
+  QVariant param;
+  bool valid;
+
+  param = pParams.value("item_id", &valid);
+
+  if (valid)
+    _item->setId(param.toInt());
+
+  return NoError;
+}
+
 bool dspPoItemReceivingsByItem::setParams(ParameterList &pParams)
 {
+  if (!display::setParams(pParams))
+    return false;
+
   if (!_item->isValid())
   {
     QMessageBox::warning( this, tr("Enter Item Number"),
