@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -19,6 +19,7 @@
 #include "guiclient.h"
 #include "poLiabilityDistrib.h"
 #include "voucher.h"
+#include "errorReporter.h"
 
 dspPoItemReceivingsByVendor::dspPoItemReceivingsByVendor(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspPoItemReceivingsByVendor", fl)
@@ -65,6 +66,9 @@ void dspPoItemReceivingsByVendor::languageChange()
 
 bool dspPoItemReceivingsByVendor::setParams(ParameterList &pParams)
 {
+  if (!display::setParams(pParams))
+    return false;
+
   if (!_vendor->isValid() && _vendor->isVisible())
   {
     QMessageBox::warning( this, tr("Enter Vendor Number"),
@@ -190,9 +194,9 @@ void dspPoItemReceivingsByVendor::sMarkAsInvoiced()
     if (newdlg.exec() == XDialog::Rejected)
       update = false;
   }
-  else if (dspMarkAsInvoiced.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                dspMarkAsInvoiced, __FILE__, __LINE__))
   {
-    systemError(this, dspMarkAsInvoiced.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   if (update)
@@ -202,9 +206,9 @@ void dspPoItemReceivingsByVendor::sMarkAsInvoiced()
           "WHERE (recv_id=:recv_id); ");
     dspMarkAsInvoiced.bindValue(":recv_id",list()->id());
     dspMarkAsInvoiced.exec();
-    if (dspMarkAsInvoiced.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Item Information"),
+                                  dspMarkAsInvoiced, __FILE__, __LINE__))
     {
-      systemError(this, dspMarkAsInvoiced.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     sFillList();
@@ -228,9 +232,9 @@ bool dspPoItemReceivingsByVendor::recvHasValue()
     valq.exec();
     if (valq.first())
       hasValue = true;
-    else if (valq.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                  valq, __FILE__, __LINE__))
     {
-      systemError(this, valq.lastError().databaseText(), __FILE__, __LINE__);
       return false;
     }
   }
