@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -25,6 +25,7 @@
 #include "purchaseOrder.h"
 #include "purchaseOrderItem.h"
 #include "reschedulePoitem.h"
+#include "errorReporter.h"
 
 dspPoItemsByDate::dspPoItemsByDate(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspPoItemsByDate", fl)
@@ -69,6 +70,9 @@ void dspPoItemsByDate::languageChange()
 
 bool dspPoItemsByDate::setParams(ParameterList &params)
 {
+  if (!display::setParams(params))
+    return false;
+
   if (!_dates->startDate().isValid())
   {
     QMessageBox::warning( this, tr("Enter Start Date"),
@@ -180,9 +184,9 @@ void dspPoItemsByDate::sRunningAvailability()
     newdlg->set(params);
     omfgThis->handleNewWindow(newdlg);
   }
-  else if (availq.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                availq, __FILE__, __LINE__))
   {
-    systemError(this, availq.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -261,9 +265,9 @@ void dspPoItemsByDate::sCloseItem()
                  " WHERE (poitem_id=:poitem_id);" );
   closeq.bindValue(":poitem_id", list()->altId());
   closeq.exec();
-  if (closeq.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Item Information"),
+                                closeq, __FILE__, __LINE__))
   {
-    systemError(this, closeq.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   sFillList();
@@ -277,9 +281,9 @@ void dspPoItemsByDate::sOpenItem()
                 " WHERE (poitem_id=:poitem_id);" );
   openq.bindValue(":poitem_id", list()->altId());
   openq.exec();
-  if (openq.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Item Information"),
+                                openq, __FILE__, __LINE__))
   {
-    systemError(this, openq.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   sFillList();
