@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -22,6 +22,7 @@
 #include "purchaseOrder.h"
 #include "purchaseOrderItem.h"
 #include "reschedulePoitem.h"
+#include "errorReporter.h"
 
 dspPoItemsByItem::dspPoItemsByItem(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspPoItemsByItem", fl)
@@ -78,9 +79,9 @@ enum SetResponse dspPoItemsByItem::set(const ParameterList &pParams)
     dspet.exec();
     if (dspet.first())
       _item->setId(dspet.value("itemsrc_item_id").toInt());
-    else if (dspet.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                  dspet, __FILE__, __LINE__))
     {
-      systemError(this, dspet.lastError().databaseText(), __FILE__, __LINE__);
       return UndefinedError;
     }
   }
@@ -96,6 +97,9 @@ enum SetResponse dspPoItemsByItem::set(const ParameterList &pParams)
 
 bool dspPoItemsByItem::setParams(ParameterList &params)
 {
+  if (!display::setParams(params))
+    return false;
+
   if (!_item->isValid())
   {
     QMessageBox::warning( this, tr("Enter Item Number"),
@@ -199,9 +203,9 @@ void dspPoItemsByItem::sRunningAvailability()
     newdlg->set(params);
     omfgThis->handleNewWindow(newdlg);
   }
-  else if (dspRA.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                dspRA, __FILE__, __LINE__))
   {
-    systemError(this, dspRA.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -280,9 +284,9 @@ void dspPoItemsByItem::sCloseItem()
              "WHERE (poitem_id=:poitem_id);" );
   dspCloseItem.bindValue(":poitem_id", list()->altId());
   dspCloseItem.exec();
-  if (dspCloseItem.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Item Information"),
+                                dspCloseItem, __FILE__, __LINE__))
   {
-    systemError(this, dspCloseItem.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   sFillList();
@@ -296,9 +300,9 @@ void dspPoItemsByItem::sOpenItem()
              "WHERE (poitem_id=:poitem_id);" );
   dspOpenItem.bindValue(":poitem_id", list()->altId());
   dspOpenItem.exec();
-  if (dspOpenItem.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Item Information"),
+                                dspOpenItem, __FILE__, __LINE__))
   {
-    systemError(this, dspOpenItem.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   sFillList();
