@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -22,6 +22,7 @@
 #include "reschedulePoitem.h"
 #include "changePoitemQty.h"
 #include "dspRunningAvailability.h"
+#include "errorReporter.h"
 
 dspPoItemsByVendor::dspPoItemsByVendor(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspPoItemsByVendor", fl)
@@ -64,6 +65,9 @@ void dspPoItemsByVendor::languageChange()
 
 bool dspPoItemsByVendor::setParams(ParameterList & params)
 {
+  if (!display::setParams(params))
+    return false;
+
   if (!_vendor->isValid())
   {
     QMessageBox::warning( this, tr("Enter Vendor Number"),
@@ -170,9 +174,9 @@ void dspPoItemsByVendor::sRunningAvailability()
     newdlg->set(params);
     omfgThis->handleNewWindow(newdlg);
   }
-  else if (raq.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                raq, __FILE__, __LINE__))
   {
-    systemError(this, raq.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -251,9 +255,9 @@ void dspPoItemsByVendor::sCloseItem()
                  "WHERE (poitem_id=:poitem_id);" );
   closeq.bindValue(":poitem_id", list()->altId());
   closeq.exec();
-  if (closeq.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Item Information"),
+                                closeq, __FILE__, __LINE__))
   {
-    systemError(this, closeq.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   sFillList();
@@ -267,9 +271,9 @@ void dspPoItemsByVendor::sOpenItem()
                  "WHERE (poitem_id=:poitem_id);" );
   openq.bindValue(":poitem_id", list()->altId());
   openq.exec();
-  if (openq.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Item Information"),
+                                openq, __FILE__, __LINE__))
   {
-    systemError(this, openq.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   sFillList();
@@ -289,9 +293,9 @@ void dspPoItemsByVendor::sPopulatePo()
     poq.bindValue(":vend_id", _vendor->vendId());
     poq.exec();
     _poNumber->populate(poq);
-    if (poq.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                  poq, __FILE__, __LINE__))
     {
-      systemError(this, poq.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
