@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -40,12 +40,10 @@ dspPricesByCustomerType::dspPricesByCustomerType(QWidget* parent, const char*, Q
   list()->addColumn(tr("Price UOM"),   _uomColumn,      Qt::AlignCenter, true,  "priceuom" );
   list()->addColumn(tr("Qty. Break"),  _qtyColumn,      Qt::AlignRight,  true,  "f_qtybreak" );
   list()->addColumn(tr("Price"),       _priceColumn,    Qt::AlignRight,  true,  "price" );
-  list()->addColumn(tr("Currency"),    _currencyColumn, Qt::AlignLeft,   true,  "currConcat" );
+  list()->addColumn(tr("Currency"),    _currencyColumn, Qt::AlignLeft,   !omfgThis->singleCurrency(),  "currConcat" );
   list()->addColumn(tr("Ext. Cost"),   _costColumn,     Qt::AlignRight,  true,  "f_cost" );
   list()->addColumn(tr("Mar. %"),      _prcntColumn,    Qt::AlignRight,  true,  "f_margin" );
 
-  if (omfgThis->singleCurrency())
-    list()->hideColumn(CURR_COL);
   sHandleCosts(_showCosts->isChecked());
 }
 
@@ -72,6 +70,9 @@ void dspPricesByCustomerType::sHandleCosts(bool pShowCosts)
 
 bool dspPricesByCustomerType::setParams(ParameterList & params)
 {
+  if (!display::setParams(params))
+    return false;
+
   if(!_custtype->isValid())
   {
     QMessageBox::warning(this, tr("Customer Type Required"),
