@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -21,6 +21,8 @@
 #include "dspRunningAvailability.h"
 #include "purchaseOrder.h"
 #include "purchaseRequest.h"
+#include "salesOrderItem.h"
+#include "errorReporter.h"
 
 dspPurchaseReqsByItem::dspPurchaseReqsByItem(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspPurchaseReqsByItem", fl)
@@ -48,10 +50,11 @@ dspPurchaseReqsByItem::dspPurchaseReqsByItem(QWidget* parent, const char*, Qt::W
 
   if (_privileges->check("MaintainPurchaseRequests"))
     connect(list(), SIGNAL(itemSelected(int)), this, SLOT(sEdit()));
-  else
-    newAction()->setEnabled(false);
+
+  newAction()->setEnabled(false);
   
   connect(omfgThis, SIGNAL(purchaseRequestsUpdated()), this, SLOT(sFillList()));
+  connect(_item,    SIGNAL(valid(bool)), newAction(), SLOT(setEnabled(bool)));
 }
 
 void dspPurchaseReqsByItem::languageChange()
@@ -62,6 +65,9 @@ void dspPurchaseReqsByItem::languageChange()
 
 bool dspPurchaseReqsByItem::setParams(ParameterList &params)
 {
+  if (!display::setParams(params))
+    return false;
+
   if (! _item->isValid())
   {
     QMessageBox::information(this, tr("Item Required"),
@@ -78,8 +84,12 @@ bool dspPurchaseReqsByItem::setParams(ParameterList &params)
   return true;
 }
 
-void dspPurchaseReqsByItem::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem *, int)
+void dspPurchaseReqsByItem::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem* pItem, int)
 {
+  XTreeWidgetItem *item = dynamic_cast<XTreeWidgetItem *>(pItem);
+  if(0 == item)
+    return;
+
   QAction *menuItem;
 
   menuItem = pMenu->addAction(tr("Running Availability..."), this, SLOT(sDspRunningAvailability()));
@@ -87,11 +97,21 @@ void dspPurchaseReqsByItem::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem *, int)
 
   pMenu->addSeparator();
 
+  menuItem = pMenu->addAction(tr("Edit P/R..."), this, SLOT(sEdit()));
+  menuItem->setEnabled(_privileges->check("MaintainPurchaseRequests"));
+
   menuItem = pMenu->addAction(tr("Release P/R..."), this, SLOT(sRelease()));
   menuItem->setEnabled(_privileges->check("MaintainPurchaseOrders"));
 
   menuItem = pMenu->addAction(tr("Delete P/R..."), this, SLOT(sDelete()));
   menuItem->setEnabled(_privileges->check("MaintainPurchaseRequests"));
+
+  if(item->rawValue("parent").toString().indexOf("S/O") == 0)
+  {
+    pMenu->addSeparator();
+    menuItem = pMenu->addAction(tr("View S/O Item"), this, SLOT(sViewSOItem()));
+    menuItem->setEnabled(_privileges->check("MaintainSalesOrders"));
+  }
 }
 
 void dspPurchaseReqsByItem::sDspRunningAvailability()
@@ -145,9 +165,9 @@ void dspPurchaseReqsByItem::sRelease()
                                         "could not be released.").arg(selected[i]->rawValue("pr_number").toString()),
                                      QMessageBox::Ok|QMessageBox::Default);
         }
-        else if (dspRelease.lastError().type() != QSqlError::NoError)
+        else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Releasing Purchase Request(s)"),
+                                      dspRelease, __FILE__, __LINE__))
         {
-          systemError(this, dspRelease.lastError().databaseText(), __FILE__, __LINE__);
           return;
         }
       }
@@ -196,3 +216,26 @@ void dspPurchaseReqsByItem::sDelete()
   omfgThis->sPurchaseRequestsUpdated();
 }
 
+void dspPurchaseReqsByItem::sViewSOItem()
+{
+  XSqlQuery soitem;
+  soitem.prepare("SELECT pr_order_id AS so FROM pr WHERE pr_id=:pr_id");
+  soitem.bindValue(":pr_id", list()->id());
+  soitem.exec();
+  if (soitem.first())
+  {
+    ParameterList params;
+    params.append("soitem_id", soitem.value("so"));
+    params.append("mode", "view");
+
+    salesOrderItem *newdlg = new salesOrderItem();
+    newdlg->set(params);
+    omfgThis->handleNewWindow(newdlg);
+  }
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error opening Sales Order item"),
+                                      soitem, __FILE__, __LINE__))
+  {
+    return;
+  }
+}
+
