@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -20,7 +20,7 @@ class dspPurchaseReqsByPlannerCode : public display, public Ui::dspPurchaseReqsB
     Q_OBJECT
 
 public:
-    dspPurchaseReqsByPlannerCode(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = Qt::Window);
+    dspPurchaseReqsByPlannerCode(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::Window);
 
     virtual bool setParams(ParameterList &);
 
@@ -28,7 +28,10 @@ public slots:
     virtual void sPopulateMenu(QMenu * pMenu, QTreeWidgetItem *, int);
     virtual void sDspRunningAvailability();
     virtual void sRelease();
+    virtual void sNew();
+    virtual void sEdit();
     virtual void sDelete();
+    virtual void sViewSOItem();
 
 protected slots:
     virtual void languageChange();
