@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -20,6 +20,7 @@
 #include "createCountTagsByItem.h"
 #include "dspInventoryLocator.h"
 #include "parameterwidget.h"
+#include "xtreewidget.h"
 
 dspQOH::dspQOH(QWidget* parent, const char*, Qt::WindowFlags fl)
     : display(parent, "dspQOH", fl)
@@ -41,6 +42,13 @@ dspQOH::dspQOH(QWidget* parent, const char*, Qt::WindowFlags fl)
   _showGroupInt->addButton(_showPositive);
   _showGroupInt->addButton(_showNegative);
 
+  QString qryABC = QString( "SELECT  'A', '%1' UNION "
+                            "SELECT  'B', '%2' UNION "
+                            "SELECT  'C', '%3' ORDER BY 1;")
+      .arg(tr("A Class"))
+      .arg(tr("B Class"))
+      .arg(tr("C Class"));
+
   parameterWidget()->appendComboBox(tr("Class Code"), "classcode_id", XComboBox::ClassCodes);
   parameterWidget()->append(tr("Class Code Pattern"), "classcode_pattern", ParameterWidget::Text);
   parameterWidget()->appendComboBox(tr("Cost Category"), "costcat_id", XComboBox::CostCategories);
@@ -48,6 +56,9 @@ dspQOH::dspQOH(QWidget* parent, const char*, Qt::WindowFlags fl)
   parameterWidget()->append(tr("Item"), "item_id", ParameterWidget::Item);
   parameterWidget()->appendComboBox(tr("Item Group"), "itemgrp_id", XComboBox::ItemGroups);
   parameterWidget()->append(tr("Item Group Pattern"), "itemgrp_pattern", ParameterWidget::Text);
+  parameterWidget()->append(tr("ABC Class"), "abc_class",
+                           ParameterWidget::Multiselect, QVariant(), false,
+                           qryABC);
   if (_metrics->boolean("MultiWhs"))
     parameterWidget()->append(tr("Site"), "warehous_id", ParameterWidget::Site);
   if (_metrics->boolean("EnableAsOfQOH"))
@@ -60,6 +71,7 @@ dspQOH::dspQOH(QWidget* parent, const char*, Qt::WindowFlags fl)
   list()->addColumn(tr("Description"),        -1,          Qt::AlignLeft,   true,  "itemdescrip"   );
   list()->addColumn(tr("Class Code"),         _itemColumn, Qt::AlignLeft,   true,  "classcode_code"   );
   list()->addColumn(tr("Cost Category"),      _itemColumn, Qt::AlignLeft,   false, "costcat_code"   );
+  list()->addColumn(tr("ABC Class"),          _itemColumn, Qt::AlignLeft,   false, "itemsite_abcclass"   );
   list()->addColumn(tr("UOM"),                _uomColumn,  Qt::AlignCenter, true,  "uom_name" );
   list()->addColumn(tr("Default Location"),   _itemColumn, Qt::AlignLeft,   true,  "defaultlocation"   );
   list()->addColumn(tr("Reorder Lvl."),       _qtyColumn,  Qt::AlignRight,  true,  "reorderlevel"  );
@@ -127,10 +139,11 @@ SetResponse dspQOH::set(const ParameterList &pParams)
   if (valid)
     parameterWidget()->setDefault(tr("Site"), param);
 
-  parameterWidget()->applyDefaultFilterSet();
-
   if (pParams.inList("run"))
+  {
+    parameterWidget()->applyDefaultFilterSet();
     sFillList();
+  }
 
   return NoError;
 }
