@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -30,7 +30,7 @@ dspQOHByLocation::dspQOHByLocation(QWidget* parent, const char*, Qt::WindowFlags
 
   connect(_warehouse, SIGNAL(updated()), this, SLOT(sPopulateLocations()));
 
-  omfgThis->inputManager()->notify(cBCLocation, this, this, SLOT(set(int)));
+  omfgThis->inputManager()->notify(cBCLocation, this, this, SLOT(setLocation(int)));
 
   _location->setAllowNull(true);
   _asOf->setDate(omfgThis->dbDate(), true);
@@ -72,17 +72,7 @@ enum SetResponse dspQOHByLocation::set(const ParameterList &pParams)
   param = pParams.value("location_id", &valid);
   if (valid)
   {
-    XSqlQuery qq;
-    qq.prepare( "SELECT location_warehous_id "
-                "FROM location "
-                "WHERE (location_id=:location_id);" );
-    qq.bindValue(":location_id", param.toInt());
-    qq.exec();
-    if (qq.first())
-    {
-      _warehouse->setId(qq.value("location_warehous_id").toInt());
-      _location->setId(param.toInt());
-    }
+    setLocation(param.toInt());
   }
   
   if (pParams.inList("run"))
@@ -94,6 +84,21 @@ enum SetResponse dspQOHByLocation::set(const ParameterList &pParams)
   return NoError;
 }
 
+void dspQOHByLocation::setLocation(int pId)
+{
+  XSqlQuery qq;
+  qq.prepare( "SELECT location_warehous_id "
+              "FROM location "
+              "WHERE (location_id=:location_id);" );
+  qq.bindValue(":location_id", pId);
+  qq.exec();
+  if (qq.first())
+  {
+    _warehouse->setId(qq.value("location_warehous_id").toInt());
+    _location->setId(pId);
+  }
+}
+
 void dspQOHByLocation::sPopulateLocations()
 {
   if (_warehouse->isAll())
@@ -102,7 +107,8 @@ void dspQOHByLocation::sPopulateLocations()
                          "            ELSE (warehous_code || '-' || formatLocationName(location_id))"
                          "       END AS locationname "
                          "FROM location, whsinfo "
-                         "WHERE (location_warehous_id=warehous_id) "
+                         "WHERE ((location_warehous_id=warehous_id) "
+                         " AND   (location_active) ) "
                          "ORDER BY locationname;" );
   else
   {
@@ -112,7 +118,8 @@ void dspQOHByLocation::sPopulateLocations()
                 "            ELSE formatLocationName(location_id)"
                 "       END AS locationname "
                 "FROM location "
-                "WHERE (location_warehous_id=:warehous_id) "
+                "WHERE ((location_warehous_id=:warehous_id) "
+                " AND   (location_active) ) "
                 "ORDER BY locationname;" );
     qq.bindValue(":warehous_id", _warehouse->id());
     qq.exec();
@@ -189,6 +196,9 @@ void dspQOHByLocation::sFillList()
 
 bool dspQOHByLocation::setParams(ParameterList &params)
 {
+  if (!display::setParams(params))
+    return false;
+
   params.append("byLocation");
   params.append("asOf", _asOf->date());
 
@@ -199,6 +209,9 @@ bool dspQOHByLocation::setParams(ParameterList &params)
 
   _warehouse->appendValue(params);
 
+  if (_metrics->boolean("MultiWhs"))
+    params.append("MultiWhs");
+
   if (_location->id() != -1)
     params.append("location_id", _location->id());
 
