@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -22,6 +22,7 @@
 #include "salesOrder.h"
 #include "transferOrder.h"
 #include "workOrder.h"
+#include "errorReporter.h"
 
 dspReservations::dspReservations(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspReservations", fl)
@@ -185,6 +186,9 @@ void dspReservations::sEditTransferOrder()
 
 bool dspReservations::setParams(ParameterList &params)
 {
+  if (!display::setParams(params))
+    return false;
+
   if(!_item->isValid())
   {
     QMessageBox::warning(this, tr("Item Required"),
@@ -220,9 +224,9 @@ void dspReservations::sFillList()
       _availableqoh->setDouble(dspFillList.value("availableqoh").toDouble());
       _available->setDouble(dspFillList.value("unreserved").toDouble());
     }
-    else if (dspFillList.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                  dspFillList, __FILE__, __LINE__))
     {
-      systemError(this, dspFillList.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
