@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -23,6 +23,7 @@
 #include "transferOrder.h"
 #include "workOrder.h"
 #include "purchaseOrder.h"
+#include "errorReporter.h"
 
 dspRunningAvailability::dspRunningAvailability(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspRunningAvailability", fl)
@@ -46,6 +47,8 @@ dspRunningAvailability::dspRunningAvailability(QWidget* parent, const char*, Qt:
   list()->addColumn(tr("Amount"),                 _moneyColumn, Qt::AlignRight, true, "amount");
   list()->addColumn(tr("Ordered"),                _qtyColumn,   Qt::AlignRight, true, "qtyordered");
   list()->addColumn(tr("Received"),               _qtyColumn,   Qt::AlignRight, true, "qtyreceived");
+  if (_metrics->boolean("EnableSOReservations"))
+    list()->addColumn(tr("Reserved"),             _qtyColumn,   Qt::AlignRight, true, "reserved");
   list()->addColumn(tr("Balance"),                _qtyColumn,   Qt::AlignRight, true, "balance");
   list()->addColumn(tr("Running Avail."),         _qtyColumn,   Qt::AlignRight, true, "runningavail");
   list()->addColumn(tr("Running Netable"),        _qtyColumn,   Qt::AlignRight, true, "runningnetable");
@@ -95,6 +98,9 @@ enum SetResponse dspRunningAvailability::set(const ParameterList &pParams)
 
 bool dspRunningAvailability::setParams(ParameterList & params)
 {
+  if (!display::setParams(params))
+    return false;
+
   if(!_item->isValid())
   {
     QMessageBox::warning(this, tr("Item Required"),
@@ -121,6 +127,9 @@ bool dspRunningAvailability::setParams(ParameterList & params)
   if (_metrics->boolean("MultiWhs"))
     params.append("MultiWhs");
 
+  if (_metrics->boolean("EnableSOReservations"))
+    params.append("showReserved");
+  
   if (_metrics->value("Application") == "Standard")
   {
     XSqlQuery xtmfg;
@@ -208,9 +217,9 @@ void dspRunningAvailability::sSoftenOrder()
              "WHERE (planord_id=:planord_id);" );
   dspSoftenOrder.bindValue(":planord_id", list()->id());
   dspSoftenOrder.exec();
-  if (dspSoftenOrder.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Planned Order Information"),
+                                dspSoftenOrder, __FILE__, __LINE__))
   {
-    systemError(this, dspSoftenOrder.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -270,9 +279,9 @@ void dspRunningAvailability::sDeleteOrder()
     }
     */
   }
-  else if (dspDeleteOrder.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Planned Order Information"),
+                                dspDeleteOrder, __FILE__, __LINE__))
   {
-    systemError(this, dspDeleteOrder.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -375,9 +384,9 @@ void dspRunningAvailability::sFillList()
 
       display::sFillList();
     }
-    else if (dspFillList.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                  dspFillList, __FILE__, __LINE__))
     {
-      systemError(this, dspFillList.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
