@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -15,6 +15,7 @@
 #include "errorReporter.h"
 #include "inputManager.h"
 #include "salesOrderList.h"
+#include "xtreewidget.h"
 
 dspSalesOrderStatus::dspSalesOrderStatus(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspSalesOrderStatus", fl)
@@ -26,6 +27,8 @@ dspSalesOrderStatus::dspSalesOrderStatus(QWidget* parent, const char*, Qt::Windo
   setMetaSQLOptions("salesOrderStatus", "detail");
 
   _so->setAllowedTypes(OrderLineEdit::Sales);
+  
+  _so->setFocus();
 
   omfgThis->inputManager()->notify(cBCSalesOrder, this, _so, SLOT(setId(int)));
 
@@ -41,7 +44,6 @@ dspSalesOrderStatus::dspSalesOrderStatus(QWidget* parent, const char*, Qt::Windo
   list()->addColumn(tr("Close Date"),          _dateColumn, Qt::AlignLeft,   true,  "closedate"  );
   list()->addColumn(tr("Close User"),          _itemColumn, Qt::AlignLeft,   true,  "closeuser"  );
   list()->addColumn(tr("Child Ord. #/Status"), _itemColumn, Qt::AlignCenter, true,  "childinfo" );
-
 }
 
 void dspSalesOrderStatus::languageChange()
@@ -58,13 +60,19 @@ enum SetResponse dspSalesOrderStatus::set(const ParameterList &pParams)
 
   param = pParams.value("sohead_id", &valid);
   if (valid)
+  {
     _so->setId(param.toInt());
+    setQueryOnStartEnabled(true);
+  }
 
   return NoError;
 }
 
 bool dspSalesOrderStatus::setParams(ParameterList &params)
 {
+  if (!display::setParams(params))
+    return false;
+
   params.append("sohead_id", _so->id());
 
   return true;
