@@ -33,6 +33,7 @@ dspSalesOrders::dspSalesOrders(QWidget* parent, const char*, Qt::WindowFlags fl)
     setNewVisible(true);
   setQueryOnStartEnabled(false);
   setAutoUpdateEnabled(true);
+  setSearchVisible(true);
 
   if (_metrics->boolean("MultiWhs"))
     parameterWidget()->append(tr("Site"), "warehous_id", ParameterWidget::Site);
@@ -43,6 +44,7 @@ dspSalesOrders::dspSalesOrders(QWidget* parent, const char*, Qt::WindowFlags fl)
   parameterWidget()->append(tr("End Date"),   "endDate",   ParameterWidget::Date, QDate::currentDate());
   parameterWidget()->append(tr("P/O Number"), "poNumber", ParameterWidget::Text);
   parameterWidget()->append(tr("Project"), "prj_id", ParameterWidget::Project);
+  parameterWidget()->append(tr("Created By"), "createdby", ParameterWidget::User);
   parameterWidget()->appendComboBox(tr("Sales Rep."), "salesrep_id", XComboBox::SalesRepsActive);
   parameterWidget()->appendComboBox(tr("Sale Type"), "saletype_id", XComboBox::SaleTypes);
   parameterWidget()->append(tr("Hide Closed"), "hideClosed", ParameterWidget::Exists);
@@ -56,7 +58,6 @@ dspSalesOrders::dspSalesOrders(QWidget* parent, const char*, Qt::WindowFlags fl)
   list()->addColumn(tr("Ship-to"),     -1,           Qt::AlignLeft,   true,  "cohead_shiptoname"   );
   list()->addColumn(tr("Cust. P/O #"), 200,          Qt::AlignLeft,   true,  "cohead_custponumber"   );
 
-  parameterWidget()->applyDefaultFilterSet();
   connect(omfgThis, SIGNAL(salesOrdersUpdated(int, bool)), this, SLOT(sFillList())  );
 }
 
@@ -92,7 +93,8 @@ void dspSalesOrders::sPopulateMenu(QMenu *menuThis, QTreeWidgetItem*, int)
 
 bool dspSalesOrders::setParams(ParameterList & params)
 {
-  parameterWidget()->appendValue(params);
+  if (!display::setParams(params))
+    return false;
   params.append("noLines", tr("No Lines"));
   params.append("closed", tr("Closed"));
   params.append("open", tr("Open"));
