@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -37,13 +37,19 @@ dspSalesOrdersByItem::dspSalesOrdersByItem(QWidget* parent, const char*, Qt::Win
 
   _item->setType(ItemLineEdit::cSold);
 
-  list()->addColumn(tr("Order #"),    _orderColumn, Qt::AlignLeft,   true,  "cohead_number"   );
-  list()->addColumn(tr("Order Date"), _dateColumn,  Qt::AlignCenter, true,  "cohead_orderdate" );
-  list()->addColumn(tr("Customer"),   -1,           Qt::AlignLeft,   true,  "cust_name"   );
-  list()->addColumn(tr("Ordered"),    _qtyColumn,   Qt::AlignRight,  true,  "coitem_qtyord"  );
-  list()->addColumn(tr("Shipped"),    _qtyColumn,   Qt::AlignRight,  true,  "coitem_qtyshipped"  );
-  list()->addColumn(tr("Returned"),   _qtyColumn,   Qt::AlignRight,  true,  "coitem_qtyreturned"  );
-  list()->addColumn(tr("Balance"),    _qtyColumn,   Qt::AlignRight,  true,  "qtybalance"  );
+  list()->addColumn(tr("Order #"),         _orderColumn, Qt::AlignLeft,   true,  "cohead_number"   );
+  list()->addColumn(tr("Order Date"),      _dateColumn,  Qt::AlignCenter, true,  "cohead_orderdate" );
+  list()->addColumn(tr("Customer"),        -1,           Qt::AlignLeft,   true,  "cust_name"   );
+  list()->addColumn(tr("Order UOM"),       _uomColumn,   Qt::AlignCenter, true,  "uom_name" );
+  list()->addColumn(tr("Ordered"),         _qtyColumn,   Qt::AlignRight,  true,  "coitem_qtyord"  );
+  list()->addColumn(tr("Shipped"),         _qtyColumn,   Qt::AlignRight,  true,  "coitem_qtyshipped"  );
+  list()->addColumn(tr("Returned"),        _qtyColumn,   Qt::AlignRight,  true,  "coitem_qtyreturned"  );
+  list()->addColumn(tr("Balance"),         _qtyColumn,   Qt::AlignRight,  true,  "qtybalance"  );
+  list()->addColumn(tr("Inv. UOM"),        _uomColumn,   Qt::AlignCenter, false, "inv_uom_name" );
+  list()->addColumn(tr("Inv. Ordered"),    _qtyColumn,   Qt::AlignRight,  false, "invqtyord"  );
+  list()->addColumn(tr("Inv. Shipped"),    _qtyColumn,   Qt::AlignRight,  false, "invqtyshipped"  );
+  list()->addColumn(tr("Inv. Returned"),   _qtyColumn,   Qt::AlignRight,  false, "invqtyreturned"  );
+  list()->addColumn(tr("Inv. Balance"),    _qtyColumn,   Qt::AlignRight,  false, "invqtybalance"  );
 
   connect(omfgThis, SIGNAL(salesOrdersUpdated(int, bool)), this, SLOT(sFillList()));
 }
@@ -172,6 +178,9 @@ void dspSalesOrdersByItem::sDspShipments()
 
 bool dspSalesOrdersByItem::setParams(ParameterList &params)
 {
+  if (!display::setParams(params))
+    return false;
+
   if (!_item->isValid())
   {
     QMessageBox::warning(this, tr("Item Required"),
@@ -187,6 +196,7 @@ bool dspSalesOrdersByItem::setParams(ParameterList &params)
   }
 
   _dates->appendValue(params);
+  params.append("orderByOrderdate", true);
   params.append("closed", tr("Closed"));
   params.append("item_id", _item->id());
 
