@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -24,6 +24,7 @@
 #include "inputManager.h"
 #include "printShippingForm.h"
 
+
 dspShipmentsBase::dspShipmentsBase(QWidget* parent, const char* name, Qt::WindowFlags fl)
   : display(parent, name, fl)
 {
@@ -71,6 +72,7 @@ enum SetResponse dspShipmentsBase::set(const ParameterList &pParams)
   {
     _salesOrder->setId(param.toInt());
     _salesOrder->setEnabled(false);
+    setQueryOnStartEnabled(true);
   }
 
   param = pParams.value("shiphead_id", &valid);
@@ -78,6 +80,7 @@ enum SetResponse dspShipmentsBase::set(const ParameterList &pParams)
   {
     _shipment->setId(param.toInt());
     _shipment->setEnabled(false);
+    setQueryOnStartEnabled(true);
   }
 
   return NoError;
@@ -95,6 +98,9 @@ void dspShipmentsBase::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem *, int)
 
 bool dspShipmentsBase::setParams(ParameterList& params)
 {
+  if (!display::setParams(params))
+    return false;
+
   if(_salesOrder->isVisibleTo(this))
     params.append("sohead_id", _salesOrder->id());
 
@@ -328,9 +334,9 @@ void dspShipmentsBase::sFillURL()
                                  "the shipper %1. The tracking number is %2.")
                                   .arg(shipvia, tracknum));
   }
-  else if (shq.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipment Information"),
+                                shq, __FILE__, __LINE__))
   {
-    systemError(this, shq.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
