@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -19,12 +19,12 @@
 #include <openreports.h>
 #include <parameter.h>
 
-#include "guiclient.h"
 #include "adjustmentTrans.h"
 #include "enterMiscCount.h"
 #include "transferTrans.h"
 #include "createCountTagsByItem.h"
 #include "mqlutil.h"
+#include "xtreewidget.h"
 
 dspSlowMovingInventoryByClassCode::dspSlowMovingInventoryByClassCode(QWidget* parent, const char*, Qt::WindowFlags fl)
     : display(parent, "dspSlowMovingInventoryByClassCode", fl)
@@ -38,6 +38,7 @@ dspSlowMovingInventoryByClassCode::dspSlowMovingInventoryByClassCode(QWidget* pa
   _costsGroupInt = new QButtonGroup(this);
   _costsGroupInt->addButton(_useStandardCosts);
   _costsGroupInt->addButton(_useActualCosts);
+  _costsGroupInt->addButton(_usePostedCosts);
 
   connect(_showValue, SIGNAL(toggled(bool)), this, SLOT(sHandleValue(bool)));
 
@@ -65,6 +66,9 @@ void dspSlowMovingInventoryByClassCode::languageChange()
 
 bool dspSlowMovingInventoryByClassCode::setParams(ParameterList & params)
 {
+  if (!display::setParams(params))
+    return false;
+
   if(!_cutoffDate->isValid())
   {
     QMessageBox::warning(this, tr("No Cutoff Date"),
@@ -77,14 +81,18 @@ bool dspSlowMovingInventoryByClassCode::setParams(ParameterList & params)
   _warehouse->appendValue(params);
   _classCode->appendValue(params);
 
+  if(_tolerance->value() > 0)
+    params.append("tolerance", _tolerance->value());
+
   if(_showValue->isChecked())
     params.append("showValue");
 
   if (_useStandardCosts->isChecked())
     params.append("useStandardCosts");
-
-  if (_useActualCosts->isChecked())
+  else if (_useActualCosts->isChecked())
     params.append("useActualCosts");
+  else if (_usePostedCosts->isChecked())
+    params.append("usePostedCosts");
 
   return true;
 }
