@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -11,6 +11,7 @@
 #include "dspSummarizedBOM.h"
 
 #include <QVariant>
+#include "xtreewidget.h"
 
 dspSummarizedBOM::dspSummarizedBOM(QWidget* parent, const char*, Qt::WindowFlags fl)
   : dspBOMBase(parent, "dspSummarizedBOM", fl)
@@ -28,6 +29,9 @@ dspSummarizedBOM::dspSummarizedBOM(QWidget* parent, const char*, Qt::WindowFlags
 
 bool dspSummarizedBOM::setParams(ParameterList &params)
 {
+  if (!display::setParams(params))
+    return false;
+
   params.append("bySummarized");
   return dspBOMBase::setParams(params);
 }
