@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -16,6 +16,7 @@
 
 #include "xtreewidget.h"
 #include "xmessagebox.h"
+#include "errorReporter.h"
 
 dspSummarizedBankrecHistory::dspSummarizedBankrecHistory(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspSummarizedBankrecHistory", fl)
@@ -48,6 +49,9 @@ void dspSummarizedBankrecHistory::languageChange()
 
 bool dspSummarizedBankrecHistory::setParams(ParameterList & params)
 {
+  if (!display::setParams(params))
+    return false;
+
   params.append("bankaccntid", _bankaccnt->id());
   return true;
 }
@@ -99,9 +103,9 @@ void dspSummarizedBankrecHistory::sReopen()
         return;
       }
     }
-    else if (open.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Bank Reconciliation Information"),
+                                  open, __FILE__, __LINE__))
     {
-      systemError(this, open.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     sFillList();
