@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -34,6 +34,7 @@ dspSummarizedSales::dspSummarizedSales(QWidget* parent, const char*, Qt::WindowF
   parameterWidget()->appendComboBox(tr("Currency"), "curr_id", XComboBox::Currencies);
   parameterWidget()->append(tr("Currency Pattern"), "currConcat_pattern", ParameterWidget::Text);
   parameterWidget()->append(tr("Customer"),   "cust_id",   ParameterWidget::Customer);
+  parameterWidget()->append(tr("Customer P/O Number"), "poNumber", ParameterWidget::Text);
   parameterWidget()->append(tr("Customer Ship-to"),   "shipto_id",   ParameterWidget::Shipto);
   parameterWidget()->appendComboBox(tr("Customer Group"), "custgrp_id", XComboBox::CustomerGroups);
   parameterWidget()->append(tr("Customer Group Pattern"), "custgrp_pattern", ParameterWidget::Text);
@@ -43,12 +44,11 @@ dspSummarizedSales::dspSummarizedSales(QWidget* parent, const char*, Qt::WindowF
   parameterWidget()->appendComboBox(tr("Product Category"), "prodcat_id", XComboBox::ProductCategories);
   parameterWidget()->append(tr("Product Category Pattern"), "prodcat_pattern", ParameterWidget::Text);
   parameterWidget()->appendComboBox(tr("Sales Rep."), "salesrep_id", XComboBox::SalesReps);
+  parameterWidget()->appendComboBox(tr("Sale Type"), "saletype_id", XComboBox::SaleTypes);
   parameterWidget()->append(tr("Include Misc. Items"), "includeMisc", ParameterWidget::Exists, true);
   if (_metrics->boolean("MultiWhs"))
     parameterWidget()->append(tr("Site"), "warehous_id", ParameterWidget::Site);
 
-  parameterWidget()->applyDefaultFilterSet();
-
   _units->append(0,tr("Base"));
   _units->append(1,tr("Local"));
 
@@ -86,6 +86,9 @@ void dspSummarizedSales::languageChange()
 
 bool dspSummarizedSales::setParams(ParameterList & params)
 {
+  if (!display::setParams(params))
+    return false;
+
   if (!_cust->isChecked() &&
       !_custtype->isChecked() &&
       !_salesrep->isChecked() &&
@@ -160,9 +163,9 @@ bool dspSummarizedSales::setParams(ParameterList & params)
 
 void dspSummarizedSales::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem*, int)
 {
-  QAction *menuItem;
+  //QAction *menuItem;
 
-  menuItem = pMenu->addAction(tr("View Sales Detail..."), this, SLOT(sViewHistory()));
+  (void)pMenu->addAction(tr("View Sales Detail..."), this, SLOT(sViewHistory()));
 
 }
 
@@ -183,11 +186,11 @@ void dspSummarizedSales::sViewHistory()
       params.append("item_id", list()->id("item_number"));
   if (_site->isChecked())
       params.append("warehous_id", list()->id("warehous_code"));
-  params.append("run");
 
   dspSalesHistory *newdlg = new dspSalesHistory();
   newdlg->set(params);
   omfgThis->handleNewWindow(newdlg);
+  newdlg->sFillList();
 }
 
 void dspSummarizedSales::sGroupByChanged()
@@ -227,8 +230,7 @@ void dspSummarizedSales::sGroupByChanged()
   list()->addColumn(tr("Wt. Avg. Price"), _priceColumn,    Qt::AlignRight,  true,  "wtavgprice"  );
   list()->addColumn(tr("Total Units"),    _qtyColumn,      Qt::AlignRight,  true,  "totalunits"  );
   list()->addColumn(tr("Total Sales"),    _bigMoneyColumn, Qt::AlignRight,  true,  "totalsales"  );
-  if (!omfgThis->singleCurrency())
-    list()->addColumn(tr("Currency"),       _currencyColumn,  Qt::AlignLeft,  true,  "currAbbr"  );
+  list()->addColumn(tr("Currency"),       _currencyColumn,  Qt::AlignLeft,  !omfgThis->singleCurrency(),  "currAbbr"  );
 
   xtsettingsSetValue("dspSummarizedSales/units", _units->id());
 }
