@@ -40,8 +40,6 @@ dspTimePhasedBookings::dspTimePhasedBookings(QWidget* parent, const char*, Qt::W
   if (_metrics->boolean("MultiWhs"))
     parameterWidget()->append(tr("Site"), "warehous_id", ParameterWidget::Site);
 
-  parameterWidget()->applyDefaultFilterSet();
-
   _groupBy->append(0,tr("Customer"));
   _groupBy->append(1,tr("Product Category"));
   _groupBy->append(2,tr("Item"));
@@ -85,12 +83,10 @@ void dspTimePhasedBookings::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem *pSelect
   if (pColumn < 4)
     return;
 
-  QAction *menuItem;
-
   _column = pColumn;
 
   if (((XTreeWidgetItem *)pSelected)->id() != -1)
-    menuItem = pMenu->addAction(tr("View Bookings..."), this, SLOT(sViewBookings()));
+    (void)pMenu->addAction(tr("View Bookings..."), this, SLOT(sViewBookings()));
 }
 
 bool dspTimePhasedBookings::setParamsTP(ParameterList & params)
