@@ -12,7 +12,7 @@
 
 #include <QAction>
 #include <QMenu>
-#include <QMessageBox>
+#include "guiErrorCheck.h"
 #include <QSqlError>
 #include <QVariant>
 
@@ -23,6 +23,7 @@
 
 #include "dspAPOpenItemsByVendor.h"
 #include "submitReport.h"
+#include "errorReporter.h"
 
 dspTimePhasedOpenAPItems::dspTimePhasedOpenAPItems(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspTimePhasedOpenAPItems", fl)
@@ -55,15 +56,17 @@ dspTimePhasedOpenAPItems::~dspTimePhasedOpenAPItems()
 
 bool dspTimePhasedOpenAPItems::setParams(ParameterList &params)
 {
-  if ((_custom->isChecked() && ! _periods->isPeriodSelected()) ||
-      (!_custom->isChecked() && ! _asOf->isValid()))
-  {
-    QMessageBox::critical(this, tr("Incomplete criteria"),
-                          tr("<p>The criteria you specified are not complete. "
-                             "Please make sure all fields are correctly filled "
-                             "out before running the report." ) );
+  if (!display::setParams(params))
+    return false;
+
+  QList<GuiErrorCheck> errors;
+  errors<< GuiErrorCheck((_custom->isChecked() && ! _periods->isPeriodSelected()) || (!_custom->isChecked() && ! _asOf->isValid()), _custom,
+              tr("The criteria you specified are not complete. "
+                  "Please make sure all fields are correctly filled "
+                  "out before running the report."))
+  ;
+  if (GuiErrorCheck::reportErrors(this, tr("Incomplete criteria"), errors))
     return false;
-  }
 
   _vendorGroup->appendValue(params);
 
@@ -160,13 +163,13 @@ void dspTimePhasedOpenAPItems::sFillList()
 void dspTimePhasedOpenAPItems::sFillCustom()
 {
   XSqlQuery dspFillCustom;
-  if (!_periods->isPeriodSelected())
-  {
-    if (isVisible())
-      QMessageBox::warning( this, tr("Select Calendar Periods"),
-                            tr("Please select one or more Calendar Periods") );
+
+  QList<GuiErrorCheck> errors;
+  errors<< GuiErrorCheck(!_periods->isPeriodSelected() && isVisible(), _periods,
+              tr("Please select one or more Calendar Periods"))
+  ;
+  if (GuiErrorCheck::reportErrors(this, tr("Select Calendar Periods"), errors))
     return;
-  }
 
   _columnDates.clear();
   list()->setColumnCount(2);
@@ -215,9 +218,9 @@ void dspTimePhasedOpenAPItems::sFillCustom()
 
   dspFillCustom = mql.toQuery(params);
   list()->populate(dspFillCustom);
-  if (dspFillCustom.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Vendor Information"),
+                                dspFillCustom, __FILE__, __LINE__))
   {
-    systemError(this, dspFillCustom.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -231,11 +234,12 @@ void dspTimePhasedOpenAPItems::sFillStd()
     return;
   dspFillStd = mql.toQuery(params);
   list()->populate(dspFillStd);
-  if (dspFillStd.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Vendor Information"),
+                                dspFillStd, __FILE__, __LINE__))
   {
-    systemError(this, dspFillStd.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
+
 }
 
 void dspTimePhasedOpenAPItems::sToggleCustom()
