@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -21,7 +21,7 @@ class dspTimePhasedOpenAPItems : public display, public Ui::dspTimePhasedOpenAPI
     Q_OBJECT
 
 public:
-    dspTimePhasedOpenAPItems(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = Qt::Window);
+    dspTimePhasedOpenAPItems(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::Window);
     ~dspTimePhasedOpenAPItems();
 
 public slots:
@@ -31,9 +31,10 @@ public slots:
     virtual void sFillStd();
     virtual void sFillCustom();
     virtual void sToggleCustom();
+    virtual void sToggleReport();
 
 protected slots:
-    virtual bool setParams(ParameterList &);
+    Q_INVOKABLE virtual bool setParams(ParameterList &);
 
 private:
     int _column;
