@@ -12,7 +12,7 @@
 
 #include <QAction>
 #include <QMenu>
-#include <QMessageBox>
+#include "guiErrorCheck.h"
 #include <QSqlError>
 #include <QVariant>
 
@@ -23,6 +23,7 @@
 
 #include "printStatementByCustomer.h"
 #include "dspAROpenItems.h"
+#include "errorReporter.h"
 
 dspTimePhasedOpenARItems::dspTimePhasedOpenARItems(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspTimePhasedOpenARItems", fl)
@@ -58,15 +59,17 @@ dspTimePhasedOpenARItems::~dspTimePhasedOpenARItems()
 
 bool dspTimePhasedOpenARItems::setParams(ParameterList &params)
 {
-  if ((_custom->isChecked() && ! _periods->isPeriodSelected()) ||
-      (!_custom->isChecked() && !_asOf->isValid()))
-  {
-    QMessageBox::critical(this, tr("Incomplete criteria"),
-                          tr("<p>The criteria you specified are not complete. "
+  if (!display::setParams(params))
+    return false;
+
+  QList<GuiErrorCheck> errors;
+  errors<< GuiErrorCheck((_custom->isChecked() && ! _periods->isPeriodSelected()) || (!_custom->isChecked() && !_asOf->isValid()), _custom,
+                         tr("The criteria you specified are not complete. "
                              "Please make sure all fields are correctly filled "
-                             "out before running the report." ) );
+                             "out before running the report."))
+  ;
+  if (GuiErrorCheck::reportErrors(this, tr("Incomplete criteria"), errors))
     return false;
-  }
 
   _customerSelector->appendValue(params);
 
@@ -148,7 +151,7 @@ void dspTimePhasedOpenARItems::sPrintStatement()
 
 void dspTimePhasedOpenARItems::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem *, int pColumn)
 {
-  QAction *menuItem;
+  //QAction *menuItem;
   _column = pColumn;
   
   if ((_custom->isChecked()) && (_column == list()->column("linetotal")))
@@ -156,11 +159,11 @@ void dspTimePhasedOpenARItems::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem *, in
 
   if ((_column > 1) && (list()->id() > 0))
   {
-    menuItem = pMenu->addAction(tr("View Open Items..."), this, SLOT(sViewOpenItems()));
+    (void)pMenu->addAction(tr("View Open Items..."), this, SLOT(sViewOpenItems()));
 
     pMenu->addSeparator();
 
-    menuItem = pMenu->addAction(tr("Print Statement..."), this, SLOT(sPrintStatement()));
+    (void)pMenu->addAction(tr("Print Statement..."), this, SLOT(sPrintStatement()));
   }
 }
 
@@ -177,10 +180,12 @@ void dspTimePhasedOpenARItems::sFillCustom()
   XSqlQuery dspFillCustom;
   if (!_periods->isPeriodSelected())
   {
-    if (isVisible())
-      QMessageBox::warning( this, tr("Select Calendar Periods"),
-                            tr("Please select one or more Calendar Periods") );
-    return;
+    QList<GuiErrorCheck> errors;
+    errors<< GuiErrorCheck(isVisible(), _periods,
+                         tr("Please select one or more Calendar Periods"))
+    ;
+    if (GuiErrorCheck::reportErrors(this, tr("Select Calendar Periods"), errors))
+     return ;
   }
 
   _columnDates.clear();
@@ -231,9 +236,9 @@ void dspTimePhasedOpenARItems::sFillCustom()
     return;
   dspFillCustom = mql.toQuery(params);
   list()->populate(dspFillCustom);
-  if (dspFillCustom.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Customer Information"),
+                                dspFillCustom, __FILE__, __LINE__))
   {
-    systemError(this, dspFillCustom.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -248,9 +253,9 @@ void dspTimePhasedOpenARItems::sFillStd()
 
   dspFillStd = mql.toQuery(params);
   list()->populate(dspFillStd);
-  if (dspFillStd.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Customer Information"),
+                                dspFillStd, __FILE__, __LINE__))
   {
-    systemError(this, dspFillStd.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
