@@ -38,11 +38,10 @@ dspTimePhasedSales::dspTimePhasedSales(QWidget* parent, const char*, Qt::WindowF
   parameterWidget()->append(tr("Item"), "item_id", ParameterWidget::Item);
   parameterWidget()->appendComboBox(tr("Product Category"), "prodcat_id", XComboBox::ProductCategories);
   parameterWidget()->append(tr("Product Category Pattern"), "prodcat_pattern", ParameterWidget::Text);
+  parameterWidget()->append(tr("Include Misc. Items"), "includeMisc", ParameterWidget::Exists, true);
   if (_metrics->boolean("MultiWhs"))
     parameterWidget()->append(tr("Site"), "warehous_id", ParameterWidget::Site);
 
-  parameterWidget()->applyDefaultFilterSet();
-
   _groupBy->append(0,tr("Customer"));
   _groupBy->append(1,tr("Product Category"));
   _groupBy->append(2,tr("Item"));
