@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -23,6 +23,7 @@
 #include "metasql.h"
 #include "parameterwidget.h"
 #include "xtreewidget.h"
+#include "errorReporter.h"
 
 dspTrialBalances::dspTrialBalances(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspTrialBalances", fl)
@@ -49,19 +50,23 @@ dspTrialBalances::dspTrialBalances(QWidget* parent, const char*, Qt::WindowFlags
 
   // Determine current period
   int periodid = -1;
+  int yearperiodid = -1;
   XSqlQuery qry;
-  qry.exec("SELECT period_id "
+  qry.exec("SELECT period_id, period_yearperiod_id "
            "FROM period "
            "WHERE (current_date BETWEEN period_start AND period_end);");
   if (qry.first())
+  {
     periodid = qry.value("period_id").toInt();
+    yearperiodid = qry.value("period_yearperiod_id").toInt();
+  }
 
   parameterWidget()->appendComboBox(tr("Period"), "period_id", XComboBox::AccountingPeriods, QVariant(periodid));
+  parameterWidget()->appendComboBox(tr("Fiscal Year"), "yearperiod_id", XComboBox::FiscalYears, QVariant(yearperiodid));
   if (_metrics->value("GLCompanySize").toInt() > 0)
     parameterWidget()->appendComboBox(tr("Company"), "company_id", XComboBox::Companies);
   parameterWidget()->append(tr("GL Account"), "accnt_id",  ParameterWidget::GLAccount);
   parameterWidget()->append(tr("Show Zero Amounts"), "showZero", ParameterWidget::Exists);
-  parameterWidget()->applyDefaultFilterSet();
 }
 
 void dspTrialBalances::languageChange()
@@ -71,13 +76,12 @@ void dspTrialBalances::languageChange()
 
 void dspTrialBalances::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem *, int)
 {
-  QAction *menuItem;
-  menuItem = pMenu->addAction(tr("View Transactions..."), this, SLOT(sViewTransactions()));
+  (void)pMenu->addAction(tr("View Transactions..."), this, SLOT(sViewTransactions()));
 
   if (_metrics->boolean("ManualForwardUpdate"))
   {
     pMenu->addSeparator();
-    menuItem = pMenu->addAction(tr("Forward Update"), this, SLOT(sForwardUpdate()));
+    (void)pMenu->addAction(tr("Forward Update"), this, SLOT(sForwardUpdate()));
   }
 }
 
@@ -108,13 +112,15 @@ void dspTrialBalances::sForwardUpdate()
     int result = dspForwardUpdate.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("forwardUpdateTrialBalance", result), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Trial Balance Information"),
+                             storedProcErrorLookup("forwardUpdateTrialBalance", result),
+                             __FILE__, __LINE__);
       return;
     }
   }
-  else if (dspForwardUpdate.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Trial Balance Information"),
+                                dspForwardUpdate, __FILE__, __LINE__))
   {
-    systemError(this, dspForwardUpdate.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -163,13 +169,15 @@ bool dspTrialBalances::forwardUpdate()
     int result = dspforwardUpdate.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("forwardUpdateTrialBalance", result), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Trial Balance Information"),
+                             storedProcErrorLookup("forwardUpdateTrialBalance", result),
+                             __FILE__, __LINE__);
       return false;
     }
   }
-  else if (dspforwardUpdate.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Trial Balance Information"),
+                                dspforwardUpdate, __FILE__, __LINE__))
   {
-    systemError(this, dspforwardUpdate.lastError().databaseText(), __FILE__, __LINE__);
     return false;
   }
   return true;
