@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -18,6 +18,7 @@
 #include "dspInventoryAvailability.h"
 #include "itemSite.h"
 #include "storedProcErrorLookup.h"
+#include "errorReporter.h"
 
 dspUnbalancedQOHByClassCode::dspUnbalancedQOHByClassCode(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspUnbalancedQOHByClassCode", fl)
@@ -55,14 +56,15 @@ void dspUnbalancedQOHByClassCode::sBalance()
     int result = dspBalance.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("balanceItemsite", result),
-                  __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                             storedProcErrorLookup("balanceItemsite", result),
+                             __FILE__, __LINE__);
       return;
     }
   }
-  else if (dspBalance.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                dspBalance, __FILE__, __LINE__))
   {
-    systemError(this, dspBalance.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   sFillList();
@@ -140,6 +142,9 @@ void dspUnbalancedQOHByClassCode::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem*,
 
 bool dspUnbalancedQOHByClassCode::setParams(ParameterList &params)
 {
+  if (!display::setParams(params))
+    return false;
+
   if ((_classCode->isSelected()) || (_classCode->isPattern()))
     _classCode->appendValue(params);
 
