@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -17,6 +17,8 @@
 #include "enterPoitemReceipt.h"
 #include "poLiabilityDistrib.h"
 #include "postPoReturnCreditMemo.h"
+#include "errorReporter.h"
+#include "xtreewidget.h"
 
 dspUninvoicedReceivings::dspUninvoicedReceivings(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspUninvoicedReceivings", fl)
@@ -87,9 +89,9 @@ void dspUninvoicedReceivings::sMarkAsInvoiced()
     if (newdlg.exec() == XDialog::Rejected)
       update = false;
   }
-  else if (dspMarkAsInvoiced.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Receiving Information"),
+                                dspMarkAsInvoiced, __FILE__, __LINE__))
   {
-    systemError(this, dspMarkAsInvoiced.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   if (update)
@@ -99,9 +101,9 @@ void dspUninvoicedReceivings::sMarkAsInvoiced()
           "WHERE (recv_id=:recv_id); ");
     dspMarkAsInvoiced.bindValue(":recv_id",list()->id());
     dspMarkAsInvoiced.exec();
-    if (dspMarkAsInvoiced.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Receiving Information"),
+                                  dspMarkAsInvoiced, __FILE__, __LINE__))
     {
-      systemError(this, dspMarkAsInvoiced.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     sFillList();
@@ -128,6 +130,9 @@ void dspUninvoicedReceivings::sCreateCreditMemo()
 
 bool dspUninvoicedReceivings::setParams(ParameterList &pParams)
 {
+  if (!display::setParams(pParams))
+    return false;
+
   _warehouse->appendValue(pParams);
   if(_selectedPurchasingAgent->isChecked())
     pParams.append("agentUsername", _agent->currentText());
