@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -13,6 +13,7 @@
 #include <QMenu>
 #include <QMessageBox>
 #include <QVariant>
+#include "xtreewidget.h"
 
 dspValidLocationsByItem::dspValidLocationsByItem(QWidget* parent, const char*, Qt::WindowFlags fl)
     : display(parent, "dspValidLocationsByItem", fl)
@@ -28,7 +29,6 @@ dspValidLocationsByItem::dspValidLocationsByItem(QWidget* parent, const char*, Q
   list()->addColumn(tr("Description"), -1,           Qt::AlignLeft,   true,  "locationdescrip"   );
   list()->addColumn(tr("Restricted"),  _orderColumn, Qt::AlignCenter, true,  "location_restrict"  );
   list()->addColumn(tr("Netable"),     _orderColumn, Qt::AlignCenter, true,  "location_netable" );
-
 }
 
 void dspValidLocationsByItem::languageChange()
@@ -39,6 +39,9 @@ void dspValidLocationsByItem::languageChange()
 
 bool dspValidLocationsByItem::setParams(ParameterList & params)
 {
+  if (!display::setParams(params))
+    return false;
+
   if (! _item->isValid())
   { 
     QMessageBox::warning(this, tr("Invalid Item"),
