@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -77,10 +77,8 @@ void dspWoHistoryByClassCode::sEdit()
 
 void dspWoHistoryByClassCode::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem *, int)
 {
-  QAction *menuItem;
-
-  menuItem = pMenu->addAction(tr("Edit..."), this, SLOT(sEdit()));
-  menuItem = pMenu->addAction(tr("View..."), this, SLOT(sView()));
+  (void)pMenu->addAction(tr("Edit..."), this, SLOT(sEdit()));
+  (void)pMenu->addAction(tr("View..."), this, SLOT(sView()));
 }
 
 void dspWoHistoryByClassCode::sHandleCosts(bool pShowCosts)
@@ -93,6 +91,9 @@ void dspWoHistoryByClassCode::sHandleCosts(bool pShowCosts)
 
 bool dspWoHistoryByClassCode::setParams(ParameterList & params)
 {
+  if (!display::setParams(params))
+    return false;
+
   if(isVisible())
   {
     if (! _dates->startDate().isValid())
