@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -79,10 +79,8 @@ void dspWoHistoryByItem::sEdit()
 
 void dspWoHistoryByItem::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem *, int)
 {
-  QAction *menuItem;
-
-  menuItem = pMenu->addAction(tr("Edit..."), this, SLOT(sEdit()));
-  menuItem = pMenu->addAction(tr("View..."), this, SLOT(sView()));
+  (void)pMenu->addAction(tr("Edit..."), this, SLOT(sEdit()));
+  (void)pMenu->addAction(tr("View..."), this, SLOT(sView()));
 }
 
 void dspWoHistoryByItem::sHandleCosts(bool pShowCosts)
@@ -98,12 +96,15 @@ void dspWoHistoryByItem::sHandleCosts(bool pShowCosts)
 
 bool dspWoHistoryByItem::setParams(ParameterList & params)
 {
+  if (!display::setParams(params))
+    return false;
+
   if(isVisible())
   {
     if (! _item->isValid())
     {
-      QMessageBox::warning(this, tr("Invalid Item"),
-                           tr("Enter a valid Item."));
+      QMessageBox::warning(this, tr("Enter a valid Item Number"),
+                           tr("You must enter a valid item number for this report."));
       _item->setFocus();
       return false;
     }
