@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -78,7 +78,7 @@ void dspWoHistoryByNumber::sView()
   params.append("mode", "view");
   params.append("wo_id", list()->id());
 
-  workOrder *newdlg = new workOrder();
+  workOrder *newdlg = new workOrder(this);
   newdlg->set(params);
   omfgThis->handleNewWindow(newdlg);
 }
@@ -89,17 +89,15 @@ void dspWoHistoryByNumber::sEdit()
   params.append("mode", "edit");
   params.append("wo_id", list()->id());
 
-  workOrder *newdlg = new workOrder();
+  workOrder *newdlg = new workOrder(this);
   newdlg->set(params);
   omfgThis->handleNewWindow(newdlg);
 }
 
 void dspWoHistoryByNumber::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem *, int)
 {
-  QAction *menuItem;
-
-  menuItem = pMenu->addAction(tr("Edit..."), this, SLOT(sEdit()));
-  menuItem = pMenu->addAction(tr("View..."), this, SLOT(sView()));
+  (void)pMenu->addAction(tr("Edit..."), this, SLOT(sEdit()));
+  (void)pMenu->addAction(tr("View..."), this, SLOT(sView()));
 }
 
 void dspWoHistoryByNumber::sHandleCosts(bool pShowCosts)
@@ -112,10 +110,13 @@ void dspWoHistoryByNumber::sHandleCosts(bool pShowCosts)
 
 bool dspWoHistoryByNumber::setParams(ParameterList &params)
 {
+  if (!display::setParams(params))
+    return false;
+
   if(_woNumber->text().isEmpty())
   {
-    QMessageBox::warning( this, tr("Invalid Work Order Number"),                                                   
-                      tr( "You must enter a work order number for this report." ) );                               
+    QMessageBox::warning( this, tr("Enter Work Order Pattern"),
+                      tr( "You must enter a work order pattern for this report." ) );
     _woNumber->setFocus();
     return false;
   }
