@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -17,6 +17,7 @@
 
 #include "inputManager.h"
 #include "woMaterialItem.h"
+#include "xtreewidget.h"
 
 dspWoMaterialsByWorkOrder::dspWoMaterialsByWorkOrder(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspWoMaterialsByWorkOrder", fl)
@@ -41,6 +42,8 @@ dspWoMaterialsByWorkOrder::dspWoMaterialsByWorkOrder(QWidget* parent, const char
   }
 
   list()->addColumn(tr("Component Item"),  _itemColumn,  Qt::AlignLeft,   true,  "item_number"   );
+  list()->addColumn(tr("Component Item Description 1"),  _itemColumn,  Qt::AlignLeft,   true,  "item_descrip1"   );
+  list()->addColumn(tr("Component Item Description 2"),  _itemColumn,  Qt::AlignLeft,   true,  "item_descrip2"   );
   if (_manufacturing)
     list()->addColumn(tr("Oper. #"),         _dateColumn,  Qt::AlignCenter, true,  "wooperseq" );
   list()->addColumn(tr("Iss. Meth.") ,     _orderColumn, Qt::AlignCenter, true,  "issuemethod" );
@@ -82,6 +85,9 @@ enum SetResponse dspWoMaterialsByWorkOrder::set(const ParameterList &pParams)
 
 bool dspWoMaterialsByWorkOrder::setParams(ParameterList & params)
 {
+  if (!display::setParams(params))
+    return false;
+
   if(!_wo->isValid())
   {
     QMessageBox::warning(this, tr("Work Order Required"),
@@ -102,9 +108,7 @@ bool dspWoMaterialsByWorkOrder::setParams(ParameterList & params)
 
 void dspWoMaterialsByWorkOrder::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem *, int)
 {
-  QAction *menuItem;
-
-  menuItem = pMenu->addAction(tr("View Requirement..."), this, SLOT(sView()));
+  (void)pMenu->addAction(tr("View Requirement..."), this, SLOT(sView()));
 }
 
 void dspWoMaterialsByWorkOrder::sView()
