@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2010 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2016 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -24,6 +24,7 @@
 #include "explodeWo.h"
 #include "implodeWo.h"
 #include "issueWoMaterialItem.h"
+#include "returnWoMaterialItem.h"
 #include "postProduction.h"
 #include "printWoTraveler.h"
 #include "reprioritizeWo.h"
@@ -32,6 +33,7 @@
 #include "storedProcErrorLookup.h"
 #include "workOrder.h"
 #include "parameterwidget.h"
+#include "errorReporter.h"
 
 dspWoSchedule::dspWoSchedule(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspWoSchedule", fl)
@@ -74,8 +76,6 @@ dspWoSchedule::dspWoSchedule(QWidget* parent, const char*, Qt::WindowFlags fl)
   parameterWidget()->append(tr("Has Closed Parent Sales Order"), "woSoStatusMismatch", ParameterWidget::Exists);
   parameterWidget()->append(tr("Project"), "prj_id", ParameterWidget::Project);
 
-  parameterWidget()->applyDefaultFilterSet();
-
   list()->addColumn(tr("Work Order #"),   _orderColumn,  Qt::AlignLeft,   true,  "wonumber"   );
   list()->addColumn(tr("Parent Type"),    _orderColumn,  Qt::AlignCenter, true,  "wo_ordtype" );
   list()->addColumn(tr("Parent Order #"), _orderColumn,  Qt::AlignLeft,   true,  "parentorder" );
@@ -92,6 +92,11 @@ dspWoSchedule::dspWoSchedule(QWidget* parent, const char*, Qt::WindowFlags fl)
   list()->addColumn(tr("Due Date"),       _dateColumn,   Qt::AlignRight,  true,  "wo_duedate"  );
   list()->addColumn(tr("Condition"),      _dateColumn,   Qt::AlignLeft,   true,  "condition"   );
 
+  list()->addColumn(tr("Wip Value"),      _bigMoneyColumn,   Qt::AlignLeft,   true,  "wo_wipvalue"   );
+  list()->addColumn(tr("Cost Est."),      _bigMoneyColumn,   Qt::AlignLeft,   true,  "plancost"   );
+  list()->addColumn(tr("Variance Est."),  _bigMoneyColumn,   Qt::AlignLeft,   true,  "wopvar"   );
+  list()->addColumn(tr("Cost Method"),    _orderColumn,   Qt::AlignLeft,   true,  "costmethod"   );
+
   if (_privileges->check("MaintainWorkOrders"))
     connect(list(), SIGNAL(itemSelected(int)), this, SLOT(sEdit()));
   else
@@ -143,10 +148,9 @@ enum SetResponse dspWoSchedule::set(const ParameterList &pParams)
     parameterWidget()->setDefault(tr("Status"), list);
   }
 
-  parameterWidget()->applyDefaultFilterSet();
-
   if (pParams.inList("run"))
   {
+    parameterWidget()->applyDefaultFilterSet();
     sFillList();
     return NoError_Run;
   }   
@@ -168,6 +172,7 @@ bool dspWoSchedule::setParams(ParameterList & params)
   params.append("planord", tr("Planned Order"));
   params.append("mps", tr("MPS"));
   params.append("so", tr("S/O"));
+  params.append("quote", tr("Quote"));
 
   params.append("overdue", tr("Overdue"));
   params.append("ontime", tr("On Time"));
@@ -302,23 +307,17 @@ void dspWoSchedule::sDeleteWO()
     dspDeleteWO.bindValue(":wo_id", list()->id());
     dspDeleteWO.exec();
 
-    if (dspDeleteWO.first())
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Work Order Information"),
+                         dspDeleteWO, __FILE__, __LINE__))
     {
-      int result = dspDeleteWO.value("returnVal").toInt();
-      if (result < 0)
-      {
-	systemError(this, storedProcErrorLookup("deleteWo", result));
-	return;
-      }
+      return;
     }
-    else if (dspDeleteWO.lastError().type() != QSqlError::NoError)
-      systemError(this, dspDeleteWO.lastError().databaseText(), __FILE__, __LINE__);
-
-    omfgThis->sWorkOrdersUpdated(-1, true);
+    else
+      omfgThis->sWorkOrdersUpdated(-1, true);
   }
-  else if (dspDeleteWO.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Work Order Information"),
+                                dspDeleteWO, __FILE__, __LINE__))
   {
-    systemError(this, dspDeleteWO.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -491,6 +490,9 @@ void dspWoSchedule::sPopulateMenu(QMenu *pMenu,  QTreeWidgetItem *pSelected, int
       menuItem->setEnabled(_privileges->check("IssueWoMaterials"));
     }
 
+    menuItem = pMenu->addAction(tr("Return Material Item..."), this, SLOT(sReturnWoMaterialItem()));
+    menuItem->setEnabled(_privileges->check("ReturnWoMaterials"));
+    
     menuItem = pMenu->addAction(tr("Post Production..."), this, SLOT(sPostProduction()));
     menuItem->setEnabled(_privileges->check("PostProduction"));
 
@@ -542,6 +544,15 @@ void dspWoSchedule::sIssueWoMaterialItem()
     newdlg.exec();
 }
 
+void dspWoSchedule::sReturnWoMaterialItem()
+{
+  returnWoMaterialItem newdlg(this);
+  ParameterList params;
+  params.append("wo_id", list()->id());
+  if (newdlg.set(params) == NoError)
+    newdlg.exec();
+}
+
 void dspWoSchedule::sDspRunningAvailability()
 {
   XSqlQuery dspDspRunningAvailability;
@@ -558,9 +569,9 @@ void dspWoSchedule::sDspRunningAvailability()
     newdlg->set(params);
     omfgThis->handleNewWindow(newdlg);
   }
-  else if (dspDspRunningAvailability.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                dspDspRunningAvailability, __FILE__, __LINE__))
   {
-    systemError(this, dspDspRunningAvailability.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -584,9 +595,9 @@ void dspWoSchedule::sViewBOM()
     newdlg->set(params);
     omfgThis->handleNewWindow(newdlg);
   }
-  else if (dspViewBOM.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                dspViewBOM, __FILE__, __LINE__))
   {
-    systemError(this, dspViewBOM.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
