@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -26,6 +26,7 @@
 
 #define cIncludeLotSerial   0x01
 #define cNoIncludeLotSerial 0x02
+#define DEBUG false
 
 distributeInventory::distributeInventory(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
@@ -84,7 +85,6 @@ distributeInventory::distributeInventory(QWidget* parent, const char* name, bool
 
   _itemlocdistid = -1;
   _transtype = "O";
-  
   _locationDefaultLit->hide();
   _locations->hide();
 }
@@ -98,31 +98,88 @@ void distributeInventory::languageChange()
 {
   retranslateUi(this);
 }
-int distributeInventory::SeriesAdjust(int pItemlocSeries, QWidget *pParent, const QString & pPresetLotnum, const QDate & pPresetLotexp, const QDate & pPresetLotwarr)
+
+int distributeInventory::SeriesCreate(int itemsiteId, double qty, const QString & orderType, 
+  const QString & transType, int orderitemId, int itemlocSeries, int itemlocdistId, int invhistId )
+{
+  bool controlled = false;
+
+  // Generate a series id to be used for itemlocdist if controlled, and for returnWoMaterial()
+  XSqlQuery parentSeries;
+  parentSeries.prepare("SELECT COALESCE(:itemlocSeries, NEXTVAL('itemloc_series_seq')) AS result, isControlledItemsite(:itemsiteId) AS controlled;");
+  parentSeries.bindValue(":itemsiteId", itemsiteId);
+  if (itemlocSeries > 0)
+    parentSeries.bindValue(":itemlocSeries", itemlocSeries);
+  parentSeries.exec();
+  if (parentSeries.first() && parentSeries.value("result").toInt() > 0)
+  {
+    itemlocSeries = parentSeries.value("result").toInt();
+    controlled = parentSeries.value("controlled").toBool();
+  }
+  else
+  {
+    ErrorReporter::error(QtCriticalMsg, 0, tr("Failed to Retrieve the Next itemloc_series_seq"),
+      parentSeries.lastError().databaseText(), __FILE__, __LINE__);
+    return -1;
+  }
+
+  if (controlled)
+  {
+    // Create the itemlocdist record if controlled
+    XSqlQuery parentItemlocdist;
+    parentItemlocdist.prepare("SELECT createItemlocdistParent(:itemsiteId, :qty, :orderType, :orderitemId, "
+                              " :itemlocSeries, :invhistId, :itemlocdistId, :transType) AS result;");
+    parentItemlocdist.bindValue(":itemsiteId", itemsiteId);
+    parentItemlocdist.bindValue(":qty", qty);
+    parentItemlocdist.bindValue(":orderType", orderType);
+    parentItemlocdist.bindValue(":transType", transType);
+    parentItemlocdist.bindValue(":itemlocSeries", itemlocSeries);
+    if (orderitemId > 0)
+      parentItemlocdist.bindValue(":orderitemId", orderitemId);
+    if (itemlocdistId > 0)
+      parentItemlocdist.bindValue(":itemlocdistId", itemlocdistId);
+    if (invhistId > 0)
+      parentItemlocdist.bindValue(":invhistId", invhistId);
+    parentItemlocdist.exec();
+    if (ErrorReporter::error(QtCriticalMsg, 0, tr("Error Creating itemlocdist record for post "
+        "production controlled item"), parentItemlocdist, __FILE__, __LINE__))
+      return -1;
+  }
+
+  return itemlocSeries;
+}
+
+int distributeInventory::SeriesAdjust(int pItemlocSeries, QWidget *pParent, 
+  const QString & pPresetLotnum, const QDate & pPresetLotexp, const QDate & pPresetLotwarr,
+  bool pPreDistributed)
 {
   int result;
-  QList<int>  ildsList; // Item Loc Dist Series 
-  QList<int>  ildList; // Item Loc Dist List
-
+  
+  if (DEBUG)
+    qDebug() << tr("DistributeInventory::SeriesAdjust pItemlocSeries: %1, pPreDistributed: %2")
+    .arg(pItemlocSeries).arg(pPreDistributed);
+  
   if (pItemlocSeries != 0)
   {
     XSqlQuery itemloc;
     itemloc.prepare( "SELECT itemlocdist_id, itemlocdist_reqlotserial," 
                      "       itemlocdist_distlotserial, itemlocdist_qty,"
-                     "       itemsite_loccntrl, itemsite_controlmethod,"
+                     "       itemsite_id, itemsite_loccntrl, itemsite_controlmethod,"
                      "       itemsite_perishable, itemsite_warrpurc,"
                      "       COALESCE(itemsite_lsseq_id,-1) AS itemsite_lsseq_id,"
                      "       COALESCE(itemlocdist_source_id,-1) AS itemlocdist_source_id,"
-                     "       CASE WHEN (invhist_transtype IN ('RM','RP','RR','RX')) THEN 'R'"
-                     "            WHEN (invhist_transtype = 'IM') THEN 'I'"
-                     "            WHEN (invhist_transtype = 'SH' AND invhist_ordtype='SO') THEN 'I'"
+                     // TODO - remove invhist altogether after #22868 is complete
+                     "       CASE WHEN (COALESCE(invhist_transtype, itemlocdist_transtype) IN ('RM','RP','RR','RX')) THEN 'R'"
+                     "            WHEN (COALESCE(invhist_transtype, itemlocdist_transtype) = 'IM') THEN 'I'"
+                     "            WHEN (COALESCE(invhist_transtype, itemlocdist_transtype) = 'SH' "
+                     "              AND COALESCE(invhist_ordtype, itemlocdist_order_type) ='SO') THEN 'I'"
                      "            ELSE 'O'"
                      "       END AS trans_type,"
-                     "       CASE WHEN (invhist_transtype IN ('RM','RP','RR','RX')"
+                     "       CASE WHEN (COALESCE(invhist_transtype, itemlocdist_transtype) IN ('RM','RP','RR','RX')"
                      "                  AND itemsite_recvlocation_dist) THEN true"
-                     "            WHEN (invhist_transtype = 'IM'"
+                     "            WHEN (COALESCE(invhist_transtype, itemlocdist_transtype) = 'IM'"
                      "                  AND itemsite_issuelocation_dist) THEN true"
-                     "            WHEN (invhist_transtype NOT IN ('RM','RP','RR','RX','IM')"
+                     "            WHEN (COALESCE(invhist_transtype, itemlocdist_transtype) NOT IN ('RM','RP','RR','RX','IM')"
                      "                  AND itemsite_location_dist) THEN true"
                      "            ELSE false"
                      "       END AS auto_dist "
@@ -134,10 +191,12 @@ int distributeInventory::SeriesAdjust(int pItemlocSeries, QWidget *pParent, cons
     itemloc.exec();
     while (itemloc.next())
     {
+      // Requires lot/serial
       if (itemloc.value("itemlocdist_reqlotserial").toBool())
       {
         int itemlocSeries = -1;
         XSqlQuery query;
+
         // Check to see if this is a lot controlled item and if we have
         // a predefined lot#/expdate to use. If so assign that information
         // with itemlocdist_qty and move on. otherwise do the normal dialog
@@ -150,72 +209,75 @@ int distributeInventory::SeriesAdjust(int pItemlocSeries, QWidget *pParent, cons
             itemlocSeries = query.value("_itemloc_series").toInt();
             query.prepare( "SELECT createlotserial(itemlocdist_itemsite_id,:lotserial,:itemlocdist_series,'I',NULL,itemlocdist_id,:qty,:expiration,:warranty) "
                            "FROM itemlocdist "
-                           "WHERE (itemlocdist_id=:itemlocdist_id);"
-                           
-                           "UPDATE itemlocdist "
-                           "SET itemlocdist_source_type='O' "
-                           "WHERE (itemlocdist_series=:itemlocdist_series);"
-
-                           "DELETE FROM itemlocdist "
-                           "WHERE (itemlocdist_id=:itemlocdist_id);" );
+                           "WHERE (itemlocdist_id=:itemlocdist_id);");
             query.bindValue(":lotserial", pPresetLotnum);
-            query.bindValue(":qty", itemloc.value("itemlocdist_qty"));
             query.bindValue(":itemlocdist_series", itemlocSeries);
             query.bindValue(":itemlocdist_id", itemloc.value("itemlocdist_id"));
-            if(itemloc.value("itemsite_perishable").toBool())
+            query.bindValue(":qty", itemloc.value("itemlocdist_qty"));
+            if (itemloc.value("itemsite_perishable").toBool())
               query.bindValue(":expiration", pPresetLotexp);
             else
               query.bindValue(":expiration", omfgThis->endOfTime());
-            if(itemloc.value("itemsite_warrpurc").toBool())
+            if (itemloc.value("itemsite_warrpurc").toBool())
               query.bindValue(":warranty", pPresetLotwarr);
+            
+            query.exec();
+            if (!query.first() || ErrorReporter::error(QtCriticalMsg, 0, tr("Error Retrieving Lot/Serial Information"),
+                                          query, __FILE__, __LINE__))
+            {
+              return XDialog::Rejected;
+            }
+
+            query.prepare( "UPDATE itemlocdist "
+                           "SET itemlocdist_source_type='O' "
+                           "WHERE (itemlocdist_series=:itemlocdist_series);");
+            query.bindValue(":itemlocdist_series", itemlocSeries);
             query.exec();
           }
         }
 
-        if(itemlocSeries == -1)
+        // Item did not have a preset lot number.
+        // Check to see if Lot/Serial distributions should be created using
+        // "from" side of transaction.  Transactions are related by itemlocdist_source_id.
+        // InterWarehouseTransfer uses this technique.
+        if(itemlocSeries == -1 && itemloc.value("itemlocdist_source_id").toInt() > -1)
         {
-          // Check to see if Lot/Serial distributions should be created using
-          // "from" side of transaction.  Transactions are related by itemlocdist_source_id.
-          // InterWarehouseTransfer uses this technique.
-          if (itemloc.value("itemlocdist_source_id").toInt() > -1)
+          XSqlQuery fromlots;
+          fromlots.exec("SELECT nextval('itemloc_series_seq') AS _itemloc_series;");
+          if(fromlots.first())
           {
-            XSqlQuery fromlots;
-            fromlots.exec("SELECT nextval('itemloc_series_seq') AS _itemloc_series;");
-            if(fromlots.first())
+            itemlocSeries = fromlots.value("_itemloc_series").toInt();
+            fromlots.prepare("SELECT  createlotserial(s.itemlocdist_itemsite_id, ls_number, "
+                             "        :itemlocdist_series, 'I', NULL, :itemlocdist_id, (d.itemlocdist_qty * -1.0), "
+                             "        itemloc_expiration, itemloc_warrpurc) "
+                             "FROM itemlocdist s JOIN itemlocdist o ON (o.itemlocdist_id=s.itemlocdist_source_id) "
+                             "                   JOIN itemlocdist d ON (d.itemlocdist_itemlocdist_id=o.itemlocdist_id) "
+                             "                   JOIN itemloc ON (itemloc_id=d.itemlocdist_source_id) "
+                             "                   JOIN ls ON (ls_id=itemloc_ls_id) "
+                             "WHERE (s.itemlocdist_id=:itemlocdist_id);");
+            fromlots.bindValue(":itemlocdist_id", itemloc.value("itemlocdist_id"));
+            fromlots.bindValue(":itemlocdist_series", itemlocSeries);
+            fromlots.exec();
+            if (ErrorReporter::error(QtCriticalMsg, 0, tr("Error Retrieving Lot/Serial Information"),
+                                          fromlots, __FILE__, __LINE__))
             {
-              itemlocSeries = fromlots.value("_itemloc_series").toInt();
-              fromlots.prepare("SELECT  createlotserial(s.itemlocdist_itemsite_id, ls_number, "
-                               "        :itemlocdist_series, 'I', NULL, :itemlocdist_id, (d.itemlocdist_qty * -1.0), "
-                               "        itemloc_expiration, itemloc_warrpurc) "
-                               "FROM itemlocdist s JOIN itemlocdist o ON (o.itemlocdist_id=s.itemlocdist_source_id) "
-                               "                   JOIN itemlocdist d ON (d.itemlocdist_itemlocdist_id=o.itemlocdist_id) "
-                               "                   JOIN itemloc ON (itemloc_id=d.itemlocdist_source_id) "
-                               "                   JOIN ls ON (ls_id=itemloc_ls_id) "
-                               "WHERE (s.itemlocdist_id=:itemlocdist_id);"
-
-                               "UPDATE itemlocdist "
-                               "SET itemlocdist_source_type='O' "
-                               "WHERE (itemlocdist_series=:itemlocdist_series);"
-
-                               "DELETE FROM itemlocdist "
-                               "WHERE (itemlocdist_id=:itemlocdist_id);" );
-              fromlots.bindValue(":itemlocdist_series", itemlocSeries);
-              fromlots.bindValue(":itemlocdist_id", itemloc.value("itemlocdist_id"));
-              fromlots.exec();
-              if (fromlots.lastError().type() != QSqlError::NoError)
-              {
-                systemError(0, fromlots.lastError().databaseText(), __FILE__, __LINE__);
-                return XDialog::Rejected;
-              }
+              return XDialog::Rejected;
             }
+
+            fromlots.prepare("UPDATE itemlocdist "
+                             "SET itemlocdist_source_type='O' "
+                             "WHERE (itemlocdist_series=:itemlocdist_series);");
+            fromlots.bindValue(":itemlocdist_series", itemlocSeries);
+            fromlots.exec();
           }
         }
 
-        if(itemlocSeries == -1)
-        { 
+        // Open Assign Lot Serial dialog, populating with auto ls info if required. 
+        if (itemlocSeries == -1)
+        {
           ParameterList params;
           params.append("itemlocdist_id", itemloc.value("itemlocdist_id").toInt());
-
+          
           // Auto assign lot/serial if applicable
           if (itemloc.value("itemsite_lsseq_id").toInt() != -1 &&
               !itemloc.value("itemsite_perishable").toBool() &&
@@ -226,13 +288,12 @@ int distributeInventory::SeriesAdjust(int pItemlocSeries, QWidget *pParent, cons
             autocreatels.exec();
             if (autocreatels.first())
               params.append("itemlocseries", autocreatels.value("itemlocseries").toInt());
-            else if (autocreatels.lastError().type() != QSqlError::NoError)
+            else if (ErrorReporter::error(QtCriticalMsg, 0, tr("Error Retrieving Lot/Serial Information"),
+                                          autocreatels, __FILE__, __LINE__))
             {
-              systemError(0, autocreatels.lastError().databaseText(), __FILE__, __LINE__);
               return XDialog::Rejected;
             }
           }
-
           assignLotSerial newdlg(pParent, "", true);
           newdlg.set(params);
           itemlocSeries = newdlg.exec();
@@ -240,6 +301,7 @@ int distributeInventory::SeriesAdjust(int pItemlocSeries, QWidget *pParent, cons
             return XDialog::Rejected;
         }
         
+        // Distribute location with Distribute Inventory dialog
         if (itemloc.value("itemsite_loccntrl").toBool())
         {
           query.prepare( "SELECT itemlocdist_id " 
@@ -258,14 +320,17 @@ int distributeInventory::SeriesAdjust(int pItemlocSeries, QWidget *pParent, cons
             if (itemloc.value("auto_dist").toBool())
             {
               if (newdlg.sDefaultAndPost())
-                ildList.append(query.value("itemlocdist_id").toInt());
+              {
+                if (DEBUG)
+                  qDebug() << tr("Pre-22868 itemlocdist_id array for distributeToLocations(). "
+                                 "Auto Dist + Default Post: ildList.append(%1) - (itemlocdist_id from "
+                                 "itemloc query above)").arg(itemloc.value("itemlocdist_id").toInt());
+              }
               else
               {
                 result = newdlg.exec();
                 if (result == XDialog::Rejected)
                   return XDialog::Rejected;
-                else
-                  ildList.append(result);
               }
             }
             else
@@ -273,24 +338,37 @@ int distributeInventory::SeriesAdjust(int pItemlocSeries, QWidget *pParent, cons
               result = newdlg.exec();
               if (result == XDialog::Rejected)
                 return XDialog::Rejected;
-              else
-                ildList.append(result);
             }
+
+            if (DEBUG && result > 0)
+              qDebug() << tr("Pre-22868 itemlocdist_id array for distributeToLocations(). "
+                             "Auto Dist: ildList.append(%1) - (itemlocdist_id from "
+                             "distributeInventory newdlg)").arg(result);
           }
         }
         else
         {
           query.prepare( "UPDATE itemlocdist "
-                         "SET itemlocdist_source_type='L', itemlocdist_source_id=-1 "
+                         "SET itemlocdist_source_type='L', itemlocdist_source_id = -1 "
                          "WHERE (itemlocdist_series=:itemlocdist_series); ");
           query.bindValue(":itemlocdist_series", itemlocSeries);
           query.exec();
 
           // Append id to list and process at the end
-          ildsList.append(itemlocSeries);
+          if (DEBUG)
+            qDebug() << tr("Pre-22868 itemloc_series array for distributeItemlocSeries(). "
+                           "Not Location Controlled: ildsList.append(%1) - (itemlocSeries)").
+                           arg(itemlocSeries);
         }
+
+        // Set itemlocdist_child_series of parent itemlocdist record
+        query.prepare("UPDATE itemlocdist SET itemlocdist_child_series = :itemlocSeries "
+                      "WHERE itemlocdist_id = :itemlocdist_id;");
+        query.bindValue(":itemlocSeries", itemlocSeries);
+        query.bindValue(":itemlocdist_id", itemloc.value("itemlocdist_id").toInt());
+        query.exec();
       }
-      else
+      else // NOT a lot/serial controlled item (or a neg. qty, requiring "distribution from")
       {
         ParameterList params;
         params.append("itemlocdist_id", itemloc.value("itemlocdist_id").toInt());
@@ -304,14 +382,17 @@ int distributeInventory::SeriesAdjust(int pItemlocSeries, QWidget *pParent, cons
         if (itemloc.value("auto_dist").toBool())
         {
           if (newdlg.sDefaultAndPost())
-            ildList.append(itemloc.value("itemlocdist_id").toInt());
+          {
+            if (DEBUG)
+              qDebug() << tr("Pre-22868 itemlocdist_id array for distributeToLocations(). "
+                             "Auto Dist + Default Post: ildList.append(%1) - (itemlocdist_id from "
+                             "itemloc query above)").arg(itemloc.value("itemlocdist_id").toInt());
+          }
           else
           {
             result = newdlg.exec();
             if (result == XDialog::Rejected)
               return XDialog::Rejected;
-            else
-              ildList.append(result);
           }
         }
         else
@@ -319,55 +400,59 @@ int distributeInventory::SeriesAdjust(int pItemlocSeries, QWidget *pParent, cons
           result = newdlg.exec();
           if (result == XDialog::Rejected)
             return XDialog::Rejected;
-          else
-            ildList.append(result);
         }
+
+        if (DEBUG && result > 0)
+          qDebug() << tr("Pre-22868 itemlocdist_id array for distributeToLocations(). "
+                         "Auto Dist: ildList.append(%1) - (itemlocdist_id from "
+                         "distributeInventory newdlg)").arg(result);
+
+        // Set itemlocdist_child_series of parent itemlocdist record. In this case there is no child, set it to itself.
+        XSqlQuery query;
+        query.prepare("UPDATE itemlocdist SET itemlocdist_child_series = itemlocdist_series "
+                      "WHERE itemlocdist_id = :itemlocdist_id AND itemlocdist_qty < 0;"); //itemlocdist_id = :itemlocdist_id
+        query.bindValue(":itemlocdist_id", itemloc.value("itemlocdist_id").toInt());
+        query.exec();
+        if (query.first())
+        {
+          if (query.numRowsAffected() != 1)
+          {
+            ErrorReporter::error(QtCriticalMsg, 0, tr("Updating Itemlocdist Parent Record Should Return One Row"),
+                           query, __FILE__, __LINE__);
+            return XDialog::Rejected;
+          }
+        }  
+        else if (ErrorReporter::error(QtCriticalMsg, 0, tr("Error Updating itemlocdist Lot/Serial Information"),
+                                      query, __FILE__, __LINE__))
+          return XDialog::Rejected;
       }
     }
 
-    XSqlQuery post;
-    
-    // Process Lot/Serial distributions
-    for (int i = 0; i < ildsList.size(); ++i) {
-      post.prepare( "SELECT distributeItemlocSeries(:itemlocdist_series) AS result;");
-      post.bindValue(":itemlocdist_series", ildsList.at(i));
-      post.exec();
-      if (post.lastError().type() != QSqlError::NoError)
+    // Post distribution detail here (pre-incident #28868)  
+    if (!pPreDistributed)
+    {
+      XSqlQuery postDistDetail;
+      postDistDetail.prepare("SELECT postdistdetail(:itemlocSeries) AS result;");
+      postDistDetail.bindValue(":itemlocSeries", pItemlocSeries);
+      postDistDetail.exec();
+      if (postDistDetail.first())
       {
-        systemError(0, post.lastError().databaseText(), __FILE__, __LINE__);
+        // If result = 0 (no dist. records were posted) and there are itemlocdist records throw error
+        if (postDistDetail.value("result").toInt() <= 0 && itemloc.size() > 0)
+        {
+          ErrorReporter::error(QtCriticalMsg, 0, tr("Posting distribution detail returned 0 results"),
+            postDistDetail, __FILE__, __LINE__);
+          return XDialog::Rejected;
+        }
+      }
+      else 
+      {
+        ErrorReporter::error(QtCriticalMsg, 0, tr("Distribution Detail Posting Failed"),
+          postDistDetail, __FILE__, __LINE__);
         return XDialog::Rejected;
       }
     }
-    
-    // Process location distributions
-    for (int i = 0; i < ildList.size(); ++i) {
-      post.prepare("SELECT distributeToLocations(:itemlocdist_id) AS result;");
-      post.bindValue(":itemlocdist_id", ildList.at(i));
-      post.exec();
-      if (post.lastError().type() != QSqlError::NoError)
-      {
-        systemError(0, post.lastError().databaseText(), __FILE__, __LINE__);
-        return XDialog::Rejected;
-      }
-    }
-    
-    //Post inventory history for any remaining non-distributed transactions and trial balance
-    post.prepare("SELECT postItemlocseries(:itemlocseries) AS result;");
-    post.bindValue(":itemlocseries",  pItemlocSeries);
-    post.exec();
-    if (post.first())
-    {
-      if (!post.value("result").toBool())
-            QMessageBox::warning( 0, tr("Inventory Distribution"), 
-        tr("There was an error posting the transaction.  Contact your administrator") );
-    }
-    else if (post.lastError().type() != QSqlError::NoError)
-    {
-      systemError(0, post.lastError().databaseText(), __FILE__, __LINE__);
-      return XDialog::Rejected;
-    }
   }
-  
   return XDialog::Accepted;
 }
 
@@ -428,9 +513,9 @@ void distributeInventory::populate()
     else
       _bcQty->clear();
   }
-  else if (distributepopulate.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, 0, tr("Error Retrieving Lot/Serial Information"),
+                                distributepopulate, __FILE__, __LINE__))
   {
-    systemError(this, distributepopulate.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -458,6 +543,8 @@ void distributeInventory::sSelectLocation()
   else if (_itemloc->altId() == cItemloc)
     params.append("itemlocdist_id", _itemloc->id());
 
+  params.append("itemsite_controlmethod", _controlMethod);
+
   distributeToLocation newdlg(this, "", true);
   newdlg.set(params);
 
@@ -545,12 +632,14 @@ bool distributeInventory::sDefault()
         int result = distributeDefault.value("result").toInt();
         if (result < 0)
         {
-          systemError(this, storedProcErrorLookup("distributetodefault", result),
-                      __FILE__, __LINE__);
+          ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Lot/Serial Information"),
+                                 storedProcErrorLookup("distributetodefault", result),
+                                 __FILE__, __LINE__);
+          
           return false;
         }
       }
-      else if (ErrorReporter::error(QtWarningMsg, this, tr("Distribute Default Location"),
+      else if (ErrorReporter::error(QtWarningMsg, this, tr("Error Retrieving Lot/Serial Information"),
                               distributeDefault, __FILE__, __LINE__))
       {
         return false;
@@ -583,7 +672,7 @@ void distributeInventory::sFillList()
   distributeFillList.prepare( "SELECT itemsite_id, "
              "       COALESCE(itemsite_location_id,-1) AS itemsite_location_id,"
              "       formatlotserialnumber(itemlocdist_ls_id) AS lotserial,"
-             "       (itemlocdist_order_type || ' ' || formatSoItemNumber(itemlocdist_order_id)) AS order,"
+             "       itemlocdist_order_type || ' ' || formatOrderLineItemNumber(itemlocdist_order_type, itemlocdist_order_id) AS order, "
              "       (itemsite_controlmethod IN ('L', 'S')) AS lscontrol,"
              "       parent.itemlocdist_qty AS qtytodistribute,"
              "       ( ( SELECT COALESCE(SUM(child.itemlocdist_qty), 0)"
@@ -648,15 +737,15 @@ void distributeInventory::sFillList()
     distributeFillList = mql.toQuery(params);
 
     _itemloc->populate(distributeFillList, true);
-    if (distributeFillList.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Lot/Serial Information"),
+                                  distributeFillList, __FILE__, __LINE__))
     {
-      systemError(this, distributeFillList.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
-  else if (distributeFillList.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Lot/Serial Information"),
+                                distributeFillList, __FILE__, __LINE__))
   {
-    systemError(this, distributeFillList.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -691,9 +780,9 @@ void distributeInventory::sBcDistribute()
 
   if(!distributeBcDistribute.first())
   {
-    if (distributeBcDistribute.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Lot/Serial Information"),
+                                  distributeBcDistribute, __FILE__, __LINE__))
     {
-      systemError(this, distributeBcDistribute.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     QMessageBox::warning(this, tr("No Match Found"),
@@ -708,6 +797,8 @@ void distributeInventory::sBcDistribute()
   params.append("qty",                   _bcQty->text());
   params.append("distribute");
 
+  params.append("itemsite_controlmethod", _controlMethod);
+
   distributeToLocation newdlg(this, "", true);
   if (newdlg.set(params) != NoError)
     return;
@@ -758,11 +849,13 @@ void distributeInventory::sPopulateDefaultSelector()
             loclist.prepare( " SELECT location_id, formatLocationName(location_id) AS locationname "
                            " FROM location "
                            " WHERE ( (location_warehous_id=:warehous_id)"
-                           "   AND (NOT location_restrict) ) "
+                           "   AND (NOT location_restrict) "
+                           "   AND (location_active) ) "
                            " UNION SELECT location_id, formatLocationName(location_id) AS locationname "
                            "  FROM location, locitem, itemsite"
                            "  WHERE ( (location_warehous_id=:warehous_id)"
                            "    AND (location_restrict)"
+                           "    AND (location_active)"
                            "    AND (locitem_location_id=location_id)"
                            "    AND (locitem_item_id=itemsite_item_id)"
                            "    AND (itemsite_id=:itemsite_id))"
@@ -821,11 +914,11 @@ void distributeInventory::sChangeDefaultLocation()
    query.bindValue(":itemsite_id", _itemsite_id);
    query.exec();
    sFillList();
-   if (query.lastError().type() != QSqlError::NoError)
-      {
-        systemError(this, query.lastError().databaseText(), __FILE__, __LINE__);
-        return;
-      }
+   if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Updating Item Site Information"),
+                                 query, __FILE__, __LINE__))
+   {
+     return;
+   }
 }
 
 void distributeInventory::updateZoneList()
