@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -19,11 +19,11 @@
 #include <QVariant>
 
 #include <metasql.h>
-#include "mqlutil.h"
+#include <mqlutil.h>
 
 #include <openreports.h>
 
-#include <currcluster.h>
+#include "currcluster.h"
 
 #include "applyARCreditMemo.h"
 #include "arOpenItem.h"
@@ -36,6 +36,7 @@
 #include "dspSalesOrderStatus.h"
 #include "dspShipmentsBySalesOrder.h"
 #include "getGLDistDate.h"
+#include "guiErrorCheck.h"
 #include "invoice.h"
 #include "incident.h"
 #include "printArOpenItem.h"
@@ -45,6 +46,8 @@
 #include "salesOrder.h"
 #include "storedProcErrorLookup.h"
 
+#include "errorReporter.h"
+
 dspAROpenItems::dspAROpenItems(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspAROpenItems", fl)
 {
@@ -80,6 +83,7 @@ dspAROpenItems::dspAROpenItems(QWidget* parent, const char*, Qt::WindowFlags fl)
   list()->addColumn(tr("Recurring"),       _ynColumn, Qt::AlignCenter, false, "recurring");
   list()->addColumn(tr("Open"),            _ynColumn, Qt::AlignCenter, false, "open");
   list()->addColumn(tr("Doc. #"),       _orderColumn, Qt::AlignLeft,   true,  "docnumber");
+  list()->addColumn(tr("Cust. PO#"),    _orderColumn, Qt::AlignLeft,   false, "custpo");
   list()->addColumn(tr("Cust./Assign To"),_itemColumn, Qt::AlignLeft,  false, "cust_number");
   list()->addColumn(tr("Name/Desc."),             -1, Qt::AlignLeft,   true,  "cust_name");
   list()->addColumn(tr("Order/Incident"),_itemColumn, Qt::AlignRight,  false, "ordernumber");
@@ -89,8 +93,8 @@ dspAROpenItems::dspAROpenItems(QWidget* parent, const char*, Qt::WindowFlags fl)
   list()->addColumn(baseAmountTitle,    _moneyColumn, Qt::AlignRight,  false,  "base_amount");
   list()->addColumn(tr("Paid"),         _moneyColumn, Qt::AlignRight,  true,  "paid");
   list()->addColumn(basePaidTitle,      _moneyColumn, Qt::AlignRight,  false,  "base_paid");
-  list()->addColumn(tr("Balance"),      _moneyColumn, Qt::AlignRight,  true,  "balance");
-  list()->addColumn(tr("Currency"),  _currencyColumn, Qt::AlignLeft,   true,  "currAbbr");
+  list()->addColumn(tr("Balance"),      _moneyColumn, Qt::AlignRight,  !omfgThis->singleCurrency(),  "balance");
+  list()->addColumn(tr("Currency"),  _currencyColumn, Qt::AlignLeft,   !omfgThis->singleCurrency(),  "currAbbr");
   list()->addColumn(baseBalanceTitle,   _moneyColumn, Qt::AlignRight,  true,  "base_balance");
   list()->addColumn(tr("Credit Card"),            -1, Qt::AlignLeft,   false, "ccard_number");
   list()->addColumn(tr("Notes"),                  -1, Qt::AlignLeft,   false, "notes");
@@ -98,12 +102,6 @@ dspAROpenItems::dspAROpenItems(QWidget* parent, const char*, Qt::WindowFlags fl)
   connect(omfgThis, SIGNAL(creditMemosUpdated()), this, SLOT(sFillList()));
   connect(omfgThis, SIGNAL(invoicesUpdated(int, bool)), this, SLOT(sFillList()));
 
-  if (omfgThis->singleCurrency())
-  {
-    list()->hideColumn("currAbbr");
-    list()->hideColumn("balance");
-  }
-
   disconnect(newAction(), SIGNAL(triggered()), this, SLOT(sNew()));
   connect(newAction(), SIGNAL(triggered()), this, SLOT(sCreateInvoice()));
 
@@ -116,7 +114,7 @@ dspAROpenItems::dspAROpenItems(QWidget* parent, const char*, Qt::WindowFlags fl)
   menuItem = newMenu->addAction(tr("Misc. Debit Memo"),   this, SLOT(sEnterMiscArDebitMemo()));
   menuItem->setEnabled(_privileges->check("MaintainARMemos"));
   newMenu->addSeparator();
-  menuItem = newMenu->addAction(tr("Return"), this, SLOT(sNewCreditMemo()));
+  menuItem = newMenu->addAction(tr("Sales Credit"), this, SLOT(sNewCreditMemo()));
   menuItem->setEnabled(_privileges->check("MaintainCreditMemos"));
   menuItem = newMenu->addAction(tr("Misc. Credit Memo"),   this, SLOT(sEnterMiscArCreditMemo()));
   menuItem->setEnabled(_privileges->check("MaintainARMemos"));
@@ -195,7 +193,7 @@ void dspAROpenItems::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem *pItem, int)
   else if (((XTreeWidgetItem *)pItem)->altId() == 1 && ((XTreeWidgetItem *)pItem)->id("docnumber") > -1)
   // Credit Memo
   {
-    menuItem = pMenu->addAction(tr("Edit Return..."), this, SLOT(sEdit()));
+    menuItem = pMenu->addAction(tr("Edit Sales Credit..."), this, SLOT(sEdit()));
     menuItem->setEnabled(_privileges->check("MaintainCreditMemos"));
   }
   else if (((XTreeWidgetItem *)pItem)->id() > 0)
@@ -242,11 +240,11 @@ void dspAROpenItems::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem *pItem, int)
   {
     if(((XTreeWidgetItem *)pItem)->rawValue("posted") != 0)
     {
-      menuItem = pMenu->addAction(tr("Void Posted Credit Memo..."), this, SLOT(sVoidCreditMemo()));
+      menuItem = pMenu->addAction(tr("Void Posted Sales Credit..."), this, SLOT(sVoidCreditMemo()));
       menuItem->setEnabled(_privileges->check("VoidPostedARCreditMemos"));
     }
 
-    menuItem = pMenu->addAction(tr("View Return..."), this, SLOT(sViewCreditMemo()));
+    menuItem = pMenu->addAction(tr("View Sales Credit..."), this, SLOT(sViewCreditMemo()));
     menuItem->setEnabled(_privileges->check("MaintainCreditMemos") || _privileges->check("ViewCreditMemos"));
   }
   else if (((XTreeWidgetItem *)pItem)->altId() == 4)
@@ -388,13 +386,14 @@ void dspAROpenItems::sApplyAropenCM()
 void dspAROpenItems::sCCRefundCM()
 {
   XSqlQuery dspCCRefundCM;
-  if (list()->id("ccard_number") < 0)
-  {
-    QMessageBox::warning(this, tr("Cannot Refund by Credit Card"),
-			 tr("<p>The application cannot refund this "
-			    "transaction using a credit card."));
+
+  QList<GuiErrorCheck> errors;
+  errors<< GuiErrorCheck(list()->id("ccard_number") < 0, _credits,
+                         tr("The application cannot refund this "
+                            "transaction using a credit card."))
+  ;
+  if (GuiErrorCheck::reportErrors(this, tr("Cannot Refund by Credit Card"), errors))
     return;
-  }
   
   int     ccardid = -1;
   double  total   =  0.0;
@@ -424,9 +423,9 @@ void dspAROpenItems::sCCRefundCM()
     currid  = dspCCRefundCM.value("aropen_curr_id").toInt();
     docnum  = dspCCRefundCM.value("aropen_docnumber").toString();
   }
-  else if (dspCCRefundCM.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving AR Information"),
+                                dspCCRefundCM, __FILE__, __LINE__))
   {
-    systemError(this, dspCCRefundCM.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   else
@@ -463,9 +462,9 @@ void dspAROpenItems::sCCRefundCM()
 
 void dspAROpenItems::sDeleteCreditMemo()
 {
-  if (QMessageBox::question(this, tr("Delete Selected Returns?"),
+  if (QMessageBox::question(this, tr("Delete Selected Credit Memos?"),
                             tr("<p>Are you sure that you want to delete the "
-			       "selected Returns?"),
+			       "selected Credit Memos?"),
                             QMessageBox::Yes, QMessageBox::No | QMessageBox::Default) == QMessageBox::Yes)
   {
     XSqlQuery delq;
@@ -478,13 +477,14 @@ void dspAROpenItems::sDeleteCreditMemo()
       if (delq.first())
       {
             if (! delq.value("result").toBool())
-              systemError(this, tr("Could not delete Return."),
-                          __FILE__, __LINE__);
+              ErrorReporter::error(QtCriticalMsg, this,
+                                 tr("Error Deleting Credit Memo %1\n").arg(list()->currentItem()->text("docnumber")),
+                                 delq, __FILE__, __LINE__);
       }
       else if (delq.lastError().type() != QSqlError::NoError)
-            systemError(this,
-                        tr("Error deleting Return %1\n").arg(list()->currentItem()->text("docnumber")) +
-                        delq.lastError().databaseText(), __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg,
+                         this, tr("Error Deleting Credit Memo %1\n").arg(list()->currentItem()->text("docnumber")),
+                         delq, __FILE__, __LINE__);
     }
 
     omfgThis->sCreditMemosUpdated();
@@ -510,14 +510,16 @@ void dspAROpenItems::sDeleteInvoice()
             int result = dspDeleteInvoice.value("result").toInt();
             if (result < 0)
             {
-              systemError(this, storedProcErrorLookup("deleteInvoice", result),
-                          __FILE__, __LINE__);
+              ErrorReporter::error(QtCriticalMsg, this,
+                                   tr("Error Deleting Invoice %1\n").arg(list()->currentItem()->text("docnumber")),
+                                   storedProcErrorLookup("deleteInvoice", result),
+                                   __FILE__, __LINE__);
             }
       }
-      else if (dspDeleteInvoice.lastError().type() != QSqlError::NoError)
-            systemError(this,
-                        tr("Error deleting Invoice %1\n").arg(list()->currentItem()->text("docnumber")) +
-                        dspDeleteInvoice.lastError().databaseText(), __FILE__, __LINE__);
+      else
+          ErrorReporter::error(QtCriticalMsg, this,
+                               tr("Error Deleting Invoice %1\n").arg(list()->currentItem()->text("docnumber")),
+                               dspDeleteInvoice, __FILE__, __LINE__);
     }
 
     omfgThis->sInvoicesUpdated(-1, true);
@@ -634,8 +636,8 @@ void dspAROpenItems::sVoidCreditMemo()
   XSqlQuery dspVoidCreditMemo;
   XTreeWidgetItem *pItem = list()->currentItem();
   if(pItem->rawValue("posted") != 0 &&
-      QMessageBox::question(this, tr("Void Posted Return?"),
-                            tr("<p>This Return has already been posted. "
+      QMessageBox::question(this, tr("Void Posted Credit Memo?"),
+                            tr("<p>This Credit Memo has already been posted. "
                                "Are you sure you want to void it?"),
                             QMessageBox::Yes,
                             QMessageBox::No | QMessageBox::Default) == QMessageBox::No)
@@ -643,29 +645,71 @@ void dspAROpenItems::sVoidCreditMemo()
     return;
   }
 
+  bool hasControlledItems = false;
+  int itemlocSeries = distributeInventory::SeriesCreate(0, 0, QString(), QString());
+  if (itemlocSeries < 0)
+    return;
+
+  XSqlQuery cleanup; // Stage cleanup function to be called on error
+  cleanup.prepare("SELECT deleteitemlocseries(:itemlocSeries, TRUE);");
+  cleanup.bindValue(":itemlocSeries", itemlocSeries);
+  
+  // Cycle through credit memo items that are controlled and have qty returned, create an itemlocdist record for each
+  XSqlQuery cmitems;
+  cmitems.prepare("SELECT cmitem_id, cmitem_itemsite_id, item_number, "
+                  " (cmitem_qtyreturned * cmitem_qty_invuomratio) * -1 AS qty "
+                  "FROM cmhead "
+                  " JOIN cmitem ON cmhead_id=cmitem_cmhead_id "
+                  " JOIN itemsite ON cmitem_itemsite_id=itemsite_id "
+                  " JOIN item ON itemsite_item_id=item_id "
+                  "WHERE cmhead_id=:cmheadId "
+                  " AND cmitem_qtyreturned <> 0 "
+                  " AND cmitem_updateinv "
+                  " AND isControlledItemsite(cmitem_itemsite_id) "
+                  "ORDER BY cmitem_id;");
+  cmitems.bindValue(":cmheadId", list()->currentItem()->id("docnumber"));
+  cmitems.exec();
+  while (cmitems.next())
+  {
+    if (distributeInventory::SeriesCreate(cmitems.value("cmitem_itemsite_id").toInt(), 
+      cmitems.value("qty").toDouble(), "CM", "RS", cmitems.value("cmitem_id").toInt(), itemlocSeries) < 0)
+    {
+      cleanup.exec();
+      return;
+    }
+
+    hasControlledItems = true;
+  }
+
+  // Distribute detail for the records created above
+  if (hasControlledItems && distributeInventory::SeriesAdjust(itemlocSeries, this, QString(), QDate(),
+    QDate(), true) == XDialog::Rejected)
+  {
+    cleanup.exec();
+    QMessageBox::information( this, tr("Void Credit Memo"), tr("Transaction Cancelled") );
+    return;
+  }
+
   XSqlQuery rollback;
   rollback.prepare("ROLLBACK;");
 
+  dspVoidCreditMemo.exec("BEGIN;"); // TODO - remove this once voidCreditMemo no longer returns negative error codes
   XSqlQuery post;
-  post.prepare("SELECT voidCreditMemo(:cmhead_id) AS result;");
-
-  dspVoidCreditMemo.exec("BEGIN;");	// because of possible lot, serial, or location distribution cancelations
+  post.prepare("SELECT voidCreditMemo(:cmhead_id, :itemlocSeries, TRUE) AS result;");
   post.bindValue(":cmhead_id", list()->currentItem()->id("docnumber"));
+  post.bindValue(":itemlocSeries", itemlocSeries);
   post.exec();
   if (post.first())
   {
     int result = post.value("result").toInt();
-    if (result < 0)
+    if (result < 0 || result != itemlocSeries)
     {
       rollback.exec();
-      systemError(this, storedProcErrorLookup("voidCreditMemo", result),
-                      __FILE__, __LINE__);
-      return;
-    }
-    else if (distributeInventory::SeriesAdjust(result, this) == XDialog::Rejected)
-    {
-      rollback.exec();
-      QMessageBox::information( this, tr("Void Return"), tr("Transaction Canceled") );
+      cleanup.exec();
+      ErrorReporter::error(QtCriticalMsg, this,
+                              tr("Error Voiding Credit Memo %1\n").arg(list()->currentItem()->text("docnumber")),
+                               storedProcErrorLookup("voidCreditMemo", result),
+                               __FILE__, __LINE__);
       return;
     }
 
@@ -675,9 +719,10 @@ void dspAROpenItems::sVoidCreditMemo()
   else if (post.lastError().type() != QSqlError::NoError)
   {
     rollback.exec();
-    systemError(this, tr("A System Error occurred voiding Return.\n%1")
-                .arg(post.lastError().databaseText()),
-                __FILE__, __LINE__);
+    cleanup.exec();
+    ErrorReporter::error(QtCriticalMsg,this,
+                         tr("Error Voiding Credit Memo %1\n").arg(list()->currentItem()->text("docnumber")),
+                         post, __FILE__, __LINE__);
   }
 }
 
@@ -797,29 +842,113 @@ void dspAROpenItems::sVoidInvoiceDetails()
     return;
   }
 
+  bool hasControlledItems = false;
+  int itemlocSeries = distributeInventory::SeriesCreate(0, 0, QString(), QString());
+  if (itemlocSeries < 0)
+    return;
+
+  XSqlQuery cleanup; // Stage cleanup function to be called on error
+  cleanup.prepare("SELECT deleteitemlocseries(:itemlocSeries, TRUE);");
+  cleanup.bindValue(":itemlocSeries", itemlocSeries);
+  
+  // Cycle through credit memo items that are controlled and have qty returned, create an itemlocdist record for each
+  XSqlQuery invcitems;
+  invcitems.prepare("SELECT invcitem_id, itemsite_id, item_number, "
+                    " (invcitem_billed * invcitem_qty_invuomratio) AS qty "
+                    "FROM invchead "
+                    " JOIN invcitem ON invcitem_invchead_id=invchead_id "
+                    "   AND invcitem_billed <> 0 "
+                    "   AND invcitem_updateinv "
+                    " JOIN itemsite ON itemsite_item_id=invcitem_item_id "
+                    "   AND itemsite_warehous_id=invcitem_warehous_id "
+                    " JOIN item ON item_id=invcitem_item_id "
+                    "WHERE invchead_id=:invcheadId "
+                    " AND isControlledItemsite(itemsite_id) "
+                    "ORDER BY invcitem_id;");
+  invcitems.bindValue(":invcheadId", list()->currentItem()->id("docnumber"));
+  invcitems.exec();
+  while (invcitems.next())
+  {
+    if (distributeInventory::SeriesCreate(invcitems.value("itemsite_id").toInt(), 
+      invcitems.value("qty").toDouble(), "IN", "SH", invcitems.value("invcitem_id").toInt(), itemlocSeries) < 0)
+    {
+      cleanup.exec();
+    }
+
+    hasControlledItems = true;
+  }
+
+  // Distribute detail for the records created above
+  if (hasControlledItems && distributeInventory::SeriesAdjust(itemlocSeries, this, QString(), QDate(),
+    QDate(), true) == XDialog::Rejected)
+  {
+    cleanup.exec();
+    QMessageBox::information( this, tr("Void Invoice"), tr("Transaction Cancelled") );
+    return;
+  }
+
+  QVariant voidDate; // starts null
+  while (_privileges->check("ChangeARInvcDistDate"))
+  {
+    getGLDistDate newdlg(this, "", true);
+    newdlg.sSetDefaultLit(tr("Invoice Posting Date"));
+    if (newdlg.exec() == XDialog::Accepted)
+    {
+      voidDate = QVariant(newdlg.date());
+
+      XSqlQuery closedPeriod;
+      closedPeriod.prepare("SELECT period_closed "
+                           "  FROM invchead "
+                           "  JOIN period ON COALESCE(:distdate, invchead_gldistdate, "
+                           "                          invchead_invcdate) BETWEEN "
+                           "                 period_start AND period_end "
+                           " WHERE invchead_id=:invchead_id;");
+      closedPeriod.bindValue(":distdate", voidDate);
+      closedPeriod.bindValue(":invchead_id", list()->currentItem()->id("docnumber"));
+      closedPeriod.exec();
+      if (closedPeriod.first() && !closedPeriod.value("period_closed").toBool())
+        break;
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error voiding invoice"),
+                                    closedPeriod, __FILE__, __LINE__))
+        return;
+      else
+      {
+        if (QMessageBox::question(this, tr("Change Date?"),
+                                  tr("Could not void invoice because the accounting period for the "
+                                     "posting date is closed. Try again with a different date?"),
+                                  QMessageBox::Yes | QMessageBox::No,
+                                  QMessageBox::No) == QMessageBox::Yes)
+          continue;
+        else
+          return;
+      }
+    }
+    else
+      return;
+  }
+
   XSqlQuery rollback;
   rollback.prepare("ROLLBACK;");
 
-  XSqlQuery post;
-  post.prepare("SELECT voidInvoice(:invchead_id) AS result;");
+  dspVoidInvoiceDetails.exec("BEGIN;"); // TODO - remove after voidInvoice no longer returns negative error codes
 
-  dspVoidInvoiceDetails.exec("BEGIN;");	// because of possible lot, serial, or location distribution cancelations
-  post.bindValue(":invchead_id", list()->currentItem()->id("docnumber"));
+  XSqlQuery post;
+  post.prepare("SELECT voidInvoice(:invchead_id, :itemlocSeries, TRUE, :voidDate) AS result;");
+  post.bindValue(":invchead_id",   list()->currentItem()->id("docnumber"));
+  post.bindValue(":itemlocSeries", itemlocSeries);
+  post.bindValue(":voidDate",      voidDate);
   post.exec();
   if (post.first())
   {
     int result = post.value("result").toInt();
-    if (result < 0)
+    if (result < 0 || result != itemlocSeries)
     {
       rollback.exec();
-      systemError(this, storedProcErrorLookup("voidInvoice", result),
-                      __FILE__, __LINE__);
-      return;
-    }
-    else if (distributeInventory::SeriesAdjust(result, this) == XDialog::Rejected)
-    {
-      rollback.exec();
-      QMessageBox::information( this, tr("Void Invoice"), tr("Transaction Canceled") );
+      cleanup.exec();
+      ErrorReporter::error(QtCriticalMsg, this,
+                           tr("Error Voiding Invoice %1\n").arg(list()->currentItem()->text("docnumber")),
+                           storedProcErrorLookup("voidInvoice", result),
+                           __FILE__, __LINE__);
       return;
     }
 
@@ -829,9 +958,10 @@ void dspAROpenItems::sVoidInvoiceDetails()
   else if (post.lastError().type() != QSqlError::NoError)
   {
     rollback.exec();
-    systemError(this, tr("A System Error occurred voiding Invoice.\n%1")
-                .arg(post.lastError().databaseText()),
-                __FILE__, __LINE__);
+    cleanup.exec();
+    ErrorReporter::error(QtCriticalMsg, this,
+                         tr("Error Voiding Invoice %1\n").arg(list()->currentItem()->text("docnumber")),
+                         post, __FILE__, __LINE__);
   }
 }
 
@@ -883,6 +1013,9 @@ void dspAROpenItems::sViewIncident()
 
 bool dspAROpenItems::setParams(ParameterList &params)
 {
+  if (!display::setParams(params))
+    return false;
+
   _customerSelector->appendValue(params);
   if (_docDate->isChecked())
     _dates->appendValue(params);
@@ -892,7 +1025,7 @@ bool dspAROpenItems::setParams(ParameterList &params)
     params.append("endDueDate", _dates->endDate());
   }
   params.append("invoice", tr("Invoice"));
-  params.append("return", tr("Return"));
+  params.append("return", tr("Sales Credit"));
   params.append("creditMemo", tr("Credit Memo"));
   params.append("debitMemo", tr("Debit Memo"));
   params.append("cashdeposit", tr("Customer Deposit"));
@@ -922,7 +1055,7 @@ void dspAROpenItems::sPreview()
     ParameterList params;
     params.append("cust_id", _customerSelector->custId());
     params.append("invoice",  tr("Invoice"));
-    params.append("return",   tr("Return"));
+    params.append("return",   tr("Sales Credit"));
     params.append("debit",    tr("Debit Memo"));
     params.append("credit",   tr("Credit Memo"));
     params.append("deposit",  tr("Deposit"));
@@ -1049,15 +1182,17 @@ void dspAROpenItems::sPostCreditMemo()
     
   bool changeDate = false;
   QDate newDate = QDate::currentDate();
+  QDate seriesDate;
 
   if (_privileges->check("ChangeSOMemoPostDate"))
   {
     getGLDistDate newdlg(this, "", true);
-    newdlg.sSetDefaultLit(tr("Return Date"));
+    newdlg.sSetDefaultLit(tr("Credit Memo Date"));
     if (newdlg.exec() == XDialog::Accepted)
     {
       newDate = newdlg.date();
       changeDate = (newDate.isValid());
+      seriesDate = newdlg.seriesDate();
     }
     else
       return;
@@ -1074,58 +1209,96 @@ void dspAROpenItems::sPostCreditMemo()
     setDate.exec();
     if (setDate.lastError().type() != QSqlError::NoError)
     {
-          systemError(this, setDate.lastError().databaseText(), __FILE__, __LINE__);
+          ErrorReporter::error(QtCriticalMsg, this,
+                               tr("Error Posting Credit Memo %1\n").arg(list()->currentItem()->text("docnumber")),
+                               setDate, __FILE__, __LINE__);
     }
   }
 
+  bool hasControlledItems = false;
+  int itemlocSeries = distributeInventory::SeriesCreate(0, 0, QString(), QString());
+  if (itemlocSeries < 0)
+    return;
+
+  XSqlQuery cleanup; // Stage cleanup function to be called on error
+  cleanup.prepare("SELECT deleteitemlocseries(:itemlocSeries, TRUE);");
+  cleanup.bindValue(":itemlocSeries", itemlocSeries);
+  
+  // Cycle through credit memo items that are controlled and have qty returned, create an itemlocdist record for each
+  XSqlQuery cmitems;
+  cmitems.prepare("SELECT itemsite_id, item_number, "
+                    " SUM(cmitem_qtyreturned * cmitem_qty_invuomratio) AS qty "
+                    "FROM cmhead JOIN cmitem ON cmitem_cmhead_id=cmhead_id "
+                    " JOIN itemsite ON itemsite_id=cmitem_itemsite_id "
+                    " JOIN item ON item_id=itemsite_item_id "
+                    " JOIN costcat ON costcat_id=itemsite_costcat_id "
+                    "WHERE cmitem_qtyreturned <> 0 "
+                    " AND cmitem_updateinv "
+                    " AND cmhead_id=:cmheadId "
+                    " AND isControlledItemsite(itemsite_id) "
+                    " AND itemsite_costmethod != 'J' "
+                    "GROUP BY itemsite_id, item_number "
+                    "ORDER BY itemsite_id;");
+  cmitems.bindValue(":cmheadId", id);
+  cmitems.exec();
+  while (cmitems.next())
+  {
+    if (distributeInventory::SeriesCreate(cmitems.value("itemsite_id").toInt(), 
+      cmitems.value("qty").toDouble(), "CM", "RS", cmitems.value("cmitem_id").toInt(), itemlocSeries) < 0)
+    {
+      cleanup.exec();
+      return;
+    }
+
+    hasControlledItems = true;
+  }
+
+  // Distribute detail for the records created above
+  if (hasControlledItems && distributeInventory::SeriesAdjust(itemlocSeries, this, QString(), QDate(),
+    QDate(), true) == XDialog::Rejected)
+  {
+    cleanup.exec();
+    QMessageBox::information( this, tr("Post Credit Memo"), tr("Transaction Cancelled") );
+    return;
+  }
+
   XSqlQuery rollback;
   rollback.prepare("ROLLBACK;");
+
+  XSqlQuery tx;
+  tx.exec("BEGIN;");  // TODO - remove this after postCreditMemo can no longer return negative error codes
     
   XSqlQuery postq;
-  postq.prepare("SELECT postCreditMemo(:cmhead_id, 0) AS result;");
- 
-  XSqlQuery tx;
-  tx.exec("BEGIN;");	// because of possible lot, serial, or location distribution cancelations
-
+  postq.prepare("SELECT postCreditMemo(:cmhead_id, fetchJournalNumber('AR-CM', :seriesDate), :itemlocSeries, TRUE) AS result;");
   postq.bindValue(":cmhead_id", id);
+  postq.bindValue(":itemlocSeries", itemlocSeries);
+  postq.bindValue(":seriesDate", seriesDate);
   postq.exec();
   if (postq.first())
   {
     int result = postq.value("result").toInt();
-    if (result < 0)
+    if (result < 0 || result != itemlocSeries)
     {
       rollback.exec();
-      systemError( this, storedProcErrorLookup("postCreditMemo", result),
-            __FILE__, __LINE__);
+      cleanup.exec();
+      ErrorReporter::error(QtCriticalMsg, this,
+                           tr("Error Posting Credit Memo %1\n").arg(list()->currentItem()->text("docnumber")),
+                           storedProcErrorLookup("postCreditMemo", result),
+                           __FILE__, __LINE__);
       return;
     }
     else
-    {
-      if (distributeInventory::SeriesAdjust(result, this) == XDialog::Rejected)
-      {
-        rollback.exec();
-        QMessageBox::information( this, tr("Post Return"), tr("Transaction Canceled") );
-        return;
-      }
-
       dspPostCreditMemo.exec("COMMIT;");
-    }
-  }
-  // contains() string is hard-coded in stored procedure
-  else if (postq.lastError().databaseText().contains("post to closed period"))
-  {
-    rollback.exec();
-    systemError(this, tr("Could not post Return #%1 because of a missing exchange rate.")
-                                      .arg(list()->currentItem()->text("docnumber")));
-    return;
   }
   else if (postq.lastError().type() != QSqlError::NoError)
   {
     rollback.exec();
-    systemError(this, tr("A System Error occurred posting Return#%1.\n%2")
-            .arg(list()->currentItem()->text("docnumber"))
-            .arg(postq.lastError().databaseText()),
-          __FILE__, __LINE__);
+    cleanup.exec();
+    ErrorReporter::error(QtCriticalMsg, this,
+                         tr("Error Posting Credit Memo#%1.\n%2")
+                                     .arg(list()->currentItem()->text("docnumber"))
+                                     .arg(postq.lastError().databaseText()),
+                         postq, __FILE__, __LINE__);
   }
 
   omfgThis->sCreditMemosUpdated();
@@ -1139,6 +1312,7 @@ void dspAROpenItems::sPostInvoice()
     
   bool changeDate = false;
   QDate newDate = QDate::currentDate();
+  QDate seriesDate;
 
   if (!checkInvoiceSitePrivs(list()->currentItem()->id("docnumber")))
     return;
@@ -1151,25 +1325,37 @@ void dspAROpenItems::sPostInvoice()
     {
       newDate = newdlg.date();
       changeDate = (newDate.isValid());
+      seriesDate = newdlg.seriesDate();
     }
     else
       return;
   }
 
   int journal = -1;
-  dspPostInvoice.exec("SELECT fetchJournalNumber('AR-IN') AS result;");
+  dspPostInvoice.prepare("SELECT fetchJournalNumber('AR-IN', :seriesDate) AS result;");
+  dspPostInvoice.bindValue(":seriesDate", seriesDate);
+  dspPostInvoice.exec();
   if (dspPostInvoice.first())
   {
     journal = dspPostInvoice.value("result").toInt();
     if (journal < 0)
     {
-      systemError(this, storedProcErrorLookup("fetchJournalNumber", journal), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this,
+                           tr("Error Posting Invoice#%1.\n%2")
+                                       .arg(list()->currentItem()->text("docnumber"))
+                                       .arg(dspPostInvoice.lastError().databaseText()),
+                             storedProcErrorLookup("fetchJournalNumber", journal),
+                             __FILE__, __LINE__);
       return;
     }
   }
   else if (dspPostInvoice.lastError().type() != QSqlError::NoError)
   {
-    systemError(this, dspPostInvoice.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this,
+                         tr("Error Posting Invoice#%1.\n%2")
+                                     .arg(list()->currentItem()->text("docnumber"))
+                                     .arg(dspPostInvoice.lastError().databaseText()),
+                         dspPostInvoice, __FILE__, __LINE__);
     return;
   }
 
@@ -1183,12 +1369,6 @@ void dspAROpenItems::sPostInvoice()
   XSqlQuery sum;
   sum.prepare("SELECT invoicetotal(:invchead_id) AS subtotal;");
 
-  XSqlQuery rollback;
-  rollback.prepare("ROLLBACK;");
-
-  XSqlQuery post;
-  post.prepare("SELECT postInvoice(:invchead_id, :journal) AS result;");
-
   XSqlQuery setDate;
   setDate.prepare("UPDATE invchead SET invchead_gldistdate=:distdate "
 		  "WHERE invchead_id=:invchead_id;");
@@ -1199,7 +1379,11 @@ void dspAROpenItems::sPostInvoice()
     setDate.bindValue(":invchead_id", list()->currentItem()->id("docnumber"));
     setDate.exec();
     if (setDate.lastError().type() != QSqlError::NoError)
-      systemError(this, setDate.lastError().databaseText(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this,
+                         tr("Error Posting Invoice#%1.\n%2")
+                                       .arg(list()->currentItem()->text("docnumber"))
+                                       .arg(setDate.lastError().databaseText()),
+                         setDate, __FILE__, __LINE__);
   }
 
   sum.bindValue(":invchead_id", list()->currentItem()->id("docnumber"));
@@ -1223,67 +1407,119 @@ void dspAROpenItems::sPostInvoice()
       xrate.exec();
       if (xrate.lastError().type() != QSqlError::NoError)
       {
-        systemError(this, tr("System Error posting Invoice #%1\n%2")
-                            .arg(list()->currentItem()->text("docnumber"))
-                            .arg(xrate.lastError().databaseText()),
-                        __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this,
+                             tr("Error Posting Invoice#%1.\n%2")
+                                         .arg(list()->currentItem()->text("docnumber"))
+                                         .arg(xrate.lastError().databaseText()),
+                             xrate, __FILE__, __LINE__);
         return;
       }
       else if (!xrate.first() || xrate.value("curr_rate").isNull())
       {
-        systemError(this, tr("Could not post Invoice #%1 because of a missing exchange rate.")
-                                        .arg(list()->currentItem()->text("docnumber")));
+        ErrorReporter::error(QtCriticalMsg, this,
+                             tr("Error Posting Invoice#%1.\n%2")
+                                         .arg(list()->currentItem()->text("docnumber"))
+                                         .arg(xrate.lastError().databaseText()),
+                             xrate, __FILE__, __LINE__);
         return;
       }
     }
   }
   else if (sum.lastError().type() != QSqlError::NoError)
   {
-    systemError(this, sum.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this,
+                         tr("Error Posting Invoice#%1.\n%2")
+                                     .arg(list()->currentItem()->text("docnumber"))
+                                     .arg(sum.lastError().databaseText()),
+                         sum, __FILE__, __LINE__);
     return;
   }
 
-  XSqlQuery tx;
-  tx.exec("BEGIN;");	// because of possible lot, serial, or location distribution cancelations
+  bool hasControlledItems = false;
+  int itemlocSeries = distributeInventory::SeriesCreate(0, 0, QString(), QString());
+  if (itemlocSeries < 0)
+    return;
 
+  XSqlQuery cleanup; // Stage cleanup function to be called on error
+  cleanup.prepare("SELECT deleteitemlocseries(:itemlocSeries, TRUE);");
+  cleanup.bindValue(":itemlocSeries", itemlocSeries);
+
+  // Cycle through credit memo items that are controlled and have qty returned, create an itemlocdist record for each
+  XSqlQuery invcitems;
+  invcitems.prepare("SELECT invcitem_id, itemsite_id, item_number, "
+                  " (invcitem_billed * invcitem_qty_invuomratio) * -1 AS qty "
+                  "FROM invchead "
+                  " JOIN invcitem ON invcitem_invchead_id=invchead_id "
+                  "   AND invcitem_billed <> 0 "
+                  "   AND invcitem_updateinv "
+                  " JOIN itemsite ON itemsite_item_id=invcitem_item_id "
+                  "   AND itemsite_warehous_id=invcitem_warehous_id "
+                  " JOIN item ON item_id=invcitem_item_id "
+                  "WHERE invchead_id=:invcheadId "
+                  " AND isControlledItemsite(itemsite_id) "
+                  " AND itemsite_costmethod != 'J' "
+                  "ORDER BY invcitem_id;");
+  invcitems.bindValue(":invcheadId", list()->currentItem()->id("docnumber"));
+  invcitems.exec();
+  while (invcitems.next())
+  {
+    if (distributeInventory::SeriesCreate(invcitems.value("itemsite_id").toInt(), 
+      invcitems.value("qty").toDouble(), "IN", "SH", invcitems.value("invcitem_id").toInt(), itemlocSeries) < 0)
+    {
+      cleanup.exec();
+      return;
+    }
+
+    hasControlledItems = true;
+  }
+
+  // Distribute detail for the records created above
+  if (hasControlledItems && distributeInventory::SeriesAdjust(itemlocSeries, this, QString(), QDate(),
+    QDate(), true) == XDialog::Rejected)
+  {
+    cleanup.exec();
+    QMessageBox::information( this, tr("Post Invoice"), tr("Transaction Cancelled") );
+    return;
+  }
+
+  XSqlQuery rollback;
+  rollback.prepare("ROLLBACK;");
+
+  XSqlQuery tx;
+  tx.exec("BEGIN;");	// TODO - remove this after postInvoice function no longer returns negative error codes.
+
+  XSqlQuery post;
+  post.prepare("SELECT postInvoice(:invchead_id, :journal, :itemlocSeries, TRUE) AS result;");
   post.bindValue(":invchead_id", list()->currentItem()->id("docnumber"));
   post.bindValue(":journal",     journal);
+  post.bindValue(":itemlocSeries", itemlocSeries);
   post.exec();
   if (post.first())
   {
     int result = post.value("result").toInt();
-    if (result < 0)
+    if (result < 0 || result != itemlocSeries)
     {
       rollback.exec();
-      systemError(this, storedProcErrorLookup("postInvoice", result),
-                  __FILE__, __LINE__);
+      cleanup.exec();
+      ErrorReporter::error(QtCriticalMsg, this,
+                           tr("Error Posting Invoice#%1.\n%2")
+                                       .arg(list()->currentItem()->text("docnumber"))
+                                       .arg(post.lastError().databaseText()),
+                           storedProcErrorLookup("postInvoice", result),
+                           __FILE__, __LINE__);
     }
     else
-    {
-      if (distributeInventory::SeriesAdjust(result, this) == XDialog::Rejected)
-      {
-        rollback.exec();
-        QMessageBox::information( this, tr("Post Invoice"), tr("Transaction Canceled") );
-        return;
-      }
-
       dspPostInvoice.exec("COMMIT;");
-    }
-  }
-  // contains() string is hard-coded in stored procedure
-  else if (post.lastError().databaseText().contains("post to closed period"))
-  {
-    rollback.exec();
-    systemError(this, tr("Could not post Invoice #%1 into a closed period.")
-                            .arg(list()->currentItem()->text("docnumber")));
   }
   else if (post.lastError().type() != QSqlError::NoError)
   {
     rollback.exec();
-    systemError(this, tr("A System Error occurred posting Invoice #%1.\n%2")
-                .arg(list()->currentItem()->text("docnumber"))
-                    .arg(post.lastError().databaseText()),
-                    __FILE__, __LINE__);
+    cleanup.exec();
+    ErrorReporter::error(QtCriticalMsg, this,
+                         tr("Error Posting Invoice#%1.\n%2")
+                                     .arg(list()->currentItem()->text("docnumber"))
+                                     .arg(post.lastError().databaseText()),
+                         post, __FILE__, __LINE__);
   }
 
   omfgThis->sInvoicesUpdated(-1, true);
@@ -1343,7 +1579,7 @@ bool dspAROpenItems::checkCreditMemoSitePrivs(int cmid)
     if (!check.value("result").toBool())
       {
         QMessageBox::critical(this, tr("Access Denied"),
-                                       tr("You may not view or edit this Return as it references "
+                                       tr("You may not view or edit this Credit Memo as it references "
                                        "a Site for which you have not been granted privileges.")) ;
         return false;
       }
