@@ -20,7 +20,7 @@ class dspBillingSelections : public XWidget, public Ui::dspBillingSelections
     Q_OBJECT
 
 public:
-    dspBillingSelections(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = Qt::Window);
+    dspBillingSelections(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::Window);
     ~dspBillingSelections();
 
 public slots:
