@@ -29,6 +29,7 @@
 #include "dspInventoryHistory.h"
 #include "dspCountSlipEditList.h"
 #include "storedProcErrorLookup.h"
+#include "errorReporter.h"
 
 dspCountTagEditList::dspCountTagEditList(QWidget* parent, const char* name, Qt::WindowFlags fl)
     : XWidget(parent, name, fl)
@@ -239,9 +240,9 @@ void dspCountTagEditList::sViewInventoryHistory()
     newdlg->set(params);
     omfgThis->handleNewWindow(newdlg);
   }
-  else if (dspViewInventoryHistory.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Inventory Information"),
+                                dspViewInventoryHistory, __FILE__, __LINE__))
   {
-    systemError(this, dspViewInventoryHistory.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -329,8 +330,9 @@ void dspCountTagEditList::sDelete()
       }
       else if (dspDelete.lastError().type() != QSqlError::NoError)
       {
-	systemError(this, dspDelete.lastError().databaseText(), __FILE__, __LINE__);
-	return;
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Count Tag"),
+                         dspDelete, __FILE__, __LINE__);
+        return;
       }
       dspDelete.prepare("SELECT itemsite_id"
 	        "  FROM invcnt JOIN itemsite ON (invcnt_itemsite_id=itemsite_id AND itemsite_freeze) "
@@ -340,8 +342,9 @@ void dspCountTagEditList::sDelete()
 	itemsiteid = dspDelete.value("itemsite_id").toInt();
       else if (dspDelete.lastError().type() != QSqlError::NoError)
       {
-	systemError(this, dspDelete.lastError().databaseText(), __FILE__, __LINE__);
-	return;
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Count Tag"),
+                         dspDelete, __FILE__, __LINE__);
+        return;
       }
 
       dspDelete.prepare( "DELETE FROM cntslip "
@@ -352,8 +355,9 @@ void dspCountTagEditList::sDelete()
       dspDelete.exec();
       if (dspDelete.lastError().type() != QSqlError::NoError)
       {
-	systemError(this, dspDelete.lastError().databaseText(), __FILE__, __LINE__);
-	return;
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Count Tag"),
+                         dspDelete, __FILE__, __LINE__);
+        return;
       }
 
       if (itemsiteid != -1)
@@ -365,21 +369,23 @@ void dspCountTagEditList::sDelete()
 				  QMessageBox::Yes | QMessageBox::Default,
 				  QMessageBox::No | QMessageBox::Escape ) == QMessageBox::Yes )
 	{
-	  dspDelete.prepare("SELECT thawItemsite(:itemsite_id) AS result;");
+      dspDelete.prepare("SELECT thawItemsite(:itemsite_id) AS result;");
 	  dspDelete.bindValue(":itemsite_id", itemsiteid);
 	  dspDelete.exec();
-	  if (dspDelete.first())
+      if (dspDelete.first())
 	  {
 	    int result = dspDelete.value("result").toInt();
 	    if (result < 0)
-	    {
-	      systemError(this, storedProcErrorLookup("thawItemsite", result),
-			  __FILE__, __LINE__);
-	      continue;
-	    }
+        {
+          ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Count Tag"),
+                                 storedProcErrorLookup("thawItemsite", result),
+                                 __FILE__, __LINE__);
+          continue;
+        }
 	    else if (dspDelete.lastError().type() != QSqlError::NoError)
 	    {
-	      systemError(this, dspDelete.lastError().databaseText(), __FILE__, __LINE__);
+          ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Count Tag"),
+                               dspDelete, __FILE__, __LINE__);
 	      return;
 	    }
 	  }
@@ -412,9 +418,9 @@ void dspCountTagEditList::sDeleteTag()
   dspDeleteTag.bindValue(":cnttag_id", _cnttag->id());
   if (dspDeleteTag.exec() && dspDeleteTag.first())
     itemsiteid = dspDeleteTag.value("itemsite_id").toInt();
-  else if (dspDeleteTag.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Count Tag"),
+                                dspDeleteTag, __FILE__, __LINE__))
   {
-    systemError(this, dspDeleteTag.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   
@@ -424,7 +430,8 @@ void dspCountTagEditList::sDeleteTag()
   dspDeleteTag.exec();
   if (dspDeleteTag.lastError().type() != QSqlError::NoError)
   {
-    systemError(this, dspDeleteTag.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Count Tag"),
+                         dspDeleteTag, __FILE__, __LINE__);
     return;
   }
 
@@ -444,21 +451,22 @@ void dspCountTagEditList::sDeleteTag()
       {
 	int result = dspDeleteTag.value("result").toInt();
 	if (result < 0)
-	{
-	  systemError(this, storedProcErrorLookup("thawItemsite", result),
-		      __FILE__, __LINE__);
-	  return;
-	}
-	else if (dspDeleteTag.lastError().type() != QSqlError::NoError)
-	{
-	  systemError(this, dspDeleteTag.lastError().databaseText(), __FILE__, __LINE__);
-	  return;
-	}
+    {
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Count Tag"),
+                             storedProcErrorLookup("thawItemsite", result),
+                             __FILE__, __LINE__);
+      return;
+    }
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Count Tag"),
+                                  dspDeleteTag, __FILE__, __LINE__))
+    {
+      return;
+    }
       }
-      else if (dspDeleteTag.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Count Tag"),
+                                    dspDeleteTag, __FILE__, __LINE__))
       {
-	systemError(this, dspDeleteTag.lastError().databaseText(), __FILE__, __LINE__);
-	return;
+        return;
       }
     }
   }
@@ -473,9 +481,9 @@ void dspCountTagEditList::sDeleteSlip()
                  "WHERE (cntslip_id=:cntslip_id);" );
   dspDeleteSlip.bindValue(":cntslip_id", _cnttag->altId());
   dspDeleteSlip.exec();
-  if (dspDeleteSlip.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Count Slip"),
+                                dspDeleteSlip, __FILE__, __LINE__))
   {
-    systemError(this, dspDeleteSlip.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -560,9 +568,10 @@ void dspCountTagEditList::sFillList()
 
   dspFillList = mql.toQuery(params);
   _cnttag->populate(dspFillList, true);
-  if (dspFillList.lastError().type() != QSqlError::NoError)
+
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Count Tag Information"),
+                                dspFillList, __FILE__, __LINE__))
   {
-    systemError(this, dspFillList.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   
