@@ -16,6 +16,7 @@
 #include <QList>
 #include <QMenu>
 #include <QMessageBox>
+#include "guiErrorCheck.h"
 #include <QSqlError>
 #include <QToolBar>
 #include <QToolButton>
@@ -30,6 +31,7 @@
 #include "dspGLTransactions.h"
 #include "financialReportNotes.h"
 #include "storedProcErrorLookup.h"
+#include "errorReporter.h"
 
 #define cFlRoot  0
 #define cFlItem  1
@@ -134,23 +136,20 @@ void dspFinancialReport::sViewTransactions()
 bool dspFinancialReport::sCheck()
 {
   XSqlQuery dspCheck;
-  if (!_periods->selectedItems().count())
-  {
-    QMessageBox::warning( this, tr("No period selected"),
-                         tr("<p>You must select at least one period.") );
-    return false;
-  }
 
   //Make sure user has upgraded period settings
   dspCheck.exec("SELECT period_id FROM period WHERE period_quarter IS NULL;");
-  if (dspCheck.first())
-  {
-    QMessageBox::warning( this, tr("Setup Incomplete"),
-                         tr("<p>Please make sure all accounting periods "
+
+  QList<GuiErrorCheck> errors;
+  errors<< GuiErrorCheck(!_periods->selectedItems().count(), _periods,
+                         tr("You must select at least one period."))
+        << GuiErrorCheck(dspCheck.first(), _periods,
+                         tr("Please make sure all accounting periods "
                             "are associated with a quarter and fiscal year "
-                            "before using this application.") );
+                            "before using this application."))
+  ;
+  if (GuiErrorCheck::reportErrors(this, tr("No period selected"), errors))
     return false;
-  }
 
   return true;
 }
@@ -455,10 +454,10 @@ void dspFinancialReport::sFillListStatement()
       dspFillListStatement.bindValue(":prjid", _prjid);
       dspFillListStatement.exec();
       list()->populate(dspFillListStatement, true);
-      if (dspFillListStatement.lastError().type() != QSqlError::NoError)
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Financial Information"),
+                                    dspFillListStatement, __FILE__, __LINE__))
       {
-	systemError(this, dspFillListStatement.lastError().databaseText(), __FILE__, __LINE__);
-	return;
+        return;
       }
       list()->expandAll();
     }
@@ -853,9 +852,9 @@ void dspFinancialReport::sFillListTrend()
   dspFillListTrend.bindValue(":spec", cFlSpec);
   dspFillListTrend.exec();
   list()->populate(dspFillListTrend, true);
-  if (dspFillListTrend.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Financial Information"),
+                                dspFillListTrend, __FILE__, __LINE__))
   {
-    systemError(this, dspFillListTrend.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   list()->expandAll();
@@ -1143,13 +1142,15 @@ bool dspFinancialReport::forwardUpdate()
     int result = dspforwardUpdate.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("forwardUpdateTrialBalance", result), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Financial Information"),
+                             storedProcErrorLookup("forwardUpdateTrialBalance", result),
+                             __FILE__, __LINE__);
       return false;
     }
   }
-  else if (dspforwardUpdate.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Financial Information"),
+                                dspforwardUpdate, __FILE__, __LINE__))
   {
-    systemError(this, dspforwardUpdate.lastError().databaseText(), __FILE__, __LINE__);
     return false;
   }
   return true;
