@@ -16,6 +16,8 @@
 //#include <QStatusBar>
 #include <parameter.h>
 #include <openreports.h>
+#include "errorReporter.h"
+#include "guiErrorCheck.h"
 
 /*
  *  Constructs a duplicateAccountNumbers as a child of 'parent', with the
@@ -54,12 +56,13 @@ void duplicateAccountNumbers::languageChange()
 void duplicateAccountNumbers::sDuplicate()
 {
   XSqlQuery duplicateDuplicate;
-  if(!(_changeCompany->isChecked() || _changeProfit->isChecked() || _changeSub->isChecked()))
-  {
-    QMessageBox::warning( this, tr("No Segments Selected for Change"),
-      tr("You have not selected any segments for changing. You must select at least one segment to be changed.") );
-    return;
-  }
+
+  QList<GuiErrorCheck> errors;
+    errors<< GuiErrorCheck(!(_changeCompany->isChecked() || _changeProfit->isChecked() || _changeSub->isChecked()), _changeCompany,
+                           tr("You have not selected any segments for changing. You must select at least one segment to be changed."))
+    ;
+    if (GuiErrorCheck::reportErrors(this, tr("No Segments Selected for Change"), errors))
+      return;
 
   QString sql ("INSERT INTO accnt"
                "      (accnt_number, accnt_descrip,"
@@ -101,12 +104,9 @@ void duplicateAccountNumbers::sDuplicate()
     duplicateDuplicate.bindValue(":accnt_id",	((XTreeWidgetItem*)selected[i])->id());
     duplicateDuplicate.bindValue(":descrip",	_descrip->text());
     duplicateDuplicate.exec();
-    if (duplicateDuplicate.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Duplicating Account Numbers"),
+                                  duplicateDuplicate, __FILE__, __LINE__))
     {
-      systemError(this, tr("A System Error occurred at %1::%2\n\n%3")
-                          .arg(__FILE__)
-                          .arg(__LINE__)
-                          .arg(duplicateDuplicate.lastError().databaseText()));
       duplicateDuplicate.exec("ROLLBACK;");
       return;
     }
