@@ -32,12 +32,14 @@ employees::employees(QWidget* parent, const char*, Qt::WindowFlags fl)
   setSearchVisible(true);
   setQueryOnStartEnabled(true);
 
+  QString qryEmpGrp = QString( "SELECT  empgrp_id, empgrp_name FROM empgrp;");
+
+  parameterWidget()->appendComboBox(tr("Employee Group"), "emp_group", qryEmpGrp);
   parameterWidget()->append(tr("Show Active Only"), "activeOnly", ParameterWidget::Exists);
   if (_metrics->boolean("MultiWhs"))
     parameterWidget()->append(tr("Site"), "warehous_id", ParameterWidget::Site);
-  parameterWidget()->applyDefaultFilterSet();
 
-  connect(omfgThis, SIGNAL(employeesUpdated(int)),     this, SLOT(sFillList()));
+  connect(omfgThis, SIGNAL(employeeUpdated(int)),     this, SLOT(sFillList()));
 
   list()->addColumn(tr("Site"),   _whsColumn,  Qt::AlignLeft, true, "warehous_code");
   list()->addColumn(tr("Active"), _ynColumn,   Qt::AlignLeft, true, "emp_active");
@@ -47,7 +49,6 @@ employees::employees(QWidget* parent, const char*, Qt::WindowFlags fl)
   list()->addColumn(tr("Last"),   _itemColumn, Qt::AlignLeft, true, "cntct_last_name");
 
   setupCharacteristics("EMP");
-  parameterWidget()->applyDefaultFilterSet();
 
   connect(list(), SIGNAL(itemSelected(int)), this, SLOT(sOpen()));
 
