@@ -16,6 +16,7 @@
 
 #include "inputManager.h"
 #include "storedProcErrorLookup.h"
+#include "errorReporter.h"
 
 explodeWo::explodeWo(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
   : XDialog(parent, name, modal, fl)
@@ -78,15 +79,16 @@ void explodeWo::sExplode()
       int result = explodeExplode.value("result").toInt();
       if (result < 0)
       {
-	systemError(this,
-		    storedProcErrorLookup("explodeWo", result).arg(_wo->woNumber()),
-		    __FILE__, __LINE__);
-	return;
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Exploding Work Order"),
+                               storedProcErrorLookup("explodeWo", result)
+                               .arg(_wo->id() ? _wo->woNumber() : ""),
+                               __FILE__, __LINE__);
+        return;
       }
     }
-    else if (explodeExplode.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Exploding Work Order"),
+                                  explodeExplode, __FILE__, __LINE__))
     {
-      systemError(this, explodeExplode.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
 
