@@ -12,10 +12,12 @@
 
 #include <QVariant>
 #include <QMessageBox>
+#include "guiErrorCheck.h"
 #include <QInputDialog>
 //#include <QStatusBar>
 #include <parameter.h>
 #include "financialLayout.h"
+#include "errorReporter.h"
 
 /*
  *  Constructs a financialLayouts as a child of 'parent', with the
@@ -115,13 +117,14 @@ void financialLayouts::sDelete()
             "   AND  (flhead_sys));");
   financialDelete.bindValue(":flhead_id", _flhead->id());
   financialDelete.exec();
-  if (financialDelete.first())
-  {
-    QMessageBox::critical(this, tr("System Report"), 
-            tr("You may not delete a system report,\n"
-               "but you may deactivate it."));
+
+  QList<GuiErrorCheck> errors;
+  errors<< GuiErrorCheck(financialDelete.first(), _new,
+                         tr("You may not delete a system report,\n"
+                            "but you may deactivate it."))
+  ;
+  if (GuiErrorCheck::reportErrors(this, tr("System Report"), errors))
     return;
-  }
 
   if(QMessageBox::question( this, tr("Confirm Delete"),
        tr("You are about to delete the selected Financial Report and all of its items.\n"
@@ -201,9 +204,9 @@ void financialLayouts::sCopy()
       }
       else
       {
-        systemError(this, tr("A System Error occurred at %1::%2.")
-                          .arg(__FILE__)
-                          .arg(__LINE__) );
+        ErrorReporter::error(QtCriticalMsg, this, tr("Copy Error"),
+                             tr("%1: An error occurred copying financial layout "
+                                "information").arg(windowTitle()),__FILE__,__LINE__);
         return;
       }
     }
