@@ -17,6 +17,7 @@
 #include <QSqlError>
 
 #include <metasql.h>
+#include "errorReporter.h"
 
 fixSerial::fixSerial(QWidget* parent, const char * name, Qt::WindowFlags fl)
     : XWidget(parent, name, fl)
@@ -71,6 +72,8 @@ void fixSerial::sFillList()
 		"    AND d.adnum = a.attnum"
 		"    AND pg_catalog.pg_get_expr(d.adbin, d.adrelid) ~* 'nextval'"
 		"    AND a.atthasdef "
+                "    AND relpersistence in ('p', 'u')"
+                "    AND pg_class.relkind != 'f'" // #27517 do not include foreign tables (relkind = 'f')
 		"ORDER BY relname;" ;
 
   XSqlQuery relq;
@@ -101,9 +104,9 @@ void fixSerial::sFillList()
     maxq = maxMql.toQuery(params);
     if (maxq.first())
       maxval = maxq.value("maxval").toInt();
-    else if (maxq.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Column Information"),
+                                  maxq, __FILE__, __LINE__))
     {
-      systemError(this, maxq.lastError().databaseText(), __FILE__, __LINE__);
       continue;
     }
 
@@ -111,9 +114,9 @@ void fixSerial::sFillList()
     seqq = seqMql.toQuery(params);
     if (seqq.first())
       currval = seqq.value("currval").toInt();
-    else if (seqq.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Column Information"),
+                                  seqq, __FILE__, __LINE__))
     {
-      systemError(this, seqq.lastError().databaseText(), __FILE__, __LINE__);
       continue;
     }
 
@@ -139,9 +142,9 @@ void fixSerial::sFillList()
   }
 
   QApplication::restoreOverrideCursor();
-  if (relq.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Column Information"),
+                                relq, __FILE__, __LINE__))
   {
-    systemError(this, relq.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -167,9 +170,9 @@ bool fixSerial::fixOne(XTreeWidgetItem *pItem)
   fixfixOne.bindValue(":value",		pItem->text(4));
 
   fixfixOne.exec();
-  if (fixfixOne.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Column Information"),
+                                fixfixOne, __FILE__, __LINE__))
   {
-    systemError(this, fixfixOne.lastError().databaseText(), __FILE__, __LINE__);
     return false;
   }
 
