@@ -1,6 +1,7 @@
 // generated 2009-04-30
 // grep "public X[Dialog,Widget]" *.h | awk '{print "CLASSITEM("$2")"}' > getscreen_classlist.h
 CLASSITEM(absoluteCalendarItem)
+CLASSITEM(account1099)
 CLASSITEM(accountingPeriod)
 CLASSITEM(accountingPeriods)
 CLASSITEM(accountingYearPeriod)
@@ -60,6 +61,7 @@ CLASSITEM(checkFormats)
 CLASSITEM(check)
 CLASSITEM(classCode)
 CLASSITEM(classCodes)
+CLASSITEM(classCodeTax)
 CLASSITEM(closePurchaseOrder)
 CLASSITEM(closeWo)
 CLASSITEM(commentType)
@@ -191,6 +193,7 @@ CLASSITEM(dspItemCostHistory)
 CLASSITEM(dspItemCostsByClassCode)
 CLASSITEM(dspItemCostSummary)
 CLASSITEM(dspItemSources)
+CLASSITEM(dspItemSourcePrices)
 CLASSITEM(dspItemsWithoutItemSources)
 CLASSITEM(dspJobCosting)
 CLASSITEM(dspMaterialUsageVarianceByBOMItem)
@@ -248,6 +251,7 @@ CLASSITEM(dspSummarizedBankrecHistory)
 CLASSITEM(dspSummarizedBOM)
 CLASSITEM(dspSummarizedGLTransactions)
 CLASSITEM(dspSummarizedSales)
+CLASSITEM(dspTax1099)
 CLASSITEM(dspTaxHistory)
 CLASSITEM(dspTimePhasedAvailability)
 CLASSITEM(dspTimePhasedBookings)
@@ -261,6 +265,7 @@ CLASSITEM(dspUndefinedManufacturedItems)
 CLASSITEM(dspUninvoicedReceivings)
 CLASSITEM(dspUnusedPurchasedItems)
 CLASSITEM(dspUsageStatistics)
+CLASSITEM(dspUserPrivileges)
 CLASSITEM(dspValidLocationsByItem)
 CLASSITEM(dspVendorAPHistory)
 CLASSITEM(dspVoucherRegister)
@@ -361,12 +366,12 @@ CLASSITEM(itemSource)
 CLASSITEM(itemSourceList)
 CLASSITEM(itemSourcePrice)
 CLASSITEM(itemSourceSearch)
-CLASSITEM(itemSources)
 CLASSITEM(itemSubstitute)
 CLASSITEM(itemtax)
 CLASSITEM(itemUOM)
 CLASSITEM(labelForm)
 CLASSITEM(labelForms)
+CLASSITEM(ledgerControl)
 CLASSITEM(listRecurringInvoices)
 CLASSITEM(locales)
 CLASSITEM(location)
@@ -418,6 +423,8 @@ CLASSITEM(postPurchaseOrdersByAgent)
 CLASSITEM(postStandardJournalGroup)
 CLASSITEM(postStandardJournal)
 CLASSITEM(postVouchers)
+CLASSITEM(poType)
+CLASSITEM(poTypes)
 CLASSITEM(prepareCheckRun)
 CLASSITEM(priceList)
 CLASSITEM(pricingScheduleAssignment)
@@ -439,6 +446,7 @@ CLASSITEM(printOptions)
 CLASSITEM(printPackingListBatchByShipvia)
 CLASSITEM(printPackingList)
 CLASSITEM(printPoForm)
+CLASSITEM(printToForm)
 CLASSITEM(printPurchaseOrder)
 CLASSITEM(printPurchaseOrdersByAgent)
 CLASSITEM(printRaForm)
@@ -457,7 +465,7 @@ CLASSITEM(profitCenter)
 CLASSITEM(profitCenters)
 CLASSITEM(project)
 CLASSITEM(projects)
-//CLASSITEM(projectTypes)
+CLASSITEM(projectTypes)
 CLASSITEM(prospect)
 CLASSITEM(prospects)
 CLASSITEM(purchaseOrder)
@@ -513,6 +521,7 @@ CLASSITEM(salesHistoryInformation)
 CLASSITEM(salesOrder)
 CLASSITEM(salesOrderInformation)
 CLASSITEM(salesOrderItem)
+CLASSITEM(salesOrderSimple)
 CLASSITEM(salesRep)
 CLASSITEM(salesReps)
 CLASSITEM(saleType)
@@ -630,7 +639,6 @@ CLASSITEM(vendors)
 CLASSITEM(vendorPriceList)
 CLASSITEM(vendorType)
 CLASSITEM(vendorTypes)
-CLASSITEM(vendorWorkBench)
 CLASSITEM(viewCheckRun)
 CLASSITEM(voidChecks)
 CLASSITEM(voucher)
