@@ -61,6 +61,7 @@
 #include "check.h"
 #include "classCode.h"
 #include "classCodes.h"
+#include "classCodeTax.h"
 #include "closePurchaseOrder.h"
 #include "closeWo.h"
 #include "commentType.h"
@@ -78,7 +79,6 @@
 #include "configurePO.h"
 #include "configureSO.h"
 #include "configureSearchPath.h"
-#include "configureWF.h"
 #include "configureWO.h"
 #include "confirmAchOK.h"
 #include "contact.h"
@@ -193,6 +193,7 @@
 #include "dspItemCostsByClassCode.h"
 #include "dspItemCostSummary.h"
 #include "dspItemSources.h"
+#include "dspItemSourcePrices.h"
 #include "dspItemsWithoutItemSources.h"
 #include "dspJobCosting.h"
 #include "dspMaterialUsageVarianceByBOMItem.h"
@@ -365,7 +366,6 @@
 #include "itemSourceList.h"
 #include "itemSourcePrice.h"
 #include "itemSourceSearch.h"
-#include "itemSources.h"
 #include "itemSubstitute.h"
 #include "itemtax.h"
 #include "itemUOM.h"
@@ -423,6 +423,8 @@
 #include "postStandardJournalGroup.h"
 #include "postStandardJournal.h"
 #include "postVouchers.h"
+#include "poType.h"
+#include "poTypes.h"
 #include "prepareCheckRun.h"
 #include "priceList.h"
 #include "pricingScheduleAssignment.h"
