@@ -13,9 +13,11 @@
 #include <math.h>
 
 #include <QMessageBox>
+#include "guiErrorCheck.h"
 #include <QSqlError>
 #include <QValidator>
 #include <QVariant>
+#include "errorReporter.h"
 
 glSeriesItem::glSeriesItem(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
@@ -115,15 +117,15 @@ void glSeriesItem::sSave()
     co.bindValue(":glseries_id", _glseriesid);
     co.bindValue(":company_id", _account->companyId());
     co.exec();
-    if (co.first())
-    {
-      QMessageBox::critical(this, tr("Can not Save Series Item"),
-                            tr("The Company of this Account does not match the "
+
+    QList<GuiErrorCheck> errors;
+    errors<< GuiErrorCheck(co.first(), _account,
+                           tr("The Company of this Account does not match the "
                                "Companies for other Accounts on this series.  This "
-                               "entry can not be saved."));
-      _account->setFocus();
+                               "entry can not be saved."))
+    ;
+    if (GuiErrorCheck::reportErrors(this, tr("Can not Save Series Item"), errors))
       return;
-    }
   }
 
   double amount = _amount->baseValue();
@@ -146,9 +148,9 @@ void glSeriesItem::sSave()
   glSave.bindValue(":amount",	amount);
   glSave.bindValue(":distdate",	_amount->effective());
   glSave.exec();
-  if (glSave.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving G/L Series Information"),
+                                glSave, __FILE__, __LINE__))
   {
-    systemError(this, glSave.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -178,9 +180,9 @@ void glSeriesItem::populate()
 
     _account->setId(glpopulate.value("glseries_accnt_id").toInt());
   }
-  else if (glpopulate.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving G/L Series Information"),
+                                glpopulate, __FILE__, __LINE__))
   {
-    systemError(this, glpopulate.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
