@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -10,10 +10,12 @@
 
 #include "group.h"
 
+#include <QMenu>
 #include <QMessageBox>
 #include <QSqlError>
 #include <QVariant>
 
+#include "user.h"
 #include "errorReporter.h"
 #include "guiErrorCheck.h"
 #include "storedProcErrorLookup.h"
@@ -37,12 +39,31 @@ group::group(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
   connect(_granted, SIGNAL(itemSelected(int)), this, SLOT(sRevoke()));
   connect(_granted, SIGNAL(valid(bool)), _revoke, SLOT(setEnabled(bool)));
   connect(_available, SIGNAL(itemSelected(int)), this, SLOT(sAdd()));
-
+  connect(_name, SIGNAL(editingFinished()), this, SLOT(sChanged()));
+  connect(_description, SIGNAL(editingFinished()), this, SLOT(sChanged()));
+  connect(_inventoryMenu, SIGNAL(clicked(bool)), this, SLOT(sChanged()));
+  connect(_productsMenu, SIGNAL(clicked(bool)), this, SLOT(sChanged()));
+  connect(_scheduleMenu, SIGNAL(clicked(bool)), this, SLOT(sChanged()));
+  connect(_manufactureMenu, SIGNAL(clicked(bool)), this, SLOT(sChanged()));
+  connect(_crmMenu2, SIGNAL(clicked(bool)), this, SLOT(sChanged()));
+  connect(_purchaseMenu, SIGNAL(clicked(bool)), this, SLOT(sChanged()));
+  connect(_salesMenu, SIGNAL(clicked(bool)), this, SLOT(sChanged()));
+  connect(_accountingMenu, SIGNAL(clicked(bool)), this, SLOT(sChanged()));
+  connect(_inventoryToolbar, SIGNAL(clicked(bool)), this, SLOT(sChanged()));
+  connect(_productsToolbar, SIGNAL(clicked(bool)), this, SLOT(sChanged()));
+  connect(_scheduleToolbar, SIGNAL(clicked(bool)), this, SLOT(sChanged()));
+  connect(_manufactureToolbar, SIGNAL(clicked(bool)), this, SLOT(sChanged()));
+  connect(_crmToolbar2, SIGNAL(clicked(bool)), this, SLOT(sChanged()));
+  connect(_purchaseToolbar, SIGNAL(clicked(bool)), this, SLOT(sChanged()));
+  connect(_salesToolbar, SIGNAL(clicked(bool)), this, SLOT(sChanged()));
+  connect(_accountingToolbar, SIGNAL(clicked(bool)), this, SLOT(sChanged()));
 
   _available->addColumn(tr("Available Privileges"), -1, Qt::AlignLeft, true, "priv_name");
   _available->addColumn(tr("Description"), -1, Qt::AlignLeft, true, "priv_descrip");
   _granted->addColumn(tr("Granted Privileges"), -1, Qt::AlignLeft, true, "priv_name");
   _granted->addColumn(tr("Description"), -1, Qt::AlignLeft, true, "priv_descrip");
+  _assigned->addColumn(tr("Username"), 100, Qt::AlignLeft, true, "usr_username");
+  _assigned->addColumn(tr("Proper Name"), -1, Qt::AlignLeft, true, "usr_propername");
 
   groupgroup.exec( "SELECT DISTINCT priv_module "
           "FROM priv "
@@ -50,7 +71,9 @@ group::group(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
   for (int i = 0; groupgroup.next(); i++)
     _module->append(i, groupgroup.value("priv_module").toString());
 
-  _trapClose = false;
+  connect(_assigned, SIGNAL(populateMenu(QMenu*,QTreeWidgetItem *)), this, SLOT(sPopulateMenu(QMenu *, QTreeWidgetItem *)));
+
+  _modified = false;
 }
 
 group::~group()
@@ -63,6 +86,15 @@ void group::languageChange()
   retranslateUi(this);
 }
 
+void group::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem *)
+{
+  QAction *menuItem;
+
+  menuItem = pMenu->addAction("Maintain User...", this, SLOT(sEditUser()));
+  if (!_privileges->check("MaintainUsers"))
+    menuItem->setEnabled(false);
+}
+
 enum SetResponse group::set(const ParameterList &pParams)
 {
   XSqlQuery groupet;
@@ -85,15 +117,14 @@ enum SetResponse group::set(const ParameterList &pParams)
       groupet.exec("SELECT NEXTVAL('grp_grp_id_seq') AS grp_id;");
       if (groupet.first())
         _grpid = groupet.value("grp_id").toInt();
-      else if (groupet.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Group Information"),
+                                    groupet, __FILE__, __LINE__))
       {
-        systemError(this, groupet.lastError().databaseText(), __FILE__, __LINE__);
         return UndefinedError;
       }
 
       _mode = cNew;
-      _trapClose = true;
-      groupet.exec("BEGIN;");
+      _modified = true;
       groupet.prepare( "INSERT INTO grp "
                  "( grp_id, grp_name, grp_descrip)"
                  "VALUES( :grp_id, :grp_id, '' );" );
@@ -101,9 +132,8 @@ enum SetResponse group::set(const ParameterList &pParams)
       groupet.exec();
       if (groupet.lastError().type() != QSqlError::NoError)
       {
-        systemError(this, groupet.lastError().databaseText(), __FILE__, __LINE__);
-        groupet.exec("ROLLBACK;");
-        _trapClose = false;
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Group Information"),
+                             groupet, __FILE__, __LINE__);
         return UndefinedError;
       }
 
@@ -114,7 +144,6 @@ enum SetResponse group::set(const ParameterList &pParams)
     {
       _mode = cEdit;
       _name->setEnabled(false);
-      groupet.exec("BEGIN;");
     }
     else if (param.toString() == "view")
     {
@@ -141,9 +170,9 @@ enum SetResponse group::set(const ParameterList &pParams)
 void group::reject()
 {
   if (DEBUG)
-    qDebug("group::reject() called with _trapClose = %d", _trapClose);
+    qDebug("group::reject() called with _modified = %d", _modified);
 
-  if(_trapClose)
+  if(_modified)
   {
     XSqlQuery endtxn;
     switch(QMessageBox::question(this, tr("Save?"),
@@ -153,12 +182,12 @@ void group::reject()
     {
       case QMessageBox::Yes:
         sSave();
-        return;
-//        endtxn.exec("COMMIT;");
-//        break;
+        if (_modified)
+          sCancel();
+        break;
 
       case QMessageBox::No:
-        endtxn.exec("ROLLBACK;");
+        sCancel();
         break;
 
       case QMessageBox::Cancel:
@@ -186,9 +215,8 @@ void group::sCheck()
     groupCheck.exec();
     if (groupCheck.first())
     {
-      XSqlQuery groupet;
-      groupet.exec("ROLLBACK;");
-      groupet.exec("BEGIN;");
+      sCancel();
+      _modified = false;
       _grpid = groupCheck.value("grp_id").toInt();
       _mode = cEdit;
       populate();
@@ -205,34 +233,78 @@ void group::sSave()
 
   QList<GuiErrorCheck> errors;
   errors << GuiErrorCheck(_name->text().trimmed().isEmpty(), _name,
-                          tr("You must enter a valid Name for this Group "
+                          tr("You must enter a valid Name for this Role "
                              "before continuing"))
      ;
 
-  if (GuiErrorCheck::reportErrors(this, tr("Cannot Save Group"), errors))
+  if (GuiErrorCheck::reportErrors(this, tr("Cannot Save Role"), errors))
     return;
 
   groupSave.prepare( "UPDATE grp "
              "   SET grp_name=:grp_name,"
-             "       grp_descrip=:grp_descrip "
+             "       grp_descrip=:grp_descrip, "
+             "       grp_showimmenu=:grp_showimmenu, "
+             "       grp_showpdmenu=:grp_showpdmenu, "
+             "       grp_showmsmenu=:grp_showmsmenu, "
+             "       grp_showwomenu=:grp_showwomenu, "
+             "       grp_showcrmmenu=:grp_showcrmmenu, "
+             "       grp_showpomenu=:grp_showpomenu, "
+             "       grp_showsomenu=:grp_showsomenu, "
+             "       grp_showglmenu=:grp_showglmenu, "
+             "       grp_showimtoolbar=:grp_showimtoolbar, "
+             "       grp_showpdtoolbar=:grp_showpdtoolbar, "
+             "       grp_showmstoolbar=:grp_showmstoolbar, "
+             "       grp_showwotoolbar=:grp_showwotoolbar, "
+             "       grp_showcrmtoolbar=:grp_showcrmtoolbar, "
+             "       grp_showpotoolbar=:grp_showpotoolbar, "
+             "       grp_showsotoolbar=:grp_showsotoolbar, "
+             "       grp_showgltoolbar=:grp_showgltoolbar  "
              " WHERE(grp_id=:grp_id);" );
   groupSave.bindValue(":grp_id", _grpid);
   groupSave.bindValue(":grp_name", _name->text());
   groupSave.bindValue(":grp_descrip", _description->text().trimmed());
+  groupSave.bindValue(":grp_showimmenu", _inventoryMenu->isChecked());
+  groupSave.bindValue(":grp_showpdmenu", _productsMenu->isChecked());
+  groupSave.bindValue(":grp_showmsmenu", _scheduleMenu->isChecked());
+  groupSave.bindValue(":grp_showwomenu", _manufactureMenu->isChecked());
+  groupSave.bindValue(":grp_showcrmmenu", _crmMenu2->isChecked());
+  groupSave.bindValue(":grp_showpomenu", _purchaseMenu->isChecked());
+  groupSave.bindValue(":grp_showsomenu", _salesMenu->isChecked());
+  groupSave.bindValue(":grp_showglmenu", _accountingMenu->isChecked());
+  groupSave.bindValue(":grp_showimtoolbar", _inventoryToolbar->isChecked());
+  groupSave.bindValue(":grp_showpdtoolbar", _productsToolbar->isChecked());
+  groupSave.bindValue(":grp_showmstoolbar", _scheduleToolbar->isChecked());
+  groupSave.bindValue(":grp_showwotoolbar", _manufactureToolbar->isChecked());
+  groupSave.bindValue(":grp_showcrmtoolbar", _crmToolbar2->isChecked());
+  groupSave.bindValue(":grp_showpotoolbar", _purchaseToolbar->isChecked());
+  groupSave.bindValue(":grp_showsotoolbar", _salesToolbar->isChecked());
+  groupSave.bindValue(":grp_showgltoolbar", _accountingToolbar->isChecked());
   groupSave.exec();
-  if (groupSave.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Group"),
+                                groupSave, __FILE__, __LINE__))
   {
-    systemError(this, groupSave.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
-  groupSave.exec("COMMIT;");
-  _trapClose = false;
+  _modified = false;
   _mode = cEdit;
 
   done(_grpid);
 }
 
+void group::sEditUser()
+{
+  ParameterList params;
+  params.append("mode", "edit");
+  params.append("username", _assigned->selectedItems().first()->text(0));
+
+  user newdlg(this);
+  newdlg.set(params);
+
+  newdlg.exec();
+  populateAssigned();
+}
+
 void group::sModuleSelected(const QString &pModule)
 {
   XSqlQuery avail;
@@ -248,9 +320,9 @@ void group::sModuleSelected(const QString &pModule)
   avail.bindValue(":grpid", _grpid);
   avail.exec();
   _available->populate(avail);
-  if (avail.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Group Information"),
+                                avail, __FILE__, __LINE__))
   {
-    systemError(this, avail.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -265,9 +337,9 @@ void group::sModuleSelected(const QString &pModule)
   grppriv.bindValue(":priv_module", _module->currentText());
   grppriv.exec();
   _granted->populate(grppriv);
-  if (grppriv.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Group Information"),
+                                grppriv, __FILE__, __LINE__))
   {
-    systemError(this, grppriv.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -284,19 +356,19 @@ void group::sAdd()
     int result = groupAdd.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("grantPrivGroup", result),
-                  __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Granting Privilege"),
+                             storedProcErrorLookup("grantPrivGroup", result),
+                             __FILE__, __LINE__);
       return;
     }
   }
-  else if (groupAdd.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Granting Privilege "),
+                                groupAdd, __FILE__, __LINE__))
   {
-    systemError(this, groupAdd.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
   sModuleSelected(_module->currentText());
-  _trapClose = true;
 }
 
 void group::sAddAll()
@@ -311,19 +383,19 @@ void group::sAddAll()
     int result = groupAddAll.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("grantAllModulePrivGroup", result),
-                  __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Granting All Privileges"),
+                             storedProcErrorLookup("grantAllModulePrivGroup", result),
+                             __FILE__, __LINE__);
       return;
     }
   }
-  else if (groupAddAll.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Granting All Privileges"),
+                                groupAddAll, __FILE__, __LINE__))
   {
-    systemError(this, groupAddAll.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
   sModuleSelected(_module->currentText());
-  _trapClose = true;
 }
 
 void group::sRevoke()
@@ -338,19 +410,19 @@ void group::sRevoke()
     int result = groupRevoke.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("revokePrivGroup", result),
-                  __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Revoking Privilege"),
+                             storedProcErrorLookup("revokePrivGroup", result),
+                             __FILE__, __LINE__);
       return;
     }
   }
-  else if (groupRevoke.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Revoking Privilege"),
+                                groupRevoke, __FILE__, __LINE__))
   {
-    systemError(this, groupRevoke.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
   sModuleSelected(_module->currentText());
-  _trapClose = true;
 }
 
 void group::sRevokeAll()
@@ -365,25 +437,31 @@ void group::sRevokeAll()
     int result = groupRevokeAll.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("revokeAllModulePrivGroup", result),
-                  __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Revoking All Privileges"),
+                             storedProcErrorLookup("revokeAllModulePrivGroup", result),
+                             __FILE__, __LINE__);
       return;
     }
   }
-  else if (groupRevokeAll.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Revoking All Privileges"),
+                                groupRevokeAll, __FILE__, __LINE__))
   {
-    systemError(this, groupRevokeAll.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
   sModuleSelected(_module->currentText());
-  _trapClose = true;
 }
 
 void group::populate()
 {
   XSqlQuery grouppopulate;
-  grouppopulate.prepare( "SELECT grp_name, grp_descrip "
+  grouppopulate.prepare( "SELECT grp_name, grp_descrip, "
+             " grp_showimmenu, grp_showpdmenu, grp_showmsmenu, "
+             " grp_showwomenu, grp_showcrmmenu, grp_showpomenu, "
+             " grp_showsomenu, grp_showglmenu, "
+             " grp_showimtoolbar, grp_showpdtoolbar, grp_showmstoolbar, "
+             " grp_showwotoolbar, grp_showcrmtoolbar, grp_showpotoolbar, "
+             " grp_showsotoolbar, grp_showgltoolbar "
              "  FROM grp "
              " WHERE(grp_id=:grp_id);" );
   grouppopulate.bindValue(":grp_id", _grpid);
@@ -392,7 +470,22 @@ void group::populate()
   {
     _name->setText(grouppopulate.value("grp_name"));
     _description->setText(grouppopulate.value("grp_descrip"));
-
+    _inventoryMenu->setChecked(grouppopulate.value("grp_showimmenu").toBool());
+    _productsMenu->setChecked(grouppopulate.value("grp_showpdmenu").toBool());
+    _scheduleMenu->setChecked(grouppopulate.value("grp_showmsmenu").toBool());
+    _manufactureMenu->setChecked(grouppopulate.value("grp_showwomenu").toBool());
+    _crmMenu2->setChecked(grouppopulate.value("grp_showcrmmenu").toBool());
+    _purchaseMenu->setChecked(grouppopulate.value("grp_showpomenu").toBool());
+    _salesMenu->setChecked(grouppopulate.value("grp_showsomenu").toBool());
+    _accountingMenu->setChecked(grouppopulate.value("grp_showglmenu").toBool());
+    _inventoryToolbar->setChecked(grouppopulate.value("grp_showimtoolbar").toBool());
+    _productsToolbar->setChecked(grouppopulate.value("grp_showpdtoolbar").toBool());
+    _scheduleToolbar->setChecked(grouppopulate.value("grp_showmstoolbar").toBool());
+    _manufactureToolbar->setChecked(grouppopulate.value("grp_showwotoolbar").toBool());
+    _crmToolbar2->setChecked(grouppopulate.value("grp_showcrmtoolbar").toBool());
+    _purchaseToolbar->setChecked(grouppopulate.value("grp_showpotoolbar").toBool());
+    _salesToolbar->setChecked(grouppopulate.value("grp_showsotoolbar").toBool());
+    _accountingToolbar->setChecked(grouppopulate.value("grp_showgltoolbar").toBool());
     grouppopulate.prepare( "SELECT priv_module "
                "FROM grppriv, priv "
                "WHERE ( (grppriv_priv_id=priv_id)"
@@ -418,9 +511,49 @@ void group::populate()
       sModuleSelected(_module->itemText(0));
     }
   }
-  else if (grouppopulate.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Role Information"),
+                                grouppopulate, __FILE__, __LINE__))
   {
-    systemError(this, grouppopulate.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
+  
+  populateAssigned();
+}
+
+void group::populateAssigned()
+{
+
+  XSqlQuery assignedpopulate;
+  assignedpopulate.prepare( "SELECT usr_username, usr_propername FROM usr "
+                            " JOIN usrgrp ON (usrgrp_username=usr_username) "
+                            " WHERE (usrgrp_grp_id=:grp_id);" );
+  assignedpopulate.bindValue(":grp_id", _grpid);
+  assignedpopulate.exec();
+  if (assignedpopulate.lastError().type() != QSqlError::NoError)
+  {
+    ErrorReporter::error(QtCriticalMsg, this, tr("User Role Assignment"),
+                         assignedpopulate.lastError(), __FILE__, __LINE__);
+    return;
+  }
+  _assigned->populate(assignedpopulate, false);
+}
+
+void group::sCancel()
+{
+  if (_mode == cNew)
+  {
+    XSqlQuery groupet;
+    groupet.prepare("DELETE FROM grp "
+                    " WHERE grp_id=:grp_id;");
+    groupet.bindValue(":grp_id", _grpid);
+    groupet.exec();
+
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error deleting group"),
+                         groupet.lastError(), __FILE__, __LINE__);
+  }
+}
+
+void group::sChanged()
+{
+  _modified = true;
 }
