@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2016 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -12,6 +12,9 @@
 #define GUIERRORCHECK_H
 
 #include <QString>
+#include <QObject>
+#include <QtScript>
+#include <QList>
 
 class QWidget;
 
@@ -19,6 +22,7 @@ class GuiErrorCheck
 {
   public:
     GuiErrorCheck(bool hasError, QWidget *widget, QString msg);
+    GuiErrorCheck();
     ~GuiErrorCheck();
 
     static bool reportErrors(QWidget *parent, QString title, QList<GuiErrorCheck> list);
@@ -28,7 +32,12 @@ class GuiErrorCheck
     bool        hasError;
     QWidget    *widget;
     QString     msg;
-
 };
 
+// Script exposure
+Q_DECLARE_METATYPE(GuiErrorCheck)
+Q_DECLARE_METATYPE(QList<GuiErrorCheck>)
+void setupGuiErrorCheck(QScriptEngine *engine);
+QScriptValue constructGuiErrorCheck(QScriptContext *context, QScriptEngine *engine);
+
 #endif // GUIERRORCHECK_H
