@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2016 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -13,14 +13,18 @@
 
 #include <QAction>
 #include <QDate>
-#include <QList>
 #include <QMainWindow>
 #include <QTimer>
 
 #include <xsqlquery.h>
 
-#include "../common/format.h"
+#include "format.h"
 #include "../hunspell/hunspell.hxx"
+#include "mqlhash.h"
+
+#include <xtuplecommon.h>
+#include <version.h>
+#include <data.h>
 
 class QCheckBox;
 class QCloseEvent;
@@ -108,7 +112,7 @@ extern Metricsenc  *_metricsenc;
 enum SetResponse
 {
   NoError, NoError_Cancel, NoError_Run, NoError_Print, NoError_Submit,
-  Error_NoSetup, UndefinedError
+  Error_NoSetup, Error_AlreadyOpen, UndefinedError
 };
 
 
@@ -175,11 +179,16 @@ class GUIClient : public QMainWindow
     Q_INVOKABLE void setCaption();
     Q_INVOKABLE void saveToolbarPositions();
 
-    Q_INVOKABLE inline QMdiArea *workspace()         { return _workspace;    }
+    Q_INVOKABLE inline QMdiArea *workspace()           { return _workspace;    }
     Q_INVOKABLE inline InputManager *inputManager()    { return _inputManager; }
     Q_INVOKABLE inline QString databaseURL()           { return _databaseURL;  }
     Q_INVOKABLE inline QString username()              { return _username;     }
 
+    // runtime versions
+    Q_INVOKABLE inline QString version()               { return _Version;         }
+    Q_INVOKABLE inline QString qtVersion()             { return QT_VERSION_STR;   }
+    Q_INVOKABLE inline QString openrptVersion()        { return OpenRPT::version; }
+
     Q_INVOKABLE inline const QDate startOfTime()       { return _startOfTime;  }
     Q_INVOKABLE inline const QDate endOfTime()         { return _endOfTime;    }
     Q_INVOKABLE inline const QDate dbDate()            { return _dbDate;       }
@@ -220,7 +229,7 @@ class GUIClient : public QMainWindow
     Q_INVOKABLE void tabifyDockWidget ( QDockWidget * first, QDockWidget * second );
     Q_INVOKABLE void setCentralWidget(QWidget * widget);
 
-	TimeoutHandler   *_timeoutHandler;
+    TimeoutHandler   *_timeoutHandler;
     ReportHandler    *_reportHandler;
 
     QMap<const QObject*,int> _customCommands;
@@ -228,19 +237,20 @@ class GUIClient : public QMainWindow
     QString _key;
     Q_INVOKABLE QString key() { return _key; }
 
+    MqlHash *_mqlhash;
     QString _singleWindow;
 
     Q_INVOKABLE        void  launchBrowser(QWidget*, const QString &);
-    Q_INVOKABLE     QWidget *myActiveWindow();
     Q_INVOKABLE inline bool  shuttingDown() { return _shuttingDown; }
 
     void loadScriptGlobals(QScriptEngine * engine);
 
-    #ifdef Q_OS_MAC
-    		void updateMacDockMenu(QWidget *w);
-    		void removeFromMacDockMenu(QWidget *w);
-    	#endif
+#ifdef Q_OS_MAC
+    void updateMacDockMenu(QWidget *w);
+    void removeFromMacDockMenu(QWidget *w);
+#endif
 
+    Q_INVOKABLE void hunspell_initialize();
     //check hunspell is ready
     Q_INVOKABLE bool hunspell_ready();
     //spellcheck word, returns 1 if word ok otherwise 0
@@ -310,8 +320,6 @@ class GUIClient : public QMainWindow
 
     void sIdleTimeout();
 
-    void sFocusChanged(QWidget* old, QWidget* now);
-
     void sClearErrorMessages();
     void sNewErrorMessage();
     void setWindowTitle();
@@ -320,6 +328,7 @@ class GUIClient : public QMainWindow
     void tick();
 
     void messageNotify();
+    void dbConnectionLost();
 
     /** @name Data Update Signals
      
@@ -398,7 +407,6 @@ class GUIClient : public QMainWindow
 
   private slots:
     void handleDocument(QString path);
-    void hunspell_initialize();
     void hunspell_uninitialize();
 
   private:
@@ -412,7 +420,6 @@ class GUIClient : public QMainWindow
     bool         _showTopLevel;
     QWidgetList  _windowList;
     QMenuBar	*_menuBar;
-    QWidget     *_activeWindow;
 
     InputManager   *_inputManager;
 
@@ -453,11 +460,10 @@ class GUIClient : public QMainWindow
     bool _shown;
     bool _shuttingDown;
 
-    QFileSystemWatcher* _fileWatcher;
-    QMap<QString, int> _fileMap;
-    QTextCodec * _spellCodec;
-    Hunspell * _spellChecker;
-    bool _spellReady;
+    QFileSystemWatcher *_fileWatcher;
+    QMap<QString, int>  _fileMap;
+    QTextCodec *_spellCodec;
+    Hunspell   *_spellChecker;
     QStringList _spellAddWords;
 
     QMenu *_menu;
