@@ -19,6 +19,7 @@
 #include <openreports.h>
 
 #include "honorific.h"
+#include "errorReporter.h"
 
 /*
    honorifics is internal name, titles is external name.  names
@@ -75,9 +76,9 @@ void honorifics::sFillList()
              "ORDER BY hnfc_code;" );
   honorificsFillList.exec();
   _honorifics->populate(honorificsFillList);
-  if (honorificsFillList.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Title Information"),
+                                honorificsFillList, __FILE__, __LINE__))
   {
-    systemError(this, honorificsFillList.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -89,9 +90,9 @@ void honorifics::sDelete()
              "WHERE (hnfc_id=:hnfc_id);" );
   honorificsDelete.bindValue(":hnfc_id", _honorifics->id());
   honorificsDelete.exec();
-  if (honorificsDelete.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Title"),
+                                honorificsDelete, __FILE__, __LINE__))
   {
-    systemError(this, honorificsDelete.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
