@@ -20,10 +20,41 @@
 #include "errorReporter.h"
 #include "returnAuthorization.h"
 #include "storedProcErrorLookup.h"
+#include "incidentHistory.h"
 #include "todoItem.h"
+#include "guiErrorCheck.h"
 
 #include <openreports.h>
 
+bool incident::userHasPriv(const int pMode, const int pId)
+{
+  if (_privileges->check("MaintainAllIncidents"))
+    return true;
+  bool personalPriv = _privileges->check("MaintainPersonalIncidents");
+  if(pMode==cView)
+  {
+    if(_privileges->check("ViewAllIncidents"))
+      return true;
+    personalPriv = personalPriv || _privileges->check("ViewPersonalIncidents");
+  }
+
+  if(pMode==cNew)
+    return personalPriv;
+  else
+  {
+    XSqlQuery usernameCheck;
+    usernameCheck.prepare( "SELECT getEffectiveXtUser() IN (incdt_owner_username, incdt_assigned_username) AS canModify "
+                           "FROM incdt "
+                            "WHERE (incdt_id=:incdt_id);" );
+    usernameCheck.bindValue(":incdt_id", pId);
+    usernameCheck.exec();
+
+    if (usernameCheck.first())
+      return usernameCheck.value("canModify").toBool()&&personalPriv;
+    return false;
+  }
+}
+
 incident::incident(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
 {
@@ -33,6 +64,7 @@ incident::incident(QWidget* parent, const char* name, bool modal, Qt::WindowFlag
   _incdtid = -1;
   _saved = false;
   _aropenid = -1;
+  _close = false;
 
   _print = _buttonBox->addButton(tr("Print"),QDialogButtonBox::ActionRole);
   _print->setObjectName("_print");
@@ -46,6 +78,7 @@ incident::incident(QWidget* parent, const char* name, bool modal, Qt::WindowFlag
   connect(_deleteTodoItem, SIGNAL(clicked()),       this,       SLOT(sDeleteTodoItem()));
   connect(_editTodoItem,  SIGNAL(clicked()),        this,       SLOT(sEditTodoItem()));
   connect(_item,          SIGNAL(newId(int)),     _lotserial,   SLOT(setItemId(int)));
+  connect(_incdthist,     SIGNAL(itemSelected(int)), this,      SLOT(sIncidentHistory()));
   connect(_newTodoItem,   SIGNAL(clicked()),        this,       SLOT(sNewTodoItem()));
   connect(_buttonBox,     SIGNAL(accepted()),        this,       SLOT(sSave()));
   connect(_print,         SIGNAL(clicked()),        this,       SLOT(sPrint()));
@@ -111,15 +144,6 @@ enum SetResponse incident::set(const ParameterList &pParams)
   QVariant param;
   bool     valid;
 
-  param = pParams.value("incdt_id", &valid);
-  if (valid)
-  {
-    _incdtid = param.toInt();
-    populate();
-    _lotserial->setItemId(_item->id());
-    _charass->setId(_incdtid);
-  }
-
   param = pParams.value("mode", &valid);
   if (valid)
   {
@@ -139,6 +163,7 @@ enum SetResponse incident::set(const ParameterList &pParams)
         _alarms->setId(_incdtid);
         _recurring->setParent(_incdtid, "INCDT");
         _print->hide();
+        _project->setAllowedStatuses(ProjectLineEdit::Concept |  ProjectLineEdit::InProcess);
       }
       else
       {
@@ -147,6 +172,11 @@ enum SetResponse incident::set(const ParameterList &pParams)
                                    "\n%1" ).arg(incidentet.lastError().text()));
         reject();
       }
+      // Characteristics update incident history so we have to save the incident first
+      // when adding a characteristic in new mode otherwise we get foreign key errors
+      QPushButton *newbutton = _charass->findChild<QPushButton*>("_newCharacteristic");
+      disconnect(newbutton, SIGNAL(clicked()), _charass, SLOT(sNew()));
+      connect(newbutton, SIGNAL(clicked()), this, SLOT(sNewCharacteristic()));
     }
     else if (param.toString() == "edit")
     {
@@ -155,32 +185,16 @@ enum SetResponse incident::set(const ParameterList &pParams)
       _crmacct->setEnabled(true);
     }
     else if (param.toString() == "view")
-    {
-      _mode = cView;
+      setViewMode();
+  }
 
-      _crmacct->setEnabled(false);
-      _cntct->setEnabled(false);
-      _assignedTo->setEnabled(false);
-      _category->setEnabled(false);
-      _status->setEnabled(false);
-      _resolution->setEnabled(false);
-      _severity->setEnabled(false);
-      _priority->setEnabled(false);
-      _item->setReadOnly(true);
-      _lotserial->setEnabled(false);
-      _description->setEnabled(false);
-      _notes->setEnabled(false);
-      _deleteTodoItem->setEnabled(false);
-      _editTodoItem->setEnabled(false);
-      _newTodoItem->setEnabled(false);
-      _charass->setReadOnly(true);
-      _owner->setEnabled(false);
-
-      _buttonBox->setStandardButtons(QDialogButtonBox::Close);
-      _comments->setReadOnly(true);
-      _documents->setReadOnly(true);
-      _alarms->setReadOnly(true);
-    }
+  param = pParams.value("incdt_id", &valid);
+  if (valid)
+  {
+    _incdtid = param.toInt();
+    populate();
+    _lotserial->setItemId(_item->id());
+    _charass->setId(_incdtid);
   }
 
   param = pParams.value("crmacct_id", &valid);
@@ -235,6 +249,34 @@ enum SetResponse incident::set(const ParameterList &pParams)
   return NoError;
 }
 
+void incident::setViewMode()
+{
+  _mode = cView;
+
+  _crmacct->setEnabled(false);
+  _cntct->setEnabled(false);
+  _assignedTo->setEnabled(false);
+  _category->setEnabled(false);
+  _status->setEnabled(false);
+  _resolution->setEnabled(false);
+  _severity->setEnabled(false);
+  _priority->setEnabled(false);
+  _item->setReadOnly(true);
+  _lotserial->setEnabled(false);
+  _description->setEnabled(false);
+  _notes->setEnabled(false);
+  _deleteTodoItem->setEnabled(false);
+  _editTodoItem->setEnabled(false);
+  _newTodoItem->setEnabled(false);
+  _charass->setReadOnly(true);
+  _owner->setEnabled(false);
+
+  _buttonBox->setStandardButtons(QDialogButtonBox::Close);
+  _comments->setReadOnly(true);
+  _documents->setReadOnly(true);
+  _alarms->setReadOnly(true);
+}
+
 int incident::id() const
 {
   return _incdtid;
@@ -268,14 +310,15 @@ void incident::sCancel()
       int result = incidentCancel.value("result").toInt();
       if (result < 0)
       {
-        systemError(this, storedProcErrorLookup("releaseNumber", result),
-                    __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Cancelling Incident"),
+                               storedProcErrorLookup("releaseNumber", result),
+                               __FILE__, __LINE__);
         return;
       }
     }
-    else if (incidentCancel.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Cancelling Incident"),
+                                  incidentCancel, __FILE__, __LINE__))
     {
-      systemError(this, incidentCancel.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
 
@@ -287,14 +330,15 @@ void incident::sCancel()
       int result = incidentCancel.value("result").toInt();
       if (result < 0)
       {
-        systemError(this, storedProcErrorLookup("deleteIncident", result),
-                    __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Cancelling Incident"),
+                               storedProcErrorLookup("deleteIncident", result),
+                               __FILE__, __LINE__);
         return;
       }
     }
-    else if (incidentCancel.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Cancelling Incident"),
+                                  incidentCancel, __FILE__, __LINE__))
     {
-      systemError(this, incidentCancel.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -316,35 +360,18 @@ bool incident::save(bool partial)
   XSqlQuery incidentave;
   if (! partial)
   {
-    if(_crmacct->id() == -1)
-    {
-      QMessageBox::critical( this, tr("Incomplete Information"),
-        tr("You must specify the Account that this incident is for.") );
+    QList<GuiErrorCheck> errors;
+    errors<< GuiErrorCheck(_crmacct->id() == -1, _number,
+                           tr("You must specify the Account that this incident is for."))
+          << GuiErrorCheck(_cntct->id() <= 0 && _cntct->name().simplified().isEmpty(), _cntct,
+                           tr("You must specify a Contact for this Incident."))
+          << GuiErrorCheck(_description->text().trimmed().isEmpty(), _description,
+                           tr("You must specify a description for this incident report."))
+          << GuiErrorCheck(_status->currentIndex() == 3 && _assignedTo->username().isEmpty(), _assignedTo,
+                           tr("You must specify an assignee when the status is assigned."))
+    ;
+    if (GuiErrorCheck::reportErrors(this, tr("Cannot Save Incident"), errors))
       return false;
-    }
-
-    if(_cntct->id() <= 0 && _cntct->name().simplified().isEmpty())
-    {
-      QMessageBox::critical( this, tr("Incomplete Information"),
-        tr("You must specify a Contact for this Incident.") );
-      return false;
-    }
-
-    if(_description->text().trimmed().isEmpty())
-    {
-      QMessageBox::critical( this, tr("Incomplete Information"),
-        tr("You must specify a description for this incident report.") );
-      _description->setFocus();
-      return false;
-    }
-
-    if (_status->currentIndex() == 3 && _assignedTo->username().isEmpty())
-    {
-      QMessageBox::critical( this, tr("Incomplete Information"),
-        tr("You must specify an assignee when the status is assigned.") );
-      _description->setFocus();
-      return false;
-    }
   }
 
   RecurrenceWidget::RecurrenceChangePolicy cp = _recurring->getChangePolicy();
@@ -356,7 +383,8 @@ bool incident::save(bool partial)
 
   if (!incidentave.exec("BEGIN"))
   {
-    systemError(this, incidentave.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Incident"),
+                         incidentave, __FILE__, __LINE__);
     return false;
   }
 
@@ -432,7 +460,8 @@ bool incident::save(bool partial)
   if(!incidentave.exec() && incidentave.lastError().type() != QSqlError::NoError)
   {
     rollback.exec();
-    systemError(this, incidentave.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Incident"),
+                         incidentave, __FILE__, __LINE__);
     return false;
   }
 
@@ -440,7 +469,8 @@ bool incident::save(bool partial)
   if (! _recurring->save(true, cp, &errmsg))
   {
     rollback.exec();
-    systemError(this, errmsg, __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Incident"),
+                         rollback, __FILE__, __LINE__);
     return false;
   }
 
@@ -448,7 +478,8 @@ bool incident::save(bool partial)
   if(incidentave.lastError().type() != QSqlError::NoError)
   {
     rollback.exec();
-    systemError(this, incidentave.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Incident"),
+                         incidentave, __FILE__, __LINE__);
     return false;
   }
 
@@ -475,15 +506,52 @@ void incident::sFillHistoryList()
   incidentFillHistoryList.bindValue(":contact", tr("Contact"));
   incidentFillHistoryList.exec();
   _incdthist->populate(incidentFillHistoryList);
-  if (incidentFillHistoryList.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Incident History"),
+                                incidentFillHistoryList, __FILE__, __LINE__))
   {
-    systemError(this, incidentFillHistoryList.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
 
 void incident::populate()
 {
+  if (_mode == cEdit && !_lock.acquire("incdt", _incdtid, AppLock::Interactive))
+    setViewMode();
+
+  _close = false;
+
+  foreach (QWidget* widget, QApplication::allWidgets())
+  {
+    if (!widget->isWindow() || !widget->isVisible())
+      continue;
+
+    incident *w = qobject_cast<incident*>(widget);
+
+    if (w && w != this && w->id()==_incdtid)
+    {
+      // detect "i'm my own grandpa"
+      QObject *p;
+      for (p = parent(); p && p != w ; p = p->parent())
+        ; // do nothing
+      if (p == w)
+      {
+        QMessageBox::warning(this, tr("Cannot Open Recursively"),
+                             tr("This incident is already open and cannot be "
+                                "raised. Please close windows to get to it."));
+        _close = true;
+      } else if (p) {
+        w->setFocus();
+        if (omfgThis->showTopLevel())
+        {
+          w->raise();
+          w->activateWindow();
+        }
+        _close = true;
+      }
+      break;
+    }
+  }
+
   XSqlQuery incidentpopulate;
   incidentpopulate.prepare("SELECT incdt_number,"
             "       incdt_crmacct_id,"
@@ -577,6 +645,13 @@ void incident::sCRMAcctChanged(const int newid)
   _cntct->setSearchAcct(newid);
 }
 
+void incident::sNewCharacteristic()
+{
+  if (! save(false))
+    return;
+  _charass->sNew();
+}
+
 void incident::sNewTodoItem()
 {
   if (! save(true))
@@ -616,6 +691,16 @@ void incident::sViewTodoItem()
   newdlg.exec();
 }
 
+void incident::sIncidentHistory()
+{
+  ParameterList params;
+  params.append("incdthist_id", _incdthist->id());
+
+  incidentHistory newdlg(this, 0, true);
+  newdlg.set(params);
+  newdlg.exec();
+}
+
 void incident::sDeleteTodoItem()
 {
   XSqlQuery incidentDeleteTodoItem;
@@ -627,15 +712,17 @@ void incident::sDeleteTodoItem()
     int result = incidentDeleteTodoItem.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("deleteTodoItem", result));
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting To-Do Item"),
+                             storedProcErrorLookup("deleteTodoItem", result),
+                             __FILE__, __LINE__);
       return;
     }
     else
       sFillTodoList();
     }
-  else if (incidentDeleteTodoItem.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting To-Do Item"),
+                                incidentDeleteTodoItem, __FILE__, __LINE__))
   {
-    systemError(this, incidentDeleteTodoItem.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -805,3 +892,20 @@ void incident::sAssigned()
   if (_status->currentIndex() < 3 && !_assignedTo->username().isEmpty())
     _status->setCurrentIndex(3);
 }
+
+void incident::setVisible(bool visible)
+{
+  if (_close)
+    close();
+  else
+    XDialog::setVisible(visible);
+}
+
+void incident::done(int result)
+{
+  if (!_lock.release())
+    ErrorReporter::error(QtCriticalMsg, this, tr("Locking Error"),
+                         _lock.lastError(), __FILE__, __LINE__);
+
+  XDialog::done(result);
+}
