@@ -19,6 +19,7 @@
 #include <openreports.h>
 
 #include "incidentPriority.h"
+#include "errorReporter.h"
 
 incidentPriorities::incidentPriorities(QWidget* parent, const char* name, Qt::WindowFlags fl)
     : XWidget(parent, name, fl)
@@ -71,9 +72,9 @@ void incidentPriorities::sFillList()
              "FROM incdtpriority "
              "ORDER BY incdtpriority_order, incdtpriority_name;" );
   incidentFillList.exec();
-  if (incidentFillList.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Incident Priority Information"),
+                                incidentFillList, __FILE__, __LINE__))
   {
-    systemError(this, incidentFillList.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   _incidentPriorities->populate(incidentFillList);
@@ -86,9 +87,9 @@ void incidentPriorities::sDelete()
              "WHERE (incdtpriority_id=:incdtpriorityid);" );
   incidentDelete.bindValue(":incdtpriorityid", _incidentPriorities->id());
   incidentDelete.exec();
-  if (incidentDelete.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Incident Priority"),
+                                incidentDelete, __FILE__, __LINE__))
   {
-    systemError(this, incidentDelete.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
