@@ -21,7 +21,7 @@ class incidentSeverities : public XWidget, public Ui::incidentSeverities
     Q_OBJECT
 
 public:
-    incidentSeverities(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = Qt::Window);
+    incidentSeverities(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::Window);
     ~incidentSeverities();
 
 public slots:
