@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -15,14 +15,18 @@
 #include <QMessageBox>
 #include <QSqlError>
 #include <QVariant>
-
 #include <QDebug>
 
+#include <metasql.h>
+
+#include "mqlutil.h"
+#include "errorReporter.h"
 #include "distributeInventory.h"
 #include "invoiceItem.h"
 #include "storedProcErrorLookup.h"
 #include "taxBreakdown.h"
 #include "allocateARCreditMemo.h"
+#include "guiErrorCheck.h"
 
 #define cViewQuote (0x20 | cView)
 
@@ -102,6 +106,7 @@ invoice::invoice(QWidget* parent, const char* name, Qt::WindowFlags fl)
   _invcitem->addColumn(tr("Margin %"),      _prcntColumn,    Qt::AlignRight,  false, "marginpercent");
 
   _charass->setType("INV");
+  _comments->setType(Comments::Invoice);
   
   _custCurrency->setLabel(_custCurrencyLit);
 
@@ -156,15 +161,16 @@ enum SetResponse invoice::set(const ParameterList &pParams)
         _recurring->setParent(_invcheadid, "I");
         _documents->setId(_invcheadid);
         _charass->setId(_invcheadid);
+        _comments->setId(_invcheadid);
       }
-      else if (invoiceet.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Invoice Information"),
+                                    invoiceet, __FILE__, __LINE__))
       {
-	    systemError(this, invoiceet.lastError().databaseText(), __FILE__, __LINE__);
-	    return UndefinedError;
+        return UndefinedError;
       }
 
       if ((_metrics->value("InvcNumberGeneration") == "A") ||
-          (_metrics->value("InvcNumberGeneration") == "O"))
+         (_metrics->value("InvcNumberGeneration") == "O"))
       {
         invoiceet.exec("SELECT fetchInvcNumber() AS number;");
         if (invoiceet.first())
@@ -174,9 +180,9 @@ enum SetResponse invoice::set(const ParameterList &pParams)
           if (_metrics->value("InvcNumberGeneration") == "A")
             _invoiceNumber->setEnabled(false);
         }
-        else if (invoiceet.lastError().type() != QSqlError::NoError)
+        else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Invoice Information"),
+                                      invoiceet, __FILE__, __LINE__))
         {
-          systemError(this, invoiceet.lastError().databaseText(), __FILE__, __LINE__);
           return UndefinedError;
         }
       }
@@ -205,10 +211,10 @@ enum SetResponse invoice::set(const ParameterList &pParams)
       invoiceet.bindValue(":invchead_orderdate", _orderDate->date());
       invoiceet.bindValue(":invchead_invcdate",	 _invoiceDate->date());
       invoiceet.exec();
-      if (invoiceet.lastError().type() != QSqlError::NoError)
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Invoice Information"),
+                                    invoiceet, __FILE__, __LINE__))
       {
-	    systemError(this, invoiceet.lastError().databaseText(), __FILE__, __LINE__);
-	    return UndefinedError;
+        return UndefinedError;
       }
 
       connect(_cust,	    SIGNAL(valid(bool)), _new, SLOT(setEnabled(bool)));
@@ -223,6 +229,7 @@ enum SetResponse invoice::set(const ParameterList &pParams)
       {
         _invcheadid = param.toInt();
         _charass->setId(_invcheadid);
+        _comments->setId(_invcheadid);
       }
 
       setObjectName(QString("invoice edit %1").arg(_invcheadid));
@@ -273,6 +280,7 @@ enum SetResponse invoice::set(const ParameterList &pParams)
 //      _documents->setReadOnly(true);
       _charass->setReadOnly(true);
       _postInvoice->setVisible(false);
+      _comments->setReadOnly(true);
 
       disconnect(_invcitem, SIGNAL(valid(bool)), _edit, SLOT(setEnabled(bool)));
       disconnect(_invcitem, SIGNAL(valid(bool)), _delete, SLOT(setEnabled(bool)));
@@ -291,6 +299,7 @@ enum SetResponse invoice::set(const ParameterList &pParams)
     _invcheadid = param.toInt();
     _documents->setId(_invcheadid);
     _charass->setId(_invcheadid);
+    _comments->setId(_invcheadid);
     populate();
     populateCMInfo();
     populateCCInfo();
@@ -328,7 +337,8 @@ void invoice::sClose()
       invoiceClose.bindValue(":invchead_id", _invcheadid);
       invoiceClose.exec();
       if (invoiceClose.lastError().type() != QSqlError::NoError)
-        systemError(this, invoiceClose.lastError().databaseText(), __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Invoice"),
+                           invoiceClose, __FILE__, __LINE__);
       if (invoiceClose.first() && invoiceClose.value("invchead_posted").toBool())
       {
         QMessageBox::warning( this, tr("Cannot delete Invoice"),
@@ -344,14 +354,14 @@ void invoice::sClose()
         int result = invoiceClose.value("result").toInt();
         if (result < 0)
         {
-          systemError(this, storedProcErrorLookup("deleteInvoice", result),
-                      __FILE__, __LINE__);
+          ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Invoice"),
+                                 storedProcErrorLookup("deleteInvoice", result),
+                                 __FILE__, __LINE__);
         }
       }
       else if (invoiceClose.lastError().type() != QSqlError::NoError)
-        systemError(this,
-                    tr("Error deleting Invoice %1\n").arg(_invcheadid) +
-                       invoiceClose.lastError().databaseText(), __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Invoice"),
+                           invoiceClose, __FILE__, __LINE__);
     }
   }
   else if (_mode == cEdit)
@@ -376,14 +386,14 @@ void invoice::sClose()
           int result = invoiceClose.value("result").toInt();
           if (result < 0)
           {
-            systemError(this, storedProcErrorLookup("deleteInvoice", result),
-                        __FILE__, __LINE__);
+            ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Invoice"),
+                                   storedProcErrorLookup("deleteInvoice", result),
+                                   __FILE__, __LINE__);
           }
         }
         else if (invoiceClose.lastError().type() != QSqlError::NoError)
-          systemError(this,
-                      tr("Error deleting Invoice %1\n").arg(_invcheadid) +
-                         invoiceClose.lastError().databaseText(), __FILE__, __LINE__);
+          ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Invoice"),
+                             invoiceClose, __FILE__, __LINE__);
       }
     }
   }
@@ -398,7 +408,7 @@ void invoice::sPopulateCustomerInfo(int pCustid)
     XSqlQuery cust;
     cust.prepare( "SELECT cust_name, COALESCE(cntct_addr_id,-1) AS addr_id, "
                   "       cust_salesrep_id, cust_commprcnt * 100 AS commission,"
-                  "       cust_creditstatus, cust_terms_id, "
+                  "       cust_creditstatus, cust_terms_id, cust_shipvia,"
                   "       COALESCE(cust_taxzone_id, -1) AS cust_taxzone_id,"
                   "       COALESCE(cust_shipchrg_id, -1) AS cust_shipchrg_id,"
                   "       cust_ffshipto, cust_ffbillto, "
@@ -422,6 +432,8 @@ void invoice::sPopulateCustomerInfo(int pCustid)
 	_taxzone->setId(cust.value("cust_taxzone_id").toInt());
 	_custCurrency->setId(cust.value("cust_curr_id").toInt());
         _shipChrgs->setId(cust.value("cust_shipchrg_id").toInt());
+        _shipVia->setText(cust.value("cust_shipvia").toString());
+        _custShipVia = _shipVia->currentText();
 
 	bool ffBillTo = cust.value("cust_ffbillto").toBool();
         if (_mode != cView)
@@ -442,10 +454,10 @@ void invoice::sPopulateCustomerInfo(int pCustid)
 	  _shipToPhone->clear();
 	}
       }
-      if (cust.lastError().type() != QSqlError::NoError)
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Customer Information"),
+                                    cust, __FILE__, __LINE__))
       {
-	systemError(this, cust.lastError().databaseText(), __FILE__, __LINE__);
-	return;
+        return;
       }
   }
   else
@@ -465,13 +477,15 @@ void invoice::populateShipto(int pShiptoid)
   {
     XSqlQuery shipto;
     shipto.prepare( "SELECT shipto_id, shipto_num, shipto_name, shipto_addr_id, "
-                    "       cntct_phone, shipto_shipvia, shipto_salesrep_id, "
+                    "       cntct_phone, shipto_salesrep_id, "
+                    "       COALESCE(shipto_shipvia, cust_shipvia, '') AS shipvia, "
                     "       COALESCE(shipto_taxzone_id, -1) AS shipto_taxzone_id,"
                     "       COALESCE(shipto_shipchrg_id, -1) AS shipto_shipchrg_id,"
                     "       COALESCE(shipto_shipzone_id, -1) AS shipto_shipzone_id,"
                     "       shipto_commission * 100 AS commission "
-                    "FROM shiptoinfo LEFT OUTER JOIN "
-		    "     cntct ON (shipto_cntct_id=cntct_id)"
+                    "FROM shiptoinfo "
+                    "LEFT OUTER JOIN cntct ON (shipto_cntct_id=cntct_id) "
+                    "JOIN custinfo ON (shipto_cust_id=cust_id) "
                     "WHERE (shipto_id=:shipto_id);" );
     shipto.bindValue(":shipto_id", pShiptoid);
     shipto.exec();
@@ -494,14 +508,14 @@ void invoice::populateShipto(int pShiptoid)
 
       _salesrep->setId(shipto.value("shipto_salesrep_id").toInt());
       _commission->setDouble(shipto.value("commission").toDouble());
-      _shipVia->setText(shipto.value("shipto_shipvia"));
+      _shipVia->setText(shipto.value("shipvia"));
       _taxzone->setId(shipto.value("shipto_taxzone_id").toInt());
       _shipChrgs->setId(shipto.value("shipto_shipchrg_id").toInt());
       _shippingZone->setId(shipto.value("shipto_shipzone_id").toInt());
     }
-    else if (shipto.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Customer Ship-To Information"),
+                                  shipto, __FILE__, __LINE__))
     {
-      systemError(this, shipto.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -525,41 +539,24 @@ void invoice::sCopyToShipto()
   _shipToAddr->setId(_billToAddr->id());
   _shipToPhone->setText(_billToPhone->text());
   _taxzone->setId(_custtaxzoneid);
+  _shipVia->setText(_custShipVia);
+
   _shipTo->blockSignals(false);
   _shipToAddr->blockSignals(false);
 }
 
 void invoice::sSave()
 {
-  struct {
-    bool	condition;
-    QString	msg;
-    QWidget*	widget;
-  } error[] = {
-    { _cust->id() <= 0,
-      tr("<p>You must enter a Customer for this Invoice before saving it."),
-      _cust
-    },
-    // TODO: add more error checks here?
-    { true, "", NULL }
-  };
-
-  if (_total->localValue() < 0 )
-  {
-    QMessageBox::information(this, tr("Total Less than Zero"),
-                             tr("<p>The Total must be a positive value.") );
-    _cust->setFocus();
-    return;
-  }
-
-  //Invoices must have atleast one line item.
-  if (_invcitem->topLevelItemCount() <= 0 )
-  {
-    QMessageBox::information(this, tr("No Line Items"),
-                             tr("<p>There must be at least one line item for an invoice.") );
-    _new->setFocus();
-    return;
-  }
+  QList<GuiErrorCheck> errors;
+    errors<< GuiErrorCheck(_cust->id() <= 0, _cust,
+                           tr("You must enter a Customer for this Invoice before saving it."))
+          << GuiErrorCheck(_total->localValue() < 0, _cust,
+                           tr("The Total must be a positive value."))
+          << GuiErrorCheck(_invcitem->topLevelItemCount() <= 0, _new,
+                           tr("There must be at least one line item for an invoice."))
+    ;
+    if (GuiErrorCheck::reportErrors(this, tr("Cannot Save Invoice"), errors))
+      return;
 
   //  We can't post a Misc. Charge without a Sales Account
   if ( (! _miscAmount->isZero()) && (!_miscChargeAccount->isValid()) )
@@ -573,18 +570,6 @@ void invoice::sSave()
     _miscChargeAccount->setFocus();
     return;
   }
-
-
-  int errIndex;
-  for (errIndex = 0; ! error[errIndex].condition; errIndex++)
-    ;
-  if (! error[errIndex].msg.isEmpty())
-  {
-    QMessageBox::critical(this, tr("Cannot Save Invoice"), error[errIndex].msg);
-    error[errIndex].widget->setFocus();
-    return;
-  }
-
   // save address info in case someone wants to use 'em again later
   // but don't make any global changes to the data and ignore errors
   _shipToAddr->blockSignals(true);
@@ -595,8 +580,46 @@ void invoice::sSave()
   if (!save())
     return;
 
+  // If this is a recurring invoice check whether the newly saved invoice
+  // should supercede the original recurring invoice settings
+  if (_recurring->isRecurring() && _mode == cEdit && _recurring->parentId() != _invcheadid)
+  {
+    if (QMessageBox::question(this, tr("Recurring Invoice"),
+                                    tr("You have edited a recurring Invoice.\n"
+                                    "Do you wish to change all future invoice recurrences?"),
+                              QMessageBox::Yes, QMessageBox::No | QMessageBox::Default)
+             == QMessageBox::Yes)
+    {
+    // Update Recurring Invoice with latest saved detail
+      XSqlQuery recurUpd;
+      recurUpd.prepare("UPDATE recur SET recur_parent_id=:newinvoice "
+                       "WHERE ((recur_parent_type = 'I') "
+                       " AND   (recur_parent_id = :oldinvoice));");
+      recurUpd.bindValue(":newinvoice", _invcheadid);
+      recurUpd.bindValue(":oldinvoice", _recurring->parentId());
+      recurUpd.exec();
+      if (recurUpd.lastError().type() != QSqlError::NoError)
+      {
+         ErrorReporter::error(QtCriticalMsg, this, tr("Error Updating Invoice Recurrence"),
+                     recurUpd, __FILE__, __LINE__);
+           return;
+      }
+      // Also update this invoice
+      recurUpd.prepare("UPDATE invchead SET invchead_recurring_invchead_id=:invoice "
+                       "WHERE (invchead_id = :invoice); ");
+      recurUpd.bindValue(":invoice",  _invcheadid);
+      recurUpd.exec();
+      if (recurUpd.lastError().type() != QSqlError::NoError)
+      {
+         ErrorReporter::error(QtCriticalMsg, this, tr("Error Updating Invoice Recurrence"),
+                     recurUpd, __FILE__, __LINE__);
+           return;
+      }
+    }
+  }
+
   // post the Invoice if user desires
-  if (_postInvoice->isChecked())
+  if (_postInvoice->isVisible() && _postInvoice->isChecked())
     postInvoice();
 
   omfgThis->sInvoicesUpdated(_invcheadid, true);
@@ -713,7 +736,8 @@ bool invoice::save()
   if (invoiceave.lastError().type() != QSqlError::NoError)
   {
     rollbackq.exec();
-    systemError(this, invoiceave.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Invoice Information"),
+                         invoiceave, __FILE__, __LINE__);
     return false;
   }
 
@@ -721,7 +745,8 @@ bool invoice::save()
   if (! _recurring->save(true, cp, &errmsg))
   {
     rollbackq.exec();
-    systemError(this, errmsg, __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Occurred"),
+                         errmsg.arg(windowTitle()),__FILE__,__LINE__);
     return false;
   }
 
@@ -734,22 +759,26 @@ void invoice::postInvoice()
 {
   XSqlQuery unpostedPost;
   int journal = -1;
+  int itemlocSeries;
   unpostedPost.exec("SELECT fetchJournalNumber('AR-IN') AS result;");
   if (unpostedPost.first())
   {
     journal = unpostedPost.value("result").toInt();
     if (journal < 0)
     {
-      systemError(this, storedProcErrorLookup("fetchJournalNumber", journal), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting Invoice"),
+                             storedProcErrorLookup("fetchJournalNumber", journal),
+                             __FILE__, __LINE__);
       return;
     }
   }
-  else if (unpostedPost.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting Invoice"),
+                                unpostedPost, __FILE__, __LINE__))
   {
-    systemError(this, unpostedPost.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
+  // Check total invoice value, if 0 open dialog, if not 0 verify that it has a curr exchange rate 
   XSqlQuery xrate;
   xrate.prepare("SELECT curr_rate "
 		"FROM curr_rate, invchead "
@@ -759,73 +788,141 @@ void invoice::postInvoice()
   // if SUM becomes dependent on curr_id then move XRATE before it in the loop
   XSqlQuery sum;
   sum.prepare("SELECT invoicetotal(:invchead_id) AS subtotal;");
-
-  XSqlQuery rollback;
-  rollback.prepare("ROLLBACK;");
-
-  XSqlQuery post;
-  post.prepare("SELECT postInvoice(:invchead_id, :journal) AS result;");
-
   sum.bindValue(":invchead_id", _invcheadid);
   if (sum.exec() && sum.first() && sum.value("subtotal").toDouble() == 0)
   {
-     if (QMessageBox::question(this, tr("Invoice Has Value 0"),
-	      		  tr("Invoice #%1 has a total value of 0.\n"
-		     	     "Would you like to post it anyway?")
-			    .arg(_invoiceNumber->text()),
-			  QMessageBox::Yes,
-			  QMessageBox::No | QMessageBox::Default)
-	     == QMessageBox::No)
-	       return;
+    if (QMessageBox::question(this, tr("Invoice Has Value 0"),
+      tr("Invoice #%1 has a total value of 0.\n""Would you like to post it anyway?")
+      .arg(_invoiceNumber->text()), QMessageBox::Yes, QMessageBox::No | QMessageBox::Default)
+      == QMessageBox::No)
+      return;
   }
   else if (sum.lastError().type() != QSqlError::NoError)
   {
-    systemError(this, sum.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting Invoice"),
+                         sum, __FILE__, __LINE__);
+    return;
   }
   else if (sum.value("subtotal").toDouble() != 0)
   {
-     xrate.bindValue(":invchead_id", _invcheadid);
-     xrate.exec();
-     if (xrate.lastError().type() != QSqlError::NoError)
-     {
-       systemError(this, tr("System Error posting Invoice #%1\n%2")
-	            .arg(_invoiceNumber->text())
-	            .arg(xrate.lastError().databaseText()),
-                __FILE__, __LINE__);
-     }
-     else if (!xrate.first() || xrate.value("curr_rate").isNull())
-     {
-       systemError(this, tr("Could not post Invoice #%1 because of a missing exchange rate.")
-				.arg(_invoiceNumber->text()));
-     }
+    xrate.bindValue(":invchead_id", _invcheadid);
+    xrate.exec();
+    if (xrate.lastError().type() != QSqlError::NoError)
+    {
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting Invoice #%1\n")
+                          .arg(_invoiceNumber->text()),
+                          xrate, __FILE__, __LINE__);
+      return;
+    }
+    else if (!xrate.first() || xrate.value("curr_rate").isNull())
+    {
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Occurred"),
+                          tr("Window:%2\nCould not post Invoice #%1 due to a missing exchange rate.")
+                          .arg(_invoiceNumber->text())
+                          .arg(windowTitle()),__FILE__,__LINE__);
+      return;
+    }
   }
 
-  unpostedPost.exec("BEGIN;");	// because of possible lot, serial, or location distribution cancelations
+  // Stage distribution cleanup function to be called on error
+  XSqlQuery cleanup;
+  cleanup.prepare("SELECT deleteitemlocseries(:itemlocSeries, TRUE);");
+
+  // Get the parent series id
+  XSqlQuery parentSeries;
+  parentSeries.prepare("SELECT NEXTVAL('itemloc_series_seq') AS result;");
+  parentSeries.exec();
+  if (parentSeries.first() && parentSeries.value("result").toInt() > 0)
+  {
+    itemlocSeries = parentSeries.value("result").toInt();
+    cleanup.bindValue(":itemlocSeries", itemlocSeries);
+  }
+  else
+  {
+    ErrorReporter::error(QtCriticalMsg, this, tr("Failed to Retrieve the Next itemloc_series_seq"),
+                            parentSeries, __FILE__, __LINE__);
+    return;
+  }
+
+  // Handle the Inventory and G/L Transactions for any billed Inventory where invcitem_updateinv is true
+  XSqlQuery items;
+  items.prepare("SELECT itemsite_id, invcitem_id, "
+                " (invcitem_billed * invcitem_qty_invuomratio) AS qty, "
+                " invchead_invcnumber "
+                "FROM invchead " 
+                " JOIN invcitem ON invcitem_invchead_id = invchead_id "
+                "   AND invcitem_billed <> 0 " 
+                "   AND invcitem_updateinv "
+                " JOIN itemsite ON itemsite_item_id = invcitem_item_id " 
+                "   AND itemsite_warehous_id = invcitem_warehous_id "
+                " JOIN item ON item_id = invcitem_item_id "
+                "WHERE invchead_id = :invchead_id "
+                " AND itemsite_costmethod != 'J' "
+                " AND isControlledItemsite(itemsite_id) "
+                "ORDER BY invcitem_id;");
+  items.bindValue(":invchead_id", _invcheadid);
+  items.exec();
+  while (items.next())
+  {
+    // Create the parent itemlocdist record for each line item requiring distribution, call distributeInventory::seriesAdjust
+    XSqlQuery parentItemlocdist;
+    parentItemlocdist.prepare("SELECT createitemlocdistparent(:itemsite_id, :qty, 'IN', "
+                              " :orderitemId, :itemlocSeries, NULL, NULL, 'SH');");
+    parentItemlocdist.bindValue(":itemsite_id", items.value("itemsite_id").toInt());
+    parentItemlocdist.bindValue(":qty", items.value("qty").toDouble() * -1);
+    parentItemlocdist.bindValue(":orderitemId", items.value("invcitem_id").toInt());
+    parentItemlocdist.bindValue(":itemlocSeries", itemlocSeries);
+    parentItemlocdist.exec();
+    if (!parentItemlocdist.first())
+    {
+      cleanup.exec();
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Creating itemlocdist Records"),
+        parentItemlocdist, __FILE__, __LINE__);
+      return;
+    }
+  }
+
+  // If the above query had results, call distributeInventory
+  if (items.size() > 0 && (distributeInventory::SeriesAdjust(itemlocSeries, this, QString(), QDate(),
+        QDate(), true) == XDialog::Rejected))
+  {
+    cleanup.exec();
+    QMessageBox::information( this, tr("Post Invoices"), tr("Error Posting Invoice Item Distribution") );
+    return;
+  }
+
+  // TODO - remove this after postInvoice has had the remaining negative error codes replaced with RAISE EXCEPTIONs
+  XSqlQuery rollback;
+  rollback.prepare("ROLLBACK;");
+
+  // Post invoice
+  XSqlQuery post;
+  post.exec("BEGIN;");
+  post.prepare("SELECT postInvoice(:invchead_id, :journal, :itemlocSeries, true) AS result;");
   post.bindValue(":invchead_id", _invcheadid);
-  post.bindValue(":journal",     journal);
+  post.bindValue(":journal", journal);
+  post.bindValue(":itemlocSeries", itemlocSeries);
   post.exec();
   if (post.first())
   {
-     int result = post.value("result").toInt();
-     if (result < 0)
-     {
-       rollback.exec();
-       systemError(this, storedProcErrorLookup("postInvoice", result),
-	           __FILE__, __LINE__);
-     }
-     else if (distributeInventory::SeriesAdjust(result, this) == XDialog::Rejected)
-     {
-       rollback.exec();
-       QMessageBox::information( this, tr("Post Invoices"), tr("Transaction Canceled") );
-       return;
-     }
-
-     unpostedPost.exec("COMMIT;");
-   }
-   // contains() string is hard-coded in stored procedure
-   else if (post.lastError().databaseText().contains("post to closed period"))
-     rollback.exec();
-
+    int result = post.value("result").toInt();
+    if (result < 0 || result != itemlocSeries)
+    {
+      rollback.exec();
+      cleanup.exec();
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting Invoice"),
+                          storedProcErrorLookup("postInvoice", result),
+                          __FILE__, __LINE__);
+      return;
+    }
+  }
+  else if (post.lastError().type() != QSqlError::NoError)
+  {
+    rollback.exec();
+    cleanup.exec();
+    return;
+  }
+  post.exec("COMMIT;");
 }
 
 void invoice::sNew()
@@ -878,9 +975,9 @@ void invoice::sDelete()
              "WHERE (invcitem_id=:invcitem_id);" );
   invoiceDelete.bindValue(":invcitem_id", _invcitem->id());
   invoiceDelete.exec();
-  if (invoiceDelete.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Line Item From Invoice"),
+                                invoiceDelete, __FILE__, __LINE__))
   {
-    systemError(this, invoiceDelete.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -1013,56 +1110,26 @@ void invoice::populate()
 
     sFillItemList();
   }
-  if (invoicepopulate.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Invoice Information"),
+                                invoicepopulate, __FILE__, __LINE__))
   {
-    systemError(this, invoicepopulate.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
 
 void invoice::sFillItemList()
 {
-  XSqlQuery invoiceFillItemList;
-  invoiceFillItemList.prepare( "SELECT invcitem_id, invcitem_linenumber,"
-             "       formatSoItemNumber(invcitem_coitem_id) AS soitemnumber, "
-             "       CASE WHEN (item_id IS NULL) THEN invcitem_number"
-             "            ELSE item_number"
-             "       END AS itemnumber,"
-             "       CASE WHEN (item_id IS NULL) THEN invcitem_descrip"
-             "            ELSE (item_descrip1 || ' ' || item_descrip2)"
-             "       END AS itemdescription,"
-             "       quom.uom_name AS qtyuom,"
-             "       invcitem_ordered, invcitem_billed,"
-             "       puom.uom_name AS priceuom,"
-             "       invcitem_price,"
-             "       round((invcitem_billed * invcitem_qty_invuomratio) * (invcitem_price / "
-	           "            (CASE WHEN(item_id IS NULL) THEN 1 "
-	           "			            ELSE invcitem_price_invuomratio END)), 2) AS extprice,"
-             "       COALESCE(coitem_unitcost, itemCost(itemsite_id), 0.0) AS unitcost,"
-             "       ROUND((invcitem_billed * invcitem_qty_invuomratio) *"
-             "             ((invcitem_price / COALESCE(invcitem_price_invuomratio,1.0)) - "
-             "              currtolocal(:curr_id, COALESCE(coitem_unitcost, itemCost(itemsite_id), 0.0), :date)),2) AS margin,"
-             "       CASE WHEN (invcitem_price = 0.0) THEN 100.0"
-             "            ELSE (((invcitem_price - currtolocal(:curr_id, COALESCE(coitem_unitcost, itemCost(itemsite_id), 0.0), :date)) / invcitem_price) * 100.0)"
-             "       END AS marginpercent,"
-             "       'qty' AS invcitem_ordered_xtnumericrole,"
-             "       'qty' AS invcitem_billed_xtnumericrole,"
-             "       'salesprice' AS invcitem_price_xtnumericrole,"
-             "       'curr' AS extprice_xtnumericrole,"
-             "       'cost' AS unitcost_xtnumericrole "
-             "FROM invcitem LEFT OUTER JOIN item on (invcitem_item_id=item_id) "
-             "  LEFT OUTER JOIN uom AS quom ON (invcitem_qty_uom_id=quom.uom_id)"
-             "  LEFT OUTER JOIN uom AS puom ON (invcitem_price_uom_id=puom.uom_id)"
-             "  LEFT OUTER JOIN coitem ON (coitem_id=invcitem_coitem_id)"
-             "  LEFT OUTER JOIN itemsite ON (itemsite_item_id=invcitem_item_id AND itemsite_warehous_id=invcitem_warehous_id)"
-             "WHERE (invcitem_invchead_id=:invchead_id) "
-             "ORDER BY invcitem_linenumber;" );
-  invoiceFillItemList.bindValue(":invchead_id", _invcheadid);
-  invoiceFillItemList.bindValue(":curr_id", _custCurrency->id());
-  invoiceFillItemList.bindValue(":date", _orderDate->date());
-  invoiceFillItemList.exec();
-  if (invoiceFillItemList.lastError().type() != QSqlError::NoError)
-      systemError(this, invoiceFillItemList.lastError().databaseText(), __FILE__, __LINE__);
+  MetaSQLQuery mql = mqlLoad("invoiceItems", "list");
+
+  ParameterList params;
+  params.append("invchead_id", _invcheadid);
+  params.append("curr_id", _custCurrency->id());
+  params.append("date", _orderDate->date());
+  XSqlQuery invoiceFillItemList = mql.toQuery(params);
+
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Invoice Items"),
+                             invoiceFillItemList, __FILE__, __LINE__))
+    return;
 
   _invcitem->clear();
   _invcitem->populate(invoiceFillItemList);
@@ -1079,7 +1146,8 @@ void invoice::sFillItemList()
   if (invoiceFillItemList.first())
     _subtotal->setLocalValue(invoiceFillItemList.value("subtotal").toDouble());
   else if (invoiceFillItemList.lastError().type() != QSqlError::NoError)
-    systemError(this, invoiceFillItemList.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Invoice Information"),
+                       invoiceFillItemList, __FILE__, __LINE__);
 
   _custCurrency->setEnabled(_invcitem->topLevelItemCount() == 0);
 
@@ -1107,7 +1175,8 @@ void invoice::closeEvent(QCloseEvent *pEvent)
     invoicecloseEvent.bindValue(":invchead_id", _invcheadid);
     invoicecloseEvent.exec();
     if (invoicecloseEvent.lastError().type() != QSqlError::NoError)
-      systemError(this, invoicecloseEvent.lastError().databaseText(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Confirming Invoice Has Not Been Posted"),
+                         invoicecloseEvent, __FILE__, __LINE__);
     if (invoicecloseEvent.first() && invoicecloseEvent.value("invchead_posted").toBool())
     {
       QMessageBox::warning( this, tr("Cannot delete Invoice"),
@@ -1120,7 +1189,8 @@ void invoice::closeEvent(QCloseEvent *pEvent)
     invoicecloseEvent.bindValue(":invoiceNumber", _invoiceNumber->text().toInt());
     invoicecloseEvent.exec();
     if (invoicecloseEvent.lastError().type() != QSqlError::NoError)
-      systemError(this, invoicecloseEvent.lastError().databaseText(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Invoice"),
+                         invoicecloseEvent, __FILE__, __LINE__);
     sReleaseNumber();
   }
 
@@ -1136,7 +1206,8 @@ void invoice::sReleaseNumber()
     invoiceReleaseNumber.bindValue(":number", _NumberGen);
     invoiceReleaseNumber.exec();
     if (invoiceReleaseNumber.lastError().type() != QSqlError::NoError)
-      systemError(this, invoiceReleaseNumber.lastError().databaseText(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Releasing Unused Invoice Number"),
+                         invoiceReleaseNumber, __FILE__, __LINE__);
     _NumberGen = -1;
   }
 }
@@ -1180,9 +1251,9 @@ void invoice::sCalculateTax()
   taxq.exec();
   if (taxq.first())
     _tax->setLocalValue(taxq.value("tax").toDouble());
-  else if (taxq.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Calculating Tax Amounts"),
+                                taxq, __FILE__, __LINE__))
   {
-    systemError(this, taxq.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   // changing _tax fires sCalculateTotal()
@@ -1361,9 +1432,9 @@ void invoice::populateCMInfo()
   cm.exec();
   if(cm.first())
     _allocatedCM->setLocalValue(cm.value("amount").toDouble());
-  else if (cm.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Invoice Information"),
+                                cm, __FILE__, __LINE__))
   {
-    systemError(this, cm.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   else
@@ -1385,9 +1456,9 @@ void invoice::populateCMInfo()
   cm.exec();
   if(cm.first())
     _outstandingCM->setLocalValue(cm.value("amount").toDouble());
-  else if (cm.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Invoice Information"),
+                                cm, __FILE__, __LINE__))
   {
-    systemError(this, cm.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   else
@@ -1432,9 +1503,9 @@ void invoice::populateCCInfo()
   cc.exec();
   if(cc.first())
     _authCC->setLocalValue(cc.value("amount").toDouble());
-  else if (cc.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Customer Credit Card Information"),
+                                cc, __FILE__, __LINE__))
   {
-    systemError(this, cc.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   else
@@ -1522,7 +1593,9 @@ bool invoice::sCheckInvoiceNumber()
         _invoiceNumber->setFocus();
       }
       else if (checkq.lastError().type() != QSqlError::NoError)
-        systemError(this, invoiceCheckInvoiceNumber.lastError().text(), __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error with Invoice Number %1")
+                           .arg(_invoiceNumber->text()),
+                           checkq, __FILE__, __LINE__);
       else
       {
         _invoiceNumber->setEnabled(false);
