@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -23,6 +23,7 @@
 #include "taxDetail.h"
 #include "errorReporter.h"
 #include "guiErrorCheck.h"
+#include "itemCharacteristicDelegate.h"
 
 /* TODO: connect(_warehouse id, SIGNAL(newId(int)), ...) to set _trackqoh?
          bug 16465 - seems too disruptive now as we're between 3.8.0RC and RC2
@@ -32,35 +33,48 @@ invoiceItem::invoiceItem(QWidget* parent, const char * name, Qt::WindowFlags fl)
 {
   setupUi(this);
 
-  connect(_billed,  SIGNAL(editingFinished()),    this, SLOT(sCalculateExtendedPrice()));
-  connect(_item,    SIGNAL(newId(int)),     this, SLOT(sPopulateItemInfo(int)));
-  connect(_item,    SIGNAL(newId(int)),     this, SLOT(sHandleUpdateInv()));
-  connect(_extended,SIGNAL(valueChanged()), this, SLOT(sLookupTax()));
-  connect(_listPrices,   SIGNAL(clicked()), this, SLOT(sListPrices()));
-  connect(_price,   SIGNAL(idChanged(int)), this, SLOT(sPriceGroup()));
-  connect(_price,   SIGNAL(valueChanged()), this, SLOT(sCalculateExtendedPrice()));
-  connect(_save,    SIGNAL(clicked()),      this, SLOT(sSave()));
-  connect(_taxLit,  SIGNAL(leftClickedURL(QString)), this, SLOT(sTaxDetail()));
-  connect(_taxtype, SIGNAL(newID(int)),     this, SLOT(sLookupTax()));
-  connect(_qtyUOM, SIGNAL(newID(int)), this, SLOT(sQtyUOMChanged()));
-  connect(_pricingUOM, SIGNAL(newID(int)), this, SLOT(sPriceUOMChanged()));
-  connect(_miscSelected, SIGNAL(toggled(bool)), this, SLOT(sMiscSelected(bool)));
+  connect(_billed,       SIGNAL(editingFinished()),       this, SLOT(sCalculateExtendedPrice()));
+  connect(_item,         SIGNAL(newId(int)),              this, SLOT(sPopulateItemInfo(int)));
+  connect(_item,         SIGNAL(newId(int)),              this, SLOT(sHandleUpdateInv()));
+  connect(_extended,     SIGNAL(valueChanged()),          this, SLOT(sLookupTax()));
+  connect(_listPrices,   SIGNAL(clicked()),               this, SLOT(sListPrices()));
+  connect(_price,        SIGNAL(idChanged(int)),          this, SLOT(sPriceGroup()));
+  connect(_price,        SIGNAL(valueChanged()),          this, SLOT(sCalculateExtendedPrice()));
+  connect(_save,         SIGNAL(clicked()),               this, SLOT(sSave()));
+  connect(_taxLit,       SIGNAL(leftClickedURL(QString)), this, SLOT(sTaxDetail()));
+  connect(_taxtype,      SIGNAL(newID(int)),              this, SLOT(sLookupTax()));
+  connect(_qtyUOM,       SIGNAL(newID(int)),              this, SLOT(sQtyUOMChanged()));
+  connect(_pricingUOM,   SIGNAL(newID(int)),              this, SLOT(sPriceUOMChanged()));
+  connect(_miscSelected, SIGNAL(toggled(bool)),           this, SLOT(sMiscSelected(bool)));
 
   _ordered->setValidator(omfgThis->qtyVal());
   _billed->setValidator(omfgThis->qtyVal());
 
   _altRevAccnt->setType(GLCluster::cRevenue);
 
+  _itemchar = new QStandardItemModel(0, 2, this);
+  _itemchar->setHeaderData( 0, Qt::Horizontal, tr("Name"), Qt::DisplayRole);
+  _itemchar->setHeaderData( 1, Qt::Horizontal, tr("Value"), Qt::DisplayRole);
+
+  _itemcharView->setModel(_itemchar);
+  ItemCharacteristicDelegate * delegate = new ItemCharacteristicDelegate(this);
+  _itemcharView->setItemDelegate(delegate);
+
+  _invcharView->setType("INVI");
+  _comments->setType(Comments::InvoiceItem);
+
   _taxtype->setEnabled(_privileges->check("OverrideTax"));
   
   _mode = cNew;
   _invcheadid	= -1;
   _custid	= -1;
+  _shiptoid	= -1;
   _invcitemid	= -1;
   _priceRatioCache = 1.0;
   _taxzoneid	= -1;
   _qtyinvuomratio = 1.0;
   _priceinvuomratio = 1.0;
+  _listprice = 0.0;
   _invuomid = -1;
   _trackqoh = true;
   
@@ -97,8 +111,9 @@ enum SetResponse invoiceItem::set(const ParameterList &pParams)
   {
     _invcheadid = param.toInt();
 
-    invoiceet.prepare("SELECT * "
-                      "FROM invchead "
+    invoiceet.prepare("SELECT i.*, crmacct_id "
+                      "FROM invchead i "
+                      "JOIN crmacct ON (crmacct_cust_id=invchead_cust_id) "
 			  "WHERE (invchead_id = :invchead_id);");
     invoiceet.bindValue(":invchead_id", _invcheadid);
     invoiceet.exec();
@@ -106,15 +121,17 @@ enum SetResponse invoiceItem::set(const ParameterList &pParams)
     {
       _invoiceNumber->setText(invoiceet.value("invchead_invcnumber").toString());
       _custid = invoiceet.value("invchead_cust_id").toInt();
+      _shiptoid = invoiceet.value("invchead_shipto_id").toInt();
       _taxzoneid = invoiceet.value("invchead_taxzone_id").toInt();
       _tax->setId(invoiceet.value("invchead_curr_id").toInt());
       _price->setId(invoiceet.value("invchead_curr_id").toInt());
       _price->setEffective(invoiceet.value("invchead_invcdate").toDate());
+      _item->setCRMAcctId(invoiceet.value("crmacct_id").toInt());
       sPriceGroup();
     }
-    else if (invoiceet.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Invoice Line Item Information"),
+                                  invoiceet, __FILE__, __LINE__))
     {
-      systemError(this, invoiceet.lastError().databaseText(), __FILE__, __LINE__);
       return UndefinedError;
     }
   }
@@ -123,6 +140,8 @@ enum SetResponse invoiceItem::set(const ParameterList &pParams)
   if (valid)
   {
     _invcitemid = param.toInt();
+    _invcharView->setId(_invcitemid);
+    _comments->setId(_invcitemid);
     populate();
   }
 
@@ -135,11 +154,15 @@ enum SetResponse invoiceItem::set(const ParameterList &pParams)
 
       invoiceet.exec("SELECT NEXTVAL('invcitem_invcitem_id_seq') AS invcitem_id;");
       if (invoiceet.first())
-        _invcitemid = invoiceet.value("invcitem_id").toInt();
-      else if (invoiceet.lastError().type() != QSqlError::NoError)
       {
-            systemError(this, invoiceet.lastError().databaseText(), __FILE__, __LINE__);
-            return UndefinedError;
+        _invcitemid = invoiceet.value("invcitem_id").toInt();
+        _invcharView->setId(_invcitemid);
+        _comments->setId(_invcitemid);
+      }
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Adding New Line Item"),
+                                    invoiceet, __FILE__, __LINE__))
+      {
+        return UndefinedError;
       }
 
       invoiceet.prepare( "SELECT (COALESCE(MAX(invcitem_linenumber), 0) + 1) AS linenumber "
@@ -149,10 +172,10 @@ enum SetResponse invoiceItem::set(const ParameterList &pParams)
       invoiceet.exec();
       if (invoiceet.first())
         _lineNumber->setText(invoiceet.value("linenumber").toString());
-      else if (invoiceet.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Adding New Line Item"),
+                                    invoiceet, __FILE__, __LINE__))
       {
-	    systemError(this, invoiceet.lastError().databaseText(), __FILE__, __LINE__);
-	    return UndefinedError;
+        return UndefinedError;
       }
 
       connect(_billed, SIGNAL(editingFinished()), this, SLOT(sDeterminePrice()));
@@ -183,6 +206,9 @@ enum SetResponse invoiceItem::set(const ParameterList &pParams)
       _altRevAccnt->setEnabled(false);
       _qtyUOM->setEnabled(false);
       _pricingUOM->setEnabled(false);
+      _itemcharView->setEnabled(false);
+      _invcharView->setEnabled(false);
+      _comments->setReadOnly(true);
 
       _save->hide();
       _close->setText(tr("&Cancel"));
@@ -228,7 +254,7 @@ void invoiceItem::sSave()
                "  invcitem_custpn,"
                "  invcitem_ordered, invcitem_billed, invcitem_updateinv,"
                "  invcitem_qty_uom_id, invcitem_qty_invuomratio,"
-               "  invcitem_custprice, invcitem_price,"
+               "  invcitem_custprice, invcitem_price, invcitem_listprice,"
                "  invcitem_price_uom_id, invcitem_price_invuomratio,"
                "  invcitem_notes, "
                "  invcitem_taxtype_id, invcitem_rev_accnt_id) "
@@ -239,7 +265,7 @@ void invoiceItem::sSave()
                "  :invcitem_custpn,"
                "  :invcitem_ordered, :invcitem_billed, :invcitem_updateinv,"
                "  :qty_uom_id, :qty_invuomratio,"
-               "  :invcitem_custprice, :invcitem_price,"
+               "  :invcitem_custprice, :invcitem_price, :invcitem_listprice,"
                "  :price_uom_id, :price_invuomratio,"
                "  :invcitem_notes, "
                "  :invcitem_taxtype_id, :invcitem_rev_accnt_id);");
@@ -286,6 +312,7 @@ void invoiceItem::sSave()
   invoiceSave.bindValue(":qty_invuomratio", _qtyinvuomratio);
   invoiceSave.bindValue(":invcitem_custprice", _custPrice->localValue());
   invoiceSave.bindValue(":invcitem_price", _price->localValue());
+  invoiceSave.bindValue(":invcitem_listprice", _listPrice->baseValue());
   if(!_miscSelected->isChecked())
     invoiceSave.bindValue(":price_uom_id", _pricingUOM->id());
   invoiceSave.bindValue(":price_invuomratio", _priceinvuomratio);
@@ -296,11 +323,25 @@ void invoiceItem::sSave()
     invoiceSave.bindValue(":invcitem_rev_accnt_id", _altRevAccnt->id());
 
   invoiceSave.exec();
-  if (invoiceSave.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Invoice Line Item Information"),
+                                invoiceSave, __FILE__, __LINE__))
   {
-    systemError(this, invoiceSave.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
+
+  invoiceSave.prepare("SELECT updateCharAssignment('INVI', :target_id, :char_id, :char_value);");
+
+  QModelIndex idx1, idx2;
+  for(int i = 0; i < _itemchar->rowCount(); i++)
+  {
+    idx1 = _itemchar->index(i, 0);
+    idx2 = _itemchar->index(i, 1);
+    invoiceSave.bindValue(":target_id", _invcitemid);
+    invoiceSave.bindValue(":char_id", _itemchar->data(idx1, Qt::UserRole));
+    invoiceSave.bindValue(":char_value", _itemchar->data(idx2, Qt::DisplayRole));
+    invoiceSave.exec();
+  }
+
   _saved = true;
   emit saved(_invcitemid);
 
@@ -312,9 +353,6 @@ void invoiceItem::populate()
   XSqlQuery invcitem;
   invcitem.prepare("SELECT invcitem.*,"
                    "       invchead_invcnumber, invchead_curr_id AS taxcurr_id,"
-                   "       CASE WHEN (item_id IS NULL) THEN :na"
-                   "            ELSE item_listprice"
-                   "       END AS f_listprice,"
                    "		   taxzone_id, itemsite_costmethod,"
                    "       COALESCE(cobill_id, -1) AS cobill_id "
                    " FROM invcitem JOIN invchead ON (invchead_id=invcitem_invchead_id)"
@@ -334,7 +372,7 @@ void invoiceItem::populate()
     _lineNumber->setText(invcitem.value("invcitem_linenumber").toString());
 
     // TODO: should this check itemsite_controlmethod == N?
-    _trackqoh = (invcitem.value("invcitem_invcitem_id").toInt() > 0 &&
+    _trackqoh = (invcitem.value("invcitem_item_id").toInt() > 0 &&
                  invcitem.value("itemsite_costmethod").toString() != "J");
 
     if (invcitem.value("invcitem_item_id").toInt() != -1)
@@ -381,7 +419,6 @@ void invoiceItem::populate()
     }
     _price->setLocalValue(invcitem.value("invcitem_price").toDouble());
     _custPrice->setLocalValue(invcitem.value("invcitem_custprice").toDouble());
-    _listPrice->setBaseValue(invcitem.value("f_listprice").toDouble() * (_priceinvuomratio / _priceRatioCache));
 
     _custPn->setText(invcitem.value("invcitem_custpn").toString());
     _notes->setText(invcitem.value("invcitem_notes").toString());
@@ -396,9 +433,9 @@ void invoiceItem::populate()
       _qtyUOM->setEnabled(false);
     }
   }
-  else if (invcitem.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Invoice Line Item Information"),
+                                invcitem, __FILE__, __LINE__))
   {
-    systemError(this, invcitem.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -410,9 +447,9 @@ void invoiceItem::populate()
   invcitem.exec();
   if (invcitem.first())
     _tax->setLocalValue(invcitem.value("lineTaxTotal").toDouble());
-  else if (invcitem.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Invoice Line Item Information"),
+                                invcitem, __FILE__, __LINE__))
   {
-    systemError(this, invcitem.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -432,57 +469,30 @@ void invoiceItem::sPopulateItemInfo(int pItemid)
   XSqlQuery invoicePopulateItemInfo;
   if ( (_itemSelected->isChecked()) && (pItemid != -1) )
   {
-    // Get list of active, valid Selling UOMs
-    MetaSQLQuery muom = mqlLoad("uoms", "item");
-
-    ParameterList params;
-    params.append("uomtype", "Selling");
-    params.append("item_id", pItemid);
-
-    // Also have to factor UOMs previously used on Invoice now inactive
-    if (_invcitemid != -1)
-    {
-      XSqlQuery invuom;
-      invuom.prepare("SELECT invcitem_qty_uom_id, invcitem_price_uom_id "
-                "  FROM invcitem"
-                " WHERE(invcitem_id=:invcitem_id);");
-      invuom.bindValue(":invcitem_id", _invcitemid);
-      invuom.exec();
-      if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting Invoice UOMs"),
-                           invuom, __FILE__, __LINE__))
-        return;
-      else if (invuom.first())
-      {
-        params.append("uom_id", invuom.value("invcitem_qty_uom_id"));
-        params.append("uom_id2", invuom.value("invcitem_price_uom_id"));
-      }
-    }
-    XSqlQuery uom = muom.toQuery(params);
-    if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting UOMs"),
-                           uom, __FILE__, __LINE__))
-      return;
-
-    _qtyUOM->populate(uom);
-    _pricingUOM->populate(uom);
+    sPopulateUOM();
 
     invoicePopulateItemInfo.prepare( "SELECT item_inv_uom_id, item_price_uom_id,"
-               "       iteminvpricerat(item_id) AS invpricerat,"
-               "       item_listprice, item_fractional, "
-               "       stdcost(item_id) AS f_unitcost,"
-               "       getItemTaxType(item_id, :taxzone) AS taxtype_id,"
-               "       itemsite_costmethod"
-               "  FROM item"
-               "  JOIN itemsite ON (item_id=itemsite_item_id)"
-               " WHERE ((item_id=:item_id)"
-               "    AND (itemsite_warehous_id=:whsid));" );
+                                    "       iteminvpricerat(item_id) AS invpricerat,"
+                                    "       listPrice(item_id, :cust_id, :shipto_id, :whsid) AS listprice,"
+                                    "       item_fractional, "
+                                    "       stdcost(item_id) AS f_unitcost,"
+                                    "       getItemTaxType(item_id, :taxzone) AS taxtype_id,"
+                                    "       itemsite_costmethod"
+                                    "  FROM item"
+                                    "  JOIN itemsite ON (item_id=itemsite_item_id)"
+                                    " WHERE ((item_id=:item_id)"
+                                    "    AND (itemsite_warehous_id=:whsid));" );
     invoicePopulateItemInfo.bindValue(":item_id", pItemid);
     invoicePopulateItemInfo.bindValue(":taxzone", _taxzoneid);
     invoicePopulateItemInfo.bindValue(":whsid",   _warehouse->id());
+    invoicePopulateItemInfo.bindValue(":cust_id", _custid);
+    invoicePopulateItemInfo.bindValue(":shipto_id", _shiptoid);
     invoicePopulateItemInfo.exec();
     if (invoicePopulateItemInfo.first())
     {
       _priceRatioCache = invoicePopulateItemInfo.value("invpricerat").toDouble();
-      _listPrice->setBaseValue(invoicePopulateItemInfo.value("item_listprice").toDouble());
+      _listprice = invoicePopulateItemInfo.value("listprice").toDouble();
+      _listPrice->setBaseValue(_listprice  * (_priceinvuomratio / _priceRatioCache));
 
       _invuomid = invoicePopulateItemInfo.value("item_inv_uom_id").toInt();
       _qtyUOM->setId(invoicePopulateItemInfo.value("item_inv_uom_id").toInt());
@@ -506,17 +516,70 @@ void invoiceItem::sPopulateItemInfo(int pItemid)
       // TODO: should this check itemsite_controlmethod == N?
       _trackqoh = (invoicePopulateItemInfo.value("itemsite_costmethod").toString() != "J");
     }
-    else if (invoicePopulateItemInfo.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Invoice Line Item Information"),
+                                  invoicePopulateItemInfo, __FILE__, __LINE__))
     {
-      systemError(this, invoicePopulateItemInfo.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
+
+    XSqlQuery item;
+    invoicePopulateItemInfo.prepare( "SELECT char_id, char_name, "
+             "  CASE WHEN char_type < 2 THEN "
+             "    charass_value "
+             "  ELSE "
+             "    formatDate(charass_value::date) "
+             " END AS f_charass_value, "
+             "  charass_value, charass_value AS charass_value_qttooltiprole "
+             " FROM ( "
+             " SELECT char_id, char_type, char_name, "
+             "   COALESCE(i.charass_value,i2.charass_value) AS charass_value "
+             " FROM "
+             "   (SELECT DISTINCT char_id, char_type, char_name "
+             "    FROM charass, char, charuse "
+             "    WHERE ((charass_char_id=char_id) "
+             "    AND (charuse_char_id=char_id AND charuse_target_type='INVI') "
+             "    AND (charass_target_type='I') "
+             "    AND (charass_target_id=:item_id) )"
+             "    UNION SELECT char_id, char_type, char_name "
+             "    FROM charass, char "
+             "    WHERE ((charass_char_id=char_id) "
+             "    AND charass_char_id IN (SELECT charuse_char_id FROM charuse"
+             "                           WHERE charuse_target_type = 'I')"
+             "    AND  (charass_target_type = 'INVI' AND charass_target_id=:invcitem_id)) ) AS data "
+             "   LEFT OUTER JOIN charass  i  ON ((:invcitem_id=i.charass_target_id) "
+             "                               AND ('INVI'=i.charass_target_type) "
+             "                               AND (i.charass_char_id=char_id)) "
+             "   LEFT OUTER JOIN item     i1 ON (i1.item_id=:item_id) "
+             "   LEFT OUTER JOIN charass  i2 ON ((i1.item_id=i2.charass_target_id) "
+             "                               AND ('I'=i2.charass_target_type) "
+             "                               AND (i2.charass_char_id=char_id) "
+             "                               AND (i2.charass_default))) data2 "
+             " ORDER BY char_name;"  );
+    invoicePopulateItemInfo.bindValue(":item_id", pItemid);
+    invoicePopulateItemInfo.bindValue(":invcitem_id", _invcitemid);
+    invoicePopulateItemInfo.exec();
+    int row = 0;
+    _itemchar->removeRows(0, _itemchar->rowCount());
+    QModelIndex idx;
+    while(invoicePopulateItemInfo.next())
+    {
+      _itemchar->insertRow(_itemchar->rowCount());
+      idx = _itemchar->index(row, 0);
+      _itemchar->setData(idx, invoicePopulateItemInfo.value("char_name"), Qt::DisplayRole);
+      _itemchar->setData(idx, invoicePopulateItemInfo.value("char_id"), Qt::UserRole);
+      idx = _itemchar->index(row, 1);
+      _itemchar->setData(idx, invoicePopulateItemInfo.value("charass_value"), Qt::DisplayRole);
+      _itemchar->setData(idx, _item->id(), Xt::IdRole);
+      _itemchar->setData(idx, _item->id(), Qt::UserRole);
+      row++;
+    }
   }
   else
   {
     _priceRatioCache = 1.0;
     _qtyinvuomratio = 1.0;
     _priceinvuomratio = 1.0;
+    _listprice = 0.0;
     _qtyUOM->clear();
     _pricingUOM->clear();
     _listPrice->clear();
@@ -530,11 +593,12 @@ void invoiceItem::sDeterminePrice()
   if ( (_itemSelected->isChecked()) && (_item->isValid()) && (_billed->toDouble()) && (_qtyUOM->id() > 0) && (_pricingUOM->id() > 0) )
   {
     XSqlQuery itemprice;
-    itemprice.prepare( "SELECT itemPrice(item_id, :cust_id, -1, "
-		       "		 :qty, :qtyUOM, :priceUOM, :curr_id, :effective) AS price "
-                       "FROM item "
-                       "WHERE (item_id=:item_id);" );
+    itemprice.prepare("SELECT itemPrice(item_id, :cust_id, :shipto_id, "
+                      "		              :qty, :qtyUOM, :priceUOM, :curr_id, :effective) AS price "
+                      "FROM item "
+                      "WHERE (item_id=:item_id);" );
     itemprice.bindValue(":cust_id", _custid);
+    itemprice.bindValue(":shipto_id", _shiptoid);
     itemprice.bindValue(":qty", _billed->toDouble());
     itemprice.bindValue(":qtyUOM", _qtyUOM->id());
     itemprice.bindValue(":priceUOM", _pricingUOM->id());
@@ -550,9 +614,9 @@ void invoiceItem::sDeterminePrice()
                                tr("<p>This item is marked as exclusive and "
                                   "no qualifying price schedule was found. "
                                   "You may click on the price list button "
-				  "(...) next to the Unit Price to determine "
+				                  "(...) next to the Unit Price to determine "
                                   "if there is a minimum quantity the selected "
-				  "Customer may purchase." ) );
+				                  "Customer may purchase." ) );
         _custPrice->clear();
         _price->clear();
         _billed->clear();
@@ -565,9 +629,9 @@ void invoiceItem::sDeterminePrice()
       _custPrice->setLocalValue(price);
       _price->setLocalValue(price);
     }
-    else if (itemprice.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Determining Item Price"),
+                                  itemprice, __FILE__, __LINE__))
     {
-      systemError(this, itemprice.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -609,8 +673,92 @@ void invoiceItem::sTaxDetail()
   }
 }
 
+void invoiceItem::sPopulateUOM()
+{
+  if ( (_itemSelected->isChecked()) && (_item->id() != -1) )
+  {
+    // Get list of active, valid Selling UOMs
+    MetaSQLQuery muom = mqlLoad("uoms", "item");
+    
+    ParameterList params;
+    params.append("uomtype", "Selling");
+    params.append("item_id", _item->id());
+    
+    // Include Global UOMs
+    if (_privileges->check("MaintainUOMs"))
+    {
+      params.append("includeGlobal", true);
+      params.append("global", tr("-Global"));
+    }
+    
+    // Also have to factor UOMs previously used on Invoice now inactive
+    if (_invcitemid != -1)
+    {
+      XSqlQuery invcuom;
+      invcuom.prepare("SELECT invcitem_qty_uom_id, invcitem_price_uom_id "
+                      "  FROM invcitem"
+                      " WHERE(invcitem_id=:invcitem_id);");
+      invcuom.bindValue(":invcitem_id", _invcitemid);
+      invcuom.exec();
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting Invoice UOMs"),
+                               invcuom, __FILE__, __LINE__))
+        return;
+      else if (invcuom.first())
+      {
+        params.append("uom_id", invcuom.value("invcitem_qty_uom_id"));
+        params.append("uom_id2", invcuom.value("invcitem_price_uom_id"));
+      }
+    }
+
+    XSqlQuery uom = muom.toQuery(params);
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting UOMs"),
+                             uom, __FILE__, __LINE__))
+      return;
+    
+    int saveqtyuomid = _qtyUOM->id();
+    int savepriceuomid = _pricingUOM->id();
+    disconnect(_qtyUOM,     SIGNAL(newID(int)), this, SLOT(sQtyUOMChanged()));
+    disconnect(_pricingUOM, SIGNAL(newID(int)), this, SLOT(sPriceUOMChanged()));
+    _qtyUOM->populate(uom);
+    _pricingUOM->populate(uom);
+    _qtyUOM->setId(saveqtyuomid);
+    _pricingUOM->setId(savepriceuomid);
+    connect(_qtyUOM,     SIGNAL(newID(int)), this, SLOT(sQtyUOMChanged()));
+    connect(_pricingUOM, SIGNAL(newID(int)), this, SLOT(sPriceUOMChanged()));
+  }
+}
+
 void invoiceItem::sQtyUOMChanged()
 {
+  // Check for Global UOM Conversion that must be setup for Item
+  if (_qtyUOM->code() == "G")
+  {
+    if (QMessageBox::question(this, tr("Use Global UOM?"),
+                              tr("<p>This Global UOM Conversion is not setup for this Item."
+                                 "<p>Do you want to add this UOM conversion to this Item?"),
+                              QMessageBox::Yes | QMessageBox::Default,
+                              QMessageBox::No  | QMessageBox::Escape) == QMessageBox::Yes)
+    {
+      // create itemuomconv and itemuom
+      XSqlQuery adduom;
+      adduom.prepare("SELECT createItemUomConv(:item_id, :uom_id, :uom_type) AS result;");
+      adduom.bindValue(":item_id", _item->id());
+      adduom.bindValue(":uom_id", _qtyUOM->id());
+      adduom.bindValue(":uom_type", "Selling");
+      adduom.exec();
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Creating Item UOM Conv"),
+                               adduom, __FILE__, __LINE__))
+        return;
+      
+      // repopulate uom comboboxes
+      sPopulateUOM();
+    }
+    else
+    {
+      _qtyUOM->setId(_invuomid);
+    }
+  }
+  
   if(_qtyUOM->id() == _invuomid)
     _qtyinvuomratio = 1.0;
   else
@@ -625,7 +773,8 @@ void invoiceItem::sQtyUOMChanged()
     if(invuom.first())
       _qtyinvuomratio = invuom.value("ratio").toDouble();
     else
-      systemError(this, invuom.lastError().databaseText(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Changing Quantity UOM"),
+                         invuom, __FILE__, __LINE__);
   }
 
   if(_qtyUOM->id() != _invuomid)
@@ -635,6 +784,7 @@ void invoiceItem::sQtyUOMChanged()
   }
   else
     _pricingUOM->setEnabled(true);
+  
   sDeterminePrice();
   sCalculateExtendedPrice();
 }
@@ -644,6 +794,34 @@ void invoiceItem::sPriceUOMChanged()
   if(_pricingUOM->id() == -1 || _qtyUOM->id() == -1)
     return;
 
+  // Check for Global UOM Conversion that must be setup for Item
+  if (_pricingUOM->code() == "G")
+  {
+    if (QMessageBox::question(this, tr("Use Global UOM?"),
+                              tr("<p>This Global UOM Conversion is not setup for this Item."
+                                 "<p>Do you want to add this UOM conversion to this Item?"),
+                              QMessageBox::Yes | QMessageBox::Default,
+                              QMessageBox::No  | QMessageBox::Escape) == QMessageBox::Yes)
+    {
+      XSqlQuery adduom;
+      adduom.prepare("SELECT createItemUomConv(:item_id, :uom_id, :uom_type) AS result;");
+      adduom.bindValue(":item_id", _item->id());
+      adduom.bindValue(":uom_id", _pricingUOM->id());
+      adduom.bindValue(":uom_type", "Selling");
+      adduom.exec();
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Creating Item UOM Conv"),
+                               adduom, __FILE__, __LINE__))
+        return;
+      
+      // repopulate uom comboboxes
+      sPopulateUOM();
+    }
+    else
+    {
+      _pricingUOM->setId(_invuomid);
+    }
+  }
+  
   if(_pricingUOM->id() == _invuomid)
     _priceinvuomratio = 1.0;
   else
@@ -658,17 +836,11 @@ void invoiceItem::sPriceUOMChanged()
     if(invuom.first())
       _priceinvuomratio = invuom.value("ratio").toDouble();
     else
-      systemError(this, invuom.lastError().databaseText(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Changing Price UOM"),
+                         invuom, __FILE__, __LINE__);
   }
 
-  XSqlQuery item;
-  item.prepare("SELECT item_listprice"
-               "  FROM item"
-               " WHERE(item_id=:item_id);");
-  item.bindValue(":item_id", _item->id());
-  item.exec();
-  item.first();
-  _listPrice->setBaseValue(item.value("item_listprice").toDouble() * (_priceinvuomratio / _priceRatioCache));
+  _listPrice->setBaseValue(_listprice * (_priceinvuomratio / _priceRatioCache));
   sDeterminePrice();
   sCalculateExtendedPrice();
 }
@@ -697,9 +869,9 @@ void invoiceItem::sHandleUpdateInv()
     {
       _updateInv->setEnabled(true);
     }
-    else if (invq.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Invoice Line Item Information"),
+                                  invq, __FILE__, __LINE__))
     {
-      systemError(this, invq.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     else
@@ -719,7 +891,7 @@ void invoiceItem::sListPrices()
 {
   ParameterList params;
   params.append("cust_id",   _custid);
-  //params.append("shipto_id", _shiptoid);
+  params.append("shipto_id", _shiptoid);
   params.append("item_id",   _item->id());
   params.append("warehous_id", _warehouse->id());
   params.append("qty",       _billed->toDouble() * _qtyinvuomratio);
@@ -751,9 +923,9 @@ void invoiceItem::sLookupTax()
     _tax->setLocalValue(taxcal.value("taxamount").toDouble());
 	_saved = false;
   }
-  else if (taxcal.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Calculating Line Item Tax"),
+                                taxcal, __FILE__, __LINE__))
   {
-    systemError(this, taxcal.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
