@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -11,6 +11,7 @@
 
 #include "issueToShipping.h"
 
+#include <QMenu>
 #include <QSqlError>
 #include <QVariant>
 
@@ -24,6 +25,9 @@
 #include "reserveSalesOrderItem.h"
 #include "shipOrder.h"
 #include "storedProcErrorLookup.h"
+#include "errorReporter.h"
+
+#define DEBUG false
 
 issueToShipping::issueToShipping(QWidget* parent, const char* name, Qt::WindowFlags fl)
     : XWidget(parent, name, fl),
@@ -100,9 +104,10 @@ issueToShipping::issueToShipping(QWidget* parent, const char* name, Qt::WindowFl
     _warehouseLit->hide();
     _warehouse->hide();
   }
-  
-  if(_metrics->boolean("EnableSOReservations"))
+
+  if(_metrics->boolean("EnableSOReservations") || _metrics->boolean("DisallowNegativeInventory"))
   {
+    _requireInventory->setForgetful(true);
     _requireInventory->setChecked(true);
     _requireInventory->setEnabled(false);
   }
@@ -185,9 +190,9 @@ void issueToShipping::sCatchSoitemid(int pSoitemid)
     _soitem->setId(pSoitemid);
     sIssueStock();
   }
-  else if (issueCatchSoitemid.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                issueCatchSoitemid, __FILE__, __LINE__))
   {
-    systemError(this, issueCatchSoitemid.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -213,9 +218,9 @@ void issueToShipping::sCatchToitemid(int porderitemid)
     _soitem->setId(porderitemid);
     sIssueStock();
   }
-  else if (issueCatchToitemid.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                issueCatchToitemid, __FILE__, __LINE__))
   {
-    systemError(this, issueCatchToitemid.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -284,9 +289,9 @@ void issueToShipping::sCatchWoid(int pWoid)
       _soitem->setId(issueCatchWoid.value("coitem_id").toInt());
       sIssueStock();
     }
-    else if (issueCatchWoid.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                  issueCatchWoid, __FILE__, __LINE__))
     {
-      systemError(this, issueCatchWoid.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -296,7 +301,7 @@ void issueToShipping::sCatchWoid(int pWoid)
 
 void issueToShipping::sPopulateMenu(QMenu *pMenu)
 {
-  if (_metrics->boolean("EnableSOReservations"))
+  if (_metrics->boolean("EnableSOReservations") && _order->isSO())
   {
     QAction *menuItem;
     menuItem = pMenu->addAction(tr("Unreserve Stock"), this, SLOT(sUnreserveStock()));
@@ -321,7 +326,7 @@ void issueToShipping::sIssueStock()
 
     if(_requireInventory->isChecked())
       params.append("requireInventory");
-    
+
     issueLineToShipping newdlg(this, "", true);
     if (newdlg.set(params) == NoError && newdlg.exec() != XDialog::Rejected)
       update = true;
@@ -366,24 +371,26 @@ bool issueToShipping::sufficientItemInventory(int porderitemid)
                        "   AND  (toitem_tohead_id=tohead_id)"
                        "   AND  (toitem_id=<? value(\"toitem_id\") ?>));"
                        "<? endif ?>" ;
-        MetaSQLQuery errm(errs);
-        issueufficientItemInventory = errm.toQuery(errp);
-        if (! issueufficientItemInventory.first() && issueufficientItemInventory.lastError().type() != QSqlError::NoError)
-          systemError(this, issueufficientItemInventory.lastError().databaseText(), __FILE__, __LINE__);
-        systemError(this,
-                    storedProcErrorLookup("sufficientInventoryToShipItem",
-                                          result)
-                    .arg(issueufficientItemInventory.value("item_number").toString())
-                    .arg(issueufficientItemInventory.value("warehous_code").toString()), __FILE__, __LINE__);
-        return false;
-      }
-    }
-    else if (issueufficientItemInventory.lastError().type() != QSqlError::NoError)
-    {
-      systemError(this, issueufficientItemInventory.lastError().databaseText(), __FILE__, __LINE__);
-      return false;
-    }
-  }
+         MetaSQLQuery errm(errs);
+         issueufficientItemInventory = errm.toQuery(errp);
+         if (! issueufficientItemInventory.first() && issueufficientItemInventory.lastError().type() != QSqlError::NoError)
+           ErrorReporter::error(QtCriticalMsg, this, tr("Insufficient Inventory To Ship"),
+                              issueufficientItemInventory, __FILE__, __LINE__);
+
+         ErrorReporter::error(QtCriticalMsg, this, tr("Insufficient Inventory To Ship"),
+                                  storedProcErrorLookup("sufficientInventoryToShipItem", result)
+                                  .arg(issueufficientItemInventory.value("item_number").toString())
+                                  .arg(issueufficientItemInventory.value("warehous_code").toString()),
+                                  __FILE__, __LINE__);
+         return false;
+       }
+     }
+     else if (ErrorReporter::error(QtCriticalMsg, this, tr("Insufficient Inventory To Ship"),
+                                  issueufficientItemInventory, __FILE__, __LINE__))
+     {
+       return false;
+     }
+   }
 
   return true;
 }
@@ -403,13 +410,15 @@ bool issueToShipping::sufficientInventory(int porderheadid)
       int result = issueufficientInventory.value("result").toInt();
       if (result < 0)
       {
-        systemError(this, storedProcErrorLookup("sufficientInventoryToShipOrder", result), __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Insufficient Inventory To Ship"),
+                               storedProcErrorLookup("sufficientInventoryToShipOrder", result),
+                               __FILE__, __LINE__);
         return false;
       }
     }
-    else if (issueufficientInventory.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Insufficient Inventory To Ship"),
+                                  issueufficientInventory, __FILE__, __LINE__))
     {
-      systemError(this, issueufficientInventory.lastError().databaseText(), __FILE__, __LINE__);
       return false;
     }
   }
@@ -442,45 +451,222 @@ bool issueToShipping::sIssueLineBalance(int id, int altId)
     if (! sufficientItemInventory(id))
       return false;
   }
-  int invhistid = 0;
-  int itemlocSeries = 0;
 
+  XSqlQuery issueDetail;
+  issueDetail.prepare("SELECT calcIssueToShippingLineBalance(:orderType, :orderitemId) AS balance, "
+                       "  orderitem_qty_invuomratio, "
+                       "  isControlledItemsite(orderitem_itemsite_id) AS controlled, "
+                       "  isControlledItemsite(wo_itemsite_id) AS woItemControlled, "
+                       "  orderitem_itemsite_id AS itemsite_id, wo_id, wo_itemsite_id, "
+                       "  CASE WHEN wo_id IS NOT NULL THEN "
+                       "    roundQty(woitem.item_fractional, calcIssueToShippingLineBalance(:orderType, :orderitemId) "
+                       "    * orderitem_qty_invuomratio) "
+                       "  ELSE NULL END AS postprodqty "
+                       "FROM orderitem "
+                       "  LEFT OUTER JOIN wo ON orderitem_id = wo_ordid "
+                       "    AND wo_ordtype = 'S' AND orderitem_orderhead_type = 'SO' "
+                       "  LEFT OUTER JOIN itemsite AS woitemsite ON woitemsite.itemsite_id = wo_itemsite_id "
+                       "  LEFT OUTER JOIN item AS woitem ON woitem.item_id = woitemsite.itemsite_item_id "
+                       "WHERE orderitem_orderhead_type = :orderType "
+                       " AND orderitem_id = :orderitemId;");
+  issueDetail.bindValue(":orderType", _order->type());
+  issueDetail.bindValue(":orderitemId", id);
+  issueDetail.exec();
+  if (issueDetail.first())
+  {
+    if (issueDetail.size() > 1)
+    {
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Gathering Order Item Info."),
+        tr("Retrieving from orderitem table returned more than one row."), __FILE__, __LINE__);
+      return false;
+    }
+
+
+  }
+  else
+  {
+    ErrorReporter::error(QtCriticalMsg, this, tr("Could not find orderitem info."),
+      issueDetail.lastError().text(), __FILE__, __LINE__);
+    return false;
+  }
+
+  double balance = issueDetail.value("balance").toDouble();
+  if (balance == 0)
+    return false;  // nothing to issue
+
+  int invhistid = 0;
+  int itemlocSeries;
+  int itemsiteId = issueDetail.value("itemsite_id").toInt();;
+  bool controlled = issueDetail.value("controlled").toBool();;
+  bool hasControlledBackflushItems = false;
+  bool jobItem = (altId == 1 && balance > 0);
+
+  XSqlQuery issue;
+  XSqlQuery cleanup;
+  XSqlQuery parentItemlocdist;
+  XSqlQuery womatlItemlocdist;
+  XSqlQuery parentSeries;
   XSqlQuery rollback;
   rollback.prepare("ROLLBACK;");
 
-  XSqlQuery issue;
-  issue.exec("BEGIN;");
+  // Stage cleanup function to be called on error
+  cleanup.prepare("SELECT deleteitemlocseries(:itemlocSeries, TRUE);");
+
+  // Series for issueToShipping
+  parentSeries.prepare("SELECT NEXTVAL('itemloc_series_seq') AS result;");
+  parentSeries.exec();
+  if (parentSeries.first() && parentSeries.value("result").toInt() > 0)
+  {
+    itemlocSeries = parentSeries.value("result").toInt();
+    cleanup.bindValue(":itemlocSeries", itemlocSeries);
+  }
+  else
+  {
+    ErrorReporter::error(QtCriticalMsg, this, tr("Failed to Retrieve the Next itemloc_series_seq"),
+      parentSeries, __FILE__, __LINE__);
+    return false;
+  }
+
+  // Stage this here so that, if job item, some of the params can be overriden with WO relavant values.
+  parentItemlocdist.prepare("SELECT createItemlocdistParent(:itemsite_id, :qty, :orderType, :orderitemId, "
+    ":itemlocSeries, NULL, NULL, :transType) AS result;");
+  parentItemlocdist.bindValue(":itemsite_id", itemsiteId);
+  parentItemlocdist.bindValue(":qty", balance * issueDetail.value("orderitem_qty_invuomratio").toDouble() * -1);
+  parentItemlocdist.bindValue(":orderitemId", id);
+  parentItemlocdist.bindValue(":itemlocSeries", itemlocSeries);
+  parentItemlocdist.bindValue(":orderType", _order->type());
+  parentItemlocdist.bindValue(":transType", "SH");
 
   // If this is a lot/serial controlled job item, we need to post production first
-  if (altId == 1)
+  if (jobItem)
+  {
+    // Handle creation of itemlocdist records for each eligible backflush item (sql below from postProduction backflush handling)
+    XSqlQuery backflushItems;
+    backflushItems.prepare(
+      "SELECT item_number, item_fractional, itemsite_id, itemsite_item_id, womatl_id, womatl_wo_id, "
+      // issueMaterial qty = noNeg(expected - consumed)
+      " noNeg(((womatl_qtyfxd + ((:qty + wo_qtyrcv) * womatl_qtyper)) * (1 + womatl_scrap)) - "
+      "   (womatl_qtyiss + "
+      "   CASE WHEN (womatl_qtywipscrap >  ((womatl_qtyfxd + (:qty + wo_qtyrcv) * womatl_qtyper) * womatl_scrap)) "
+      "        THEN (womatl_qtyfxd + (:qty + wo_qtyrcv) * womatl_qtyper) * womatl_scrap "
+      "        ELSE womatl_qtywipscrap END)) AS qtyToIssue "
+      "FROM womatl, wo, itemsite, item "
+      "WHERE womatl_issuemethod IN ('L', 'M') "
+      " AND womatl_wo_id=wo_id "
+      " AND womatl_itemsite_id=itemsite_id "
+      " AND wo_ordid = :coitem_id "
+      " AND wo_ordtype = 'S' "
+      " AND itemsite_item_id=item_id "
+      " AND isControlledItemsite(itemsite_id) "
+      "ORDER BY womatl_id;");
+    backflushItems.bindValue(":qty", issueDetail.value("postprodqty").toDouble());
+    backflushItems.bindValue(":coitem_id", id);
+    backflushItems.exec();
+    while (backflushItems.next())
+    {
+      if (backflushItems.value("qtyToIssue").toDouble() > 0)
+      {
+        hasControlledBackflushItems = true;
+        womatlItemlocdist.prepare("SELECT createItemlocdistParent(:itemsite_id, roundQty(:item_fractional, itemuomtouom(:item_id, womatl_uom_id, NULL, :qty)) * -1, 'WO', womatl_wo_id, "
+                                  " :itemlocSeries, NULL, NULL, 'IM') AS result "
+                                  "FROM womatl "
+                                  "WHERE womatl_id = :womatl_id;");
+        womatlItemlocdist.bindValue(":itemsite_id", backflushItems.value("itemsite_id").toInt());
+        womatlItemlocdist.bindValue(":item_id", backflushItems.value("itemsite_item_id").toInt());
+        womatlItemlocdist.bindValue(":item_fractional", backflushItems.value("item_fractional").toBool());
+        womatlItemlocdist.bindValue(":womatl_id", backflushItems.value("womatl_id").toInt());
+        womatlItemlocdist.bindValue(":qty", backflushItems.value("qtyToIssue").toDouble());
+        womatlItemlocdist.bindValue(":itemlocSeries", itemlocSeries);
+        womatlItemlocdist.exec();
+        if (!womatlItemlocdist.first())
+        {
+          cleanup.exec();
+          QMessageBox::information( this, tr("Issue Line to Shipping"),
+            tr("Failed to Create an itemlocdist record for work order backflushed material item %1.")
+            .arg(backflushItems.value("item_number").toString()) );
+          return false;
+        }
+        else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Creating itemlocdist Records"),
+          womatlItemlocdist, __FILE__, __LINE__))
+        {
+          cleanup.exec();
+          return false;
+        }
+      }
+    }
+
+    // If it's a controlled job item, set the relavant params
+    if (issueDetail.value("woItemControlled").toBool())
+    {
+      parentItemlocdist.bindValue(":itemsite_id", issueDetail.value("wo_itemsite_id").toInt());
+      parentItemlocdist.bindValue(":orderitemId", issueDetail.value("wo_id").toInt());
+      parentItemlocdist.bindValue(":orderType", "WO");
+      parentItemlocdist.bindValue(":transType", "RM");
+      parentItemlocdist.bindValue(":qty", issueDetail.value("postprodqty").toDouble());
+    }
+  }
+
+  // Create the itemlocdist record if controlled item and distribute detail if controlled or controlled backflush items
+  if (controlled || (issueDetail.value("woItemControlled").toBool() && jobItem) || hasControlledBackflushItems)
+  {
+    // If controlled item, execute the sql to create the parent itemlocdist record
+    // (for WO post prod item if job, else for issue to shipping transaction).
+    if (controlled || (issueDetail.value("woItemControlled").toBool() && jobItem))
+    {
+      parentItemlocdist.exec();
+      if (!parentItemlocdist.first())
+      {
+        cleanup.exec();
+        QMessageBox::information( this, tr("Issue to Shipping"), tr("Error creating itemlocdist records for controlled item") );
+        return false;
+      }
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Creating itemlocdist Records"),
+                                parentItemlocdist, __FILE__, __LINE__))
+      {
+        cleanup.exec();
+        return false;
+      }
+    }
+
+    if (distributeInventory::SeriesAdjust(itemlocSeries, this, QString(), QDate(),
+      QDate(), true) == XDialog::Rejected)
+    {
+      cleanup.exec();
+      QMessageBox::information( this, tr("Issue to Shipping"), tr("Issue Canceled") );
+      return false;
+    }
+  }
+
+  // Wrap remaining sql in a transaction block - perform postSoItemProduction if Job item, then issue to shipping
+  issue.exec("BEGIN;");
+
+  // If Job item, postSoItemProduction
+  if (jobItem)
   {
     XSqlQuery prod;
-    prod.prepare("SELECT postSoItemProduction(:soitem_id, :ts) AS result;");
+    prod.prepare("SELECT postSoItemProduction(:soitem_id, :ts, :itemlocSeries, TRUE) AS result;");
     prod.bindValue(":soitem_id", id);
     prod.bindValue(":ts", _transDate->date());
+    prod.bindValue(":itemlocSeries", itemlocSeries);
     prod.exec();
     if (prod.first())
     {
-      itemlocSeries = prod.value("result").toInt();
+      int result = prod.value("result").toInt();
 
-      if (itemlocSeries < 0)
+      if (result < 0 || result != itemlocSeries)
       {
         rollback.exec();
-        systemError(this, storedProcErrorLookup("postProduction", itemlocSeries),
-                    __FILE__, __LINE__);
+        cleanup.exec();
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting Production"),
+                               storedProcErrorLookup("postProduction", result),
+                               __FILE__, __LINE__);
         return false;
       }
-      
-      if (itemlocSeries > 0)
+
+      // If controlled item, get the inventory history from post production trans.
+      // so we can create itemlocdist records for issue to shipping transaction and auto-distribute to them in postInvTrans.
+      if (issueDetail.value("woItemControlled").toBool())
       {
-        if (distributeInventory::SeriesAdjust(itemlocSeries, this) == XDialog::Rejected)
-        {
-          rollback.exec();
-          QMessageBox::information( this, tr("Issue to Shipping"), tr("Issue Canceled") );
-          return false;
-        }
-        
-        // Need to get the inventory history id so we can auto reverse the distribution when issuing
         prod.prepare("SELECT invhist_id "
                      "FROM invhist "
                      "WHERE ((invhist_series = :itemlocseries) "
@@ -492,77 +678,57 @@ bool issueToShipping::sIssueLineBalance(int id, int altId)
         else
         {
           rollback.exec();
-          systemError(this, tr("Inventory history not found"),
-                      __FILE__, __LINE__);
+          cleanup.exec();
+          ErrorReporter::error(QtCriticalMsg, this, tr("Error Occurred"),
+                               tr("Inventory history not found")
+                               .arg(windowTitle()),__FILE__,__LINE__);
           return false;
         }
       }
     }
-    else if (prod.lastError().type() != QSqlError::NoError)
+    else
     {
       rollback.exec();
-      systemError(this, prod.lastError().databaseText(), __FILE__, __LINE__);
+      cleanup.exec();
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting Production for Job Item"),
+        prod, __FILE__,__LINE__);
       return false;
     }
   }
 
-  issue.prepare("SELECT issueLineBalanceToShipping(:ordertype, :soitem_id, :ts, :itemlocseries, :invhist_id) AS result;");
+  // Issue to Shipping
+  issue.prepare("SELECT issueToShipping(:ordertype::text, :soitem_id, :qty, :itemlocSeries, :ts, "
+    ":invhist_id, false, true) AS result;");
   issue.bindValue(":ordertype", _order->type());
   issue.bindValue(":soitem_id", id);
-  issue.bindValue(":ts",        _transDate->date());
-  if (invhistid)
+  issue.bindValue(":qty", balance);
+  issue.bindValue(":itemlocSeries", itemlocSeries);
+  issue.bindValue(":ts", _transDate->date());
+  if (invhistid > 0)
     issue.bindValue(":invhist_id", invhistid);
-  if (itemlocSeries)
-    issue.bindValue(":itemlocseries", itemlocSeries);
   issue.exec();
+
   if (issue.first())
   {
     int result = issue.value("result").toInt();
-    if (result < 0)
+    if (result < 0 || result != itemlocSeries)
     {
       rollback.exec();
-      systemError(this, storedProcErrorLookup("issueLineBalanceToShipping", result),
-                  __FILE__, __LINE__);
+      cleanup.exec();
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Issuing Item"),
+                             storedProcErrorLookup("issueToShipping", result),
+                             __FILE__, __LINE__);
       return false;
     }
-    else if (distributeInventory::SeriesAdjust(result, this) == XDialog::Rejected)
-    {
-      rollback.exec();
-      QMessageBox::information( this, tr("Issue to Shipping"), tr("Issue Canceled") );
-      return false;
-    }
-	
-	// If Transfer Order then insert special pre-assign records for the lot/serial#
-	// so they are available when the Transfer Order is received
-	if (_order->type() == "TO")
-	{
-      XSqlQuery lsdetail;
-      lsdetail.prepare("INSERT INTO lsdetail "
-	                   "            (lsdetail_itemsite_id, lsdetail_created, lsdetail_source_type, "
-	  				   "             lsdetail_source_id, lsdetail_source_number, lsdetail_ls_id, lsdetail_qtytoassign, "
-                                           "             lsdetail_expiration, lsdetail_warrpurc ) "
-					   "SELECT invhist_itemsite_id, NOW(), 'TR', "
-					   "       :orderitemid, invhist_ordnumber, invdetail_ls_id, (invdetail_qty * -1.0), "
-                                           "       invdetail_expiration, invdetail_warrpurc "
-					   "FROM invhist JOIN invdetail ON (invdetail_invhist_id=invhist_id) "
-					   "WHERE (invhist_series=:itemlocseries);");
-      lsdetail.bindValue(":orderitemid", id);
-      lsdetail.bindValue(":itemlocseries", result);
-      lsdetail.exec();
-      if (lsdetail.lastError().type() != QSqlError::NoError)
-      {
-        rollback.exec();
-        systemError(this, lsdetail.lastError().databaseText(), __FILE__, __LINE__);
-        return false;
-      }
-	}
-	
+
     issue.exec("COMMIT;");
   }
   else if (issue.lastError().type() != QSqlError::NoError)
   {
     rollback.exec();
-    systemError(this, issue.lastError().databaseText(), __FILE__, __LINE__);
+    cleanup.exec();
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Issuing Item"),
+                         issue, __FILE__, __LINE__);
     return false;
   }
   return true;
@@ -581,8 +747,9 @@ void issueToShipping::sIssueAllBalance()
     XTreeWidgetItem *cursor = (XTreeWidgetItem*)_soitem->topLevelItem(i);
     if (sIssueLineBalance(cursor->id(),cursor->altId()))
       refresh = true;
-    else
-      break;
+    // attempt to issue all lines
+    //else
+    //  break;
   }
 
   if (refresh)
@@ -596,39 +763,17 @@ void issueToShipping::sReturnStock()
   for (int i = 0; i < selected.size(); i++)
   {
     XTreeWidgetItem *cursor = (XTreeWidgetItem*)selected[i];
-    
-    XSqlQuery rollback;
-    rollback.prepare("ROLLBACK;");
 
-    issueReturnStock.exec("BEGIN");
     issueReturnStock.prepare("SELECT returnItemShipments(:ordertype, :soitem_id, 0, :ts) AS result;");
     issueReturnStock.bindValue(":ordertype", _order->type());
     issueReturnStock.bindValue(":soitem_id", cursor->id());
     issueReturnStock.bindValue(":ts",        _transDate->date());
     issueReturnStock.exec();
-    if (issueReturnStock.first())
+    if (issueReturnStock.lastError().type() != QSqlError::NoError)
     {
-      int result = issueReturnStock.value("result").toInt();
-      if (result < 0)
-      {
-        rollback.exec();
-        systemError( this, storedProcErrorLookup("returnItemShipments", result),
-                     __FILE__, __LINE__);
-        return;
-      }
-      else if (distributeInventory::SeriesAdjust(result, this) == XDialog::Rejected)
-      {
-        rollback.exec();
-        QMessageBox::information( this, tr("Issue to Shipping"), tr("Return Canceled") );
-        return;
-      }      
-      issueReturnStock.exec("COMMIT;"); 
-    }
-    else if (issueReturnStock.lastError().type() != QSqlError::NoError)
-    {
-      rollback.exec();
-      systemError(this, issueReturnStock.lastError().databaseText(), __FILE__, __LINE__);
-      return;
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Returning Stock"),
+                           issueReturnStock, __FILE__, __LINE__);
+      continue;
     }
   }
 
@@ -648,24 +793,22 @@ void issueToShipping::sShip()
   {
     // Reset _order so that lock is released prior to shipping and potentially auto receiving
     // to avoid locking conflicts
-    _transDate->setDate(omfgThis->dbDate());
     _order->setId(-1);
     _order->setFocus();
 
     ParameterList params;
     params.append("shiphead_id", issueShip.value("shiphead_id").toInt());
+    params.append("transdate", _transDate->date());
 
     shipOrder newdlg(this, "", true);
     if (newdlg.set(params) == NoError && newdlg.exec() != XDialog::Rejected)
     {
-      //_transDate->setDate(omfgThis->dbDate());
-      //_order->setId(-1);
-      //_order->setFocus();
+      _transDate->setDate(omfgThis->dbDate());
     }
   }
-  else if (issueShip.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Issuing Item"),
+                                issueShip, __FILE__, __LINE__))
   {
-    systemError(this, issueShip.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   else
@@ -686,12 +829,12 @@ void issueToShipping::sReserveStock()
   {
     ParameterList params;
     params.append("soitem_id", ((XTreeWidgetItem *)(selected[i]))->id());
-    
+
     reserveSalesOrderItem newdlg(this, "", true);
     newdlg.set(params);
     newdlg.exec();
   }
-  
+
   sFillList();
 }
 
@@ -709,20 +852,19 @@ void issueToShipping::sReserveLineBalance()
       int result = reserveSales.value("result").toInt();
       if (result < 0)
       {
-        systemError(this, storedProcErrorLookup("reserveSoLineBalance", result) +
-                    tr("<br>Line Item %1").arg(selected[i]->text(0)),
-                    __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                               storedProcErrorLookup("reserveSoLineBalance", result),
+                               __FILE__, __LINE__);
         return;
       }
     }
-    else if (reserveSales.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                  reserveSales, __FILE__, __LINE__))
     {
-      systemError(this, tr("Line Item %1\n").arg(selected[i]->text(0)) +
-                  reserveSales.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
-  
+
   sFillList();
 }
 
@@ -740,20 +882,19 @@ void issueToShipping::sUnreserveStock()
       int result = unreserveSales.value("result").toInt();
       if (result < 0)
       {
-        systemError(this, storedProcErrorLookup("unreservedSoLineQty", result) +
-                    tr("<br>Line Item %1").arg(selected[i]->text(0)),
-                    __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                               storedProcErrorLookup("unreservedSoLineQty", result),
+                               __FILE__, __LINE__);
         return;
       }
     }
-    else if (unreserveSales.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                  unreserveSales, __FILE__, __LINE__))
     {
-      systemError(this, tr("Line Item %1\n").arg(selected[i]->text(0)) +
-                  unreserveSales.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
-  
+
   sFillList();
 }
 
@@ -769,43 +910,45 @@ void issueToShipping::sFillList()
   }
   else if (_order->isSO())
   {
-    issueFillList.prepare( "SELECT cohead_holdtype,"
-                           "       (calcSalesOrderAmt(cohead_id) - "
-                           "       COALESCE(SUM(currToCurr(aropenalloc_curr_id, cohead_curr_id,"
-                           "                               aropenalloc_amount, cohead_orderdate)),0)) AS balance "
-                           "FROM cohead LEFT OUTER JOIN aropenalloc ON (aropenalloc_doctype='S' AND"
-                           "                                            aropenalloc_doc_id=cohead_id) "
-                           "WHERE (cohead_id=:sohead_id) "
-                           "GROUP BY cohead_holdtype, cohead_id;" );
+    issueFillList.prepare("SELECT soHoldType(cohead_id) AS holdtype "
+                          "FROM cohead "
+                          "WHERE (cohead_id=:sohead_id);" );
     issueFillList.bindValue(":sohead_id", _order->id());
     issueFillList.exec();
     if (issueFillList.first())
     {
-      if ( (issueFillList.value("cohead_holdtype").toString() == "C") && (issueFillList.value("balance").toDouble() > 0.0) )
+      if (issueFillList.value("holdtype").toString() == "C")
       {
         QMessageBox::critical( this, tr("Cannot Issue Stock"),
                               storedProcErrorLookup("issuetoshipping", -12));
         _order->setId(-1);
         _order->setFocus();
       }
-      else if (issueFillList.value("cohead_holdtype").toString() == "P")
+      else if (issueFillList.value("holdtype").toString() == "P")
       {
         QMessageBox::critical( this, tr("Cannot Issue Stock"),
                               storedProcErrorLookup("issuetoshipping", -13));
         _order->setId(-1);
         _order->setFocus();
       }
-      else if (issueFillList.value("cohead_holdtype").toString() == "R")
+      else if (issueFillList.value("holdtype").toString() == "R")
       {
         QMessageBox::critical( this, tr("Cannot Issue Stock"),
                               storedProcErrorLookup("issuetoshipping", -14));
         _order->setId(-1);
         _order->setFocus();
       }
+      else if (issueFillList.value("holdtype").toString() == "T")
+      {
+        QMessageBox::critical( this, tr("Cannot Issue Stock"),
+                              storedProcErrorLookup("issuetoshipping", -16));
+        _order->setId(-1);
+        _order->setFocus();
+      }
     }
-    else if (issueFillList.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                  issueFillList, __FILE__, __LINE__))
     {
-      systemError(this, issueFillList.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     else
@@ -823,8 +966,10 @@ void issueToShipping::sFillList()
   }
   else
   {
-    systemError(this, tr("Unrecognized order type %1").arg(_order->type()),
-                __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Occurred"),
+                         tr("Window:%1:\n Unrecognized order type %2")
+                         .arg(windowTitle())
+                         .arg(_order->type()),__FILE__,__LINE__);
     return;
   }
 
@@ -833,7 +978,7 @@ void issueToShipping::sFillList()
 
   if (_metrics->boolean("EnableSOReservationsByLocation"))
     listp.append("includeReservations");
-  
+
   MetaSQLQuery listm = mqlLoad("issueToShipping", "detail");
   XSqlQuery listq = listm.toQuery(listp);
   _soitem->populate(listq, true);
@@ -843,9 +988,9 @@ void issueToShipping::sFillList()
   {
     _shipment->setText(listq.value("shiphead_number").toString());
   }
-  if (listq.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                listq, __FILE__, __LINE__))
   {
-    systemError(this, listq.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -911,9 +1056,9 @@ void issueToShipping::sBcFind()
 
   if(!issueBcFind.first())
   {
-    if (issueBcFind.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Barcode Information"),
+                                  issueBcFind, __FILE__, __LINE__))
     {
-      systemError(this, issueBcFind.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     XMessageBox::message(this, QMessageBox::Warning, tr("No Match Found"),
@@ -991,9 +1136,9 @@ void issueToShipping::sBcFind()
   // If there are no records then the order is complete
   if(!issueBcFind.first())
   {
-    if (issueBcFind.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Barcode Information"),
+                                  issueBcFind, __FILE__, __LINE__))
     {
-      systemError(this, issueBcFind.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     XMessageBox::message( this, QMessageBox::Information, tr("Order Complete"),
