@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -13,10 +13,13 @@
 #include <QSqlError>
 #include <QVariant>
 #include <QMessageBox>
+#include "guiErrorCheck.h"
 #include <QValidator>
 
 #include "inputManager.h"
 #include "distributeInventory.h"
+#include "storedProcErrorLookup.h"
+#include "errorReporter.h"
 
 issueWoMaterialItem::issueWoMaterialItem(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
@@ -133,29 +136,54 @@ void issueWoMaterialItem::sCatchItemsiteid(int pItemsiteid)
 void issueWoMaterialItem::sIssue()
 {
   XSqlQuery issueIssue;
-  if (!_transDate->isValid())
+  int itemlocSeries;
+  
+  QList<GuiErrorCheck> errors;
+  errors<< GuiErrorCheck(!_transDate->isValid(), _transDate,
+                         tr("You must enter a valid transaction date."))
+  ;
+  if (GuiErrorCheck::reportErrors(this, tr("Invalid date"), errors))
+    return;
+
+  // Stage distribution cleanup function to be called on error
+  XSqlQuery cleanup;
+  cleanup.prepare("SELECT deleteitemlocseries(:itemlocSeries, TRUE);");
+
+  // Get the parent series id
+  XSqlQuery parentSeries;
+  parentSeries.prepare("SELECT NEXTVAL('itemloc_series_seq') AS result;");
+  parentSeries.exec();
+  if (parentSeries.first() && parentSeries.value("result").toInt() > 0)
   {
-    QMessageBox::critical(this, tr("Invalid date"),
-                          tr("You must enter a valid transaction date.") );
-    _transDate->setFocus();
+    itemlocSeries = parentSeries.value("result").toInt();
+    cleanup.bindValue(":itemlocSeries", itemlocSeries);
+  }
+  else
+  {
+    ErrorReporter::error(QtCriticalMsg, this, tr("Failed to Retrieve the Next itemloc_series_seq"),
+                            parentSeries, __FILE__, __LINE__);
     return;
   }
   
-  issueIssue.prepare("SELECT itemsite_id, item_number, warehous_code, "
+  issueIssue.prepare("SELECT womatl_wo_id, womatl_id, itemsite_id, item_number, warehous_code, "
             "       (COALESCE((SELECT SUM(itemloc_qty) "
             "                    FROM itemloc "
-            "                   WHERE (itemloc_itemsite_id=itemsite_id)), 0.0) >= roundQty(item_fractional, itemuomtouom(itemsite_item_id, womatl_uom_id, NULL, :qty))) AS isqtyavail "
+            "                   WHERE (itemloc_itemsite_id=itemsite_id)), 0.0) >= roundQty(item_fractional, itemuomtouom(itemsite_item_id, womatl_uom_id, NULL, :qty))) AS isqtyavail, "
+            "       formatWoNumber(womatl_wo_id) AS wo_number, "
+            "       roundQty(item_fractional, itemuomtouom(itemsite_item_id, womatl_uom_id, NULL, :qty)) * -1 AS post_qty "
             "  FROM womatl, itemsite, item, whsinfo "
             " WHERE ((womatl_itemsite_id=itemsite_id) "
             "   AND (itemsite_item_id=item_id) "
             "   AND (itemsite_warehous_id=warehous_id) "
             "   AND (NOT ((item_type = 'R') OR (itemsite_controlmethod = 'N'))) "
-            "   AND ((itemsite_controlmethod IN ('L', 'S')) OR (itemsite_loccntrl)) "
-            "   AND (womatl_id=:womatl_id)); ");
+            "   AND isControlledItemsite(itemsite_id) "
+            "   AND (womatl_id=:womatl_id)) "
+            " ORDER BY womatl_id; ");
   issueIssue.bindValue(":womatl_id", _womatl->id());
   issueIssue.bindValue(":qty", _qtyToIssue->toDouble());
   issueIssue.exec();
-  while(issueIssue.next())
+  // How could there be more than one record??
+  if (issueIssue.first())
   {
     if(!(issueIssue.value("isqtyavail").toBool()))
     {
@@ -168,70 +196,73 @@ void issueWoMaterialItem::sIssue()
           .arg(issueIssue.value("warehous_code").toString()));
       return;
     }
+
+    // Create the parent itemlocdist record for each line item requiring distribution, call distributeInventory::seriesAdjust
+    XSqlQuery parentItemlocdist;
+    parentItemlocdist.prepare("SELECT createitemlocdistparent(:itemsite_id, :qty, 'WO', "
+                              " :orderitemId, :itemlocSeries, NULL, NULL, 'IM');");
+    parentItemlocdist.bindValue(":itemsite_id", issueIssue.value("itemsite_id").toInt());
+    parentItemlocdist.bindValue(":qty", issueIssue.value("post_qty").toDouble());
+    parentItemlocdist.bindValue(":orderitemId", issueIssue.value("womatl_id").toInt());
+    parentItemlocdist.bindValue(":itemlocSeries", itemlocSeries);
+    parentItemlocdist.exec();
+    if (parentItemlocdist.first())
+    {
+      if (distributeInventory::SeriesAdjust(itemlocSeries, this, QString(), QDate(),
+        QDate(), true) == XDialog::Rejected)
+      {
+        cleanup.exec();
+        QMessageBox::information( this, tr("Material Issue"), tr("Detail Distribution was Cancelled") );
+        return;
+      }
+    }
+    else
+    {
+      cleanup.exec();
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Creating itemlocdist Records"),
+                              parentItemlocdist, __FILE__, __LINE__);
+      return;
+    }
   }
 
+  // Wrap issueWoMaterial and INSERT in a transaction in case insert fails.
   XSqlQuery rollback;
   rollback.prepare("ROLLBACK;");
 
-  issueIssue.exec("BEGIN;");	// because of possible lot, serial, or location distribution cancelations
-  issueIssue.prepare("SELECT issueWoMaterial(:womatl_id, :qty, true, :date) AS result;");
+  // Post inventory
+  issueIssue.exec("BEGIN;");
+  issueIssue.prepare("SELECT issueWoMaterial(:womatl_id, :qty, :itemlocSeries, true, :date, true) AS result;");
   issueIssue.bindValue(":womatl_id", _womatl->id());
   issueIssue.bindValue(":qty", _qtyToIssue->toDouble());
+  issueIssue.bindValue(":itemlocSeries", itemlocSeries);
   issueIssue.bindValue(":date",  _transDate->date());
   issueIssue.exec();
   if (issueIssue.first())
   {
-    if (issueIssue.value("result").toInt() < 0)
+    int result = issueIssue.value("result").toInt();
+    if (result < 0 || result != itemlocSeries)
     {
       rollback.exec();
-      systemError( this, tr("A System Error occurred at issueWoMaterialItem::%1, Work Order ID #%2, Error #%3.")
-                         .arg(__LINE__)
-                         .arg(_wo->id())
-                         .arg(issueIssue.value("result").toInt()) );
+      cleanup.exec();
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Issuing Material To Work Order #: %1")
+                           .arg(_wo->id()),
+                            storedProcErrorLookup("issueWoMaterial", result),
+                            __FILE__, __LINE__);
       return;
     }
-    else if (distributeInventory::SeriesAdjust(issueIssue.value("result").toInt(), this) == XDialog::Rejected)
-    {
-      rollback.exec();
-      QMessageBox::information( this, tr("Material Issue"), tr("Transaction Canceled") );
-      return;
-    }
-
-    if (_metrics->boolean("LotSerialControl"))
-    {
-      // Insert special pre-assign records for the lot/serial#
-      // so they are available when the material is returned
-      XSqlQuery lsdetail;
-      lsdetail.prepare("INSERT INTO lsdetail "
-                       "            (lsdetail_itemsite_id, lsdetail_created, lsdetail_source_type, "
-                       "             lsdetail_source_id, lsdetail_source_number, lsdetail_ls_id, lsdetail_qtytoassign) "
-                       "SELECT invhist_itemsite_id, NOW(), 'IM', "
-                       "       :orderitemid, invhist_ordnumber, invdetail_ls_id, (invdetail_qty * -1.0) "
-                       "FROM invhist JOIN invdetail ON (invdetail_invhist_id=invhist_id) "
-                       "WHERE (invhist_series=:itemlocseries)"
-                       "  AND (COALESCE(invdetail_ls_id, -1) > 0);");
-      lsdetail.bindValue(":orderitemid", _womatl->id());
-      lsdetail.bindValue(":itemlocseries", issueIssue.value("result").toInt());
-      lsdetail.exec();
-      if (lsdetail.lastError().type() != QSqlError::NoError)
-      {
-        rollback.exec();
-        systemError(this, lsdetail.lastError().databaseText(), __FILE__, __LINE__);
-        return;
-      }
-    }
-
-    issueIssue.exec("COMMIT;");
   }
   else
   {
     rollback.exec();
-    systemError( this, tr("A System Error occurred at issueWoMaterialItem::%1, Work Order ID #%2.")
-                       .arg(__LINE__)
-                       .arg(_wo->id()) );
+    cleanup.exec();
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Issuing Material to Work Order #: %1")
+                         .arg(_wo->id()),
+                         issueIssue, __FILE__, __LINE__);
     return;
   }
 
+  issueIssue.exec("COMMIT;");
+
   if (_captive)
     close();
   else
@@ -275,9 +306,8 @@ void issueWoMaterialItem::sSetQOH(int pWomatlid)
       _beforeQty->setDouble(_cachedQOH);
     }
     else
-      systemError(this, tr("A System Error occurred at %1::%2.")
-                        .arg(__FILE__)
-                        .arg(__LINE__) );
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Setting Quantity On Hand Information"),
+                         qoh, __FILE__, __LINE__);
   }
   sPopulateQOH();
 }
