@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -118,6 +118,7 @@ item::item(QWidget* parent, const char* name, Qt::WindowFlags fl)
   _inventoryUOM->setType(XComboBox::UOMs);
 
   _charass->addColumn(tr("Characteristic"), _itemColumn, Qt::AlignLeft, true, "char_name" );
+  _charass->addColumn(tr("Group"),          _itemColumn, Qt::AlignLeft, true, "char_group" );
   _charass->addColumn(tr("Value"),          -1,          Qt::AlignLeft, true, "charass_value" );
   _charass->addColumn(tr("Default"),        _ynColumn*2,   Qt::AlignCenter, true, "charass_default" );
   _charass->addColumn(tr("List Price"),     _priceColumn,Qt::AlignRight, true, "charass_price" );
@@ -152,13 +153,6 @@ item::item(QWidget* parent, const char* name, Qt::WindowFlags fl)
 //  connect(moreButton, SIGNAL(clicked()), this, SLOT(sEditBOM()));
 //  _bom->findChild<QVBoxLayout*>("_formButtonsLayout")->addWidget(moreButton);
 
-  ParameterList plist;
-  if(_privileges->check("MaintainBOMs"))
-    plist.append("mode", "edit");
-  else if (_privileges->check("ViewBOMs"))
-    plist.append("mode", "view");
-  _bom->set(plist);
-
   _uomconv->addColumn(tr("Conversions/Where Used"), _itemColumn*2, Qt::AlignLeft, true, "uomname");
   _uomconv->addColumn(tr("Ratio"),      -1, Qt::AlignRight, true, "uomvalue"  );
   _uomconv->addColumn(tr("Global"),     _ynColumn*2,    Qt::AlignCenter, true, "global" );
@@ -171,7 +165,7 @@ item::item(QWidget* parent, const char* name, Qt::WindowFlags fl)
   _itemsrc->addColumn(tr("Vendor Item"), _itemColumn, Qt::AlignLeft, true, "itemsrc_vend_item_number" );
   _itemsrc->addColumn(tr("Manufacturer"), _itemColumn, Qt::AlignLeft, true, "itemsrc_manuf_name" );
   _itemsrc->addColumn(tr("Manuf. Item#"), _itemColumn, Qt::AlignLeft, true, "itemsrc_manuf_item_number" );
-  _itemsrc->addColumn(tr("Default"),      _dateColumn,   Qt::AlignCenter, true, "default");
+  _itemsrc->addColumn(tr("Default"),      _dateColumn,   Qt::AlignCenter, true, "itemsrc_default");
 
   _itemalias->addColumn(tr("Alias Number"),    _itemColumn, Qt::AlignLeft,   true, "itemalias_number"  );
   _itemalias->addColumn(tr("Account"),         _itemColumn, Qt::AlignLeft,   true, "crmacct_name"  );
@@ -190,7 +184,9 @@ item::item(QWidget* parent, const char* name, Qt::WindowFlags fl)
   _itemSite->addColumn(tr("Active"),        _dateColumn, Qt::AlignCenter, true, "itemsite_active" );
   _itemSite->addColumn(tr("Site"),          _whsColumn,  Qt::AlignCenter, true, "warehous_code" );
   _itemSite->addColumn(tr("Description"),   -1,          Qt::AlignLeft, true, "warehous_descrip"   );
-  _itemSite->addColumn(tr("Cntrl. Method"), _itemColumn, Qt::AlignCenter, true, "itemsite_controlmethod" );
+  _itemSite->addColumn(tr("Cntrl. Method"), _itemColumn, Qt::AlignCenter, true, "controlmethod" );
+  _itemSite->addColumn(tr("Cost Method"),   _itemColumn, Qt::AlignCenter, true, "costmethod" );
+  _itemSite->addColumn(tr("Avg. Cost"),     _moneyColumn, Qt::AlignRight, true, "avgcost" );
   _itemSite->setDragString("itemsiteid=");
 
   connect(omfgThis, SIGNAL(itemsitesUpdated()), SLOT(sFillListItemSites()));
@@ -290,7 +286,6 @@ enum SetResponse item::set(const ParameterList &pParams)
       itemet.exec("SELECT NEXTVAL('item_item_id_seq') AS item_id");
       if (itemet.first())
         _itemid = itemet.value("item_id").toInt();
-//  ToDo
 
       _comments->setId(_itemid);
       _documents->setId(_itemid);
@@ -318,6 +313,7 @@ enum SetResponse item::set(const ParameterList &pParams)
       setObjectName(QString("item edit %1").arg(_itemid));
       _mode = cEdit;
 
+      connect(_classcode, SIGNAL(newID(int)), this, SLOT(sDefaultItemTaxes()));
       connect(_charass, SIGNAL(valid(bool)), _editCharacteristic, SLOT(setEnabled(bool)));
       connect(_charass, SIGNAL(valid(bool)), _deleteCharacteristic, SLOT(setEnabled(bool)));
       connect(_uomconv, SIGNAL(valid(bool)), _editUOM, SLOT(setEnabled(bool)));
@@ -433,6 +429,8 @@ void item::saveCore()
                           tr("You must select an Inventory Unit of Measure for this Item before continuing."))
          << GuiErrorCheck(_classcode->id() == -1, _classcode,
                           tr("You must select a Class Code before continuing."))
+         << GuiErrorCheck(_itemNumber->text().trimmed() == "", _itemNumber,
+                          tr("You must enter an Item Number before continuing."))
   ;
   
   if (GuiErrorCheck::reportErrors(this, tr("Cannot Save Item"), errors))
@@ -476,6 +474,7 @@ void item::saveCore()
   _elements->findChild<ItemCluster*>("_item")->setId(_itemid);
   
   sHandleRightButtons();
+  sDefaultItemTaxes();
   
   emit saved(_itemid);
 }
@@ -603,6 +602,23 @@ void item::sSave()
                               tr("You may not rename this Item to the entered Item Number as it is in use by another Item."));
     }
   }
+
+  if (_metrics->boolean("EnforceUniqueBarcodes"))
+  {
+    itemSave.prepare( "SELECT EXISTS(SELECT 1 FROM item "
+                      "WHERE ((item_active) "
+                      "AND (item_upccode = :item_upccode) "
+                      "AND (item_id <> :item_id))) AS result;");
+    itemSave.bindValue(":item_id", _itemid);
+    itemSave.bindValue(":item_upccode", _upcCode->text());
+    itemSave.exec();
+    if (itemSave.first() && itemSave.value("result").toBool())
+    {
+      errors << GuiErrorCheck(true, _upcCode,
+                              tr("You may not use this Item Bar Code (%1) as it is used by another Item.")
+                                .arg(_upcCode->text()));
+    }
+  }
   
   errors << GuiErrorCheck(_disallowPlanningType && QString(_itemTypes[_itemtype->currentIndex()]) == "L", _itemtype,
                           tr("This item is part of one or more Bills of Materials and cannot be a Planning Item."))
@@ -714,9 +730,9 @@ void item::sSave()
   itemSave.exec();
   while (itemSave.next())
     knownunits.append(itemSave.value("uom_id").toString());
-  if (itemSave.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Information"),
+                                itemSave, __FILE__, __LINE__))
   {
-    systemError(this, itemSave.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -733,9 +749,9 @@ void item::sSave()
   QStringList missingunitnames;
   while (itemSave.next())
     missingunitnames.append(itemSave.value("uom_name").toString());
-  if (itemSave.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Information"),
+                                itemSave, __FILE__, __LINE__))
   {
-    systemError(this, itemSave.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   if (missingunitnames.size() > 0)
@@ -825,7 +841,8 @@ void item::sSave()
   itemSave.bindValue(":item_price_uom_id", _priceUOM->id());
   itemSave.bindValue(":item_listprice", _listprice->toDouble());
   itemSave.bindValue(":item_listcost", _listcost->toDouble());
-  itemSave.bindValue(":item_upccode", _upcCode->text());
+  if (_upcCode->text().trimmed().length() > 0)
+    itemSave.bindValue(":item_upccode", _upcCode->text());
   itemSave.bindValue(":item_active", QVariant(_active->isChecked()));
   itemSave.bindValue(":item_picklist", QVariant(_pickListItem->isChecked()));
   itemSave.bindValue(":item_fractional", QVariant(_fractional->isChecked()));
@@ -846,7 +863,8 @@ void item::sSave()
     XSqlQuery itemTrxn;
     itemTrxn.exec("ROLLBACK;");
     _inTransaction = false;
-    systemError(this, itemSave.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Information"),
+                         itemSave, __FILE__, __LINE__);
     return;
   }
 
@@ -950,12 +968,10 @@ void item::sDelete()
 void item::sFillList()
 {
   XSqlQuery itemFillList;
-  itemFillList.prepare( "SELECT charass_id, char_name, "
-             " CASE WHEN char_type < 2 THEN "
-             "   charass_value "
-             " ELSE "
-             "   formatDate(charass_value::date) "
-             "END AS charass_value, "
+  itemFillList.prepare( "SELECT charass_id, char_name, char_group, "
+             "           CASE WHEN char_type = 2 THEN formatDate(charass_value::date)"
+             "                ELSE charass_value"
+             "           END AS charass_value,"
              " charass_default, "
              " charass_price, 'salesprice' AS charass_price_xtnumericrole "
              "FROM charass, char "
@@ -1082,7 +1098,6 @@ void item::populate()
     _taxRecoverable->setChecked(item.value("item_tax_recoverable").toBool());
 
     sFillList();
-    _bom->findChild<ItemCluster*>("_item")->setId(_itemid);
     _elements->findChild<ItemCluster*>("_item")->setId(_itemid);
     sFillUOMList();
     sFillSourceList();
@@ -1154,15 +1169,14 @@ void item::sPopulateUOMs()
 void item::sHandleItemtype()
 {
   QString itemType = QString(*(_itemTypes + _itemtype->currentIndex()));
+
   bool pickList  = false;
   bool sold      = false;
   bool weight    = false;
   bool config    = false;
-  bool shipUOM   = false;
-  bool capUOM    = false;
-  bool planType  = false;
   bool purchased = false;
   bool freight   = false;
+  bool upc       = false;
   
   _configured->setEnabled(false);
 
@@ -1171,11 +1185,9 @@ void item::sHandleItemtype()
     pickList = true;
     sold     = true;
     weight   = true;
-    capUOM   = true;
-    shipUOM  = true;
-    planType = true;
     purchased = true;
     freight  = true;
+    upc      = true;
   }
 
   if (itemType == "M")
@@ -1184,20 +1196,15 @@ void item::sHandleItemtype()
     sold     = true;
     weight   = true;
     config   = true;
-    capUOM   = true;
-    shipUOM  = true;
-    planType = true;
     purchased = true;
     freight  = true;
+    upc      = true;
   }
 
-  if (itemType == "F")
-    planType = true;
-
+  // nothing to do if (itemType == "F")
+  
   if (itemType == "B")
   {
-    capUOM   = true;
-    planType = true;
     purchased = true;
     freight  = true;
   }
@@ -1207,10 +1214,8 @@ void item::sHandleItemtype()
     pickList = true;
     sold     = true;
     weight   = true;
-    capUOM   = true;
-    shipUOM  = true;
-    planType = true;
     freight  = true;
+    upc      = true;
   }
 
   if (itemType == "Y")
@@ -1218,37 +1223,31 @@ void item::sHandleItemtype()
     pickList = true;
     sold     = true;
     weight   = true;
-    capUOM   = true;
-    shipUOM  = true;
-    planType = true;
     freight  = true;
+    upc      = true;
   }
 
   if (itemType == "R")
   {
     sold     = true;
     weight   = true;
-    capUOM   = true;
-    shipUOM  = true;
     freight  = true;
     config   = true;
+    upc      = true;
   }
 
   if (itemType == "T")
   {
     pickList = true;
     weight   = true;
-    capUOM   = true;
-    shipUOM  = true;
     freight  = true;
     purchased = true;
     sold = true;
+    upc      = true;
   }
 
   if (itemType == "O")
   {
-    capUOM   = true;
-    planType = true;
     purchased = true;
     freight  = true;
   }
@@ -1256,7 +1255,6 @@ void item::sHandleItemtype()
   if (itemType == "A")
   {
     sold     = true;
-    planType = true;
     freight  = true;
   }
 
@@ -1264,6 +1262,7 @@ void item::sHandleItemtype()
   {
     sold     = true;
     weight   = true;
+    upc      = true;
     _fractional->setChecked(false);
   }
   _fractional->setEnabled(itemType!="K");
@@ -1285,6 +1284,8 @@ void item::sHandleItemtype()
 
   _freightClass->setEnabled(freight);
 
+  _upcCode->setEnabled(upc);
+
   _tab->setTabEnabled(_tab->indexOf(_sourcesTab), 
         (_privileges->check("ViewItemSources") || 
 	 _privileges->check("MaintainItemSources")) && 
@@ -1553,12 +1554,29 @@ void item::sEditBOM()
 {
   ParameterList params;
 
-  if(_privileges->check("MaintainBOMs") && _mode != cView)
-    params.append("mode", "edit");
-  else
-    params.append("mode", "view");
   params.append("item_id", _itemid);
 
+  XSqlQuery rev;
+  rev.prepare("SELECT bomhead_rev_id "
+              "  FROM bomhead "
+              " WHERE bomhead_item_id=:itemid "
+              "   AND bomhead_rev_id=getActiveRevId('BOM', :itemid);");
+  rev.bindValue(":itemid", _itemid);
+  rev.exec();
+  if (rev.first())
+  {
+    if(_privileges->check("MaintainBOMs") && _mode != cView)
+      params.append("mode", "edit");
+    else
+      params.append("mode", "view");
+    params.append("revision_id", rev.value("bomhead_rev_id").toInt());
+  }
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Fetching BOM"),
+                                rev, __FILE__, __LINE__))
+    return;
+  else
+    params.append("mode", "new");
+
   BOM *newdlg = new BOM(this);
   newdlg->set(params);
   omfgThis->handleNewWindow(newdlg);
@@ -1689,78 +1707,43 @@ void item::sDeleteItemSite()
   itemDeleteItemSite.exec();
   if (itemDeleteItemSite.first())
   {
-    switch (itemDeleteItemSite.value("result").toInt())
+    int result = itemDeleteItemSite.value("result").toInt();
+    if (result < 0)
     {
-      case -1:
-        QMessageBox::warning( this, tr("Cannot Delete Item Site"),
-                              tr( "The selected Item Site cannot be deleted as there is Inventory History posted against it.\n"
-                                  "You may edit the Item Site and deactivate it." ) );
-        return;
-
-      case -2:
-        QMessageBox::warning( this, tr("Cannot Delete Item Site"),
-                              tr( "The selected Item Site cannot be deleted as there is Work Order History posted against it.\n"
-                                  "You may edit the Item Site and deactivate it." ) );
-        return;
-
-      case -3:
-        QMessageBox::warning( this, tr("Cannot Delete Item Site"),
-                              tr( "The selected Item Site cannot be deleted as there is Sales History posted against it.\n"
-                                  "You may edit the Item Site and deactivate it." ) );
-        return;
-
-      case -4:
-        QMessageBox::warning( this, tr("Cannot Delete Item Site"),
-                              tr( "The selected Item Site cannot be deleted as there is Purchasing History posted against it.\n"
-                                  "You may edit the Item Site and deactivate it." ) );
-        return;
-
-      case -5:
-        QMessageBox::warning( this, tr("Cannot Delete Item Site"),
-                              tr( "The selected Item Site cannot be deleted as there is Planning History posted against it.\n"
-                                  "You may edit the Item Site and deactivate it." ) );
-        return;
-
-      case -9:
-        QMessageBox::warning( this, tr("Cannot Delete Item Site"),
-                              tr("The selected Item Site cannot be deleted as there is a non-zero Inventory Quantity posted againt it.") );
-        return;
-
-
-      default:
-//  ToDo
-
-      case 0:
-        sFillListItemSites();
-        break;
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Item Site"),
+                             storedProcErrorLookup("deleteItemSite", result),
+                             __FILE__, __LINE__);
+      return;
     }
   }
-//  ToDo
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Item Site"),
+                                itemDeleteItemSite, __FILE__, __LINE__))
+  {
+    return;
+  }
+
+  sFillListItemSites();
 }
 
 void item::sFillListItemSites()
 {
   XSqlQuery itemFillListItemSites;
-  QString sql( "SELECT itemsite_id, itemsite_active,"
-               "       warehous_code, warehous_descrip, "
-               "       CASE WHEN itemsite_controlmethod='R' THEN :regular"
-               "            WHEN itemsite_controlmethod='N' THEN :none"
-               "            WHEN itemsite_controlmethod='L' THEN :lotNumber"
-               "            WHEN itemsite_controlmethod='S' THEN :serialNumber"
-               "       END AS itemsite_controlmethod "
-               "FROM itemsite, item, whsinfo "
-               "WHERE ( (itemsite_item_id=item_id)"
-               " AND (itemsite_warehous_id=warehous_id) "
-               " AND (item_id=:item_id) ) "
-               "ORDER BY item_number, warehous_code;" );
+  MetaSQLQuery mql = mqlLoad("itemSites", "detail");
 
-  itemFillListItemSites.prepare(sql);
-  itemFillListItemSites.bindValue(":item_id", _itemid);
-  itemFillListItemSites.bindValue(":regular", tr("Regular"));
-  itemFillListItemSites.bindValue(":none", tr("None"));
-  itemFillListItemSites.bindValue(":lotNumber", tr("Lot #"));
-  itemFillListItemSites.bindValue(":serialNumber", tr("Serial #"));
-  itemFillListItemSites.exec();
+  ParameterList params;
+  params.append("item_id", _itemid);
+  params.append("regular", tr("Regular"));
+  params.append("none", tr("None"));
+  params.append("lot", tr("Lot #"));
+  params.append("serial", tr("Serial #"));
+  params.append("standard", tr("Standard"));
+  params.append("job", tr("Job"));
+  params.append("average", tr("Average"));
+  params.append("na", tr("N/A"));
+  params.append("never", tr("Never"));
+  params.append("showInactive");
+
+  itemFillListItemSites  = mql.toQuery(params);
   _itemSite->populate(itemFillListItemSites);
 }
 
@@ -1810,9 +1793,9 @@ void item::sFillListItemtax()
   itemFillListItemtax.bindValue(":any", tr("Any"));
   itemFillListItemtax.exec();
   _itemtax->populate(itemFillListItemtax, _itemtax->id());
-  if (itemFillListItemtax.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Information"),
+                                itemFillListItemtax, __FILE__, __LINE__))
   {
-    systemError(this, itemFillListItemtax.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -1855,14 +1838,15 @@ void item::sDeleteUOM()
     int result = itemDeleteUOM.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("deleteItemUOMConv", result),
-                  __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Item UOM Conversion Information"),
+                             storedProcErrorLookup("deleteItemUOMConv", result),
+                             __FILE__, __LINE__);
       return;
     }
   }
-  else if (itemDeleteUOM.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Dleting Item UOM Conversion Information"),
+                                itemDeleteUOM, __FILE__, __LINE__))
   {
-    systemError(this, itemDeleteUOM.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -1987,16 +1971,32 @@ bool item::checkSitePrivs(int itemsiteid)
 
 void item::sHandleButtons()
 {
-  if(_mode == cNew)
+  if (_bomButton->isChecked())
   {
     ParameterList params;
 
-    if(_privileges->check("MaintainBOMs") && _mode != cView)
-      params.append("mode", "edit");
-    else
-      params.append("mode", "view");
     params.append("item_id", _itemid);
 
+    XSqlQuery rev;
+    rev.prepare("SELECT bomhead_rev_id "
+                "  FROM bomhead "
+                " WHERE bomhead_item_id=:itemid "
+                "   AND bomhead_rev_id=getActiveRevId('BOM', :itemid);");
+    rev.bindValue(":itemid", _itemid);
+    rev.exec();
+    if (rev.first())
+    {
+      if(_privileges->check("MaintainBOMs") && _mode != cView)
+        params.append("mode", "edit");
+      else
+        params.append("mode", "view");
+      params.append("revision_id", rev.value("bomhead_rev_id").toInt());
+    }
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Fetching BOM"),
+                                  rev, __FILE__, __LINE__))
+      return;
+    else
+      params.append("mode", "new");
     _bom->set(params);
 
     _bom->sFillList();
@@ -2024,7 +2024,7 @@ void item::sHandleButtons()
 
 void item::sHandleRightButtons()
 {
-  if ((_inventoryUOM->isValid()) && (_classcode->isValid()))
+  if ((_itemNumber->text().trimmed() != "") && (_inventoryUOM->isValid()) && (_classcode->isValid()))
   {
     QString itemtype = _itemTypes[_itemtype->currentIndex()];
     if(!_privileges->check("ViewItemAvailabilityWorkbench"))
@@ -2086,22 +2086,11 @@ void item::sHandleRightButtons()
 void item::sFillSourceList()
 {
   XSqlQuery itemFillSourceList;
-  QString sql( "SELECT itemsrc_id, vend_number,"
-               "       vend_name, itemsrc_vend_item_number, "
-	       "       itemsrc_active, itemsrc_manuf_name, "
-               "       itemsrc_manuf_item_number, "
-			   "       CASE WHEN itemsrc_default = 'true' THEN 'Yes' "
-			   "       ELSE 'No' "
-			   "       END AS default "
-               "FROM item, vendinfo, itemsrc "
-               "WHERE ( (itemsrc_item_id=item_id)"
-               " AND (itemsrc_vend_id=vend_id)"
-	       " AND (itemsrc_item_id=<? value(\"item_id\") ?>) "
-               ") ORDER BY vend_number, vend_name;" );
+  MetaSQLQuery mql = mqlLoad("itemSources", "detail");
                
   ParameterList params;
-  MetaSQLQuery mql(sql);
   params.append("item_id", _itemid);
+  params.append("showInactive", true);
   itemFillSourceList = mql.toQuery(params);
   _itemsrc->populate(itemFillSourceList);
 }
@@ -2214,3 +2203,25 @@ void item::setId(int p)
   emit newId(_itemid);
 }
 
+void item::sDefaultItemTaxes()
+{
+  XSqlQuery itemDefaultTaxes;
+  itemDefaultTaxes.prepare("INSERT INTO itemtax (itemtax_item_id, itemtax_taxzone_id, itemtax_taxtype_id) "
+                           "SELECT :item_id, classcodetax_taxzone_id, classcodetax_taxtype_id "
+                           "FROM classcodetax "
+                           "WHERE classcodetax_classcode_id = :classcode_id "
+                           "AND NOT EXISTS(SELECT 1 FROM itemtax "
+                           "               WHERE itemtax_item_id=:item_id "
+                           "               AND COALESCE(itemtax_taxtype_id,-1)=COALESCE(classcodetax_taxtype_id, -1) "
+                           "               AND COALESCE(itemtax_taxzone_id, -1)=COALESCE(classcodetax_taxzone_id,-1));");
+  itemDefaultTaxes.bindValue(":item_id", _itemid);
+  itemDefaultTaxes.bindValue(":classcode_id", _classcode->id());
+  itemDefaultTaxes.exec();
+  if (itemDefaultTaxes.lastError().type() != QSqlError::NoError)
+  {
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Updating Item Taxes"),
+                         itemDefaultTaxes, __FILE__, __LINE__);
+    return;
+  }
+  sFillListItemtax();
+}
