@@ -106,13 +106,17 @@ void itemAlias::sSave()
                              "before continuing"))
      ;
 
+  _number->setText(_number->text().trimmed().toUpper());
+
   itemSave.prepare( "SELECT itemalias_id "
              "FROM itemalias "
              "WHERE ( (itemalias_item_id=:itemalias_item_id)"
+             " AND (COALESCE(itemalias_crmacct_id, -1)=COALESCE(:itemalias_crmacct_id, -1))"
              " AND (itemalias_number=:itemalias_number)"
              " AND (itemalias_id != :itemalias_id) );" );
   itemSave.bindValue(":itemalias_id", _itemaliasid);
   itemSave.bindValue(":itemalias_item_id", _itemid);
+  itemSave.bindValue(":itemalias_crmacct_id", _crmacct->id());
   itemSave.bindValue(":itemalias_number", _number->text());
   itemSave.exec();
   if (itemSave.first())
@@ -152,8 +156,11 @@ void itemAlias::sSave()
   itemSave.bindValue(":itemalias_number", _number->text().trimmed());
   if (_crmacct->id() != -1)
     itemSave.bindValue(":itemalias_crmacct_id", _crmacct->id());
-  itemSave.bindValue(":itemalias_descrip1", _descrip1->text().trimmed());
-  itemSave.bindValue(":itemalias_descrip2", _descrip2->text().trimmed());
+  if (_useDescription->isChecked())
+  {
+    itemSave.bindValue(":itemalias_descrip1", _descrip1->text().trimmed());
+    itemSave.bindValue(":itemalias_descrip2", _descrip2->text().trimmed());
+  }
   itemSave.bindValue(":itemalias_comments", _comments->toPlainText());
   itemSave.bindValue(":itemalias_usedescrip", QVariant(_useDescription->isChecked()));
   itemSave.exec();
