@@ -30,6 +30,7 @@
 #include "dspSalesHistory.h"
 #include "dspSalesOrdersByItem.h"
 #include "dspSingleLevelWhereUsed.h"
+#include "dspSingleLevelBOM.h"
 #include "item.h"
 #include "parameterwidget.h"
 
@@ -82,6 +83,13 @@ itemAvailabilityWorkbench::itemAvailabilityWorkbench(QWidget* parent, const char
   _dspSingleLevelWhereUsed->setQueryOnStartEnabled(false);
   _dspSingleLevelWhereUsed->findChild<QWidget*>("_item")->hide();
   
+  _dspSingleLevelBOM = new dspSingleLevelBOM(this, "dspSingleLevelBOM", Qt::Widget);
+  _dspSingleLevelBOM->setObjectName("dspSingleLevelBOM");
+  _singleLevelBOMPage->layout()->addWidget(_dspSingleLevelBOM);
+  _dspSingleLevelBOM->setCloseVisible(false);
+  _dspSingleLevelBOM->setQueryOnStartEnabled(false);
+  _dspSingleLevelBOM->findChild<QWidget*>("_item")->hide();
+  
   _dspInventoryHistory = new dspInventoryHistory(this, "dspInventoryHistory", Qt::Widget);
   _dspInventoryHistory->setObjectName("dspInventoryHistory");
   _inventoryHistoryPage->layout()->addWidget(_dspInventoryHistory);
@@ -127,6 +135,7 @@ itemAvailabilityWorkbench::itemAvailabilityWorkbench(QWidget* parent, const char
   _dspSalesOrdersByItem->setCloseVisible(false);
   _dspSalesOrdersByItem->setQueryOnStartEnabled(false);
   _dspSalesOrdersByItem->findChild<QWidget*>("_item")->hide();
+  _dspSalesOrdersByItem->findChild<DateCluster*>("_dates")->setStartDate(QDate().currentDate().addDays(-30));
   
   _dspQuotesByItem = new dspQuotesByItem(this, "dspQuotesByItem", Qt::Widget);
   _dspQuotesByItem->setObjectName("dspQuotesByItem");
@@ -182,11 +191,13 @@ itemAvailabilityWorkbench::itemAvailabilityWorkbench(QWidget* parent, const char
   _itemMaster->findChild<QWidget*>("_sold")->setEnabled(false);
   _itemMaster->findChild<QWidget*>("_itemGroup")->setEnabled(false);
   _itemMaster->findChild<QWidget*>("_weightGroup")->setEnabled(false);
+  _itemMaster->findChild<QWidget*>("_remarksTab")->findChild<Comments*>("_comments")->setReadOnly(true);
   
   connect(_tab, SIGNAL(currentChanged(int)), this, SLOT(sFillList()));
   connect(_availabilityButton, SIGNAL(clicked()), this, SLOT(sHandleButtons()));
   connect(_runningAvailabilityButton, SIGNAL(clicked()), this, SLOT(sHandleButtons()));
   connect(_locationDetailButton, SIGNAL(clicked()), this, SLOT(sHandleButtons()));
+  connect(_singleLevelBOMButton, SIGNAL(clicked()), this, SLOT(sHandleButtons()));
   connect(_costedIndentedBOMButton, SIGNAL(clicked()), this, SLOT(sHandleButtons()));
   connect(_whereUsedButton, SIGNAL(clicked()), this, SLOT(sHandleButtons()));
   connect(_inventoryHistoryButton, SIGNAL(clicked()), this, SLOT(sHandleButtons()));
@@ -200,7 +211,7 @@ itemAvailabilityWorkbench::itemAvailabilityWorkbench(QWidget* parent, const char
 
   // General
   if (!_privileges->check("ViewInventoryAvailability") && !_privileges->check("ViewQOH"))
-    _tab->removeTab(0);
+    _tab->removeTab(_tab->indexOf(_availabilityTab));
   else
   {
     if (!_privileges->check("ViewInventoryAvailability"))
@@ -213,7 +224,7 @@ itemAvailabilityWorkbench::itemAvailabilityWorkbench(QWidget* parent, const char
   }
   
   if (!_privileges->check("ViewPurchaseOrders") && !_privileges->check("ViewSalesOrders") && !_privileges->check("ViewQuotes"))
-    _tab->removeTab(1);
+    _tab->removeTab(_tab->indexOf(_ordersTab));
   else
   {
     if (!_privileges->check("ViewPurchaseOrders"))
@@ -225,7 +236,7 @@ itemAvailabilityWorkbench::itemAvailabilityWorkbench(QWidget* parent, const char
   }
   
   if (!_privileges->check("ViewInventoryHistory") && !_privileges->check("ViewReceiptsReturns") && !_privileges->check("ViewSalesHistory"))
-    _tab->removeTab(2);
+    _tab->removeTab(_tab->indexOf(_historyTab));
   else
   {
     if (!_privileges->check("ViewInventoryHistory"))
@@ -236,17 +247,22 @@ itemAvailabilityWorkbench::itemAvailabilityWorkbench(QWidget* parent, const char
       _salesHistoryButton->hide();
   }
 
-  if (!_privileges->check("ViewItemMaster") && !_privileges->check("MaintainItemMasters"))
-    _tab->removeTab(3);
+  if (!_privileges->check("ViewItemMasters MaintainItemMasters"))
+    _tab->removeTab(_tab->indexOf(_itemTab));
 
   if (!_privileges->check("ViewCosts") && !_privileges->check("ViewBOMs"))
-    _tab->removeTab(4);
+    _tab->removeTab(_tab->indexOf(_bomTab));
   else
   {
     if (!_privileges->check("ViewCosts"))
       _costedIndentedBOMButton->hide();
     if (!_privileges->check("ViewBOMs"))
+    {
+      _singleLevelBOMButton->hide();
       _whereUsedButton->hide();
+      _costedIndentedBOMButton->setChecked(true);
+      sHandleButtons();
+    }
   }
 }
 
@@ -276,32 +292,34 @@ enum SetResponse itemAvailabilityWorkbench::set( const ParameterList & pParams )
 void itemAvailabilityWorkbench::sHandleButtons()
 {
   if (_availabilityButton->isChecked())
-    _availabilityStack->setCurrentIndex(0);
+    _availabilityStack->setCurrentWidget(_availabilityPage);
   else if (_runningAvailabilityButton->isChecked())
-    _availabilityStack->setCurrentIndex(1);
+    _availabilityStack->setCurrentWidget(_runningAvailabilityPage);
   else if (_locationDetailButton->isChecked())
-    _availabilityStack->setCurrentIndex(2);
+    _availabilityStack->setCurrentWidget(_locationDetailPage);
   
-  if (_costedIndentedBOMButton->isChecked())
-    _bomStack->setCurrentIndex(0);
+  if (_singleLevelBOMButton->isChecked())
+    _bomStack->setCurrentWidget(_singleLevelBOMPage);
+  else if (_costedIndentedBOMButton->isChecked())
+    _bomStack->setCurrentWidget(_costedIndentedBOMPage);
   else if (_whereUsedButton->isChecked())
-    _bomStack->setCurrentIndex(1);
+    _bomStack->setCurrentWidget(_whereUsedPage);
 
   if (_inventoryHistoryButton->isChecked())
-    _historyStack->setCurrentIndex(0);
+    _historyStack->setCurrentWidget(_inventoryHistoryPage);
   else if (_receivingHistoryButton->isChecked())
-    _historyStack->setCurrentIndex(1);
+    _historyStack->setCurrentWidget(_receivingHistoryPage);
   else if (_salesHistoryButton->isChecked())
-    _historyStack->setCurrentIndex(2);
+    _historyStack->setCurrentWidget(_salesHistoryPage);
   
   if (_purchaseOrderItemsButton->isChecked())
-    _ordersStack->setCurrentIndex(0);
+    _ordersStack->setCurrentWidget(_purchaseOrderItemsPage);
   else if (_salesOrderItemsButton->isChecked())
-    _ordersStack->setCurrentIndex(1);
+    _ordersStack->setCurrentWidget(_salesOrderItemsPage);
   else if (_quoteItemsButton->isChecked())
-    _ordersStack->setCurrentIndex(2);
+    _ordersStack->setCurrentWidget(_quoteItemsPage);
   else if (_customerPricesButton->isChecked())
-    _ordersStack->setCurrentIndex(3);
+    _ordersStack->setCurrentWidget(_customerPricesPage);
   
   sFillList();
 }
@@ -311,6 +329,7 @@ void itemAvailabilityWorkbench::populate()
   _dspInventoryAvailability->setItemId(_item->id());
   _dspRunningAvailability->findChild<ItemCluster*>("_item")->setId(_item->id());
   _dspInventoryLocator->findChild<ItemCluster*>("_item")->setId(_item->id());
+  _dspSingleLevelBOM->findChild<ItemCluster*>("_item")->setId(_item->id());
   _dspCostedIndentedBOM->findChild<ItemCluster*>("_item")->setId(_item->id());
   _dspSingleLevelWhereUsed->findChild<ItemCluster*>("_item")->setId(_item->id());
   _dspInventoryHistory->setItemId(_item->id());
@@ -353,7 +372,14 @@ void itemAvailabilityWorkbench::sFillList()
     if (_costedIndentedBOMButton->isChecked())
       _dspCostedIndentedBOM->sFillList();
     else if (_whereUsedButton->isChecked())
-      _dspSingleLevelWhereUsed->sFillList();
+    {
+      if (_dspSingleLevelWhereUsed->findChild<ItemCluster*>("_item")->isValid())
+        _dspSingleLevelWhereUsed->sFillList();
+      else
+        _dspSingleLevelWhereUsed->list()->clear();
+    }
+    else if (_singleLevelBOMButton->isChecked())
+      _dspSingleLevelBOM->sFillList();
   }
   else if (_tab->currentIndex() == _tab->indexOf(_historyTab))
   {
@@ -366,6 +392,12 @@ void itemAvailabilityWorkbench::sFillList()
   }
   else if (_tab->currentIndex() == _tab->indexOf(_ordersTab))
   {
+      _salesOrderItemsButton->setEnabled(_sold);
+      _quoteItemsButton->setEnabled(_sold);
+      _customerPricesButton->setEnabled(_sold);
+    if (!_sold){
+      _purchaseOrderItemsButton->setChecked(true);
+    }
     if (_purchaseOrderItemsButton->isChecked())
       _dspPoItemsByItem->sFillList();
     else if (_salesOrderItemsButton->isChecked() && _sold)
