@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -13,8 +13,11 @@
 #include <QVariant>
 #include <QMessageBox>
 #include <QSqlError>
+#include <metasql.h>
 #include <parameter.h>
 #include "itemPricingSchedule.h"
+#include "errorReporter.h"
+#include "mqlutil.h"
 
 itemPricingSchedules::itemPricingSchedules(QWidget* parent, const char* name, Qt::WindowFlags fl)
     : XWidget(parent, name, fl)
@@ -30,7 +33,6 @@ itemPricingSchedules::itemPricingSchedules(QWidget* parent, const char* name, Qt
   connect(_searchFor, SIGNAL(textChanged(const QString&)), this, SLOT(sSearch(const QString&)));
   connect(_close, SIGNAL(clicked()), this, SLOT(close()));
   connect(_showFuture, SIGNAL(toggled(bool)), this, SLOT(sFillList()));
-  connect(_ipshead, SIGNAL(valid(bool)), _view, SLOT(setEnabled(bool)));
   connect(_view, SIGNAL(clicked()), this, SLOT(sView()));
   connect(_copy, SIGNAL(clicked()), this, SLOT(sCopy()));
 
@@ -41,18 +43,21 @@ itemPricingSchedules::itemPricingSchedules(QWidget* parent, const char* name, Qt
 
   if (_privileges->check("MaintainPricingSchedules"))
   {
+    _new->setEnabled(true);
     connect(_ipshead, SIGNAL(valid(bool)), _edit, SLOT(setEnabled(bool)));
     connect(_ipshead, SIGNAL(valid(bool)), _copy, SLOT(setEnabled(bool)));
     connect(_ipshead, SIGNAL(valid(bool)), _delete, SLOT(setEnabled(bool)));
-    connect(_ipshead, SIGNAL(itemSelected(int)), _edit, SLOT(animateClick()));
   }
   else
-  {
     _new->setEnabled(false);
+
+  if (_privileges->check("MaintainPricingSchedules ViewPricingSchedules"))
+  {
+    connect(_ipshead, SIGNAL(valid(bool)), _view, SLOT(setEnabled(bool)));
     connect(_ipshead, SIGNAL(itemSelected(int)), _view, SLOT(animateClick()));
   }
 
-  sFillList();
+  _listpricesched = false;
 }
 
 itemPricingSchedules::~itemPricingSchedules()
@@ -65,10 +70,29 @@ void itemPricingSchedules::languageChange()
   retranslateUi(this);
 }
 
+enum SetResponse itemPricingSchedules::set(const ParameterList &pParams)
+{
+  bool     valid;
+  QVariant param;
+  
+  param = pParams.value("listpricesched", &valid);
+  if (valid)
+  {
+    _listpricesched = true;
+    setWindowTitle(tr("List Pricing Schedules"));
+  }
+  
+  sFillList();
+  
+  return NoError;
+}
+
 void itemPricingSchedules::sNew()
 {
   ParameterList params;
   params.append("mode", "new");
+  if (_listpricesched)
+    params.append("listpricesched", true);
 
   itemPricingSchedule newdlg(this, "", true);
   newdlg.set(params);
@@ -83,6 +107,8 @@ void itemPricingSchedules::sEdit()
   ParameterList params;
   params.append("mode", "edit");
   params.append("ipshead_id", _ipshead->id());
+  if (_listpricesched)
+    params.append("listpricesched", true);
 
   itemPricingSchedule newdlg(this, "", true);
   newdlg.set(params);
@@ -111,8 +137,8 @@ void itemPricingSchedules::sCopy()
     sFillList();
   }
   else if (itemCopy.lastError().type() != QSqlError::NoError)
-	systemError(this,itemCopy.lastError().databaseText(),
-                  __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Copying Item Pricing Schedule"),
+                       itemCopy, __FILE__, __LINE__);
 }
 
 void itemPricingSchedules::sView()
@@ -184,7 +210,8 @@ void itemPricingSchedules::sDeleteExpired()
   itemDeleteExpired.prepare("SELECT deleteexpiredips() AS result;");
   itemDeleteExpired.exec();
   if (itemDeleteExpired.lastError().type() != QSqlError::NoError)
-	systemError(this,itemDeleteExpired.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Item Pricing Schedule"),
+                       itemDeleteExpired, __FILE__, __LINE__);
 	
   sFillList();
 }
@@ -196,36 +223,21 @@ void itemPricingSchedules::sFillList()
 
 void itemPricingSchedules::sFillList(int pIpsheadid)
 {
-  QString sql( "SELECT ipshead_id, ipshead_name, ipshead_descrip,"
-               "       ipshead_effective, ipshead_expires,"
-               "       CASE WHEN COALESCE(ipshead_effective, startOfTime()) <= startOfTime() THEN :always END AS ipshead_effective_qtdisplayrole,"
-               "       CASE WHEN COALESCE(ipshead_expires, endOfTime()) >= endOfTime() THEN :never END AS ipshead_expires_qtdisplayrole "
-               "FROM ipshead " );
-
+  ParameterList params;
+  
   if (!_showExpired->isChecked())
-  {
-    sql += "WHERE ( (ipshead_expires > CURRENT_DATE)";
-
-    if (!_showFuture->isChecked())
-      sql += " AND (ipshead_effective <= CURRENT_DATE) ) ";
-    else
-      sql += " ) ";
-  }
-  else if (!_showFuture->isChecked())
-    sql += "WHERE (ipshead_effective <= CURRENT_DATE) ";
-
-  sql += "ORDER BY ipshead_name, ipshead_effective;";
-
-  XSqlQuery r;
-  r.prepare(sql);
-  r.bindValue(":always", tr("Always"));
-  r.bindValue(":never", tr("Never"));
-  r.exec();
+    params.append("showUnexpired", true);
+  if (!_showFuture->isChecked())
+    params.append("showUneffective", true);
+  if (_listpricesched)
+    params.append("listpricesched", true);
 
+  MetaSQLQuery mql = mqlLoad("pricingSchedules", "detail");
+  XSqlQuery qry = mql.toQuery(params);
   if (pIpsheadid == -1)
-    _ipshead->populate(r);
+    _ipshead->populate(qry);
   else
-    _ipshead->populate(r, pIpsheadid);
+    _ipshead->populate(qry, pIpsheadid);
 }
 
 void itemPricingSchedules::sSearch( const QString & pTarget)
