@@ -68,6 +68,8 @@ private:
     char _itemType;
     bool _wasLotSerial;
     bool _wasLocationControl;
+    bool _wasCostJob;
+    QString _wasControlMethod;
 
 };
 
