@@ -17,6 +17,8 @@
 #include <parameter.h>
 #include "location.h"
 #include <metasql.h>
+#include "errorReporter.h"
+#include "storedProcErrorLookup.h"
 
 locations::locations(QWidget* parent, const char* name, Qt::WindowFlags fl)
   : XWidget(parent, name, fl)
@@ -25,16 +27,17 @@ locations::locations(QWidget* parent, const char* name, Qt::WindowFlags fl)
 
 
   // signals and slots connections
-  connect(_print, SIGNAL(clicked()), this, SLOT(sPrint()));
-  connect(_new, SIGNAL(clicked()), this, SLOT(sNew()));
-  connect(_edit, SIGNAL(clicked()), this, SLOT(sEdit()));
-  connect(_delete, SIGNAL(clicked()), this, SLOT(sDelete()));
-  connect(_close, SIGNAL(clicked()), this, SLOT(close()));
-  connect(_location, SIGNAL(valid(bool)), _view, SLOT(setEnabled(bool)));
-  connect(_view, SIGNAL(clicked()), this, SLOT(sView()));
-  connect(_warehouse, SIGNAL(updated()), this, SLOT(sFillList()));
-  connect(_warehouse, SIGNAL(updated()), this, SLOT(updateZoneList()));
-  connect(_zone, SIGNAL(currentIndexChanged(int)), this, SLOT(sFillList()));
+  connect(_print, 	SIGNAL(clicked()), this, SLOT(sPrint()));
+  connect(_new, 	SIGNAL(clicked()), this, SLOT(sNew()));
+  connect(_edit, 	SIGNAL(clicked()), this, SLOT(sEdit()));
+  connect(_delete, 	SIGNAL(clicked()), this, SLOT(sDelete()));
+  connect(_close, 	SIGNAL(clicked()), this, SLOT(close()));
+  connect(_location, 	SIGNAL(valid(bool)), _view, SLOT(setEnabled(bool)));
+  connect(_view, 	SIGNAL(clicked()), this, SLOT(sView()));
+  connect(_warehouse, 	SIGNAL(updated()), this, SLOT(sFillList()));
+  connect(_warehouse, 	SIGNAL(updated()), this, SLOT(updateZoneList()));
+  connect(_zone, 	SIGNAL(currentIndexChanged(int)), this, SLOT(sFillList()));
+  connect(_active, 	SIGNAL(toggled(bool)), this, SLOT(sFillList()));
 
   _location->addColumn(tr("Site"),        _whsColumn,  Qt::AlignCenter, true,  "warehous_code" );
   _location->addColumn(tr("Zone"),        -1, Qt::AlignLeft,   true,  "zone"   );
@@ -43,6 +46,7 @@ locations::locations(QWidget* parent, const char* name, Qt::WindowFlags fl)
   _location->addColumn(tr("Netable"),     80,          Qt::AlignCenter, true,  "netable" );
   _location->addColumn(tr("Usable"),      80,          Qt::AlignCenter, true,  "usable" );
   _location->addColumn(tr("Restricted"),  80,          Qt::AlignCenter, true,  "restricted" );
+  _location->addColumn(tr("Active"),      80,          Qt::AlignCenter, true,  "active" );
 
   if (_privileges->check("MaintainLocations"))
   {
@@ -117,49 +121,34 @@ void locations::sDelete()
   locationsDelete.bindValue(":location_id", _location->id());
   locationsDelete.exec();
   if (locationsDelete.first())
-  {
-    QString caption = tr("Cannot Delete Selected Location");
-    QString msg;
-
-    switch (locationsDelete.value("result").toInt())
+  {  
+    int result = locationsDelete.value("result").toInt();
+    if (result < 0)
     {
-      case -1:
-        msg = tr("<p>There are one or more Item Sites that use the selected Location as their default Location. "
-		 "You must assign the default Location for all Item Sites that use the selected Location before you may delete it.</p>" );
-        break;
-
-      case -2:
-        msg = tr("<p>There is Inventory contained in the selected Location. "
-		 "You must move all Inventory out of the selected Location and may then set its status to inactive.</p>" );
-        break;
-
-      case -3:
-        msg = tr("<p>There are one or more undistributed Location records that have been posted against the selected Locations. "
-		 "This probably indicates a system error.<br><br>"
-		 "Please contact your Systems Adminstrator to have this resolved.</p>" );
-        break;
-
-      case -4:
-        msg = tr("<p>The selected Location cannot be deleted as there has been Inventory Transaction History posted against it.</p> " );
-        break;
+       ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Location"),
+                            storedProcErrorLookup("deleteLocation", result),
+                            __FILE__, __LINE__);
+       return;
     }
-    if (! msg.isEmpty())
-    {
-      QMessageBox::critical(this, caption, msg);
-      return;
-    }
-
-    sFillList();
   }
+  else if (!ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Location"),
+                         locationsDelete, __FILE__, __LINE__))
+       return;
 
-//  ToDo
+  sFillList();
 }
 
 
 void locations::sPrint()
 {
   ParameterList params;
-  _warehouse->appendValue(params);
+  if (_warehouse->isSelected())
+    params.append("warehous_id", _warehouse->id());
+  if (_zone->id() > 0)
+    params.append("zone_id", _zone->id());
+  if (_active->isChecked())
+    params.append("showInactive", true);
+
   orReport report("WarehouseLocationMasterList", params);
   if (report.isValid())
     report.print();
@@ -175,7 +164,8 @@ void locations::sFillList()
                "       firstLine(location_descrip) AS locationname,"
                "       formatBoolYN(location_netable) AS netable,"
                "       formatBoolYN(location_usable) AS usable,"
-               "       formatBoolYN(location_restrict) AS restricted "
+               "       formatBoolYN(location_restrict) AS restricted, "
+               "       formatBoolYN(location_active) AS active "
                "FROM location  "
                " JOIN whsinfo ON (location_warehous_id=warehous_id) "
                " LEFT OUTER JOIN whsezone ON (location_whsezone_id=whsezone_id) "
@@ -185,6 +175,9 @@ void locations::sFillList()
                " <? endif ?> "
                " <? if exists('zone_id') ?>"
                " AND (location_whsezone_id=<? value('zone_id') ?>) "
+               " <? endif ?> "
+               " <? if not exists('showInactive') ?> "
+               "   AND (location_active) "
                " <? endif ?> ) "
                "ORDER BY warehous_code, locationname;");
 
@@ -195,10 +188,11 @@ void locations::sFillList()
     params.append("warehous_id", _warehouse->id());
   if (_zone->id() > 0)
     params.append("zone_id", _zone->id());
+  if (_active->isChecked())
+    params.append("showInactive", true);
 
   XSqlQuery locationsFillList = mql.toQuery(params);
-  if (locationsFillList.first())
-    _location->populate(locationsFillList);
+  _location->populate(locationsFillList);
 }
 
 void locations::updateZoneList()
@@ -217,7 +211,6 @@ void locations::updateZoneList()
     params.append("warehous_id", _warehouse->id());
 
   XSqlQuery zoneFillList = mql.toQuery(params);
-  if (zoneFillList.first())
-    _zone->populate(zoneFillList);
+  _zone->populate(zoneFillList);
 }
 
