@@ -21,7 +21,7 @@ class lotSerialSequence : public XDialog, public Ui::lotSerialSequence
     Q_OBJECT
 
 public:
-    lotSerialSequence(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    lotSerialSequence(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~lotSerialSequence();
 
     virtual void populate();
