@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2013 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -16,6 +16,7 @@
 #include <QtSql>
 #include <dlineedit.h>
 #include <xcombobox.h>
+#include <QRegExpValidator>
 
 LotSerialUtils::LotSerialUtils()
 {
@@ -86,10 +87,12 @@ void LotSerialUtils::updateLotCharacteristics(int ls_id, const QList<QWidget *>
         }
 
         XSqlQuery q;
-        bool success = q.exec(QString("INSERT INTO charass "
-                              "(charass_value, charass_target_type, charass_target_id, charass_char_id) "
-                              "VALUES ('%1', 'LS', %2, %3)").arg(char_text).arg(ls_id).arg(_charIds.at(i)));
-        if (!success)
+        q.prepare("INSERT INTO charass (charass_value, charass_target_type, charass_target_id, charass_char_id)"
+                              " VALUES (:value, 'LS', :target, :char)");
+        q.bindValue(":value", char_text);
+        q.bindValue(":target", ls_id);
+        q.bindValue(":char", _charIds.at(i));
+        if (!q.exec())
         {
             qDebug() << "UpdateLotCharacteristics Error:" << q.lastError().text();
             continue;
@@ -169,19 +172,34 @@ QList<QWidget *> LotSerialUtils::addLotCharsToGridLayout(QWidget *parent, QGridL
         }
         else if (charTypes.at(i) == 1)
         {
-            XComboBox *x = new XComboBox(parent);
-            XSqlQuery q;
-            q.prepare(QString("SELECT charopt_value FROM charopt WHERE charopt_char_id=%1 ORDER BY charopt_id ASC;").arg(charIds.at(i)));
-            q.exec();
-            while(q.next()) {
-                x->addItem(q.value("charopt_value").toString());
-            }
-            edit = x;
+          XComboBox *x = new XComboBox(parent);
+          XSqlQuery q;
+          q.prepare(QString("SELECT charopt_value FROM charopt WHERE charopt_char_id=%1 ORDER BY charopt_id ASC;").arg(charIds.at(i)));
+          q.exec();
+          while(q.next()) {
+              x->addItem(q.value("charopt_value").toString());
+          }
+          edit = x;
 
         }
         else
         {
-            edit = new QLineEdit(parent);
+          XLineEdit *x = new XLineEdit(parent);
+          XSqlQuery mask;
+          mask.prepare( "SELECT COALESCE(char_mask, '') AS char_mask,"
+                        "       COALESCE(char_validator, '.*') AS char_validator "
+                        "FROM char "
+                        "WHERE (char_id=:char_id);" );
+          mask.bindValue(":char_id", charIds.at(i));
+          mask.exec();
+          if (mask.first())
+          {
+            x->setInputMask(mask.value("char_mask").toString());
+            QRegExp rx(mask.value("char_validator").toString());
+            QValidator *validator = new QRegExpValidator(rx, x);
+            x->setValidator(validator);
+          }
+          edit = x;
         }
         layout->addWidget(label, rowCount + i, 0, 0);
         layout->addWidget(edit, rowCount + i, 1, 0);
