@@ -29,6 +29,7 @@
 #include "printShippingForm.h"
 #include "issueToShipping.h"
 #include "storedProcErrorLookup.h"
+#include "errorReporter.h"
 
 maintainShipping::maintainShipping(QWidget* parent, const char* name, Qt::WindowFlags fl)
     : XWidget(parent, name, fl)
@@ -173,38 +174,17 @@ void maintainShipping::sShipOrder()
 void maintainShipping::sReturnAllOrderStock()
 {
   XSqlQuery maintainReturnAllOrderStock;
-  XSqlQuery rollback;
-  rollback.prepare("ROLLBACK;");
-
-  maintainReturnAllOrderStock.exec("BEGIN");
   maintainReturnAllOrderStock.prepare("SELECT returnCompleteShipment(:ship_id) AS result;");
   maintainReturnAllOrderStock.bindValue(":ship_id", _ship->id());
   maintainReturnAllOrderStock.exec();
-  if (maintainReturnAllOrderStock.first())
+  if (maintainReturnAllOrderStock.lastError().type() != QSqlError::NoError)
   {
-    int result = maintainReturnAllOrderStock.value("result").toInt();
-    if (result < 0)
-    {
-      rollback.exec();
-      systemError(this, storedProcErrorLookup("returnCompleteShipment", result),
-		  __FILE__, __LINE__);
-      return;
-    }
-    else if (distributeInventory::SeriesAdjust(result, this) == XDialog::Rejected)
-    {
-      rollback.exec();
-      QMessageBox::information( this, tr("Issue to Shipping"), tr("Return Canceled") );
-      return;
-    }
-    maintainReturnAllOrderStock.exec("COMMIT;"); 
-    sFillList();
-  }
-  else if (maintainReturnAllOrderStock.lastError().type() != QSqlError::NoError)
-  {
-    rollback.exec();
-    systemError(this, maintainReturnAllOrderStock.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipment Information"),
+                         maintainReturnAllOrderStock, __FILE__, __LINE__);
     return;
   }
+
+  sFillList();
 }
 
 void maintainShipping::sViewOrder()
@@ -222,9 +202,9 @@ void maintainShipping::sViewOrder()
     else if (maintainViewOrder.value("shiphead_order_type").toString() == "TO")
       transferOrder::viewTransferOrder(maintainViewOrder.value("shiphead_order_id").toInt());
   }
-  else if (maintainViewOrder.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipment Information"),
+                                maintainViewOrder, __FILE__, __LINE__))
   {
-    systemError(this, maintainViewOrder.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -242,7 +222,7 @@ void maintainShipping::sPrintShippingForm()
 void maintainShipping::sIssueStock()
 {
   ParameterList params;
-  if (_ship->currentItem()->rawValue("ordertype").toString() == "SO")
+  if (_ship->currentItem()->rawValue("ordertype").toString() == "SO Item")
 	  params.append("sohead_id", _ship->id());
   else if (_ship->currentItem()->rawValue("ordertype").toString() == "TO")
     params.append("tohead_id", _ship->id());
@@ -255,38 +235,17 @@ void maintainShipping::sIssueStock()
 void maintainShipping::sReturnAllLineStock()
 {
   XSqlQuery maintainReturnAllLineStock;
-  XSqlQuery rollback;
-  rollback.prepare("ROLLBACK;");
-
-  maintainReturnAllLineStock.exec("BEGIN");
   maintainReturnAllLineStock.prepare("SELECT returnItemShipments(:ship_id) AS result;");
   maintainReturnAllLineStock.bindValue(":ship_id", _ship->altId());
   maintainReturnAllLineStock.exec();
-  if (maintainReturnAllLineStock.first())
+  if (maintainReturnAllLineStock.lastError().type() != QSqlError::NoError)
   {
-    int result = maintainReturnAllLineStock.value("result").toInt();
-    if (maintainReturnAllLineStock.value("result").toInt() < 0)
-    {
-      rollback.exec();
-      systemError(this, storedProcErrorLookup("returnItemShipments", result),
-		  __FILE__, __LINE__);
-      return;
-    }
-    else if (distributeInventory::SeriesAdjust(result, this) == XDialog::Rejected)
-    {
-      rollback.exec();
-      QMessageBox::information( this, tr("Issue to Shipping"), tr("Return Canceled") );
-      return;
-    }    
-    maintainReturnAllLineStock.exec("COMMIT;"); 
-    sFillList();
-  }
-  else if (maintainReturnAllLineStock.lastError().type() != QSqlError::NoError)
-  {
-    rollback.exec();
-    systemError(this, maintainReturnAllLineStock.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipment Information"),
+                         maintainReturnAllLineStock, __FILE__, __LINE__);
     return;
   }
+
+  sFillList();
 }
 
 void maintainShipping::sViewLine()
@@ -314,39 +273,17 @@ void maintainShipping::sViewLine()
 void maintainShipping::sReturnAllStock()
 {
   XSqlQuery maintainReturnAllStock;
-  XSqlQuery rollback;
-  rollback.prepare("ROLLBACK;");
-
-  maintainReturnAllStock.exec("BEGIN");
   maintainReturnAllStock.prepare("SELECT returnShipmentTransaction(:ship_id) AS result;");
   maintainReturnAllStock.bindValue(":ship_id", _ship->altId());
   maintainReturnAllStock.exec();
-  if (maintainReturnAllStock.first())
+  if (maintainReturnAllStock.lastError().type() != QSqlError::NoError)
   {
-    int result = maintainReturnAllStock.value("result").toInt();
-    if (maintainReturnAllStock.value("result").toInt() < 0)
-    {
-      rollback.exec();
-      systemError(this,
-		  storedProcErrorLookup("returnShipmentTransaction", result),
-		  __FILE__, __LINE__);
-      return;
-    }
-    else if (distributeInventory::SeriesAdjust(result, this) == XDialog::Rejected)
-    {
-      rollback.exec();
-      QMessageBox::information( this, tr("Issue to Shipping"), tr("Return Canceled") );
-      return;
-    }    
-    maintainReturnAllStock.exec("COMMIT;"); 
-    sFillList();
-  }
-  else if (maintainReturnAllStock.lastError().type() != QSqlError::NoError)
-  {
-    rollback.exec();
-    systemError(this, maintainReturnAllStock.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipment Information"),
+                         maintainReturnAllStock, __FILE__, __LINE__);
     return;
   }
+
+  sFillList();
 }
 
 void maintainShipping::sFillList()
@@ -371,9 +308,9 @@ void maintainShipping::sFillList()
   maintainFillList.exec();
   _ship->populate(maintainFillList, true);
   _ship->expandAll();
-  if (maintainFillList.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipment Information"),
+                                maintainFillList, __FILE__, __LINE__))
   {
-    systemError(this, maintainFillList.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
