@@ -22,7 +22,7 @@ class massReplaceComponent : public XWidget, public Ui::massReplaceComponent
     Q_OBJECT
 
 public:
-    massReplaceComponent(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = Qt::Window);
+    massReplaceComponent(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::Window);
     ~massReplaceComponent();
 
 public slots:
