@@ -22,7 +22,7 @@ class materialReceiptTrans : public XWidget, public Ui::materialReceiptTrans
     Q_OBJECT
 
 public:
-    materialReceiptTrans(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = Qt::Window);
+    materialReceiptTrans(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::Window);
     ~materialReceiptTrans();
 
 public slots:
@@ -37,8 +37,10 @@ protected slots:
 
 private:
     int _mode;
+    int _itemsiteId;
     double _cachedQOH;
     bool _captive;
+    bool _controlledItem;
 
 };
 
