@@ -46,6 +46,7 @@ class menuSystem : public QObject
     void sHotKeys();
     void sRescanPrivileges();
     void sMaintainUsers();
+    void sUserPrivileges();
     void sMaintainGroups();
     void sNewEmployee();
     void sListEmployees();
@@ -62,6 +63,7 @@ class menuSystem : public QObject
 
     void sFixACL();
     void sFixSerial();
+    void sProcessManager();
     void sExportData();
     void sImportData();
     void sCSVAtlases();
@@ -71,8 +73,6 @@ class menuSystem : public QObject
     void sExit();
 
     void sAbout();
-    void sTOC();
-    void sDownload();
     void sReferenceGuide();
 
     void sCommunityHome();
