@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -15,6 +15,8 @@
 #include <QMessageBox>
 #include <QSqlError>
 #include <QVariant>
+#include <metasql.h>
+#include <mqlutil.h>
 
 #include "guiErrorCheck.h"
 #include "errorReporter.h"
@@ -29,7 +31,6 @@ miscVoucher::miscVoucher(QWidget* parent, const char* name, Qt::WindowFlags fl)
   connect(_amountToDistribute,   SIGNAL(valueChanged()),                 this, SLOT(sPopulateBalanceDue()));
   connect(_amountToDistribute,   SIGNAL(effectiveChanged(const QDate&)), this, SLOT(sFillMiscList()));
   connect(_amountToDistribute,   SIGNAL(idChanged(int)),                 this, SLOT(sFillMiscList()));
-  connect(_amountToDistribute,   SIGNAL(valueChanged()),                 this, SLOT(sPopulateBalanceDue()));
   connect(_delete,               SIGNAL(clicked()),                      this, SLOT(sDeleteMiscDistribution()));
   connect(_edit,                 SIGNAL(clicked()),                      this, SLOT(sEditMiscDistribution()));
   connect(_invoiceDate,          SIGNAL(newDate(const QDate&)),          this, SLOT(sPopulateDistDate()));
@@ -52,6 +53,7 @@ miscVoucher::miscVoucher(QWidget* parent, const char* name, Qt::WindowFlags fl)
   _miscDistrib->addColumn(tr("Notes"),      -1,           Qt::AlignLeft,   true,  "vodist_notes"  );
 
   _charass->setType("VCH");
+  _comments->setType(Comments::Voucher);
 }
 
 miscVoucher::~miscVoucher()
@@ -99,6 +101,7 @@ enum SetResponse miscVoucher::set(const ParameterList &pParams)
         _recurring->setParent(_voheadid, "V");
         _documents->setId(_voheadid);
         _charass->setId(_voheadid);
+        _comments->setId(_voheadid);
       }
       else if (ErrorReporter::error(QtCriticalMsg, this, tr("Creating Voucher"),
                                insq, __FILE__, __LINE__))
@@ -123,7 +126,6 @@ enum SetResponse miscVoucher::set(const ParameterList &pParams)
       _invoiceDate->setEnabled(false);
       _dueDate->setEnabled(false);
       _terms->setEnabled(false);
-      _terms->setType(XComboBox::Terms);
       _vendor->setShowInactive(true);
       _invoiceNum->setEnabled(false);
       _reference->setEnabled(false);
@@ -136,7 +138,7 @@ enum SetResponse miscVoucher::set(const ParameterList &pParams)
       _close->setText(tr("&Close"));
       _save->hide();
       _postVoucher->setVisible(false);
-
+      _comments->setReadOnly(true);
     }
   }
 
@@ -150,9 +152,13 @@ enum SetResponse miscVoucher::set(const ParameterList &pParams)
     _voheadid = param.toInt();
     _documents->setId(_voheadid);
     _charass->setId(_voheadid);
+    _comments->setId(_voheadid);
     populate();
   }
 
+  connect(_taxzone,              SIGNAL(newID(int)),                     this, SLOT(sDistributionDateUpdated()));
+  connect(_distributionDate,     SIGNAL(newDate(const QDate&)),          this, SLOT(sDistributionDateUpdated()));
+
   return NoError;
 }
 
@@ -437,6 +443,7 @@ void miscVoucher::sNewMiscDistribution()
   newdlg.set(params);
   if (newdlg.exec() != XDialog::Rejected)
   {
+    sUpdateVoucherTax();
     sFillMiscList();
     sPopulateDistributed();
   }
@@ -458,6 +465,8 @@ void miscVoucher::sEditMiscDistribution()
   newdlg.set(params);
   if (newdlg.exec() != XDialog::Rejected)
   {
+    if (_miscDistrib->altId() != 4) // Don't auto update taxes to allow manual override
+      sUpdateVoucherTax();
     sFillMiscList();
     sPopulateDistributed();
   }
@@ -473,44 +482,22 @@ void miscVoucher::sDeleteMiscDistribution()
                            delq, __FILE__, __LINE__))
     return;
 
+  sUpdateVoucherTax();
   sFillMiscList();
   sPopulateDistributed();
 }
 
 void miscVoucher::sFillMiscList()
 {
-  XSqlQuery getq;
-  getq.prepare("SELECT vodist_id,"
-             " (formatGLAccount(accnt_id) || ' - ' || accnt_descrip) AS account,"
-             "       vodist_notes,"
-             "       vodist_amount, 'curr' AS vodist_amount_xtnumericrole "
-             "FROM vodist, accnt "
-             "WHERE ( (vodist_poitem_id=-1)"
-             " AND (vodist_accnt_id=accnt_id)"
-             " AND (vodist_vohead_id=:vohead_id) ) "
-             "UNION ALL "
-             "SELECT vodist_id, (expcat_code || ' - ' || expcat_descrip) AS account,"
-             "       vodist_notes,"
-             "       vodist_amount, 'curr' AS vodist_amount_xtnumericrole "
-             "  FROM vodist, expcat "
-             " WHERE ( (vodist_poitem_id=-1)"
-             "   AND   (vodist_expcat_id=expcat_id)"
-             "   AND   (vodist_vohead_id=:vohead_id) ) "
-             "UNION ALL "
-             "SELECT vodist_id, (tax_code || ' - ' || tax_descrip) AS account,"
-             "       vodist_notes,"
-             "       vodist_amount, 'curr' AS vodist_amount_xtnumericrole "
-             "  FROM vodist, tax "
-             " WHERE ( (vodist_poitem_id=-1)"
-             "   AND   (vodist_tax_id=tax_id)"
-             "   AND   (vodist_vohead_id=:vohead_id) ) "
-             "ORDER BY account;" );
-  getq.bindValue(":vohead_id", _voheadid);
-  getq.exec();
-  _miscDistrib->populate(getq);
-  if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting Misc. Distributions"),
-                           getq, __FILE__, __LINE__))
-    return;
+  MetaSQLQuery mql = mqlLoad("voucher", "miscDistr");
+  ParameterList params;
+
+  params.append("vohead_id", _voheadid);
+  XSqlQuery miscFillList = mql.toQuery(params);
+  _miscDistrib->populate(miscFillList, true);
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Misc Distributions"),
+                              miscFillList, __FILE__, __LINE__))
+      return;
 }
 
 void miscVoucher::sPopulateDistributed()
@@ -624,7 +611,7 @@ void miscVoucher::sPopulateDistDate()
 
 void miscVoucher::sPopulateDueDate()
 {
-  if ( _invoiceDate->isValid() && _terms->isValid() && (!_dueDate->isValid()) )
+  if ( _invoiceDate->isValid() && _terms->isValid() )
   {
     XSqlQuery dateq;
     dateq.prepare("SELECT determineDueDate(:terms_id, :invoiceDate) AS duedate;");
@@ -639,6 +626,30 @@ void miscVoucher::sPopulateDueDate()
   }
 }
 
+void miscVoucher::sDistributionDateUpdated()
+{
+  sUpdateVoucherTax();
+  sFillMiscList();
+  sPopulateDistributed();
+}
+
+void miscVoucher::sUpdateVoucherTax()
+{
+  if (_amountToDistribute->localValue() <= 0 || !_taxzone->isValid() || !_distributionDate->isValid())
+    return;
+
+  XSqlQuery updTax;
+  updTax.prepare("SELECT updatemiscvouchertax(:voheadid,:taxzone,:distdate,:curr) as ret;");
+  updTax.bindValue(":voheadid", _voheadid);
+  updTax.bindValue(":taxzone",  _taxzone->id());
+  updTax.bindValue(":distdate", _distributionDate->date());
+  updTax.bindValue(":curr", _amountToDistribute->id());
+  updTax.exec();
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Adding Tax to Voucher"),
+                         updTax, __FILE__, __LINE__))
+    return;
+}
+
 void miscVoucher::keyPressEvent( QKeyEvent * e )
 {
   if(e->key() == Qt::Key_Return || e->key() == Qt::Key_Enter)
