@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -37,22 +37,41 @@ openSalesOrders::openSalesOrders(QWidget* parent, const char*, Qt::WindowFlags f
   setNewVisible(true);
   setQueryOnStartEnabled(true);
   setAutoUpdateEnabled(true);
+  setSearchVisible(true);
 
   _custid = -1;
   optionsWidget()->hide();
 
   _dates->setStartNull(tr("Earliest"), omfgThis->startOfTime(), true);
-  _dates->setStartDate(QDate().currentDate().addDays(-90));
+  int confDays = _metrics->value("DefaultOrderStartDays").toInt() ? : -90;
+  _dates->setStartDate(QDate().currentDate().addDays(confDays));
   _dates->setEndNull(tr("Latest"), omfgThis->endOfTime(), true);
 
+  QString holdSql = QString("SELECT 0 AS code, '%1' AS desc "
+                            " UNION SELECT 1, '%2' " 
+                            " UNION SELECT 2, '%3' " 
+                            " UNION SELECT 3, '%4' " 
+                            " UNION SELECT 4, '%5' "
+                            " UNION SELECT 5, '%6' "
+                            " ORDER BY code; ")
+          .arg(tr("None"))
+          .arg(tr("Credit"))
+          .arg(tr("Shipping"))
+          .arg(tr("Packing"))
+          .arg(tr("Return"))
+          .arg(tr("Tax"));
+
   if (_metrics->boolean("MultiWhs"))
     parameterWidget()->append(tr("Site"), "warehous_id", ParameterWidget::Site);
   parameterWidget()->append(tr("Customer"), "cust_id", ParameterWidget::Customer);
   parameterWidget()->appendComboBox(tr("Customer Type"), "custtype_id", XComboBox::CustomerTypes);
   parameterWidget()->append(tr("Customer Type Pattern"), "custtype_pattern", ParameterWidget::Text);
-  parameterWidget()->append(tr("P/O Number"), "poNumber", ParameterWidget::Text);
+  parameterWidget()->append(tr("Customer P/O Number"), "poNumber", ParameterWidget::Text);
   parameterWidget()->appendComboBox(tr("Sales Rep."), "salesrep_id", XComboBox::SalesRepsActive);
   parameterWidget()->appendComboBox(tr("Sale Type"), "saletype_id", XComboBox::SaleTypes);
+  parameterWidget()->append(tr("Project"), "prj_id", ParameterWidget::Project);
+  parameterWidget()->append(tr("Created By"), "createdby", ParameterWidget::User);
+  parameterWidget()->appendComboBox(tr("Hold Type"), "holdtype", holdSql);
 
   list()->addColumn(tr("Order #"),          _orderColumn,    Qt::AlignLeft,  true,  "cohead_number");
   list()->addColumn(tr("Sale Type"),        _orderColumn,    Qt::AlignLeft,  true,  "saletype_descr");
@@ -69,10 +88,10 @@ openSalesOrders::openSalesOrders(QWidget* parent, const char*, Qt::WindowFlags f
     list()->addColumn(tr("Margin %"),       _prcntColumn,    Qt::AlignRight, true,  "ordermarginpercent");
   }
   list()->addColumn(tr("Status"),           _statusColumn,   Qt::AlignCenter,false, "status");
+  list()->addColumn(tr("Hold Type"),        -1,              Qt::AlignLeft  ,false, "holdtype");
   list()->addColumn(tr("Notes"),            -1,              Qt::AlignLeft,  false, "notes");
   
   setupCharacteristics("SO");
-  parameterWidget()->applyDefaultFilterSet();
   
   if (_privileges->check("MaintainSalesOrders"))
     connect(list(), SIGNAL(itemSelected(int)), this, SLOT(sEdit()));
@@ -128,6 +147,13 @@ bool openSalesOrders::setParams(ParameterList &params)
   if (_showClosed->isChecked() && _showClosed->isVisible())
     params.append("showClosed");
 
+  params.append("none",   tr("None"));
+  params.append("credit", tr("Credit"));
+  params.append("ship",   tr("Shipping"));
+  params.append("pack",   tr("Packing"));
+  params.append("tax",    tr("Tax"));
+  params.append("return", tr("Return"));
+   
   return true;
 }
 
@@ -193,13 +219,15 @@ void openSalesOrders::sAddToPackingListBatch()
     int result = openAddToPackingListBatch.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("addToPackingListBatch", result), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Packing List Information"),
+                             storedProcErrorLookup("addToPackingListBatch", result),
+                             __FILE__, __LINE__);
       return;
     }
   }
-  else if (openAddToPackingListBatch.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Packing List Information"),
+                                openAddToPackingListBatch, __FILE__, __LINE__))
   {
-    systemError(this, openAddToPackingListBatch.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
