@@ -26,6 +26,7 @@
 #include "quoteList.h"
 #include "printQuote.h"
 #include "printSoForm.h"
+#include "guiErrorCheck.h"
 
 bool opportunity::userHasPriv(const int pMode, const int pId)
 {
@@ -112,6 +113,7 @@ opportunity::opportunity(QWidget* parent, const char* name, bool modal, Qt::Wind
   _assignedTo->setType(UsernameLineEdit::UsersActive);
 
   _saved = false;
+  _close = false;
 }
 
 /*
@@ -138,13 +140,6 @@ enum SetResponse opportunity::set(const ParameterList &pParams)
   QVariant param;
   bool     valid;
 
-  param = pParams.value("ophead_id", &valid);
-  if (valid)
-  {
-    _opheadid = param.toInt();
-    populate();
-  }
-
   param = pParams.value("mode", &valid);
   if (valid)
   {
@@ -191,35 +186,14 @@ enum SetResponse opportunity::set(const ParameterList &pParams)
       _buttonBox->setFocus();
     }
     else if (param.toString() == "view")
-    {
-      _mode = cView;
+      setViewMode();
+  }
 
-      _crmacct->setEnabled(false);
-      _owner->setEnabled(false);
-      _oppstage->setEnabled(false);
-      _oppsource->setEnabled(false);
-      _opptype->setEnabled(false);
-      _notes->setReadOnly(true);
-      _name->setEnabled(false);
-      _targetDate->setEnabled(false);
-      _actualDate->setEnabled(false);
-      _amount->setEnabled(false);
-      _probability->setEnabled(false);
-      _deleteTodoItem->setEnabled(false);
-      _editTodoItem->setEnabled(false);
-      _newTodoItem->setEnabled(false);
-      _deleteSale->setEnabled(false);
-      _editSale->setEnabled(false);
-      _printSale->setEnabled(false);
-      _newSale->setEnabled(false);
-      _attachSale->setEnabled(false);
-
-      _buttonBox->setStandardButtons(QDialogButtonBox::Close);
-      _cntct->setReadOnly(true);
-      _comments->setReadOnly(true);
-      _documents->setReadOnly(true);
-      _charass->setReadOnly(true);
-    }
+  param = pParams.value("ophead_id", &valid);
+  if (valid)
+  {
+    _opheadid = param.toInt();
+    populate();
   }
 
   param = pParams.value("crmacct_id", &valid);
@@ -234,6 +208,37 @@ enum SetResponse opportunity::set(const ParameterList &pParams)
   return NoError;
 }
 
+void opportunity::setViewMode()
+{
+  _mode = cView;
+
+  _crmacct->setEnabled(false);
+  _owner->setEnabled(false);
+  _oppstage->setEnabled(false);
+  _oppsource->setEnabled(false);
+  _opptype->setEnabled(false);
+  _notes->setReadOnly(true);
+  _name->setEnabled(false);
+  _targetDate->setEnabled(false);
+  _actualDate->setEnabled(false);
+  _amount->setEnabled(false);
+  _probability->setEnabled(false);
+  _deleteTodoItem->setEnabled(false);
+  _editTodoItem->setEnabled(false);
+  _newTodoItem->setEnabled(false);
+  _deleteSale->setEnabled(false);
+  _editSale->setEnabled(false);
+  _printSale->setEnabled(false);
+  _newSale->setEnabled(false);
+  _attachSale->setEnabled(false);
+
+  _buttonBox->setStandardButtons(QDialogButtonBox::Close);
+  _cntct->setReadOnly(true);
+  _comments->setReadOnly(true);
+  _documents->setReadOnly(true);
+  _charass->setReadOnly(true);
+}
+
 void opportunity::sCancel()
 {
   XSqlQuery cancelOppo;
@@ -287,20 +292,14 @@ bool opportunity::save(bool partial)
   XSqlQuery opportunityave;
   if (! partial)
   {
-    if(_crmacct->id() == -1)
-    {
-      QMessageBox::critical( this, tr("Incomplete Information"),
-	tr("You must specify the Account that this opportunity is for.") );
+    QList<GuiErrorCheck> errors;
+    errors<< GuiErrorCheck(_crmacct->id() == -1, _crmacct,
+                           tr("You must specify the Account that this opportunity is for."))
+          << GuiErrorCheck(_name->text().trimmed().isEmpty(), _name,
+                           tr("You must specify a Name for this opportunity report."))
+    ;
+    if (GuiErrorCheck::reportErrors(this, tr("Cannot Save Incident"), errors))
       return false;
-    }
-
-    if(_name->text().trimmed().isEmpty())
-    {
-      QMessageBox::critical( this, tr("Incomplete Information"),
-	tr("You must specify a Name for this opportunity report.") );
-      _name->setFocus();
-      return false;
-    }
   }
 
   XSqlQuery rollback;
@@ -415,6 +414,43 @@ bool opportunity::save(bool partial)
 
 void opportunity::populate()
 { 
+  if (_mode == cEdit && !_lock.acquire("ophead", _opheadid, AppLock::Interactive))
+    setViewMode();
+
+  _close = false;
+
+  foreach (QWidget* widget, QApplication::allWidgets())
+  {
+    if (!widget->isWindow() || !widget->isVisible())
+      continue;
+
+    opportunity *w = qobject_cast<opportunity*>(widget);
+
+    if (w && w != this && w->id()==_opheadid)
+    {
+      // detect "i'm my own grandpa"
+      QObject *p;
+      for (p = parent(); p && p != w ; p = p->parent())
+        ; // do nothing
+      if (p == w)
+      {
+        QMessageBox::warning(this, tr("Cannot Open Recursively"),
+                             tr("This opportunity is already open and cannot be "
+                                "raised. Please close windows to get to it."));
+        _close = true;
+      } else if (p) {
+        w->setFocus();
+        if (omfgThis->showTopLevel())
+        {
+          w->raise();
+          w->activateWindow();
+        }
+        _close = true;
+      }
+      break;
+    }
+  }
+
   XSqlQuery opportunitypopulate;
   opportunitypopulate.prepare("SELECT ophead_name,"
             "       ophead_crmacct_id,"
@@ -853,7 +889,10 @@ void opportunity::sAttachQuote()
 {
   XSqlQuery opportunityAttachQuote;
   ParameterList params;
-  params.append("cust_id", _custid);
+  if (_custid > -1)
+    params.append("cust_id", _custid);
+  else
+    params.append("cust_id", _prospectid);
   params.append("openOnly", true);
   
   quoteList newdlg(this, "", true);
@@ -1187,3 +1226,24 @@ void opportunity::sHandleAssigned()
     _assignDate->setDate(omfgThis->dbDate());
 }
 
+int opportunity::id()
+{
+  return _opheadid;
+}
+
+void opportunity::setVisible(bool visible)
+{
+  if (_close)
+    close();
+  else
+    XDialog::setVisible(visible);
+}
+
+void opportunity::done(int result)
+{
+  if (!_lock.release())
+    ErrorReporter::error(QtCriticalMsg, this, tr("Locking Error"),
+                         _lock.lastError(), __FILE__, __LINE__);
+
+  XDialog::done(result);
+}
