@@ -11,6 +11,7 @@
 #ifndef OPPORTUNITY_H
 #define OPPORTUNITY_H
 
+#include "applock.h"
 #include "guiclient.h"
 #include "xdialog.h"
 #include <parameter.h>
@@ -25,8 +26,13 @@ public:
     opportunity(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~opportunity();
 
+    static bool userHasPriv(const int = cView, const int = 0);
+
+    Q_INVOKABLE virtual int id();
+
 public slots:
     virtual SetResponse set( const ParameterList & pParams );
+    virtual void setViewMode();
     virtual void populate();
     virtual void sCancel();
     virtual void sSave();
@@ -60,6 +66,9 @@ public slots:
     virtual void sPrintSalesOrder();
     virtual void sViewSalesOrder();
     virtual void sHandleCrmacct(int);
+    virtual void done(int);
+
+    virtual void setVisible(bool);
 
 protected slots:
     virtual void languageChange();
@@ -70,6 +79,8 @@ private:
 	int		_prospectid;
     int		_mode;
     bool	_saved;
+    bool        _close;
+    AppLock     _lock;
 };
 
 #endif // OPPORTUNITY_H
