@@ -48,17 +48,13 @@ void postCashReceipts::sPost()
     journalNumber = postPost.value("journalnumber").toInt();
   else
   {
-    systemError(this, tr("A System Error occurred at %1::%2.")
-                      .arg(__FILE__)
-                      .arg(__LINE__) );
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting Cash Receipts Information"),
+                         postPost, __FILE__, __LINE__);
     return;
   }
 
-  postPost.exec( "SELECT cashrcpt_id, cust_number "
-          "FROM cashrcpt, custinfo "
-          "WHERE ( (NOT cashrcpt_posted)"
-          "  AND   (NOT cashrcpt_void)"
-          "  AND   (cashrcpt_cust_id=cust_id) );" );
+  postPost.exec( "SELECT cashrcpt_id FROM cashrcpt "
+                 "WHERE ((NOT cashrcpt_posted) AND (NOT cashrcpt_void));");
   if (postPost.first())
   {
     int counter = 0;
@@ -68,8 +64,7 @@ void postCashReceipts::sPost()
 
     do
     {
-      message( tr("Posting Cash Receipt #%1...")
-               .arg(postPost.value("cust_number").toString()) );
+      message( tr("Posting Cash Receipt...") );
 
       post.bindValue(":cashrcpt_id", postPost.value("cashrcpt_id"));
       post.bindValue(":journalNumber", journalNumber);
@@ -86,24 +81,21 @@ void postCashReceipts::sPost()
 
           case -5:
             QMessageBox::critical( this, tr("Cannot Post Cash Receipt"),
-                                   tr( "A Cash Receipt for Customer #%1 cannot be posted as the A/R Account cannot be determined.\n"
+                                   tr( "The selected Cash Receipt cannot be posted as the A/R Account cannot be determined.\n"
                                        "You must make a A/R Account Assignment for the Customer Type to which this Customer\n"
-                                       "is assigned for you may post this Cash Receipt." )
-                                   .arg(postPost.value("cust_number").toString())  );
+                                       "is assigned for you may post this Cash Receipt." ));
             break;
 
           case -6:
             QMessageBox::critical( this, tr("Cannot Post Cash Receipt"),
-                                   tr( "A Cash Receipt for Customer #%1 cannot be posted as the Bank Account cannot be determined.\n"
-                                       "You must make a Bank Account Assignment for this Cash Receipt before you may post it." )
-                                   .arg(postPost.value("cust_number").toString())  );
+                                   tr( "The selected Cash Receipt cannot be posted as the Bank Account cannot be determined.\n"
+                                       "You must make a Bank Account Assignment for this Cash Receipt before you may post it." ));
             break;
 
           case -7:
             QMessageBox::critical( this, tr("Cannot Post Cash Receipt"),
-                                   tr( "A Cash Receipt for Customer #%1 cannot be posted due to an unknown error.\n"
-                                       "Contact you Systems Administrator." )
-                                   .arg(postPost.value("cust_number").toString())  );
+                                   tr( "The selected Cash Receipt cannot be posted due to an unknown error.\n"
+                                       "Contact you Systems Administrator." ));
 
           default:
             counter++;
