@@ -21,7 +21,7 @@ class postPurchaseOrder : public XDialog, public Ui::postPurchaseOrder
     Q_OBJECT
 
 public:
-    postPurchaseOrder(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    postPurchaseOrder(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~postPurchaseOrder();
 
 public slots:
