@@ -13,6 +13,8 @@
 #include <QVariant>
 #include <QMessageBox>
 #include "glSeries.h"
+#include "errorReporter.h"
+#include "guiErrorCheck.h"
 
 postStandardJournal::postStandardJournal(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
@@ -66,13 +68,13 @@ enum SetResponse postStandardJournal::set(const ParameterList &pParams)
 void postStandardJournal::sPost()
 {
   XSqlQuery postPost;
-  if (!_distDate->isValid())
-  {
-    QMessageBox::critical( this, tr("Cannot Post Standard Journal"),
-                           tr("You must enter a Distribution Date before you may post this Standard Journal.") );
-    _distDate->setFocus();
-    return;
-  }
+
+  QList<GuiErrorCheck> errors;
+    errors<< GuiErrorCheck(!_distDate->isValid(), _distDate,
+                           tr("You must enter a Distribution Date before you may post this Standard Journal."))
+    ;
+    if (GuiErrorCheck::reportErrors(this, tr("Cannot Post Standard Journal"), errors))
+      return;
 
   postPost.prepare("SELECT postStandardJournal(:stdjrnl_id, :distDate, :reverse) AS result;");
   postPost.bindValue(":stdjrnl_id", _stdjrnl->id());
@@ -92,9 +94,8 @@ void postStandardJournal::sPost()
     newdlg.exec();
   }
   else
-    systemError(this, tr("A System Error occurred at %1::%2.")
-                      .arg(__FILE__)
-                      .arg(__LINE__) );
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting Standard Journal"),
+                       postPost, __FILE__, __LINE__);
 
   if (_captive)
     accept();
