@@ -11,8 +11,9 @@
 #include "postStandardJournalGroup.h"
 
 #include <QVariant>
-#include <QMessageBox>
+#include "guiErrorCheck.h"
 #include "glSeries.h"
+#include "errorReporter.h"
 
 postStandardJournalGroup::postStandardJournalGroup(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
@@ -66,13 +67,13 @@ enum SetResponse postStandardJournalGroup::set(const ParameterList &pParams)
 void postStandardJournalGroup::sPost()
 {
   XSqlQuery postPost;
-  if (!_distDate->isValid())
-  {
-    QMessageBox::critical( this, tr("Cannot Post Standard Journal Group"),
-                           tr("You must enter a Distribution Date before you may post this Standard Journal Group.") );
-    _distDate->setFocus();
-    return;
-  }
+
+  QList<GuiErrorCheck> errors;
+    errors<< GuiErrorCheck(!_distDate->isValid(), _distDate,
+                           tr("You must enter a Distribution Date before you may post this Standard Journal Group."))
+    ;
+    if (GuiErrorCheck::reportErrors(this, tr("Cannot Post Standard Journal Group"), errors))
+      return;
 
   postPost.prepare("SELECT postStandardJournalGroup(:stdjrnlgrp_id, :distDate, :reverse) AS result;");
   postPost.bindValue(":stdjrnlgrp_id", _stdjrnlgrp->id());
@@ -92,9 +93,8 @@ void postStandardJournalGroup::sPost()
     newdlg.exec();
   }
   else
-    systemError(this, tr("A System Error occurred at %1::%2.")
-                      .arg(__FILE__)
-                      .arg(__LINE__) );
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting Standard Journal Group"),
+                       postPost, __FILE__, __LINE__);
 
   if (_captive)
     accept();
