@@ -21,7 +21,7 @@ class postStandardJournalGroup : public XDialog, public Ui::postStandardJournalG
     Q_OBJECT
 
 public:
-    postStandardJournalGroup(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    postStandardJournalGroup(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~postStandardJournalGroup();
 
 public slots:
