@@ -11,9 +11,11 @@
 #include "postVouchers.h"
 
 #include <QMessageBox>
+#include "guiErrorCheck.h"
 #include <QSqlError>
 
 #include <openreports.h>
+#include "errorReporter.h"
 
 postVouchers::postVouchers(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
@@ -41,12 +43,13 @@ void postVouchers::sPost()
   XSqlQuery postPost;
   postPost.prepare("SELECT count(*) AS unposted FROM vohead WHERE (NOT vohead_posted)");
   postPost.exec();
-  if(postPost.first() && postPost.value("unposted").toInt()==0)
-  {
-    QMessageBox::critical( this, tr("No Vouchers to Post"),
-                           tr("There are no Vouchers to post.") );
+
+  QList<GuiErrorCheck> errors;
+  errors<< GuiErrorCheck(postPost.first() && postPost.value("unposted").toInt()==0, _post,
+                         tr("There are no Vouchers to post."))
+  ;
+  if (GuiErrorCheck::reportErrors(this, tr("No Vouchers to Post"), errors))
     return;
-  }
 
   postPost.prepare("SELECT postVouchers(false) AS result;");
   postPost.exec();
@@ -66,10 +69,8 @@ void postVouchers::sPost()
     }
     else if (result < 0)
     {
-      systemError( this, tr("A System Error occurred at %1::%2, Error #%3.")
-                         .arg(__FILE__)
-                         .arg(__LINE__)
-                         .arg(result) );
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting Voucher"),
+                           postPost, __FILE__, __LINE__);
       return;
     }
 
@@ -104,10 +105,8 @@ void postVouchers::sPost()
   }
   else
   {
-    systemError( this, tr("A System Error occurred at %1::%2.\n%3")
-                       .arg(__FILE__)
-                       .arg(__LINE__)
-                       .arg(postPost.lastError().databaseText()) );
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting Voucher"),
+                         postPost, __FILE__, __LINE__);
     return;
   }
 
