@@ -14,6 +14,7 @@
 #include <QVariant>
 
 #include "mqlutil.h"
+#include "errorReporter.h"
 
 priceList::priceList(QWidget* parent, const char * name, Qt::WindowFlags fl)
     : XDialog(parent, name, fl)
@@ -38,21 +39,17 @@ priceList::priceList(QWidget* parent, const char * name, Qt::WindowFlags fl)
   _price->addColumn(tr("Percent"),         _prcntColumn, Qt::AlignRight, true, "discountpercent");
   _price->addColumn(tr("Fixed Amt."),      _priceColumn, Qt::AlignRight, true, "discountfixed");
   _price->addColumn(tr("Type"),             _itemColumn, Qt::AlignLeft,  true, "price_type");
-  _price->addColumn(tr("Currency"),     _currencyColumn, Qt::AlignLeft,  true, "currency");
-  _price->addColumn(tr("Price (in Base)"), _priceColumn, Qt::AlignRight, true, "price");
+  _price->addColumn(tr("Currency"),     _currencyColumn, Qt::AlignLeft,  !omfgThis->singleCurrency(), "currency");
+  _price->addColumn(tr("Price (in Base)"), _priceColumn, Qt::AlignRight, !omfgThis->singleCurrency(), "price");
   // column title reset in priceList::set
-
-  if (omfgThis->singleCurrency())
-  {
-    _price->hideColumn(_price->column("price"));
-    _price->hideColumn(_price->column("currency"));
-  }
+  _price->addColumn(tr("Method"), -1, Qt::AlignRight, false, "method");
 
   _shiptoid = -1;
   _shiptonum = "";
   _prodcatid = -1;
   _custtypeid = -1;
   _custtypecode = "";
+  _listpriceschedule = "";
   _iteminvpricerat = 1.0;
 
   _qty->setValidator(omfgThis->qtyVal());
@@ -101,7 +98,13 @@ enum SetResponse priceList::set(const ParameterList &pParams)
     _warehouse->setId(param.toInt());
     _warehouse->setEnabled(false);
   }
-    
+
+  param = pParams.value("shipzone_id", &valid);
+  _shipzoneid = (valid) ? param.toInt() : -1;   
+
+  param = pParams.value("saletype_id", &valid);
+  _saletypeid = (valid) ? param.toInt() : -1;
+ 
   param = pParams.value("curr_id", &valid);
   if (valid)
   {
@@ -158,9 +161,9 @@ void priceList::sNewCust()
       _custtypeid = custq.value("custtype_id").toInt();
       _custtypecode = custq.value("custtype_code").toString();
     }
-    else if (custq.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Price List Information"),
+                                  custq, __FILE__, __LINE__))
     {
-      systemError(this, custq.lastError().text(), __FILE__, __LINE__);
       return;
     }
   }
@@ -180,9 +183,9 @@ void priceList::sNewShipto()
     {
       _shiptonum = shiptoq.value("shipto_num").toString();
     }
-    else if (shiptoq.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Price List Information"),
+                                  shiptoq, __FILE__, __LINE__))
     {
-      systemError(this, shiptoq.lastError().text(), __FILE__, __LINE__);
       return;
     }
   }
@@ -199,7 +202,9 @@ void priceList::sNewItem()
     itemq.prepare("SELECT item_listprice, item_listcost, item_prodcat_id,"
                   "       itemuomtouomratio(item_id, item_inv_uom_id, item_price_uom_id) AS iteminvpricerat,"
                   "       itemCost(:item_id, :cust_id, :shipto_id, :qty, item_inv_uom_id, item_price_uom_id,"
-                  "                :curr_id, :effective, :asof, :warehous_id) AS item_unitcost"
+                  "                :curr_id, :effective, :asof, :warehous_id) AS item_unitcost,"
+                  "       listPrice(:item_id, :cust_id, :shipto_id, :warehous_id) AS item_unitprice,"
+                  "       listPriceSchedule(:item_id, :cust_id, :shipto_id, :warehous_id) AS listpriceschedule"
                   "  FROM item LEFT OUTER JOIN itemsite ON (itemsite_item_id=item_id AND itemsite_warehous_id=:warehous_id)"
                   " WHERE (item_id=:item_id);");
     itemq.bindValue(":item_id",          _item->id());
@@ -213,15 +218,16 @@ void priceList::sNewItem()
     itemq.exec();
     if (itemq.first())
     {
-      _listPrice->setDouble(itemq.value("item_listprice").toDouble());
+      _listPrice->setDouble(itemq.value("item_unitprice").toDouble());
       _listCost->setDouble(itemq.value("item_listcost").toDouble());
       _unitCost->setDouble(itemq.value("item_unitcost").toDouble());
       _iteminvpricerat = itemq.value("iteminvpricerat").toDouble();
       _prodcatid = itemq.value("item_prodcat_id").toInt();
+      _listpriceschedule = itemq.value("listpriceschedule").toString();
     }
-    else if (itemq.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Price List Information"),
+                                  itemq, __FILE__, __LINE__))
     {
-      systemError(this, itemq.lastError().text(), __FILE__, __LINE__);
       return;
     }
   }
@@ -229,7 +235,14 @@ void priceList::sNewItem()
 
 void priceList::sSelect()
 {
-  _selectedPrice = _price->rawValue("base_price").toDouble();
+  _selectedPrice = _price->rawValue("price").toDouble();
+  _selectedMethod = _price->rawValue("method").toString();
+  _selectedType = _price->rawValue("price_type").toString();
+  _selectedSchedule = _price->rawValue("schedulename").toString();
+  _selectedBasis = _listPrice->toDouble();
+  _selectedModifierPct = _price->rawValue("discountpercent").toDouble();
+  _selectedModifierAmt = _price->rawValue("discountfixed").toDouble();
+  _selectedQtyBreak = _price->rawValue("qty_break").toDouble();
 
   accept();
 }
@@ -242,7 +255,10 @@ void priceList::sFillList()
                                           errString, &ok);
   if (! ok)
   {
-    systemError(this, errString, __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Occurred"),
+                         tr("%1: %2")
+                         .arg(windowTitle())
+                         .arg(errString),__FILE__,__LINE__);
     return;
   }
 
@@ -253,24 +269,31 @@ void priceList::sFillList()
   pricelistp.append("shipToPattern",    tr("Cust. Ship-To Pattern"));
   pricelistp.append("custType",         tr("Cust. Type"));
   pricelistp.append("custTypePattern",  tr("Cust. Type Pattern"));
+  pricelistp.append("shipZone",         tr("Shipping Zone"));
+  pricelistp.append("saleType",         tr("Sale Type"));
   pricelistp.append("sale",             tr("Sale"));
   pricelistp.append("listPrice",        tr("List Price"));
   pricelistp.append("nominal",          tr("Nominal"));
   pricelistp.append("discount",         tr("Discount"));
   pricelistp.append("markup",           tr("Markup"));
   pricelistp.append("item_id",          _item->id());
+  pricelistp.append("warehous_id",      _warehouse->id());
   pricelistp.append("prodcat_id",       _prodcatid);
   pricelistp.append("cust_id",          _cust->id());
   pricelistp.append("custtype_id",      _custtypeid);
   pricelistp.append("custtype_code",    _custtypecode);
+  pricelistp.append("saletype_id",      _saletypeid);
+  pricelistp.append("shipzone_id",      _shipzoneid);
   pricelistp.append("shipto_id",        _shiptoid);
   pricelistp.append("shipto_num",       _shiptonum);
   pricelistp.append("curr_id",          _curr_id);
   pricelistp.append("effective",        _effective);
   pricelistp.append("asof",             _asOf);
-  pricelistp.append("item_listprice",   _listPrice->toDouble());
+  pricelistp.append("qty",              _qty->toDouble());
   pricelistp.append("item_listcost",    _listCost->toDouble());
-  pricelistp.append("item_unitcost",    (_unitCost->toDouble() / _iteminvpricerat));
+  pricelistp.append("item_unitcost",    _unitCost->toDouble());
+  pricelistp.append("item_listprice",   _listPrice->toDouble());
+  pricelistp.append("listpricesched",   _listpriceschedule);
 
   XSqlQuery pricelistq = pricelistm.toQuery(pricelistp);
   _price->populate(pricelistq, true);
