@@ -13,8 +13,10 @@
 #include <QSqlError>
 #include <QVariant>
 
+#include "printChecksReviewEdit.h"
 #include "guiclient.h"
 #include "storedProcErrorLookup.h"
+#include "errorReporter.h"
 
 printChecksReview::printChecksReview(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
@@ -22,6 +24,7 @@ printChecksReview::printChecksReview(QWidget* parent, const char* name, bool mod
   setupUi(this);
 
   connect(_complete,  SIGNAL(clicked()), this, SLOT(sComplete()));
+  connect(_edit,      SIGNAL(clicked()), this, SLOT(sEdit()));
   connect(_printed,   SIGNAL(clicked()), this, SLOT(sMarkPrinted()));
   connect(_replace,   SIGNAL(clicked()), this, SLOT(sMarkReplaced()));
   connect(_selectAll, SIGNAL(clicked()), this, SLOT(sSelectAll()));
@@ -82,10 +85,13 @@ void printChecksReview::sComplete()
 	{
 	  int result = checkPrint.value("result").toInt();
 	  if (result < 0)
-	    systemError(this, storedProcErrorLookup("markCheckPrinted", result), __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Check Information"),
+                               storedProcErrorLookup("markCheckPrinted", result),
+                               __FILE__, __LINE__);
 	}
 	else if (checkPrint.lastError().type() != QSqlError::NoError)
-	  systemError(this, checkPrint.lastError().databaseText(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Check Information"),
+                         checkPrint, __FILE__, __LINE__);
         break;
       case ActionVoided:
         checkVoid.bindValue(":check_id", cursor->id());
@@ -94,10 +100,13 @@ void printChecksReview::sComplete()
 	{
 	  int result = checkVoid.value("result").toInt();
 	  if (result < 0)
-	    systemError(this, storedProcErrorLookup("voidCheck", result), __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Check Information"),
+                             storedProcErrorLookup("voidCheck", result),
+                             __FILE__, __LINE__);
 	}
 	else if (checkVoid.lastError().type() != QSqlError::NoError)
-	  systemError(this, checkVoid.lastError().databaseText(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Check Information"),
+                         checkVoid, __FILE__, __LINE__);
         break;
       case ActionReplaced:
         checkVoid.bindValue(":check_id", cursor->id());
@@ -106,20 +115,26 @@ void printChecksReview::sComplete()
 	{
 	  int result = checkVoid.value("result").toInt();
 	  if (result < 0)
-	    systemError(this, storedProcErrorLookup("voidCheck", result), __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Check Information"),
+                             storedProcErrorLookup("voidCheck", result),
+                             __FILE__, __LINE__);
 	}
 	else if (checkVoid.lastError().type() != QSqlError::NoError)
-	  systemError(this, checkVoid.lastError().databaseText(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Check Information"),
+                         checkVoid, __FILE__, __LINE__);
         checkReplace.bindValue(":check_id", cursor->id());
         checkReplace.exec();
 	if (checkReplace.first())
 	{
 	  int result = checkReplace.value("result").toInt();
 	  if (result < 0)
-	    systemError(this, storedProcErrorLookup("replaceVoidedCheck", result), __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Check Information"),
+                             storedProcErrorLookup("replaceVoidedCheck", result),
+                             __FILE__, __LINE__);
 	}
 	else if (checkReplace.lastError().type() != QSqlError::NoError)
-	  systemError(this, checkReplace.lastError().databaseText(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Check Information"),
+                         checkReplace, __FILE__, __LINE__);
         break;
     }
   }
@@ -128,6 +143,36 @@ void printChecksReview::sComplete()
   close();
 }
 
+void printChecksReview::sEdit()
+{
+  QList<XTreeWidgetItem*> selected = _checks->selectedItems();
+  for (int i = 0; i < selected.size(); i++)
+  {
+    XTreeWidgetItem *cursor = (XTreeWidgetItem*)selected[i];
+
+    ParameterList params;
+    params.append("checkhead_id", cursor->id());
+
+    printChecksReviewEdit newdlg(this, "", true);
+    newdlg.set(params);
+    newdlg.exec();
+
+    XSqlQuery checkNumber;
+    checkNumber.prepare( "SELECT checkhead_number "
+                         "FROM checkhead "
+                         "WHERE checkhead_id=:checkhead_id;");
+    checkNumber.bindValue(":checkhead_id", cursor->id());
+    checkNumber.exec();
+    if(checkNumber.first())
+    {
+      cursor->setText(0, checkNumber.value("checkhead_number").toString());
+    }
+    else if (checkNumber.lastError().type() != QSqlError::NoError)
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error fetching Check Number"),
+                           checkNumber, __FILE__, __LINE__);
+  }
+}
+
 void printChecksReview::sUnmark()
 {
   markSelected(ActionUnmark);
