@@ -20,9 +20,11 @@ class printInvoice : public printMulticopyDocument,
     Q_OBJECT
 
   public:
-    printInvoice(QWidget *parent = 0, const char *name = 0, bool modal = false, Qt::WFlags fl = 0);
+    printInvoice(QWidget *parent = 0, const char *name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~printInvoice();
 
+    Q_INVOKABLE virtual int distributeInventory(XSqlQuery *qry);
+
   protected slots:
     virtual void languageChange();
     virtual void sHandlePopulated(XSqlQuery *qry);
