@@ -20,11 +20,12 @@ class printInvoices : public printMulticopyDocument,
     Q_OBJECT
 
   public:
-    printInvoices(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    printInvoices(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~printInvoices();
 
     Q_INVOKABLE virtual ParameterList getParamsDocList();
     Q_INVOKABLE virtual ParameterList getParamsOneCopy(const int row, XSqlQuery *qry);
+    Q_INVOKABLE virtual int distributeInventory(XSqlQuery *qry);
 
   protected slots:
     virtual void languageChange();
