@@ -18,7 +18,6 @@
 #include <metasql.h>
 #include <openreports.h>
 
-#include "distributeInventory.h"
 #include "errorReporter.h"
 #include "storedProcErrorLookup.h"
 
@@ -79,8 +78,8 @@ printMulticopyDocument::printMulticopyDocument(QWidget    *parent,
           this, SLOT(sPrintOneDoc(XSqlQuery*)));
   connect(this, SIGNAL(timeToMarkOnePrinted(XSqlQuery*)),
           this, SLOT(sMarkOnePrinted(XSqlQuery*)));
-  connect(this, SIGNAL(timeToPostOneDoc(XSqlQuery*)),
-          this, SLOT(sPostOneDoc(XSqlQuery*)));
+  connect(this, SIGNAL(timeToPostOneDoc(XSqlQuery*, int)),
+          this, SLOT(sPostOneDoc(XSqlQuery*, int)));
 
   _distributeInventory = false;
 }
@@ -108,8 +107,8 @@ printMulticopyDocument::printMulticopyDocument(QString numCopiesMetric,
           this, SLOT(sPrintOneDoc(XSqlQuery*)));
   connect(this, SIGNAL(timeToMarkOnePrinted(XSqlQuery*)),
           this, SLOT(sMarkOnePrinted(XSqlQuery*)));
-  connect(this, SIGNAL(timeToPostOneDoc(XSqlQuery*)),
-          this, SLOT(sPostOneDoc(XSqlQuery*)));
+  connect(this, SIGNAL(timeToPostOneDoc(XSqlQuery*, int)),
+          this, SLOT(sPostOneDoc(XSqlQuery*, int)));
 
   _distributeInventory = false;
 
@@ -209,7 +208,7 @@ bool printMulticopyDocument::sMarkOnePrinted(XSqlQuery *docq)
   return wasMarked;
 }
 
-bool printMulticopyDocument::sPostOneDoc(XSqlQuery *docq)
+bool printMulticopyDocument::sPostOneDoc(XSqlQuery *docq, int itemlocSeries)
 {
   if (! _postQuery.isEmpty()    &&
       _data->_post->isChecked() &&
@@ -223,6 +222,13 @@ bool printMulticopyDocument::sPostOneDoc(XSqlQuery *docq)
     postp.append("docid",     docq->value("docid"));
     postp.append("docnumber", docq->value("docnumber"));
 
+    XSqlQuery cleanup; // Stage cleanup function to be called on error
+    cleanup.prepare("SELECT deleteitemlocseries(:itemlocSeries, TRUE);");
+    cleanup.bindValue(":itemlocSeries", itemlocSeries);
+  
+    if (itemlocSeries > 0)
+      postp.append("itemlocSeries", itemlocSeries);
+
     if (! _askBeforePostingQry.isEmpty())
     {
       MetaSQLQuery askm(_askBeforePostingQry);
@@ -230,14 +236,21 @@ bool printMulticopyDocument::sPostOneDoc(XSqlQuery *docq)
       if (ErrorReporter::error(QtCriticalMsg, this,
                                tr("Cannot Post %1").arg(docnumber),
                                askq, __FILE__, __LINE__))
+      {
+        cleanup.exec();
         return false;
+      }
+        
       else if (askq.value("ask").toBool() &&
                QMessageBox::question(this, tr("Post Anyway?"),
                                      _askBeforePostingMsg.arg(docnumber),
                                       QMessageBox::Yes,
                                       QMessageBox::No | QMessageBox::Default)
                   == QMessageBox::No)
+      {
+        cleanup.exec();
         return false;
+      }
     }
 
     if (! _errCheckBeforePostingQry.isEmpty())
@@ -247,13 +260,19 @@ bool printMulticopyDocument::sPostOneDoc(XSqlQuery *docq)
       if (ErrorReporter::error(QtCriticalMsg, this,
                                tr("Cannot Post %1").arg(docnumber),
                                errq, __FILE__, __LINE__))
+      {
+        cleanup.exec();
         return false;
+      }
       else if (! errq.first() || ! errq.value("ok").toBool())
       {
         ErrorReporter::error(QtCriticalMsg, this,
                              tr("Cannot Post %1").arg(docnumber),
                              _errCheckBeforePostingMsg, __FILE__, __LINE__);
-        return false;
+        {
+          cleanup.exec();
+          return false;
+        }
       }
     }
 
@@ -271,24 +290,18 @@ bool printMulticopyDocument::sPostOneDoc(XSqlQuery *docq)
       if (result < 0)
       {
         rollback.exec();
+        cleanup.exec();
         ErrorReporter::error(QtCriticalMsg, this,
                              tr("Cannot Post %1").arg(docnumber),
                              storedProcErrorLookup(_postFunction, result),
                              __FILE__, __LINE__);
         return false;
       }
-      if (_distributeInventory &&
-          (distributeInventory::SeriesAdjust(result, this) == XDialog::Rejected))
-      {
-        rollback.exec();
-        QMessageBox::information(this, tr("Posting Canceled"),
-                                 tr("Transaction Canceled") );
-        return false;
-      }
     }
     else if (postq.lastError().type() != QSqlError::NoError)
     {
       rollback.exec();
+      cleanup.exec();
       ErrorReporter::error(QtCriticalMsg, this,
                            tr("Cannot Post %1").arg(docnumber),
                            postq, __FILE__, __LINE__);
@@ -308,7 +321,7 @@ void printMulticopyDocument::sPrint()
   if (! isOkToPrint())
     return;
 
-  bool mpStartedInitialized = _data->_mpIsInitialized;
+  //bool mpStartedInitialized = _data->_mpIsInitialized;
 
   _data->_printed.clear();
 
@@ -324,7 +337,18 @@ void printMulticopyDocument::sPrint()
     emit aboutToStart(&docinfoq);
     emit timeToPrintOneDoc(&docinfoq);
     emit timeToMarkOnePrinted(&docinfoq);
-    emit timeToPostOneDoc(&docinfoq);
+
+    // Distribute inventory detail
+    int itemlocSeries = 0;
+
+    if (_distributeInventory)
+    {
+      itemlocSeries = distributeInventory(&docinfoq);
+      if (itemlocSeries <= 0)
+        return;
+    }
+    
+    emit timeToPostOneDoc(&docinfoq, itemlocSeries);
 
     message("");
   }
@@ -380,7 +404,9 @@ bool printMulticopyDocument::sPrintOneDoc(XSqlQuery *docq)
     if (orReport::beginMultiPrint(_data->_printer, userCanceled) == false)
     {
       if(!userCanceled)
-        systemError(this, tr("Could not initialize printing system for multiple reports."));
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Occurred"),
+                           tr("%1: Could not initialize printing system "
+                              "for multiple reports. ").arg(windowTitle()),__FILE__,__LINE__);
       return false;
     }
   }
@@ -455,6 +481,11 @@ XDocCopySetter *printMulticopyDocument::copies()
   return _data->_copies;
 }
 
+int printMulticopyDocument::distributeInventory(XSqlQuery *qry)
+{
+  return 0;
+}
+
 QString printMulticopyDocument::doctype()
 {
   return _data->_doctype;
@@ -519,7 +550,7 @@ void printMulticopyDocument::setDoctype(QString doctype)
   if (doctype == "AR")
     _data->_doctypefull = tr("A/R Statement");
   else if (doctype == "CM")
-    _data->_doctypefull = tr("Return");
+    _data->_doctypefull = tr("Sales Credit");
   else if (doctype == "IN")
     _data->_doctypefull = tr("Invoice");
   else if (doctype == "L")
