@@ -20,7 +20,7 @@ class printShippingForms : public printMulticopyDocument,
     Q_OBJECT
 
   public:
-    printShippingForms(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    printShippingForms(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~printShippingForms();
 
   public slots:
