@@ -11,6 +11,7 @@
 #include "printStatementByCustomer.h"
 
 #include <QMessageBox>
+#include "guiErrorCheck.h"
 #include <QVariant>
 
 #include <metasql.h>
@@ -71,6 +72,7 @@ void printStatementByCustomer::clear()
 
 ParameterList printStatementByCustomer::getParams(XSqlQuery *docq)
 {
+  Q_UNUSED(docq);
   return getParams();
 }
 
@@ -81,7 +83,7 @@ ParameterList printStatementByCustomer::getParams()
   params.append("docid",    _cust->id());
   params.append("cust_id",  _cust->id());
   params.append("invoice",  tr("Invoice"));
-  params.append("return",   tr("Return"));
+  params.append("return",   tr("Sales Credit"));
   params.append("debit",    tr("Debit Memo"));
   params.append("credit",   tr("Credit Memo"));
   params.append("deposit",  tr("Deposit"));
@@ -92,14 +94,14 @@ ParameterList printStatementByCustomer::getParams()
 
 bool printStatementByCustomer::isOkToPrint()
 {
-  if (!_cust->isValid())
-  {
-    QMessageBox::warning(this, tr("Enter a Valid Customer Number"),
-                         tr("<p>You must enter a valid Customer Number for "
-                            "this Statement.") );
-    _cust->setFocus();
+
+  QList<GuiErrorCheck> errors;
+  errors<< GuiErrorCheck(!_cust->isValid(), _cust,
+                         tr("You must enter a valid Customer Number for "
+                            "this Statement."))
+  ;
+  if (GuiErrorCheck::reportErrors(this, tr("Enter a Valid Customer Number"), errors))
     return false;
-  }
 
   ParameterList params = getParams();
   if (params.value("checkParamsReturn") == false)
