@@ -13,6 +13,8 @@
 #include <QMessageBox>
 #include <QSqlError>
 #include <QVariant>
+#include "errorReporter.h"
+#include "guiErrorCheck.h"
 
 profitCenter::profitCenter(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
@@ -75,12 +77,6 @@ enum SetResponse profitCenter::set(const ParameterList &pParams )
 void profitCenter::sSave()
 {
   XSqlQuery profitSave;
-  if (_number->text().length() == 0)
-  {
-      QMessageBox::warning( this, tr("Cannot Save Profit Center"),
-                            tr("You must enter a valid Number.") );
-      return;
-  }
   
   profitSave.prepare("SELECT prftcntr_id"
             "  FROM prftcntr"
@@ -89,21 +85,24 @@ void profitCenter::sSave()
   profitSave.bindValue(":prftcntr_id", _prftcntrid);
   profitSave.bindValue(":prftcntr_number", _number->text());
   profitSave.exec();
-  if(profitSave.first())
-  {
-    QMessageBox::critical(this, tr("Duplicate Profit Center Number"),
-      tr("A Profit Center Number already exists for the one specified.") );
-    return;
-  }
+
+  QList<GuiErrorCheck> errors;
+    errors<< GuiErrorCheck(_number->text().trimmed().length() == 0, _number,
+                           tr("You must enter a valid Number."))
+          << GuiErrorCheck(profitSave.first(), _number,
+                           tr("A Profit Center Number already exists for the one specified."))
+    ;
+    if (GuiErrorCheck::reportErrors(this, tr("Cannot Save Profit Center"), errors))
+      return;
 
   if (_mode == cNew)
   {
     profitSave.exec("SELECT NEXTVAL('prftcntr_prftcntr_id_seq') AS prftcntr_id;");
     if (profitSave.first())
       _prftcntrid = profitSave.value("prftcntr_id").toInt();
-    else if (profitSave.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Profit Center Information"),
+                                  profitSave, __FILE__, __LINE__))
     {
-      systemError(this, profitSave.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
 
@@ -137,9 +136,9 @@ void profitCenter::sSave()
   profitSave.bindValue(":prftcntr_number", _number->text());
   profitSave.bindValue(":prftcntr_descrip", _descrip->toPlainText());
   profitSave.exec();
-  if (profitSave.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Profit Center Information"),
+                                           profitSave, __FILE__, __LINE__))
   {
-    systemError(this, profitSave.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -152,9 +151,9 @@ void profitCenter::sSave()
     profitSave.bindValue(":prftcntr_number", _number->text());
     profitSave.bindValue(":old_prftcntr_number", _cachedNumber);
     profitSave.exec();
-    if (profitSave.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Profit Center Information"),
+                                  profitSave, __FILE__, __LINE__))
     {
-      systemError(this, profitSave.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -177,9 +176,10 @@ void profitCenter::populate()
 
     _cachedNumber = profitpopulate.value("prftcntr_number").toString();
   }
-  else if (profitpopulate.lastError().type() != QSqlError::NoError)
+
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Profit Center Information"),
+                                profitpopulate, __FILE__, __LINE__))
   {
-    systemError(this, profitpopulate.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
