@@ -19,6 +19,7 @@
 
 #include "profitCenter.h"
 #include "storedProcErrorLookup.h"
+#include "errorReporter.h"
 
 profitCenters::profitCenters(QWidget* parent, const char* name, Qt::WindowFlags fl)
     : XWidget(parent, name, fl)
@@ -104,16 +105,15 @@ void profitCenters::sDelete()
     int result = profitDelete.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("deleteProfitCenter", result),
-                  __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Profit Center"),
+                             storedProcErrorLookup("deleteProfitCenter", result),
+                             __FILE__, __LINE__);
       return;
     }
   }
-  else if (profitDelete.lastError().type() != QSqlError::NoError)
-  {
-    systemError(this, profitDelete.lastError().databaseText(), __FILE__, __LINE__);
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Profit Center"),
+                                profitDelete, __FILE__, __LINE__))
     return;
-  }
 
   sFillList();
 }
@@ -140,9 +140,9 @@ void profitCenters::sFillList()
                  "ORDER BY prftcntr_number;" );
   profitFillList.exec();
   _prftcntr->populate(profitFillList);
-  if (profitFillList.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Profit Center Information"),
+                                         profitFillList, __FILE__, __LINE__))
   {
-    systemError(this, profitFillList.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
