@@ -1,7 +1,7 @@
   /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -35,6 +35,35 @@
 
 const char *_projectStatuses[] = { "P", "O", "C" };
 
+bool project::userHasPriv(const int pMode, const int pId)
+{
+  if (_privileges->check("MaintainAllProjects"))
+    return true;
+  bool personalPriv = _privileges->check("MaintainPersonalProjects");
+  if(pMode==cView)
+  {
+    if(_privileges->check("ViewAllProjects"))
+      return true;
+    personalPriv = personalPriv || _privileges->check("ViewPersonalProjects");
+  }
+
+  if(pMode==cNew)
+    return personalPriv;
+  else
+  {
+    XSqlQuery usernameCheck;
+    usernameCheck.prepare( "SELECT getEffectiveXtUser() IN (prj_owner_username, prj_username) AS canModify "
+                           "FROM prj "
+                            "WHERE (prj_id=:prj_id);" );
+    usernameCheck.bindValue(":prj_id", pId);
+    usernameCheck.exec();
+
+    if (usernameCheck.first())
+      return usernameCheck.value("canModify").toBool()&&personalPriv;
+    return false;
+  }
+}
+
 project::project(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl),
       _prjid(-1)
@@ -57,6 +86,7 @@ project::project(QWidget* parent, const char* name, bool modal, Qt::WindowFlags
   connect(_buttonBox,     SIGNAL(accepted()),        this, SLOT(sSave()));
   connect(_queryTasks,    SIGNAL(clicked()),         this, SLOT(sFillTaskList()));
   connect(_newTask,       SIGNAL(clicked()),         this, SLOT(sNewTask()));
+  connect(_print,         SIGNAL(clicked()),         this, SLOT(sPrintTasks()));
   connect(_editTask,      SIGNAL(clicked()),         this, SLOT(sEditTask()));
   connect(_viewTask,      SIGNAL(clicked()),         this, SLOT(sViewTask()));
   connect(_deleteTask,    SIGNAL(clicked()),         this, SLOT(sDeleteTask()));
@@ -67,6 +97,12 @@ project::project(QWidget* parent, const char* name, bool modal, Qt::WindowFlags
   connect(_showPo, SIGNAL(toggled(bool)), this, SLOT(sFillTaskList()));
   connect(_showWo, SIGNAL(toggled(bool)), this, SLOT(sFillTaskList()));
   connect(_showIn, SIGNAL(toggled(bool)), this, SLOT(sFillTaskList()));
+  connect(_showCompleted, SIGNAL(toggled(bool)), this, SLOT(sFillTaskList()));
+
+  connect(omfgThis, SIGNAL(salesOrdersUpdated(int, bool)), this, SLOT(sFillTaskList()));
+  connect(omfgThis, SIGNAL(quotesUpdated(int, bool)), this, SLOT(sFillTaskList()));
+  connect(omfgThis, SIGNAL(workOrdersUpdated(int, bool)), this, SLOT(sFillTaskList()));
+  connect(omfgThis, SIGNAL(purchaseOrdersUpdated(int, bool)), this, SLOT(sFillTaskList()));
 
   _charass->setType("PROJ");
 
@@ -91,6 +127,7 @@ project::project(QWidget* parent, const char* name, bool modal, Qt::WindowFlags
   _prjtask->addColumn(tr("Exp. Budget"),      _priceColumn,   Qt::AlignRight,  true,  "exp_budget"  );
   _prjtask->addColumn(tr("Exp. Actual"),      _priceColumn,   Qt::AlignRight,  true,  "exp_actual"  );
   _prjtask->addColumn(tr("Exp. Balance"),      _priceColumn,   Qt::AlignRight,  true,  "exp_balance"  );
+  _prjtask->setSortingEnabled(false);
 
   _owner->setUsername(omfgThis->username());
   _assignedTo->setUsername(omfgThis->username());
@@ -105,6 +142,7 @@ project::project(QWidget* parent, const char* name, bool modal, Qt::WindowFlags
   _totalExpBal->setPrecision(omfgThis->moneyVal());
   
   _saved=false;
+  _close = false;
 
   QMenu * newMenu = new QMenu;
   QAction *menuItem;
@@ -150,6 +188,75 @@ enum SetResponse project::set(const ParameterList &pParams)
   if (valid)
     _assignedTo->setUsername(param.toString());
 
+  param = pParams.value("mode", &valid);
+  if (valid)
+  {
+    if (param.toString() == "new")
+    {
+      _mode = cNew;
+
+      connect(_assignedTo, SIGNAL(newId(int)), this, SLOT(sAssignedToChanged(int)));
+      connect(_status,  SIGNAL(currentIndexChanged(int)), this, SLOT(sStatusChanged(int)));
+      connect(_completed,  SIGNAL(newDate(QDate)), this, SLOT(sCompletedChanged()));
+      connect(_pctCompl,  SIGNAL(valueChanged(int)), this, SLOT(sCompletedChanged()));
+      connect(_prjtask, SIGNAL(valid(bool)), this, SLOT(sHandleButtons(bool)));
+      connect(_prjtask, SIGNAL(valid(bool)), this, SLOT(sHandleButtons(bool)));
+      connect(_prjtask, SIGNAL(itemSelected(int)), _editTask, SLOT(animateClick()));
+
+      projectet.exec("SELECT NEXTVAL('prj_prj_id_seq') AS prj_id;");
+      if (projectet.first())
+        _prjid = projectet.value("prj_id").toInt();
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Project Information"),
+                                    projectet, __FILE__, __LINE__))
+      {
+        return UndefinedError;
+      }
+
+      _comments->setId(_prjid);
+      _documents->setId(_prjid);
+      _charass->setId(_prjid);
+      _recurring->setParent(_prjid, "J");
+    }
+    else if (param.toString() == "edit")
+    {
+      _mode = cEdit;
+
+      _number->setEnabled(false);
+
+      connect(_assignedTo, SIGNAL(newId(int)), this, SLOT(sAssignedToChanged(int)));
+      connect(_status,  SIGNAL(currentIndexChanged(int)), this, SLOT(sStatusChanged(int)));
+      connect(_completed,  SIGNAL(newDate(QDate)), this, SLOT(sCompletedChanged()));
+      connect(_pctCompl,  SIGNAL(valueChanged(int)), this, SLOT(sCompletedChanged()));
+      connect(_prjtask, SIGNAL(valid(bool)), this, SLOT(sHandleButtons(bool)));
+      connect(_prjtask, SIGNAL(valid(bool)), this, SLOT(sHandleButtons(bool)));
+      connect(_prjtask, SIGNAL(itemSelected(int)), _editTask, SLOT(animateClick()));
+
+      QMenu * newMenu = new QMenu;
+      QAction *menuItem;
+      newMenu->addAction(tr("Task..."), this, SLOT(sNewTask()));
+      newMenu->addSeparator();
+      menuItem = newMenu->addAction(tr("Incident"), this, SLOT(sNewIncident()));
+      menuItem->setEnabled(_privileges->check("MaintainPersonalIncidents") ||
+                       _privileges->check("MaintainAllIncidents"));
+      menuItem = newMenu->addAction(tr("Quote"), this, SLOT(sNewQuotation()));
+      menuItem->setEnabled(_privileges->check("MaintainQuotes"));
+      menuItem = newMenu->addAction(tr("Sales Order"), this, SLOT(sNewSalesOrder()));
+      menuItem->setEnabled(_privileges->check("MaintainSalesOrders"));
+      menuItem = newMenu->addAction(tr("Purchase Order"),   this, SLOT(sNewPurchaseOrder()));
+      menuItem->setEnabled(_privileges->check("MaintainPurchaseOrders"));
+      menuItem = newMenu->addAction(tr("Work Order"),   this, SLOT(sNewWorkOrder()));
+      menuItem->setEnabled(_privileges->check("MaintainWorkOrders"));
+      _newTask->setMenu(newMenu);
+
+      QMenu * printMenu = new QMenu;
+      printMenu->addAction(tr("Print Tasks"), this, SLOT(sPrintTasks()));
+      printMenu->addAction(tr("Print Orders"), this, SLOT(sPrintOrders()));
+      _print->setMenu(printMenu);
+    }
+    else if (param.toString() == "view")
+      setViewMode();
+  }
+
   param = pParams.value("prj_id", &valid);
   if (valid)
   {
@@ -170,81 +277,41 @@ enum SetResponse project::set(const ParameterList &pParams)
   {
     _cntct->setId(param.toInt());
   }
-
-  param = pParams.value("mode", &valid);
-  if (valid)
-  {
-    if (param.toString() == "new")
-    {
-      _mode = cNew;
-
-      connect(_assignedTo, SIGNAL(newId(int)), this, SLOT(sAssignedToChanged(int)));
-      connect(_status,  SIGNAL(currentIndexChanged(int)), this, SLOT(sStatusChanged(int)));
-      connect(_prjtask, SIGNAL(valid(bool)), this, SLOT(sHandleButtons(bool)));
-      connect(_prjtask, SIGNAL(valid(bool)), this, SLOT(sHandleButtons(bool)));
-      connect(_prjtask, SIGNAL(itemSelected(int)), _editTask, SLOT(animateClick()));
-
-      projectet.exec("SELECT NEXTVAL('prj_prj_id_seq') AS prj_id;");
-      if (projectet.first())
-        _prjid = projectet.value("prj_id").toInt();
-      else if (projectet.lastError().type() == QSqlError::NoError)
-      {
-        systemError(this, projectet.lastError().text(), __FILE__, __LINE__);
-        return UndefinedError;
-      }
-
-      _comments->setId(_prjid);
-      _documents->setId(_prjid);
-      _charass->setId(_prjid);
-      _recurring->setParent(_prjid, "J");
-    }
-    else if (param.toString() == "edit")
-    {
-      _mode = cEdit;
-
-      _number->setEnabled(false);
-
-      connect(_assignedTo, SIGNAL(newId(int)), this, SLOT(sAssignedToChanged(int)));
-      connect(_status,  SIGNAL(currentIndexChanged(int)), this, SLOT(sStatusChanged(int)));
-      connect(_prjtask, SIGNAL(valid(bool)), this, SLOT(sHandleButtons(bool)));
-      connect(_prjtask, SIGNAL(valid(bool)), this, SLOT(sHandleButtons(bool)));
-      connect(_prjtask, SIGNAL(itemSelected(int)), _editTask, SLOT(animateClick()));
-    }
-    else if (param.toString() == "view")
-    {
-      _mode = cView;
-      
-      _owner->setEnabled(false);
-      _number->setEnabled(false);
-      _status->setEnabled(false);
-      _name->setEnabled(false);
-      _descrip->setEnabled(false);
-      _so->setEnabled(false);
-      _wo->setEnabled(false);
-      _po->setEnabled(false);
-      _assignedTo->setEnabled(false);
-      _crmacct->setEnabled(false);
-      _cntct->setEnabled(false);
-      _newTask->setEnabled(false);
-      connect(_prjtask, SIGNAL(itemSelected(int)), _viewTask, SLOT(animateClick()));
-      _comments->setReadOnly(true);
-      _charass->setReadOnly(true);
-      _documents->setReadOnly(true);
-      _started->setEnabled(false);
-      _assigned->setEnabled(false);
-      _due->setEnabled(false);
-      _completed->setEnabled(false);
-      _recurring->setEnabled(false);
-      _projectType->setEnabled(false);
-      _buttonBox->removeButton(_buttonBox->button(QDialogButtonBox::Save));
-      _buttonBox->removeButton(_buttonBox->button(QDialogButtonBox::Cancel));
-      _buttonBox->addButton(QDialogButtonBox::Close);
-    }
-  }
     
   return NoError;
 }
 
+void project::setViewMode()
+{
+  _mode = cView;
+
+  _infoGroup->setEnabled(false);
+  _number->setEnabled(false);
+  _status->setEnabled(false);
+  _name->setEnabled(false);
+  _descrip->setEnabled(false);
+  _so->setEnabled(false);
+  _wo->setEnabled(false);
+  _po->setEnabled(false);
+  _cntct->setEnabled(false);
+  _newTask->setEnabled(false);
+  connect(_prjtask, SIGNAL(itemSelected(int)), _viewTask, SLOT(animateClick()));
+  _comments->setReadOnly(true);
+  _charass->setReadOnly(true);
+  _documents->setReadOnly(true);
+  _scheduleGroup->setEnabled(false);
+  _recurring->setEnabled(false);
+  _projectType->setEnabled(false);
+  _buttonBox->removeButton(_buttonBox->button(QDialogButtonBox::Save));
+  _buttonBox->removeButton(_buttonBox->button(QDialogButtonBox::Cancel));
+  _buttonBox->addButton(QDialogButtonBox::Close);
+
+  QMenu * printMenu = new QMenu;
+  printMenu->addAction(tr("Print Tasks"), this, SLOT(sPrintTasks()));
+  printMenu->addAction(tr("Print Orders"), this, SLOT(sPrintOrders()));
+  _print->setMenu(printMenu);
+}
+
 void project::sHandleButtons(bool valid)
 {
   if(_prjtask->altId() == 5)
@@ -261,6 +328,7 @@ void project::sHandleButtons(bool valid)
 
 void project::sPopulateMenu(QMenu *pMenu,  QTreeWidgetItem *selected)
 {
+  Q_UNUSED(selected);
   QAction *menuItem;
 
   if(_prjtask->altId() == 5)
@@ -388,6 +456,43 @@ void project::sPopulateMenu(QMenu *pMenu,  QTreeWidgetItem *selected)
 
 void project::populate()
 {
+  if (_mode == cEdit && !_lock.acquire("prj", _prjid, AppLock::Interactive))
+    setViewMode();
+
+  _close = false;
+
+  foreach (QWidget* widget, QApplication::allWidgets())
+  {
+    if (!widget->isWindow() || !widget->isVisible())
+      continue;
+
+    project *w = qobject_cast<project*>(widget);
+
+    if (w && w != this && w->id()==_prjid)
+    {
+      // detect "i'm my own grandpa"
+      QObject *p;
+      for (p = parent(); p && p != w ; p = p->parent())
+        ; // do nothing
+      if (p == w)
+      {
+        QMessageBox::warning(this, tr("Cannot Open Recursively"),
+                             tr("This project is already open and cannot be "
+                                "raised. Please close windows to get to it."));
+        _close = true;
+      } else if (p) {
+        w->setFocus();
+        if (omfgThis->showTopLevel())
+        {
+          w->raise();
+          w->activateWindow();
+        }
+        _close = true;
+      }
+      break;
+    }
+  }
+
   XSqlQuery projectpopulate;
   projectpopulate.prepare( "SELECT * "
              "FROM prj "
@@ -405,6 +510,7 @@ void project::populate()
     _wo->setChecked(projectpopulate.value("prj_wo").toBool());
     _po->setChecked(projectpopulate.value("prj_po").toBool());
     _assignedTo->setUsername(projectpopulate.value("prj_username").toString());
+    _dept->setId(projectpopulate.value("prj_dept_id").toInt());
     _cntct->setId(projectpopulate.value("prj_cntct_id").toInt());
     _crmacct->setId(projectpopulate.value("prj_crmacct_id").toInt());
     _started->setDate(projectpopulate.value("prj_start_date").toDate());
@@ -412,6 +518,8 @@ void project::populate()
     _due->setDate(projectpopulate.value("prj_due_date").toDate());
     _completed->setDate(projectpopulate.value("prj_completed_date").toDate());
     _projectType->setId(projectpopulate.value("prj_prjtype_id").toInt());
+    _priority->setId(projectpopulate.value("prj_priority_id").toInt());
+    _pctCompl->setValue(projectpopulate.value("prj_pct_complete").toInt());
     for (int counter = 0; counter < _status->count(); counter++)
     {
       if (QString(projectpopulate.value("prj_status").toString()[0]) == _projectStatuses[counter])
@@ -475,6 +583,14 @@ void project::sStatusChanged(const int pStatus)
   }
 }
 
+void project::sCompletedChanged()
+{
+  if (_completed->isValid())
+    _pctCompl->setValue(100);
+  if (_pctCompl->value() == 100)
+    _completed->setDate(omfgThis->dbDate());
+}
+
 void project::sCRMAcctChanged(const int newid)
 {
   _cntct->setSearchAcct(newid);
@@ -521,13 +637,15 @@ bool project::sSave(bool partial)
                "  prj_so, prj_wo, prj_po, prj_status, prj_owner_username, "
                "  prj_start_date, prj_due_date, prj_assigned_date,"
                "  prj_completed_date, prj_username, prj_recurring_prj_id,"
-               "  prj_crmacct_id, prj_cntct_id, prj_prjtype_id) "
+               "  prj_crmacct_id, prj_cntct_id, prj_prjtype_id, "
+               "  prj_dept_id, prj_priority_id, prj_pct_complete ) "
                "VALUES "
                "( :prj_id, :prj_number, :prj_name, :prj_descrip,"
                "  :prj_so, :prj_wo, :prj_po, :prj_status, :prj_owner_username,"
                "  :prj_start_date, :prj_due_date, :prj_assigned_date,"
                "  :prj_completed_date, :username, :prj_recurring_prj_id,"
-               "  :prj_crmacct_id, :prj_cntct_id, :prj_prjtype_id);" );
+               "  :prj_crmacct_id, :prj_cntct_id, :prj_prjtype_id, "
+               "  :prj_dept_id, :prj_priority_id, :prj_pct_complete );" );
   else
     projectSave.prepare( "UPDATE prj "
                "SET prj_number=:prj_number, prj_name=:prj_name, prj_descrip=:prj_descrip,"
@@ -539,7 +657,10 @@ bool project::sSave(bool partial)
                "    prj_recurring_prj_id=:prj_recurring_prj_id,"
                "    prj_crmacct_id=:prj_crmacct_id,"
                "    prj_cntct_id=:prj_cntct_id, "
-               "    prj_prjtype_id=:prj_prjtype_id "
+               "    prj_prjtype_id=:prj_prjtype_id, "
+               "    prj_dept_id=:prj_dept_id, "
+               "    prj_priority_id=:prj_priority_id, "
+               "    prj_pct_complete=:prj_pct_complete "
                "WHERE (prj_id=:prj_id);" );
 
   projectSave.bindValue(":prj_id", _prjid);
@@ -547,6 +668,9 @@ bool project::sSave(bool partial)
   projectSave.bindValue(":prj_name", _name->text());
   projectSave.bindValue(":prj_descrip", _descrip->toPlainText());
   projectSave.bindValue(":prj_status", _projectStatuses[_status->currentIndex()]);
+  projectSave.bindValue(":prj_priority_id", _priority->id());
+  if (_dept->id() > 0)
+    projectSave.bindValue(":prj_dept_id", _dept->id());
   projectSave.bindValue(":prj_so", QVariant(_so->isChecked()));
   projectSave.bindValue(":prj_wo", QVariant(_wo->isChecked()));
   projectSave.bindValue(":prj_po", QVariant(_po->isChecked()));
@@ -562,6 +686,7 @@ bool project::sSave(bool partial)
   projectSave.bindValue(":prj_due_date",	_due->date());
   projectSave.bindValue(":prj_assigned_date", _assigned->date());
   projectSave.bindValue(":prj_completed_date", _completed->date());
+  projectSave.bindValue(":prj_pct_complete", _pctCompl->value());
   if (_recurring->isRecurring())
     projectSave.bindValue(":prj_recurring_prj_id", _recurring->parentId());
 
@@ -569,7 +694,8 @@ bool project::sSave(bool partial)
   if (projectSave.lastError().type() != QSqlError::NoError)
   {
     rollbackq.exec();
-    systemError(this, projectSave.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Project Information"),
+                         projectSave, __FILE__, __LINE__);
     return false;
   }
 
@@ -578,7 +704,10 @@ bool project::sSave(bool partial)
   {
     qDebug("recurring->save failed");
     rollbackq.exec();
-    systemError(this, errmsg, __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Occurred"),
+                         tr("%1: Error Saving Project Information \n %2")
+                         .arg(windowTitle())
+                         .arg(errmsg),__FILE__,__LINE__);
     return false;
   }
 
@@ -607,6 +736,13 @@ void project::sPrintTasks()
 
 void project::sPrintOrders()
 {
+  if (!_showSo->isChecked() && !_showWo->isChecked() && !_showPo->isChecked())
+  {
+    QMessageBox::critical( this, tr("Cannot Print Orders"),
+        tr("Please first select an order type or types to print."));
+    return;
+  }
+
   ParameterList params;
 
   params.append("prj_id", _prjid);
@@ -766,50 +902,47 @@ void project::sFillTaskList()
     _totalExpAct->setDouble(qry.value("totalexpact").toDouble());
     _totalExpBal->setDouble(qry.value("totalexpbal").toDouble());
   }
-  if (qry.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Project Information"),
+                                qry, __FILE__, __LINE__))
   {
-    systemError(this, qry.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
-/* Not sure why the totals are zeroed out 
-   else
-  {
-    _totalHrBud->setDouble(0.0);
-    _totalHrAct->setDouble(0.0);
-    _totalHrBal->setDouble(0.0);
-    _totalExpBud->setDouble(0.0);
-    _totalExpAct->setDouble(0.0);
-    _totalExpBal->setDouble(0.0);
-  }  */
 
 // Populate Task List
   MetaSQLQuery mqltask = mqlLoad("orderActivityByProject", "detail");
-  
-  params.append("so", tr("Sales Order"));
-  params.append("wo", tr("Work Order"));
-  params.append("po", tr("Purchase Order"));
-  params.append("pr", tr("Purchase Request"));
-  params.append("sos", tr("Sales Orders"));
-  params.append("wos", tr("Work Orders"));
-  params.append("pos", tr("Purchase Orders"));
-  params.append("prs", tr("Purchase Requests"));
-  params.append("quote", tr("Quote"));
-  params.append("quotes", tr("Quotes"));
-  params.append("invoice", tr("Invoice"));
-  params.append("invoices", tr("Invoices"));
-  params.append("open", tr("Open"));
-  params.append("closed", tr("Closed"));
-  params.append("converted", tr("Converted"));
-  params.append("canceled", tr("Canceled"));
-  params.append("expired", tr("Expired"));
-  params.append("unposted", tr("Unposted"));
-  params.append("posted", tr("Posted"));
-  params.append("exploded", tr("Exploded"));
-  params.append("released", tr("Released"));
-  params.append("planning", tr("Concept"));
-  params.append("inprocess", tr("In Process"));
-  params.append("complete", tr("Complete"));
+
+  params.append("assigned",   tr("Assigned"));
+  params.append("canceled",   tr("Canceled"));
+  params.append("closed",     tr("Closed"));
+  params.append("complete",   tr("Complete"));
+  params.append("confirmed",  tr("Confirmed"));
+  params.append("converted",  tr("Converted"));
+  params.append("expired",    tr("Expired"));
+  params.append("exploded",   tr("Exploded"));
+  params.append("feedback",   tr("Feedback"));
+  params.append("inprocess",  tr("In Process"));
+  params.append("invoice",    tr("Invoice"));
+  params.append("invoices",   tr("Invoices"));
+  params.append("new",        tr("New"));
+  params.append("open",       tr("Open"));
+  params.append("planning",   tr("Concept"));
+  params.append("po",         tr("Purchase Order"));
+  params.append("pos",        tr("Purchase Orders"));
+  params.append("posted",     tr("Posted"));
+  params.append("pr",         tr("Purchase Request"));
+  params.append("prs",        tr("Purchase Requests"));
+  params.append("quote",      tr("Quote"));
+  params.append("quotes",     tr("Quotes"));
+  params.append("released",   tr("Released"));
+  params.append("resolved",   tr("Resolved"));
+  params.append("so",         tr("Sales Order"));
+  params.append("sos",        tr("Sales Orders"));
+  params.append("total",      tr("Total"));
+  params.append("unposted",   tr("Unposted"));
   params.append("unreleased", tr("Unreleased"));
+  params.append("wo",         tr("Work Order"));
+  params.append("wos",        tr("Work Orders"));
+
   params.append("total", tr("Total"));
 
   if(_showSo->isChecked())
@@ -824,12 +957,17 @@ void project::sFillTaskList()
   if(_showIn->isChecked())
     params.append("showIn");
 
+  if (_showCompleted->isChecked())
+    params.append("showCompleted");
+
   if (! _privileges->check("ViewAllProjects") && ! _privileges->check("MaintainAllProjects"))
     params.append("owner_username", omfgThis->username());
 
   XSqlQuery qrytask = mqltask.toQuery(params);
 
   _prjtask->populate(qrytask, true);
+  (void)ErrorReporter::error(QtCriticalMsg, this, tr("Could not get Task Information"),
+                           qrytask, __FILE__, __LINE__);
   _prjtask->expandAll();
 }
 
@@ -883,7 +1021,6 @@ void project::sNewQuotation()
   salesOrder *newdlg = new salesOrder(this);
   newdlg->set(params);
   omfgThis->handleNewWindow(newdlg);
-  sFillTaskList();
 }
 
 void project::sNewSalesOrder()
@@ -895,7 +1032,6 @@ void project::sNewSalesOrder()
   salesOrder *newdlg = new salesOrder(this);
   newdlg->set(params);
   omfgThis->handleNewWindow(newdlg);
-  sFillTaskList();
 }
 
 
@@ -908,7 +1044,6 @@ void project::sNewPurchaseOrder()
   purchaseOrder *newdlg = new purchaseOrder(this);
   newdlg->set(params);
   omfgThis->handleNewWindow(newdlg);
-  sFillTaskList();
 }
 
 void project::sNewWorkOrder()
@@ -920,7 +1055,6 @@ void project::sNewWorkOrder()
   workOrder *newdlg = new workOrder(this);
   newdlg->set(params);
   omfgThis->handleNewWindow(newdlg);
-  sFillTaskList();
 }
 
 void project::sEditOrder()
@@ -1125,3 +1259,24 @@ void project::sViewOrder()
   }
 }
 
+int project::id()
+{
+  return _prjid;
+}
+
+void project::setVisible(bool visible)
+{
+  if (_close)
+    close();
+  else
+    XDialog::setVisible(visible);
+}
+
+void project::done(int result)
+{
+  if (!_lock.release())
+    ErrorReporter::error(QtCriticalMsg, this, tr("Locking Error"),
+                         _lock.lastError(), __FILE__, __LINE__);
+
+  XDialog::done(result);
+}
