@@ -11,6 +11,7 @@
 #ifndef PROJECT_H
 #define PROJECT_H
 
+#include "applock.h"
 #include "guiclient.h"
 #include "xdialog.h"
 #include <parameter.h>
@@ -24,12 +25,18 @@ public:
     project(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~project();
 
+    Q_INVOKABLE virtual int id();
+
+    static bool userHasPriv(const int = cView, const int = 0);
+
 public slots:
     virtual SetResponse set( const ParameterList & pParams );
+    virtual void setViewMode();
     virtual void populate();
     virtual void sPopulateMenu( QMenu * pMenu, QTreeWidgetItem * selected );
     virtual void sAssignedToChanged(const int);
     virtual void sStatusChanged(const int);
+    virtual void sCompletedChanged();
     virtual void sCRMAcctChanged(const int);
     virtual void sClose();
     virtual bool sSave(bool partial = false);
@@ -49,6 +56,9 @@ public slots:
     virtual void sNewWorkOrder();
     virtual void sNumberChanged();
     virtual void sHandleButtons(bool valid = false);
+    virtual void done(int);
+
+    virtual void setVisible(bool);
 
 protected slots:
     virtual void languageChange();
@@ -57,6 +67,8 @@ private:
     int _mode;
     int _prjid;
     bool _saved;
+    bool _close;
+    AppLock _lock;
 
 signals:
     void saved(int);
