@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to y6ou under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -12,6 +12,7 @@
 
 #include <parameter.h>
 #include "projectType.h"
+#include "errorReporter.h"
 #include "guiclient.h"
 
 projectTypes::projectTypes(QWidget* parent, const char* name, Qt::WindowFlags fl)
@@ -64,6 +65,9 @@ void projectTypes::sDelete()
   typeDelete.bindValue(":prjtype_id", _projecttype->id());
   typeDelete.exec();
 
+  ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Project Type"),
+                            typeDelete, __FILE__, __LINE__);
+
   sFillList();
 }
 
@@ -73,9 +77,11 @@ void projectTypes::sNew()
   ParameterList params;
   params.append("mode", "new");
 
-  projectType *newdlg = new projectType();
-  newdlg->set(params);
-  omfgThis->handleNewWindow(newdlg);
+  projectType newdlg(this, "", true);
+  newdlg.set(params);
+
+  if (newdlg.exec() != XDialog::Rejected)
+    sFillList();
 }
 
 void projectTypes::sEdit()
@@ -84,9 +90,11 @@ void projectTypes::sEdit()
   params.append("mode", "edit");
   params.append("prjtype_id", _projecttype->id());
 
-  projectType *newdlg = new projectType();
-  newdlg->set(params);
-  omfgThis->handleNewWindow(newdlg);
+  projectType newdlg(this, "", true);
+  newdlg.set(params);
+
+  if (newdlg.exec() != XDialog::Rejected)
+    sFillList();
 }
 
 void projectTypes::sFillList()
