@@ -57,6 +57,7 @@ prospect::prospect(QWidget* parent, const char* name, Qt::WindowFlags fl)
   _quotes->addColumn(tr("Quote Date"),       _dateColumn,  Qt::AlignLeft, true, "quhead_quotedate" );
 
   _NumberGen = -1;
+  _closed = false;
 }
 
 prospect::~prospect()
@@ -83,10 +84,6 @@ enum SetResponse prospect::set(const ParameterList &pParams)
   if (valid)
     _prospectid = param.toInt();
 
-  if (_crmacctid >= 0 || _prospectid >= 0)
-    if (! sPopulate())
-      return UndefinedError;
-
   param = pParams.value("mode", &valid);
   if (valid)
   {
@@ -112,29 +109,20 @@ enum SetResponse prospect::set(const ParameterList &pParams)
       _mode = cEdit;
     }
     else if (param.toString() == "view")
-    {
-      _mode = cView;
-
-      _save->hide();
-
-      disconnect(_quotes, SIGNAL(itemSelected(int)), _editQuote, SLOT(animateClick()));
-      disconnect(_quotes, SIGNAL(valid(bool)),       _editQuote, SLOT(setEnabled(bool)));
-      disconnect(_quotes, SIGNAL(valid(bool)),       _editQuote, SLOT(setEnabled(bool)));
-      disconnect(_quotes, SIGNAL(valid(bool)),     _deleteQuote, SLOT(setEnabled(bool)));
-
-      connect(_quotes, SIGNAL(itemSelected(int)), _viewQuote, SLOT(animateClick()));
-
-      _close->setText(tr("&Close"));
-    }
+      setViewMode();
   }
 
+  if (_crmacctid >= 0 || _prospectid >= 0)
+    if (! sPopulate())
+      return UndefinedError;
+
   bool canEdit = (cEdit == _mode || cNew == _mode);
   _number->setEnabled(canEdit &&
                       _metrics->value("CRMAccountNumberGeneration") != "A");
   _active->setEnabled(canEdit);
   _contact->setEnabled(canEdit);
   _name->setEnabled(canEdit);
-  _newQuote->setEnabled(canEdit);
+  _newQuote->setEnabled(cEdit == _mode);
   _notes->setReadOnly(! canEdit);
   _salesrep->setEnabled(canEdit);
   _site->setEnabled(canEdit);
@@ -143,6 +131,22 @@ enum SetResponse prospect::set(const ParameterList &pParams)
   return NoError;
 }
 
+void prospect::setViewMode()
+{
+  _mode = cView;
+
+  _save->hide();
+
+  disconnect(_quotes, SIGNAL(itemSelected(int)), _editQuote, SLOT(animateClick()));
+  disconnect(_quotes, SIGNAL(valid(bool)),       _editQuote, SLOT(setEnabled(bool)));
+  disconnect(_quotes, SIGNAL(valid(bool)),       _editQuote, SLOT(setEnabled(bool)));
+  disconnect(_quotes, SIGNAL(valid(bool)),     _deleteQuote, SLOT(setEnabled(bool)));
+
+  connect(_quotes, SIGNAL(itemSelected(int)), _viewQuote, SLOT(animateClick()));
+
+  _close->setText(tr("&Close"));
+}
+
 int prospect::id() const
 {
   return _prospectid;
@@ -383,6 +387,43 @@ bool prospect::sPopulate()
   XSqlQuery getq;
   if (_prospectid >= 0)
   {
+    if (_mode == cEdit && !_lock.acquire("prospect", _prospectid, AppLock::Interactive))
+      setViewMode();
+ 
+    _closed = false;
+ 
+    foreach (QWidget* widget, QApplication::allWidgets())
+    {
+      if (!widget->isWindow() || !widget->isVisible())
+        continue;
+ 
+      prospect *w = qobject_cast<prospect*>(widget);
+ 
+      if (w && w != this && w->id()==_prospectid)
+      {
+        // detect "i'm my own grandpa"
+        QObject *p;
+        for (p = parent(); p && p != w ; p = p->parent())
+          ; // do nothing
+        if (p == w)
+        {
+          QMessageBox::warning(this, tr("Cannot Open Recursively"),
+                               tr("This prospect is already open and cannot be "
+                                  "raised. Please close windows to get to it."));
+          _closed = true;
+        } else if (p) {
+          w->setFocus();
+          if (omfgThis->showTopLevel())
+          {
+            w->raise();
+            w->activateWindow();
+          }
+          _closed = true;
+        }
+        break;
+      }
+    }
+
     getq.prepare("SELECT prospect.*, crmacct_id, crmacct_owner_username"
                  "  FROM prospect, crmacct"
                  " WHERE ((prospect_id=:prospect_id)"
@@ -448,6 +489,11 @@ void prospect::closeEvent(QCloseEvent *pEvent)
     ErrorReporter::error(QtCriticalMsg, this, tr("Getting Prospect"),
                          query, __FILE__, __LINE__);
   }
+
+  if (!_lock.release())
+    ErrorReporter::error(QtCriticalMsg, this, tr("Locking Error"),
+                         _lock.lastError(), __FILE__, __LINE__);
+
   XWidget::closeEvent(pEvent);
 }
 
@@ -482,3 +528,11 @@ void prospect::sCrmAccount()
   newdlg->set(params);
   omfgThis->handleNewWindow(newdlg);
 }
+
+void prospect::setVisible(bool visible)
+{
+  if (_closed)
+    close();
+  else
+    XWidget::setVisible(visible);
+}
