@@ -20,7 +20,7 @@ class purgeClosedWorkOrders : public XDialog, public Ui::purgeClosedWorkOrders
     Q_OBJECT
 
 public:
-    purgeClosedWorkOrders(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    purgeClosedWorkOrders(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~purgeClosedWorkOrders();
 
 public slots:
