@@ -14,6 +14,7 @@
 #include "xsqlquery.h"
 #include <QSqlError>
 #include <QMessageBox>
+#include "errorReporter.h"
 
 quickRelocateLot::quickRelocateLot(QWidget *parent, const char *name, bool modal, Qt::WindowFlags f1)
     : XDialog(parent, name, modal, f1)
@@ -30,6 +31,7 @@ quickRelocateLot::quickRelocateLot(QWidget *parent, const char *name, bool modal
     connect(_warehous, SIGNAL(newID()), this, SLOT(sFillList()));
     connect(_assign, SIGNAL(clicked()), this, SLOT(sPost()));
     connect(_cancel, SIGNAL(clicked()), this, SLOT(reject()));
+    connect(_itemloc, SIGNAL(valid(bool)), _assign, SLOT(setEnabled(bool)));
 }
 
 quickRelocateLot::~quickRelocateLot()
@@ -59,9 +61,9 @@ void quickRelocateLot::sFillList()
   _itemloc->clear();
   _itemloc->populate(lotQuery);
   _itemloc->selectAll();
-  if (lotQuery.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Lot Information"),
+                                lotQuery, __FILE__, __LINE__))
   {
-    systemError(this, lotQuery.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -99,9 +101,9 @@ void quickRelocateLot::sPost()
     {
         ls_id = lotQuery.value("ls_id").toInt();
     }
-    else if (lotQuery.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Lot Information"),
+                                  lotQuery, __FILE__, __LINE__))
     {
-      systemError(this, lotQuery.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
 
@@ -125,9 +127,9 @@ void quickRelocateLot::sPost()
     {
         location_id = locationQuery.value("location_id").toInt();
     }
-    else if (locationQuery.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Location Information"),
+                                  locationQuery, __FILE__, __LINE__))
     {
-      systemError(this, locationQuery.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
 
@@ -150,9 +152,9 @@ void quickRelocateLot::sPost()
     {
         qoh = qohQuery.value("itemloc_qty").toInt();
     }
-    else if (qohQuery.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Lot/Quantity Information"),
+                                  qohQuery, __FILE__, __LINE__))
     {
-      systemError(this, qohQuery.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
 
@@ -176,9 +178,9 @@ void quickRelocateLot::sPost()
     updateQuery.bindValue(":ls_id", ls_id);
     updateQuery.bindValue(":itemsite_id", itemsite_id);
     updateQuery.exec();
-    if (qohQuery.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting Lot Information"),
+                                  updateQuery, __FILE__, __LINE__))
     {
-      systemError(this, qohQuery.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   
