@@ -16,6 +16,7 @@
 #include <QSqlError>
 #include <QVariant>
 
+#include <applock.h>
 #include <parameter.h>
 #include <openreports.h>
 
@@ -40,6 +41,7 @@ quotes::quotes(QWidget* parent, const char *name, Qt::WindowFlags fl)
   setParameterWidgetVisible(true);
   setNewVisible(true);
   setQueryOnStartEnabled(true);
+  setAutoUpdateEnabled(true);
 
   _convertedtoSo->setVisible(false);
 
@@ -74,7 +76,7 @@ quotes::quotes(QWidget* parent, const char *name, Qt::WindowFlags fl)
     connect(list(), SIGNAL(itemSelected(int)), this, SLOT(sView()));
   }
 
-  connect(omfgThis, SIGNAL(quotesUpdated(int, bool)), this, SLOT(sFillList()));
+  connect(omfgThis, SIGNAL(quotesUpdated(int, bool)), this, SLOT(sHandleQuoteEvent(int, bool)));
 }
 
 enum SetResponse quotes::set(const ParameterList& pParams)
@@ -90,6 +92,12 @@ enum SetResponse quotes::set(const ParameterList& pParams)
   return NoError;
 }
 
+void quotes::sHandleQuoteEvent(int pQuheadid, bool)
+{
+  if (pQuheadid == -1)
+    sFillList();
+}
+
 void quotes::sPopulateMenu(QMenu * pMenu, QTreeWidgetItem *, int)
 {
   QAction *menuItem;
@@ -147,6 +155,7 @@ void quotes::sPrint()
 
 void quotes::sConvert(int pType)
 {
+  AppLock _lock;
   QString docType = "Sales Order";
   if (pType == 1)
     docType = "Invoice";
@@ -177,6 +186,21 @@ void quotes::sConvert(int pType)
 
       foreach (XTreeWidgetItem *item, list()->selectedItems())
       {
+        if (!_lock.acquire("quhead", item->id(), AppLock::Interactive))
+        {
+          QMessageBox::critical(this, tr("Cannot Convert"),
+                                tr("<p>One or more of the selected Quotes is"
+                                   " being edited.  You cannot convert a Quote"
+                                   " that is being edited."));
+          return;
+        }
+        if (! _lock.release())
+        {
+          ErrorReporter::error(QtCriticalMsg, this, tr("Locking Error"),
+                               _lock.lastError(), __FILE__, __LINE__);
+          return;
+        }
+        
         if (checkSitePrivs(item->id()))
         {
           int quheadid = item->id();
@@ -186,7 +210,7 @@ void quotes::sConvert(int pType)
           check.exec();
           if (check.first())
           {
-            QMessageBox::critical(this, tr("Can not Convert"),
+            QMessageBox::critical(this, tr("Cannot Convert"),
                                 tr("<p>One or more of the selected Quotes have"
                                    " been converted.  You cannot convert an already"
                                    " converted Quote."));
@@ -223,11 +247,11 @@ void quotes::sConvert(int pType)
                       int result = prospectq.value("result").toInt();
                       if (result < 0)
                       {
-                        systemError(this,
-                                    storedProcErrorLookup("convertProspectToCustomer",
-                                    result), __FILE__, __LINE__);
-                        notConverted.append(item);
-                        continue;
+                          ErrorReporter::error(QtCriticalMsg, this, tr("Error Converting Prospect To Customer"),
+                                               storedProcErrorLookup("convertProspectToCustomer", result),
+                                               __FILE__, __LINE__);
+                          notConverted.append(item);
+                          continue;
                       }
                       convert.exec();
                       if (convert.first())
@@ -243,12 +267,11 @@ void quotes::sConvert(int pType)
                         }
                       }
                     }
-                    else if (prospectq.lastError().type() != QSqlError::NoError)
+                    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Converting Prospect To Customer"),
+                                                  prospectq, __FILE__, __LINE__))
                     {
-                      systemError(this, prospectq.lastError().databaseText(),
-                              __FILE__, __LINE__);
-                      notConverted.append(item);
-                      continue;
+                        notConverted.append(item);
+                        continue;
                     }
                   }
                   else
@@ -321,11 +344,13 @@ void quotes::sConvert(int pType)
 void quotes::sConvertSalesOrder()
 {
   sConvert(0);
+  sFillList();
 }
 
 void quotes::sConvertInvoice()
 {
   sConvert(1);
+  sFillList();
 }
 
 void quotes::sCopy()
@@ -343,15 +368,15 @@ void quotes::sCopy()
       {
         lastid = qq.value("result").toInt();
       }
-      else if(qq.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Copying Quote"),
+                                    qq, __FILE__, __LINE__))
       {
-        systemError(this, qq.lastError().text(), __FILE__, __LINE__);
         return;
       }
     }
   }
   if(lastid != -1)
-    omfgThis->sQuotesUpdated(lastid);
+    omfgThis->sQuotesUpdated(-1);
 }
 
 void quotes::sCopyToCustomer()
@@ -371,7 +396,7 @@ void quotes::sCopyToCustomer()
       }
     }
     if(lastid != -1)
-      omfgThis->sQuotesUpdated(lastid);
+      omfgThis->sQuotesUpdated(-1);
 }
 
 void quotes::sNew()
@@ -445,18 +470,18 @@ void quotes::sDelete()
           int result = quotesDelete.value("result").toInt();
           if (result < 0)
           {
-            systemError(this, storedProcErrorLookup("deleteQuote", result),
-                        __FILE__, __LINE__);
-            continue;
+              ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Quote"),
+                                   storedProcErrorLookup("deleteQuote", result),
+                                   __FILE__, __LINE__);
+              continue;
           }
           counter++;
         }
-        else if (quotesDelete.lastError().type() != QSqlError::NoError)
+        else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Quote #%1")
+                                      .arg(item->text(0)),
+                                      quotesDelete, __FILE__, __LINE__))
         {
-          systemError(this, tr("A System Error occurred deleting Quote #%1\n%2.")
-                             .arg(item->text(0))
-                             .arg(quotesDelete.lastError().databaseText()), __FILE__, __LINE__);
-          continue;
+            continue;
         }
       }
     }
