@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2018 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -27,7 +27,7 @@ public slots:
     virtual enum SetResponse set(const ParameterList & pParams );
     virtual void sCheckKey();
     virtual void sSelect();
-    virtual void sClose();
+    virtual void sRequest();
 
 protected slots:
     virtual void languageChange();
@@ -35,10 +35,6 @@ protected slots:
 protected:
     virtual void closeEvent(QCloseEvent*);
 
-private:
-    int _bankaccntid;
-    QString _type;
-
 };
 
 #endif // REGISTRATIONKEYDIALOG_H
