@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -16,6 +16,7 @@
 
 #include "inputManager.h"
 #include "storedProcErrorLookup.h"
+#include "errorReporter.h"
 
 relocateInventory::relocateInventory(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
@@ -28,7 +29,7 @@ relocateInventory::relocateInventory(QWidget* parent, const char* name, bool mod
 
   _captive = false;
 
-  _item->setType(ItemLineEdit::cLocationControlled);
+  _item->setType(ItemLineEdit::cLocationControlled | ItemLineEdit::cActive);
   _qty->setValidator(omfgThis->transQtyVal());
 
   omfgThis->inputManager()->notify(cBCItem, this, _item, SLOT(setItemid(int)));
@@ -160,6 +161,8 @@ void relocateInventory::sHandleItem()
     _qty->setValidator(omfgThis->transQtyVal());
   else
     _qty->setValidator(new QIntValidator(this));
+
+  sFillList();
 }
 
 void relocateInventory::sMove()
@@ -218,14 +221,15 @@ void relocateInventory::sMove()
     int result = relocate.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("relocateInventory", result),
-                  __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Inventory Information"),
+                             storedProcErrorLookup("relocateInventory", result),
+                             __FILE__, __LINE__);
       return;
     }
   }
-  else if (relocate.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Inventory Information"),
+                                relocate, __FILE__, __LINE__))
   {
-    systemError(this, relocate.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   
@@ -249,7 +253,7 @@ void relocateInventory::sFillList()
 {
   sShowHideDefaultToTarget();
   
-  if (_item->isValid())
+  if (_item->isValid() && _warehouse->isValid())
   {
     XSqlQuery query;
     query.prepare( "SELECT itemloc.*,"
@@ -270,9 +274,9 @@ void relocateInventory::sFillList()
     query.bindValue(":warehous_id", _warehouse->id());
     query.exec();
     _source->populate(query, true);
-    if (query.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Inventory Information"),
+                                  query, __FILE__, __LINE__))
     {
-      systemError(this, query.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
 
@@ -295,15 +299,16 @@ void relocateInventory::sFillList()
                    "WHERE ( (itemsite_warehous_id=:warehous_id)"
                    " AND (location_warehous_id=:warehous_id)"
                    " AND (itemsite_item_id=:item_id)"
-                   " AND  (validLocation(location_id, itemsite_id)) ) "
+                   " AND (location_active) "
+                   " AND (validLocation(location_id, itemsite_id)) ) "
                    "ORDER BY locationname ) AS data" );
     query.bindValue(":warehous_id", _warehouse->id());
     query.bindValue(":item_id", _item->id());
     query.exec();
     _target->populate(query);
-    if (query.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Inventory Information"),
+                                  query, __FILE__, __LINE__))
     {
-      systemError(this, query.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -325,20 +330,16 @@ void relocateInventory::sShowHideDefaultToTarget()
                  "  WHERE (itemsite_item_id=:item_id) "
                  "  AND (itemsite_warehous_id=:warehous_id);");
    query.bindValue(":item_id", _item->id());
-    query.bindValue(":warehous_id", _warehouse->id());
+   query.bindValue(":warehous_id", _warehouse->id());
    query.exec();
    if(query.first())
    {
       if(query.value("itemsite_id").toInt() != -1
          && query.value("itemsite_loccntrl").toBool())
       {
-         _defaultToTarget->show();
-        //Allow default location update with correct privileges
-        if (_privileges->check("MaintainItemSites"))
-            _defaultToTarget->setEnabled(true);
-        else
-            _defaultToTarget->setEnabled(false);
-            _defaultToTarget->setChecked(false);
+        _defaultToTarget->show();
+        _defaultToTarget->setEnabled(_privileges->check("MaintainItemSites"));
+        _defaultToTarget->setChecked(false);
       }
       else
       {
@@ -365,11 +366,11 @@ void relocateInventory::sChangeDefaultLocation()
    query.bindValue(":item_id", _item->id());
    query.exec();
    sFillList();
-   if (query.lastError().type() != QSqlError::NoError)
-      {
-        systemError(this, query.lastError().databaseText(), __FILE__, __LINE__);
-        return;
-      }
+   if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Inventory Information"),
+                                 query, __FILE__, __LINE__))
+   {
+     return;
+   }
    }
 
 }
