@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -44,6 +44,20 @@ returnAuthorization::returnAuthorization(QWidget* parent, const char* name, Qt::
   _shipTo->setNameVisible(false);
   _shipTo->setDescriptionVisible(false);
 
+  _disposition->append(1, tr("Credit"),     "C");
+  _disposition->append(2, tr("Return"),     "R");
+  _disposition->append(3, tr("Replace"),    "P");
+  _disposition->append(4, tr("Service"),    "V");
+  _disposition->append(5, tr("Substitute"), "M");
+
+  _timing->append(1, tr("Immediately"),  "I");
+  _timing->append(2, tr("Upon Receipt"), "R");
+
+  _creditBy->append(1, tr("None"),         "N");
+  _creditBy->append(2, tr("Sales Credit"), "M");
+  _creditBy->append(3, tr("Cash Payment"), "K");
+  _creditBy->append(4, tr("Credit Card"),  "C");
+
   connect(_copyToShipto, SIGNAL(clicked()), this, SLOT(sCopyToShipto()));
   connect(_delete, SIGNAL(clicked()), this, SLOT(sDelete()));
   connect(_edit, SIGNAL(clicked()), this, SLOT(sEdit()));
@@ -56,7 +70,7 @@ returnAuthorization::returnAuthorization(QWidget* parent, const char* name, Qt::
   connect(_subtotal, SIGNAL(valueChanged()), this, SLOT(sCalculateTotal()));
   connect(_tax, SIGNAL(valueChanged()), this, SLOT(sCalculateTotal()));
   connect(_miscCharge, SIGNAL(valueChanged()), this, SLOT(sCalculateTotal()));
-  connect(_freight, SIGNAL(valueChanged()), this, SLOT(sFreightChanged()));
+  connect(_freight, SIGNAL(editingFinished()), this, SLOT(sFreightChanged()));
   connect(_taxzone, SIGNAL(newID(int)), this, SLOT(sTaxZoneChanged()));
   connect(_warehouse, SIGNAL(newID(int)), this, SLOT(sRecvWhsChanged()));
   connect(_shipWhs, SIGNAL(newID(int)), this, SLOT(sShipWhsChanged()));
@@ -99,7 +113,9 @@ returnAuthorization::returnAuthorization(QWidget* parent, const char* name, Qt::
   _authNumber->setValidator(omfgThis->orderVal());
   _comments->setType(Comments::ReturnAuth);
 
+  _custType->setText("");
   _currency->setLabel(_currencyLit);
+  _charass->setType("RA");
 
   _raitem->addColumn(tr("#"),             _seqColumn,   Qt::AlignCenter,true,  "f_linenumber");
   _raitem->addColumn(tr("Kit Seq. #"),    _seqColumn,   Qt::AlignRight, false, "raitem_subnumber");
@@ -140,7 +156,7 @@ returnAuthorization::returnAuthorization(QWidget* parent, const char* name, Qt::
   }
 
   if (! _metrics->boolean("CCAccept"))
-    _creditBy->removeItem(3);
+    _creditBy->removeItem(4);
 
   _printRA->setChecked(_metrics->boolean("DefaultPrintRAOnSave"));
 
@@ -192,10 +208,11 @@ enum SetResponse returnAuthorization::set(const ParameterList &pParams)
         _raheadid = returnet.value("rahead_id").toInt();
         _comments->setId(_raheadid);
         _documents->setId(_raheadid);
+        _charass->setId(_raheadid);
       }
-      else if (returnet.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                    returnet, __FILE__, __LINE__))
       {
-        systemError(this, returnet.lastError().databaseText(), __FILE__, __LINE__);
         return UndefinedError;
       }
 
@@ -212,45 +229,15 @@ enum SetResponse returnAuthorization::set(const ParameterList &pParams)
       if (!_authNumber->text().isEmpty())
         returnet.bindValue(":rahead_number", _authNumber->text());
       returnet.exec();
-      if (returnet.lastError().type() != QSqlError::NoError)
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                    returnet, __FILE__, __LINE__))
       {
-        systemError(this, returnet.lastError().databaseText(), __FILE__, __LINE__);
         return UndefinedError;
       }
 
-      metric = _metrics->value("DefaultRaDisposition");
-      if (metric == "C")
-        _disposition->setCurrentIndex(0);
-      else if (metric == "R")
-        _disposition->setCurrentIndex(1);
-      else if (metric == "P")
-        _disposition->setCurrentIndex(2);
-      else if (metric == "V")
-        _disposition->setCurrentIndex(3);
-      else if (metric == "M")
-        _disposition->setCurrentIndex(4);
-      else
-        _disposition->setCurrentIndex(5);
-
-      metric = _metrics->value("DefaultRaTiming");
-      if (metric == "I")
-        _timing->setCurrentIndex(0);
-      else if (metric == "R")
-        _timing->setCurrentIndex(1);
-      else
-        _timing->setCurrentIndex(2);
-
-      metric = _metrics->value("DefaultRaCreditMethod");
-      if (metric == "N")
-        _creditBy->setCurrentIndex(0);
-      else if (metric == "M")
-        _creditBy->setCurrentIndex(1);
-      else if (metric == "K")
-        _creditBy->setCurrentIndex(2);
-      else if (metric == "C")
-        _creditBy->setCurrentIndex(3);
-      else
-        _creditBy->setCurrentIndex(4);
+      _disposition->setCode(_metrics->value("DefaultRaDisposition"));
+      _timing->setCode(_metrics->value("DefaultRaTiming"));
+      _creditBy->setCode(_metrics->value("DefaultRaCreditMethod"));
 
       connect(_cust, SIGNAL(newId(int)), this, SLOT(sPopulateCustomerInfo()));
       connect(_cust, SIGNAL(valid(bool)), _new, SLOT(setEnabled(bool)));
@@ -289,6 +276,7 @@ enum SetResponse returnAuthorization::set(const ParameterList &pParams)
     _raheadid = param.toInt();
     _comments->setId(_raheadid);
     _documents->setId(_raheadid);
+    _charass->setId(_raheadid);
     populate();
   }
 
@@ -323,6 +311,7 @@ enum SetResponse returnAuthorization::set(const ParameterList &pParams)
       _notes->setEnabled(false);
       _comments->setReadOnly(true);
       _documents->setReadOnly(true);
+      _charass->setReadOnly(true);
       _copyToShipto->setEnabled(false);
       _shipTo->setEnabled(false);
       _shipToName->setEnabled(false);
@@ -375,9 +364,9 @@ void returnAuthorization::setNumber()
       if (_metrics->value("RANumberGeneration") == "A")
         _authNumber->setEnabled(false);
     }
-    else if (returnetNumber.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                  returnetNumber, __FILE__, __LINE__))
     {
-      systemError(this, returnetNumber.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -388,27 +377,20 @@ void returnAuthorization::setNumber()
 bool returnAuthorization::sSave(bool partial)
 {
   XSqlQuery returnSave;
-  const char *dispositionTypes[] = { "C", "R", "P", "V", "M", "" };
-  const char *timingTypes[] = { "I", "R", "" };
-  const char *creditMethods[] = { "N", "M", "K", "C", "" };
-
-  QString disposition = QString(dispositionTypes[_disposition->currentIndex()]);
-  QString timing = QString(timingTypes[_timing->currentIndex()]);
-  QString creditBy = QString(creditMethods[_creditBy->currentIndex()]);
 
   QList<GuiErrorCheck> errors;
   errors
-  << GuiErrorCheck((!partial && disposition.isEmpty()), _disposition,
+  << GuiErrorCheck((!partial && _disposition->code().isEmpty()), _disposition,
                    tr("<p>You must enter a Disposition."))
-  << GuiErrorCheck((!partial && timing.isEmpty()), _timing,
-                   tr("<p>You must enter a Timing."))
-  << GuiErrorCheck((!partial && creditBy.isEmpty()), _creditBy,
+  << GuiErrorCheck((!partial && _timing->code().isEmpty()), _timing,
+                   tr("<p>You must enter a Credit/Ship Timing."))
+  << GuiErrorCheck((!partial && _creditBy->code().isEmpty()), _creditBy,
                    tr("<p>You must enter a Credit Method."))
-  << GuiErrorCheck(_authNumber->text().isEmpty(), _authNumber,
+  << GuiErrorCheck(!partial && _authNumber->text().isEmpty(), _authNumber,
                    tr("You must enter a valid Authorization Number."))
   << GuiErrorCheck(!_authDate->isValid(), _authDate,
                    tr("You must enter a valid Authorization Date."))
-  << GuiErrorCheck((!partial && (disposition == "C") && (creditBy == "N")), _creditBy,
+  << GuiErrorCheck((!partial && (_disposition->code() == "C") && (_creditBy->code() == "N")), _creditBy,
                    tr("<p>You may not enter a Disposition of Credit "
                       "and a Credit By of None."))
   << GuiErrorCheck((! _miscCharge->isZero() && (!_miscChargeAccount->isValid())), _miscChargeAccount,
@@ -474,12 +456,12 @@ bool returnAuthorization::sSave(bool partial)
     returnSave.bindValue(":rahead_taxzone_id", _taxzone->id());
   if (_rsnCode->id() > 0)
     returnSave.bindValue(":rahead_rsncode_id", _rsnCode->id());
-  if (!disposition.isEmpty())
-    returnSave.bindValue(":rahead_disposition", disposition);
-  if (!timing.isEmpty())
-    returnSave.bindValue(":rahead_timing", timing);
-  if (!creditBy.isEmpty())
-    returnSave.bindValue(":rahead_creditmethod", creditBy);
+  if (!_disposition->code().isEmpty())
+    returnSave.bindValue(":rahead_disposition", _disposition->code());
+  if (!_timing->code().isEmpty())
+    returnSave.bindValue(":rahead_timing", _timing->code());
+  if (!_creditBy->code().isEmpty())
+    returnSave.bindValue(":rahead_creditmethod", _creditBy->code());
   if (_origso->isValid())
     returnSave.bindValue(":rahead_orig_cohead_id", _origso->id());
   if (_newso->isValid())
@@ -519,13 +501,14 @@ bool returnAuthorization::sSave(bool partial)
   returnSave.bindValue(":rahead_calcfreight", _calcfreight);
   returnSave.bindValue(":rahead_warehous_id", _warehouse->id());
   returnSave.bindValue(":rahead_cohead_warehous_id", _shipWhs->id());
-  returnSave.bindValue(":rahead_shipzone_id", _shippingZone->id());
+  if (_shippingZone->id() != -1)
+    returnSave.bindValue(":rahead_shipzone_id", _shippingZone->id());
   returnSave.bindValue(":rahead_saletype_id", _saleType->id());
 
   returnSave.exec();
-  if (returnSave.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving RA Information"),
+                                returnSave, __FILE__, __LINE__))
   {
-    systemError(this, returnSave.lastError().databaseText(), __FILE__, __LINE__);
     return false;
   }
 
@@ -623,24 +606,6 @@ void returnAuthorization::sOrigSoChanged()
   {
     if (_origso->isValid())
     {
-      if ( !_warehouse->isValid() )
-      {
-        QMessageBox::warning( this, tr("Invalid Receiving Site"),
-                             tr("<p>You must enter a valid Receiving Site." ) );
-        _warehouse->setFocus();
-        _origso->setId(-1);
-        return;
-      }
-
-      if ( !_shipWhs->isValid() )
-      {
-        QMessageBox::warning( this, tr("Invalid Shipping Site"),
-                             tr("<p>You must enter a valid Shipping Site." ) );
-        _shipWhs->setFocus();
-        _origso->setId(-1);
-        return;
-      }
-
       XSqlQuery sohead;
       sohead.prepare("SELECT cohead_salesrep_id, cohead_commission,"
                      "       cohead_taxzone_id, cohead_custponumber, cohead_prj_id,"
@@ -653,13 +618,15 @@ void returnAuthorization::sOrigSoChanged()
                      "       cohead_shiptoname, cohead_shiptoaddress1,"
                      "       cohead_shiptoaddress2, cohead_shiptoaddress3,"
                      "       cohead_shiptocity, cohead_shiptostate,"
-                     "       cohead_shiptozipcode, cohead_shiptocountry,"
+                     "       cohead_shiptozipcode, cohead_shiptocountry, cohead_warehous_id,"
                      "       cust_ffshipto, custtype_code, cohead_commission, "
-                     "       shipto_num"
+                     "       shipto_num, "
+                     "       warehous_code "
                      "  FROM cohead"
                      "  JOIN custinfo ON (cohead_cust_id=cust_id)"
                      "  JOIN custtype ON (cust_custtype_id=custtype_id)"
                      "  LEFT OUTER JOIN shiptoinfo ON (cohead_shipto_id=shipto_id)"
+                     "  LEFT OUTER JOIN warehous ON (cohead_warehous_id=warehous_id)"
                      " WHERE (cohead_id=:cohead_id)"
                      " LIMIT 1;");
       // TODO: why left outer join shipto if we don't use the shipto_num?
@@ -667,6 +634,79 @@ void returnAuthorization::sOrigSoChanged()
       sohead.exec();
       if (sohead.first())
       {
+        // If the sites don't match the cohead ask the user if they want to make changes
+        if (_metrics->boolean("MultiWhs") && sohead.value("cohead_warehous_id").toInt() > 0 &&
+            (_warehouse->id() != sohead.value("cohead_warehous_id").toInt() ||
+             _shipWhs->id() != sohead.value("cohead_warehous_id").toInt()) )
+        {
+          if (_warehouse->id() != sohead.value("cohead_warehous_id").toInt() &&
+              _shipWhs->id() != sohead.value("cohead_warehous_id").toInt())
+          {
+            if (QMessageBox::question(this, tr("Sites Do Not Match"),
+                tr("The Orig. Sales Order Site (%1) does not match the Receiving Site (%2) nor Shipping Site (%3). <p>"
+                   "Do you want to update both of them to match the Sales Order?")
+                .arg(sohead.value("warehous_code").toString())
+                .arg(_warehouse->currentText())
+                .arg(_shipWhs->currentText()),
+                QMessageBox::Yes | QMessageBox::No,
+                QMessageBox::Yes) == QMessageBox::Yes)
+            {
+              _ignoreWhsSignals = true;
+              _warehouse->setId(sohead.value("cohead_warehous_id").toInt());
+              _shipWhs->setId(sohead.value("cohead_warehous_id").toInt());
+              _ignoreWhsSignals = false;
+            }
+          }
+          else if (_warehouse->id() != sohead.value("cohead_warehous_id").toInt())
+          {
+            if (QMessageBox::question(this, tr("Sites Do Not Match"),
+                tr("The Original Sales Order Site (%1) does not match the Receiving Site (%2). <p>"
+                   "Do you want to update it to match the Sales Order?")
+                .arg(sohead.value("warehous_code").toString())
+                .arg(_warehouse->currentText()),
+                QMessageBox::Yes | QMessageBox::No,
+                QMessageBox::Yes) == QMessageBox::Yes)
+            {
+              _ignoreWhsSignals = true;
+              _warehouse->setId(sohead.value("cohead_warehous_id").toInt());
+              _ignoreWhsSignals = false;
+            }
+          }
+          else if (_shipWhs->id() != sohead.value("cohead_warehous_id").toInt())
+          {
+            if (QMessageBox::question(this, tr("Sites Do Not Match"),
+                tr("The Original Sales Order Site (%1) does not match the Shipping Site (%2). <p>"
+                   "Do you want to update it to match the Sales Order?")
+                .arg(sohead.value("warehous_code").toString())
+                .arg(_shipWhs->currentText()),
+                QMessageBox::Yes | QMessageBox::No,
+                QMessageBox::Yes) == QMessageBox::Yes)
+            {
+              _ignoreWhsSignals = true;
+              _shipWhs->setId(sohead.value("cohead_warehous_id").toInt());
+              _ignoreWhsSignals = false;
+            }
+          }
+        }
+
+        if ( !_warehouse->isValid() )
+        {
+          QMessageBox::warning( this, tr("Invalid Receiving Site"),
+                               tr("<p>You must enter a valid Receiving Site." ) );
+          _warehouse->setFocus();
+          _origso->setId(-1);
+          return;
+        }
+
+        if ( !_shipWhs->isValid() )
+        {
+          QMessageBox::warning( this, tr("Invalid Shipping Site"),
+                               tr("<p>You must enter a valid Shipping Site." ) );
+          _shipWhs->setFocus();
+          _origso->setId(-1);
+          return;
+        }
+
         _salesRep->setId(sohead.value("cohead_salesrep_id").toInt());
         _commission->setDouble(sohead.value("cohead_commission").toDouble() * 100);
 
@@ -750,9 +790,9 @@ void returnAuthorization::populateShipto(int pShiptoid)
       _commission->setDouble(query.value("shipto_commission").toDouble() * 100);
       _ignoreShiptoSignals = false;
     }
-    else if (query.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Ship To Information"),
+                                  query, __FILE__, __LINE__))
     {
-      systemError(this, query.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -817,14 +857,15 @@ void returnAuthorization::sPopulateCustomerInfo()
         _custEmail = query.value("cust_soemaildelivery").toBool();
         populateShipto(query.value("shiptoid").toInt());
       }
-      else if (query.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Customer Information"),
+                                    query, __FILE__, __LINE__))
       {
-        systemError(this, query.lastError().databaseText(), __FILE__, __LINE__);
         return;
       }
     }
     else
     {
+      _custType->setText("");
       _origso->setCustId(-1);
       _salesRep->setCurrentIndex(-1);
       _taxzone->setId(-1);
@@ -870,9 +911,9 @@ void returnAuthorization::sCheckAuthorizationNumber()
 
       _mode = cEdit;
     }
-    else if (query.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                  query, __FILE__, __LINE__))
     {
-      systemError(this, query.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -1025,16 +1066,17 @@ void returnAuthorization::sDelete()
       }
       else if (checkwo.lastError().type() != QSqlError::NoError)
       {
-        systemError(this, checkwo.lastError().databaseText(), __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Work Order Information"),
+                             checkwo, __FILE__, __LINE__);
       }
 
       returnDelete.prepare( "DELETE FROM raitem "
                  "WHERE (raitem_id=:raitem_id);" );
       returnDelete.bindValue(":raitem_id", ((XTreeWidgetItem*)(selected[i]))->id());
       returnDelete.exec();
-      if (returnDelete.lastError().type() != QSqlError::NoError)
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting RA Item"),
+                                    returnDelete, __FILE__, __LINE__))
       {
-        systemError(this, returnDelete.lastError().databaseText(), __FILE__, __LINE__);
         return;
       }
     }
@@ -1044,92 +1086,32 @@ void returnAuthorization::sDelete()
 
 void returnAuthorization::sFillList()
 {
-  XSqlQuery returnFillList;
-  returnFillList.prepare( "SELECT raitem_id, "
-             "       CASE WHEN (raitem_status='C')     THEN -1"
-             "            WHEN (raitem_disposition='C') THEN 0"
-             "            WHEN (raitem_disposition='R') THEN 1"
-             "            WHEN (raitem_disposition='P') THEN 2"
-             "            WHEN (raitem_disposition='V') THEN 3"
-             "            WHEN (raitem_disposition='S') THEN 4"
-             "            ELSE -1 END AS disposition_code,"
-             "       formatRaLineNumber(raitem_id) AS f_linenumber, raitem_subnumber,"
-             "       item_number, uom_name, item_type,"
-             "       (item_descrip1 || ' ' || item_descrip2) AS item_descrip,"
-             "       (SELECT warehous_code FROM whsinfo JOIN itemsite ON itemsite_warehous_id=warehous_id" 
-             "            WHERE itemsite_id = COALESCE(raitem_coitem_itemsite_id, raitem_itemsite_id)) AS warehous_code,"
-             "       raitem_status, "
-             "       CASE WHEN (raitem_disposition='C') THEN :credit "
-             "            WHEN (raitem_disposition='R') THEN :return "
-             "            WHEN (raitem_disposition='P') THEN :replace "
-             "            WHEN (raitem_disposition='V') THEN :service "
-             "            WHEN (raitem_disposition='S') THEN :ship "
-             "            ELSE 'Error' END AS disposition, "
-             "       raitem_warranty,"
-             "       COALESCE(oc.coitem_qtyshipped,0) AS oldcoitem_qtyshipped,"
-             "       raitem_qtyauthorized,"
-             "       raitem_qtyreceived,"
-             "       qtyToReceive('RA', raitem_id) AS raitem_qtytoreceive,"
-             "       COALESCE(nc.coitem_qtyshipped,0) AS newcoitem_qtyshipped,"
-             "       raitem_unitprice,"
-             "       round((raitem_qtyauthorized * raitem_qty_invuomratio) * (raitem_unitprice / raitem_price_invuomratio),2) AS raitem_extprice,"
-             "       raitem_amtcredited,"
-             "       raitem_saleprice,"
-             "       round((raitem_qtyauthorized * raitem_qty_invuomratio) * (raitem_saleprice / raitem_price_invuomratio),2) AS raitem_extsaleprice,"
-             "       round((raitem_qtyauthorized * raitem_qty_invuomratio) * (raitem_unitprice / raitem_price_invuomratio),2) - "
-             "       round((raitem_qtyauthorized * raitem_qty_invuomratio) * (raitem_saleprice / raitem_price_invuomratio),2) AS raitem_netdue,"
-             "       och.cohead_number::text || '-' || oc.coitem_linenumber::text AS oldcohead_number, "
-             "       COALESCE(och.cohead_id,-1) AS oldcohead_number_xtidrole, "
-             "       nch.cohead_number::text || '-' || nc.coitem_linenumber::text AS newcohead_number, "
-             "       COALESCE(nch.cohead_id,-1) AS newcohead_number_xtidrole, "
-             "       raitem_scheddate, "
-             "       'qty' AS oldcoitem_qtyshipped_xtnumericrole, "
-             "       'qty' AS raitem_qtyauthorized_xtnumericrole, "
-             "       'qty' AS raitem_qtyreceived_xtnumericrole, "
-             "       'qty' AS raitem_qtytoreceive_xtnumericrole, "
-             "       'qty' AS newcoitem_qtyshipped_xtnumericrole, "
-             "       'salesprice' AS raitem_unitprice_xtnumericrole, "
-             "       'extprice' AS raitem_extprice_xtnumericrole, "
-             "       'curr' AS raitem_amtcredited_xtnumericrole, "
-             "       'salesprice' AS raitem_saleprice_xtnumericrole, "
-             "       'extprice' AS raitem_extsaleprice_xtnumericrole, "
-             "       'extprice' AS raitem_netdue_xtnumericrole, "
-             "        :na AS raitem_scheddate_xtnullrole,"
-             "       CASE WHEN raitem_subnumber = 0 THEN 0"
-             "            ELSE 1 END AS xtindentrole "
-             "FROM raitem "
-             " LEFT OUTER JOIN coitem oc ON (raitem_orig_coitem_id=oc.coitem_id) "
-             " LEFT OUTER JOIN cohead och ON (och.cohead_id=oc.coitem_cohead_id) "
-             " LEFT OUTER JOIN coitem nc ON (raitem_new_coitem_id=nc.coitem_id) "
-             " LEFT OUTER JOIN cohead nch ON (nch.cohead_id=nc.coitem_cohead_id),"
-             " itemsite, item, uom "
-             "WHERE ( (raitem_itemsite_id=itemsite_id)"
-             " AND (itemsite_item_id=item_id)"
-             " AND (raitem_rahead_id=:rahead_id) "
-             " AND (raitem_qty_uom_id=uom_id) ) "
-             "ORDER BY raitem_linenumber, raitem_subnumber;" );
-  returnFillList.bindValue(":rahead_id", _raheadid);
-  returnFillList.bindValue(":credit", tr("Credit"));
-  returnFillList.bindValue(":return", tr("Return"));
-  returnFillList.bindValue(":replace", tr("Replace"));
-  returnFillList.bindValue(":service", tr("Service"));
-  returnFillList.bindValue(":ship", tr("Ship"));
-  returnFillList.bindValue(":na", tr("N/A"));
-  returnFillList.exec();
+  MetaSQLQuery recvm = mqlLoad("returnAuthorizationItems", "list");
+
+  ParameterList params;
+  params.append("rahead_id", _raheadid);
+  params.append("credit",  tr("Credit"));
+  params.append("return",  tr("Return"));
+  params.append("replace", tr("Replace"));
+  params.append("service", tr("Service"));
+  params.append("ship",    tr("Ship"));
+  params.append("na",      tr("N/A"));
+
+  XSqlQuery returnFillList = recvm.toQuery(params);
   _raitem->populate(returnFillList, true);
-  if ((returnFillList.first()) && (_mode == cEdit))
+  if (returnFillList.first() && _mode == cEdit)
   {
     _cust->setDisabled(_cust->isValid());
   }
-  else if (returnFillList.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                returnFillList, __FILE__, __LINE__))
   {
-    systemError(this, returnFillList.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
   sCalculateSubtotal();
 
-  if (_creditBy->currentIndex() != 0 && _calcfreight)
+  if (_calcfreight)
   {
     returnFillList.prepare("SELECT SUM(freightdata_total) AS freight "
               "FROM freightDetail('RA', :head_id, :cust_id, :shipto_id, :orderdate, :shipvia, :curr_id);");
@@ -1143,13 +1125,11 @@ void returnAuthorization::sFillList()
     if (returnFillList.first())
     {
       _freightCache = returnFillList.value("freight").toDouble();
-      disconnect(_freight, SIGNAL(valueChanged()), this, SLOT(sFreightChanged()));
       _freight->setLocalValue(_freightCache);
-      connect(_freight, SIGNAL(valueChanged()), this, SLOT(sFreightChanged()));
     }
-    else if (returnFillList.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                  returnFillList, __FILE__, __LINE__))
     {
-      systemError(this, returnFillList.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -1197,7 +1177,8 @@ void returnAuthorization::sFillList()
     _commission->setEnabled(true);
     _taxzone->setEnabled(true);
     _disposition->setEnabled(true);
-    _timing->setEnabled(true);
+// this overrides the setting of _timing in sDispositionChanged
+//    _timing->setEnabled(true);
     _cust->setEnabled(!_origso->isValid());
     _billToName->setEnabled(true);
     _billToAddr->setEnabled(true);
@@ -1275,19 +1256,9 @@ void returnAuthorization::populate()
     if (!rahead.value("rahead_rsncode_id").isNull() && rahead.value("rahead_rsncode_id").toInt() != -1)
       _rsnCode->setId(rahead.value("rahead_rsncode_id").toInt());
 
-    if (rahead.value("rahead_timing").toString() == "I")
-      _timing->setCurrentIndex(0);
-    else
-      _timing->setCurrentIndex(1);
+    _timing->setCode(rahead.value("rahead_timing").toString());
 
-    if (rahead.value("rahead_creditmethod").toString() == "N")
-      _creditBy->setCurrentIndex(0);
-    else if (rahead.value("rahead_creditmethod").toString() == "M")
-      _creditBy->setCurrentIndex(1);
-    else if (rahead.value("rahead_creditmethod").toString() == "K")
-      _creditBy->setCurrentIndex(2);
-    else if (rahead.value("rahead_creditmethod").toString() == "C")
-      _creditBy->setCurrentIndex(3);
+    _creditBy->setCode(rahead.value("rahead_creditmethod").toString());
 
     _ignoreSoSignals = true;
     _origso->setId(rahead.value("rahead_orig_cohead_id").toInt());
@@ -1333,10 +1304,8 @@ void returnAuthorization::populate()
     // Auto calculated _freight is populated in sFillItemList
     if (!_calcfreight)
     {
-      disconnect(_freight, SIGNAL(valueChanged()), this, SLOT(sFreightChanged()));
       _freightCache = rahead.value("rahead_freight").toDouble();
       _freight->setLocalValue(_freightCache);
-      connect(_freight, SIGNAL(valueChanged()), this, SLOT(sFreightChanged()));
     }
 
 
@@ -1363,23 +1332,14 @@ void returnAuthorization::populate()
     }
 
     _ignoreSoSignals = true;
-    if (rahead.value("rahead_disposition").toString() == "C")
-      sDispositionChanged();
-    else if (rahead.value("rahead_disposition").toString() == "R")
-      _disposition->setCurrentIndex(1);
-    else if (rahead.value("rahead_disposition").toString() == "P")
-      _disposition->setCurrentIndex(2);
-    else if (rahead.value("rahead_disposition").toString() == "V")
-      _disposition->setCurrentIndex(3);
-    else if (rahead.value("rahead_disposition").toString() == "M")
-      _disposition->setCurrentIndex(4);
+    _disposition->setCode(rahead.value("rahead_disposition").toString());
     _ignoreSoSignals = false;
     _saved = true;
     sFillList();
   }
-  else if (rahead.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                rahead, __FILE__, __LINE__))
   {
-    systemError(this, rahead.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -1396,9 +1356,9 @@ void returnAuthorization::closeEvent(QCloseEvent *pEvent)
                "WHERE (rahead_id=:rahead_id);" );
     returncloseEvent.bindValue(":rahead_id", _raheadid);
     returncloseEvent.exec();
-    if (returncloseEvent.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Updating RA Item Information"),
+                                  returncloseEvent, __FILE__, __LINE__))
     {
-      systemError(this, returncloseEvent.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     else
@@ -1413,9 +1373,9 @@ void returnAuthorization::closeEvent(QCloseEvent *pEvent)
       returncloseEvent.prepare("SELECT releaseRaNumber(:number) AS result;");
       returncloseEvent.bindValue(":number", _authNumber->text());
       returncloseEvent.exec();
-      if (returncloseEvent.lastError().type() != QSqlError::NoError)
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Updating RA Item Information"),
+                                    returncloseEvent, __FILE__, __LINE__))
       {
-        systemError(this, returncloseEvent.lastError().databaseText(), __FILE__, __LINE__);
         return;
       }
     }
@@ -1451,9 +1411,9 @@ void returnAuthorization::sCalculateTax()
   taxq.exec();
   if (taxq.first())
     _tax->setLocalValue(taxq.value("tax").toDouble());
-  else if (taxq.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Calculating Tax"),
+                                taxq, __FILE__, __LINE__))
   {
-    systemError(this, taxq.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -1514,25 +1474,25 @@ void returnAuthorization::sShipWhsChanged()
 void returnAuthorization::sDispositionChanged()
 {
   _new->setEnabled(_cust->isValid() ||
-              (_disposition->currentIndex() == 1 && _creditBy->currentIndex() == 0));
+              (_disposition->code() == "R" && _creditBy->code() == "N"));
 
   bool enableReceipt = _privileges->check("EnterReceipts") &&
-                      (_disposition->currentIndex() != 0);
+                      (_disposition->code() != "C");
 
   _receiveAll->setEnabled(enableReceipt);
   _postReceipts->setEnabled(enableReceipt);
 
-  if (_disposition->currentIndex() == 0)
+  if (_disposition->code() == "C")
   {
-    _timing->setCurrentIndex(0);
+    _timing->setCode("I");
     _timing->setEnabled(false);
-    if (_creditBy->currentIndex() == 0)
-      _creditBy->setCurrentIndex(1);
+    if (_creditBy->code() == "N")
+      _creditBy->setCode("M");
   }
   else
     _timing->setEnabled(true);
 
-  _refund->setEnabled(_creditBy->currentIndex() == 3);
+  _refund->setEnabled(_creditBy->code() == "C");
 
   if (!_ignoreSoSignals)
   {
@@ -1545,17 +1505,17 @@ void returnAuthorization::sDispositionChanged()
 void returnAuthorization::sCreditByChanged()
 {
   _new->setEnabled(_cust->isValid() ||
-                   (_disposition->currentIndex() == 1 && _creditBy->currentIndex() == 0));
+                   (_disposition->code() == "R" && _creditBy->code() == "N"));
   
-  if (_creditBy->currentIndex() == 0 && _total->localValue() > 0)
+  if (_creditBy->code() == "N" && _total->localValue() > 0)
   {
     QMessageBox::information(this, tr("Credit By 'None' not allowed"),
                           tr("<p>This Return Authorization has authorized "
                              "credit amounts. You may not set the Credit By "
                              "to 'None' unless all credit amounts are zero."));
-    _creditBy->setCurrentIndex(1);
+    _creditBy->setCode("M");
   }
-  else if (_creditBy->currentIndex() == 0  || _total->localValue() == 0)
+  else if (_creditBy->code() == "N" || _total->localValue() == 0)
   {
     _currency->setEnabled(true);
     _miscChargeDescription->setEnabled(false);
@@ -1583,9 +1543,9 @@ void returnAuthorization::sAuthorizeLine()
     returnAuthorizeLine.prepare(sql);
     returnAuthorizeLine.bindValue(":raitem_id",  ((XTreeWidgetItem*)(selected[i]))->id());
     returnAuthorizeLine.exec();
-    if (returnAuthorizeLine.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                  returnAuthorizeLine, __FILE__, __LINE__))
     {
-      systemError(this, returnAuthorizeLine.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -1605,10 +1565,10 @@ void returnAuthorization::sClearAuthorization()
         returnClearAuthorization.prepare ( "SELECT clearReturnItem(:raitem_id) AS result;" );
         returnClearAuthorization.bindValue(":raitem_id",  ((XTreeWidgetItem*)(selected[i]))->id());
         returnClearAuthorization.exec();
-        if (returnClearAuthorization.lastError().type() != QSqlError::NoError)
+        if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                      returnClearAuthorization, __FILE__, __LINE__))
         {
-           systemError(this, returnClearAuthorization.lastError().databaseText(), __FILE__, __LINE__);
-           return;
+          return;
         }
   }
 
@@ -1627,9 +1587,9 @@ void returnAuthorization::sAuthorizeAll()
   returnAuthorizeAll.prepare(sql);
   returnAuthorizeAll.bindValue(":raitem_rahead_id",  _raheadid);
   returnAuthorizeAll.exec();
-  if (returnAuthorizeAll.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                returnAuthorizeAll, __FILE__, __LINE__))
   {
-    systemError(this, returnAuthorizeAll.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -1655,9 +1615,9 @@ void returnAuthorization::sEnterReceipt()
     params.append("recv_id", returnEnterReceipt.value("recv_id"));
     params.append("mode", "edit");
   }
-  else if (returnEnterReceipt.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                returnEnterReceipt, __FILE__, __LINE__))
   {
-    systemError(this, returnEnterReceipt.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   else
@@ -1693,15 +1653,16 @@ void returnAuthorization::sReceiveAll()
     int result = returnReceiveAll.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("enterReceipt", result),
-                  __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Receiving RA Items"),
+                             storedProcErrorLookup("enterReceipt", result),
+                             __FILE__, __LINE__);
       return;
     }
     omfgThis->sPurchaseOrderReceiptsUpdated();
   }
-  if (returnReceiveAll.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Receiving RA Items"),
+                                returnReceiveAll, __FILE__, __LINE__))
   {
-    systemError(this, returnReceiveAll.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -1750,9 +1711,9 @@ void returnAuthorization::sAction()
   else
     returnAction.bindValue(":status",QString("C"));
   returnAction.exec();
-  if (returnAction.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Updating RA Item Information"),
+                                        returnAction, __FILE__, __LINE__))
   {
-    systemError(this, returnAction.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   else
@@ -1774,9 +1735,7 @@ void returnAuthorization::sRefund()
 
   XSqlQuery begin("BEGIN;");
 
-  bool _post = _disposition->currentIndex() == 0 &&
-               _timing->currentIndex() == 0 &&
-               _creditBy->currentIndex() == 3;
+  bool _post = _disposition->code() == "C" && _timing->code() == "I" && _creditBy->code() == "C";
 
   XSqlQuery cmq;
   cmq.prepare("SELECT createRaCreditMemo(:rahead_id,:post) AS result;");
@@ -1794,8 +1753,8 @@ void returnAuthorization::sRefund()
     if (ccq.first())
     {
       int ccpayid = ccq.value("ccpay_id").toInt();
-      QMessageBox::information( this, tr("New Return Created"),
-                                tr("<p>A new Return has been created and "
+      QMessageBox::information( this, tr("New Sales Credit Created"),
+                                tr("<p>A new Sales Credit has been created and "
                                    "assigned #%1")
                                    .arg(ccq.value("cmhead_number").toString()));
       CreditCardProcessor *cardproc = CreditCardProcessor::getProcessor();
@@ -1859,9 +1818,10 @@ void returnAuthorization::sRefund()
     }
     else if (ccq.lastError().type() != QSqlError::NoError)
     {
-      systemError(this, ccq.lastError().databaseText(), __FILE__, __LINE__);
-      XSqlQuery rollback("ROLLBACK;");
-      return;
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Credit Memo Information"),
+                                              ccq, __FILE__, __LINE__);
+       XSqlQuery rollback("ROLLBACK;");
+       return;
     }
     else
     {
@@ -1875,7 +1835,7 @@ void returnAuthorization::sRefund()
   else if (cmq.lastError().type() != QSqlError::NoError)
   {
     XSqlQuery rollback("ROLLBACK;");
-    ErrorReporter::error(QtCriticalMsg, this, tr("Creating R/A Return"),
+    ErrorReporter::error(QtCriticalMsg, this, tr("Creating R/A Sales Credit"),
                          cmq, __FILE__, __LINE__);
     return;
   }
@@ -2031,9 +1991,9 @@ void returnAuthorization::sCheckNumber()
     _authNumber->setEnabled(false);
     _cust->setReadOnly(true);
   }
-  else if (returnCheckNumber.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                        returnCheckNumber, __FILE__, __LINE__))
   {
-    systemError(this, returnCheckNumber.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   sSave(true);
@@ -2060,9 +2020,7 @@ void returnAuthorization::sFreightChanged()
         _calcfreight = false;
       else
       {
-        disconnect(_freight, SIGNAL(valueChanged()), this, SLOT(sFreightChanged()));
         _freight->setLocalValue(_freightCache);
-        connect(_freight, SIGNAL(valueChanged()), this, SLOT(sFreightChanged()));
       }
     }
     else if ( (!_calcfreight) &&
@@ -2079,9 +2037,7 @@ void returnAuthorization::sFreightChanged()
       if (answer == QMessageBox::Yes)
       {
         _calcfreight = true;
-        disconnect(_freight, SIGNAL(valueChanged()), this, SLOT(sFreightChanged()));
         _freight->setLocalValue(_freightCache);
-        connect(_freight, SIGNAL(valueChanged()), this, SLOT(sFreightChanged()));
       }
     }
     else
