@@ -24,6 +24,8 @@ class returnAuthorizationItem : public XDialog, public Ui::returnAuthorizationIt
 public:
     returnAuthorizationItem(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~returnAuthorizationItem();
+    Q_INVOKABLE virtual int raitemId() { return _raitemid; }
+    Q_INVOKABLE virtual int coitemId() { return _coitemid; }
 
 public slots:
 	
@@ -40,8 +42,10 @@ public slots:
     void sCalculateTax();
     void sPriceGroup();
     void sTaxDetail();
+    void sPopulateUOM();
     void sQtyUOMChanged();
     void sPriceUOMChanged();
+    void sSalePriceUOMChanged();
     void sDispositionChanged();
     void sDetermineAvailability();
     void sHandleWo( bool pCreate );
