@@ -12,11 +12,13 @@
 
 #include <QVariant>
 #include <QMessageBox>
+#include "guiErrorCheck.h"
 #include <QSqlError>
 
 #include "distributeInventory.h"
 #include "inputManager.h"
 #include "storedProcErrorLookup.h"
+#include "errorReporter.h"
 
 returnWoMaterialItem::returnWoMaterialItem(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
@@ -80,59 +82,108 @@ enum SetResponse returnWoMaterialItem::set(const ParameterList &pParams)
    }
   }
 
+  param = pParams.value("wo_id", &valid);
+  if (valid)
+  {
+    _wo->setId(param.toInt());
+    _wo->setEnabled(false);
+    _womatl->setFocus();
+  }
+
   return NoError;
 }
 
 void returnWoMaterialItem::sReturn()
 {
-  if (!_transDate->isValid())
-  {
-    QMessageBox::critical(this, tr("Invalid date"),
-                          tr("You must enter a valid transaction date.") );
-    _transDate->setFocus();
+  int itemlocSeries;
+
+  QList<GuiErrorCheck> errors;
+  errors<< GuiErrorCheck(!_transDate->isValid(), _transDate,
+                         tr("You must enter a valid transaction date."));
+  errors<< GuiErrorCheck(!_wo->isValid(), _wo,
+                         tr("You must select the Work Order from which you wish to return Material"))
+  ;
+  if (GuiErrorCheck::reportErrors(this, tr("Invalid Return"), errors))
     return;
-  }
-  else if (!_wo->isValid())
+
+  // Stage cleanup function to be called on error
+  XSqlQuery cleanup;
+  cleanup.prepare("SELECT deleteitemlocseries(:itemlocSeries, TRUE);");
+  
+  // Get parent series id
+  XSqlQuery parentSeries;
+  parentSeries.prepare("SELECT NEXTVAL('itemloc_series_seq') AS result;");
+  parentSeries.exec();
+  if (parentSeries.first() && parentSeries.value("result").toInt() > 0)
   {
-    QMessageBox::critical( this, tr("Select Work Order"),
-                           tr("You must select the Work Order from which you with to return Materal") );
-    _wo->setFocus();
+    itemlocSeries = parentSeries.value("result").toInt();
+    cleanup.bindValue(":itemlocSeries", itemlocSeries);
+  }
+  else
+  {
+    ErrorReporter::error(QtCriticalMsg, this, tr("Failed to Retrieve the Next itemloc_series_seq"),
+      parentSeries, __FILE__, __LINE__);
     return;
   }
 
-  XSqlQuery rollback;
-  rollback.prepare("ROLLBACK;");
+  // If controlled item: create the parent itemlocdist record, call distributeInventory::seriesAdjust
+  if (_controlledItem)
+  {
+    XSqlQuery parentItemlocdist;
+    parentItemlocdist.prepare("SELECT createitemlocdistparent(:itemsite_id, "
+                              " itemuomtouom(itemsite_item_id, womatl_uom_id, NULL, :qty), "
+                              " 'WO', womatl_id, :itemlocSeries, NULL, NULL, 'IM') AS result "
+                              "FROM womatl "
+                              " JOIN itemsite ON womatl_itemsite_id = itemsite_id "
+                              "WHERE womatl_id = :womatl_id;");
+    parentItemlocdist.bindValue(":itemsite_id", _itemsiteId);
+    parentItemlocdist.bindValue(":qty", _qty->toDouble());
+    parentItemlocdist.bindValue(":womatl_id", _womatl->id());
+    parentItemlocdist.bindValue(":itemlocSeries", itemlocSeries);
+    parentItemlocdist.exec();
+    if (parentItemlocdist.first())
+    {
+      if (distributeInventory::SeriesAdjust(itemlocSeries, this, QString(), QDate(),
+        QDate(), true) == XDialog::Rejected)
+      {
+        cleanup.exec();
+        QMessageBox::information(this, tr("Material Return"), 
+          tr("Transaction Canceled") );
+        return;
+      }
+    }
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Creating itemlocdist Records"),
+                              parentItemlocdist, __FILE__, __LINE__))
+    {
+      return;
+    }
+  }
 
+  // Proceed to posting inventory transaction
   XSqlQuery returnItem;
-  returnItem.exec("BEGIN;");	// because of possible lot, serial, or location distribution cancelations
-  returnItem.prepare("SELECT returnWoMaterial(:womatl_id, :qty, :date) AS result;");
+  returnItem.prepare("SELECT returnWoMaterial(:womatl_id, :qty, :itemlocSeries, :date, false, TRUE) AS result;");
   returnItem.bindValue(":womatl_id", _womatl->id());
   returnItem.bindValue(":qty", _qty->toDouble());
+  returnItem.bindValue(":itemlocSeries", itemlocSeries);
   returnItem.bindValue(":date",  _transDate->date());
   returnItem.exec();
   if (returnItem.first())
   {
     int result = returnItem.value("result").toInt();
-    if (result < 0)
+    if (result < 0 || result != itemlocSeries)
     {
-      rollback.exec();
-      systemError(this, storedProcErrorLookup("returnWoMaterial", result),
-                  __FILE__, __LINE__);
+      cleanup.exec();
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Work Order Information"),
+                           storedProcErrorLookup("returnWoMaterial", result),
+                           __FILE__, __LINE__);
       return;
     }
-    else if (distributeInventory::SeriesAdjust(returnItem.value("result").toInt(), this) == XDialog::Rejected)
-    {
-      rollback.exec();
-      QMessageBox::information( this, tr("Material Return"), tr("Transaction Canceled") );
-      return;
-    }
-
-    returnItem.exec("COMMIT;");
   }
   else if (returnItem.lastError().type() != QSqlError::NoError)
   {
-    rollback.exec();
-    systemError(this, returnItem.lastError().databaseText(), __FILE__, __LINE__);
+    cleanup.exec();
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Work Order Information"),
+                         returnItem, __FILE__, __LINE__);
     return;
   }
 
@@ -169,24 +220,26 @@ void returnWoMaterialItem::sSetQOH(int pWomatlid)
     }
     
     XSqlQuery qoh;
-    qoh.prepare( "SELECT itemuomtouom(itemsite_item_id, NULL, womatl_uom_id, qtyAvailable(itemsite_id)) AS availableqoh,"
-                 "       uom_name "
-                 "  FROM womatl, itemsite, uom"
-                 " WHERE((womatl_itemsite_id=itemsite_id)"
-                 "   AND (womatl_uom_id=uom_id)"
-                 "   AND (womatl_id=:womatl_id) );" );
+    qoh.prepare( "SELECT itemuomtouom(itemsite_item_id, NULL, womatl_uom_id, "
+                 "  qtyAvailable(itemsite_id)) AS availableqoh, uom_name, itemsite_id, "
+                 "  isControlledItemsite(itemsite_id) AS controlled "
+                 "FROM womatl, itemsite, uom "
+                 "WHERE womatl_itemsite_id = itemsite_id "
+                 "  AND womatl_uom_id = uom_id "
+                 "  AND womatl_id=:womatl_id;" );
     qoh.bindValue(":womatl_id", pWomatlid);
     qoh.exec();
     if (qoh.first())
     {
+      _itemsiteId = qoh.value("itemsite_id").toInt();
+      _controlledItem = qoh.value("controlled").toBool();
       _uom->setText(qoh.value("uom_name").toString());
       _cachedQOH = qoh.value("availableqoh").toDouble();
       _beforeQty->setDouble(_cachedQOH);
     }
     else
-      systemError(this, tr("A System Error occurred at %1::%2.")
-                        .arg(__FILE__)
-                        .arg(__LINE__) );
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Work Order Item Information"),
+                         qoh, __FILE__, __LINE__);
   }
   sUpdateQty();
 }
