@@ -21,7 +21,7 @@ class returnWoMaterialItem : public XDialog, public Ui::returnWoMaterialItem
     Q_OBJECT
 
 public:
-    returnWoMaterialItem(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    returnWoMaterialItem(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~returnWoMaterialItem();
 
 public slots:
@@ -34,6 +34,8 @@ protected slots:
     virtual void languageChange();
 
 private:
+    int _itemsiteId;
+    bool _controlledItem;
     bool _captive;
     double _cachedQOH;
 
