@@ -21,7 +21,7 @@ class sale : public XDialog, public Ui::sale
     Q_OBJECT
 
 public:
-    sale(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    sale(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~sale();
 
 public slots:
