@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -13,12 +13,12 @@
 
 #include "applock.h"
 #include "guiclient.h"
-#include "xwidget.h"
+#include "xdocumentwindow.h"
 #include <parameter.h>
 #include "ui_salesOrder.h"
 #include "dspShipmentsBySalesOrder.h"
 
-class salesOrder : public XWidget, public Ui::salesOrder
+class salesOrder : public XDocumentWindow, public Ui::salesOrder
 {
   Q_OBJECT
 
@@ -34,6 +34,7 @@ class salesOrder : public XWidget, public Ui::salesOrder
     Q_INVOKABLE virtual int   id() { return _soheadid; }
     Q_INVOKABLE virtual int   modeType() const;
     Q_INVOKABLE virtual int   modeState() const;
+    Q_INVOKABLE virtual bool  creditLimitCheckIssue();
 
   public slots:
     virtual SetResponse set(const ParameterList &pParams );
@@ -55,6 +56,7 @@ class salesOrder : public XWidget, public Ui::salesOrder
     virtual void        sFirm();
     virtual void        sSoften();
     virtual void        sAction();
+    virtual void        sCancel();
     virtual void        sDelete();
     virtual void        populate();
     virtual void        sFillItemList();
@@ -107,6 +109,8 @@ class salesOrder : public XWidget, public Ui::salesOrder
     virtual void        sViewPR();
     virtual void        sPopulateShipments();
     virtual void        sViewItemWorkbench();
+    virtual bool        creditLimitCheck();
+    virtual void	sHoldTypeChanged();
 
   protected slots:
     virtual void  languageChange();
@@ -126,29 +130,29 @@ class salesOrder : public XWidget, public Ui::salesOrder
     bool    deleteForCancel();
 
     bool    _saved;
+    bool    _saving;
     bool    _calcfreight;
     int     _orderNumberGen;
     double  _freightCache;
-    double  _amountOutstanding;
-    double  _amountAllocated;
-    double  _taxableSubtotal;
     bool    _userEnteredOrderNumber;
     bool    _ignoreSignals;
     bool    _blanketPos;
     bool    _usesPos;
-    bool    _ffShipto;
     bool    _captive;
+    bool    _holdOverride;
     int     _soheadid;
     int     _lineMode;
+    int     _lineFirm;
     AppLock _lock;
     int     _mode;
     int     _numSelected;
-    int     _originalPrjid;
     int     _custtaxzoneid;
     int     _taxzoneidCache;
     int     _crmacctid;
+    int     _custWhs;
     QDate   _orderDateCache;
     QDate   _shipDateCache;
+    QString _holdTypeCache;
 };
 
 #endif  // SALESORDER_H
