@@ -76,9 +76,9 @@ enum SetResponse salesRep::set(const ParameterList &pParams)
       saleset.exec("SELECT NEXTVAL('salesrep_salesrep_id_seq') AS salesrep_id;");
       if (saleset.first())
         _salesrepid = saleset.value("salesrep_id").toInt();
-      else if (saleset.lastError().type() == QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Sales Rep Information"),
+                                    saleset, __FILE__, __LINE__))
       {
-        systemError(this, saleset.lastError().text(), __FILE__, __LINE__);
         return UndefinedError;
       }
 
