@@ -21,13 +21,13 @@ class scrapTrans : public XWidget, public Ui::scrapTrans
     Q_OBJECT
 
 public:
-    scrapTrans(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = Qt::Window);
+    scrapTrans(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::Window);
     ~scrapTrans();
 
 public slots:
     virtual enum SetResponse set(const ParameterList & pParams );
     virtual void sPost();
-    virtual void sPopulateQOH( int pWarehousid );
+    virtual void sPopulateQOH();
     virtual void sPopulateQty();
 
 protected slots:
@@ -35,7 +35,9 @@ protected slots:
 
 private:
     int _mode;
+    int _itemsiteId;
     bool _captive;
+    bool _controlledItem;
     double _cachedQOH;
 
 };
