@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2018 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -9,6 +9,7 @@
  */
 
 #include "scripttoolbox.h"
+#include "scriptapi_internal.h"
 
 #include <QBoxLayout>
 #include <QDateTime>
@@ -30,7 +31,9 @@
 #include <QTextStream>
 #include <QUrl>
 #include <QWidget>
+#if QT_VERSION < 0x050900
 #include <QWebView>
+#endif
 
 #include <parameter.h>
 #include <metasql.h>
@@ -38,7 +41,6 @@
 
 #include "creditCard.h"
 #include "creditcardprocessor.h"
-#include "mqlutil.h"
 #include "storedProcErrorLookup.h"
 #include "xdialog.h"
 #include "xmainwindow.h"
@@ -46,6 +48,7 @@
 #include "display.h"
 #include "xuiloader.h"
 #include "getscreen.h"
+#include "errorReporter.h"
 
 /** @ingroup scriptapi
 
@@ -200,7 +203,6 @@ XSqlQuery ScriptToolbox::executeQuery(const QString & query, const ParameterList
 
   @return The XSqlQuery object after calling XSqlQuery::exec()
 
-  @see mqlLoad
   @see MetaSQLQuery
   @see XSqlQuery
   @see QSqlQuery
@@ -208,7 +210,7 @@ XSqlQuery ScriptToolbox::executeQuery(const QString & query, const ParameterList
 XSqlQuery ScriptToolbox::executeDbQuery(const QString & group, const QString & name)
 {
   ParameterList params;
-  MetaSQLQuery mql = mqlLoad(group, name);
+  MetaSQLQuery mql(omfgThis->_mqlhash->value(group, name));
   return mql.toQuery(params);
 }
 
@@ -250,7 +252,7 @@ XSqlQuery ScriptToolbox::executeDbQuery(const QString & group, const QString & n
  */
 XSqlQuery ScriptToolbox::executeDbQuery(const QString & group, const QString & name, const ParameterList & params)
 {
-  MetaSQLQuery mql = mqlLoad(group, name);
+  MetaSQLQuery mql(omfgThis->_mqlhash->value(group, name));
   return mql.toQuery(params);
 }
 /** @example ccvoid.js */
@@ -289,6 +291,7 @@ XSqlQuery ScriptToolbox::executeRollback()
  */
 QObject * ScriptToolbox::qtyVal()
 {
+  scriptDeprecated("Use mainwindow.qtyVal() instead of toolbox.qtyVal()");
   QValidator * val = omfgThis->qtyVal();
   return val;
 }
@@ -298,6 +301,7 @@ QObject * ScriptToolbox::qtyVal()
  */
 QObject * ScriptToolbox::TransQtyVal()
 {
+  scriptDeprecated("Use mainwindow.transQtyVal() instead of toolbox.TransQtyVal()");
   QValidator * val = omfgThis->transQtyVal();
   return val;
 }
@@ -307,6 +311,7 @@ QObject * ScriptToolbox::TransQtyVal()
  */
 QObject * ScriptToolbox::qtyPerVal()
 {
+  scriptDeprecated("Use mainwindow.qtyPerVal() instead of toolbox.qtyPerVal()");
   QValidator * val = omfgThis->qtyPerVal();
   return val;
 }
@@ -316,6 +321,7 @@ QObject * ScriptToolbox::qtyPerVal()
  */
 QObject * ScriptToolbox::percentVal()
 {
+  scriptDeprecated("Use mainwindow.percentVal() instead of toolbox.percentVal()");
   QValidator * val = omfgThis->percentVal();
   return val;
 }
@@ -325,6 +331,7 @@ QObject * ScriptToolbox::percentVal()
  */
 QObject * ScriptToolbox::moneyVal()
 {
+  scriptDeprecated("Use mainwindow.moneyVal() instead of toolbox.moneyVal()");
   QValidator * val = omfgThis->moneyVal();
   return val;
 }
@@ -334,6 +341,7 @@ QObject * ScriptToolbox::moneyVal()
  */
 QObject * ScriptToolbox::negMoneyVal()
 {
+  scriptDeprecated("Use mainwindow.negMoneyVal() instead of toolbox.negMoneyVal()");
   QValidator * val = omfgThis->negMoneyVal();
   return val;
 }
@@ -343,6 +351,7 @@ QObject * ScriptToolbox::negMoneyVal()
  */
 QObject * ScriptToolbox::priceVal()
 {
+  scriptDeprecated("Use mainwindow.priceVal() instead of toolbox.priceVal()");
   QValidator * val = omfgThis->priceVal();
   return val;
 }
@@ -352,6 +361,7 @@ QObject * ScriptToolbox::priceVal()
  */
 QObject * ScriptToolbox::costVal()
 {
+  scriptDeprecated("Use mainwindow.costVal() instead of toolbox.costVal()");
   QValidator * val = omfgThis->costVal();
   return val;
 }
@@ -361,6 +371,7 @@ QObject * ScriptToolbox::costVal()
  */
 QObject * ScriptToolbox::ratioVal()
 {
+  scriptDeprecated("Use mainwindow.ratioVal() instead of toolbox.ratioVal()");
   QValidator * val = omfgThis->ratioVal();
   return val;
 }
@@ -370,6 +381,7 @@ QObject * ScriptToolbox::ratioVal()
  */
 QObject * ScriptToolbox::weightVal()
 {
+  scriptDeprecated("Use mainwindow.weightVal() instead of toolbox.weightVal()");
   QValidator * val = omfgThis->weightVal();
   return val;
 }
@@ -379,6 +391,7 @@ QObject * ScriptToolbox::weightVal()
  */
 QObject * ScriptToolbox::runTimeVal()
 {
+  scriptDeprecated("Use mainwindow.runTimeVal() instead of toolbox.runTimeVal()");
   QValidator * val = omfgThis->runTimeVal();
   return val;
 }
@@ -388,6 +401,7 @@ QObject * ScriptToolbox::runTimeVal()
  */
 QObject * ScriptToolbox::orderVal()
 {
+  scriptDeprecated("Use mainwindow.orderVal() instead of toolbox.orderVal()");
   QValidator * val = omfgThis->orderVal();
   return val;
 }
@@ -397,6 +411,7 @@ QObject * ScriptToolbox::orderVal()
  */
 QObject * ScriptToolbox::dayVal()
 {
+  scriptDeprecated("Use mainwindow.dayVal() instead of toolbox.dayVal()");
   QValidator * val = omfgThis->dayVal();
   return val;
 }
@@ -406,6 +421,7 @@ QObject * ScriptToolbox::dayVal()
  */
 QObject * ScriptToolbox::createGridLayout()
 {
+  scriptDeprecated("Use QGridLayout() instead of toolbox.createGridLayout()");
   QGridLayout * layout = new QGridLayout;
   return layout;
 }
@@ -445,6 +461,7 @@ QObject * ScriptToolbox::widgetGetLayout(QWidget * w)
   */
 void ScriptToolbox::layoutGridAddLayout(QObject * parent, QObject * child, int row, int column, int alignment)
 {
+  scriptDeprecated("Use QGridLayout::addLayout instead of toolbox.layoutGridAddLayout()");
   QGridLayout * parentLayout = qobject_cast<QGridLayout*>(parent);
   QGridLayout * childLayout = qobject_cast<QGridLayout*>(child);
 
@@ -457,6 +474,7 @@ void ScriptToolbox::layoutGridAddLayout(QObject * parent, QObject * child, int r
   */
 void ScriptToolbox::layoutBoxInsertWidget(QObject * obj, int index, QWidget * widget, int stretch, int alignment)
 {
+  scriptDeprecated("Use QBoxLayout::insertWidget() instead of toolbox.layoutBoxInsertWidget()");
   QBoxLayout * layout = qobject_cast<QBoxLayout*>(obj);
   if(layout && widget)
     layout->insertWidget(index, widget, stretch, (Qt::Alignment)alignment);
@@ -467,6 +485,7 @@ void ScriptToolbox::layoutBoxInsertWidget(QObject * obj, int index, QWidget * wi
   */
 void ScriptToolbox::layoutGridAddWidget(QObject * obj, QWidget * widget, int row, int column, int alignment)
 {
+  scriptDeprecated("Use QGridLayout::addWidget() instead of toolbox.layoutGridAddWidget()");
   QGridLayout * layout = qobject_cast<QGridLayout*>(obj);
   if(layout && widget)
     layout->addWidget(widget, row, column, (Qt::Alignment)alignment);
@@ -477,6 +496,7 @@ void ScriptToolbox::layoutGridAddWidget(QObject * obj, QWidget * widget, int row
   */
 void ScriptToolbox::layoutGridAddWidget(QObject * obj, QWidget * widget, int fromRow, int fromColumn, int rowSpan, int columnSpan, int alignment)
 {
+  scriptDeprecated("Use QGridLayout::addWidget() instead of toolbox.layoutGridAddWidget()");
   QGridLayout * layout = qobject_cast<QGridLayout*>(obj);
   if(layout && widget)
     layout->addWidget(widget, fromRow, fromColumn, rowSpan, columnSpan, (Qt::Alignment)alignment);
@@ -487,6 +507,7 @@ void ScriptToolbox::layoutGridAddWidget(QObject * obj, QWidget * widget, int fro
   */
 void ScriptToolbox::layoutStackedInsertWidget(QObject * obj, int index, QWidget * widget)
 {
+  scriptDeprecated("Use QStackedWidget::insertWidget instead of toolbox.layoutStackedInsertWidget()");
   QStackedLayout * layout = qobject_cast<QStackedLayout*>(obj);
   if(layout && widget)
     layout->insertWidget(index, widget);
@@ -497,6 +518,7 @@ void ScriptToolbox::layoutStackedInsertWidget(QObject * obj, int index, QWidget
   */
 QObject * ScriptToolbox::menuAddAction(QObject * menu, const QString & text, const bool enabled)
 {
+  scriptDeprecated("Use QMenu::addAction instead of toolbox.menuAddAction()");
   QMenu * m = qobject_cast<QMenu*>(menu);
   QAction * act = 0;
   if(m)
@@ -512,6 +534,7 @@ QObject * ScriptToolbox::menuAddAction(QObject * menu, const QString & text, con
   */
 QObject * ScriptToolbox::menuAddMenu(QObject * menu, const QString & text, const QString & name)
 {
+  scriptDeprecated("Use QMenu::addAction instead of toolbox.menuAddMenu()");
   QMenu * m = qobject_cast<QMenu*>(menu);
   QMenu * nm = 0;
   if(m)
@@ -528,6 +551,7 @@ QObject * ScriptToolbox::menuAddMenu(QObject * menu, const QString & text, const
   */
 QObject * ScriptToolbox::menuAddSeparator(QObject * menu)
 {
+  scriptDeprecated("Use QMenu::addSeparator instead of toolbox.menuAddSeparator");
   QMenu * m = qobject_cast<QMenu*>(menu);
   QAction * na = 0;
   if(m)
@@ -540,6 +564,7 @@ QObject * ScriptToolbox::menuAddSeparator(QObject * menu)
   */
 QObject * ScriptToolbox::menuInsertAction(QObject * menu, QObject * before, const QString & name, const bool enabled)
 {
+  scriptDeprecated("Use QMenu::insertAction instead of toolbox.menuInsertAction");
   QMenu * m = qobject_cast<QMenu*>(menu);
   QAction * ba = qobject_cast<QAction*>(before);
   QAction * na = new QAction(name,m);
@@ -557,6 +582,7 @@ QObject * ScriptToolbox::menuInsertAction(QObject * menu, QObject * before, cons
   */
 QObject * ScriptToolbox::menuInsertMenu(QObject * menu, QObject * before, const QString & name)
 {
+  scriptDeprecated("Use QMenu::insertMenu instead of toolbox.menuInsertMenu()");
   QMenu * m = qobject_cast<QMenu*>(menu);
   QAction * ba = qobject_cast<QAction*>(before);
   QMenu * nm = new QMenu(name);
@@ -578,6 +604,7 @@ QObject * ScriptToolbox::menuInsertMenu(QObject * menu, QObject * before, const
   */
 QObject * ScriptToolbox::menuInsertSeparator(QObject * menu, QObject * before)
 {
+  scriptDeprecated("Use QMenu::insertSeparator instead of toolbox.menuInsertSeparator()");
   QMenu * m = qobject_cast<QMenu*>(menu);
   QAction * ba = qobject_cast<QAction*>(before);
   if (!ba)
@@ -600,6 +627,7 @@ QObject * ScriptToolbox::menuInsertSeparator(QObject * menu, QObject * before)
   */
 void ScriptToolbox::menuRemove(QObject * menu, QObject * action)
 {
+  scriptDeprecated("Use QMenu::removeAction instead of toolbox.menuRemove()");
   QMenu * m = qobject_cast<QMenu*>(menu);
   QAction * act = qobject_cast<QAction*>(action);
   if (!act)
@@ -628,6 +656,7 @@ int ScriptToolbox::menuActionCount(QObject * menu)
  */
 int ScriptToolbox::tabCount(QWidget * tab)
 {
+  scriptDeprecated("Use QTabWidget::count() instead of toolbox.tabCount()");
   QTabWidget *tw = qobject_cast<QTabWidget*>(tab);
   if(tw)
     return tw->count();
@@ -639,6 +668,7 @@ int ScriptToolbox::tabCount(QWidget * tab)
   */
 QWidget * ScriptToolbox::tabWidget(QWidget * tab, int idx)
 {
+  scriptDeprecated("Use QTabWidget::widget() instead of toolbox.tabWidget()");
   QTabWidget *tw = qobject_cast<QTabWidget*>(tab);
   QWidget * w = 0;
   if(tw)
@@ -651,6 +681,7 @@ QWidget * ScriptToolbox::tabWidget(QWidget * tab, int idx)
   */
 int ScriptToolbox::tabInsertTab(QWidget * tab, int idx, QWidget * page, const QString & text)
 {
+  scriptDeprecated("Use QTabWidget::insertTab() instead of toolbox.tabInsertTab()");
   QTabWidget *tw = qobject_cast<QTabWidget*>(tab);
   int i = -1;
   if(tw)
@@ -663,6 +694,7 @@ int ScriptToolbox::tabInsertTab(QWidget * tab, int idx, QWidget * page, const QS
   */
 int ScriptToolbox::tabTabIndex(QWidget * tab, QWidget * page)
 {
+  scriptDeprecated("Use QTabWidget::indexOf() instead of toolbox.tabTabIndex()");
   QTabWidget *tw = qobject_cast<QTabWidget*>(tab);
   if(tw)
     return tw->indexOf(page);
@@ -674,6 +706,7 @@ int ScriptToolbox::tabTabIndex(QWidget * tab, QWidget * page)
   */
 void ScriptToolbox::tabRemoveTab(QWidget * tab, int idx)
 {
+  scriptDeprecated("Use QTabWidget::removeTab() instead of toolbox.tabRemoveTab()");
   QTabWidget *tw = qobject_cast<QTabWidget*>(tab);
   if(tw)
     tw->removeTab(idx);
@@ -684,6 +717,7 @@ void ScriptToolbox::tabRemoveTab(QWidget * tab, int idx)
   */
 void ScriptToolbox::tabSetTabEnabled(QWidget * tab, int idx, bool enable)
 {
+  scriptDeprecated("Use QTabWidget::setTabEnabled() instead of toolbox.tabSetTabEnabled()");
   QTabWidget *tw = qobject_cast<QTabWidget*>(tab);
   if(tw)
     tw->setTabEnabled(idx, enable);
@@ -694,6 +728,7 @@ void ScriptToolbox::tabSetTabEnabled(QWidget * tab, int idx, bool enable)
   */
 void ScriptToolbox::tabSetTabText(QWidget * tab, int idx, const QString & text)
 {
+  scriptDeprecated("Use QTabWidget::setTabText() instead of toolbox.tabSetTabText()");
   QTabWidget *tw = qobject_cast<QTabWidget*>(tab);
   if(tw)
     tw->setTabText(idx, text);
@@ -704,6 +739,7 @@ void ScriptToolbox::tabSetTabText(QWidget * tab, int idx, const QString & text)
   */
 QString ScriptToolbox::tabtabText(QWidget * tab, int idx)
 {
+  scriptDeprecated("Use QTabWidget::tabText() instead of toolbox.tabtabText()");
   QTabWidget *tw = qobject_cast<QTabWidget*>(tab);
   QString str;
   if(tw)
@@ -893,7 +929,8 @@ bool ScriptToolbox::printReportCopies(const QString & name, const ParameterList
   if (orReport::beginMultiPrint(&printer, userCanceled) == false)
   {
     if(!userCanceled)
-      systemError(NULL, tr("Could not initialize printing system for multiple reports."));
+      ErrorReporter::error(QtCriticalMsg, 0, tr("Error Occurred"),
+                         tr("Could not initialize the printing system for multiple reports."),__FILE__,__LINE__);
     return userCanceled;
   }
   userCanceled = false;
@@ -1043,32 +1080,37 @@ void ScriptToolbox::openUrl(const QString & fileUrl)
 /** @deprecated Use QFile.copyFile(oldName, newName) instead */
 bool ScriptToolbox::copyFile(const QString & oldName, const QString & newName)
 {
-   return QFile::copy(oldName, newName);
+  scriptDeprecated("Use QFile.copyFile(oldName, newName) instead of toolbox.copyFile()");
+  return QFile::copy(oldName, newName);
 }
 
 /** @deprecated Use var file = new QFileInfo(path).fileName() instead */
 QString ScriptToolbox::getFileName(const QString & path)
 {
-    QFileInfo fi(path);
-    return fi.fileName();
+  scriptDeprecated("Use new QFileInfo(path).fileName() instead of toolbox.getFileName(path)");
+  QFileInfo fi(path);
+  return fi.fileName();
 }
 
 /** @deprecated Use QFile.rename(oldName, newName) instead */
 bool ScriptToolbox::renameFile(const QString & oldName, const QString & newName)
 {
-   return QFile::rename(oldName, newName);
+  scriptDeprecated("Use QFile.rename(oldName, newName) instead of toolbox.renameFile()");
+  return QFile::rename(oldName, newName);
 }
 
 /** @deprecated Use QFile.removeFile(oldName, newName) instead */
 bool ScriptToolbox::removeFile(const QString & name)
 {
-   return QFile::remove(name);
+  scriptDeprecated("Use QFile.removeFile() instead of toolbox.removeFile()");
+  return QFile::remove(name);
 }
 
 /** @deprecated Use QFile.exists(name) instead */
 bool ScriptToolbox::fileExists(const QString & name)
 {
-   return QFile::exists(name);
+  scriptDeprecated("Use QFile.exists(name) instead of toolbox.fileExists()");
+  return QFile::exists(name);
 }
 
 /** @brief Read the entire contents of a text %file.
@@ -1126,37 +1168,43 @@ bool ScriptToolbox::textStreamWrite(const QString & pName, const QString & Write
 /** @deprecated use QFile.exists(name) instead */
 bool fileExists(const QString & name)
 {
-   QFile file(name);
-   return file.exists();
+  scriptDeprecated("use QFile.exists(name) instead of toolbox.fileExist()");
+  QFile file(name);
+  return file.exists();
 }
 
 /** @deprecated Use QDir.homePath() instead */
 QString ScriptToolbox::getHomeDir()
 {
-   return QDir::homePath();
+  scriptDeprecated("Use QDir.homePath() instead of toolbox.getHomeDir()");
+  return QDir::homePath();
 }
 
 /** @deprecated Use QDir.currentPath() instead */
 QString ScriptToolbox::getCurrentDir()
 {
-   return QDir::currentPath();
+  scriptDeprecated("Use QDir.currentPath() instead of toolbox.getCurrentDir()");
+  return QDir::currentPath();
 }
 
 /** @deprecated Use QDir.tempPath() instead */
 QString ScriptToolbox::getTempDir()
 {
+  scriptDeprecated("Use QDir.tempPath() instead of toolbox.getTempDir()");
    return QDir::tempPath();
 }
 
 /** @deprecated Use QDir.rootPath() instead */
 QString ScriptToolbox::rootPath()
 {
+  scriptDeprecated("Use QDir.rootPath() instead of toolbox.rootPath()");
    return QDir::rootPath();
 }
 
 /** @deprecated Use var dir = new QDir(rootPath) followed by dir.mkpath(mkPath) instead */
 bool ScriptToolbox::makePath(const QString & mkPath, const QString & rootPath)
 {
+  scriptDeprecated("Use var dir = new QDir(rootPath) ; dir.mkpath(mkPath) instead of toolbox.makePath(mkPath, rootPath)");
    QDir dir(rootPath);
 
    return dir.mkpath(mkPath);
@@ -1165,6 +1213,7 @@ bool ScriptToolbox::makePath(const QString & mkPath, const QString & rootPath)
 /** @deprecated Use var dir = new QDir(rootPath) followed by dir.rmpath(rmPath) instead */
 bool ScriptToolbox::removePath(const QString & rmPath, const QString & rootPath)
 {
+  scriptDeprecated("Use var dir = new QDir(rootPath) ; dir.rmpath(rmPath) instead of toolbox.removePath(rmPath, rootPath)");
    QDir dir(rootPath);
 
    return dir.rmpath(rmPath);
@@ -1231,6 +1280,8 @@ void ScriptToolbox::listProperties(const QScriptValue &obj, const bool showProto
  */
 int ScriptToolbox::messageBox(const QString & type, QWidget * parent, const QString & title, const QString & text, int buttons, int defaultButton)
 {
+  scriptDeprecated("Use QMessageBox.critical(), QMessageBox.information(), "
+                   "QMessageBox.question(), or QMessageBox.warning() instead of toolbox.messageBox()");
   int btn;
   if(type == "critical")
     btn = QMessageBox::critical(parent, title, text, (QMessageBox::StandardButton)buttons, (QMessageBox::StandardButton)defaultButton);
@@ -1282,10 +1333,7 @@ QWidget *ScriptToolbox::openWindow(QString pname, QWidget *parent, Qt::WindowMod
   if(returnVal)
   {
     if(!returnVal->inherits("QDialog"))
-    {
-      returnVal->setWindowModality(modality);
-      omfgThis->handleNewWindow(returnVal);
-    }
+      omfgThis->handleNewWindow(returnVal, modality);
     _lastWindow = returnVal;
     return returnVal;
   }
@@ -1335,7 +1383,6 @@ QWidget *ScriptToolbox::openWindow(QString pname, QWidget *parent, Qt::WindowMod
 
     window->setCentralWidget(ui);
     window->setWindowTitle(ui->windowTitle());
-    window->setWindowModality(modality);
     window->resize(size);
 
     if (ui->inherits("QDialog"))
@@ -1356,19 +1403,19 @@ QWidget *ScriptToolbox::openWindow(QString pname, QWidget *parent, Qt::WindowMod
         qWarning("Could not find the script engine to embed a dialog inside "
                "a placeholder window");
 
-      omfgThis->handleNewWindow(window);
+      omfgThis->handleNewWindow(window, modality);
       returnVal = ui;
     }
     else
     {
-      omfgThis->handleNewWindow(window);
+      omfgThis->handleNewWindow(window, modality);
       returnVal = window;
     }
     _lastWindow = window;
   }
-  else if (screenq.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, 0, tr("Error Opening New Window"),
+                                screenq, __FILE__, __LINE__))
   {
-    systemError(0, screenq.lastError().databaseText(), __FILE__, __LINE__);
     return 0;
   }
 
@@ -1390,6 +1437,7 @@ QWidget *ScriptToolbox::newDisplay(QString pname, QWidget *parent, Qt::WindowMod
   */
 void ScriptToolbox::addColumnXTreeWidget(QWidget * tree, const QString & pString, int pWidth, int pAlignment, bool pVisible, const QString pEditColumn, const QString pDisplayColumn)
 {
+  scriptDeprecated("Use XTreeWidget::addColumn() instead of toolbox.addColumnXTreeWidget()");
   XTreeWidget *xt = qobject_cast<XTreeWidget*>(tree);
   if(xt)
     xt->addColumn(pString, pWidth, pAlignment, pVisible, pEditColumn, pDisplayColumn);
@@ -1400,107 +1448,23 @@ void ScriptToolbox::addColumnXTreeWidget(QWidget * tree, const QString & pString
   */
 void ScriptToolbox::populateXTreeWidget(QWidget * tree, XSqlQuery pSql, bool pUseAltId)
 {
+  scriptDeprecated("Use XTreeWidget::populate() instead of toolbox.populateXtreeWidget()");
   XTreeWidget *xt = qobject_cast<XTreeWidget*>(tree);
   if(xt)
     xt->populate(pSql, pUseAltId);
 }
 
+#if QT_VERSION < 0x050900
 /** @brief Load the given QWebView with the given URL. */
 void ScriptToolbox::loadQWebView(QWidget * webView, const QString & url)
 {
+  scriptDeprecated("QWebView will not be available in Qt 5.9");
   QUrl p = QUrl(url);
   QWebView *wv = qobject_cast<QWebView*>(webView);
   if(wv)
     wv->load(p);
 }
-
-/** @internal */
-QScriptValue SetResponsetoScriptValue(QScriptEngine *engine, const enum SetResponse &sr)
-{
-  return QScriptValue(engine, (int)sr);
-}
-
-/** @internal */
-void SetResponsefromScriptValue(const QScriptValue &obj, enum SetResponse &sr)
-{
-  sr = (enum SetResponse)obj.toInt32();
-}
-
-/** @internal */
-QScriptValue ParameterGroupStatestoScriptValue(QScriptEngine *engine, const enum ParameterGroup::ParameterGroupStates &en)
-{
-  return QScriptValue(engine, (int)en);
-}
-
-/** @internal */
-void ParameterGroupStatesfromScriptValue(const QScriptValue &obj, enum ParameterGroup::ParameterGroupStates &en)
-{
-  en = (enum ParameterGroup::ParameterGroupStates)obj.toInt32();
-}
-
-/** @internal */
-QScriptValue ParameterGroupTypestoScriptValue(QScriptEngine *engine, const enum ParameterGroup::ParameterGroupTypes &en)
-{
-  return QScriptValue(engine, (int)en);
-}
-
-/** @internal */
-void ParameterGroupTypesfromScriptValue(const QScriptValue &obj, enum ParameterGroup::ParameterGroupTypes &en)
-{
-  en = (enum ParameterGroup::ParameterGroupTypes)obj.toInt32();
-}
-
-/** @internal */
-QScriptValue QtWindowModalitytoScriptValue(QScriptEngine *engine, const enum Qt::WindowModality &en)
-{
-  return QScriptValue(engine, (int)en);
-}
-
-/** @internal */
-void QtWindowModalityfromScriptValue(const QScriptValue &obj, enum Qt::WindowModality &en)
-{
-  en = (enum Qt::WindowModality)obj.toInt32();
-}
-
-/** @internal */
-QScriptValue WindowSystemtoScriptValue(QScriptEngine *engine, const enum GUIClient::WindowSystem &en)
-{
-  return QScriptValue(engine, (int)en);
-}
-
-/** @internal */
-void WindowSystemfromScriptValue(const QScriptValue &obj, enum GUIClient::WindowSystem &en)
-{
-  en = (enum GUIClient::WindowSystem)obj.toInt32();
-}
-
-/** @internal */
-QScriptValue SaveFlagstoScriptValue(QScriptEngine *engine, const enum SaveFlags &en)
-{
-  return QScriptValue(engine, (int)en);
-}
-
-/** @internal */
-void SaveFlagsfromScriptValue(const QScriptValue &obj, enum SaveFlags &en)
-{
-  if (obj.isNumber())
-    en = (enum SaveFlags)obj.toInt32();
-  else if (obj.isString())
-  {
-    if (obj.toString() == "CHECK")
-      en = CHECK;
-    else if (obj.toString() == "CHANGEONE")
-      en = CHANGEONE;
-    else if (obj.toString() == "CHANGEALL")
-      en = CHANGEALL;
-    else
-      qWarning("string %s could not be converted to SaveFlags",
-               qPrintable(obj.toString()));
-  }
-  else
-    qWarning("object %s could not be converted to SaveFlags",
-             qPrintable(obj.toString()));
-}
+#endif
 
 /** @brief Save a credit card definition from a script.
 
@@ -1570,6 +1534,7 @@ QObject *ScriptToolbox::getCreditCardProcessor()
  */
 QString scriptHandleIncludes(QString source)
 {
+  scriptDeprecated("See include.cpp for the proper way to include scripts");
   QString returnVal = source;
   if (returnVal.contains(QRegExp("#include")))
   {
@@ -1592,16 +1557,11 @@ QString scriptHandleIncludes(QString source)
           name = words.at(1);
 
         line.replace(i, "// " + line.at(i));
-        XSqlQuery inclq;
-        inclq.prepare("SELECT script_source "
-                      "FROM script "
-                      "WHERE ((script_name=:name)"
-                      "  AND  ((script_order=:order) OR (:order = -1))"
-                      "  AND  script_enabled) "
-                      "ORDER BY script_order;");
-        inclq.bindValue(":name",  name);
-        inclq.bindValue(":order", order);
-        inclq.exec();
+        ParameterList params;
+        params.append("jsonlist", QString("{\"1\": \"%1\"}").arg(name));
+        params.append("order", order);
+        MetaSQLQuery mql(omfgThis->_mqlhash->value("scripts", "fetch"));
+        XSqlQuery inclq = mql.toQuery(params);
         bool found = false;
         while (inclq.next())
         {
