@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2016 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -12,10 +12,12 @@
 #include "mqlutil.h"
 
 #include <QSqlError>
+#include <QInputDialog>
 
 #include <metasql.h>
 #include <openreports.h>
 #include <parameter.h>
+#include <xdateinputdialog.h>
 #include <QMessageBox>
 
 #include "apOpenItem.h"
@@ -41,6 +43,7 @@ selectPayments::selectPayments(QWidget* parent, const char* name, Qt::WindowFlag
   connect(_startDate, SIGNAL(newDate(const QDate&)), this, SLOT(sFillList()));
   connect(_endDate, SIGNAL(newDate(const QDate&)), this, SLOT(sFillList()));
   connect(_onOrBeforeDate, SIGNAL(newDate(const QDate&)), this, SLOT(sFillList()));
+  connect(_query, SIGNAL(clicked()), this, SLOT(sFillList()));
   connect(_print, SIGNAL(clicked()), this, SLOT(sPrint()));
   connect(_select, SIGNAL(clicked()), this, SLOT(sSelect()));
   connect(_selectDiscount, SIGNAL(clicked()), this, SLOT(sSelectDiscount()));
@@ -70,6 +73,7 @@ selectPayments::selectPayments(QWidget* parent, const char* name, Qt::WindowFlag
   _apopen->addColumn(tr("Discount (%1)").arg(CurrDisplay::baseCurrAbbr()), _moneyColumn, Qt::AlignRight, false, "base_discount"  );
   _apopen->addColumn(tr("Currency"),       _currencyColumn, Qt::AlignLeft,       true, "curr_concat" );
   _apopen->addColumn(tr("Status"),         _currencyColumn, Qt::AlignCenter,     true, "apopen_status" );
+  _apopen->addColumn(tr("On Hold Comment"), -1,             Qt::AlignLeft,       true, "apopen_hold_comment" );
 
 //  if (omfgThis->singleCurrency())
 //  {
@@ -145,15 +149,16 @@ void selectPayments::sSelectDue()
       int result = selectSelectDue.value("result").toInt();
       if (result < 0)
       {
-        systemError(this, storedProcErrorLookup("selectPayment", result),
-                    __FILE__, __LINE__);
+          ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Payment Information"),
+                               storedProcErrorLookup("selectPayment", result),
+                               __FILE__, __LINE__);
         return;
       }
     }
     else
     {
-      ErrorReporter::error(QtCriticalMsg, this, tr("Select Due"),
-                           selectSelectDue, __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Payment Information"),
+                                        selectSelectDue, __FILE__, __LINE__);
       return;
     }
 
@@ -188,14 +193,15 @@ void selectPayments::sSelectDiscount()
       int result = selectSelectDiscount.value("result").toInt();
       if (result < 0)
       {
-        systemError(this, storedProcErrorLookup("selectPayment", result),
-                    __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Payment Information"),
+                               storedProcErrorLookup("selectPayment", result),
+                               __FILE__, __LINE__);
         return;
       }
     }
     else
     {
-      ErrorReporter::error(QtCriticalMsg, this, tr("Select Discount"),
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Payment Information"),
                            selectSelectDiscount, __FILE__, __LINE__);
       return;
     }
@@ -217,14 +223,15 @@ void selectPayments::sClearAll()
     int result = selectClearAll.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("clearPayment", result),
-                  __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Clearing Payment Information"),
+                             storedProcErrorLookup("clearPayment", result),
+                             __FILE__, __LINE__);
       return;
     }
   }
-  else if (selectClearAll.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Clearing Payment Information"),
+                                selectClearAll, __FILE__, __LINE__))
   {
-    systemError(this, selectClearAll.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -314,15 +321,15 @@ void selectPayments::sSelectLine()
         int result = selectSelectLine.value("result").toInt();
         if (result < 0)
         {
-          systemError(this, cursor->text(0) + " " + cursor->text(2) + "\n" +
-                            storedProcErrorLookup("selectPayment", result),
-                      __FILE__, __LINE__);
+          ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Payment Information"),
+                                 storedProcErrorLookup("selectPayment", result),
+                                 __FILE__, __LINE__);
           return;
         }
       }
-      else if (selectSelectLine.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Payment Information"),
+                                    selectSelectLine, __FILE__, __LINE__))
       {
-        systemError(this, selectSelectLine.lastError().databaseText(), __FILE__, __LINE__);
         return;
       }
                 }
@@ -351,15 +358,16 @@ void selectPayments::sClear()
       int result = selectClear.value("result").toInt();
       if (result < 0)
       {
-        systemError(this, cursor->text(0) + " " + cursor->text(2) + "\n" +
-                          storedProcErrorLookup("clearPayment", result),
-                    __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, cursor->text(0) + " " + cursor->text(2) + "\n" +
+                               tr(" Error Clearing Payment Information"),
+                               storedProcErrorLookup("clearPayment", result),
+                               __FILE__, __LINE__);
         return;
       }
     }
-    else if (selectClear.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Clearing Payment Information"),
+                                  selectClear, __FILE__, __LINE__))
     {
-      systemError(this, selectClear.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     update = true;
@@ -433,9 +441,9 @@ void selectPayments::sFillList()
   MetaSQLQuery mql = mqlLoad("apOpenItems", "selectpayments");
   selectFillList = mql.toQuery(params);
   _apopen->populate(selectFillList,true);
-  if (selectFillList.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Payment Information"),
+                                selectFillList, __FILE__, __LINE__))
   {
-    systemError(this, selectFillList.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -529,13 +537,14 @@ void selectPayments::sViewGLSeries()
     omfgThis->handleNewWindow(newdlg);
   }
   else
-    systemError( this, dspViewGLSeries.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Payment Information"),
+                       dspViewGLSeries, __FILE__, __LINE__);
 }
 
 void selectPayments::sOpen()
 {
   XSqlQuery open;
-  open.prepare("UPDATE apopen SET apopen_status = 'O' WHERE apopen_id=:apopen_id;");
+  open.prepare("UPDATE apopen SET apopen_status = 'O', apopen_hold_comment = NULL WHERE apopen_id=:apopen_id;");
   open.bindValue(":apopen_id", _apopen->id());
   open.exec();
   sFillList();
@@ -544,6 +553,9 @@ void selectPayments::sOpen()
 void selectPayments::sOnHold()
 {
   XSqlQuery selectpayment;
+  bool ok;
+  QString comment;
+
   selectpayment.prepare("SELECT * FROM apselect WHERE apselect_apopen_id = :apopen_id;");
   selectpayment.bindValue(":apopen_id", _apopen->id());
   selectpayment.exec();
@@ -555,9 +567,16 @@ void selectPayments::sOnHold()
     return;
   }
 
+  comment = QInputDialog::getText(this, tr("On Hold Comment"),
+        tr("Enter an On Hold Comment:"), QLineEdit::Normal, comment, &ok);
+  if(!ok)
+    return;
+
   XSqlQuery onhold;
-  onhold.prepare("UPDATE apopen SET apopen_status = 'H' WHERE apopen_id=:apopen_id;");
+  onhold.prepare("UPDATE apopen SET apopen_status = 'H', apopen_hold_comment = :comment "
+                 " WHERE apopen_id=:apopen_id;");
   onhold.bindValue(":apopen_id", _apopen->id());
+  onhold.bindValue(":comment", comment);
   onhold.exec();
   sFillList();
 }
@@ -607,33 +626,42 @@ void selectPayments::sVoidVoucher()
   QList<XTreeWidgetItem*> list = _apopen->selectedItems();
   XTreeWidgetItem * cursor = 0;
   XSqlQuery dspVoidVoucher;
-  dspVoidVoucher.prepare("SELECT voidApopenVoucher(:apopen_id) AS result;");
+  dspVoidVoucher.prepare("SELECT voidApopenVoucher(:apopen_id, :voidDate) AS result;");
   for(int i = 0; i < list.size(); i++)
   {
     cursor = (XTreeWidgetItem*)list.at(i);
     if ( (cursor->rawValue("doctype") == tr("Voucher")) && (cursor->rawValue("selected") == 0.0) )
     {
-      dspVoidVoucher.bindValue(":apopen_id", cursor->id());
-      dspVoidVoucher.exec();
-      
-      if(dspVoidVoucher.first())
+      XDateInputDialog newdlg(this, "", true);
+      ParameterList params;
+      params.append("label", tr("On what date did you void the Voucher?"));
+      params.append("default", cursor->rawValue("apopen_docdate"));
+      newdlg.set(params);
+      int returnVal = newdlg.exec();
+      if (returnVal == XDialog::Accepted)
       {
-        if(dspVoidVoucher.value("result").toInt() < 0)
+        QDate voidDate = newdlg.getDate();
+        dspVoidVoucher.bindValue(":apopen_id", cursor->id());
+        dspVoidVoucher.bindValue(":voidDate", voidDate);
+        dspVoidVoucher.exec();
+      
+        if(dspVoidVoucher.first())
         {
-          systemError( this, tr("A System Error occurred at %1::%2, Error #%3.")
-                      .arg(__FILE__)
-                      .arg(__LINE__)
-                      .arg(dspVoidVoucher.value("result").toInt()) );
+          if(dspVoidVoucher.value("result").toInt() < 0)
+          {
+            ErrorReporter::error(QtCriticalMsg, this, tr("Error Voiding Voucher"),
+                                 dspVoidVoucher, __FILE__, __LINE__);
+            return;
+          }
+        }
+        else
+        {
+          ErrorReporter::error(QtCriticalMsg, this, tr("Voiding Voucher"),
+                               dspVoidVoucher, __FILE__, __LINE__);
           return;
         }
+        update = true;
       }
-      else
-      {
-        ErrorReporter::error(QtCriticalMsg, this, tr("Voiding Voucher"),
-                             dspVoidVoucher, __FILE__, __LINE__);
-        return;
-      }
-      update = true;
     }
   }
   if(update)
