@@ -1,20 +1,25 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
  * to be bound by its terms.
  */
+#include <QDebug>
 #include <QBuffer>
 #include <QtUiTools/QUiLoader>
 #include <QMessageBox>
 #include <QPushButton>
 #include <QScriptEngine>
 #include <QScriptEngineDebugger>
+#include <QScrollArea>
 
 #include "getscreen.h"
+#include "include.h"
+#include "metasql.h"
+#include "mqlutil.h"
 #include "scripttoolbox.h"
 #include "setup.h"
 #include "xt.h"
@@ -37,7 +42,9 @@ setup::setup(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
 {
   setupUi(this);
-
+  if (! name)
+    setObjectName("setup");
+    
   _modules->addItem(tr("All"), Xt::AllModules);
   _modules->addItem(tr("Accounting"), Xt::AccountingModule);
   _modules->addItem(tr("Sales"), Xt::SalesModule);
@@ -73,6 +80,7 @@ setup::setup(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
   insert(tr("Purchase"), "configurePO", Configure, Xt::PurchaseModule, mode("ConfigurePO"), 0 );
   insert(tr("Registration"), "registrationKey", Configure, Xt::SystemModule, mode("MaintainRegistrationKey"), 0 );
   insert(tr("Schedule"), "configureMS", Configure, Xt::ScheduleModule, mode("ConfigureMS"), 0 );
+
   insert(tr("Search Path"), "configureSearchPath", Configure, Xt::SystemModule, _privileges->isDba() ? cEdit : 0, 0);
 
   // Account Mappings
@@ -177,6 +185,9 @@ setup::setup(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
   modeVal = mode("MaintainOpportunityTypes");
   insert(tr("Opportunity Types"), "opportunityTypes", MasterInformation, Xt::CRMModule, modeVal, modeVal);
 
+  modeVal = mode("MaintainProjectTypes");
+  insert(tr("Project Types"), "projectTypes", MasterInformation, Xt::CRMModule, modeVal, modeVal);
+
   modeVal = mode("MaintainPlannerCodes", "ViewPlannerCodes");
   insert(tr("Planner Codes"), "plannerCodes", MasterInformation, Xt::InventoryModule | Xt::ScheduleModule | Xt::PurchaseModule, modeVal, modeVal);
 
@@ -186,6 +197,9 @@ setup::setup(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
   modeVal = mode("MaintainReasonCodes");
   insert(tr("Reason Codes"), "reasonCodes", MasterInformation, Xt::InventoryModule | Xt::AccountingModule, modeVal, modeVal);
 
+  modeVal = mode("MaintainPurchaseTypes", "ViewPurchaseTypes");
+  insert(tr("Purchase Types"), "poTypes", MasterInformation, Xt::PurchaseModule, modeVal, modeVal);
+
   modeVal = mode("MaintainRejectCodes", "ViewRejectCodes");
   insert(tr("Reject Codes"), "rejectCodes", MasterInformation, Xt::PurchaseModule, modeVal, modeVal);
 
@@ -495,23 +509,22 @@ void setup::setCurrentIndex(XTreeWidgetItem* item)
         QUiLoader loader;
         QByteArray ba = screenq.value("uiform_source").toByteArray();
         QBuffer uiFile(&ba);
-        if (!uiFile.open(QIODevice::ReadOnly))
-          QMessageBox::critical(0, tr("Could not load UI"),
-                                tr("<p>There was an error loading the UI Form "
-                                   "from the database."));
+
+         if (!uiFile.open(QIODevice::ReadOnly))
+            QMessageBox::critical(0, tr("Could not load UI"),
+                 tr("<p>There was an error loading the UI Form "
+                    "from the database."));
+
+
         w = loader.load(&uiFile);
         w->setObjectName(uiName);
         uiFile.close();
 
         // Load scripts if applicable
-        XSqlQuery scriptq;
-        scriptq.prepare("SELECT script_source, script_order"
-                        "  FROM script"
-                        " WHERE((script_name=:script_name)"
-                        "   AND (script_enabled))"
-                        " ORDER BY script_order;");
-        scriptq.bindValue(":script_name", uiName);
-        scriptq.exec();
+        ParameterList params;
+        params.append("jsonlist", QString("{\"1\": \"%1\"}").arg(uiName));
+        MetaSQLQuery mql = mqlLoad("scripts", "fetch");
+        XSqlQuery scriptq = mql.toQuery(params);
 
         QScriptEngine* engine = new QScriptEngine();
         if (_preferences->boolean("EnableScriptDebug"))
@@ -520,12 +533,13 @@ void setup::setCurrentIndex(XTreeWidgetItem* item)
           debugger->attachTo(engine);
         }
         omfgThis->loadScriptGlobals(engine);
+        setupInclude(engine);
         QScriptValue mywindow = engine->newQObject(w);
         engine->globalObject().setProperty("mywindow", mywindow);
 
         while(scriptq.next())
         {
-          QString script = scriptHandleIncludes(scriptq.value("script_source").toString());
+          QString script = scriptq.value("script_source").toString();
           QScriptValue result = engine->evaluate(script, uiName);
           if (engine->hasUncaughtException())
           {
@@ -560,10 +574,12 @@ void setup::setCurrentIndex(XTreeWidgetItem* item)
         if (x)
           x->set(params);
       }
-
       int idx = _stack->count();
       _itemMap[uiName].index = idx;
-      _stack->addWidget(w);
+      QScrollArea *scroll = new QScrollArea();
+      scroll->setWidget(w);
+      scroll->setWidgetResizable(true);
+      _stack->addWidget(scroll);
       _stack->setCurrentIndex(idx);
 
       _stackLit->setText(label.arg(item->text(0)));
