@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -13,6 +13,8 @@
 #include <QVariant>
 #include <QMessageBox>
 #include "standardJournalItem.h"
+#include "errorReporter.h"
+#include "guiErrorCheck.h"
 
 standardJournal::standardJournal(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
@@ -36,8 +38,9 @@ standardJournal::standardJournal(QWidget* parent, const char* name, bool modal,
   _stdjrnlitem->addColumn(tr("Debit"),   _priceColumn, Qt::AlignRight,  true,  "debit" );
   _stdjrnlitem->addColumn(tr("Credit"),  _priceColumn, Qt::AlignRight,  true,  "credit" );
 
-  _debits->setValidator(omfgThis->moneyVal());
-  _credits->setValidator(omfgThis->moneyVal());
+  _credits->setPrecision(omfgThis->moneyVal());
+  _debits->setPrecision(omfgThis->moneyVal());
+
 }
 
 standardJournal::~standardJournal()
@@ -121,12 +124,6 @@ enum SetResponse standardJournal::set(const ParameterList &pParams)
 void standardJournal::sSave()
 {
   XSqlQuery standardSave;
-  if (_name->text().length() == 0)
-  {
-    QMessageBox::warning( this, tr("Cannot Save Standard Journal"),
-                          tr("You must enter a valid Name.") );
-    return;
-  }
   
   standardSave.prepare( "SELECT stdjrnl_id"
              "  FROM stdjrnl "
@@ -135,14 +132,16 @@ void standardJournal::sSave()
   standardSave.bindValue(":stdjrnl_name", _name->text());
   standardSave.bindValue(":stdjrnl_id", _stdjrnlid);
   standardSave.exec();
-  if (standardSave.first())
-  {
-    QMessageBox::warning( this, tr("Cannot Save Standard Journal"),
-                          tr("The Name you have entered for this Standard Journal already exists. "
-                             "Please enter in a different Name for this Standard Journal."));
-    _name->setFocus();
+
+  QList<GuiErrorCheck> errors;
+  errors<< GuiErrorCheck(_name->text().length() == 0, _name,
+                         tr("You must enter a valid Name.")),
+  errors<< GuiErrorCheck(standardSave.first(), _name,
+                         tr("The Name you have entered for this Standard Journal already exists. "
+                           "Please enter in a different Name for this Standard Journal."))
+  ;
+  if (GuiErrorCheck::reportErrors(this, tr("Cannot Save Standard Journal"), errors))
     return;
-  }
 
   if (_mode == cNew)
     standardSave.prepare( "INSERT INTO stdjrnl "
@@ -259,12 +258,12 @@ void standardJournal::sFillList()
   standardFillList.exec();
   _stdjrnlitem->populate(standardFillList);
 
-  standardFillList.prepare( "SELECT SUM( CASE WHEN (stdjrnlitem_amount < 0) THEN (stdjrnlitem_amount * -1)"
+  standardFillList.prepare( "SELECT formatMoney(SUM( CASE WHEN (stdjrnlitem_amount < 0) THEN (stdjrnlitem_amount * -1)"
              "                 ELSE 0"
-             "            END ) AS debit,"
-             "       SUM( CASE WHEN (stdjrnlitem_amount > 0) THEN stdjrnlitem_amount"
+             "            END )) AS debit,"
+             "       formatMoney(SUM( CASE WHEN (stdjrnlitem_amount > 0) THEN stdjrnlitem_amount"
              "                 ELSE 0"
-             "            END ) AS credit,"
+             "            END )) AS credit,"
              "       (SUM(stdjrnlitem_amount) <> 0) AS oob "
              "FROM stdjrnlitem "
              "WHERE (stdjrnlitem_stdjrnl_id=:stdjrnl_id);" );
@@ -272,8 +271,8 @@ void standardJournal::sFillList()
   standardFillList.exec();
   if (standardFillList.first())
   {
-    _debits->setDouble(standardFillList.value("debit").toDouble());
-    _credits->setDouble(standardFillList.value("credit").toDouble());
+    _debits->setText(standardFillList.value("debit"));
+    _credits->setText(standardFillList.value("credit"));
 
     QString stylesheet;
     if (standardFillList.value("oob").toBool())
@@ -282,9 +281,8 @@ void standardJournal::sFillList()
     _credits->setStyleSheet(stylesheet);
   }
   else
-    systemError(this, tr("A System Error occurred at %1::%2.")
-                      .arg(__FILE__)
-                      .arg(__LINE__) );
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Journal Information"),
+                       standardFillList, __FILE__, __LINE__);
 }
 
 void standardJournal::populate()
@@ -304,8 +302,7 @@ void standardJournal::populate()
     sFillList();
   }
   else
-    systemError(this, tr("A System Error occurred at %1::%2.")
-                      .arg(__FILE__)
-                      .arg(__LINE__) );
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Journal Information"),
+                       standardpopulate, __FILE__, __LINE__);
 }
 
