@@ -21,7 +21,7 @@ class states : public XWidget, public Ui::states
     Q_OBJECT
 
 public:
-    states(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = Qt::Window);
+    states(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::Window);
     ~states();
 
 public slots:
