@@ -13,6 +13,8 @@
 #include <QMessageBox>
 #include <QSqlError>
 #include <QVariant>
+#include "errorReporter.h"
+#include "guiErrorCheck.h"
 
 subaccount::subaccount(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
@@ -72,13 +74,7 @@ enum SetResponse subaccount::set(const ParameterList &pParams )
 void subaccount::sSave()
 {
   XSqlQuery subaccountSave;
-  if (_number->text().length() == 0)
-  {
-      QMessageBox::warning( this, tr("Cannot Save Sub Account"),
-                            tr("You must enter a valid Number.") );
-      return;
-  }
-  
+
   subaccountSave.prepare("SELECT subaccnt_id"
             "  FROM subaccnt"
             " WHERE((subaccnt_id != :subaccnt_id)"
@@ -86,21 +82,24 @@ void subaccount::sSave()
   subaccountSave.bindValue(":subaccnt_id", _subaccntid);
   subaccountSave.bindValue(":subaccnt_number", _number->text());
   subaccountSave.exec();
-  if(subaccountSave.first())
-  {
-    QMessageBox::critical(this, tr("Duplicate Sub Account Number"),
-      tr("A Sub Account Number already exists for the one specified.") );
-    return;
-  }
+
+  QList<GuiErrorCheck> errors;
+    errors<< GuiErrorCheck(_number->text().length() == 0, _number,
+                           tr("You must enter a valid Number."))
+          << GuiErrorCheck(subaccountSave.first(), _number,
+                           tr("A Sub Account Number already exists for the one specified."))
+    ;
+    if (GuiErrorCheck::reportErrors(this, tr("Cannot Save Sub Account"), errors))
+      return;
 
   if (_mode == cNew)
   {
     subaccountSave.exec("SELECT NEXTVAL('subaccnt_subaccnt_id_seq') AS subaccnt_id;");
     if (subaccountSave.first())
       _subaccntid = subaccountSave.value("subaccnt_id").toInt();
-    else if (subaccountSave.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Sub Account Information"),
+                                  subaccountSave, __FILE__, __LINE__))
     {
-      systemError(this, subaccountSave.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     
@@ -135,9 +134,9 @@ void subaccount::sSave()
   subaccountSave.bindValue(":subaccnt_number", _number->text());
   subaccountSave.bindValue(":subaccnt_descrip", _descrip->toPlainText());
   subaccountSave.exec();
-  if (subaccountSave.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Sub Account Information"),
+                                subaccountSave, __FILE__, __LINE__))
   {
-    systemError(this, subaccountSave.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   
