@@ -21,7 +21,7 @@ class subaccount : public XDialog, public Ui::subaccount
     Q_OBJECT
 
 public:
-    subaccount(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    subaccount(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~subaccount();
 
 public slots:
