@@ -24,6 +24,8 @@ public:
     task(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~task();
 
+    static bool userHasPriv(const int = cView, const int = 0);
+
     virtual SetResponse set(const ParameterList & pParams );
 
 public slots:
@@ -34,6 +36,7 @@ public slots:
     virtual void sFillUserList();
     virtual void sAssignedToChanged(const int);
     virtual void sStatusChanged(const int);
+    virtual void sCompletedChanged();
     virtual void sHoursAdjusted();
     virtual void sExpensesAdjusted();
 
