@@ -16,6 +16,8 @@
 #include <QCloseEvent>
 
 #include "storedProcErrorLookup.h"
+#include "errorReporter.h"
+#include "guiErrorCheck.h"
 
 taxClass::taxClass(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
@@ -101,12 +103,13 @@ void taxClass::sCheck()
 void taxClass::sSave()
 {
   XSqlQuery taxSave;
-  if (_taxClass->text().length() == 0)
-  {
-      QMessageBox::warning( this, tr("Cannot Save Tax Class"),
-                            tr("You must enter a valid Code.") );
+
+  QList<GuiErrorCheck> errors;
+    errors<< GuiErrorCheck(_taxClass->text().length() == 0, _taxClass,
+                           tr("You must enter a valid Code."))
+    ;
+    if (GuiErrorCheck::reportErrors(this, tr("Cannot Save Tax Class"), errors))
       return;
-  }
   
   if (_mode == cEdit)
   {
@@ -132,9 +135,9 @@ void taxClass::sSave()
     _taxClass->setFocus();
     return;
   }
-  else if (taxSave.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Tax Class Information"),
+                                taxSave, __FILE__, __LINE__))
   {
-    systemError(this, taxSave.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -159,7 +162,8 @@ void taxClass::sSave()
     else if (taxSave.lastError().type() != QSqlError::NoError)
     {
       rollback.exec();
-      systemError(this, taxSave.lastError().databaseText(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Tax Class Information"),
+                           taxSave, __FILE__, __LINE__);
       return;
     }
 
@@ -176,7 +180,8 @@ void taxClass::sSave()
   if (taxSave.lastError().type() != QSqlError::NoError)
   {
     rollback.exec();
-    systemError(this, taxSave.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Tax Class Information"),
+                         taxSave, __FILE__, __LINE__);
     return;
   }
 
@@ -199,9 +204,9 @@ void taxClass::populate()
     _description->setText(taxpopulate.value("taxclass_descrip").toString());
 	_seq->setValue(taxpopulate.value("taxclass_sequence").toInt());
   }
-  else if (taxpopulate.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Tax Class Information"),
+                                taxpopulate, __FILE__, __LINE__))
   {
-    systemError(this, taxpopulate.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
