@@ -13,6 +13,8 @@
 #include <QMessageBox>
 #include <QSqlError>
 #include <QVariant>
+#include "errorReporter.h"
+#include "guiErrorCheck.h"
 
 taxRegistration::taxRegistration(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
   : XDialog(parent, name, modal, fl)
@@ -46,7 +48,7 @@ void taxRegistration::languageChange()
   retranslateUi(this);
 }
 
-enum SetResponse taxRegistration::set(const ParameterList pParams)
+enum SetResponse taxRegistration::set(const ParameterList& pParams)
 {
   XSqlQuery taxet;
   XDialog::set(pParams);
@@ -80,9 +82,9 @@ enum SetResponse taxRegistration::set(const ParameterList pParams)
 		{
           _taxregid = taxet.value("_taxreg_id").toInt();
 		}
-      else if (taxet.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Tax Registration Information"),
+                                    taxet, __FILE__, __LINE__))
       {
-        systemError(this, taxet.lastError().databaseText(), __FILE__, __LINE__);
         return UndefinedError;
       }
     }
@@ -118,12 +120,7 @@ enum SetResponse taxRegistration::set(const ParameterList pParams)
 void taxRegistration::sSave() 
 {
   XSqlQuery taxSave;
- if( _dates->startDate() > _dates->endDate()) 
-	{
-	  QMessageBox::critical(this, tr("Incorrect Date Entry"),
-	   tr("The start date should be earlier than the end date.") );
-	  return;
-	}
+
   taxSave.prepare("SELECT taxreg_id"
             "  FROM taxreg"
             " WHERE((taxreg_id != :taxreg_id)"
@@ -143,13 +140,15 @@ void taxRegistration::sSave()
   if(!_reltype.isEmpty())
     taxSave.bindValue(":taxreg_rel_type", _reltype);
   taxSave.exec();
-  if(taxSave.first())
-  {
-    QMessageBox::critical(this, tr("Duplicate Tax Registration"),
-      tr("A Tax Registration already exists for the parameters specified.") );
-    _taxZone->setFocus();
-    return;
-  }
+
+  QList<GuiErrorCheck> errors;
+    errors<< GuiErrorCheck(_dates->startDate() > _dates->endDate(), _dates,
+                           tr("The start date should be earlier than the end date.")),
+    errors<< GuiErrorCheck(taxSave.first(), _taxZone,
+                           tr("A Tax Registration already exists for the parameters specified."))
+    ;
+    if (GuiErrorCheck::reportErrors(this, tr("Incorrect Date Entry"), errors))
+      return;
 
 
   if (cNew == _mode) 
@@ -188,9 +187,9 @@ void taxRegistration::sSave()
   taxSave.bindValue(":taxreg_notes", _notes->toPlainText());
 	
   taxSave.exec();
-  if (taxSave.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Tax Registration Information"),
+                                taxSave, __FILE__, __LINE__))
   {
-    systemError(this, taxSave.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   accept();
@@ -217,9 +216,9 @@ void taxRegistration::sPopulate()
 	if (handleReltype() < 0)
       return;
   }
-  else if (taxPopulate.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Tax Registration Information"),
+                                taxPopulate, __FILE__, __LINE__))
   {
-    systemError(this, taxPopulate.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
