@@ -16,6 +16,8 @@
 #include <QCloseEvent>
 
 #include "storedProcErrorLookup.h"
+#include "errorReporter.h"
+#include "guiErrorCheck.h"
 
 taxZone::taxZone(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
@@ -100,12 +102,13 @@ void taxZone::sCheck()
 void taxZone::sSave()
 {
   XSqlQuery taxSave;
-  if (_taxZone->text().length() == 0)
-  {
-      QMessageBox::warning( this, tr("Cannot Save Tax Zone"),
-                            tr("You must enter a valid Code.") );
+
+  QList<GuiErrorCheck> errors;
+    errors<< GuiErrorCheck(_taxZone->text().length() == 0, _taxZone,
+                           tr("You must enter a valid Code."))
+    ;
+    if (GuiErrorCheck::reportErrors(this, tr("Cannot Save Tax Zone"), errors))
       return;
-  }
   
   if (_mode == cEdit)
   {
@@ -131,9 +134,9 @@ void taxZone::sSave()
     _taxZone->setFocus();
     return;
   }
-  else if (taxSave.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Tax Zone Information"),
+                                taxSave, __FILE__, __LINE__))
   {
-    systemError(this, taxSave.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -157,7 +160,8 @@ void taxZone::sSave()
     else if (taxSave.lastError().type() != QSqlError::NoError)
     {
       rollback.exec();
-      systemError(this, taxSave.lastError().databaseText(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Tax Zone Information"),
+                           taxSave, __FILE__, __LINE__);
       return;
     }
 
@@ -173,7 +177,8 @@ void taxZone::sSave()
   if (taxSave.lastError().type() != QSqlError::NoError)
   {
     rollback.exec();
-    systemError(this, taxSave.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Tax Zone Information"),
+                         taxSave, __FILE__, __LINE__);
     return;
   }
 
@@ -196,9 +201,9 @@ void taxZone::populate()
     _description->setText(taxpopulate.value("taxzone_descrip").toString());
 	
   }
-  else if (taxpopulate.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Tax Zone Information"),
+                                taxpopulate, __FILE__, __LINE__))
   {
-    systemError(this, taxpopulate.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
