@@ -18,6 +18,7 @@
 #include <openreports.h>
 #include "taxZone.h"
 #include "storedProcErrorLookup.h"
+#include "errorReporter.h"
 
 /*
  *  Constructs a taxZones as a child of 'parent', with the
@@ -89,14 +90,16 @@ void taxZones::sDelete()
     int returnVal = taxDelete.value("result").toInt();
     if (returnVal < 0)
     {
-      systemError(this, storedProcErrorLookup("deleteTaxZone", returnVal), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Tax Zone Information"),
+                             storedProcErrorLookup("deleteTaxZone", returnVal),
+                             __FILE__, __LINE__);
       return;
     }
 	sFillList(-1);
   }
-  else if (taxDelete.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Tax Zone Information"),
+                                taxDelete, __FILE__, __LINE__))
   {
-    systemError(this, taxDelete.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
