@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -18,6 +18,7 @@
 #include <openreports.h>
 
 #include "terms.h"
+#include "errorReporter.h"
 
 termses::termses(QWidget* parent, const char* name, Qt::WindowFlags fl)
     : XWidget(parent, name, fl)
@@ -96,9 +97,9 @@ void termses::sDelete()
                               "you may delete the selected Terms Code." ) );
     return;
   }
-  else if (termsesDelete.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Customer Information"),
+                                termsesDelete, __FILE__, __LINE__))
   {
-    systemError(this, termsesDelete.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -116,9 +117,45 @@ void termses::sDelete()
                               "may delete the selected Terms Code." ) );
     return;
   }
-  else if (termsesDelete.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Vendor Information"),
+                                termsesDelete, __FILE__, __LINE__))
+  {
+    return;
+  }
+
+  termsesDelete.prepare( "SELECT apopen_id FROM apopen "
+                         "WHERE apopen_terms_id=:terms_id;");
+  termsesDelete.bindValue(":terms_id", _terms->id());
+  termsesDelete.exec();
+  if (termsesDelete.first())
+  {
+    QMessageBox::critical( this, tr("Cannot Delete Terms Code"),
+                           tr("<p>You may not delete the selected Terms Code "
+                              "as there are one or more A/P documents "
+                              "assigned to it." ) );
+    return;
+  }
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving A/P Information"),
+                                termsesDelete, __FILE__, __LINE__))
+  {
+    return;
+  }
+
+  termsesDelete.prepare( "SELECT aropen_id FROM aropen "
+                         "WHERE aropen_terms_id=:terms_id;");
+  termsesDelete.bindValue(":terms_id", _terms->id());
+  termsesDelete.exec();
+  if (termsesDelete.first())
+  {
+    QMessageBox::critical( this, tr("Cannot Delete Terms Code"),
+                           tr("<p>You may not delete the selected Terms Code "
+                              "as there are one or more A/R documents "
+                              "assigned to it." ) );
+    return;
+  }
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving A/P Information"),
+                                termsesDelete, __FILE__, __LINE__))
   {
-    systemError(this, termsesDelete.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -126,7 +163,11 @@ void termses::sDelete()
              "WHERE (terms_id=:terms_id);" );
   termsesDelete.bindValue(":terms_id", _terms->id());
   termsesDelete.exec();
-
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Terms"),
+                                  termsesDelete, __FILE__, __LINE__))
+  {
+    return;
+  }
   sFillList();
 }
 
