@@ -16,6 +16,36 @@
 #include <QVariant>
 
 #include "storedProcErrorLookup.h"
+#include "errorReporter.h"
+
+bool todoItem::userHasPriv(const int pMode, const int pId)
+{
+  if (_privileges->check("MaintainAllToDoItems"))
+    return true;
+  bool personalPriv = _privileges->check("MaintainPersonalToDoItems");
+  if(pMode==cView)
+  {
+    if(_privileges->check("ViewAllToDoItems"))
+      return true;
+    personalPriv = personalPriv || _privileges->check("ViewPersonalToDoItems");
+  }
+
+  if(pMode==cNew)
+    return personalPriv;
+  else
+  {
+    XSqlQuery usernameCheck;
+    usernameCheck.prepare( "SELECT getEffectiveXtUser() IN (todoitem_owner_username, todoitem_username) AS canModify "
+                           "FROM todoitem "
+                            "WHERE (todoitem_id=:todoitem_id);" );
+    usernameCheck.bindValue(":todoitem_id", pId);
+    usernameCheck.exec();
+
+    if (usernameCheck.first())
+      return usernameCheck.value("canModify").toBool()&&personalPriv;
+    return false;
+  }
+}
 
 todoItem::todoItem(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
@@ -41,6 +71,7 @@ todoItem::todoItem(QWidget* parent, const char* name, bool modal, Qt::WindowFlag
 
   _assignedTo->setEnabled(_privileges->check("MaintainAllToDoItems"));
 
+  _close = false;
 }
 
 void todoItem::languageChange()
@@ -92,32 +123,7 @@ enum SetResponse todoItem::set(const ParameterList &pParams)
       _description->setEnabled(false);
     }
     else if (param.toString() == "view")
-    {
-      _mode = cView;
-
-      _owner->setEnabled(false);
-      _name->setEnabled(false);
-      _priority->setEnabled(false);
-      _incident->setEnabled(false);
-      _ophead->setEnabled(false);
-      _started->setEnabled(false);
-      _assigned->setEnabled(false);
-      _due->setEnabled(false);
-      _completed->setEnabled(false);
-      _pending->setEnabled(false);
-      _deferred->setEnabled(false);
-      _neither->setEnabled(false);
-      _assignedTo->setEnabled(false);
-      _description->setEnabled(false);
-      _notes->setEnabled(false);
-      _alarms->setReadOnly(true);
-      _comments->setReadOnly(true);
-      _documents->setReadOnly(true);
-      _crmacct->setReadOnly(true);
-      _cntct->setReadOnly(true);
-
-      _buttonBox->setStandardButtons(QDialogButtonBox::Close);
-    }
+      setViewMode();
   }
 
   param = pParams.value("incdt_id", &valid);
@@ -153,6 +159,34 @@ enum SetResponse todoItem::set(const ParameterList &pParams)
   return NoError;
 }
 
+void todoItem::setViewMode()
+{
+  _mode = cView;
+
+  _owner->setEnabled(false);
+  _name->setEnabled(false);
+  _priority->setEnabled(false);
+  _incident->setEnabled(false);
+  _ophead->setEnabled(false);
+  _started->setEnabled(false);
+  _assigned->setEnabled(false);
+  _due->setEnabled(false);
+  _completed->setEnabled(false);
+  _pending->setEnabled(false);
+  _deferred->setEnabled(false);
+  _neither->setEnabled(false);
+  _assignedTo->setEnabled(false);
+  _description->setEnabled(false);
+  _notes->setEnabled(false);
+  _alarms->setReadOnly(true);
+  _comments->setReadOnly(true);
+  _documents->setReadOnly(true);
+  _crmacct->setReadOnly(true);
+  _cntct->setReadOnly(true);
+
+  _buttonBox->setStandardButtons(QDialogButtonBox::Close);
+}
+
 void todoItem::sSave()
 {
   XSqlQuery todoSave;
@@ -223,15 +257,18 @@ void todoItem::sSave()
     int result = todoSave.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup(storedProc, result), __FILE__, __LINE__);
       rollbackq.exec();
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving To Do Information"),
+                             storedProcErrorLookup(storedProc, result),
+                             __FILE__, __LINE__);
       return;
     }
   }
   else if (todoSave.lastError().type() != QSqlError::NoError)
   {
     rollbackq.exec();
-    systemError(this, todoSave.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving To Do Information"),
+                         todoSave, __FILE__, __LINE__);
     return;
   }
 
@@ -247,7 +284,8 @@ void todoItem::sSave()
     if (! recurq.exec())
     {
       rollbackq.exec();
-      systemError(this, recurq.lastError().text(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving To Do Information"),
+                           recurq, __FILE__, __LINE__);
       return;
     }
   }
@@ -261,7 +299,8 @@ void todoItem::sSave()
     if (! recurq.exec())
     {
       rollbackq.exec();
-      systemError(this, recurq.lastError().text(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving To Do Information"),
+                           recurq, __FILE__, __LINE__);
       return;
     }
   }
@@ -270,7 +309,10 @@ void todoItem::sSave()
   if (! _recurring->save(true, cp, &errmsg))
   {
     rollbackq.exec();
-    systemError(this, errmsg, __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Occurred"),
+                         tr("%1: %2")
+                         .arg(windowTitle())
+                         .arg(errmsg),__FILE__,__LINE__);
     return;
   }
 
@@ -281,6 +323,43 @@ void todoItem::sSave()
 
 void todoItem::sPopulate()
 {
+  if (_mode == cEdit && !_lock.acquire("todoitem", _todoitemid, AppLock::Interactive))
+    setViewMode();
+
+  _close = false;
+
+  foreach (QWidget* widget, QApplication::allWidgets())
+  {
+    if (!widget->isWindow() || !widget->isVisible())
+      continue;
+
+    todoItem *w = qobject_cast<todoItem*>(widget);
+
+    if (w && w != this && w->id()==_todoitemid)
+    {
+      // detect "i'm my own grandpa"
+      QObject *p;
+      for (p = parent(); p && p != w ; p = p->parent())
+        ; // do nothing
+      if (p == w)
+      {
+        QMessageBox::warning(this, tr("Cannot Open Recursively"),
+                             tr("This todo is already open and cannot be "
+                                "raised. Please close windows to get to it."));
+        _close = true;
+      } else if (p) {
+        w->setFocus();
+        if (omfgThis->showTopLevel())
+        {
+          w->raise();
+          w->activateWindow();
+        }
+        _close = true;
+      }
+      break;
+    }
+  }
+
   XSqlQuery todoPopulate;
   todoPopulate.prepare( "SELECT * "
              "FROM todoitem "
@@ -334,9 +413,9 @@ void todoItem::sPopulate()
                           "TODO");
     _cntct->setId(todoPopulate.value("todoitem_cntct_id").toInt());
   }
-  else if (todoPopulate.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving To Do Information"),
+                                todoPopulate, __FILE__, __LINE__))
   {
-    systemError(this, todoPopulate.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -360,10 +439,32 @@ void todoItem::sHandleIncident()
     incdtq.exec();
     if (incdtq.first())
       _crmacct->setId(incdtq.value("incdt_crmacct_id").toInt());
-    else if (incdtq.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Incident Information"),
+                                  incdtq, __FILE__, __LINE__))
     {
-      systemError(this, incdtq.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
 }
+
+int todoItem::id()
+{
+  return _todoitemid;
+}
+
+void todoItem::setVisible(bool visible)
+{
+  if (_close)
+    close();
+  else
+    XDialog::setVisible(visible);
+}
+
+void todoItem::done(int result)
+{
+  if (!_lock.release())
+    ErrorReporter::error(QtCriticalMsg, this, tr("Locking Error"),
+                         _lock.lastError(), __FILE__, __LINE__);
+
+  XDialog::done(result);
+}
