@@ -213,9 +213,8 @@ enum SetResponse transferOrder::set(const ParameterList &pParams)
       }
       else
       {
-        systemError(this, tr("A System Error occurred at %1::%2.")
-                          .arg(__FILE__)
-                          .arg(__LINE__) );
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Planned Order Information"),
+                             transferet, __FILE__, __LINE__);
         return UndefinedError;
       }
 
@@ -744,7 +743,8 @@ void transferOrder::sPopulateMenu(QMenu *pMenu)
       }
     }
 
-    if(_metrics->boolean("EnableTOShipping"))
+    if(_metrics->boolean("EnableTOShipping") &&
+       (_lineMode == cActiveOpen || _lineMode == cInactiveOpen))
     {
       if(_numSelected == 1)
         pMenu->addSeparator();
@@ -1026,8 +1026,9 @@ void transferOrder::sAction()
         int result = transferAction.value("result").toInt();
         if (result < 0)
         {
-          systemError(this, storedProcErrorLookup("closeToItem", result),
-                      __FILE__, __LINE__);
+          ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Transfer Order Information"),
+                                 storedProcErrorLookup("closeToItem", result),
+                                 __FILE__, __LINE__);
           return;
         }
       }
@@ -1091,8 +1092,9 @@ void transferOrder::sDelete()
           {
             int result = transferDelete.value("result").toInt();
             if (result < 0)
-              systemError(this, storedProcErrorLookup("deleteTO", result),
-			  __FILE__, __LINE__);            
+              ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Transfer Order"),
+                                  storedProcErrorLookup("deleteTO", result),
+                                  __FILE__, __LINE__);
           }
           ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Order"),
                                transferDelete, __FILE__, __LINE__);
@@ -1378,7 +1380,7 @@ bool transferOrder::deleteForCancel()
     deleteTo = true;
   }
 
-  if (cNew == _mode && _toitem->topLevelItemCount() > 0)
+  if (cNew == _mode && _toitem->topLevelItemCount() > 0 && !_captive)
   {
     if (QMessageBox::question(this, tr("Delete Transfer Order?"),
                           tr("<p>Are you sure you want to delete this "
@@ -1411,8 +1413,9 @@ bool transferOrder::deleteForCancel()
     {
       int result = query.value("result").toInt();
       if (result < 0)
-        systemError(this, storedProcErrorLookup("deleteTO", result),
-                    __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Transfer Order"),
+                             storedProcErrorLookup("deleteTO", result),
+                             __FILE__, __LINE__);
 
       sReleaseNumber();
     }
@@ -1763,46 +1766,20 @@ void transferOrder::viewTransferOrder( int pId )
 void transferOrder::sReturnStock()
 {
   XSqlQuery transferReturnStock;
-  XSqlQuery rollback;
-  rollback.prepare("ROLLBACK;");
-
-  transferReturnStock.exec("BEGIN;");	// because of possible lot, serial, or location distribution cancelations
   transferReturnStock.prepare("SELECT returnItemShipments('TO', :toitem_id, 0, CURRENT_TIMESTAMP) AS result;");
   QList<XTreeWidgetItem*> selected = _toitem->selectedItems();
   for (int i = 0; i < selected.size(); i++)
   {
     transferReturnStock.bindValue(":toitem_id", ((XTreeWidgetItem*)(selected[i]))->id());
     transferReturnStock.exec();
-    if (transferReturnStock.first())
+  if (transferReturnStock.lastError().type() != QSqlError::NoError)
     {
-      int result = transferReturnStock.value("result").toInt();
-      if (result < 0)
-      {
-        rollback.exec();
-        systemError(this, storedProcErrorLookup("returnItemShipments", result) +
-                          tr("<br>Line Item %1").arg(selected[i]->text(0)),
-                           __FILE__, __LINE__);
-        return;
-      }
-      if (distributeInventory::SeriesAdjust(transferReturnStock.value("result").toInt(), this) == XDialog::Rejected)
-      {
-        rollback.exec();
-        QMessageBox::information( this, tr("Return Stock"), tr("Transaction Canceled") );
-        return;
-      }
-
-    }
-    else if (transferReturnStock.lastError().type() != QSqlError::NoError)
-    {
-      rollback.exec();
-      systemError(this, tr("Line Item %1\n").arg(selected[i]->text(0)) +
-                        transferReturnStock.lastError().databaseText(), __FILE__, __LINE__);
-      return;
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Return Item Information"),
+                           transferReturnStock, __FILE__, __LINE__);
+      continue;
     }
   }
 
-  transferReturnStock.exec("COMMIT;");
-
   sFillItemList();
 }
 
@@ -1835,11 +1812,51 @@ void transferOrder::sIssueStock()
 
 void transferOrder::sIssueLineBalance()
 {
+  int itemsiteId;
+  int succeeded = 0;
+  double balance;
+  QString itemNumber;
+  QList<QString> failedItems;
+  QList<QString> errors;
+
   XSqlQuery transferIssueLineBalance;
   QList<XTreeWidgetItem*> selected = _toitem->selectedItems();
   for (int i = 0; i < selected.size(); i++)
   {
     XTreeWidgetItem* toitem = (XTreeWidgetItem*)(selected[i]);
+
+    // Get toitem and related info
+    XSqlQuery toitemInfo;
+    toitemInfo.prepare("SELECT itemsite_id, item_number, tohead_srcname, calcIssueToShippingLineBalance('TO', toitem_id) AS balance "
+                       "FROM toitem "
+                       "  JOIN tohead ON toitem_tohead_id=tohead_id "
+                       "  JOIN item ON toitem_item_id=item_id "
+                       "  JOIN itemsite ON item_id=itemsite_item_id "
+                       "    AND tohead_src_warehous_id=itemsite_warehous_id "
+                       "WHERE toitem_id=:toitem_id; ");
+    toitemInfo.bindValue(":toitem_id", toitem->id());
+    toitemInfo.exec();
+    if (toitemInfo.first())
+    {
+      itemsiteId = toitemInfo.value("itemsite_id").toInt();
+      itemNumber = toitemInfo.value("item_number").toString();
+      balance = toitemInfo.value("balance").toDouble();
+    }
+    else 
+    {
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Checking Item Availability"),
+        transferIssueLineBalance, __FILE__, __LINE__);
+      failedItems.append(selected[i]->text(1)); // todo - don't hardcode
+      errors.append(tr("Failed to retrieve Transfer Order Item information for Line # %1. %2")
+        .arg(selected[i]->text(0))
+        .arg(toitemInfo.lastError().databaseText()));
+      continue;
+    }
+
+    // balance is 0, nothing to do
+    if (balance == 0)
+      continue;
+
     // skip if status = C or X or U
     if (toitem->altId() != 1 && toitem->altId() != 4 && toitem->altId() != 5)
     {
@@ -1854,68 +1871,115 @@ void transferOrder::sIssueLineBalance()
           int result = transferIssueLineBalance.value("result").toInt();
           if (result < 0)
           {
-            transferIssueLineBalance.prepare("SELECT item_number, tohead_srcname "
-                "  FROM toitem, tohead, item "
-                " WHERE ((toitem_item_id=item_id)"
-                "   AND  (toitem_tohead_id=tohead_id)"
-                "   AND  (toitem_id=:toitem_id)); ");
-            transferIssueLineBalance.bindValue(":toitem_id", toitem->id());
-            transferIssueLineBalance.exec();
-            if (! transferIssueLineBalance.first() && transferIssueLineBalance.lastError().type() != QSqlError::NoError)
-            {
-              ErrorReporter::error(QtCriticalMsg, this, tr("Error Getting Item"),
-                                   transferIssueLineBalance, __FILE__, __LINE__);
-              systemError(this,
-                storedProcErrorLookup("sufficientInventoryToShipItem",
-                          result)
-                .arg(transferIssueLineBalance.value("item_number").toString())
-                .arg(transferIssueLineBalance.value("tohead_srcname").toString()), __FILE__, __LINE__);
-              return;
-            }
+            failedItems.append(itemNumber);
+            errors.append(tr("Error Checking Item Availability %1")
+              .arg(storedProcErrorLookup("sufficientInventoryToShipItem", result)));
+            continue;
           }
         }
         else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Checking Inventory"),
                                transferIssueLineBalance, __FILE__, __LINE__))
-          return;
+        {
+          failedItems.append(itemNumber);
+          errors.append(tr("Error Checking Inventory %1")
+            .arg(transferIssueLineBalance.lastError().databaseText()));
+          continue;
+        }
       }
+      
+      // Stage cleanup query
+      XSqlQuery cleanup;
+      cleanup.prepare("SELECT deleteitemlocseries(:itemlocSeries, TRUE);");
 
+      // Generate the series and itemlocdist record (if controlled)
+      int itemlocSeries = distributeInventory::SeriesCreate(itemsiteId, balance * -1, "TO", "SH", toitem->id());
+      if (itemlocSeries <= 0)
+      {
+        failedItems.append(itemNumber);
+        errors.append(tr("Error Issuing Item at distributeInventory::SeriesCreate."));
+        continue;
+      }
+      else 
+        cleanup.bindValue(":itemlocSeries", itemlocSeries);
+
+      // Distribute inventory
+      if (distributeInventory::SeriesAdjust(itemlocSeries, this, QString(), QDate(),
+        QDate(), true) == XDialog::Rejected)
+      {
+        cleanup.exec();
+        failedItems.append(itemNumber);
+        errors.append("Detail Distribution Cancelled");
+
+        // If it's not the last item in the loop, ask the user to exit loop or continue
+        if (i != (selected.size() -1))
+        {
+          if (QMessageBox::question(this,  tr("Issue Line Balance to Shipping"),
+            tr("Posting distribution detail for item number %1 was cancelled but "
+              "there are more items to Issue to Shipping. Continue issuing the remaining materials?")
+            .arg(itemNumber),
+            QMessageBox::Yes | QMessageBox::No, QMessageBox::No) == QMessageBox::Yes)
+          {
+            continue;
+          }
+          else
+          {
+            break;
+          }
+        }
+        else
+          continue;
+      }
+    
       XSqlQuery rollback;
       rollback.prepare("ROLLBACK;");
 
-      transferIssueLineBalance.exec("BEGIN;");	// because of possible lot, serial, or location distribution cancelations
-      transferIssueLineBalance.prepare("SELECT issueLineBalanceToShipping('TO', :toitem_id, CURRENT_TIMESTAMP) AS result;");
-      transferIssueLineBalance.bindValue(":toitem_id", toitem->id());
+      transferIssueLineBalance.exec("BEGIN;");	// TODO - remove after negative return values have been removed from issueToShipping() 
+      transferIssueLineBalance.prepare("SELECT issueToShipping('TO', :toitemId, :qty, :itemlocSeries, CURRENT_TIMESTAMP, NULL, FALSE, TRUE) AS result;");
+      transferIssueLineBalance.bindValue(":toitemId", toitem->id());
+      transferIssueLineBalance.bindValue(":qty", balance);
+      transferIssueLineBalance.bindValue(":itemlocSeries", itemlocSeries);
       transferIssueLineBalance.exec();
       if (transferIssueLineBalance.first())
       {
         int result = transferIssueLineBalance.value("result").toInt();
-        if (result < 0)
+        if (result < 0 || result != itemlocSeries)
         {
           rollback.exec();
-          systemError(this, storedProcErrorLookup("issueLineBalance", result) +
-                            tr("<br>Line Item %1").arg(selected[i]->text(0)),
-                      __FILE__, __LINE__);
-          return;
-        }
-        if (distributeInventory::SeriesAdjust(transferIssueLineBalance.value("result").toInt(), this) == XDialog::Rejected)
-        {
-          rollback.exec();
-          QMessageBox::information( this, tr("Issue to Shipping"), tr("Transaction Canceled") );
-          return;
+          cleanup.exec();
+          failedItems.append(itemNumber);
+          errors.append(tr("Error Issuing Item. %1")
+            .arg(storedProcErrorLookup("issueLineBalance", result)));
+          continue;
         }
 
+        succeeded++;
         transferIssueLineBalance.exec("COMMIT;");
       }
       else
       {
         rollback.exec();
-        systemError(this, tr("Line Item %1\n").arg(selected[i]->text(0)) +
-                          transferIssueLineBalance.lastError().databaseText(), __FILE__, __LINE__);
-        return;
+        cleanup.exec();    
+        failedItems.append(itemNumber);
+        errors.append(tr("Error Issuing Item. %1")
+          .arg(transferIssueLineBalance.lastError().databaseText()));
+        continue;
       }
     }
   }
 
+  if (errors.size() > 0)
+  {
+    QMessageBox dlg(QMessageBox::Critical, "Errors Issuing to Shipping", "", QMessageBox::Ok, this);
+    dlg.setText(tr("%1 Items succeeded.\n%2 Items failed.").arg(succeeded).arg(failedItems.size()));
+
+    QString details;
+    for (int i=0; i<failedItems.size(); i++)
+      details += tr("Item %1 failed with:\n%2\n").arg(failedItems[i]).arg(errors[i]);
+    dlg.setDetailedText(details);
+
+    dlg.exec();
+  }
+
   sFillItemList();
 }
 
@@ -1951,7 +2015,10 @@ bool transferOrder::sQESave()
   if (! _qeitem->submitAll())
   {
     if (! _qeitem->lastError().databaseText().isEmpty())
-      systemError(this, _qeitem->lastError().databaseText(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Occurred"),
+                         tr("%1: Error Saving Quick Item Information: \n %2")
+                        .arg(windowTitle())
+                        .arg(_qeitem->lastError().databaseText()),__FILE__,__LINE__);
     return false;
   }
   sFillItemList();
@@ -1962,7 +2029,9 @@ void transferOrder::sQEDelete()
 {
   if (! _qeitem->removeRow(_qeitemView->currentIndex().row()))
   {
-    systemError(this, tr("Removing row from view failed"), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Occurred"),
+                         tr("%1: Removing row from view failed ")
+                            .arg(windowTitle()),__FILE__,__LINE__);
     return;
   }
 }
