@@ -21,7 +21,7 @@ class transferTrans : public XWidget, public Ui::transferTrans
     Q_OBJECT
 
 public:
-    transferTrans(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = Qt::Window);
+    transferTrans(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::Window);
     ~transferTrans();
 
 public slots:
@@ -32,7 +32,7 @@ protected slots:
 
     virtual void sHandleItem();
     virtual void sPost();
-    virtual void sPopulateFromQty( int pWarehousid );
+    virtual void sPopulateFromQty();
     virtual void sPopulateToQty( int pWarehousid );
     virtual void sUpdateQty( const QString & pQty );
 
