@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -18,6 +18,7 @@
 #include "distributeInventory.h"
 #include "inputManager.h"
 #include "storedProcErrorLookup.h"
+#include "errorReporter.h"
 
 transformTrans::transformTrans(QWidget* parent, const char* name, Qt::WindowFlags fl)
     : XWidget(parent, name, fl)
@@ -25,11 +26,12 @@ transformTrans::transformTrans(QWidget* parent, const char* name, Qt::WindowFlag
   setupUi(this);
 
   connect(_item,       SIGNAL(newId(int)), this, SLOT(sPopulateQOH()));
-  connect(_post,        SIGNAL(clicked()), this, SLOT(sPost()));
-  connect(_qty, SIGNAL(editingFinished()), this, SLOT(sRecalculateAfter()));
+  connect(_item,       SIGNAL(newId(int)), this, SLOT(sFillList()));
+  connect(_post,       SIGNAL(clicked()), this, SLOT(sPost()));
+  connect(_qty,        SIGNAL(editingFinished()), this, SLOT(sRecalculateAfter()));
   connect(_source,     SIGNAL(newId(int)), this, SLOT(sPopulateQOH()));
-  connect(_source,    SIGNAL(valid(bool)), this, SLOT(sHandleButtons()));
-  connect(_target,     SIGNAL(newID(int)), this, SLOT(sPopulateTarget(int)));
+  connect(_source,     SIGNAL(valid(bool)), this, SLOT(sHandleButtons()));
+  connect(_target,     SIGNAL(newID(int)), this, SLOT(sPopulateTarget()));
   connect(_target,     SIGNAL(newID(int)), this, SLOT(sHandleButtons()));
   connect(_warehouse,  SIGNAL(newID(int)), this, SLOT(sFillList()));
   connect(_warehouse,  SIGNAL(newID(int)), this, SLOT(sPopulateQOH()));
@@ -198,7 +200,7 @@ void transformTrans::sPost()
   XSqlQuery rollback;
   rollback.prepare("ROLLBACK;");
 
-  transformPost.exec("BEGIN;");	// because of possible distribution cancelations
+  transformPost.exec("BEGIN;"); // Until postTransformTrans no longer returns 0/-. 
   transformPost.prepare( "SELECT postTransformTrans(s.itemsite_id, t.itemsite_id,"
              "                          :itemloc_id, :qty, :docnumber,"
               "                          :comments, :date) AS result "
@@ -222,17 +224,20 @@ void transformTrans::sPost()
     if (result < 0)
     {
       rollback.exec();
-      systemError(this, storedProcErrorLookup("postTransformTrans", result),
-                  __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting Transaction"),
+                             storedProcErrorLookup("postTransformTrans", result),
+                             __FILE__, __LINE__);
       return;
     }
     else if (transformPost.lastError().type() != QSqlError::NoError)
     {
       rollback.exec();
-      systemError(this, transformPost.lastError().databaseText(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting Transaction"),
+                           transformPost, __FILE__, __LINE__);
       return;
     }
-
+    // TODO - this does nothing currently. There is a feature gap/bug for items that have different control metrics, 
+    // in that case SeriesAdjust should be called but additional logic will be needed here and in the postTransformTrans function.
     if (distributeInventory::SeriesAdjust(transformPost.value("result").toInt(), this) == XDialog::Rejected)
     {
       rollback.exec();
@@ -246,17 +251,20 @@ void transformTrans::sPost()
   else if (transformPost.lastError().type() != QSqlError::NoError)
   {
     rollback.exec();
-    systemError(this, transformPost.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting Transaction"),
+                         transformPost, __FILE__, __LINE__);
     return;
   }
   else
   {
     rollback.exec();
-    systemError( this,
-                tr("<p>No transaction was done because either Item %1 or Item "
-                   "%2 was not found at Site %3.")
-                .arg(_item->itemNumber(), _target->currentText(),
-                     _warehouse->currentText()));
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Occurred"),
+                         tr("%1: <p>No transaction occurred due to either Item %2 or Item "
+                            "%3 was not found at Site %4.")
+                         .arg(windowTitle())
+                         .arg(_item->itemNumber())
+                         .arg(_target->currentText())
+                         .arg(_warehouse->currentText()),__FILE__,__LINE__);
   }
 
   if (_captive)
@@ -279,12 +287,12 @@ void transformTrans::sPost()
   }
 }
 
-void transformTrans::sPopulateTarget(int /*pItemid*/)
+void transformTrans::sPopulateTarget()
 {
   XSqlQuery transformPopulateTarget;
-  if (!_item->isValid())
+  if (!_item->isValid() || !_target->isValid() || !_warehouse->isValid())
     return;
-	
+  
   transformPopulateTarget.prepare( "SELECT item_descrip1, item_descrip2, itemsite_qtyonhand "
              "FROM itemsite JOIN item ON (item_id=itemsite_item_id) "
              "WHERE ( (itemsite_item_id=:item_id) "
@@ -303,10 +311,10 @@ void transformTrans::sPopulateTarget(int /*pItemid*/)
     _targetIsValid = true;
 
   }
-  else if (transformPopulateTarget.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving ItemSite Information"),
+                                transformPopulateTarget, __FILE__, __LINE__))
   {
     _targetIsValid = false;
-    systemError(this, transformPopulateTarget.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   else
@@ -338,9 +346,9 @@ void transformTrans::sFillList()
   transformFillList.exec();
   if (transformFillList.first())
     _target->populate(transformFillList);
-  else if (transformFillList.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                transformFillList, __FILE__, __LINE__))
   {
-    systemError(this, transformFillList.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   else
@@ -392,17 +400,19 @@ void transformTrans::sFillList()
     transformFillList.bindValue(":itemsite_id", itemsiteid);
     transformFillList.exec();
     _source->populate(transformFillList, true);
-    if (transformFillList.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                  transformFillList, __FILE__, __LINE__))
     {
-      systemError(this, transformFillList.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
-  else if (transformFillList.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                transformFillList, __FILE__, __LINE__))
   {
-    systemError(this, transformFillList.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
+
+  sPopulateTarget();
 }
 
 void  transformTrans::sPopulateQOH()
@@ -434,9 +444,9 @@ void  transformTrans::sPopulateQOH()
       sRecalculateAfter();
 
     }
-    else if (transformPopulateQOH.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                  transformPopulateQOH, __FILE__, __LINE__))
     {
-      systemError(this, transformPopulateQOH.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -451,4 +461,4 @@ void transformTrans::sRecalculateAfter()
 void transformTrans::sHandleButtons()
 {
   _post->setEnabled((! _controlled || _source->id() > -1) && _target->id() > -1 && _targetIsValid);
-}
+}
\ No newline at end of file
