@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -21,7 +21,7 @@ class transformTrans : public XWidget, public Ui::transformTrans
     Q_OBJECT
 
 public:
-    transformTrans(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = Qt::Window);
+    transformTrans(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::Window);
     ~transformTrans();
 
 public slots:
@@ -29,7 +29,7 @@ public slots:
     virtual void sFillList();
     virtual void sHandleButtons();
     virtual void sPopulateQOH();
-    virtual void sPopulateTarget(int pItemid);
+    virtual void sPopulateTarget();
     virtual void sPost();
     virtual void sRecalculateAfter();
 
