@@ -43,15 +43,14 @@ unappliedARCreditMemos::unappliedARCreditMemos(QWidget* parent, const char* name
   _aropen->addColumn( tr("Customer"), -1,              Qt::AlignLeft,   true,  "cust_name"   );
   _aropen->addColumn( tr("Amount"),   _moneyColumn,    Qt::AlignRight,  true,  "aropen_amount"  );
   _aropen->addColumn( tr("Applied"),  _moneyColumn,    Qt::AlignRight,  true,  "applied"  );
-  _aropen->addColumn( tr("Balance"),  _moneyColumn,    Qt::AlignRight,  true,  "balance"  );
+  _aropen->addColumn( tr("Balance"),  _moneyColumn,    Qt::AlignRight,  !omfgThis->singleCurrency(),  "balance"  );
   _aropen->addColumn( tr("Currency"), _currencyColumn, Qt::AlignLeft,   true,  "currAbbr" );
 
-  if (omfgThis->singleCurrency())
-    _aropen->hideColumn(5);
-
   if (_privileges->check("ApplyARMemos"))
     connect(_aropen, SIGNAL(valid(bool)), _apply, SLOT(setEnabled(bool)));
 
+  connect(_customerGroup, SIGNAL(updated()), this, SLOT(sFillList()));
+
   sFillList();
 }
 
@@ -72,9 +71,18 @@ void unappliedARCreditMemos::languageChange()
   retranslateUi(this);
 }
 
+bool unappliedARCreditMemos::setParams(ParameterList &params)
+{
+  _customerGroup->appendValue(params);
+  return true;
+}
+
 void unappliedARCreditMemos::sPrint()
 {
   ParameterList params;
+  if (! setParams(params))
+    return;
+
   params.append("isReport", true);
 
   orReport report("UnappliedARCreditMemos", params);
@@ -84,7 +92,6 @@ void unappliedARCreditMemos::sPrint()
     report.reportError(this);
 }
 
-
 void unappliedARCreditMemos::sNew()
 {
   ParameterList params;
@@ -118,12 +125,14 @@ void unappliedARCreditMemos::sFillList()
   MetaSQLQuery mql = mqlLoad("arCreditMemos", "unapplied");
   
   ParameterList params;
+  if (! setParams(params))
+    return;
   
   XSqlQuery qry = mql.toQuery(params);
   _aropen->populate(qry);
-  if (qry.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Unapplied AR Credit Memo Information"),
+                                qry, __FILE__, __LINE__))
   {
-    systemError(this, qry.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
