@@ -20,8 +20,9 @@ class unappliedARCreditMemos : public XWidget, public Ui::unappliedARCreditMemos
     Q_OBJECT
 
 public:
-    unappliedARCreditMemos(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = Qt::Window);
+    unappliedARCreditMemos(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::Window);
     ~unappliedARCreditMemos();
+    virtual bool setParams(ParameterList &params);
 
 public slots:
     virtual void sPrint();
