@@ -21,7 +21,7 @@ class unpostedGlSeries : public XWidget, public Ui::unpostedGlSeries
     Q_OBJECT
 
 public:
-    unpostedGlSeries(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = Qt::Window);
+    unpostedGlSeries(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::Window);
     ~unpostedGlSeries();
 
 public slots:
