@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -11,6 +11,7 @@
 #ifndef UNPOSTEDPURCHASEORDERS_H
 #define UNPOSTEDPURCHASEORDERS_H
 
+#include "applock.h"
 #include "display.h"
 
 #include "ui_unpostedPurchaseOrders.h"
@@ -32,8 +33,12 @@ public slots:
     virtual void sRelease();
     virtual void sUnrelease();
     virtual void sPrint();
+    virtual void sPrintForms();
     virtual void sView();
     virtual bool setParams(ParameterList &);
+
+private:
+    AppLock _lock;
 };
 
 #endif // UNPOSTEDPURCHASEORDERS_H
