@@ -20,7 +20,10 @@
 #include "storedProcErrorLookup.h"
 
 #include <parameter.h>
+#include <metasql.h>
+#include "mqlutil.h"
 
+#include "errorReporter.h"
 #include "hotkey.h"
 #include "imageList.h"
 #include "timeoutHandler.h"
@@ -39,6 +42,11 @@ userPreferences::userPreferences(QWidget* parent, const char* name, bool modal,
 
   if(!_privileges->check("MaintainPreferencesOthers"))
     _selectedUser->setEnabled(false);
+  if(!_privileges->check("MaintainPreferencesSelf"))
+  {
+    _currentUser->setEnabled(false);
+    _selectedUser->setChecked(true);
+  }
 
   QPushButton* apply = _buttonBox->button(QDialogButtonBox::Apply);
   connect(apply, SIGNAL(clicked()), this, SLOT(sApply()));
@@ -86,7 +94,21 @@ userPreferences::userPreferences(QWidget* parent, const char* name, bool modal,
   _backgroundList->setMaximumWidth(25);
 #endif
 
-  _user->setType(XComboBox::Users);
+  //_user->setType(XComboBox::Users);
+  XSqlQuery userPref;
+  QString qryUser = "SELECT usr_id, usr_username, usr_username "
+                    "FROM usr "
+                    "WHERE ((true) "
+                    "<? if not exists('maintainSelf') ?> "
+                    " AND NOT (usr_username = geteffectivextuser()) "
+                    "<? endif ?>) "
+                    "ORDER BY usr_username;";
+  MetaSQLQuery mql(qryUser);
+  ParameterList params;
+  if(_privileges->check("MaintainPreferencesSelf"))
+    params.append("maintainSelf", true);
+  userPref = mql.toQuery(params);
+  _user->populate(userPref);
 
   _ellipsesAction->append(1, tr("List"));
   _ellipsesAction->append(2, tr("Search"));
@@ -99,6 +121,12 @@ userPreferences::userPreferences(QWidget* parent, const char* name, bool modal,
   _translations->setEnabled(_privileges->check("MaintainTranslations"));
   _dictionaries->setEnabled(_privileges->check("MaintainDictionaries"));
 
+  _deprecationLevel->append(1, tr("Debug"),    "debug");
+  _deprecationLevel->append(2, tr("Info"),     "info");
+  _deprecationLevel->append(3, tr("Warning"),  "warning");
+  _deprecationLevel->append(4, tr("Critical"), "critical");
+  _deprecationLevel->append(5, tr("Fatal"),    "fatal");
+
   sPopulate();
   adjustSize();
 }
@@ -115,6 +143,19 @@ void userPreferences::languageChange()
   retranslateUi(this);
 }
 
+enum SetResponse userPreferences::set(const ParameterList &pParams)
+{
+  XDialog::set(pParams);
+  QVariant  param;
+  bool      valid;
+
+  param = pParams.value("passwordReset", &valid);
+  if (valid)
+    _tab->setCurrentIndex(_tab->indexOf(_password));
+
+  return NoError;
+}
+
 void userPreferences::setBackgroundImage(int pImageid)
 {
   XSqlQuery useretBackgroundImage;
@@ -128,9 +169,9 @@ void userPreferences::setBackgroundImage(int pImageid)
     _backgroundImageid = useretBackgroundImage.value("image_id").toInt();
     _background->setText(useretBackgroundImage.value("description").toString());
   }
-  else if (useretBackgroundImage.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Setting Background Image"),
+                                useretBackgroundImage, __FILE__, __LINE__))
   {
-    systemError(this, useretBackgroundImage.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -199,6 +240,8 @@ void userPreferences::sPopulate()
   else
     _richText->setChecked(true);
 
+  _richLimit->setValue(_pref->value("XTreeWidgetDataLimit").toDouble());
+
   _enableSpell->setChecked(_pref->boolean("SpellCheck"));
 
   //_rememberCheckBoxes->setChecked(! _pref->boolean("XCheckBox/forgetful"));
@@ -222,6 +265,7 @@ void userPreferences::sPopulate()
   _accountingToolbar->setChecked(_pref->boolean("ShowGLToolbar"));
   
   _listNumericItemsFirst->setChecked(_pref->boolean("ListNumericItemNumbersFirst"));
+  _disableWheelEvent->setChecked(_pref->boolean("DisableXComboBoxWheelEvent"));
   _ignoreTranslation->setChecked(_pref->boolean("IngoreMissingTranslationFiles"));
 
   _idleTimeout->setValue(_pref->value("IdleTimeout").toInt());
@@ -250,6 +294,7 @@ void userPreferences::sPopulate()
     _warehouseGroup->hide();
     
   _debug->setChecked(_pref->boolean("EnableScriptDebug"));
+  _deprecationLevel->setCode(_pref->value("DeprecationLevel"));
 
   sFillList();
   sFillWarehouseList();
@@ -270,28 +315,8 @@ void userPreferences::sSave(bool close)
 
   _pref->set("SpellCheck", _enableSpell->isChecked());
 
-  if(_enableSpell->isChecked())
-  {
-      QString langName = QLocale::languageToString(QLocale().language());
-      QString appPath = QApplication::applicationDirPath();
-      QString fullPathWithoutExt = appPath + "/" + langName;
-      QFile affFile(fullPathWithoutExt + ".aff");
-      QFile dicFile(fullPathWithoutExt + ".dic");
-      // If we don't have files for the first name lets try a more common naming convention
-      if(!(affFile.exists() && dicFile.exists()))
-      {
-        langName = QLocale().name().toLower(); // retruns lang_cntry format en_us for example
-        fullPathWithoutExt = appPath + "/" + langName;
-        affFile.setFileName(fullPathWithoutExt + tr(".aff"));
-        dicFile.setFileName(fullPathWithoutExt + tr(".dic"));
-      }
-      if(!affFile.exists() || !dicFile.exists())
-      {
-        QMessageBox::warning( this, tr("Spell Dictionary Missing"),
-                   tr("The following Hunspell files are required for spell checking: <p>")
-                   + fullPathWithoutExt + tr(".aff <p>") + fullPathWithoutExt + tr(".dic"));
-      }
-  }
+  if (_enableSpell->isChecked())
+    omfgThis->hunspell_initialize();
   
   _pref->set("ShowIMMenu", _inventoryMenu->isChecked());
   _pref->set("ShowPDMenu", _productsMenu->isChecked());
@@ -314,6 +339,7 @@ void userPreferences::sSave(bool close)
   _pref->set("PreferredWarehouse", ((_noWarehouse->isChecked()) ? -1 : _warehouse->id())  );
  
   _pref->set("ListNumericItemNumbersFirst", _listNumericItemsFirst->isChecked());
+  _pref->set("DisableXComboBoxWheelEvent", _disableWheelEvent->isChecked());
   _pref->set("IngoreMissingTranslationFiles", _ignoreTranslation->isChecked());
 
   _pref->set("IdleTimeout", _idleTimeout->value());
@@ -332,6 +358,7 @@ void userPreferences::sSave(bool close)
     _pref->set("InterfaceWindowOption", QString("TopLevel"));
     
   _pref->set("CopyListsPlainText", _plainText->isChecked());
+  _pref->set("XTreeWidgetDataLimit", QString::number(_richLimit->value()));
   _pref->set("EmailEvents", _emailEvents->isChecked());
 
   _pref->set("AlarmEventDefault", _alarmEvent->isChecked());
@@ -339,6 +366,7 @@ void userPreferences::sSave(bool close)
   _pref->set("AlarmSysmsgDefault", _alarmSysmsg->isChecked());
 
   _pref->set("EnableScriptDebug", _debug->isChecked());
+  _pref->set("DeprecationLevel",  _deprecationLevel->code());
 
   if (_currentUser->isChecked())
   {
@@ -434,11 +462,20 @@ bool userPreferences::save()
            .arg(_username->text()) );
     userave.bindValue(":password", passwd);
     userave.exec();
-    if (userave.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving User Information"),
+                                  userave, __FILE__, __LINE__))
     {
-      systemError(this, userave.lastError().databaseText(), __FILE__, __LINE__);
       return false;
     }
+
+    XSqlQuery usrq;
+    usrq.prepare("SELECT setUserPreference(:username, 'PasswordResetDate', :passdate);");
+    usrq.bindValue(":username", _username->text());
+    usrq.bindValue(":passdate", QDate::currentDate());
+    usrq.exec();
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Saving User Account"),
+                             usrq, __FILE__, __LINE__))
+      return false;
   }
   return true;
 }
@@ -605,9 +642,9 @@ void userPreferences::sAllWarehousesToggled(int pEvnttypeid)
     userAllWarehousesToggled.bindValue(":username", _user->currentText());
   userAllWarehousesToggled.bindValue(":evnttype_id", pEvnttypeid);
   userAllWarehousesToggled.exec();
-  if (userAllWarehousesToggled.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Selecting/De-Selecting All Warehouses For Selected Event"),
+                                userAllWarehousesToggled, __FILE__, __LINE__))
   {
-    systemError(this, userAllWarehousesToggled.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -638,9 +675,9 @@ void userPreferences::sWarehouseToggled(QTreeWidgetItem *selected)
   userWarehouseToggled.bindValue(":evnttype_id", _event->id());
   userWarehouseToggled.bindValue(":warehous_id", ((XTreeWidgetItem *)selected)->id());
   userWarehouseToggled.exec();
-  if (userWarehouseToggled.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Selecting/De-Selecting Warehouse For Selected Event"),
+                                userWarehouseToggled, __FILE__, __LINE__))
   {
-    systemError(this, userWarehouseToggled.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -668,9 +705,9 @@ void userPreferences::sFillWarehouseList()
       _warehouses->topLevelItem(i)->setText(0, tr("Yes"));
     else
       _warehouses->topLevelItem(i)->setText(0, tr("No"));
-    if (userFillWarehouseList.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving User Warehouse Information"),
+                                  userFillWarehouseList, __FILE__, __LINE__))
     {
-      systemError(this, userFillWarehouseList.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
