@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -18,6 +18,15 @@
 #include "contactcluster.h"
 #include "ui_vendor.h"
 
+class ParameterList;
+
+class dspCheckRegister;
+class dspPOsByVendor;
+class dspPoItemReceivingsByVendor;
+class dspVendorAPHistory;
+class unappliedAPCreditMemos;
+class selectPayments;
+
 class vendor : public XWidget, public Ui::vendor
 {
     Q_OBJECT
@@ -31,8 +40,11 @@ public:
 
 public slots:
     virtual SetResponse set(const ParameterList & pParams );
-    virtual void sSave();
+    virtual void setId(int);
+    virtual bool sSave();
+    virtual void sSaveClicked();
     virtual void sCheck();
+    virtual bool sCheckRequired();
     virtual bool sPopulate();
     virtual void sPrintAddresses();
     virtual void sNewAddress();
@@ -47,12 +59,17 @@ public slots:
     virtual void sViewTaxreg();
     virtual void sDeleteTaxreg();
     virtual void sHandleButtons();
+    virtual void sHandleDistButtons();
     virtual void sNumberEdited();
+    virtual void sSaveAndNewComment();
 
     virtual void sNext();
     virtual void sPrevious();
 
     virtual void clear();
+    virtual void close();
+    virtual void sNumberEditable(bool);
+    virtual void sPrepare();
 
 signals:
     void populated();
@@ -68,17 +85,24 @@ protected slots:
 
 protected:
     virtual void closeEvent(QCloseEvent*);
-    QValidator *_accountValidator;
-    QValidator *_routingValidator;
+    QValidator                  *_accountValidator;
+    QValidator                  *_routingValidator;
+    unappliedAPCreditMemos      *_credits;
+    selectPayments              *_payables;
+    dspPOsByVendor              *_po;
+    dspPoItemReceivingsByVendor *_receipts;
+    dspVendorAPHistory          *_history;
+    dspCheckRegister            *_checks;
 
 private:
     int _mode;
+    int _tempMode;
     int _vendid;
     int _crmacctid;
     int _NumberGen;
     AppLock _lock;
     QString _cachedNumber;
-    bool _ignoreClose;
+    bool _captive;
     bool _notice;
     QString _crmowner;
 
