@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2018 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -14,6 +14,8 @@
 #include <QSqlError>
 #include <QVariant>
 #include <vendorcluster.h>
+#include "errorReporter.h"
+#include "storedProcErrorLookup.h"
 
 vendorAddress::vendorAddress(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
@@ -70,10 +72,10 @@ enum SetResponse vendorAddress::set(const ParameterList &pParams)
         _address->setSearchAcct(setVendor.value("crmacct_id").toInt());
         _contact->setSearchAcct(setVendor.value("crmacct_id").toInt());
       }
-      else if (setVendor.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Vendor Information"),
+                                    setVendor, __FILE__, __LINE__))
       {
-	systemError(this, setVendor.lastError().databaseText(), __FILE__, __LINE__);
-	return UndefinedError;
+        return UndefinedError;
       }
     }
     else if (param.toString() == "edit")
@@ -99,38 +101,33 @@ enum SetResponse vendorAddress::set(const ParameterList &pParams)
 
 void vendorAddress::sSave()
 {
-  XSqlQuery vendorSave;
-  if (! vendorSave.exec("BEGIN"))
-  {
-    systemError(this, vendorSave.lastError().databaseText(), __FILE__, __LINE__);
-    return;
-  }
-
   XSqlQuery rollback;
   rollback.prepare("ROLLBACK;");
 
+  AddressCluster::SaveFlags addrSaveMode = AddressCluster::CHECK;
   int saveResult = _address->save(AddressCluster::CHECK);
   if (-2 == saveResult)
   {
-    int answer = QMessageBox::question(this,
-		    tr("Question Saving Address"),
-		    tr("There are multiple uses of this Vendor "
-		       "Address.\nWhat would you like to do?"),
-		    tr("Change This One"),
-		    tr("Change Address for All"),
-		    tr("Cancel"),
-		    2, 2);
-    if (0 == answer)
-      saveResult = _address->save(AddressCluster::CHANGEONE);
-    else if (1 == answer)
-      saveResult = _address->save(AddressCluster::CHANGEALL);
+    addrSaveMode = AddressCluster::askForSaveMode(_address->id());
+    if (addrSaveMode == AddressCluster::CHECK)
+      return;
   }
-  if (saveResult < 0)	// not else-if: this is error check for CHANGE{ONE,ALL}
+
+  XSqlQuery vendorSave;
+  if (! vendorSave.exec("BEGIN"))
+  {
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Vendor Information"),
+                         vendorSave, __FILE__, __LINE__);
+    return;
+  }
+
+  saveResult = _address->save(addrSaveMode);
+  if (saveResult < 0)
   {
-    systemError(this, tr("There was an error saving this address (%1).\n"
-			 "Check the database server log for errors.")
-		      .arg(saveResult), __FILE__, __LINE__);
     rollback.exec();
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Occurred"),
+                         storedProcErrorLookup("saveAddr", saveResult),
+                         __FILE__, __LINE__);
     _address->setFocus();
     return;
   }
@@ -143,7 +140,8 @@ void vendorAddress::sSave()
     else
     {
       rollback.exec();
-      systemError(this, vendorSave.lastError().databaseText(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Vendor Information"),
+                           vendorSave, __FILE__, __LINE__);
       return;
     }
 
@@ -177,7 +175,8 @@ void vendorAddress::sSave()
   if (vendorSave.lastError().type() != QSqlError::NoError)
   {
     rollback.exec();
-    systemError(this, vendorSave.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Vendor Information"),
+                         vendorSave, __FILE__, __LINE__);
     return;
   }
 
