@@ -21,16 +21,18 @@ class viewCheckRun : public XWidget, public Ui::viewCheckRun
     Q_OBJECT
 
 public:
-    viewCheckRun(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = Qt::Window);
+    viewCheckRun(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::Window);
     ~viewCheckRun();
 
 public slots:
+    virtual void sAltExchRate();
     virtual void sDelete();
     virtual void sEdit();
     virtual void sFillList();
     virtual void sFillList(int);
     virtual void sHandleItemSelection();
     virtual void sNewMiscCheck();
+    virtual void sPopulateMenu(QMenu *pMenu);
     virtual void sPost();
     virtual void sPostChecks();
     virtual void sPrepareCheckRun();
