@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2015 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -20,12 +20,15 @@
 #include <QSqlError>
 #include <QValidator>
 #include <QVariant>
+#include <metasql.h>
+#include <mqlutil.h>
 
 #include "errorReporter.h"
 #include "guiErrorCheck.h"
 #include "purchaseOrderList.h"
 #include "purchaseOrderItem.h"
 #include "storedProcErrorLookup.h"
+#include "taxBreakdown.h"
 #include "voucherItem.h"
 #include "voucherMiscDistrib.h"
 
@@ -47,10 +50,15 @@ voucher::voucher(QWidget* parent, const char* name, Qt::WindowFlags fl)
   connect(_edit,                     SIGNAL(clicked()),                                 this,          SLOT(sEditMiscDistribution()));
   connect(_delete,                   SIGNAL(clicked()),                                 this,          SLOT(sDeleteMiscDistribution()));
   connect(_invoiceDate,              SIGNAL(newDate(const QDate&)),                     this,          SLOT(sPopulateDistDate()));
+  connect(_distributionDate,         SIGNAL(newDate(const QDate&)),                     this,          SLOT(sNewDistDate()));
+  connect(_taxLit,                   SIGNAL(leftClickedURL(const QString&)),            this,          SLOT(sTaxDetail()));
   connect(_terms,                    SIGNAL(newID(int)),                                this,          SLOT(sPopulateDueDate()));
   connect(_poitem,                   SIGNAL(populateMenu(QMenu*,QTreeWidgetItem*,int)), this,          SLOT(sPopulateMenu(QMenu*)));
   connect(_amountToDistribute,       SIGNAL(idChanged(int)),                            this,          SLOT(sFillList()));
   connect(_amountDistributed,        SIGNAL(valueChanged()),                            this,          SLOT(sPopulateBalanceDue()));
+  connect(_freight,                  SIGNAL(valueChanged()),                            this,          SLOT(sPopulateDistributed()));
+  connect(_freightDistr,             SIGNAL(toggled(bool)),                             this,          SLOT(sFreightDistribution(bool)));
+  connect(_distributeFreight,        SIGNAL(clicked()),                                 this,          SLOT(sDistributeFreight()));
 
   _terms->setType(XComboBox::APTerms);
   _poNumber->setAllowedStatuses(OrderLineEdit::Open);
@@ -82,9 +90,22 @@ voucher::voucher(QWidget* parent, const char* name, Qt::WindowFlags fl)
 
   _miscDistrib->addColumn(tr("Account"),    -1,           Qt::AlignLeft,   true,  "account"  );
   _miscDistrib->addColumn(tr("Amount"),     _moneyColumn, Qt::AlignRight,  true,  "vodist_amount" );
+  _miscDistrib->addColumn(tr("Notes"),      -1,           Qt::AlignLeft,   true,  "vodist_notes" );
 
   _charass->setType("VCH");
+  _comments->setType(Comments::Voucher);
 
+  _freightCostElement->populate("SELECT costelem_id, costelem_type "
+                                "FROM costelem "
+                                "WHERE ( (costelem_active)"
+                                " AND (NOT costelem_sys)"
+                                " AND (costelem_po) ) "
+                                "ORDER BY costelem_type;");
+
+  if (_metrics->value("DefaultFreightCostElement") > 0)
+    _freightCostElement->setId(_metrics->value("DefaultFreightCostElement").toInt());
+
+  _frghtdistr = 0;
   _vendid = -1;
 
   setWindowModified(false);
@@ -127,6 +148,7 @@ enum SetResponse voucher::set(const ParameterList &pParams)
         _voheadid = insq.value("vohead_id").toInt();
         _documents->setId(_voheadid);
         _charass->setId(_voheadid);
+        _comments->setId(_voheadid);
       }
       else if (ErrorReporter::error(QtCriticalMsg, this, tr("Inserting Voucher"),
                                     insq, __FILE__, __LINE__))
@@ -156,7 +178,6 @@ enum SetResponse voucher::set(const ParameterList &pParams)
       _dueDate->setEnabled(false);
       _invoiceNum->setEnabled(false);
       _reference->setEnabled(false);
-      _distributions->setEnabled(false);
       _miscDistrib->setEnabled(false);
       _new->setEnabled(false);
       _terms->setEnabled(false);
@@ -167,9 +188,11 @@ enum SetResponse voucher::set(const ParameterList &pParams)
 //      _documents->setReadOnly(true);
       _charass->setReadOnly(true);
       _close->setText(tr("&Close"));
+      _freightGroup->setEnabled(false);
+      _comments->setReadOnly(true);
+
       _save->hide();
 
-      disconnect(_poitem, SIGNAL(valid(bool)), _distributions, SLOT(setEnabled(bool)));
       disconnect(_poitem, SIGNAL(valid(bool)), _distributeline, SLOT(setEnabled(bool)));
       disconnect(_poitem, SIGNAL(valid(bool)), _clear, SLOT(setEnabled(bool)));
       disconnect(_poNumber, SIGNAL(valid(bool)), _distributeall, SLOT(setEnabled(bool)));
@@ -186,6 +209,7 @@ enum SetResponse voucher::set(const ParameterList &pParams)
     _voheadid = param.toInt();
     _documents->setId(_voheadid);
     _charass->setId(_voheadid);
+    _comments->setId(_voheadid);
     populate();
     enableWindowModifiedSetting();
   }
@@ -223,7 +247,13 @@ bool voucher::sSave()
                           _invoiceNum,
                           tr("<p>You must enter a Vendor Invoice Number "
                              "before you may save this Voucher."))
-   ;
+         << GuiErrorCheck(_freight->localValue() > 0 && !_freightDistr->isChecked() && !_freightExpcat->isValid(), _freightExpcat,
+                           tr("<p>You must select an Expense Category to post the  "
+                              "freight value to."))
+         << GuiErrorCheck(_freight->localValue() > 0 && _freightDistr->isChecked() && _frghtdistr == 0, _freight,
+                           tr("<p>You must first distribute freight to item cost elements."))
+  ;
+
   if (GuiErrorCheck::reportErrors(this, tr("Cannot Save Voucher"), errors))
     return false;
 
@@ -267,6 +297,9 @@ bool voucher::sSave()
              "    vohead_invcnumber=:vohead_invcnumber,"
              "    vohead_reference=:vohead_reference,"
              "    vohead_amount=:vohead_amount,"
+             "    vohead_freight=:vohead_freight,"
+             "    vohead_freight_expcat_id=:vohead_freight_expcat,"
+             "    vohead_freight_distributed=:frghtDistr,"
              "    vohead_1099=:vohead_1099, "
              "    vohead_curr_id=:vohead_curr_id, "
              "    vohead_notes=:vohead_notes "
@@ -284,6 +317,10 @@ bool voucher::sSave()
   updq.bindValue(":vohead_invcnumber", _invoiceNum->text().trimmed());
   updq.bindValue(":vohead_reference", _reference->text().trimmed());
   updq.bindValue(":vohead_amount", _amountToDistribute->localValue());
+  updq.bindValue(":vohead_freight", _freight->localValue());
+  updq.bindValue(":frghtDistr", _frghtdistr);
+  if (_freightExpcat->isValid())
+    updq.bindValue(":vohead_freight_expcat", _freightExpcat->id());
   updq.bindValue(":vohead_1099", QVariant(_flagFor1099->isChecked()));
   updq.bindValue(":vohead_curr_id", _amountToDistribute->id());
   updq.bindValue(":vohead_notes", _notes->toPlainText());
@@ -318,6 +355,9 @@ bool voucher::sSave()
   _miscDistrib->clear();
   _notes->setText("");
   _charass->setId(-1);
+  _comments->setId(-1);
+  _freight->clear();
+  _freightExpcat->setId(-1);
 
   setWindowModified(false);
 
@@ -383,7 +423,10 @@ void voucher::sDistributions()
   foreach (XTreeWidgetItem *item, _poitem->selectedItems())
   {
     ParameterList params;
-    params.append("mode", "new");
+    if (_mode == cView)
+      params.append("mode", "view");
+    else
+      params.append("mode", "new");
     params.append("vohead_id", _voheadid);
     params.append("poitem_id",  item->id());
     params.append("curr_id", _amountToDistribute->id());
@@ -406,11 +449,12 @@ void voucher::sDistributions()
 void voucher::sDistributeLine()
 {
   saveDetail();
+  XSqlQuery distq;
+  distq.prepare("SELECT distributeVoucherLine(:vohead_id,:poitem_id,:curr_id) "
+                "AS result;");
+
   foreach (XTreeWidgetItem *item, _poitem->selectedItems())
   {
-    XSqlQuery distq;
-    distq.prepare("SELECT distributeVoucherLine(:vohead_id,:poitem_id,:curr_id) "
-              "AS result;");
     distq.bindValue(":vohead_id", _voheadid);
     distq.bindValue(":poitem_id",  item->id());
     distq.bindValue(":curr_id", _amountToDistribute->id());
@@ -418,6 +462,25 @@ void voucher::sDistributeLine()
     if (distq.first())
     {
       int result = distq.value("result").toInt();
+      if (result == -5 &&
+          QMessageBox::question(this, tr("Error Distributing"),
+            tr("Item has multiple cost elements. Do you wish to distribute "
+               "to the Material cost element only?"),
+                                  QMessageBox::Yes | QMessageBox::No, QMessageBox::No) == QMessageBox::No)
+      {
+        return;
+      } 
+      else
+      {
+         distq.prepare("SELECT distributeVoucherLine(:vohead_id,:poitem_id,:curr_id, true) "
+                       "AS result;");
+         distq.bindValue(":vohead_id", _voheadid);
+         distq.bindValue(":poitem_id", item->id());
+         distq.bindValue(":curr_id", _amountToDistribute->id());
+         distq.exec();
+         if (distq.first())
+            result = distq.value("result").toInt();
+      }
       if (result < 0)
         ErrorReporter::error(QtCriticalMsg, this, tr("Error Distributing"),
                     storedProcErrorLookup("distributeVoucherLine", result),
@@ -489,6 +552,26 @@ void voucher::sDistributeAll()
     if (distq.first())
     {
       int result = distq.value("result").toInt();
+      if (result == -5 &&
+          QMessageBox::question(this, tr("Error Distributing"),
+            tr("Item has multiple cost elements. Do you wish to distribute "
+               "to the Material cost element only?"),
+                                  QMessageBox::Yes | QMessageBox::No, QMessageBox::No) == QMessageBox::No)
+      {
+        return;
+      } 
+      else
+      {
+         distq.prepare("SELECT distributeVoucherLine(:vohead_id,:poitem_id,:curr_id, true) "
+                       "AS result;");
+         distq.bindValue(":vohead_id", _voheadid);
+         distq.bindValue(":poitem_id", item->id());
+         distq.bindValue(":curr_id", _amountToDistribute->id());
+         distq.exec();
+         if (distq.first())
+            result = distq.value("result").toInt();
+
+      }
       if (result < 0)
         ErrorReporter::error(QtCriticalMsg, this, tr("Error Distributing"),
                              storedProcErrorLookup("distributeVoucherLine", result),
@@ -520,6 +603,7 @@ void voucher::sNewMiscDistribution()
   newdlg.set(params);
   if (newdlg.exec() != XDialog::Rejected)
   {
+    sUpdateVoucherTax();
     sFillMiscList();
     sPopulateDistributed();
   }
@@ -541,6 +625,8 @@ void voucher::sEditMiscDistribution()
   newdlg.set(params);
   if (newdlg.exec() != XDialog::Rejected)
   {
+    if (_miscDistrib->altId() != 4) // Don't auto update taxes to allow manual override
+      sUpdateVoucherTax();
     sFillMiscList();
     sPopulateDistributed();
   }
@@ -566,95 +652,36 @@ void voucher::sDeleteMiscDistribution()
   if (ErrorReporter::error(QtCriticalMsg, this, tr("Deleting Distributions"),
                            delq, __FILE__, __LINE__))
     return;
-
-  sFillMiscList();
-  sPopulateDistributed();
+  else
+  {
+    sUpdateVoucherTax();
+    sFillMiscList();
+    sPopulateDistributed();
+  }
 }
 
 void voucher::sFillList()
 {
   if (_poNumber->isValid())
   {
-    XSqlQuery getq;
-    getq.prepare( "SELECT poitem_id, poitem_linenumber,"
-               "       CASE WHEN(poitem_status='C') THEN :closed"
-               "            WHEN(poitem_status='U') THEN :unposted"
-               "            WHEN(poitem_status='O' AND ((poitem_qty_received-poitem_qty_returned) > 0) AND (poitem_qty_ordered>(poitem_qty_received-poitem_qty_returned))) THEN :partial"
-               "            WHEN(poitem_status='O' AND ((poitem_qty_received-poitem_qty_returned) > 0) AND (poitem_qty_ordered<=(poitem_qty_received-poitem_qty_returned))) THEN :received"
-               "            WHEN(poitem_status='O') THEN :open"
-               "            ELSE poitem_status"
-               "       END AS poitemstatus,"
-               "       COALESCE(item_number, poitem_vend_item_number) AS itemnumber,"
-               "       COALESCE(uom_name, poitem_vend_uom) AS uom,"
-               "       poitem_vend_item_number, poitem_vend_uom,"
-               "       poitem_qty_ordered,"
-               "       ( SELECT COALESCE(SUM(recv_qty), 0)"
-               "         FROM recv"
-               "         WHERE ( (recv_posted)"
-               "           AND (recv_invoiced)"
-               "           AND (recv_order_type='PO')"
-               "           AND (recv_orderitem_id=poitem_id) ) ) AS qtyinvoiced,"
-               "       ( SELECT COALESCE(SUM(recv_qty), 0)"
-               "         FROM recv"
-               "         WHERE ( (recv_posted)"
-               "           AND (NOT recv_invoiced)"
-               "           AND (recv_vohead_id IS NULL)"
-               "           AND (recv_order_type='PO')"
-               "           AND (recv_orderitem_id=poitem_id) ) ) AS qtyreceived,"
-               "       ( SELECT COALESCE(SUM(poreject_qty), 0)"
-               "         FROM poreject"
-               "         WHERE ( (poreject_posted)"
-               "           AND (NOT poreject_invoiced)"
-               "           AND (poreject_vohead_id IS NULL)"
-               "           AND (poreject_poitem_id=poitem_id) ) ) AS qtyrejected,"
-               "       ( SELECT COALESCE(SUM(vodist_qty), 0)"
-               "         FROM vodist"
-               "         WHERE vodist_poitem_id=poitem_id"
-               "           AND vodist_vohead_id=:vohead_id ) AS invoiceqty, "
-               "       ( SELECT COALESCE(SUM(vodist_amount), 0)"
-               "         FROM vodist"
-               "         WHERE vodist_poitem_id=poitem_id"
-               "           AND vodist_vohead_id=:vohead_id ) "
-               "     + ( SELECT COALESCE(SUM(COALESCE(voitem_freight,0)), 0)"
-               "         FROM voitem"
-               "         WHERE voitem_poitem_id=poitem_id"
-               "           AND   voitem_vohead_id=:vohead_id ) AS invoiceamount,"
-               "       poitem_unitprice,"
-               "       (poitem_unitprice * poitem_qty_ordered) AS extprice,"
-               "       poitem_freight,"
-               "       'qty' AS poitem_qty_ordered_xtnumericrole,"
-               "       'qty' AS qtyinvoiced_xtnumericrole,"
-               "       'qty' AS qtyreceived_xtnumericrole,"
-               "       'qty' AS qtyrejected_xtnumericrole,"
-               "       'qty' AS invoiceqty_xtnumericrole,"
-               "       'curr' AS invoiceamount_xtnumericrole,"
-               "       'purchprice' AS poitem_unitprice_xtnumericrole,"
-               "       'curr' AS extprice_xtnumericrole,"
-               "       'curr' AS poitem_freight_xtnumericrole "
-               "FROM poitem JOIN pohead ON (pohead_id=poitem_pohead_id)"
-               "            LEFT OUTER JOIN ( itemsite JOIN item ON (itemsite_item_id=item_id)"
-               "                                       JOIN uom ON (item_inv_uom_id=uom_id) )"
-               "                                ON (poitem_itemsite_id=itemsite_id) "
-               "WHERE (poitem_pohead_id=:pohead_id) "
-               "GROUP BY poitem_id, poitem_linenumber, poitem_status,"
-               "         item_number, uom_name,"
-               "         poitem_vend_item_number, poitem_vend_uom,"
-               "         poitem_unitprice, poitem_freight,"
-               "         poitem_qty_ordered, poitem_qty_received, poitem_qty_returned,"
-               "         itemsite_id, pohead_curr_id "
-               "ORDER BY poitem_linenumber;" );
-    getq.bindValue(":vohead_id", _voheadid);
-    getq.bindValue(":pohead_id", _poNumber->id());
-    getq.bindValue(":closed", tr("Closed"));
-    getq.bindValue(":unposted", tr("Unposted"));
-    getq.bindValue(":partial", tr("Partial"));
-    getq.bindValue(":received", tr("Received"));
-    getq.bindValue(":open", tr("Open"));
-    getq.exec();
-    _poitem->populate(getq);
-    if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting P/O Information"),
-                             getq, __FILE__, __LINE__))
-      return;
+    MetaSQLQuery mql = mqlLoad("voucher", "poitems");
+    ParameterList params;
+
+    params.append("vohead_id", _voheadid);
+    params.append("pohead_id", _poNumber->id());
+    params.append("closed",    tr("Closed"));
+    params.append("unposted",  tr("Unposted"));
+    params.append("partial",   tr("Partial"));
+    params.append("received",  tr("Received"));
+    params.append("open",      tr("Open"));
+
+    XSqlQuery poitemFillList = mql.toQuery(params);
+    _poitem->populate(poitemFillList, true);
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving PO Items"),
+                                poitemFillList, __FILE__, __LINE__))
+        return;
+
+    _freight->setId(_amountToDistribute->id());
   }
   else
     _poitem->clear();
@@ -664,35 +691,15 @@ void voucher::sFillMiscList()
 {
   if (_poNumber->isValid())
   {
-    XSqlQuery getq;
-    getq.prepare( "SELECT vodist_id, (formatGLAccount(accnt_id) || ' - ' || accnt_descrip) AS account,"
-               "       vodist_amount, 'curr' AS vodist_amount_xtnumericrole "
-               "FROM vodist, accnt "
-               "WHERE ( (vodist_poitem_id=-1)"
-               " AND (vodist_accnt_id=accnt_id)"
-               " AND (vodist_vohead_id=:vohead_id) ) "
-               "UNION ALL "
-               "SELECT vodist_id, (expcat_code || ' - ' || expcat_descrip) AS account,"
-               "       vodist_amount, 'curr' AS vodist_amount_xtnumericrole "
-               "  FROM vodist, expcat "
-               " WHERE ( (vodist_poitem_id=-1)"
-               "   AND   (vodist_expcat_id=expcat_id)"
-               "   AND   (vodist_vohead_id=:vohead_id) ) "
-               "UNION ALL "
-               "SELECT vodist_id, (tax_code || ' - ' || tax_descrip) AS account,"
-               "       vodist_amount, 'curr' AS vodist_amount_xtnumericrole "
-               "  FROM vodist, tax "
-               " WHERE ( (vodist_poitem_id=-1)"
-               "   AND   (vodist_tax_id=tax_id)"
-               "   AND   (vodist_vohead_id=:vohead_id) ) "
-               "ORDER BY account;" );
-    getq.bindValue(":vohead_id", _voheadid);
-    getq.exec();
-    _miscDistrib->populate(getq);
-    if (ErrorReporter::error(QtCriticalMsg, this,
-                             tr("Getting Misc. Distributions"),
-                             getq, __FILE__, __LINE__))
-      return;
+    MetaSQLQuery mql = mqlLoad("voucher", "miscDistr");
+    ParameterList params;
+
+    params.append("vohead_id", _voheadid);
+    XSqlQuery miscFillList = mql.toQuery(params);
+    _miscDistrib->populate(miscFillList, true);
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Misc Distributions"),
+                                miscFillList, __FILE__, __LINE__))
+        return;
   }
 }
 
@@ -700,7 +707,7 @@ void voucher::sPopulatePoInfo()
 {
   XSqlQuery po;
   po.prepare("SELECT pohead_terms_id, pohead_taxzone_id, vend_1099, "
-             "       pohead_curr_id, vend_id, vend_number, vend_name,"
+             "       pohead_curr_id, pohead_freight, vend_id, vend_number, vend_name,"
              "       addr_line1, addr_line2"
              "  FROM pohead"
              "  JOIN vendinfo ON (pohead_vend_id=vend_id)"
@@ -716,14 +723,20 @@ void voucher::sPopulatePoInfo()
     _terms->setId(po.value("pohead_terms_id").toInt());
     _taxzone->setId(po.value("pohead_taxzone_id").toInt());
     _amountToDistribute->setId(po.value("pohead_curr_id").toInt());
+    _freight->setLocalValue(po.value("pohead_freight").toDouble());
+    _freight->setId(po.value("pohead_curr_id").toInt());
     _vendid = po.value("vend_id").toInt();
     _vendor->setText(po.value("vend_number").toString());
     _vendName->setText(po.value("vend_name").toString());
     // TODO: replace with a compact AddressCluster when such exists
     _vendAddress1->setText(po.value("addr_line1").toString());
     _vendAddress2->setText(po.value("addr_line2").toString());
+    if (_mode != cView)
+      _new->setEnabled(true);
+  } else {
+    _freight->setLocalValue(0.00);
   }
-  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting P/O Information"),
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting P/O Information"),
                                 po, __FILE__, __LINE__))
     return;
 }
@@ -732,6 +745,7 @@ void voucher::sPopulateDistributed()
 {
   if (_poNumber->isValid())
   {
+    sCalculateTax();
     XSqlQuery getq;
     getq.prepare( "SELECT (COALESCE(dist,0) + COALESCE(freight,0) + COALESCE(tax,0)) AS distrib"
                "  FROM (SELECT SUM(COALESCE(voitem_freight,0)) AS freight"
@@ -751,7 +765,7 @@ void voucher::sPopulateDistributed()
     getq.exec();
     if (getq.first())
     {
-      _amountDistributed->setLocalValue(getq.value("distrib").toDouble());
+      _amountDistributed->setLocalValue(getq.value("distrib").toDouble() + _freight->localValue() - _frghtdistr);
     }
     else if (ErrorReporter::error(QtCriticalMsg, this,
                                   tr("Getting Distributions"),
@@ -762,11 +776,19 @@ void voucher::sPopulateDistributed()
 
 void voucher::sPopulateBalanceDue()
 {
-  _balance->setLocalValue(_amountToDistribute->localValue() - _amountDistributed->localValue());
+  _balance->setLocalValue(_amountToDistribute->localValue() -
+                          _amountDistributed->localValue());
+
   if (_balance->isZero())
     _balance->setPaletteForegroundColor(QColor("black"));
   else
     _balance->setPaletteForegroundColor(namedColor("error"));
+
+  if (_freight->localValue() <= 0)
+    _freightExpcat->setId(-1);
+
+  _freightGroup->setEnabled(_frghtdistr == 0 && _mode != cView);
+
 }
 
 void voucher::populateNumber()
@@ -788,7 +810,8 @@ void voucher::populate()
   XSqlQuery vohead;
   vohead.prepare( "SELECT vohead_number, vohead_pohead_id, vohead_taxzone_id, vohead_terms_id,"
                   "       vohead_distdate, vohead_docdate, vohead_duedate,"
-                  "       vohead_invcnumber, vohead_reference,"
+                  "       vohead_invcnumber, vohead_reference, vohead_freight, vohead_freight_expcat_id,"
+                  "       vohead_freight_distributed,"
                   "       vohead_1099, vohead_amount, vohead_curr_id, vohead_notes "
                   "FROM vohead "
                   "WHERE (vohead_id=:vohead_id);" );
@@ -803,6 +826,16 @@ void voucher::populate()
     _amountToDistribute->set(vohead.value("vohead_amount").toDouble(),
                              vohead.value("vohead_curr_id").toInt(),
                              vohead.value("vohead_docdate").toDate(), false);
+    _freight->set(vohead.value("vohead_freight").toDouble(),
+                             vohead.value("vohead_curr_id").toInt(),
+                             vohead.value("vohead_docdate").toDate(), false);
+
+    _frghtdistr = vohead.value("vohead_freight_distributed").toDouble();
+    if (_frghtdistr > 0)
+      sIsDistributed();
+
+    _freightDistr->setChecked(_frghtdistr > 0);
+    _freightExpcat->setId(vohead.value("vohead_freight_expcat_id").toInt());
 
     _distributionDate->setDate(vohead.value("vohead_distdate").toDate(), true);
     _invoiceDate->setDate(vohead.value("vohead_docdate").toDate());
@@ -816,6 +849,7 @@ void voucher::populate()
     sFillMiscList();
     sPopulateDistributed();
     _charass->setId(_voheadid);
+    _comments->setId(_voheadid);
   }
   else if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting Voucher"),
                                 vohead, __FILE__, __LINE__))
@@ -878,7 +912,7 @@ void voucher::sPopulateDistDate()
 
 void voucher::sPopulateDueDate()
 {
-  if ( (_invoiceDate->isValid()) && (!_dueDate->isValid()) )
+  if ( (_invoiceDate->isValid()) )
   {
     XSqlQuery dateq;
     dateq.prepare("SELECT determineDueDate(:terms_id, :invoiceDate) AS duedate;");
@@ -923,7 +957,7 @@ void voucher::keyPressEvent( QKeyEvent * e )
     e->ignore();
 }
 
-void voucher::saveDetail()
+bool voucher::saveDetail()
 {
   if (_mode != cView)
   {
@@ -939,6 +973,9 @@ void voucher::saveDetail()
                "    vohead_invcnumber=:vohead_invcnumber,"
                "    vohead_reference=:vohead_reference,"
                "    vohead_amount=:vohead_amount,"
+               "    vohead_freight=:vohead_freight,"
+               "    vohead_freight_expcat_id=:vohead_freightexpcat,"
+               "    vohead_freight_distributed=:frghtDistr,"
                "    vohead_1099=:vohead_1099, "
                "    vohead_curr_id=:vohead_curr_id, "
                "    vohead_notes=:vohead_notes "
@@ -956,19 +993,92 @@ void voucher::saveDetail()
     updq.bindValue(":vohead_invcnumber", _invoiceNum->text().trimmed());
     updq.bindValue(":vohead_reference", _reference->text().trimmed());
     updq.bindValue(":vohead_amount", _amountToDistribute->localValue());
+    updq.bindValue(":vohead_freight", _freight->localValue());
+    updq.bindValue(":frghtDistr", _frghtdistr);
+    if (_freightExpcat->isValid())
+      updq.bindValue(":vohead_freightexpcat", _freightExpcat->id());
     updq.bindValue(":vohead_1099", QVariant(_flagFor1099->isChecked()));
     updq.bindValue(":vohead_curr_id", _amountToDistribute->id());
     updq.bindValue(":vohead_notes", _notes->toPlainText());
     updq.exec();
     if (ErrorReporter::error(QtCriticalMsg, this, tr("Updating Voucher"),
                              updq, __FILE__, __LINE__))
-      return;
+      return false;
+  }
+  return true;
+}
+
+void voucher::sCalculateTax()
+{
+  if (_vendid == -1)
+    return;
+
+  if (!saveDetail())
+    return;
+
+  XSqlQuery taxq;
+  taxq.prepare( "SELECT ABS(SUM(tax)) AS tax "
+                "FROM ("
+                "SELECT ROUND(SUM(taxdetail_tax),2) AS tax "
+                "FROM tax "
+                " JOIN calculateTaxDetailSummary('VO', :vohead_id, 'T') ON (taxdetail_tax_id=tax_id)"
+                "GROUP BY tax_id) AS data;" );
+  taxq.bindValue(":vohead_id", _voheadid);
+  taxq.exec();
+  if (taxq.first())
+    _tax->setLocalValue(taxq.value("tax").toDouble());
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Calculating Voucher Tax"),
+                                    taxq, __FILE__, __LINE__))
+        return;
+}
+
+void voucher::sTaxDetail()
+{
+  if (!saveDetail())
+    return;
+
+  ParameterList params;
+  params.append("order_id", _voheadid);
+  params.append("order_type", "VO");
+  // mode => view since there are no fields to hold modified tax data
+  if (_mode == cView)
+    params.append("mode", "view");
+
+  taxBreakdown newdlg(this, "", true);
+  newdlg.set(params);
+  newdlg.exec();
+}
+
+void voucher::sUpdateVoucherTax()
+{
+  if (!_taxzone->isValid() || !_distributionDate->isValid())
+    return;
+
+  XSqlQuery updTax;
+  updTax.prepare("SELECT updatemiscvouchertax(:voheadid,:taxzone,:distdate,:curr) as ret;");
+  updTax.bindValue(":voheadid", _voheadid);
+  updTax.bindValue(":taxzone",  _taxzone->id());
+  updTax.bindValue(":distdate", _distributionDate->date());
+  updTax.bindValue(":curr", _amountToDistribute->id());
+  updTax.exec();
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Adding Tax to Voucher"),
+                         updTax, __FILE__, __LINE__))
+    return;
+}
+
+void voucher::sNewDistDate()
+{
+  if (_miscDistrib->topLevelItemCount())
+  {
+     sUpdateVoucherTax();
+     sFillMiscList();
   }
 }
 
 void voucher::enableWindowModifiedSetting()
 {
   connect(_amountToDistribute,     SIGNAL(valueChanged()), this, SLOT(sDataChanged()));
+  connect(_freight,                SIGNAL(valueChanged()), this, SLOT(sDataChanged()));
   connect(_distributionDate,SIGNAL(newDate(const QDate&)), this, SLOT(sDataChanged()));
   connect(_dueDate,         SIGNAL(newDate(const QDate&)), this, SLOT(sDataChanged()));
   connect(_flagFor1099,             SIGNAL(toggled(bool)), this, SLOT(sDataChanged()));
@@ -986,3 +1096,79 @@ void voucher::sDataChanged()
   setWindowModified(true);
 }
 
+void voucher::sFreightDistribution(bool distFreight)
+{
+  if (distFreight)
+    _freightStack->setCurrentIndex(1);
+  else
+    _freightStack->setCurrentIndex(0);
+}
+
+void voucher::sIsDistributed()
+{
+  _freight->setEnabled(false);
+  _freightDistr->setEnabled(false);
+  _freightDistr->setText(tr("Freight Distributed"));
+  _freightStack->setVisible(false);
+//  _freightCostElement->setVisible(false);
+//  _freightDistrMethod->setVisible(false);
+//  _distributeFreight->setVisible(false);
+}
+
+void voucher::sDistributeFreight()
+{
+  XSqlQuery distr;
+
+  if (!saveDetail())
+    return;
+
+  // Check for partial PO distributions before proceeding
+  distr.prepare("SELECT COALESCE(SUM(recv_qty),0) AS qtyreceived"
+                " FROM poitem "
+                " JOIN recv ON poitem_id=recv_orderitem_id"
+                " WHERE poitem_pohead_id=:pohead_id"
+                "  AND recv_posted"
+                "  AND NOT recv_invoiced"
+                "  AND recv_vohead_id IS NULL"
+                "  AND recv_order_type='PO';");
+  distr.bindValue(":pohead_id", _poNumber->id());
+  distr.exec();
+  if (distr.first())
+  {
+    if (distr.value("qtyreceived").toInt() > 0)
+    {
+      if (QMessageBox::question(this, tr("Incomplete Distribution"),
+                              tr("The Purchase Order has not been "
+                                 "completely distributed. "
+                                 "Do you wish to continue?"),
+                                  QMessageBox::Yes | QMessageBox::No, QMessageBox::No) == QMessageBox::No)
+        return;
+    }
+  }
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Distributing Voucher Freight"),
+                              distr, __FILE__, __LINE__))
+    return;
+
+  distr.prepare("SELECT distributeVoucherFreight(:vohead_id, :costelem, :dist_type, :freight) AS result;");
+  distr.bindValue(":vohead_id", _voheadid);
+  distr.bindValue(":costelem", _freightCostElement->id());
+  if (_freightDistQty->isChecked())
+    distr.bindValue(":dist_type", "Q");
+  else if (_freightDistVal->isChecked())
+    distr.bindValue(":dist_type", "V");
+  else if (_freightDistWgt->isChecked())
+    distr.bindValue(":dist_type", "W");
+  distr.bindValue(":freight", _freight->localValue());
+  distr.exec();
+  if (distr.first())  
+  {
+    _frghtdistr = _freight->localValue();
+    sIsDistributed();
+    sFillList();
+    sDataChanged();
+  }
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Distributing Voucher Freight"),
+                              distr, __FILE__, __LINE__))
+    return;
+}
+
