@@ -36,6 +36,7 @@ public slots:
     virtual void sDataChanged();
     virtual void sDistributions();
     virtual void sDistributeLine();
+    virtual void sDistributeFreight();
     virtual void sClear();
     virtual void sDistributeAll();
     virtual void sNewMiscDistribution();
@@ -43,9 +44,14 @@ public slots:
     virtual void sDeleteMiscDistribution();
     virtual void sFillList();
     virtual void sFillMiscList();
+    virtual void sFreightDistribution(bool);
     virtual void sPopulatePoInfo();
     virtual void sPopulateDistributed();
     virtual void sPopulateBalanceDue();
+    virtual void sCalculateTax();
+    virtual void sUpdateVoucherTax();
+    virtual void sNewDistDate();
+    virtual void sTaxDetail();
     virtual void populateNumber();
     virtual void populate();
     virtual void clear();
@@ -53,22 +59,21 @@ public slots:
     virtual void sPopulateDistDate();
     virtual void sPopulateDueDate();
     virtual void sView();
-    virtual void saveDetail();
+    virtual bool saveDetail();
+    virtual void sIsDistributed();
 
 protected:
     virtual void keyPressEvent( QKeyEvent * e );
 
 protected slots:
     virtual void languageChange();
-
     virtual void sPopulateMenu( QMenu * pMenu );
 
-
 private:
     int _vendid;
     int _voheadid;
     int _mode;
-
+    double _frghtdistr;
 };
 
 #endif // VOUCHER_H
