@@ -30,7 +30,9 @@ woMaterialItem::woMaterialItem(QWidget* parent, const char* name, bool modal, Qt
   connect(_qtyFxd, SIGNAL(textChanged(const QString&)), this, SLOT(sUpdateQtyRequired()));
   connect(_qtyPer, SIGNAL(textChanged(const QString&)), this, SLOT(sUpdateQtyRequired()));
   connect(_scrap, SIGNAL(textChanged(const QString&)), this, SLOT(sUpdateQtyRequired()));
-  connect(_item, SIGNAL(valid(bool)), _save, SLOT(setEnabled(bool)));
+  connect(_uom, SIGNAL(newID(int)), this, SLOT(sUOMChanged()));
+  connect(_wo, SIGNAL(valid(bool)), this, SLOT(sEnableSave()));
+  connect(_item, SIGNAL(valid(bool)), this, SLOT(sEnableSave()));
   connect(_item, SIGNAL(newId(int)), this, SLOT(sItemIdChanged()));
   connect(_close, SIGNAL(clicked()), this, SLOT(reject()));
   connect(_save, SIGNAL(clicked()), this, SLOT(sSave()));
@@ -190,6 +192,11 @@ enum SetResponse woMaterialItem::set(const ParameterList &pParams)
   return NoError;
 }
 
+void woMaterialItem::sEnableSave()
+{
+  _save->setEnabled(_wo->isValid() && _item->isValid());
+}
+
 void woMaterialItem::sSave()
 {
   XSqlQuery woSave;
@@ -212,7 +219,8 @@ void woMaterialItem::sSave()
 
   if (_mode == cNew)
   {
-    woSave.prepare( "SELECT component.itemsite_id AS itemsiteid "
+    woSave.prepare( "SELECT component.itemsite_id AS itemsiteid, "
+                    "component.itemsite_costmethod AS costmethod "
                "FROM wo, itemsite AS parent, itemsite AS component "
                "WHERE ( (parent.itemsite_warehous_id=component.itemsite_warehous_id)"
                " AND (parent.itemsite_id=wo_itemsite_id)"
@@ -221,7 +229,20 @@ void woMaterialItem::sSave()
     woSave.bindValue(":item_id", _item->id());
     woSave.bindValue(":wo_id", _wo->id());
     woSave.exec();
-    if (!woSave.first())
+    if (woSave.first())
+    {
+      if (woSave.value("costmethod")=="J")
+      {
+        QMessageBox::warning( this, tr("Cannot Create W/O Material Requirement"),
+                            tr( "A W/O Material Requirement cannot be created for the selected\n"
+                                "Work Order/Item as the selected Item is Job Costed,\n"
+                                "and Work Order Component Items may not be Job Costed." ));
+        _item->setId(-1);
+        _item->setFocus();
+        return;
+      }
+    }
+    else
     {
       QMessageBox::warning( this, tr("Cannot Create W/O Material Requirement"),
                             tr( "A W/O Material Requirement cannot be created for the selected\n"
@@ -234,6 +255,35 @@ void woMaterialItem::sSave()
 
     int itemsiteid = woSave.value("itemsiteid").toInt();
 
+    woSave.prepare("WITH RECURSIVE _wo AS ( "
+                   " SELECT itemsite_item_id, wo_ordtype, wo_ordid "
+                   "   FROM wo "
+                   "   JOIN itemsite ON wo_itemsite_id=itemsite_id "
+                   "  WHERE wo_id=:wo_id "
+                   " UNION "
+                   " SELECT parent.itemsite_item_id, wo.wo_ordtype, wo.wo_ordid "
+                   "   FROM _wo "
+                   "   JOIN wo ON _wo.wo_ordtype='W' "
+                   "          AND _wo.wo_ordid=wo.wo_id "
+                   "   JOIN itemsite parent ON wo.wo_itemsite_id=parent.itemsite_id "
+                   ") "
+                   "SELECT BOOL_OR(itemsite_item_id=:item_id) AS recursive "
+                   "  FROM _wo;");
+    woSave.bindValue(":wo_id", _wo->id());
+    woSave.bindValue(":item_id", _item->id());
+    woSave.exec();
+    if (woSave.first() && woSave.value("recursive").toBool())
+    {
+      QMessageBox::warning(this, tr("Cannot Create W/O Material Requirement"),
+                           tr("Cannot add an Item as a Material to itself."));
+      _item->setId(-1);
+      _item->setFocus();
+      return;
+    }
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Checking Parents"),
+                                  woSave, __FILE__, __LINE__))
+      return;
+
     woSave.prepare("SELECT createWoMaterial(:wo_id, :itemsite_id, :issueMethod,"
                    "                        :uom_id, :qtyFxd, :qtyPer,"
                    "                        :scrap, :bomitem_id, :notes,"
@@ -365,61 +415,126 @@ void woMaterialItem::populate()
 
 void woMaterialItem::sItemIdChanged()
 {
-  // Get list of active, valid MaterialIssue UOMs
-  MetaSQLQuery muom = mqlLoad("uoms", "item");
+  sPopulateUOM();
 
-  ParameterList params;
-  params.append("uomtype", "MaterialIssue");
-  params.append("item_id", _item->id());
-
-  // Also have to factor UOMs previously used on WO Material now inactive
-  if (_womatlid != -1)
-  {
-    XSqlQuery wouom;
-    wouom.prepare("SELECT womatl_uom_id "
-                "  FROM womatl"
-                " WHERE(womatl_id=:womatl_id);");
-    wouom.bindValue(":womatl_id", _womatlid);
-    wouom.exec();
-    if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting Sales Order UOMs"),
-                         wouom, __FILE__, __LINE__))
-      return;
-    else if (wouom.first())
-        params.append("uom_id", wouom.value("womatl_uom_id"));
-  }
-  XSqlQuery uom = muom.toQuery(params);
-  if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting UOM"),
-                         uom, __FILE__, __LINE__))
-    return;
-  _uom->populate(uom);
-
-  uom.prepare("SELECT item_inv_uom_id, item_type "
+  XSqlQuery uom;
+  uom.prepare("SELECT item_inv_uom_id, item_type, item_picklist "
               "  FROM item"
               " WHERE(item_id=:item_id);");
   uom.bindValue(":item_id", _item->id());
   uom.exec();
   if(uom.first())
   {
-    _uom->setId(uom.value("item_inv_uom_id").toInt());
+    _invuomid = uom.value("item_inv_uom_id").toInt();
+    _uom->setId(_invuomid);
     if (uom.value("item_type").toString() != "T" && uom.value("item_type").toString() != "R")
-	{
-	  if (_qtyPer->text().length() == 0)
-	  {
-	    _qtyFxd->setDouble(0.0);
-		_qtyPer->setDouble(1.0);
-	  }
-	}
-	else
-	{
-	  if (_qtyPer->text().length() == 0)
-	  {
-	    _qtyFxd->setDouble(1.0);
-		_qtyPer->setDouble(0.0);
-	  }
-	}
-	
-	if (_scrap->text().length() == 0)
-	  _scrap->setDouble(0.0);
+    {
+      if (_qtyPer->text().length() == 0)
+      {
+        _qtyFxd->setDouble(0.0);
+        _qtyPer->setDouble(1.0);
+      }
+    }
+    else
+    {
+      if (_qtyPer->text().length() == 0)
+      {
+        _qtyFxd->setDouble(1.0);
+        _qtyPer->setDouble(0.0);
+      }
+    }
+    
+    if (_scrap->text().length() == 0)
+      _scrap->setDouble(0.0);
+    
+    _pickList->setChecked(uom.value("item_picklist").toBool());
+  }
+  else
+  {
+    _qtyFxd->clear();
+    _qtyPer->clear();
+    _qtyRequired->clear();
+    _scrap->clear();
+    _pickList->setChecked(false);
   }
 }
 
+void woMaterialItem::sPopulateUOM()
+{
+  // Get list of active, valid MaterialIssue UOMs
+  MetaSQLQuery muom = mqlLoad("uoms", "item");
+  
+  ParameterList params;
+  params.append("uomtype", "MaterialIssue");
+  params.append("item_id", _item->id());
+  
+  // Include Global UOMs
+  if (_privileges->check("MaintainUOMs"))
+  {
+    params.append("includeGlobal", true);
+    params.append("global", tr("-Global"));
+  }
+  
+  // Also have to factor UOMs previously used on WO Material now inactive
+  if (_womatlid != -1)
+  {
+    XSqlQuery wouom;
+    wouom.prepare("SELECT womatl_uom_id "
+                  "  FROM womatl"
+                  " WHERE(womatl_id=:womatl_id);");
+    wouom.bindValue(":womatl_id", _womatlid);
+    wouom.exec();
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting WoMatl UOMs"),
+                             wouom, __FILE__, __LINE__))
+      return;
+    else if (wouom.first())
+      params.append("uom_id", wouom.value("womatl_uom_id"));
+  }
+  
+  XSqlQuery uom = muom.toQuery(params);
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting UOM"),
+                           uom, __FILE__, __LINE__))
+    return;
+  
+  int saveuomid = _uom->id();
+  disconnect(_uom, SIGNAL(newID(int)), this, SLOT(sUOMChanged()));
+  _uom->populate(uom);
+  _uom->setId(saveuomid);
+  connect(_uom, SIGNAL(newID(int)), this, SLOT(sUOMChanged()));
+}
+
+void woMaterialItem::sUOMChanged()
+{
+  // Check for Global UOM Conversion that must be setup for Item
+  if (_uom->code() == "G")
+  {
+    if (QMessageBox::question(this, tr("Use Global UOM?"),
+                              tr("<p>This Global UOM Conversion is not setup for this Item."
+                                 "<p>Do you want to add this UOM conversion to this Item?"),
+                              QMessageBox::Yes | QMessageBox::Default,
+                              QMessageBox::No  | QMessageBox::Escape) == QMessageBox::Yes)
+    {
+      // create itemuomconv and itemuom
+      XSqlQuery adduom;
+      adduom.prepare("SELECT createItemUomConv(:item_id, :uom_id, :uom_type) AS result;");
+      adduom.bindValue(":item_id", _item->id());
+      adduom.bindValue(":uom_id", _uom->id());
+      adduom.bindValue(":uom_type", "MaterialIssue");
+      adduom.exec();
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Creating Item UOM Conv"),
+                               adduom, __FILE__, __LINE__))
+        return;
+
+      // repopulate uom combobox
+      int saveuomid = _uom->id();
+      disconnect(_uom, SIGNAL(newID(int)), this, SLOT(sUOMChanged()));
+      sPopulateUOM();
+      _uom->setId(saveuomid);
+      connect(_uom, SIGNAL(newID(int)), this, SLOT(sUOMChanged()));
+    }
+    else
+    {
+      _uom->setId(_invuomid);
+    }
+  }
+}
