@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -14,10 +14,11 @@
 
 #include <parameter.h>
 #include <guiclient.h>
+#include "scriptablePrivate.h"
 
 class XDialogPrivate;
 
-class XDialog : public QDialog
+class XDialog : public QDialog, protected ScriptablePrivate
 {
   Q_OBJECT
 
@@ -25,13 +26,13 @@ class XDialog : public QDialog
     XDialog(QWidget * parent = 0, Qt::WindowFlags flags = 0);
     XDialog(QWidget * parent, const char * name, bool modal = false, Qt::WindowFlags flags = 0);
     virtual ~XDialog();
-
     Q_INVOKABLE virtual ParameterList get() const;
 
   public slots:
     virtual enum SetResponse set(const ParameterList &);
     virtual void setRememberPos(bool);
     virtual void setRememberSize(bool);
+    int exec();
 
   protected:
     virtual void closeEvent(QCloseEvent * event);
@@ -44,9 +45,6 @@ class XDialog : public QDialog
   private:
     friend class XDialogPrivate;
     XDialogPrivate *_private;
-
-    ParameterList _lastSetParams;
-    void loadScriptEngine();
 };
 
 #endif // __XDIALOG_H__
