@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2013 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -23,78 +23,68 @@
 #include "xcheckbox.h"
 #include "xtsettings.h"
 #include "guiclient.h"
-#include "scriptablePrivate.h"
 #include "shortcuts.h"
 
 #define DEBUG false
-//
-// XWidgetPrivate
-//
-class XWidgetPrivate : public ScriptablePrivate
+
+// Obsolete now?
+class XWidgetPrivate
 {
   friend class XWidget;
 
   public:
     XWidgetPrivate(XWidget *parent);
     virtual ~XWidgetPrivate();
-
-    XWidget * _parent;
-    bool _shown;
 };
 
-XWidgetPrivate::XWidgetPrivate(XWidget *parent) : ScriptablePrivate(false, parent), _parent(parent)
+XWidgetPrivate::XWidgetPrivate(XWidget *parent)
 {
-  _shown = false;
+  Q_UNUSED(parent);
 }
 
 XWidgetPrivate::~XWidgetPrivate()
 {
 }
 
-XWidget::XWidget(QWidget * parent, Qt::WindowFlags flags)
+XWidget::XWidget(QWidget *parent, Qt::WindowFlags flags)
   : QWidget(parent,
             ((flags & (Qt::Dialog | Qt::Window)) && parent && parent->isModal()) ? (flags | Qt::Dialog)
-                  : flags )
+                  : flags ),
+    ScriptablePrivate(parent, this),
+    _private(0)
 {
   if(parent && parent->isModal() && (flags & (Qt::Dialog | Qt::Window)))
   {
     setWindowModality(Qt::ApplicationModal);
   }
-  if(!parent || !parent->isModal())
-    setParent(omfgThis);
-
-  _private = new XWidgetPrivate(this);
 }
 
 XWidget::XWidget(QWidget * parent, const char * name, Qt::WindowFlags flags)
   : QWidget(parent,
             ((flags & (Qt::Dialog | Qt::Window)) && parent && parent->isModal()) ? (flags | Qt::Dialog)
-                  : flags )
+                  : flags ),
+    ScriptablePrivate(parent, this),
+    _private(0)
 {
   if(parent && parent->isModal() && (flags & (Qt::Dialog | Qt::Window)))
   {
     setWindowModality(Qt::ApplicationModal);
-    //setWindowFlags(windowFlags() | Qt::Dialog);
   }
 
   if(name)
     setObjectName(name);
-  if(!parent || !parent->isModal())
-    setParent(omfgThis);
-
-  _private = new XWidgetPrivate(this);
 }
 
 XWidget::~XWidget()
 {
-  if(_private)
+  if (_private)
     delete _private;
 }
 
 void XWidget::closeEvent(QCloseEvent *event)
 {
   event->accept(); // we have no reason not to accept and let the script change it if needed
-  _private->callCloseEvent(event);
+  callCloseEvent(event);
 
   if(event->isAccepted())
   {
@@ -111,9 +101,9 @@ void XWidget::closeEvent(QCloseEvent *event)
 
 void XWidget::showEvent(QShowEvent *event)
 {
-  if(!_private->_shown)
+  if (! _shown)
   {
-    _private->_shown = true;
+    _shown = true;
     if (windowFlags() & (Qt::Window | Qt::Dialog))
     {
       QRect availableGeometry = QApplication::desktop()->availableGeometry();
@@ -123,6 +113,7 @@ void XWidget::showEvent(QShowEvent *event)
       QString objName = objectName();
       QPoint pos = xtsettingsValue(objName + "/geometry/pos").toPoint();
       QSize lsize = xtsettingsValue(objName + "/geometry/size").toSize();
+      QSize currsize = size();
 
       setAttribute(Qt::WA_DeleteOnClose);
       if(omfgThis->showTopLevel() || isModal())
@@ -133,6 +124,8 @@ void XWidget::showEvent(QShowEvent *event)
         QRect r(pos, size());
         if(!pos.isNull() && availableGeometry.contains(r) && xtsettingsValue(objName + "/geometry/rememberPos", true).toBool())
           move(pos);
+        else if(currsize!=size())
+          move(QPoint(1, 1));
       }
       else
       {
@@ -160,7 +153,7 @@ void XWidget::showEvent(QShowEvent *event)
       }
     }
 
-    _private->loadScriptEngine();
+    loadScriptEngine();
 
     QList<XCheckBox*> allxcb = findChildren<XCheckBox*>();
     for (int i = 0; i < allxcb.size(); ++i)
@@ -169,34 +162,24 @@ void XWidget::showEvent(QShowEvent *event)
     shortcuts::setStandardKeys(this);
   }
 
-  _private->callShowEvent(event);
+  callShowEvent(event);
   QWidget::showEvent(event);
 }
 
-enum SetResponse XWidget::set( const ParameterList & pParams )
+enum SetResponse XWidget::set(const ParameterList & pParams)
 {
   _lastSetParams = pParams;
-
-  _private->loadScriptEngine();
-
+  loadScriptEngine();
   QTimer::singleShot(0, this, SLOT(postSet()));
-
   return NoError;
 }
 
 enum SetResponse XWidget::postSet()
 {
-  return _private->callSet(_lastSetParams);
+  return callSet(_lastSetParams);
 }
 
 ParameterList XWidget::get() const
 {
   return _lastSetParams;
 }
-
-QScriptEngine *XWidget::engine()
-{
-  _private->loadScriptEngine();
-  return _private->_engine;
-}
- 
