@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -12,34 +12,39 @@
 
 #include <QSqlError>
 
+#include "qsqldatabaseproto.h"
+
 void setupXSqlQueryProto(QScriptEngine *engine)
 {
   QScriptValue proto = engine->newQObject(new XSqlQueryProto(engine));
   engine->setDefaultPrototype(qMetaTypeId<XSqlQuery*>(), proto);
   engine->setDefaultPrototype(qMetaTypeId<XSqlQuery>(),  proto);
 
-/*
-  QScriptValue constructor = engine->newFunction(constructXSqlQuery,
-                                                 proto);
+  QScriptValue constructor = engine->newFunction(constructXSqlQuery, proto);
   engine->globalObject().setProperty("XSqlQuery",  constructor);
-*/
 }
 
-/*
-QScriptValue constructXSqlQuery(QScriptContext * context,
-                                QScriptEngine  *engine)
+QScriptValue constructXSqlQuery(QScriptContext *context, QScriptEngine  *engine)
 {
   XSqlQuery *obj = 0;
-  if (context->argumentCount() == 1)
-    obj = new XSqlQuery(context->argument(1).toQObject());
+  if (context->argumentCount() >= 2)
+    obj = new XSqlQuery(context->argument(0).toString(),
+                        context->argument(1).toVariant().value<QSqlDatabase>());
+  else if (context->argumentCount() == 1 && context->argument(0).isString())
+    obj = new XSqlQuery(context->argument(0).toString());
+  else if (context->argumentCount() == 1)
+    obj = new XSqlQuery(context->argument(0).toVariant().value<QSqlDatabase>());
   else
     obj = new XSqlQuery();
+
   return engine->toScriptValue(obj);
 }
-*/
 
-XSqlQueryProto::XSqlQueryProto(QObject * parent)
-  : QObject(parent)
+XSqlQueryProto::XSqlQueryProto(QObject * parent) : QObject(parent)
+{
+}
+
+XSqlQueryProto::~XSqlQueryProto()
 {
 }
 
@@ -197,4 +202,3 @@ int XSqlQueryProto::findFirst(const QString & col, const QString & val)
     return item->findFirst(col, val);
   return -1;
 }
-
