@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2018 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -22,8 +22,9 @@
 #include <QPushButton>
 #include <QWidget>
 
-class QGridLayout;
 class AddressCluster;
+class QGridLayout;
+class QScriptEngine;
 
 class XTUPLEWIDGETS_EXPORT AddressList : public VirtualList
 {
@@ -78,7 +79,7 @@ class XTUPLEWIDGETS_EXPORT AddressSearch : public VirtualSearch
 class XTUPLEWIDGETS_EXPORT AddressCluster : public VirtualCluster
 {
     Q_OBJECT
-    Q_ENUMS   (Mode)
+
     Q_PROPERTY(bool     activeVisible 	      READ activeVisible            WRITE setActiveVisible)
     Q_PROPERTY(QString  fieldNameAddressChange READ fieldNameAddrChange     WRITE setFieldNameAddrChange)
     Q_PROPERTY(QString  fieldNameNumber        READ fieldNameNumber         WRITE setFieldNameNumber)
@@ -98,9 +99,13 @@ class XTUPLEWIDGETS_EXPORT AddressCluster : public VirtualCluster
 
     public:
 	enum SaveFlags { CHECK = 0, CHANGEONE = 1, CHANGEALL = 2 }; 
+        Q_ENUM(SaveFlags)
         enum Mode      { Edit, View, Select };
+        Q_ENUM(Mode)
 
 	AddressCluster(QWidget*, const char* = 0);
+
+        static SaveFlags askForSaveMode(int pAddrId, QWidget *pParent = 0);
 	
 	inline virtual bool    activeVisible() const { return _active->isVisible(); };
         inline virtual QString addrChange()   const { return _addrChange->text(); };
@@ -134,7 +139,7 @@ class XTUPLEWIDGETS_EXPORT AddressCluster : public VirtualCluster
 
     public slots:
 	inline virtual void clearExtraClause()	{ };
-	inline virtual void setExtraClause(const QString&)  { };
+	inline virtual void setExtraClause(const QString&, const QString&)  { };
 	virtual void        setActiveVisible(const bool p);
         virtual void        setAddrChange(QString p);
 	inline virtual void setCity(const QString& p)	{ _city->setText(p); };
@@ -156,7 +161,7 @@ class XTUPLEWIDGETS_EXPORT AddressCluster : public VirtualCluster
 	virtual void	sInfo();
 	virtual void	sList();
 	virtual void	sSearch();
-	virtual void	setId(const int);
+	virtual void	setId(const int, const QString& = QString::null);
         virtual void	setSearchAcct(const int crmAcctId) { _searchAcctId = crmAcctId; }
         virtual int	save(enum SaveFlags);
         virtual void    check();
@@ -242,4 +247,6 @@ class XTUPLEWIDGETS_EXPORT AddressCluster : public VirtualCluster
 	QString  _fieldNameCountry;
 };
 
+void setupAddressCluster(QScriptEngine *engine);
+
 #endif
