@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -86,11 +86,7 @@ void alarmMaint::set( const ParameterList & pParams )
   
   param = pParams.value("source", &valid);
   if (valid)
-  {
     _source = (enum Alarms::AlarmSources)param.toInt();
-    if ( (QString(Alarms::_alarmMap[_source].ident) == QString("TODO")) || (QString(Alarms::_alarmMap[_source].ident) == QString("J")) )
-      _alarmDate->setEnabled(false);
-  }
     
   param = pParams.value("source_id", &valid);
   if(valid)
