@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -10,6 +10,7 @@
 
 #include <QDesktopServices>
 #include <QDialog>
+#include <QtScript>
 
 #include <parameter.h>
 #include <xsqlquery.h>
@@ -292,4 +293,16 @@ void Alarms::refresh()
   _alarms->populate(query);
 }
 
+// script exposure /////////////////////////////////////////////////////////////
 
+void setupAlarms(QScriptEngine *engine)
+{
+  if (! engine->globalObject().property("Alarms").isObject())
+  {
+    QScriptValue ctor = engine->newObject(); //engine->newFunction(scriptconstructor);
+    QScriptValue meta = engine->newQMetaObject(&Alarms::staticMetaObject, ctor);
+
+    engine->globalObject().setProperty("Alarms", meta,
+                                       QScriptValue::ReadOnly | QScriptValue::Undeletable);
+  }
+}
