@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -22,6 +22,8 @@
 #if QT_VERSION >= 0x050000
 #include <QUrlQuery>
 #endif
+#include <QtScript>
+
 #include <parameter.h>
 #include <xsqlquery.h>
 
@@ -93,8 +95,8 @@ QMap<QString, struct CommentMap *> &Comments::commentMap() {
     addToMap(MaintOrder,        "FAMAINT", tr("Maintenance Order"),      "maintord_id",  "maintOrder"        );
     addToMap(ExchangeRate,      "FX",    tr("Exchange Rate")                                                 );
     addToMap(Incident,          "INCDT", tr("Incident"),                 "incdt_id",     "incident",     "MaintainPersonalIncidents MaintainAllIncidents");
-//  addToMap(Invoice,           "INV",   tr("Invoice"),                  "invchead_id",  "invoice"           );
-//  addToMap(InvoiceItem,       "INVI",  tr("Invoice Item")                                                  );
+    addToMap(Invoice,           "INV",   tr("Invoice"),                  "invchead_id",  "invoice"           );
+    addToMap(InvoiceItem,       "INVI",  tr("Invoice Item")                                                  );
     addToMap(Item,              "I",     tr("Item"),                     "item_id",      "item"              );
     addToMap(ItemSite,          "IS",    tr("Item Site")                                                     );
     addToMap(ItemSource,        "IR",    tr("Item Source"),              "itemsrc_id",   "itemSource"        );
@@ -118,7 +120,7 @@ QMap<QString, struct CommentMap *> &Comments::commentMap() {
     addToMap(TransferOrder,     "TO",    tr("Transfer Order"),           "tohead_id",   "transferOrder"      );
     addToMap(TransferOrderItem, "TI",    tr("Transfer Order Item")                                           );
     addToMap(Vendor,            "V",     tr("Vendor"),                   "vend_id",     "vendor"             );
-//  addToMap(Voucher,           "VCH",   tr("Voucher"),                  "vohead_id",   "voucher"            );
+    addToMap(Voucher,           "VCH",   tr("Voucher"),                  "vohead_id",   "voucher"            );
     addToMap(Warehouse,         "WH",    tr("Site")                                                          );
     addToMap(WorkOrder,         "W",     tr("Work Order"),               "wo_id",       "workOrder"          );
     addToMap(Task,              "TA",    tr("Project Task"),             "prjtask_id",  "projectTask"        );
@@ -201,7 +203,7 @@ Comments::Comments(QWidget *pParent, const char *name) :
   connect(_viewComment, SIGNAL(clicked()), this, SLOT( sView()));
   connect(_editComment, SIGNAL(clicked()), this, SLOT(sEdit()));
   connect(_comment, SIGNAL(valid(bool)), this, SLOT(sCheckButtonPriv(bool)));
-  connect(_comment, SIGNAL(itemSelected(int)), _viewComment, SLOT(animateClick()));
+  connect(_comment, SIGNAL(itemSelected(int)), this, SLOT(sOpen()));
   connect(_browser, SIGNAL(anchorClicked(QUrl)), this, SLOT(anchorClicked(QUrl)));
   connect(_verbose, SIGNAL(toggled(bool)), this, SLOT(setVerboseCommentList(bool)));
 
@@ -229,7 +231,7 @@ void Comments::setType(QString sourceType)
 void Comments::setId(int pSourceid)
 {
   _sourceid = pSourceid;
-  _newComment->setEnabled(true); 
+  _newComment->setEnabled(_editable);
   refresh();
 }
 
@@ -288,6 +290,14 @@ void Comments::sEdit()
   refresh();
 }
 
+void Comments::sOpen()
+{
+  if (_editComment->isEnabled())
+    _editComment->animateClick();
+  else
+    _viewComment->animateClick();
+}
+
 void Comments::refresh()
 {
   _browser->document()->clear();
@@ -492,3 +502,17 @@ bool Comments::userCanEdit(int id)
   }
   return false;
 }
+
+// script exposure /////////////////////////////////////////////////////////////
+
+void setupComments(QScriptEngine *engine)
+{
+  if (! engine->globalObject().property("Comments").isObject())
+  {
+    QScriptValue ctor = engine->newObject(); //engine->newFunction(scriptconstructor);
+    QScriptValue meta = engine->newQMetaObject(&Comments::staticMetaObject, ctor);
+
+    engine->globalObject().setProperty("Comments", meta,
+                                       QScriptValue::ReadOnly | QScriptValue::Undeletable);
+  }
+}
