@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -52,8 +52,6 @@ class XTUPLEWIDGETS_EXPORT Comments : public QWidget
 {
   Q_OBJECT
 
-  Q_ENUMS(CommentSources)
-
   Q_PROPERTY(int type READ type WRITE setType)
   
   friend class comment;
@@ -61,26 +59,35 @@ class XTUPLEWIDGETS_EXPORT Comments : public QWidget
   public:
     Comments(QWidget *, const char * = 0);
 
-    // if you add to this then add to the _commentMap[] below
+    // if you add to this then add to the _commentMap[] in the .cpp
     enum CommentSources
     {
       Uninitialized,
       Address,
-      BBOMHead,		BBOMItem,
-      BOMHead,		BOMItem,
-      BOOHead,		BOOItem,
-      CRMAccount,	Contact,	Customer,
-      Employee,         Incident,
-      Item,		ItemSite,	ItemSource,
-      Location,		LotSerial,
-      Opportunity,
-      Project,		PurchaseOrder,	PurchaseOrderItem,
-      ReturnAuth, ReturnAuthItem, Quote, QuoteItem,
-      SalesOrder,	SalesOrderItem, Task, TimeAttendance,
-      TodoItem,      TransferOrder,	TransferOrderItem,
-      Vendor,
-      Warehouse,	WorkOrder
+      BBOMHead,          BBOMItem,
+      BOMHead,           BOMItem,
+      BOOHead,           BOOItem,
+      CRMAccount,        Contact,
+      Customer,          Employee,
+      FixedAsset,        MaintOrder,
+      ExchangeRate,      Incident,
+      Invoice,           InvoiceItem,
+      Item,              ItemSite,
+      ItemSource,        Location,
+      LotSerial,         Opportunity,
+      Project,           PurchaseOrder,
+      PurchaseOrderItem, ReturnAuth,
+      ReturnAuthItem,    RentalItem,
+      Quote,             QuoteItem,
+      SalesOrder,        SalesOrderItem,
+      Task,              TimeAttendance,
+      TodoItem,          TransferOrder,
+      TransferOrderItem, Vendor,
+      Voucher,           Warehouse,
+      WorkOrder
     };
+    Q_ENUM(CommentSources)
+
 
     inline int sourceid()             { return _sourceid; }
     int         type() const;
@@ -100,6 +107,7 @@ class XTUPLEWIDGETS_EXPORT Comments : public QWidget
     void sNew();
     void sView();
     void sEdit();
+    void sOpen();
     void refresh();
 
     void anchorClicked(const QUrl &);
@@ -132,4 +140,6 @@ class XTUPLEWIDGETS_EXPORT Comments : public QWidget
     XCheckBox *_verbose;
 };
 
+void setupComments(QScriptEngine *engine);
+
 #endif
