@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -9,12 +9,13 @@
  */
 
 #include <QDateTime>
+#include <QGridLayout>
 #include <QLabel>
-#include <QValidator>
 #include <QMessageBox>
 #include <QRect>
-#include <QGridLayout>
 #include <QSqlError>
+#include <QValidator>
+#include <QtScript>
 
 #include "xsqlquery.h"
 #include "xcombobox.h"
@@ -29,7 +30,7 @@
 #define EPSILON(d) (pow(10.0, -1 * (1 + d + decimals())) * 5)
 #define ABS(f)	((f) > 0 ? (f) : 0 - (f))
 
-#define MINWIDTH       120
+#define MINWIDTH       155
 #define MAXWIDTH       200
 #define MAXHEIGHT    32767
 #define MAXCURRWIDTH    80	/* max width of _currency combobox */
@@ -190,18 +191,18 @@ CurrCluster::CurrCluster(QWidget * parent, const char* name)
 
     _valueBaseLit->setText(_baseAbbr);
 
-    _valueBaseLit->setHidden(TRUE);
-    _valueBaseWidget->setHidden(TRUE);
+    _valueBaseLit->setHidden(true);
+    _valueBaseWidget->setHidden(true);
 
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
     setMinimumWidth(160);
 #endif
 
     clear();
 
-    setEnabled(TRUE);
-    setBaseVisible(TRUE);
-    setCurrencyEnabled(TRUE);
+    setEnabled(true);
+    setBaseVisible(true);
+    setCurrencyEnabled(true);
 }
 
 void CurrCluster::clear()
@@ -226,8 +227,8 @@ void CurrCluster::setBaseVisible(bool showBase)
     }
     else
     {
-	_valueBaseLit->setHidden(TRUE);
-	_valueBaseWidget->setHidden(TRUE);
+	_valueBaseLit->setHidden(true);
+	_valueBaseWidget->setHidden(true);
     }
 }
 
@@ -472,7 +473,7 @@ int CurrDisplay::baseId()
     baseQuery.prepare("SELECT curr_id, "
                       "  ' ' || curr_abbr || '-' || curr_symbol AS f_curr " //currConcat(curr_id) AS curr_symbol "
 		      "FROM curr_symbol "
-		      "WHERE curr_base = TRUE;");
+		      "WHERE curr_base = true;");
     baseQuery.exec();
     if (baseQuery.first())
     {
@@ -534,8 +535,8 @@ CurrDisplay::CurrDisplay(QWidget * parent, const char* name)
     _default = 0;
     clear();
 
-    setEnabled(FALSE);
-    setLocalControl(TRUE);
+    setEnabled(false);
+    setLocalControl(true);
     setMinimumSize(MINWIDTH, 0);
     setMaximumSize(MAXWIDTH, MAXHEIGHT);
     
@@ -984,3 +985,17 @@ void CurrDisplay::setDataWidgetMap(XDataWidgetMapper* m)
   m->addMapping(this, _fieldNameValue, QByteArray("localValue"));
   _mapper=m;
 }
+
+// script api //////////////////////////////////////////////////////////////////
+
+void setupCurrDisplay(QScriptEngine *engine)
+{
+  if (! engine->globalObject().property("CurrDisplay").isObject())
+  {
+    QScriptValue ctor = engine->newObject(); //engine->newFunction(scriptconstructor);
+    QScriptValue meta = engine->newQMetaObject(&CurrDisplay::staticMetaObject, ctor);
+
+    engine->globalObject().setProperty("CurrDisplay", meta,
+                                       QScriptValue::ReadOnly | QScriptValue::Undeletable);
+  }
+}
