@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -11,6 +11,7 @@
 #include <QApplication>
 #include <QCalendarWidget>
 #include <QDateTime>
+#include <QDebug>
 #include <QDesktopWidget>
 #include <QHBoxLayout>
 #include <QMessageBox>
@@ -20,13 +21,14 @@
 #include <QSqlError>
 #include <QVBoxLayout>
 #include <QValidator>
-#include <QDebug>
+#include <QtScript>
 
 #include <xsqlquery.h>
 #include <parameter.h>
 
 #include "datecluster.h"
 #include "dcalendarpopup.h"
+#include "errorReporter.h"
 #include "format.h"
 
 #define DEBUG false
@@ -152,7 +154,9 @@ void XDateEdit::parseDate()
 
   _valid = false;
 
-  if (dateString.contains(QRegExp("[0-9]+[-][0-9]+"))) //user enters hyphens instead of slashes
+  if (dateString.contains(QRegExp("[0-9]+[-][0-9]+")) &&
+      QLocale().dateFormat(QLocale::ShortFormat).contains(QRegExp("(M+|y+|d+)/(M+|y+|d+)")))
+  //user enters hyphens instead of slashes and locale uses slashes
   {
     dateString.replace("-", "/");
   }
@@ -202,6 +206,17 @@ void XDateEdit::parseDate()
              qPrintable(dateFormatStr));
 
     QDate tmp = QDate::fromString(dateString, dateFormatStr);
+
+    if (tmp.month() < 10 && dateFormatStr.indexOf(QRegExp("M{2}")) >= 0) // handle partial entered months
+    {
+      dateFormatStr.replace(QRegExp("M{2}"), "M");
+      if (DEBUG)
+        qDebug("%s::parseDate() rewriting 2-digit month format string to %s",
+               qPrintable(parent() ? parent()->objectName() : objectName()),
+               qPrintable(dateFormatStr));
+      tmp = QDate::fromString(dateString, dateFormatStr);
+    }
+
     bool twodigitformat = !(dateFormatStr.indexOf(QRegExp("y{4}")) >= 0);
     if (tmp.isValid())
     {
@@ -389,6 +404,18 @@ void XDateEdit::setDataWidgetMap(XDataWidgetMapper* m)
   m->addMapping(this, _fieldName, "date", "currentDefault");
 }
 
+void XDateEdit::setCalendarSiteId(int siteId)
+{
+  int tmpSite = _siteId;
+  _siteId = siteId;
+  if(siteId!=tmpSite&&tmpSite!=-1)
+  {
+    QDate tmpDate = _currentDate;
+    setNull();
+    checkDate(tmpDate);
+  }
+}
+
 void XDateEdit::setNull()
 {
   if (DEBUG)
@@ -474,13 +501,12 @@ void XDateEdit::checkDate(const QDate &pDate)
     workday.bindValue(":desired", 0);
     workday.exec();
     if (workday.first())
-      nextWorkDate = workday.value("result").toDate();
-    else if (workday.lastError().type() != QSqlError::NoError)
     {
-      QMessageBox::warning(this, tr("No work week calendar found"),
-                            tr("<p>The selected Site has no work week defined. "
-                               "Please go to Schedule Setup and define "
-                               "the working days for this site."));
+      nextWorkDate = workday.value("result").toDate();
+    }
+    else if (ErrorReporter::error(QtWarningMsg, this, tr("No work week calendar found"),
+                                  workday, __FILE__, __LINE__))
+    {
       return;
     }
   }
@@ -758,3 +784,17 @@ void DateCluster::setEndVisible(bool p)
   _endDate->setVisible(p);
   _endDateLit->setVisible(p);
 }
+
+// script api //////////////////////////////////////////////////////////////////
+
+void setupXDateEdit(QScriptEngine *engine)
+{
+  if (! engine->globalObject().property("XDateEdit").isObject())
+  {
+    QScriptValue ctor = engine->newObject(); //engine->newFunction(scriptconstructor);
+    QScriptValue meta = engine->newQMetaObject(&XDateEdit::staticMetaObject, ctor);
+
+    engine->globalObject().setProperty("XDateEdit", meta,
+                                       QScriptValue::ReadOnly | QScriptValue::Undeletable);
+  }
+}
