@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -14,17 +14,15 @@
 #include "widgets.h"
 #include "ui_filemoveselector.h"
 
-void setupFileMoveSelector(QScriptEngine *engine);
-
 class FileMoveSelector : public QWidget, public Ui::FileMoveSelector
 {
     Q_OBJECT
-    Q_ENUMS(FileMoveOption)
 
   public:
     enum FileMoveOption { Nothing, Suffix, ChangeDir, Delete };
+    Q_ENUM(FileMoveOption)
 
-    FileMoveSelector(QWidget* parent = 0, Qt::WFlags fl = 0);
+    FileMoveSelector(QWidget* parent = 0, Qt::WindowFlags fl = 0);
     ~FileMoveSelector();
 
     Q_INVOKABLE QString        code()           const;
@@ -46,6 +44,6 @@ class FileMoveSelector : public QWidget, public Ui::FileMoveSelector
 
 };
 
-Q_DECLARE_METATYPE(FileMoveSelector*)
+void setupFileMoveSelector(QScriptEngine *engine);
 
 #endif
