@@ -23,7 +23,7 @@ class imageAssignment : public QDialog, public Ui::imageAssignment
     Q_OBJECT
 
 public:
-    imageAssignment(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    imageAssignment(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~imageAssignment();
 
     virtual void populate();
@@ -39,11 +39,10 @@ protected slots:
     virtual void languageChange();
 
 private:
-    enum Documents::DocumentSources _source;
-    int  _mode;
-    int  _sourceid;
-    int  _imageassid;
-    
+    int     _imageassid;
+    int     _mode;
+    int     _sourceid;
+    QString _sourcetype;
 
 };
 
