@@ -1,22 +1,28 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
  * to be bound by its terms.
  */
 
-#include <QApplication>
-#include <QMessageBox>
+#include <QCompleter>
+#include <QGridLayout>
+#include <QLabel>
+#include <QMenu>
 #include <QSqlRecord>
-#include <QStandardItemEditorCreator>
+#include <QTreeView>
+#include <QtScript>
 
 #include <xsqlquery.h>
 
+#include "guiclientinterface.h"
 #include "itemcluster.h"
 #include "itemAliasList.h"
+#include "xcheckbox.h"
+#include "xtreewidget.h"
 #include "xsqltablemodel.h"
 
 #define DEBUG false
@@ -28,19 +34,19 @@ QString buildItemLineEditQuery(const QString pPre, const QStringList pClauses, c
 {
   QStringList clauses = pClauses;
   QString sql;
-  if (unionAlias) 
+  if (unionAlias)
   {
     sql = pPre + " FROM ("
                  "   SELECT item_id, item_number, item_descrip1, item_descrip2, "
                  "      item_upccode, item_type, item_fractional, item_config, item_inv_uom_id, "
-                 "      item_sold, item_active "
+                 "      item_sold, item_active, NULL AS itemalias_crmacct_id "
                  "   FROM item "
                  "   UNION "
                  "   SELECT item_id, itemalias_number, "
                  "     CASE WHEN LENGTH(itemalias_descrip1) > 1 THEN itemalias_descrip1 ELSE item_descrip1 END, "
                  "     CASE WHEN LENGTH(itemalias_descrip2) > 1 THEN itemalias_descrip1 ELSE item_descrip2 END, "
                  "      item_upccode, item_type, item_fractional, item_config, item_inv_uom_id,"
-                 "      item_sold, item_active "
+                 "      item_sold, item_active, itemalias_crmacct_id "
                  "   FROM item "
                  "     JOIN itemalias ON item_id = itemalias_item_id "
                  ") AS item "
@@ -65,13 +71,13 @@ QString buildItemLineEditQuery(const QString pPre, const QStringList pClauses, c
     sql += ", bomitem";
     clauses << "(bomitem_parent_item_id=item_id)";
   }
-  
+
   if (pType & (ItemLineEdit::cUsedOnBom))
   {
     sql += ", bomitem";
     clauses << "(bomitem_item_id=item_id)";
   }
-  
+
   if (pType & ItemLineEdit::cAllItemTypes_Mask)
   {
     QStringList types;
@@ -262,7 +268,7 @@ ItemLineEdit::ItemLineEdit(QWidget* pParent, const char* pName) :
   setNewPriv("MaintainItemMasters");
 
   setAcceptDrops(true);
-  
+
   _type = cUndefined;
   _defaultType = cUndefined;
   _useQuery = false;
@@ -273,16 +279,13 @@ ItemLineEdit::ItemLineEdit(QWidget* pParent, const char* pName) :
   _id = -1;
   _configured = false;
   _fractional = false;
+  _crmacct = 0;
   _delegate = new ItemLineEditDelegate(this);
 
   connect(_aliasAct, SIGNAL(triggered()), this, SLOT(sAlias()));
 
-  // Add alias
-  if (_x_preferences)
-  {
-    if (!_x_preferences->boolean("ClusterButtons"))
-      menu()->insertAction(menu()->actions().at(2),_aliasAct);
-  }
+  if (_x_preferences && ! _x_preferences->boolean("ClusterButtons"))
+    menu()->insertAction(menu()->actions().at(2), _aliasAct);
 }
 
 void ItemLineEdit::setItemNumber(const QString& pNumber)
@@ -326,9 +329,9 @@ void ItemLineEdit::setItemNumber(const QString& pNumber)
       item.prepare(buildItemLineEditQuery(pre, clauses, QString::null, _type, false));
       item.bindValue(":item_number", pNumber);
       item.exec();
-      
+
       if (item.size() > 1)
-      { 
+      {
         ParameterList params;
         params.append("search", pNumber);
         params.append("searchNumber");
@@ -361,7 +364,7 @@ void ItemLineEdit::setItemNumber(const QString& pNumber)
     emit configured(item.value("item_config").toBool());
     emit fractional(item.value("item_fractional").toBool());
     emit upcChanged(item.value("item_upccode").toString());
-    
+
     emit valid(true);
   }
   else
@@ -455,7 +458,7 @@ void ItemLineEdit::silentSetId(const int pId)
     emit configured(item.value("item_config").toBool());
     emit fractional(item.value("item_fractional").toBool());
     emit upcChanged(item.value("item_upccode").toString());
-    
+
     emit valid(true);
 
     if (completer())
@@ -483,7 +486,7 @@ void ItemLineEdit::silentSetId(const int pId)
 
     emit valid(false);
   }
-} 
+}
 
 void ItemLineEdit::setId(int pId)
 {
@@ -520,6 +523,11 @@ void ItemLineEdit::setItemsiteid(int pItemsiteid)
   }
 }
 
+void ItemLineEdit::setCRMAcctId(unsigned int pAcct)
+{
+  _crmacct = pAcct;
+}
+
 void ItemLineEdit::sInfo()
 {
   ParameterList params;
@@ -532,12 +540,12 @@ void ItemLineEdit::sCopy()
 {
   ParameterList params;
   params.append("item_id", id());
-  
+
   QString uiName = "copyItem";
 
   QWidget* w = 0;
   w = _guiClientInterface->openWindow(uiName, params, parentWidget()->window() , Qt::WindowModal, Qt::Dialog);
-  
+
   QDialog* newdlg = qobject_cast<QDialog*>(w);
   int id = newdlg->exec();
   if (id != QDialog::Rejected)
@@ -546,7 +554,7 @@ void ItemLineEdit::sCopy()
     emit newId(_id);
     emit valid(_id != -1);
   }
-  
+
   return;
 }
 
@@ -571,7 +579,7 @@ void ItemLineEdit::sHandleCompleter()
     numQ.prepare(QString("SELECT *"
                          "  FROM (%1) data"
                          " WHERE (POSITION(:number IN item_number)=1)"
-                         " LIMIT 10")
+                         " ORDER BY item_number LIMIT 10")
                  .arg(QString(_sql)).remove(";"));
     numQ.bindValue(":number", stripped);
   }
@@ -585,6 +593,9 @@ void ItemLineEdit::sHandleCompleter()
     clauses = _extraClauses;
     clauses << "((POSITION(:searchString IN item_number) = 1)"
             " OR (POSITION(:searchString IN item_upccode) = 1))";
+    if (_crmacct > 0)
+      clauses << QString("(itemalias_crmacct_id IS NULL OR itemalias_crmacct_id = %1)")
+                    .arg(_crmacct);
     numQ.prepare(buildItemLineEditQuery(pre, clauses, QString::null, _type, true)
                               .replace(";"," ORDER BY item_number LIMIT 10;"));
     numQ.bindValue(":searchString", QString(text().trimmed().toUpper()));
@@ -719,6 +730,9 @@ void ItemLineEdit::sAlias()
   if (!_extraClauses.isEmpty())
     params.append("extraClauses", _extraClauses);
 
+  if (_crmacct > 0)
+    params.append("crmacct", _crmacct);
+
   itemAliasList newdlg(parentWidget(), "", true);
   newdlg.set(params);
 
@@ -783,11 +797,18 @@ void ItemLineEdit::sParse()
            "_useValidationQuery %d, _useQuery %d",
            qPrintable(objectName()), _parsed, qPrintable(text()),
            _useValidationQuery, _useQuery);
-  if (!_parsed)
+  if (_completerId)
   {
+    int id = _completerId;
+    _completerId = 0;
+    setId(id);
+  }
+  else if (!_parsed)
+  {
+    QString stripped = text().trimmed().toUpper();
     _parsed = true;
 
-    if (text().length() == 0)
+    if (stripped.isEmpty())
     {
       setId(-1);
       return;
@@ -806,7 +827,7 @@ void ItemLineEdit::sParse()
                      " WHERE (item_number = :searchString"
                      "     OR item_upccode = :searchString);");
 
-      item.bindValue(":searchString", text().trimmed().toUpper());
+      item.bindValue(":searchString", stripped);
       item.exec();
       while (item.next())
       {
@@ -818,7 +839,7 @@ void ItemLineEdit::sParse()
         if (oneq.size() > 1)
         {
           ParameterList params;
-          params.append("search", text().trimmed().toUpper());
+          params.append("search", stripped);
           params.append("searchNumber");
           params.append("searchUpc");
           sSearch(params);
@@ -833,21 +854,15 @@ void ItemLineEdit::sParse()
     }
     else if (_useQuery)
     {
-      XSqlQuery item;
-      item.prepare(_sql);
-      item.exec();
-      if (item.first())
+      XSqlQuery item(_sql);
+      while (item.next())
       {
-        do
+        if (item.value("item_number").toString().startsWith(stripped) ||
+            item.value("item_upccode").toString().startsWith(stripped))
         {
-          if (item.value("item_number").toString().startsWith(text().trimmed().toUpper()) ||
-              item.value("item_upccode").toString().startsWith(text().trimmed().toUpper()))
-          {
-            setId(item.value("item_id").toInt());
-            return;
-          }
+          setId(item.value("item_id").toInt());
+          return;
         }
-        while (item.next());
       }
     }
     else
@@ -862,13 +877,18 @@ void ItemLineEdit::sParse()
       // first check item number
       clauses = _extraClauses;
       clauses << "(POSITION(:searchString IN item_number) = 1)";
+      if (_crmacct > 0)
+        clauses << QString("(itemalias_crmacct_id IS NULL OR itemalias_crmacct_id = %1)")
+                      .arg(_crmacct);
       item.prepare(buildItemLineEditQuery(pre, clauses, QString::null, _type, true)
                                .replace(";"," ORDER BY item_number LIMIT 1;"));
-      item.bindValue(":searchString", QString(text().trimmed().toUpper()));
+      item.bindValue(":searchString", stripped);
       item.exec();
       if (item.first())
       {
         setId(item.value("item_id").toInt());
+        if(_itemNumber != item.value("number").toString())
+          emit aliasChanged(item.value("number").toString());
         return;
       }
       // item number not found, check upccode
@@ -876,18 +896,18 @@ void ItemLineEdit::sParse()
       clauses << "(POSITION(:searchString IN item_upccode) = 1)";
       item.prepare(buildItemLineEditQuery(pre, clauses, QString::null, _type, true)
                    .replace(";"," ORDER BY item_number LIMIT 1;"));
-      item.bindValue(":searchString", QString(text().trimmed().toUpper()));
+      item.bindValue(":searchString", stripped);
       item.exec();
       if (item.first())
       {
         setId(item.value("item_id").toInt());
         return;
       }
-      if (_x_metrics->boolean("AutoItemSearch"))
+      if (_x_metrics && _x_metrics->boolean("AutoItemSearch"))
       {
         // nothing found, start search
         ParameterList params;
-        params.append("search", text().trimmed().toUpper());
+        params.append("search", stripped);
         params.append("searchNumber");
         params.append("searchUpc");
         params.append("searchAlias");
@@ -935,11 +955,9 @@ ItemCluster::ItemCluster(QWidget* pParent, const char* pName) :
   ItemLineEdit* itemNumber = static_cast<ItemLineEdit* >(_number);
   connect(itemNumber, SIGNAL(aliasChanged(const QString &)), this, SIGNAL(aliasChanged(const QString &)));
   connect(itemNumber, SIGNAL(privateIdChanged(int)), this, SIGNAL(privateIdChanged(int)));
-  connect(itemNumber, SIGNAL(newId(int)), this, SIGNAL(newId(int)));
   connect(itemNumber, SIGNAL(uomChanged(const QString &)), this, SIGNAL(uomChanged(const QString &)));
   connect(itemNumber, SIGNAL(descrip1Changed(const QString &)), this, SIGNAL(descrip1Changed(const QString &)));
   connect(itemNumber, SIGNAL(descrip2Changed(const QString &)), this, SIGNAL(descrip2Changed(const QString &)));
-  connect(itemNumber, SIGNAL(valid(bool)), this, SIGNAL(valid(bool)));
   connect(itemNumber, SIGNAL(warehouseIdChanged(int)), this, SIGNAL(warehouseIdChanged(int)));
   connect(itemNumber, SIGNAL(typeChanged(const QString &)), this, SIGNAL(typeChanged(const QString &)));
   connect(itemNumber, SIGNAL(upcChanged(const QString &)), this, SIGNAL(upcChanged(const QString &)));
@@ -951,18 +969,24 @@ ItemCluster::ItemCluster(QWidget* pParent, const char* pName) :
   connect(itemNumber, SIGNAL(descrip2Changed(const QString &)), _descrip2, SLOT(setText(const QString &)));
 }
 
-void ItemCluster::addNumberWidget(ItemLineEdit* pNumberWidget)
+void ItemCluster::addNumberWidget(VirtualClusterLineEdit* pNumberWidget)
 {
-    _number = pNumberWidget;
-    if (! _number)
-      return;
+  VirtualClusterLineEdit *matchType = qobject_cast<VirtualClusterLineEdit *>(pNumberWidget);
 
-    _grid->addWidget(_number, 0, 1);
-    setFocusProxy(pNumberWidget);
+  if(matchType == 0)
+    return;
 
-    connect(_number,	SIGNAL(newId(int)),	this,	 SIGNAL(newId(int)));
-    connect(_number,	SIGNAL(parsed()), 	this, 	 SLOT(sRefresh()));
-    connect(_number,	SIGNAL(valid(bool)),	this,	 SIGNAL(valid(bool)));
+  _number = matchType;
+
+  if (! _number)
+    return;
+
+  _grid->addWidget(_number, 0, 1);
+  setFocusProxy(pNumberWidget);
+
+  connect(_number,      SIGNAL(newId(int)),     this,    SIGNAL(newId(int)));
+  connect(_number,      SIGNAL(parsed()),       this,    SLOT(sRefresh()));
+  connect(_number,      SIGNAL(valid(bool)),    this,    SIGNAL(valid(bool)));
 }
 
 void ItemCluster::setDescriptionVisible(const bool p)
@@ -989,7 +1013,7 @@ void ItemCluster::setDisabled(bool pDisabled)
   setReadOnly(pDisabled);
 }
 
-void ItemCluster::setId(const int pId)
+void ItemCluster::setId(const int pId, const QString&)
 {
   _number->setId(pId);
 }
@@ -1004,6 +1028,11 @@ void ItemCluster::setItemsiteid(int intPItemsiteid)
   static_cast<ItemLineEdit* >(_number)->setItemsiteid(intPItemsiteid);
 }
 
+void ItemCluster::setCRMAcctId(unsigned int intPCRMAcctid)
+{
+  static_cast<ItemLineEdit* >(_number)->setCRMAcctId(intPCRMAcctid);
+}
+
 void ItemCluster::setOrientation(Qt::Orientation orientation)
 {
   _descrip2->setVisible(orientation == Qt::Vertical);
@@ -1092,7 +1121,6 @@ void itemList::set(const ParameterList &pParams)
   param = pParams.value("caption", &valid);
   if (valid)
     setWindowTitle(param.toString());
-
   sFillList();
 }
 
@@ -1130,15 +1158,15 @@ void itemList::sFillList()
       QString post;
       if(_x_preferences && _x_preferences->boolean("ListNumericItemNumbersFirst"))
       {
-        pre =  "SELECT DISTINCT ON (toNumeric(item_number, 999999999999999), item_number) item_id, item_number,"
-               "(item_descrip1 || ' ' || item_descrip2) AS itemdescrip, item_upccode ";
-        post = "ORDER BY toNumeric(item_number, 999999999999999), item_number, item_upccode ";
+        pre = "SELECT DISTINCT item_id, item_number,  "
+              "(item_descrip1 || ' ' || item_descrip2) AS itemdescrip, item_upccode, isNumeric(item_number) ";
+        post = "ORDER BY isNumeric(item_number) DESC, item_number, item_upccode;";
       }
       else
       {
         pre =  "SELECT DISTINCT item_id, item_number,"
                "(item_descrip1 || ' ' || item_descrip2) AS itemdescrip, item_upccode ";
-        post = "ORDER BY item_number";
+        post = "ORDER BY item_number;";
       }
 
       QStringList clauses;
@@ -1167,6 +1195,11 @@ void itemList::reject()
   done(_itemid);
 }
 
+void itemList::showEvent(QShowEvent *e)
+{
+  QDialog::showEvent(e);
+}
+
 //////////////////////////////////////////////////////////////////////
 
 itemSearch::itemSearch(QWidget* pParent, Qt::WindowFlags pFlags)
@@ -1195,7 +1228,7 @@ itemSearch::itemSearch(QWidget* pParent, Qt::WindowFlags pFlags)
   _searchAlias->setChecked( true );
   _searchAlias->setObjectName("_searchAlias");
   selectorsLyt->addWidget(_searchAlias, 4, 0);
-  
+
   _showInactive = new XCheckBox(tr("Show &Inactive Items"), this);
   _showInactive->setObjectName("_showInactive");
   selectorsLyt->addWidget(_showInactive, 5, 0);
@@ -1270,7 +1303,7 @@ void itemSearch::set(const ParameterList &pParams)
   param = pParams.value("searchAlias", &valid);
   if (valid)
     _searchAlias->setChecked(true);
-  
+
   sFillList();
 }
 
@@ -1303,7 +1336,7 @@ void itemSearch::sFillList()
 
     if (_searchAlias->isChecked())
       subClauses << "(itemalias_number ~* :searchString)";
-    
+
     if(!subClauses.isEmpty())
       clauses << QString("( " + subClauses.join(" OR ") + " )");
 
@@ -1359,7 +1392,7 @@ void itemSearch::sFillList()
 
     if (_searchAlias->isChecked())
       subClauses << "(itemalias_number ~* :searchString)";
-    
+
     if(!subClauses.isEmpty())
       clauses << QString("( " + subClauses.join(" OR ") + " )");
 
@@ -1372,3 +1405,46 @@ void itemSearch::sFillList()
   search.exec();
   _listTab->populate(search, _itemid);
 }
+
+// script api //////////////////////////////////////////////////////////////////
+
+void setupItemLineEdit(QScriptEngine *engine)
+{
+  // cannot use engine->newQMetaObject because cActive goes negative
+  QScriptValue::PropertyFlags ro = QScriptValue::ReadOnly | QScriptValue::Undeletable;
+  QScriptValue widget = engine->globalObject().property("ItemLineEdit");
+  if (! widget.isObject()) {
+    widget = engine->newObject();
+    engine->globalObject().setProperty("ItemLineEdit", widget, ro);
+  }
+
+  widget.setProperty("cUndefined",          QScriptValue(engine, ItemLineEdit::cUndefined),          ro);
+  widget.setProperty("cPurchased",          QScriptValue(engine, ItemLineEdit::cPurchased),          ro);
+  widget.setProperty("cManufactured",       QScriptValue(engine, ItemLineEdit::cManufactured),       ro);
+  widget.setProperty("cPhantom",            QScriptValue(engine, ItemLineEdit::cPhantom),            ro);
+  widget.setProperty("cBreeder",            QScriptValue(engine, ItemLineEdit::cBreeder),            ro);
+  widget.setProperty("cCoProduct",          QScriptValue(engine, ItemLineEdit::cCoProduct),          ro);
+  widget.setProperty("cByProduct",          QScriptValue(engine, ItemLineEdit::cByProduct),          ro);
+  widget.setProperty("cReference",          QScriptValue(engine, ItemLineEdit::cReference),          ro);
+  widget.setProperty("cCosting",            QScriptValue(engine, ItemLineEdit::cCosting),            ro);
+  widget.setProperty("cTooling",            QScriptValue(engine, ItemLineEdit::cTooling),            ro);
+  widget.setProperty("cOutsideProcess",     QScriptValue(engine, ItemLineEdit::cOutsideProcess),     ro);
+  widget.setProperty("cPlanning",           QScriptValue(engine, ItemLineEdit::cPlanning),           ro);
+  widget.setProperty("cKit",                QScriptValue(engine, ItemLineEdit::cKit),                ro);
+  widget.setProperty("cAllItemTypes_Mask",  QScriptValue(engine, ItemLineEdit::cAllItemTypes_Mask),  ro);
+  widget.setProperty("cPlanningMRP",        QScriptValue(engine, ItemLineEdit::cPlanningMRP),        ro);
+  widget.setProperty("cPlanningMPS",        QScriptValue(engine, ItemLineEdit::cPlanningMPS),        ro);
+  widget.setProperty("cPlanningNone",       QScriptValue(engine, ItemLineEdit::cPlanningNone),       ro);
+  widget.setProperty("cPlanningAny",        QScriptValue(engine, ItemLineEdit::cPlanningAny),        ro);
+  widget.setProperty("cItemActive",         QScriptValue(engine, ItemLineEdit::cItemActive),         ro);
+  widget.setProperty("cSold",               QScriptValue(engine, ItemLineEdit::cSold),               ro);
+  widget.setProperty("cLocationControlled", QScriptValue(engine, ItemLineEdit::cLocationControlled), ro);
+  widget.setProperty("cLotSerialControlled",QScriptValue(engine, ItemLineEdit::cLotSerialControlled),ro);
+  widget.setProperty("cDefaultLocation",    QScriptValue(engine, ItemLineEdit::cDefaultLocation),    ro);
+  widget.setProperty("cActive",             QScriptValue(engine, ItemLineEdit::cActive),             ro);
+  widget.setProperty("cGeneralManufactured",QScriptValue(engine, ItemLineEdit::cGeneralManufactured),ro);
+  widget.setProperty("cGeneralPurchased",   QScriptValue(engine, ItemLineEdit::cGeneralPurchased),   ro);
+  widget.setProperty("cGeneralComponents",  QScriptValue(engine, ItemLineEdit::cGeneralComponents),  ro);
+  widget.setProperty("cGeneralInventory",   QScriptValue(engine, ItemLineEdit::cGeneralInventory),   ro);
+  widget.setProperty("cKitComponents",      QScriptValue(engine, ItemLineEdit::cKitComponents),      ro);
+}
