@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -20,13 +20,9 @@
 
 #include "virtualCluster.h"
 
-class ParameterList;
-
-class QPushButton;
+class QCheckBox;
 class QLabel;
-class QDragEnterEvent;
-class QDropEvent;
-class QMouseEvent;
+
 class ItemLineEditDelegate;
 class itemList;
 class itemSearch;
@@ -54,6 +50,7 @@ private:
     bool _useQuery;
     QString _sql;
     QStringList _extraClauses;
+    void showEvent(QShowEvent *);
 };
 
 class XTUPLEWIDGETS_EXPORT itemSearch : public VirtualSearch
@@ -162,6 +159,7 @@ friend class ItemLineEditDelegate;
     Q_INVOKABLE QString itemType();
     Q_INVOKABLE bool    isConfigured();
     Q_INVOKABLE bool    isFractional();
+    Q_INVOKABLE void setCRMAcctId(unsigned int);
 
   public slots:
     void sHandleCompleter();
@@ -209,6 +207,7 @@ friend class ItemLineEditDelegate;
     QString _upc;
     QString _itemType;
     QStringList _extraClauses;
+    unsigned int _crmacct;
     unsigned int _type;
     unsigned int _defaultType;
     bool    _configured;
@@ -265,13 +264,14 @@ class XTUPLEWIDGETS_EXPORT ItemCluster : public VirtualCluster
     Q_INVOKABLE inline QStringList getExtraClauseList() const       { return static_cast<ItemLineEdit*>(_number)->getExtraClauseList(); }
     Q_INVOKABLE inline void clearExtraClauseList()                  { static_cast<ItemLineEdit*>(_number)->clearExtraClauseList();      }
     Q_INVOKABLE ItemLineEdit *itemLineEdit() { return static_cast<ItemLineEdit*>(_number); }
+    Q_INVOKABLE void setCRMAcctId(unsigned int);
 
     void setOrientation(Qt::Orientation orientation);
 
   public slots:
     QItemDelegate *itemDelegate() { return static_cast<ItemLineEdit*>(_number)->itemDelegate(); }
     void setDescriptionVisible(const bool p);
-    void setId(const int pId);
+    void setId(const int pId, const QString& = QString::null);
     void setItemNumber(QString);
     void setItemsiteid(int);
 
@@ -288,7 +288,7 @@ class XTUPLEWIDGETS_EXPORT ItemCluster : public VirtualCluster
     void fractional(bool);
 
   protected:
-    void addNumberWidget(ItemLineEdit* pNumberWidget);
+    void addNumberWidget(VirtualClusterLineEdit* pNumberWidget);
 
   private:
     QLabel       *_uom;
@@ -296,4 +296,6 @@ class XTUPLEWIDGETS_EXPORT ItemCluster : public VirtualCluster
     QString       _default;
 };
 
+void setupItemLineEdit(QScriptEngine *engine);
+
 #endif
