@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -16,6 +16,11 @@
 #include "parameter.h"
 #include "virtualCluster.h"
 
+class QLabel;
+class QGridLayout;
+class QScriptEngine;
+class XTreeWidgetItem;
+
 class XTUPLEWIDGETS_EXPORT OrderLineEdit : public VirtualClusterLineEdit
 {
   Q_OBJECT
@@ -33,6 +38,7 @@ class XTUPLEWIDGETS_EXPORT OrderLineEdit : public VirtualClusterLineEdit
       AnyStatus = 0x00,
       Unposted  = 0x01, Open = 0x02, Closed = 0x04
     };
+    Q_ENUM(OrderStatus)
     Q_DECLARE_FLAGS(OrderStatuses, OrderStatus)
 
     enum OrderType
@@ -40,6 +46,7 @@ class XTUPLEWIDGETS_EXPORT OrderLineEdit : public VirtualClusterLineEdit
       AnyType = 0x00,
       Purchase = 0x01, Return = 0x02, Sales = 0x04, Transfer = 0x08
     };
+    Q_ENUM(OrderType)
     Q_DECLARE_FLAGS(OrderTypes, OrderType)
 
     Q_INVOKABLE virtual OrderStatuses allowedStatuses()        const;
@@ -57,7 +64,7 @@ class XTUPLEWIDGETS_EXPORT OrderLineEdit : public VirtualClusterLineEdit
     Q_INVOKABLE virtual bool      toSitePrivsEnforced()   const {return _toPrivs; }
     Q_INVOKABLE virtual void      setExtraClause(const QString & p) {VirtualClusterLineEdit::setExtraClause(p);}
     Q_INVOKABLE virtual void          setExtraClause(const QString &, const QString &);
-    Q_INVOKABLE virtual void          setExtraClause(const OrderTypes, const QString &);
+    virtual void                      setExtraClause(const OrderTypes, const QString &);  // Not Q_INVOKABLE because scripts can't handle the overload. See issue #30620
     Q_INVOKABLE virtual void      setFromSitePrivsEnforced(const bool p);
     Q_INVOKABLE virtual void      setToSitePrivsEnforced(const bool p);
     Q_INVOKABLE virtual OrderStatus          status();
@@ -144,8 +151,8 @@ class XTUPLEWIDGETS_EXPORT OrderCluster : public VirtualCluster
     Q_INVOKABLE virtual bool    fromSitePrivsEnforced() const;
     Q_INVOKABLE virtual bool    toSitePrivsEnforced()   const;
     Q_INVOKABLE virtual void    setExtraClause(const QString &, const QString &);
-    Q_INVOKABLE virtual void    setExtraClause(const OrderLineEdit::OrderTypes,
-                                               const QString &);
+    virtual void                setExtraClause(const OrderLineEdit::OrderTypes,
+                                               const QString &); // Not Q_INVOKABLE because scripts can't handle the overload. See issue #30620
     Q_INVOKABLE virtual OrderLineEdit::OrderStatus status()            const;
     Q_INVOKABLE virtual QString                    to()                const;
     Q_INVOKABLE virtual QString                    type()              const;
@@ -153,10 +160,10 @@ class XTUPLEWIDGETS_EXPORT OrderCluster : public VirtualCluster
     Q_INVOKABLE virtual bool lockSelected();
     Q_INVOKABLE virtual void setLockSelected(bool lock);
 
-    virtual bool        nameVisible();
+    virtual bool        nameVisible() const;
     virtual void        setNameVisible(const bool p);
 
-    virtual bool        descriptionVisible();
+    virtual bool        descriptionVisible() const;
     virtual void        setDescriptionVisible(const bool p);
 
   public slots:
@@ -223,4 +230,6 @@ class XTUPLEWIDGETS_EXPORT OrderSearch : public VirtualSearch
     virtual void done(int);
 };
 
+void setupOrderLineEdit(QScriptEngine *engine);
+
 #endif
