@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -130,16 +130,30 @@ ParameterWidget::~ParameterWidget()
       delete pp;
 }
 
+QString ParameterWidget::preferencePrefix() const
+{
+  QWidget* child = parentWidget();
+  while (child->parentWidget())
+  {
+    if (child->parentWidget()->objectName().isEmpty())
+      break;
+    child = child->parentWidget();
+  }
+
+  QString result = child->objectName() + "/" + objectName();
+
+  if (DEBUG)
+    qDebug() << objectName() << __FUNCTION__ << "returning" << result;
+  return result;
+}
+
 void ParameterWidget::showEvent(QShowEvent * event)
 {
 
-  if(!_initialized)
+  if (! _initialized)
   {
-    if(window())
-      _name = window()->objectName() + "/";
-    _settingsName = _name + objectName();
-    if(_x_preferences)
-      setFiltersVisible(_x_preferences->boolean(_settingsName + "/checked"));
+    if (_x_preferences)
+      setFiltersVisible(_x_preferences->boolean(preferencePrefix() + "/checked"));
     _initialized = true;
   }
   QWidget::showEvent(event);
@@ -185,39 +199,19 @@ void ParameterWidget::appendValue(ParameterList &pParams)
 
 void ParameterWidget::applyDefaultFilterSet()
 {
-  XSqlQuery qry;
-  QString   filter_name;
-  int       filter_id = -1;
-  QString   pname;
-
-  if(window())
-    pname = window()->objectName() + "/";
-  _settingsName2 = pname + this->objectName();
-
-  if(_x_preferences)
-  {
-    _settingsName2 += "/filter_default";
-    filter_id = _x_preferences->value(_settingsName2).toInt();
-  }
-
-  QString query = "SELECT filter_name "
-                  "FROM filter "
-                  "WHERE filter_id=:id ";
-
+  int filter_id = -1;
+  if (_x_preferences)
+    filter_id = _x_preferences->value(preferencePrefix() + "/filter_default").toInt();
   if (parent())
   {
-    QString classname(parent()->objectName());
-    if (classname.isEmpty())
-      classname = parent()->metaObject()->className();
-
-    qry.prepare(query);
+    XSqlQuery qry;
+    qry.prepare("SELECT filter_name FROM filter WHERE filter_id = :id;");
     qry.bindValue(":id", filter_id);
     qry.exec();
 
     if (qry.first())
     {
-      filter_name = qry.value("filter_name").toString();
-      setSavedFiltersIndex(filter_name);
+      setSavedFiltersIndex(qry.value("filter_name").toString());
       applySaved(0, filter_id);
     }
     else
@@ -333,17 +327,15 @@ void ParameterWidget::addParam()
 void ParameterWidget::append(QString pName, QString pParam, ParameterWidgetTypes pType, QVariant pDefault, bool pRequired, QString pExtraInfo)
 {
   if (DEBUG)
-    qDebug("%s::append(%s, %s, %d, %s, %d, %s) entered",
-           qPrintable(objectName()), qPrintable(pName), qPrintable(pParam),
-           pType, qPrintable(pDefault.toString()), pRequired,
-           qPrintable(pExtraInfo.left(50)));
+    qDebug() << objectName() << __FUNCTION__ << "() entered with"
+             << pName << pParam << pType << pDefault << pRequired
+             << pExtraInfo.left(50);
   ParamProps *pp = _params.value(_params.count(), 0);
 
   if (! pp)
   {
     if (DEBUG)
-      qDebug("%s::append() instantiating a new ParamProps",
-             qPrintable(objectName()));
+      qDebug() << objectName() << __FUNCTION__ << "instantiating a new ParamProps";
     pp = new ParamProps();
     pp->name = pName;
     pp->param = pParam;
@@ -364,8 +356,6 @@ void ParameterWidget::append(QString pName, QString pParam, ParameterWidgetTypes
                pType, qPrintable(pDefault.toString()),
                qPrintable(pExtraInfo));
   }
-
-  resetEmptyParameter();
 }
 
 /** @brief Add a new XComboBox-based filter to the set of available filters.
@@ -394,8 +384,6 @@ void ParameterWidget::appendComboBox(QString pName, QString pParam, int pType, Q
     pp->enabled = true;
     _params.insert(_params.count(), pp);
   }
-
-  resetEmptyParameter();
 }
 
 /** @brief Add a new XComboBox-based filter to the set of available filters.
@@ -424,35 +412,6 @@ void ParameterWidget::appendComboBox(QString pName, QString pParam, QString pQue
     pp->enabled = true;
     _params.insert(_params.count(), pp);
   }
-
-  resetEmptyParameter();
-}
-
-/**
-  If the last parameter that has been added has not been selected, then reset it.
-  */
-void ParameterWidget::resetEmptyParameter()
-{
-  if (_filtersLayout->rowCount() > 1)
-  {
-    int id;
-    int row;
-    QLayoutItem *item;
-    XComboBox *box;
-    QToolButton *btn;
-
-    row = _filtersLayout->rowCount() - 1;
-    item = _filtersLayout->itemAtPosition(row, 0)->layout()->itemAt(0);
-    box = (XComboBox*)item->widget();
-    id = box->id();
-
-    if (id < 0) {
-      QString btnName = "button" + QString().setNum(row);
-      btn = _filterGroup->findChild<QToolButton *>(btnName);
-      btn->click();
-      addParam();
-    }
-  }
 }
 
 void ParameterWidget::applySaved(int pId, int filter_id)
@@ -462,9 +421,7 @@ void ParameterWidget::applySaved(int pId, int filter_id)
   QString query;
   QString filterValue;
   QDate today = QDate::currentDate();
-  int xid, init_filter_id;
-
-  init_filter_id = filter_id;
+  int xid;
 
   clearFilters();
 
@@ -519,6 +476,8 @@ void ParameterWidget::applySaved(int pId, int filter_id)
     if ( !(tempFilter.isEmpty()) )
     {
       //0 is filterType, 1 is filterValue, 2 is parameterwidgettype
+      //Text fields may have : in them, in this case, the last value
+      // is parameterwidgettype, and filterValue spans all middle indices
       QStringList tempFilterList = tempFilter.split(":");
       QString key = this->getParameterTypeKey(tempFilterList[0]);
       if (key.isEmpty())
@@ -550,10 +509,10 @@ void ParameterWidget::applySaved(int pId, int filter_id)
         XComboBox *mybox = qobject_cast<XComboBox*>(test->widget());
         if (mybox)
           mybox->setCurrentIndex(mybox->findText(key));
-	
+
         found = getFilterWidget(windowIdx);
 
-        int widgetType = tempFilterList[2].toInt();
+        int widgetType = tempFilterList[tempFilterList.size()-1].toInt();
 
         switch (widgetType)
         {
@@ -724,7 +683,7 @@ void ParameterWidget::applySaved(int pId, int filter_id)
             QLineEdit *lineEdit = qobject_cast<QLineEdit*>(found);
             if (lineEdit != 0)
             {
-              lineEdit->setText(tempFilterList[1]);
+              lineEdit->setText(QStringList(tempFilterList.mid(1, tempFilterList.size()-2)).join(":"));
               storeFilterValue(-1, lineEdit);
             }
           }
@@ -945,8 +904,8 @@ void ParameterWidget::resetMultiselect(QTableWidgetItem *item)
     return;
 
   if (DEBUG)
-    qDebug("%s::resetMultiselect(%p) entered for %p",
-           qPrintable(objectName()), item, tab);
+    qDebug() << objectName() << __FUNCTION__ << "entered with"
+             << item << "for" << tab;
 
   disconnect(tab, SIGNAL(itemClicked(QTableWidgetItem*)), this, SLOT(resetMultiselect(QTableWidgetItem*)));
 
@@ -989,7 +948,8 @@ void ParameterWidget::changeFilterObject(int index)
 
   int pidx = paramIndex(mybox->currentText());
   if (DEBUG)
-    qDebug() << __FUNCTION__ << index << mybox << mybox->currentText() << pidx;
+    qDebug() << objectName() << __FUNCTION__ << index << mybox
+             << mybox->currentText() << pidx;
   if (pidx >= 0)
   {
     mybox->setDisabled(_params[pidx]->required);
@@ -1232,9 +1192,8 @@ void ParameterWidget::changeFilterObject(int index)
         item->setData(Qt::UserRole, msq.value(0));
         item->setFlags(item->flags() & (~ Qt::ItemIsEditable));
         if (DEBUG)
-          qDebug("%s::changeFilterObject() Multiselect added %s (%s) at %d",
-                 qPrintable(objectName()), qPrintable(item->text()),
-                 qPrintable(item->data(Qt::UserRole).toString()), i);
+          qDebug() << objectName() << __FUNCTION__ << "Multiselect added"
+                   << item->text() << item->data(Qt::UserRole) << "at" << i;
         tab->setItem(i, 0, item);
         tab->setRowHeight(i, (tab->fontMetrics().height() * 110) / 100); // TODO: a better way?
       }
@@ -1468,7 +1427,8 @@ void ParameterWidget::save()
   params.append("classname", classname);
   if (_shared)
     params.append("shared", true);
-
+  if (!_x_privileges->check("AllowSharedFilterEdit"))
+    params.append("disableshare", true);
   filterSave newdlg(this);
   newdlg.set(params);
   filter_id = newdlg.exec();
@@ -1536,11 +1496,8 @@ void ParameterWidget::setFiltersVisible(bool visible)
 
 void ParameterWidget::setFiltersVisabiltyPreference()
 {
-  _settingsName = _name + objectName();
-  if (!_settingsName.isEmpty() && _x_preferences)
-  {
-    _x_preferences->set(_settingsName + "/checked", _filterGroup->isVisible());
-  }
+  if (_x_preferences)
+    _x_preferences->set(preferencePrefix() + "/checked", _filterGroup->isVisible());
 }
 
 /*!
@@ -1548,18 +1505,9 @@ void ParameterWidget::setFiltersVisabiltyPreference()
 */
 void ParameterWidget::setFiltersDefault()
 {
-  QString pname;
-  int filter_id;
-
-  if(parent())
-    pname = parent()->metaObject()->className();
-  pname += "/";
-  _settingsName2 = pname + this->objectName();
-
-  filter_id = _filterList->id(_filterList->currentIndex());
-
-  if (!_settingsName2.isEmpty() && _x_preferences)
-    _x_preferences->set(_settingsName2 + "/filter_default", filter_id);
+  if (_x_preferences)
+    _x_preferences->set(preferencePrefix() + "/filter_default",
+                        _filterList->id(_filterList->currentIndex()));
 }
 
 /*!
@@ -1702,8 +1650,7 @@ QWidget *ParameterWidget::getFilterWidget(const int index)
 void ParameterWidget::storeFilterValue(int pId, QObject* filter)
 {
   if (DEBUG)
-    qDebug("%s::storeFilterValue(%d, %p)",
-           qPrintable(objectName()), pId, filter);
+    qDebug() << objectName() << __FUNCTION__ << "entered with" << pId << filter;
 
   QWidget *wfilter = qobject_cast<QWidget *>(filter);
   if (! wfilter)
@@ -1777,10 +1724,10 @@ void ParameterWidget::storeFilterValue(int pId, QObject* filter)
   _filterWidgets[foundRow] = wfilter;
 
   if (pp && ! newvalue.isNull())
-  {
     _filterValues[foundRow] = qMakePair(pp->param, newvalue);
-    emit updated();
-  }
+  else
+    _filterValues.remove(foundRow);
+  emit updated();
 
   if (!_usedTypes.isEmpty())
     _usedTypes.remove(foundRow);
@@ -1974,37 +1921,25 @@ QScriptValue constructParameterWidget(QScriptContext *context,
     obj = new ParameterWidget(qscriptvalue_cast<QWidget*>(context->argument(0)),
                               qPrintable(context->argument(1).toString()));
 
+#if QT_VERSION >= 0x050000
   return engine->toScriptValue(obj);
+#else
+  Q_UNUSED(engine); return QScriptValue();
+#endif
 }
 
 void setupParameterWidget(QScriptEngine *engine)
 {
-  qScriptRegisterMetaType(engine, ParameterWidgettoScriptValue, ParameterWidgetfromScriptValue);
+  if (! engine->globalObject().property("ParameterWidget").isFunction())
+  {
+#if QT_VERSION >= 0x050000
+    qScriptRegisterMetaType(engine, ParameterWidgettoScriptValue, ParameterWidgetfromScriptValue);
+#endif
 
-  QScriptValue widget = engine->newFunction(constructParameterWidget);
+    QScriptValue ctor = engine->newFunction(constructParameterWidget);
+    QScriptValue meta = engine->newQMetaObject(&ParameterWidget::staticMetaObject, ctor);
 
-  widget.setProperty("Crmacct", QScriptValue(engine, ParameterWidget::Crmacct), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  widget.setProperty("Customer", QScriptValue(engine, ParameterWidget::Customer), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  widget.setProperty("Shipto", QScriptValue(engine, ParameterWidget::Shipto), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  widget.setProperty("Vendor", QScriptValue(engine, ParameterWidget::Vendor), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  widget.setProperty("User", QScriptValue(engine, ParameterWidget::User), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  widget.setProperty("Text", QScriptValue(engine, ParameterWidget::Text), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  widget.setProperty("Date", QScriptValue(engine, ParameterWidget::Date), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  widget.setProperty("XComBox", QScriptValue(engine, ParameterWidget::XComBox), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  widget.setProperty("Contact", QScriptValue(engine, ParameterWidget::Contact), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  widget.setProperty("GLAccount", QScriptValue(engine, ParameterWidget::GLAccount), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  widget.setProperty("Multiselect", QScriptValue(engine, ParameterWidget::Multiselect), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  widget.setProperty("Exists", QScriptValue(engine, ParameterWidget::Exists), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  widget.setProperty("CheckBox", QScriptValue(engine, ParameterWidget::CheckBox), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  widget.setProperty("Project", QScriptValue(engine, ParameterWidget::Project), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  widget.setProperty("Item", QScriptValue(engine, ParameterWidget::Item), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  widget.setProperty("Employee", QScriptValue(engine, ParameterWidget::Employee), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  widget.setProperty("Site", QScriptValue(engine, ParameterWidget::Site), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  widget.setProperty("SalesOrder", QScriptValue(engine, ParameterWidget::SalesOrder), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  widget.setProperty("WorkOrder", QScriptValue(engine, ParameterWidget::WorkOrder), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  widget.setProperty("PurchaseOrder", QScriptValue(engine, ParameterWidget::PurchaseOrder), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  widget.setProperty("TransferOrder", QScriptValue(engine, ParameterWidget::TransferOrder), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-
-  engine->globalObject().setProperty("ParameterWidget", widget, QScriptValue::ReadOnly | QScriptValue::Undeletable);
+    engine->globalObject().setProperty("ParameterWidget", meta,
+                                       QScriptValue::ReadOnly | QScriptValue::Undeletable);
+  }
 }
-
