@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -11,6 +11,10 @@
 #include "shipmentcluster.h"
 
 #include <QMessageBox>
+#include <QtScript>
+
+#include "xcheckbox.h"
+#include "xtreewidget.h"
 
 ShipmentCluster::ShipmentCluster(QWidget* pParent, const char* pName) :
     VirtualCluster(pParent, pName)
@@ -60,7 +64,7 @@ void ShipmentCluster::limitToOrder(const int head_id)
 //    removeOrderLimit();
 }
 
-void ShipmentCluster::setId(const int pid)
+void ShipmentCluster::setId(const int pid, const QString&)
 {
 //  removeOrderLimit();
   static_cast<ShipmentClusterLineEdit*>(_number)->setId(pid);
@@ -243,3 +247,17 @@ ShipmentSearch::ShipmentSearch(QWidget* pParent, Qt::WindowFlags pFlags) :
   _searchName->setText(tr("Search through Shipped Date"));
   _searchDescrip->setText(tr("Search through Tracking Number"));
 }
+
+// script api //////////////////////////////////////////////////////////////////
+
+void setupShipmentClusterLineEdit(QScriptEngine *engine)
+{
+  if (! engine->globalObject().property("ShipmentClusterLineEdit").isObject())
+  {
+    QScriptValue ctor = engine->newObject(); //engine->newFunction(scriptconstructor);
+    QScriptValue meta = engine->newQMetaObject(&ShipmentClusterLineEdit::staticMetaObject, ctor);
+
+    engine->globalObject().setProperty("ShipmentClusterLineEdit", meta,
+                                       QScriptValue::ReadOnly | QScriptValue::Undeletable);
+  }
+}
