@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -19,6 +19,8 @@
 
 #include "parameter.h"
 
+#include <xtuplecommon.h>
+
 #ifdef Q_OS_WIN
   #ifdef MAKEDLL
     #define XTUPLEWIDGETS_EXPORT __declspec(dllexport)
@@ -29,11 +31,17 @@
   #define XTUPLEWIDGETS_EXPORT
 #endif
 
-class Preferences;
+#ifndef Q_ENUM  /* introduced in Qt 5.5 */
+#define Q_ENUM(ENUMERATION) Q_ENUMS(ENUMERATION)
+#endif
+
 class Metrics;
-class QMdiArea;
+class Preferences;
 class Privileges;
+class QMdiArea;
+class QScriptEngine;
 class QWidget;
+class GuiClientInterface;
 
 extern Preferences *_x_preferences;
 extern Metrics     *_x_metrics;
@@ -41,5 +49,7 @@ extern QMdiArea    *_x_workspace;
 extern Privileges  *_x_privileges;
 extern QString     _x_username;
 
+void setupWidgetsScriptApi(QScriptEngine *engine, GuiClientInterface *client = 0);
+
 #endif
 
