@@ -1,54 +1,42 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
  * to be bound by its terms.
  */
 
-#include <QLabel>
-#include <QPushButton>
-#include <QLayout>
+#include "wocluster.h"
+
+#include <QCompleter>
 #include <QHBoxLayout>
+#include <QLabel>
+#include <QMessageBox>
+#include <QSqlQueryModel>
 #include <QVBoxLayout>
 #include <QtScript>
 
 #include <parameter.h>
 #include <xsqlquery.h>
-#include <QMessageBox>
 
+#include "format.h"
 #include "warehousegroup.h"
+#include "xcheckbox.h"
 #include "xcombobox.h"
 #include "xlineedit.h"
-
-#include "wocluster.h"
-
-#include "../common/format.h"
-
-void setupWoCluster(QScriptEngine *engine)
-{
-  QScriptValue widget = engine->newObject();
-
-  widget.setProperty("Open", QScriptValue(engine, WoLineEdit::Open), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  widget.setProperty("Exploded", QScriptValue(engine, WoLineEdit::Exploded), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  widget.setProperty("Issued", QScriptValue(engine, WoLineEdit::Issued), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  widget.setProperty("Released", QScriptValue(engine, WoLineEdit::Released), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  widget.setProperty("Closed", QScriptValue(engine, WoLineEdit::Closed), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-
-  engine->globalObject().setProperty("WoLineEdit", widget, QScriptValue::ReadOnly | QScriptValue::Undeletable);
-}
+#include "xtreewidget.h"
 
 WoLineEdit::WoLineEdit(QWidget *pParent, const char *pName) :
-    VirtualClusterLineEdit(pParent, "wo", "wo_id", "wo_number::text || '-' || wo_subnumber::text", "item_number", "(item_descrip1 || ' ' || item_descrip2) ", 0, pName)
+    VirtualClusterLineEdit(pParent, "wo", "wo_id", "formatWoNumber(wo_id)", "item_number", "(item_descrip1 || ' ' || item_descrip2) ", 0, pName)
 {
   _woType = 0;
   init();
 }
 
 WoLineEdit::WoLineEdit(int pWoType, QWidget* pParent, const char* pName) :
-    VirtualClusterLineEdit(pParent, "wo", "wo_id", "wo_number::text || '-' || wo_subnumber::text", "item_number", "(item_descrip1 || ' ' || item_descrip2) ", 0, pName)
+    VirtualClusterLineEdit(pParent, "wo", "wo_id", "formatWoNumber(wo_id)", "item_number", "(item_descrip1 || ' ' || item_descrip2) ", 0, pName)
 {
   _woType = pWoType;
   init();
@@ -68,7 +56,7 @@ void WoLineEdit::init()
   _qtyReceived = 0.0;
 
   _query =  QString("SELECT wo_id AS id, "
-                    "       wo_number::text || '-' || wo_subnumber::text AS number,"
+                    "       formatWoNumber(wo_id) AS number,"
                     "       item_number AS name, "
                     "       (item_descrip1 || ' ' || item_descrip2) AS description, "
                     "       warehous_id,"
@@ -157,13 +145,18 @@ void WoLineEdit::silentSetId(const int pId)
   }
   else
   {
-    _id    = -1;
-    _valid = false;
+    clear();
+  }
+
+  _parsed = true;
+  emit parsed();
+}
+
+void WoLineEdit::clear()
+{
+    VirtualClusterLineEdit::clear();
     _currentWarehouseid = -1;
 
-    XLineEdit::clear();
-
-    emit newId(-1);
     emit newItemid(-1);
     emit warehouseChanged("");
     emit itemNumberChanged("");
@@ -177,14 +170,9 @@ void WoLineEdit::silentSetId(const int pId)
     emit qtyBalanceChanged(0);
     emit statusChanged("");
     emit methodChanged("");
-    emit valid(false);
-      
+
     _qtyOrdered  = 0;
     _qtyReceived = 0;
-  }
-
-  _parsed = true;
-  emit parsed();
 }
 
 void WoLineEdit::sList()
@@ -350,14 +338,14 @@ void WoCluster::constructor()
   _line2Layout->setContentsMargins(0, 0, 0, 0);
   _statusLayout->setContentsMargins(0, 0, 0, 0);
 
-  _mainLayout->setSpacing(2); 
-  _woLayout->setSpacing(5); 
-  _warehouseLayout->setSpacing(5); 
-  _line1Layout->setSpacing(7); 
-  _itemLayout->setSpacing(5); 
-  _uomLayout->setSpacing(5); 
-  _line2Layout->setSpacing(7); 
-  _statusLayout->setSpacing(5); 
+  _mainLayout->setSpacing(2);
+  _woLayout->setSpacing(5);
+  _warehouseLayout->setSpacing(5);
+  _line1Layout->setSpacing(7);
+  _itemLayout->setSpacing(5);
+  _uomLayout->setSpacing(5);
+  _line2Layout->setSpacing(7);
+  _statusLayout->setSpacing(5);
 
   _woNumberLit = new QLabel(tr("Work Order #:"), this);
   _woNumberLit->setObjectName("woNumberLit");
@@ -491,7 +479,7 @@ void WoCluster::setWoNumber(const QString& number)
 {
   if (_woNumber->text() == number)
     return;
-    
+
   _woNumber->setText(number);
   _woNumber->sParse();
 }
@@ -680,7 +668,7 @@ void WomatlCluster::setWooperid(int pWooperid)
     _valid = false;
     _required = 0.0;
     _issued  = 0.0;
-    
+
     emit newId(-1);
     emit newQtyRequired(0.0);
     emit newQtyIssued(0.0);
@@ -764,7 +752,7 @@ void WomatlCluster::setWoid(int pWoid)
     _valid = false;
     _required = 0.0;
     _issued  = 0.0;
-    
+
     emit newId(-1);
     emit newQtyRequired(0.0);
     emit newQtyIssued(0.0);
@@ -1124,3 +1112,29 @@ void woSearch::sFillList()
   _listTab->populate(wo, _woid);
 }
 
+// script api //////////////////////////////////////////////////////////////////
+
+void setupWoCluster(QScriptEngine *engine)
+{
+  QScriptValue widget = engine->newObject();
+
+  widget.setProperty("Open", QScriptValue(engine, WoLineEdit::Open), QScriptValue::ReadOnly | QScriptValue::Undeletable);
+  widget.setProperty("Exploded", QScriptValue(engine, WoLineEdit::Exploded), QScriptValue::ReadOnly | QScriptValue::Undeletable);
+  widget.setProperty("Issued", QScriptValue(engine, WoLineEdit::Issued), QScriptValue::ReadOnly | QScriptValue::Undeletable);
+  widget.setProperty("Released", QScriptValue(engine, WoLineEdit::Released), QScriptValue::ReadOnly | QScriptValue::Undeletable);
+  widget.setProperty("Closed", QScriptValue(engine, WoLineEdit::Closed), QScriptValue::ReadOnly | QScriptValue::Undeletable);
+
+  engine->globalObject().setProperty("WoLineEdit", widget, QScriptValue::ReadOnly | QScriptValue::Undeletable);
+}
+
+void setupWomatlCluster(QScriptEngine *engine)
+{
+  if (! engine->globalObject().property("WomatlCluster").isObject())
+  {
+    QScriptValue ctor = engine->newObject(); //engine->newFunction(scriptconstructor);
+    QScriptValue meta = engine->newQMetaObject(&WomatlCluster::staticMetaObject, ctor);
+
+    engine->globalObject().setProperty("WomatlCluster", meta,
+                                       QScriptValue::ReadOnly | QScriptValue::Undeletable);
+  }
+}
