@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -18,9 +18,8 @@
 #include <QSqlField>
 
 #include "datecluster.h"
-#include "xsqltablemodel.h"
 
-#define DEBUG true
+#define DEBUG false
 
 XItemDelegate::XItemDelegate(QObject *parent)
   : QItemDelegate(parent)
