@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -109,8 +109,6 @@ XTreeWidget::XTreeWidget(QWidget *pParent) :
   _menu    = new QMenu(this);
   _menu->setObjectName("_menu");
   _savedId = false; // was -1;
-  _scol    = -1;
-  _sord    = Qt::AscendingOrder;
   _linear  = false;
   _alwaysLinear = true;
 
@@ -127,11 +125,7 @@ XTreeWidget::XTreeWidget(QWidget *pParent) :
   setContextMenuPolicy(Qt::CustomContextMenu);
   setSelectionBehavior(QAbstractItemView::SelectRows);
   header()->setStretchLastSection(false);
-#if QT_VERSION >= 0x050000
   header()->setSectionsClickable(true);
-#else
-  header()->setClickable(true);
-#endif
   if (_x_preferences)
     setAlternatingRowColors(!_x_preferences->boolean("NoAlternatingRowColors"));
 
@@ -164,12 +158,6 @@ XTreeWidget::XTreeWidget(QWidget *pParent) :
   connect(menuAct, SIGNAL(triggered()), this, SLOT(sShowMenu()));
   addAction(menuAct);
 
-  QAction* itemSelectedAct = new QAction(this);
-  itemSelectedAct->setShortcut(QKeySequence(QKeySequence::InsertParagraphSeparator));
-  itemSelectedAct->setShortcutContext(Qt::WidgetShortcut);
-  connect(itemSelectedAct, SIGNAL(triggered()), this, SLOT(sItemSelected()));
-  addAction(itemSelectedAct);
-
 }
 
 XTreeWidget::~XTreeWidget()
@@ -207,12 +195,19 @@ XTreeWidget::~XTreeWidget()
       }
       xtsettingsSetValue(_settingsName + "/columnWidths", savedString);
     }
-    if (!_forgetfulOrder && header()->isSortIndicatorShown())
-      savedString = QString::number(header()->sortIndicatorSection()) + " "                                                        +
-                    (header()->sortIndicatorOrder() == Qt::AscendingOrder ? "ASC" : "DESC" );
+    if (!_forgetfulOrder && !_sort.isEmpty())
+    {
+      savedString = "";
+      QPair<int, Qt::SortOrder> sort;
+      foreach (sort, _sort)
+      {
+        savedString += QString::number(sort.first) + " " +
+                       (sort.second == Qt::AscendingOrder ? "ASC" : "DESC") + "|";
+      }
+    }
     else
       savedString = "-1,ASC";
-      xtsettingsSetValue(_settingsName + "/sortOrder", savedString);
+    xtsettingsSetValue(_settingsName + "/sortOrder", savedString);
   }
 
   for (int i = 0; i < _roles.size(); i++)
@@ -762,7 +757,7 @@ void XTreeWidget::populateWorker()
     cleanupAfterPopulate();
 
     populateCalculatedColumns();
-    if (sortColumn() >= 0 && header()->isSortIndicatorShown())
+    if (!_sort.isEmpty())
       sortItems(sortColumn(), header()->sortIndicatorOrder());
 
     if (DEBUG)
@@ -849,15 +844,16 @@ void XTreeWidget::addColumn(const QString &pString, int pWidth, int pAlignment,
     }
     if (!_forgetfulOrder)
     {
-      part = xtsettingsValue(_settingsName + "/sortOrder", "-1,ASC").toString();
-      key  = part.left(part.indexOf(" "));
-      val  = part.right(part.length() - part.indexOf(" ") - 1);
-      b1   = false;
-      int k = key.toInt(&b1);
-      if (b1)
+      savedString = xtsettingsValue(_settingsName + "/sortOrder", "-1,ASC").toString();
+      savedParts  = savedString.split("|", QString::SkipEmptyParts);
+      foreach (part, savedParts)
       {
-        _scol  = k;
-        _sord  = ("ASC" == val ? Qt::AscendingOrder : Qt::DescendingOrder);
+        key  = part.left(part.indexOf(" "));
+        val  = part.right(part.length() - part.indexOf(" ") - 1);
+        b1   = false;
+        int k = key.toInt(&b1);
+        if (b1)
+          _sort.append(qMakePair(k, "ASC" == val ? Qt::AscendingOrder : Qt::DescendingOrder));
       }
     }
 
@@ -884,7 +880,7 @@ void XTreeWidget::addColumn(const QString &pString, int pWidth, int pAlignment,
   setColumnCount(column + 1);
 
   QTreeWidgetItem *hitem = headerItem();
-  hitem->setText(column, pString);
+  hitem->setText(column, "   " + pString);
   hitem->setTextAlignment(column, pAlignment);
   hitem->setData(column, Xt::ScaleRole, scale);
 
@@ -904,30 +900,29 @@ void XTreeWidget::addColumn(const QString &pString, int pWidth, int pAlignment,
   if (pWidth >= 0)
   {
     header()->resizeSection(column, pWidth);
-    #if QT_VERSION >= 0x050000
     header()->setSectionResizeMode(column, QHeaderView::Interactive);
-    #else
-    header()->setResizeMode(column, QHeaderView::Interactive);
-    #endif
   }
   else
   {
-    #if QT_VERSION >= 0x050000
     header()->setSectionResizeMode(column, QHeaderView::Interactive);
-    #else
-    header()->setResizeMode(column, QHeaderView::Interactive);
-    #endif
     _stretch.append(column);
   }
   bool forgetCache = _forgetful;
   _forgetful = true;
   setColumnVisible(column, _savedVisibleColumns.value(column, pVisible));
   _forgetful = forgetCache;
-  if (_scol >= 0 && column == _scol)
+  if (_sort.contains(qMakePair(column, Qt::AscendingOrder)) ||
+      _sort.contains(qMakePair(column, Qt::DescendingOrder)))
   {
-    if (!header()->isSortIndicatorShown())
-      header()->setSortIndicatorShown(true);
-    sortItems(_scol, _sord);
+    if (_roles.size() <= 0 && _sort[0].first == column)
+    {
+      if (!header()->isSortIndicatorShown())
+        header()->setSortIndicatorShown(true);
+
+      sortItems(_sort[0].first, _sort[0].second);
+    }
+    if (_roles.size() > 0)
+      sortItems();
   }
 }
 
@@ -945,6 +940,14 @@ int XTreeWidget::column(const QString pName) const
   return colIdx;
 }
 
+QString XTreeWidget::column(const int pIndex) const
+{
+  if (pIndex >= 0 && pIndex < _roles.size())
+    return _roles.value(pIndex)->value("qteditrole").toString();
+  else
+    return QString();
+}
+
 XTreeWidgetItem *XTreeWidget::currentItem() const
 {
   return (XTreeWidgetItem *)QTreeWidget::currentItem();
@@ -978,104 +981,118 @@ QString XTreeWidgetItem::toString() const
 
 bool XTreeWidgetItem::operator<(const XTreeWidgetItem &other) const
 {
-  QVariant  v1         = data(treeWidget()->sortColumn(), Xt::RawRole);
-  QVariant  v2         = other.data(other.treeWidget()->sortColumn(), Xt::RawRole);
+  bool returnVal = false;
+  bool sorted    = false;
 
-  bool      returnVal  = false;
-  switch (v1.type())
+  QPair<int, Qt::SortOrder> sort;
+  foreach (sort, ((XTreeWidget*)treeWidget())->sortColumnOrder())
   {
-    case QVariant::Bool:
-      returnVal = (!v1.toBool() && v1 != v2);
+    if (sort.first < 0 || sort.first >= columnCount() ||
+        treeWidget()->headerItem()->data(sort.first, Qt::UserRole).toString() == "xtrunningrole")
       break;
 
-    case QVariant::Date:
-      returnVal = (v1.toDate() < v2.toDate());
-      break;
+    QVariant v1 = data(sort.first, Xt::RawRole);
+    QVariant v2 = other.data(sort.first, Xt::RawRole);
 
-    case QVariant::DateTime:
-      returnVal = (v1.toDateTime() < v2.toDateTime());
-      break;
+    // bugs 17968 & 32496: sort strings according to user expectations AND preserve raw role [
+    bool ok1, ok2;
+    (void)v1.toString().toDouble(&ok1); // we want ok1 & ok2, not the numeric value
+    (void)v2.toString().toDouble(&ok2);
 
-    case QVariant::Double:
-      returnVal = (v1.toDouble() < v2.toDouble());
-      break;
+    if (v1.type() == QVariant::String && ! ok1 && v2.type() == QVariant::String && ! ok2)
+    {
+      QVariant d1 = data(sort.first,       Qt::DisplayRole);
+      QVariant d2 = other.data(sort.first, Qt::DisplayRole);
 
-    case QVariant::Int:
-      returnVal = (v1.toInt() < v2.toInt());
-      break;
+      (void)d1.toString().toDouble(&ok1);
+      (void)d2.toString().toDouble(&ok2);
 
-    case QVariant::LongLong:
-      returnVal = (v1.toLongLong() < v2.toLongLong());
-      break;
+      if (d1.type() == QVariant::String && ! ok1 && d2.type() == QVariant::String && ! ok2)
+      {
+        v1 = d1;
+        v2 = d2;
+      }
+    } // ] end 17968/32496
 
-    case QVariant::String:
-      bool ok;
-      if (v1.toString().toDouble() == 0.0 && v2.toDouble() == 0.0)
-        returnVal = (v1.toString() < v2.toString());
-      else if (v1.toString().toDouble() == 0.0 && v2.toDouble(&ok)) //v1 is string, v2 is number
-        returnVal = false; //the number should always be treated as greater than a string
-      else if (v1.toDouble(&ok) && v2.toString().toDouble() == 0.0)
-        returnVal = true;
-      else
+    if (sorted || v1==v2)
+      continue;
+
+    switch (v1.type())
+    {
+      case QVariant::Bool:
+        sorted = true;
+        returnVal = !v1.toBool();
+        break;
+
+      case QVariant::Date:
+        sorted = true;
+        returnVal = (v1.toDate() < v2.toDate());
+        break;
+
+      case QVariant::DateTime:
+        sorted = true;
+        returnVal = (v1.toDateTime() < v2.toDateTime());
+        break;
+
+      case QVariant::Double:
+        sorted = true;
         returnVal = (v1.toDouble() < v2.toDouble());
-      break;
+        break;
 
-    default:
-      returnVal = false;
+      case QVariant::Int:
+        sorted = true;
+        returnVal = (v1.toInt() < v2.toInt());
+        break;
+
+      case QVariant::LongLong:
+        sorted = true;
+        returnVal = (v1.toLongLong() < v2.toLongLong());
+        break;
+
+      case QVariant::String:
+        sorted = true;
+        if (v1.toString().toDouble() == 0.0 && v2.toDouble() == 0.0)
+          returnVal = (v1.toString() < v2.toString());
+        else if (v1.toString().toDouble() == 0.0 && v2.toDouble(&ok2)) //v1 is string, v2 is number
+          returnVal = false; //the number should always be treated as greater than a string
+        else if (v1.toDouble(&ok1) && v2.toString().toDouble() == 0.0)
+          returnVal = true;
+        else
+          returnVal = (v1.toDouble() < v2.toDouble());
+        break;
+
+      default:
+        break;
+    }
+
+    if (sort.second==Qt::DescendingOrder)
+      returnVal = !returnVal;
+
+    if (DEBUG)
+      qDebug("returning %d for %s < %s", returnVal,
+             qPrintable(v1.toString()), qPrintable(v2.toString()));
   }
 
-  if (DEBUG)
-    qDebug("returning %d for %s < %s", returnVal,
-           qPrintable(v1.toString()), qPrintable(v2.toString()));
   return returnVal;
 }
 
 bool XTreeWidgetItem::operator==(const XTreeWidgetItem &other) const
 {
-  QVariant  v1         = data(treeWidget()->sortColumn(), Xt::RawRole);
-  QVariant  v2         = other.data(other.treeWidget()->sortColumn(), Xt::RawRole);
-
-  bool      returnVal  = false;
-  switch (v1.type())
+  bool returnVal = true;
+  QPair<int, Qt::SortOrder> sort;
+  foreach (sort, ((XTreeWidget*)treeWidget())->sortColumnOrder())
   {
-    case QVariant::Bool:
-      returnVal = (v1.toBool() && v1 == v2);
-      break;
+    QVariant v1 = data(sort.first, Xt::RawRole);
+    QVariant v2 = other.data(sort.first, Xt::RawRole);
 
-    case QVariant::Date:
-      returnVal = (v1.toDate() == v2.toDate());
-      break;
-
-    case QVariant::DateTime:
-      returnVal = (v1.toDateTime() == v2.toDateTime());
-      break;
-
-    case QVariant::Double:
-      returnVal = (v1.toDouble() == v2.toDouble());
-      break;
-
-    case QVariant::Int:
-      returnVal = (v1.toInt() == v2.toInt());
-      break;
-
-    case QVariant::LongLong:
-      returnVal = (v1.toLongLong() == v2.toLongLong());
-      break;
-
-    case QVariant::String:
-      if (v1.toString().toDouble() == 0.0 && v2.toDouble() == 0.0)
-        returnVal = (v1.toString() == v2.toString());
-      else
-        returnVal = (v1.toDouble() == v2.toDouble());
-      break;
-
-    default:
+    if (v1!=v2)
       returnVal = false;
+
+    if (DEBUG)
+      qDebug("returning %d for %s == %s", returnVal,
+             qPrintable(v1.toString()), qPrintable(v2.toString()));
   }
 
-  if (DEBUG)
-    qDebug("returning %d for %s == %s", returnVal,
-           qPrintable(v1.toString()), qPrintable(v2.toString()));
   return returnVal;
 }
 
@@ -1085,10 +1102,50 @@ bool XTreeWidgetItem::operator>(const XTreeWidgetItem &other) const
   return !(this < other || this == other);
 }
 */
+void XTreeWidget::mergeSort(int low, int high){
+    int mid;
+    if(low<high){
+        mid=(low+high)/2;
+        // Split the data into two half.
+        mergeSort(low, mid);
+        mergeSort(mid+1, high);
+
+        // Merge them to get sorted output.
+        merge(low, high, mid);
+    }
+}
+void XTreeWidget::merge(int low, int high, int mid){
+
+    QList<QTreeWidgetItem *> temp;
+    int i = low, j = mid + 1;//i is for left-hand,j is for right-hand
+    int k = 0;//k is for the temporary array
+
+   while(i <= mid && j <=high){
+        XTreeWidgetItem *lhsItem = static_cast<XTreeWidgetItem *>(topLevelItem(i));
+        XTreeWidgetItem *rhsItem = static_cast<XTreeWidgetItem *>(topLevelItem(j));
+
+        if(*lhsItem < *rhsItem || *lhsItem == *rhsItem)
+            temp.insert(k++, static_cast<XTreeWidgetItem *>(topLevelItem(i++)->clone()));
+        else
+            temp.insert(k++, static_cast<XTreeWidgetItem *>(topLevelItem(j++)->clone()));
+    }
+    //rest elements of left-half
+    while(i <= mid)
+        temp.insert(k++, static_cast<XTreeWidgetItem *>(topLevelItem(i++)->clone()));
+    //rest elements of right-half
+    while(j <= high)
+        temp.insert(k++, static_cast<XTreeWidgetItem *>(topLevelItem(j++)->clone()));
+
+    //copy the mergered temporary array to the original array
+    for(k = 0, i = low; i <= high; ++i, ++k){
+        QTreeWidgetItem *item =  topLevelItem(i);
+        //XTreeWidgetItem *delItem = (XTreeWidgetItem*)item;
+        *item = *temp.at(k);
+
+    }
+}
 void XTreeWidget::sortItems(int column, Qt::SortOrder order)
 {
-  int previd = id();
-
   // if old style then maintain backwards compatibility
   if (_roles.size() <= 0)
   {
@@ -1096,15 +1153,47 @@ void XTreeWidget::sortItems(int column, Qt::SortOrder order)
     return;
   }
 
-  if (column < 0 || column >= columnCount() ||
-      headerItem()->data(column, Qt::UserRole).toString() == "xtrunningrole")
+  for (int column=0; column<columnCount(); column++)
+  {
+    if (!QRegExp("[\\d ][\\d ] ").exactMatch(headerItem()->text(column).left(3)))
+      headerItem()->setText(column, "   " + headerItem()->text(column));
+    headerItem()->setIcon(column, QIcon());
+    headerItem()->setText(column, "   " + headerItem()->text(column).mid(3));
+  }
+
+  bool sortable = false;
+  int priority = 0;
+  QPair<int, Qt::SortOrder> sort;
+  foreach (sort, _sort)
+  {
+    if (sort.first < 0 || sort.first >= columnCount() ||
+        headerItem()->data(sort.first, Qt::UserRole).toString() == "xtrunningrole")
+      continue;
+    sortable = true;
+    priority++;
+
+    if (sort.second == Qt::AscendingOrder)
+      headerItem()->setIcon(sort.first, QIcon(QPixmap(":/widgets/images/arrow_down.png")));
+    else
+      headerItem()->setIcon(sort.first, QIcon(QPixmap(":/widgets/images/arrow_up.png")));
+
+    if (_sort.size()>1)
+      headerItem()->setText(sort.first, (priority < 10 ? " " : "") + QString::number(priority) +
+                                        " " + headerItem()->text(sort.first).mid(3));
+  }
+
+  if (!sortable)
     return;
 
-  header()->setSortIndicator(column, order);
+  int previd = id();
 
   // simple insertion sort using binary search to find the right insertion pt
   QString totalrole("totalrole");
   int     itemcount      = topLevelItemCount();
+   
+#ifdef USENEWSORT
+  mergeSort(0,itemcount-1);
+#else
   XTreeWidgetItem *prev  = dynamic_cast<XTreeWidgetItem *>(topLevelItem(0));
   for (int i = 1; i < itemcount; i++)
   {
@@ -1124,7 +1213,7 @@ void XTreeWidget::sortItems(int column, Qt::SortOrder order)
       itemcount--;
       i--;
     }
-    else if (*item < *prev && order == Qt::AscendingOrder)
+    else if (*item < *prev)
     {
       int left   = 0;
       int right  = i;
@@ -1166,54 +1255,21 @@ void XTreeWidget::sortItems(int column, Qt::SortOrder order)
     {
       ; // nothing to do - make the > case easier to write
     }
-    else if (!(*item < *prev) && order == Qt::DescendingOrder)
-    {
-      int left   = 0;
-      int right  = i;
-      int middle = 0;
-      XTreeWidgetItem *test = 0;
-      while (left <= right)
-      {
-        middle = (left + right) / 2;
-        test   = static_cast<XTreeWidgetItem *>(topLevelItem(middle));
-        if (*test == *item)
-          break;
-        else if (!(*test < *item))
-        {
-          if (!(*item < *(static_cast<XTreeWidgetItem *>(topLevelItem(middle + 1)))))
-            break;
-          else
-            left = middle + 1;
-        }
-        else
-          right = middle - 1;
-      }
-      // can't call takeTopLevelItem() until after < and == are done
-      if (!(*item < *test) || *item == *test)
-      {
-        if (DEBUG)
-          qDebug(">= so moving %d to %d", i, middle);
-        takeTopLevelItem(i);
-        insertTopLevelItem(middle, item);
-      }
-      else
-      {
-        if (DEBUG)
-          qDebug("< so moving %d to %d", i, middle + 1);
-        takeTopLevelItem(i);
-        insertTopLevelItem(middle + 1, item);
-      }
-    }
     // can't reuse item because the thing in position i may have changed
     prev = static_cast<XTreeWidgetItem *>(topLevelItem(i));
   }
-
+#endif
   populateCalculatedColumns();
 
   setId(previd);
   emit resorted();
 }
 
+QList<QPair<int, Qt::SortOrder> > XTreeWidget::sortColumnOrder()
+{
+  return _sort;
+}
+
 void XTreeWidget::populateCalculatedColumns()
 {
   QMap<int, QMap<int, double> > totals; // <col <totalset, subtotal> >
@@ -1502,6 +1558,7 @@ void XTreeWidget::sShowMenu(const QPoint &pntThis)
       copyMenu->addAction(tr("All"),  this, SLOT(sCopyVisibleToClipboard()));
       copyMenu->addAction(tr("Row"),  this, SLOT(sCopyRowToClipboard()));
       copyMenu->addAction(tr("Cell"),  this, SLOT(sCopyCellToClipboard()));
+      copyMenu->addAction(tr("Column"),this, SLOT(sCopyColumnToClipboard()));
       _menu->addSeparator();
       _menu->addAction(tr("Export As..."),  this, SLOT(sExport()));
     }
@@ -1657,10 +1714,42 @@ void XTreeWidget::mouseMoveEvent(QMouseEvent *event)
 
 void XTreeWidget::sHeaderClicked(int column)
 {
-  // Qt::SortOrder sortOrder = Qt::DescendingOrder;
-  if (!header()->isSortIndicatorShown())
-    header()->setSortIndicatorShown(true);
-  sortItems(column, header()->sortIndicatorOrder());
+  if (_roles.size() <= 0)
+  {
+    _sort.clear();
+    _sort.append(qMakePair(column, header()->sortIndicatorOrder()));
+
+    if (!header()->isSortIndicatorShown())
+      header()->setSortIndicatorShown(true);
+
+    sortItems(column, header()->sortIndicatorOrder());
+    return;
+  }
+
+  bool multisort = QGuiApplication::keyboardModifiers() == Qt::ShiftModifier;
+
+  int sort = -1;
+  for (int i=0; i<_sort.size(); i++)
+    if (_sort.at(i).first==column)
+      sort = i;
+
+  if (!multisort && sort==-1)
+    _sort.clear();
+  if (multisort && sort!=-1)
+  {
+    _sort.removeAt(sort);
+    sortItems();
+    return;
+  }
+
+  if (sort==-1)
+    _sort.append(qMakePair(column, Qt::AscendingOrder));
+  else if (_sort.at(sort).second==Qt::DescendingOrder)
+    _sort[sort].second = Qt::AscendingOrder;
+  else
+    _sort[sort].second = Qt::DescendingOrder;
+
+  sortItems();
 }
 
 void XTreeWidget::sColumnSizeChanged(int logicalIndex, int /*oldSize*/, int /*newSize*/)
@@ -1974,6 +2063,38 @@ QVariant XTreeWidgetItem::rawValue(const QString pName)
     return data(colIdx, Xt::RawRole);
 }
 
+void XTreeWidgetItem::setNumber(int pColumn, const QVariant pValue, const QString pNumericRole)
+{
+  setData(pColumn, Xt::RawRole, pValue);
+  if (!setNumericRole(pColumn, pNumericRole))
+  {
+    // pValue is not a valid number
+    setText(pColumn, pValue);
+  }
+}
+
+void XTreeWidgetItem::setDate(int pColumn, const QVariant pDate)
+{
+  setData(pColumn, Xt::RawRole, pDate);
+  setData(pColumn, Qt::DisplayRole, formatDate(pDate.toDate()));
+}
+
+bool XTreeWidgetItem::setNumericRole(int pColIdx, const QString pRole)
+{
+  bool canConvert = false;
+
+  if (pColIdx >= 0)
+  {
+    double value = data(pColIdx, Xt::RawRole).toDouble(&canConvert);
+
+    if (canConvert)
+    {
+      setData(pColIdx, Qt::DisplayRole, QLocale().toString(value, 'f', decimalPlaces(pRole)));
+    }
+  }
+  return canConvert;
+}
+
 /* Calculate the total for a particular XTreeWidgetItem, including any children.
    pcol is the column for which we want the total.
    prole is the value of xttotalrole for which we want the total.
@@ -1990,17 +2111,19 @@ double XTreeWidgetItem::totalForItem(const int pcol, const int pset) const
   return total;
 }
 
-QScriptValue XTreeWidgetItemtoScriptValue(QScriptEngine *engine, XTreeWidgetItem *const &item)
+// script exposure of xtreewidgetitem /////////////////////////////////////////
+
+QScriptValue XTreeWidgetItemToScriptValue(QScriptEngine *engine, XTreeWidgetItem *const &item)
 {
   return engine->newQObject(item);
 }
 
-void XTreeWidgetItemfromScriptValue(const QScriptValue &obj, XTreeWidgetItem * &item)
+void XTreeWidgetItemFromScriptValue(const QScriptValue &obj, XTreeWidgetItem * &item)
 {
   item = qobject_cast<XTreeWidgetItem *>(obj.toQObject());
 }
 
-QScriptValue XTreeWidgetItemListtoScriptValue(QScriptEngine *engine, QList<XTreeWidgetItem *> const &cpplist)
+QScriptValue XTreeWidgetItemListToScriptValue(QScriptEngine *engine, QList<XTreeWidgetItem *> const &cpplist)
 {
   QScriptValue scriptlist = engine->newArray(cpplist.size());
   for (int i = 0; i < cpplist.size(); i++)
@@ -2008,7 +2131,7 @@ QScriptValue XTreeWidgetItemListtoScriptValue(QScriptEngine *engine, QList<XTree
   return scriptlist;
 }
 
-void XTreeWidgetItemListfromScriptValue(const QScriptValue &scriptlist, QList<XTreeWidgetItem *> &cpplist)
+void XTreeWidgetItemListFromScriptValue(const QScriptValue &scriptlist, QList<XTreeWidgetItem *> &cpplist)
 {
   cpplist.clear();
   int listlen = scriptlist.property("length").toInt32();
@@ -2019,21 +2142,6 @@ void XTreeWidgetItemListfromScriptValue(const QScriptValue &scriptlist, QList<XT
   }
 }
 
-QScriptValue XTreeWidgettoScriptValue(QScriptEngine *engine, XTreeWidget *const &item)
-{
-  return engine->newQObject(item);
-}
-
-void XTreeWidgetfromScriptValue(const QScriptValue &obj, XTreeWidget * &item)
-{
-  item = qobject_cast<XTreeWidget *>(obj.toQObject());
-}
-
-// script exposure of xtreewidgetitem /////////////////////////////////////////
-
-Q_DECLARE_METATYPE(QList<XTreeWidgetItem *>)
-Q_DECLARE_METATYPE(QList<XTreeWidget *>)
-
 QScriptValue constructXTreeWidgetItem(QScriptContext *context,QScriptEngine  *engine)
 {
   XTreeWidget     *tree  = qscriptvalue_cast<XTreeWidget *>(context->argument(0));
@@ -2126,49 +2234,65 @@ QScriptValue constructXTreeWidgetItem(QScriptContext *context,QScriptEngine  *en
 
 void setupXTreeWidgetItem(QScriptEngine *engine)
 {
-  qScriptRegisterMetaType(engine, XTreeWidgetItemtoScriptValue,     XTreeWidgetItemfromScriptValue);
-  qScriptRegisterMetaType(engine, XTreeWidgetItemListtoScriptValue, XTreeWidgetItemListfromScriptValue);
+  if (! engine->globalObject().property("XTreeWidgetItem").isFunction())
+  {
+    qScriptRegisterMetaType(engine, XTreeWidgetItemToScriptValue,     XTreeWidgetItemFromScriptValue);
+    qScriptRegisterMetaType(engine, XTreeWidgetItemListToScriptValue, XTreeWidgetItemListFromScriptValue);
+    QScriptValue ctor = engine->newFunction(constructXTreeWidgetItem);
+    QScriptValue meta = engine->newQMetaObject(&XTreeWidgetItem::staticMetaObject, ctor);
 
-  QScriptValue constructor = engine->newFunction(constructXTreeWidgetItem);
-  engine->globalObject().setProperty("XTreeWidgetItem", constructor, QScriptValue::ReadOnly | QScriptValue::Undeletable);
+    engine->globalObject().setProperty("XTreeWidgetItem", meta,
+                                       QScriptValue::ReadOnly | QScriptValue::Undeletable);
+  }
 }
 
 // xtreewidget ////////////////////////////////////////////////////////////////
 
-void setupXTreeWidget(QScriptEngine *engine)
+QScriptValue XTreeWidgetToScriptValue(QScriptEngine *engine, XTreeWidget *const &item)
 {
-  QScriptValue glob = engine->newObject();
-
-  qScriptRegisterMetaType(engine, XTreeWidgettoScriptValue,     XTreeWidgetfromScriptValue);
-
-  glob.setProperty("Replace", QScriptValue(engine, XTreeWidget::Replace), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  glob.setProperty("Append",  QScriptValue(engine, XTreeWidget::Append), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-
-  glob.setProperty("itemColumn",     QScriptValue(engine, _itemColumn),    QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  glob.setProperty("whsColumn",      QScriptValue(engine, _whsColumn),     QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  glob.setProperty("userColumn",     QScriptValue(engine, _userColumn),    QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  glob.setProperty("dateColumn",     QScriptValue(engine, _dateColumn),    QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  glob.setProperty("timeDateColumn", QScriptValue(engine, _timeDateColumn),QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  glob.setProperty("timeColumn",     QScriptValue(engine, _timeColumn),    QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  glob.setProperty("qtyColumn",      QScriptValue(engine, _qtyColumn),     QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  glob.setProperty("priceColumn",    QScriptValue(engine, _priceColumn),   QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  glob.setProperty("moneyColumn",    QScriptValue(engine, _moneyColumn),   QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  glob.setProperty("bigMoneyColumn", QScriptValue(engine, _bigMoneyColumn),QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  glob.setProperty("costColumn",     QScriptValue(engine, _costColumn),    QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  glob.setProperty("prcntColumn",    QScriptValue(engine, _prcntColumn),   QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  glob.setProperty("transColumn",    QScriptValue(engine, _transColumn),   QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  glob.setProperty("uomColumn",      QScriptValue(engine, _uomColumn),     QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  glob.setProperty("orderColumn",    QScriptValue(engine, _orderColumn),   QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  glob.setProperty("statusColumn",   QScriptValue(engine, _statusColumn),  QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  glob.setProperty("seqColumn",      QScriptValue(engine, _seqColumn),     QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  glob.setProperty("ynColumn",       QScriptValue(engine, _ynColumn),      QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  glob.setProperty("docTypeColumn",  QScriptValue(engine, _docTypeColumn), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-  glob.setProperty("currencyColumn", QScriptValue(engine, _currencyColumn),QScriptValue::ReadOnly | QScriptValue::Undeletable);
-
-  engine->globalObject().setProperty("XTreeWidget", glob, QScriptValue::ReadOnly | QScriptValue::Undeletable);
+  return engine->newQObject(item);
 }
 
-Q_DECLARE_METATYPE(XTreeWidget *)
+void XTreeWidgetFromScriptValue(const QScriptValue &obj, XTreeWidget * &item)
+{
+  item = qobject_cast<XTreeWidget *>(obj.toQObject());
+}
+
+void setupXTreeWidget(QScriptEngine *engine)
+{
+  qScriptRegisterMetaType(engine, XTreeWidgetToScriptValue, XTreeWidgetFromScriptValue);
+
+  if (! engine->globalObject().property("XTreeWidget").isObject())
+  {
+    QScriptValue::PropertyFlags ro = QScriptValue::ReadOnly | QScriptValue::Undeletable;
+    QScriptValue ctor = engine->newObject(); //engine->newFunction(scriptconstructor);
+    QScriptValue meta = engine->newQMetaObject(&XTreeWidget::staticMetaObject, ctor);
+
+    engine->globalObject().setProperty("XTreeWidget", meta, ro);
+
+    // these aren't in an enum so need to be exposed explicitly
+    meta.setProperty("itemColumn",     QScriptValue(engine, _itemColumn),     ro);
+    meta.setProperty("whsColumn",      QScriptValue(engine, _whsColumn),      ro);
+    meta.setProperty("userColumn",     QScriptValue(engine, _userColumn),     ro);
+    meta.setProperty("dateColumn",     QScriptValue(engine, _dateColumn),     ro);
+    meta.setProperty("timeDateColumn", QScriptValue(engine, _timeDateColumn), ro);
+    meta.setProperty("timeColumn",     QScriptValue(engine, _timeColumn),     ro);
+    meta.setProperty("qtyColumn",      QScriptValue(engine, _qtyColumn),      ro);
+    meta.setProperty("priceColumn",    QScriptValue(engine, _priceColumn),    ro);
+    meta.setProperty("moneyColumn",    QScriptValue(engine, _moneyColumn),    ro);
+    meta.setProperty("bigMoneyColumn", QScriptValue(engine, _bigMoneyColumn), ro);
+    meta.setProperty("costColumn",     QScriptValue(engine, _costColumn),     ro);
+    meta.setProperty("prcntColumn",    QScriptValue(engine, _prcntColumn),    ro);
+    meta.setProperty("transColumn",    QScriptValue(engine, _transColumn),    ro);
+    meta.setProperty("uomColumn",      QScriptValue(engine, _uomColumn),      ro);
+    meta.setProperty("orderColumn",    QScriptValue(engine, _orderColumn),    ro);
+    meta.setProperty("statusColumn",   QScriptValue(engine, _statusColumn),   ro);
+    meta.setProperty("seqColumn",      QScriptValue(engine, _seqColumn),      ro);
+    meta.setProperty("ynColumn",       QScriptValue(engine, _ynColumn),       ro);
+    meta.setProperty("docTypeColumn",  QScriptValue(engine, _docTypeColumn),  ro);
+    meta.setProperty("currencyColumn", QScriptValue(engine, _currencyColumn), ro);
+  }
+}
 
 void XTreeWidget::sCopyRowToClipboard()
 {
@@ -2278,6 +2402,28 @@ void XTreeWidget::sCopyVisibleToClipboard()
   clipboard->setMimeData(mime);
 }
 
+void XTreeWidget::sCopyColumnToClipboard()
+{
+  QTextEdit       text;
+  QMimeData       *mime      = new QMimeData();
+  QClipboard      *clipboard = QApplication::clipboard();
+  QString opText = "";
+  XTreeWidgetItem *item = topLevelItem(0);
+  if (item)
+  {
+    for (QModelIndex idx = indexFromItem(item); idx.isValid(); idx = indexBelow(idx))
+    {
+      item = (XTreeWidgetItem *)itemFromIndex(idx);
+      if (item)
+        opText = opText + item->text(currentColumn()) + "\t\r\n";
+    }
+  }
+  text.setText(opText);
+  mime->setText(text.toPlainText());
+  mime->setHtml(text.toHtml());
+  clipboard->setMimeData(mime);
+}
+
 void XTreeWidget::sSearch(const QString &pTarget)
 {
   clearSelection();
@@ -2313,8 +2459,7 @@ QString XTreeWidget::toTxt() const
   XTreeWidgetItem *item = topLevelItem(0);
   if (item)
   {
-    QModelIndex idx = indexFromItem(item);
-    while (idx.isValid())
+    for (QModelIndex idx = indexFromItem(item); idx.isValid(); idx = indexBelow(idx))
     {
       item = (XTreeWidgetItem *)itemFromIndex(idx);
       if (item)
@@ -2327,7 +2472,6 @@ QString XTreeWidget::toTxt() const
         }
       }
       opText = opText + line + "\r\n";
-      idx    = indexBelow(idx);
     }
   }
   return opText;
@@ -2356,8 +2500,7 @@ QString XTreeWidget::toCsv() const
   XTreeWidgetItem *item = topLevelItem(0);
   if (item)
   {
-    QModelIndex idx = indexFromItem(item);
-    while (idx.isValid())
+    for (QModelIndex idx = indexFromItem(item); idx.isValid(); idx = indexBelow(idx))
     {
       colcount = 0;
       item     = (XTreeWidgetItem *)itemFromIndex(idx);
@@ -2380,7 +2523,6 @@ QString XTreeWidget::toCsv() const
         }
       }
       opText = opText + line + "\r\n";
-      idx    = indexBelow(idx);
     }
   }
   return opText;
@@ -2497,6 +2639,17 @@ QString XTreeWidget::toHtml() const
   QString font;
   int     colcnt = 0;
   int     rowcnt = 0;
+  int     row = 0;
+  double  limit = 0;
+  qlonglong maxDataCount = 0;
+  qlonglong dataCount = 0;
+
+  if (_x_preferences)
+  {
+    limit = _x_preferences->value("XTreeWidgetDataLimit").toDouble();
+    if (limit > 0)
+      maxDataCount = (qlonglong)(limit * 1e9);
+  }
 
   tableFormat.setHeaderRowCount(1);
 
@@ -2508,13 +2661,11 @@ QString XTreeWidget::toHtml() const
   XTreeWidgetItem *item = topLevelItem(0);
   if (item)
   {
-    QModelIndex idx = indexFromItem(item);
-    while (idx.isValid())
+    for (QModelIndex idx = indexFromItem(item); idx.isValid(); idx = indexBelow(idx))
     {
       item = (XTreeWidgetItem *)itemFromIndex(idx);
       if (item)
         rowcnt++;
-      idx = indexBelow(idx);
     }
   }
 
@@ -2530,14 +2681,15 @@ QString XTreeWidget::toHtml() const
       cell.setFormat(format);
       cursor->insertText(header->text(counter));
       cursor->movePosition(QTextCursor::NextCell);
+
+      dataCount += (qlonglong)(header->text(counter).size());
     }
   }
 
   item = topLevelItem(0);
   if (item)
   {
-    QModelIndex idx = indexFromItem(item);
-    while (idx.isValid())
+    for (QModelIndex idx = indexFromItem(item); idx.isValid() && (maxDataCount <= 0 || dataCount < maxDataCount); idx = indexBelow(idx), row++)
     {
       item = (XTreeWidgetItem *)itemFromIndex(idx);
       if (item)
@@ -2563,10 +2715,17 @@ QString XTreeWidget::toHtml() const
             cell.setFormat(format);
             cursor->insertText(item->text(counter));
             cursor->movePosition(QTextCursor::NextCell);
+
+            dataCount += (qlonglong)(item->text(counter).size());
           }
         }
       }
-      idx = indexBelow(idx);
+    }
+
+    if (maxDataCount > 0 && dataCount > maxDataCount)
+    {
+      QString overflowMsg = tr("Maximum data limit was encountered.  Only %1 of %2 rows could be processed.");
+      QMessageBox::warning(NULL, tr("Data Limit Reached"), overflowMsg.arg(row).arg(rowcnt));
     }
   }
   return doc->toHtml();
@@ -2698,3 +2857,21 @@ QVariant XTreeWidget::rawValue(const QString colname) const
     }
     return QVariant();
 }
+
+void XTreeWidget::keyPressEvent(QKeyEvent* e)
+{
+  if (e->key()==Qt::Key_Enter)
+  {
+    bool hasDefault = false;
+    QList<QPushButton*> buttons = window()->findChildren<QPushButton*>();
+    foreach (QPushButton* button, buttons)
+      if (button->isDefault())
+        hasDefault = true;
+    if (window()->inherits("QDialog") && hasDefault)
+      QTreeWidget::keyPressEvent(e);
+    else
+      sItemSelected();
+  }
+  else
+    QTreeWidget::keyPressEvent(e);
+}
