@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -59,9 +59,6 @@ class QScriptEngine;
 class XTreeWidget;
 class XTreeWidgetProgress;
 
-void  setupXTreeWidgetItem(QScriptEngine *engine);
-void  setupXTreeWidget(QScriptEngine *engine);
-
 class XTUPLEWIDGETS_EXPORT XTreeWidgetItem : public QObject, public QTreeWidgetItem
 {
   Q_OBJECT
@@ -134,6 +131,10 @@ class XTUPLEWIDGETS_EXPORT XTreeWidgetItem : public QObject, public QTreeWidgetI
     Q_INVOKABLE virtual QVariant        rawValue(const QString colname);
     Q_INVOKABLE virtual int             id(const QString);
 
+    Q_INVOKABLE virtual void            setDate(int pColIdx, const QVariant pDate);
+    Q_INVOKABLE virtual void            setNumber(int pColIdx, const QVariant pValue, const QString pRole);
+    Q_INVOKABLE virtual bool            setNumericRole(int pColIdx, const QString pRole);
+
     virtual bool operator               <(const XTreeWidgetItem &other) const;
     virtual bool operator               ==(const XTreeWidgetItem &other) const;
 
@@ -157,9 +158,6 @@ class XTUPLEWIDGETS_EXPORT XTreeWidgetItem : public QObject, public QTreeWidgetI
     int _altId;
 };
 
-Q_DECLARE_METATYPE(XTreeWidgetItem *)
-// Q_DECLARE_METATYPE(XTreeWidgetItem)
-
 class XTreeWidgetPopulateParams;
 
 class XTUPLEWIDGETS_EXPORT XTreeWidget : public QTreeWidget
@@ -168,10 +166,10 @@ class XTUPLEWIDGETS_EXPORT XTreeWidget : public QTreeWidget
   Q_PROPERTY( QString altDragString READ altDragString WRITE setAltDragString)
   Q_PROPERTY( bool populateLinear READ populateLinear WRITE setPopulateLinear)
 
-  Q_ENUMS(PopulateStyle)
-
   public :
     enum PopulateStyle { Replace, Append };
+    Q_ENUM(PopulateStyle)
+
     XTreeWidget(QWidget *);
     ~XTreeWidget();
 
@@ -187,6 +185,10 @@ class XTUPLEWIDGETS_EXPORT XTreeWidget : public QTreeWidget
     bool    populateLinear();
     void    setPopulateLinear(bool alwaysLinear = true);
 
+    void keyPressEvent(QKeyEvent* e);
+    void mergeSort(int low, int high);
+    void merge(int low, int high, int mid);
+    
     Q_INVOKABLE int   altId() const;
     Q_INVOKABLE int   id()    const;
     Q_INVOKABLE int   id(const QString)     const;
@@ -194,11 +196,15 @@ class XTUPLEWIDGETS_EXPORT XTreeWidget : public QTreeWidget
     Q_INVOKABLE void  setId(int pId,int pAltId, bool pClear = true);
 
     Q_INVOKABLE virtual int column(const QString) const;
+    Q_INVOKABLE virtual QString column(const int) const;
     Q_INVOKABLE virtual XTreeWidgetItem *currentItem()         const;
     Q_INVOKABLE virtual void            setColumnCount(int columns);
     Q_INVOKABLE virtual void            setColumnLocked(int, bool);
     Q_INVOKABLE virtual void            setColumnVisible(int, bool);
-    Q_INVOKABLE virtual void            sortItems(int column, Qt::SortOrder order);
+    Q_INVOKABLE virtual void            sortItems(int column = 0,
+                                                  Qt::SortOrder order = Qt::AscendingOrder);
+    Q_INVOKABLE QList<QPair<int, Qt::SortOrder> > sortColumnOrder();
+
     Q_INVOKABLE virtual XTreeWidgetItem *topLevelItem(int idx) const;
 
     Q_INVOKABLE XTreeWidgetItem         *findXTreeWidgetItemWithId(const XTreeWidget *ptree, const int pid);
@@ -240,8 +246,6 @@ class XTUPLEWIDGETS_EXPORT XTreeWidget : public QTreeWidget
     Q_INVOKABLE inline void               setHeaderLabel(const QString &label)                              {        QTreeWidget::setHeaderLabel(label); }
     Q_INVOKABLE inline void               setHeaderLabels(const QStringList &labels)                        {        QTreeWidget::setHeaderLabels(labels); }
     Q_INVOKABLE inline void               setItemWidget(XTreeWidgetItem *item, int column, QWidget *widget) {        QTreeWidget::setItemWidget(item, column, widget); }
-    Q_INVOKABLE inline int                sortColumn() const { return QTreeWidget::sortColumn(); }
-    Q_INVOKABLE inline Qt::SortOrder      sortOrder()  const { return header()->sortIndicatorOrder(); } // temporary(?) until we expose all of qt
     Q_INVOKABLE inline QTreeWidgetItem    *takeTopLevelItem(int index)                                      { return QTreeWidget::takeTopLevelItem(index); }
     Q_INVOKABLE inline int                topLevelItemCount() const { return QTreeWidget::topLevelItemCount(); }
     Q_INVOKABLE inline QRect              visualItemRect(const XTreeWidgetItem *item) const                 { return QTreeWidget::visualItemRect(item); }
@@ -261,6 +265,7 @@ class XTUPLEWIDGETS_EXPORT XTreeWidget : public QTreeWidget
     void  sCopyVisibleToClipboard();
     void  sCopyRowToClipboard();
     void  sCopyCellToClipboard();
+    void  sCopyColumnToClipboard();
     void  sSearch(const QString&);
 
   signals:
@@ -321,8 +326,7 @@ class XTUPLEWIDGETS_EXPORT XTreeWidget : public QTreeWidget
     bool          _settingsLoaded;
     QString       _settingsName;
     int           _resetWhichWidth;
-    int           _scol;
-    Qt::SortOrder _sord;
+    QList<QPair<int, Qt::SortOrder> > _sort;
     static void   loadLocale();
     QList<XTreeWidgetPopulateParams> _workingParams;
     QTimer        _workingTimer;
@@ -363,4 +367,7 @@ class XTreeWidgetPopulateParams
     XTreeWidget::PopulateStyle _workingPopstyle;
 };
 
+void  setupXTreeWidgetItem(QScriptEngine *engine);
+void  setupXTreeWidget(QScriptEngine *engine);
+
 #endif
